@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -108,7 +108,7 @@ public class DOMObjectModel extends TreeModel implements ExternalObjectModel {
      */
 
     public PJConverter getPJConverter(Class targetClass) {
-        if (Node.class.isAssignableFrom(targetClass) && !(NodeOverNodeInfo.class.isAssignableFrom(targetClass))) {
+        if (Node.class.isAssignableFrom(targetClass) && !NodeOverNodeInfo.class.isAssignableFrom(targetClass)) {
             return new PJConverter() {
                 public Object convert(Sequence value, Class targetClass, XPathContext context) throws XPathException {
                     return convertXPathValueToObject(value, targetClass);
@@ -127,9 +127,9 @@ public class DOMObjectModel extends TreeModel implements ExternalObjectModel {
 
     //public NodeSet ns;
     public JPConverter getJPConverter(Class sourceClass, Configuration config) {
-        if (Node.class.isAssignableFrom(sourceClass) && !(NodeOverNodeInfo.class.isAssignableFrom(sourceClass))) {
+        if (Node.class.isAssignableFrom(sourceClass) && !NodeOverNodeInfo.class.isAssignableFrom(sourceClass)) {
             return new JPConverter() {
-                public Sequence convert(Object obj, XPathContext context) throws XPathException {
+                public Sequence convert(Object obj, XPathContext context) {
                     return wrapOrUnwrapNode((Node) obj, context.getConfiguration());
                 }
 
@@ -139,7 +139,7 @@ public class DOMObjectModel extends TreeModel implements ExternalObjectModel {
             };
         } else if (NodeList.class.isAssignableFrom(sourceClass)) {
             return new JPConverter() {
-                public Sequence convert(Object obj, XPathContext context) throws XPathException {
+                public Sequence convert(Object obj, XPathContext context) {
                     Configuration config = context.getConfiguration();
                     NodeList list = (NodeList) obj;
                     final int len = list.getLength();
@@ -160,7 +160,7 @@ public class DOMObjectModel extends TreeModel implements ExternalObjectModel {
             };
         } else if (DOMSource.class.isAssignableFrom(sourceClass)) {
             return new JPConverter() {
-                public Sequence convert(Object obj, XPathContext context) throws XPathException {
+                public Sequence convert(Object obj, XPathContext context) {
                     return unravel((DOMSource) obj, context.getConfiguration());
                 }
 
@@ -170,7 +170,7 @@ public class DOMObjectModel extends TreeModel implements ExternalObjectModel {
             };
         } else if (DocumentWrapper.class == sourceClass) {
             return new JPConverter() {
-                public Sequence convert(Object obj, XPathContext context) throws XPathException {
+                public Sequence convert(Object obj, XPathContext context) {
                     return ((DocumentWrapper) obj).getRootNode();
                 }
 
@@ -271,8 +271,10 @@ public class DOMObjectModel extends TreeModel implements ExternalObjectModel {
         Node startNode = source.getNode();
         if (startNode == null) {
             // Send an empty document
+            receiver.open();
             receiver.startDocument(0);
             receiver.endDocument();
+            receiver.close();
         } else {
             DOMSender driver = new DOMSender(startNode, receiver);
             driver.setSystemId(source.getSystemId());
@@ -356,11 +358,9 @@ public class DOMObjectModel extends TreeModel implements ExternalObjectModel {
      * @param config the Saxon Configuration. This must be the same Configuration, or one that is compatible with,
      *               the Configuration used to compile and execute the query or stylesheet.
      * @return the new wrapper node
-     * @throws net.sf.saxon.trans.XPathException
-     *          if the operation fails
      */
 
-    private NodeInfo wrapOrUnwrapNode(Node node, Configuration config) throws XPathException {
+    private NodeInfo wrapOrUnwrapNode(Node node, Configuration config) {
         if (node instanceof NodeOverNodeInfo) {
             return ((NodeOverNodeInfo) node).getUnderlyingNodeInfo();
         } else {
@@ -383,25 +383,25 @@ public class DOMObjectModel extends TreeModel implements ExternalObjectModel {
      *                        supplied value cannot be converted to the appropriate class
      */
 
-    public static Object convertXPathValueToObject(Sequence value, Class target) throws XPathException {
+    public static Object convertXPathValueToObject(Sequence value, Class<?> target) throws XPathException {
         // We accept the object if (a) the target class is Node, Node[], or NodeList,
         // or (b) the supplied object is a node, or sequence of nodes, that wrap DOM nodes,
         // provided that the target class is Object or a collection class
         boolean requireDOM =
-                (Node.class.isAssignableFrom(target) || (target == NodeList.class) ||
-                        (target.isArray() && Node.class.isAssignableFrom(target.getComponentType())));
+                Node.class.isAssignableFrom(target) || (target == NodeList.class) ||
+                        (target.isArray() && Node.class.isAssignableFrom(target.getComponentType()));
 
         // Note: we allow the declared type of the method argument to be a subclass of Node. If the actual
         // node supplied is the wrong kind of node, this will result in a Java exception.
 
         boolean allowDOM =
-                (target == Object.class || target.isAssignableFrom(ArrayList.class) ||
+                target == Object.class || target.isAssignableFrom(ArrayList.class) ||
                         target.isAssignableFrom(HashSet.class) ||
-                        (target.isArray() && target.getComponentType() == Object.class));
+                        (target.isArray() && target.getComponentType() == Object.class);
         if (!(requireDOM || allowDOM)) {
             return null;
         }
-        List<Node> nodes = new ArrayList<Node>(20);
+        List<Node> nodes = new ArrayList<>(20);
 
         SequenceIterator iter = value.iterate();
         Item item;
@@ -439,11 +439,11 @@ public class DOMObjectModel extends TreeModel implements ExternalObjectModel {
         } else if (target == NodeList.class) {
             return new DOMNodeList(nodes);
         } else if (target.isArray() && target.getComponentType() == Node.class) {
-            return nodes.toArray(new Node[nodes.size()]);
+            return nodes.toArray(new Node[0]);
         } else if (target.isAssignableFrom(ArrayList.class)) {
             return nodes;
         } else if (target.isAssignableFrom(HashSet.class)) {
-            return new HashSet<Node>(nodes);
+            return new HashSet<>(nodes);
         } else {
             // after all this work, give up
             return null;
