@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -11,29 +11,25 @@ import net.sf.saxon.expr.parser.ExplicitLocation;
 import net.sf.saxon.expr.parser.Location;
 import net.sf.saxon.expr.parser.RoleDiagnostic;
 import net.sf.saxon.expr.parser.Token;
-import net.sf.saxon.om.Item;
-import net.sf.saxon.om.NamespaceBindingSet;
-import net.sf.saxon.om.NodeName;
+import net.sf.saxon.om.*;
 import net.sf.saxon.pattern.CombinedNodeTest;
 import net.sf.saxon.pattern.ContentTypeTest;
 import net.sf.saxon.pattern.NameTest;
 import net.sf.saxon.pattern.NodeKindTest;
 import net.sf.saxon.trans.XPathException;
-import net.sf.saxon.type.ItemType;
-import net.sf.saxon.type.SchemaType;
-import net.sf.saxon.type.SimpleType;
-import net.sf.saxon.type.Type;
+import net.sf.saxon.tree.util.Orphan;
+import net.sf.saxon.type.*;
 import net.sf.saxon.value.Cardinality;
 
 import java.util.HashSet;
+import java.util.function.Supplier;
 
 /**
  * A filter on the push pipeline that performs type checking, both of the item type and the
  * cardinality.
- * <p/>
- * Note that the TypeCheckingFilter cannot currently check document node tests of the form
+ * <p>Note that the TypeCheckingFilter cannot currently check document node tests of the form
  * document-node(element(X,Y)), so it is not invoked in such cases. This isn't a big problem, because most
- * instructions that return document nodes materialize them anyway.
+ * instructions that return document nodes materialize them anyway.</p>
  */
 
 public class TypeCheckingFilter extends ProxyReceiver {
@@ -44,7 +40,7 @@ public class TypeCheckingFilter extends ProxyReceiver {
     private Location locator;
     private int count = 0;
     private int level = 0;
-    private HashSet<Long> checkedElements = new HashSet<Long>(10);
+    private HashSet<Long> checkedElements = new HashSet<>(10);
     // used to avoid repeated checking when a template creates large numbers of elements of the same type
     // The key is a (namecode, typecode) pair, packed into a single long
 
@@ -63,12 +59,15 @@ public class TypeCheckingFilter extends ProxyReceiver {
      * Notify an attribute. Attributes are notified after the startElement event, and before any
      * children. Namespaces and attributes may be intermingled.
      *
-     * @param nameCode   The name of the attribute, as held in the name pool
-     * @param typeCode   The type of the attribute, as held in the name pool
-     * @param locationId
+     * @param nameCode   The name of the attribute
+     * @param typeCode   The type of the attribute
+     * @param locationId the location of the node in the source, or of the instruction that created it
      * @param properties Bit significant value. The following bits are defined:
+     *                   <dl>
      *                   <dd>DISABLE_ESCAPING</dd>    <dt>Disable escaping for this attribute</dt>
-     *                   <dd>NO_SPECIAL_CHARACTERS</dd>      <dt>Attribute value contains no special characters</dt>  @throws IllegalStateException: attempt to output an attribute when there is no open element
+     *                   <dd>NO_SPECIAL_CHARACTERS</dd>      <dt>Attribute value contains no special characters</dt>
+     *                   </dl>
+     * @throws IllegalStateException: attempt to output an attribute when there is no open element
      *                                start tag
      */
 
@@ -81,7 +80,7 @@ public class TypeCheckingFilter extends ProxyReceiver {
                     new NameTest(Type.ATTRIBUTE, nameCode, getNamePool()),
                     Token.INTERSECT,
                     new ContentTypeTest(Type.ATTRIBUTE, typeCode, getConfiguration(), false));
-            checkItemType(type, locationId);
+            checkItemType(type, nodeSupplier(Type.ATTRIBUTE, nameCode, value), locationId);
         }
         nextReceiver.attribute(nameCode, typeCode, value, locationId, properties);
     }
@@ -95,7 +94,7 @@ public class TypeCheckingFilter extends ProxyReceiver {
             if (++count == 2) {
                 checkAllowsMany(locationId);
             }
-            checkItemType(NodeKindTest.TEXT, locationId);
+            checkItemType(NodeKindTest.TEXT, nodeSupplier(Type.TEXT, null, chars), locationId);
         }
         nextReceiver.characters(chars, locationId, properties);
     }
@@ -109,9 +108,9 @@ public class TypeCheckingFilter extends ProxyReceiver {
             if (++count == 2) {
                 checkAllowsMany(locationId);
             }
-            checkItemType(NodeKindTest.COMMENT, locationId);
+            checkItemType(NodeKindTest.COMMENT, nodeSupplier(Type.COMMENT, null, chars), locationId);
         }
-        nextReceiver.comment(chars, locationId, properties);    //To change body of overridden methods use File | Settings | File Templates.
+        nextReceiver.comment(chars, locationId, properties);
     }
 
     /**
@@ -130,7 +129,7 @@ public class TypeCheckingFilter extends ProxyReceiver {
             if (++count == 2) {
                 checkAllowsMany(ExplicitLocation.UNKNOWN_LOCATION);
             }
-            checkItemType(NodeKindTest.NAMESPACE, ExplicitLocation.UNKNOWN_LOCATION);
+            checkItemType(NodeKindTest.NAMESPACE, null, ExplicitLocation.UNKNOWN_LOCATION);
         }
         nextReceiver.namespace(namespaceBindings, properties);    //To change body of overridden methods use File | Settings | File Templates.
     }
@@ -144,7 +143,8 @@ public class TypeCheckingFilter extends ProxyReceiver {
             if (++count == 2) {
                 checkAllowsMany(locationId);
             }
-            checkItemType(NodeKindTest.PROCESSING_INSTRUCTION, locationId);
+            checkItemType(NodeKindTest.PROCESSING_INSTRUCTION,
+                          nodeSupplier(Type.PROCESSING_INSTRUCTION, new NoNamespaceName(target), data), locationId);
         }
         nextReceiver.processingInstruction(target, data, locationId, properties);
     }
@@ -158,7 +158,8 @@ public class TypeCheckingFilter extends ProxyReceiver {
             if (++count == 2) {
                 checkAllowsMany(ExplicitLocation.UNKNOWN_LOCATION);
             }
-            checkItemType(NodeKindTest.DOCUMENT, ExplicitLocation.UNKNOWN_LOCATION);
+            checkItemType(NodeKindTest.DOCUMENT,
+                          nodeSupplier(Type.DOCUMENT, null, ""), ExplicitLocation.UNKNOWN_LOCATION);
         }
         level++;
         nextReceiver.startDocument(properties);
@@ -166,9 +167,9 @@ public class TypeCheckingFilter extends ProxyReceiver {
 
     /**
      * Notify the start of an element
-     *  @param nodeName   integer code identifying the name of the element within the name pool.
-     * @param typeCode   integer code identifying the element's type within the name pool.
-     * @param location
+     * @param nodeName   the name of the element
+     * @param typeCode   the element's type
+     * @param location   the location of the element
      * @param properties properties of the element node
      */
 
@@ -180,7 +181,7 @@ public class TypeCheckingFilter extends ProxyReceiver {
                         new NameTest(Type.ELEMENT, nodeName, getNamePool()),
                         Token.INTERSECT,
                         new ContentTypeTest(Type.ELEMENT, typeCode, getConfiguration(), false));
-                checkItemType(type, location);
+                checkItemType(type, nodeSupplier(Type.ELEMENT, nodeName, ""), location);
             } else {
                 if (count == 2) {
                     checkAllowsMany(location);
@@ -191,7 +192,7 @@ public class TypeCheckingFilter extends ProxyReceiver {
                             new NameTest(Type.ELEMENT, nodeName, getNamePool()),
                             Token.INTERSECT,
                             new ContentTypeTest(Type.ELEMENT, typeCode, getConfiguration(), false));
-                    checkItemType(type, location);
+                    checkItemType(type, nodeSupplier(Type.ELEMENT, nodeName, ""), location);
                     checkedElements.add(key);
                 }
             }
@@ -236,6 +237,18 @@ public class TypeCheckingFilter extends ProxyReceiver {
         // don't pass on the close event
     }
 
+    private Supplier<NodeInfo> nodeSupplier(short nodeKind, NodeName name, CharSequence value) {
+        return () -> {
+            Orphan o = new Orphan(getConfiguration());
+            o.setNodeKind(nodeKind);
+            if (name != null) {
+                o.setNodeName(name);
+            }
+            o.setStringValue(value);
+            return o;
+        };
+    }
+
     /**
      * Output an item (atomic value or node) to the sequence
      */
@@ -248,7 +261,7 @@ public class TypeCheckingFilter extends ProxyReceiver {
             checkItem(item, locationId);
         }
         if (nextReceiver instanceof SequenceReceiver) {
-            ((SequenceReceiver) nextReceiver).append(item, locationId, copyNamespaces);
+            nextReceiver.append(item, locationId, copyNamespaces);
         } else {
             super.append(item, locationId, copyNamespaces);
         }
@@ -266,20 +279,28 @@ public class TypeCheckingFilter extends ProxyReceiver {
         return true;
     }
 
-    private void checkItemType(ItemType type, Location locationId) throws XPathException {
+    private void checkItemType(ItemType type, Supplier<? extends Item> itemSupplier, Location locationId) throws XPathException {
         if (!getConfiguration().getTypeHierarchy().isSubType(type, itemType)) {
-            throwTypeError(type, locationId);
+            throwTypeError(type, itemSupplier == null? null : itemSupplier.get(), locationId);
         }
     }
 
     private void checkItem(Item item, Location locationId) throws XPathException {
-        if (!itemType.matches(item, getConfiguration().getTypeHierarchy())) {
-            throwTypeError(Type.getItemType(item, getConfiguration().getTypeHierarchy()), locationId);
+        TypeHierarchy th = getConfiguration().getTypeHierarchy();
+        if (!itemType.matches(item, th)) {
+            throwTypeError(null, item, locationId);
         }
     }
 
-    private void throwTypeError(ItemType type, Location locationId) throws XPathException {
-        String message = role.composeErrorMessage(itemType, type);
+
+    private void throwTypeError(ItemType suppliedType, Item item, Location locationId) throws XPathException {
+        String message;
+        TypeHierarchy th = getConfiguration().getTypeHierarchy();
+        if (item == null) {
+            message = role.composeErrorMessage(itemType, suppliedType);
+        } else {
+            message = role.composeErrorMessage(itemType, item, th);
+        }
         String errorCode = role.getErrorCode();
         XPathException err = new XPathException(message);
         err.setErrorCode(errorCode);
