@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,7 +7,7 @@
 
 package net.sf.saxon.expr;
 
-import net.sf.saxon.event.SequenceReceiver;
+import net.sf.saxon.event.Receiver;
 import net.sf.saxon.expr.instruct.Block;
 import net.sf.saxon.expr.instruct.Choose;
 import net.sf.saxon.expr.instruct.ValueOf;
@@ -73,7 +73,7 @@ public class AdjacentTextNodeMerger extends UnaryExpression {
     /*@NotNull*/
     @Override
     public Expression typeCheck(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
-        typeCheckChildren(visitor, contextInfo);
+        getOperand().typeCheck(visitor, contextInfo);
 
         // This wrapper expression is unnecessary if the base expression cannot return text nodes,
         // or if it can return at most one item
@@ -225,7 +225,7 @@ public class AdjacentTextNodeMerger extends UnaryExpression {
 
     /*@NotNull*/
     @Override
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<? extends Item> iterate(XPathContext context) throws XPathException {
         return new AdjacentTextNodeMergingIterator(getBaseExpression().iterate(context));
     }
 
@@ -237,7 +237,7 @@ public class AdjacentTextNodeMerger extends UnaryExpression {
      */
 
     public void process(XPathContext context, Location locationId, int options) throws XPathException {
-        SequenceReceiver out = context.getReceiver();
+        Receiver out = context.getReceiver();
         FastStringBuffer fsb = new FastStringBuffer(FastStringBuffer.C256);
         SequenceIterator iter = getBaseExpression().iterate(context);
         boolean prevText = false;
