@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,6 +7,7 @@
 
 package net.sf.saxon.expr;
 
+import net.sf.saxon.om.Item;
 import net.sf.saxon.om.SequenceIterator;
 import net.sf.saxon.regex.RegexIterator;
 import net.sf.saxon.trans.XPathException;
@@ -18,7 +19,7 @@ import net.sf.saxon.tree.iter.EmptyIterator;
  * expressions respectively to each such string
  */
 
-public class AnalyzeMappingFunction implements ContextMappingFunction {
+public class AnalyzeMappingFunction implements ContextMappingFunction<Item> {
 
     private RegexIterator base;
     private XPathContext c2;
@@ -43,7 +44,7 @@ public class AnalyzeMappingFunction implements ContextMappingFunction {
      *         sequence.
      */
 
-    public SequenceIterator map(XPathContext context) throws XPathException {
+    public SequenceIterator<? extends Item> map(XPathContext context) throws XPathException {
         if (base.isMatching()) {
             if (matchingExpr != null) {
                 return matchingExpr.iterate(c2);
