@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,10 +8,7 @@
 package net.sf.saxon.expr;
 
 import net.sf.saxon.Configuration;
-import net.sf.saxon.expr.parser.ContextItemStaticInfo;
-import net.sf.saxon.expr.parser.ExpressionTool;
-import net.sf.saxon.expr.parser.ExpressionVisitor;
-import net.sf.saxon.expr.parser.RebindingMap;
+import net.sf.saxon.expr.parser.*;
 import net.sf.saxon.lib.ConversionRules;
 import net.sf.saxon.om.GroundedValue;
 import net.sf.saxon.om.Item;
@@ -20,7 +17,6 @@ import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.type.*;
 import net.sf.saxon.value.AtomicValue;
-import net.sf.saxon.value.SequenceExtent;
 import net.sf.saxon.value.StringValue;
 
 
@@ -34,7 +30,9 @@ public class AtomicSequenceConverter extends UnaryExpression {
     public static ToStringMappingFunction TO_STRING_MAPPER = new ToStringMappingFunction();
 
     private PlainType requiredItemType;
-    /*@Nullable*/ protected Converter converter;
+    protected Converter converter;
+    private RoleDiagnostic roleDiagnostic; // may be null
+
 
     /**
      * Constructor
@@ -49,20 +47,26 @@ public class AtomicSequenceConverter extends UnaryExpression {
         this.requiredItemType = requiredItemType;
     }
 
-    public Converter allocateConverter(Configuration config, boolean allowNull) {
+    public void allocateConverterStatically(Configuration config, boolean allowNull) {
         converter = allocateConverter(config, allowNull, getBaseExpression().getItemType());
-        return converter;
     }
 
-    private Converter getDynamicConverter(Configuration config) {
-        return allocateConverter(config, false, getBaseExpression().getItemType());
+    public Converter allocateConverter(Configuration config, boolean allowNull) {
+        return allocateConverter(config, allowNull, getBaseExpression().getItemType());
+    }
+
+    protected Converter getConverterDynamically(XPathContext context) {
+        if (converter != null) {
+            return converter;
+        }
+        return allocateConverter(context.getConfiguration(), false);
     }
 
     public Converter allocateConverter(Configuration config, boolean allowNull, ItemType sourceType) {
         final ConversionRules rules = config.getConversionRules();
         Converter converter = null;
         if (sourceType instanceof ErrorType) {
-            converter = Converter.IDENTITY_CONVERTER;
+            converter = StringConverter.IdentityConverter.INSTANCE;
         } else if (!(sourceType instanceof AtomicType)) {
             converter = null;
         } else if (requiredItemType instanceof AtomicType) {
@@ -117,6 +121,35 @@ public class AtomicSequenceConverter extends UnaryExpression {
         this.converter = converter;
     }
 
+
+    /**
+     * Set a RoleDiagnostic, used to give more precise error information if the conversion
+     * fails
+     *
+     * @param role the RoleDiagnostic, which gives more precise error information
+     */
+
+    public void setRoleDiagnostic(RoleDiagnostic role) {
+        // TODO: pragmatic kludge in the run-up to 9.9. If the role says error=XPTY0004, ignore it, because
+        // in XQuery the cast failure should be FORG0001. We only use the error code in XSLT paths where it
+        // might be something like XTTE0780
+        if (role != null && !"XPTY0004".equals(role.getErrorCode())) {
+            this.roleDiagnostic = role;
+        }
+    }
+
+    /**
+     * Get the RoleDiagnostic, used to give more precise error information if the conversion
+     * fails
+     *
+     * @return the RoleDiagnostic, or null if none has been set
+     */
+
+    public RoleDiagnostic getRoleDiagnostic() {
+        return this.roleDiagnostic;
+    }
+
+
     /**
      * Simplify an expression
      *
@@ -131,13 +164,10 @@ public class AtomicSequenceConverter extends UnaryExpression {
                 return operand;
             }
             Configuration config = getConfiguration();
-            allocateConverter(config, true);
+            allocateConverterStatically(config, true);
             if (converter != null) {
-                GroundedValue val = SequenceExtent.makeSequenceExtent(
-                        iterate(new EarlyEvaluationContext(config)));
-                Literal result = Literal.makeLiteral(val);
-                result.setRetainedStaticContext(operand.getRetainedStaticContext());
-                return result;
+                GroundedValue val = iterate(new EarlyEvaluationContext(config)).materialize();
+                return Literal.makeLiteral(val, operand);
             }
         }
         return this;
@@ -157,7 +187,7 @@ public class AtomicSequenceConverter extends UnaryExpression {
             return operand;
         } else {
             if (converter == null) {
-                allocateConverter(config, true);
+                allocateConverterStatically(config, true);
             }
             return this;
         }
@@ -223,11 +253,11 @@ public class AtomicSequenceConverter extends UnaryExpression {
     /**
      * Determine the special properties of this expression
      *
-     * @return {@link StaticProperty#NON_CREATIVE}.
+     * @return {@link StaticProperty#NO_NODES_NEWLY_CREATED}.
      */
 
     public int computeSpecialProperties() {
-        int p = super.computeSpecialProperties() | StaticProperty.NON_CREATIVE;
+        int p = super.computeSpecialProperties() | StaticProperty.NO_NODES_NEWLY_CREATED;
         if (requiredItemType == BuiltInAtomicType.UNTYPED_ATOMIC) {
             p &= ~StaticProperty.NOT_UNTYPED_ATOMIC;
         } else {
@@ -246,7 +276,7 @@ public class AtomicSequenceConverter extends UnaryExpression {
      * Copy an expression. This makes a deep copy.
      *
      * @return the copy of the original expression
-     * @param rebindings
+     * @param rebindings variables that need to be re-bound
      */
 
     /*@NotNull*/
@@ -254,6 +284,7 @@ public class AtomicSequenceConverter extends UnaryExpression {
         AtomicSequenceConverter atomicConverter = new AtomicSequenceConverter(getBaseExpression().copy(rebindings), requiredItemType);
         ExpressionTool.copyLocationInfo(this, atomicConverter);
         atomicConverter.setConverter(converter);
+        atomicConverter.setRoleDiagnostic(getRoleDiagnostic());
         return atomicConverter;
     }
 
@@ -262,17 +293,17 @@ public class AtomicSequenceConverter extends UnaryExpression {
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(final XPathContext context) throws XPathException {
-        SequenceIterator base = getBaseExpression().iterate(context);
-        Converter conv = converter;
-        if (conv == null) {
-            conv = getDynamicConverter(context.getConfiguration());
-        }
-        if (conv == Converter.TO_STRING) {
+    public SequenceIterator<? extends Item> iterate(final XPathContext context) throws XPathException {
+        SequenceIterator<? extends Item> base = getBaseExpression().iterate(context);
+        Converter conv = getConverterDynamically(context);
+        if (conv == Converter.ToStringConverter.INSTANCE) {
             return new ItemMappingIterator(base, TO_STRING_MAPPER, true);
         } else {
             AtomicSequenceMappingFunction mapper = new AtomicSequenceMappingFunction();
             mapper.setConverter(conv);
+            if (roleDiagnostic != null) {
+                mapper.setErrorCode(roleDiagnostic.getErrorCode());
+            }
             return new ItemMappingIterator(base, mapper, true);
         }
     }
@@ -283,13 +314,22 @@ public class AtomicSequenceConverter extends UnaryExpression {
 
     public static class AtomicSequenceMappingFunction implements ItemMappingFunction {
         private Converter converter;
+        private String errorCode;
 
         public void setConverter(Converter converter) {
             this.converter = converter;
         }
 
+        public void setErrorCode(String code) {
+            this.errorCode = code;
+        }
+
         public AtomicValue mapItem(Item item) throws XPathException {
-            return converter.convert((AtomicValue) item).asAtomic();
+            ConversionResult result = converter.convert((AtomicValue) item);
+            if (errorCode != null && result instanceof ValidationFailure) {
+                ((ValidationFailure)result).setErrorCode(errorCode);
+            }
+            return result.asAtomic();
         }
     }
 
@@ -298,8 +338,8 @@ public class AtomicSequenceConverter extends UnaryExpression {
      */
 
     public static class ToStringMappingFunction
-            implements ItemMappingFunction {
-        public StringValue mapItem(Item item) throws XPathException {
+            implements ItemMappingFunction<Item, StringValue> {
+        public StringValue mapItem(Item item) {
             return StringValue.makeStringValue(item.getStringValueCS());
         }
     }
@@ -310,15 +350,17 @@ public class AtomicSequenceConverter extends UnaryExpression {
      */
 
     public AtomicValue evaluateItem(XPathContext context) throws XPathException {
-        Converter conv = converter;
-        if (conv == null) {
-            conv = getDynamicConverter(context.getConfiguration());
-        }
+        Converter conv = getConverterDynamically(context);
         AtomicValue item = (AtomicValue) getBaseExpression().evaluateItem(context);
         if (item == null) {
             return null;
         }
-        return conv.convert(item).asAtomic();
+        ConversionResult result = conv.convert(item);
+        if (roleDiagnostic != null && result instanceof ValidationFailure) {
+            // TODO: use more of the information in the roleDiagnostic to form the error message
+            ((ValidationFailure)result).setErrorCode(roleDiagnostic.getErrorCode());
+        }
+        return result.asAtomic();
     }
 
     /**
@@ -356,8 +398,8 @@ public class AtomicSequenceConverter extends UnaryExpression {
      */
 
     @Override
-    public int hashCode() {
-        return super.hashCode() ^ requiredItemType.hashCode();
+    public int computeHashCode() {
+        return super.computeHashCode() ^ requiredItemType.hashCode();
     }
 
 
@@ -390,7 +432,9 @@ public class AtomicSequenceConverter extends UnaryExpression {
         if (converter instanceof Converter.PromoterToDouble || converter instanceof Converter.PromoterToFloat) {
             destination.emitAttribute("flags", "p");
         }
-        //destination.emitAttribute("using", converter.getClass().toString());
+        if (getRoleDiagnostic() != null) {
+            destination.emitAttribute("diag", getRoleDiagnostic().save());
+        }
         getBaseExpression().export(destination);
         destination.endElement();
     }
