@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -13,15 +13,19 @@ import net.sf.saxon.expr.instruct.Choose;
 import net.sf.saxon.expr.instruct.ValueOf;
 import net.sf.saxon.expr.parser.*;
 import net.sf.saxon.ma.arrays.ArrayItemType;
+import net.sf.saxon.ma.map.MapType;
 import net.sf.saxon.om.*;
 import net.sf.saxon.pattern.NameTest;
 import net.sf.saxon.pattern.NodeKindTest;
 import net.sf.saxon.pattern.NodeTest;
 import net.sf.saxon.trans.XPathException;
-import net.sf.saxon.tree.iter.*;
+import net.sf.saxon.tree.iter.AtomizingIterator;
+import net.sf.saxon.tree.iter.EmptyIterator;
+import net.sf.saxon.tree.iter.UntypedAtomizingIterator;
 import net.sf.saxon.type.*;
 import net.sf.saxon.value.AtomicValue;
 import net.sf.saxon.value.Cardinality;
+import net.sf.saxon.value.EmptySequence;
 
 
 /**
@@ -111,8 +115,14 @@ public final class Atomizer extends UnaryExpression {
                 if (i instanceof Function) {
                     if (((Function)i).isArray()) {
                         return this;
+                    } else if (((Function)i).isMap()) {
+                        XPathException err = new XPathException("Cannot atomize a map", "FOTY0013");
+                        err.setIsTypeError(true);
+                        err.setLocation(getLocation());
+                        throw err;
                     } else {
                         XPathException err = new XPathException("Cannot atomize a function item", "FOTY0013");
+                        err.setIsTypeError(true);
                         err.setLocation(getLocation());
                         throw err;
                     }
@@ -135,7 +145,7 @@ public final class Atomizer extends UnaryExpression {
 
     /*@NotNull*/
     public Expression typeCheck(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
-        typeCheckChildren(visitor, contextInfo);
+        getOperand().typeCheck(visitor, contextInfo);
         untyped = untyped | !visitor.getStaticContext().getPackageData().isSchemaAware();
 
         // If the configuration allows typed data, check whether the content type of these particular nodes is untyped
@@ -146,11 +156,12 @@ public final class Atomizer extends UnaryExpression {
         if (th.isSubType(operandType, BuiltInAtomicType.ANY_ATOMIC)) {
             return getBaseExpression();
         }
-        if (!operandType.isAtomizable()) {
+        if (!operandType.isAtomizable(th)) {
             XPathException err;
             if (operandType instanceof FunctionItemType) {
+                String thing = operandType instanceof MapType ? "map" : "function item";
                 err = new XPathException(
-                        "Cannot atomize a function item", "FOTY0013");
+                        "Cannot atomize a " + thing, "FOTY0013");
             } else {
                 err = new XPathException(
                         "Cannot atomize an element that is defined in the schema to have element-only content", "FOTY0012");
@@ -187,7 +198,6 @@ public final class Atomizer extends UnaryExpression {
 
     /**
      * Perform optimisation of an expression and its subexpressions.
-     * <p/>
      * <p>This method is called after all references to functions and variables have been resolved
      * to the declaration of the function or variable, and after all type checking has been done.</p>
      *
@@ -272,13 +282,13 @@ public final class Atomizer extends UnaryExpression {
     /**
      * Determine the special properties of this expression
      *
-     * @return {@link StaticProperty#NON_CREATIVE}.
+     * @return {@link StaticProperty#NO_NODES_NEWLY_CREATED}.
      */
 
     public int computeSpecialProperties() {
         int p = super.computeSpecialProperties();
         p &= ~StaticProperty.NODESET_PROPERTIES;
-        return p | StaticProperty.NON_CREATIVE;
+        return p | StaticProperty.NO_NODES_NEWLY_CREATED;
     }
 
     /**
@@ -317,7 +327,7 @@ public final class Atomizer extends UnaryExpression {
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<? extends Item> iterate(XPathContext context) throws XPathException {
         SequenceIterator base = getBaseExpression().iterate(context);
         return getAtomizingIterator(base, untyped && operandItemType instanceof NodeTest);
     }
@@ -445,7 +455,6 @@ public final class Atomizer extends UnaryExpression {
     /**
      * Add a representation of this expression to a PathMap. The PathMap captures a map of the nodes visited
      * by an expression in a source tree.
-     * <p/>
      * <p>The default implementation of this method assumes that an expression does no navigation other than
      * the navigation done by evaluating its subexpressions, and that the subexpressions are evaluated in the
      * same context as the containing expression. The method must be overridden for any expression
@@ -489,7 +498,7 @@ public final class Atomizer extends UnaryExpression {
      *          if a dynamic evaluation error occurs
      */
 
-    public static SequenceIterator getAtomizingIterator(SequenceIterator base, boolean oneToOne) throws XPathException {
+    public static SequenceIterator<? extends AtomicValue> getAtomizingIterator(SequenceIterator base, boolean oneToOne) throws XPathException {
         int properties = base.getProperties();
         if ((properties & SequenceIterator.LAST_POSITION_FINDER) != 0) {
             int count = ((LastPositionFinder) base).getLength();
@@ -509,6 +518,17 @@ public final class Atomizer extends UnaryExpression {
         }
     }
 
+    public static AtomicSequence atomize(Sequence sequence) throws XPathException {
+        if (sequence instanceof AtomicSequence) {
+            return (AtomicSequence)sequence;
+        } else if (sequence instanceof EmptySequence) {
+            return EmptyAtomicSequence.getInstance();
+        } else {
+            SequenceIterator iter = getAtomizingIterator(sequence.iterate(), false);
+            return new AtomicArray(iter);
+        }
+    }
+
     /**
      * Diagnostic print of expression structure. The abstract expression tree
      * is written to the supplied output destination.
@@ -533,42 +553,6 @@ public final class Atomizer extends UnaryExpression {
         return getBaseExpression().toShortString();
     }
 
-    /**
-     * Implement the mapping function. This is stateless, so there is a singleton instance.
-     */
-
-    public static class AtomizingFunction implements MappingFunction {
-
-        /**
-         * Private constructor, ensuring that everyone uses the singleton instance
-         */
-
-        private AtomizingFunction() {
-        }
-
-        private static final AtomizingFunction theInstance = new AtomizingFunction();
-
-        /**
-         * Get the singleton instance
-         *
-         * @return the singleton instance of this mapping function
-         */
-
-        public static AtomizingFunction getInstance() {
-            return theInstance;
-        }
-
-        public AtomicIterator map(Item item) throws XPathException {
-            if (item instanceof NodeInfo) {
-                return ((NodeInfo) item).atomize().iterate();
-            } else if (item instanceof AtomicValue) {
-                return new SingleAtomicIterator((AtomicValue)item);
-            } else {
-                throw new XPathException("Cannot atomize a function item or external object", "FOTY0013");
-            }
-        }
-    }
-
 
 }
 
