@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -17,20 +17,18 @@ import net.sf.saxon.tree.iter.AxisIterator;
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.type.*;
 import net.sf.saxon.value.Cardinality;
+import net.sf.saxon.z.IntHashSet;
 import net.sf.saxon.z.IntIterator;
 import net.sf.saxon.z.IntSet;
 
-import java.util.HashSet;
-import java.util.Iterator;
-import java.util.Set;
+import java.util.*;
 
 
 /**
  * An AxisExpression is always obtained by simplifying a PathExpression.
- * It represents a PathExpression that starts either at the context node, and uses
+ * It represents a PathExpression that starts at the context node, and uses
  * a simple node-test with no filters. For example "*", "title", "./item",
  * "@*", or "ancestor::chapter*".
- * <p/>
  * <p>An AxisExpression delivers nodes in axis order (not in document order).
  * To get nodes in document order, in the case of a reverse axis, the expression
  * should be wrapped in a call on reverse().</p>
@@ -109,12 +107,17 @@ public final class AxisExpression extends Expression {
 
     /*@NotNull*/
     public Expression typeCheck(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
-        boolean noWarnings = doneOptimize || (doneTypeCheck && this.staticInfo.getItemType().equals(contextInfo.getItemType()));
+        ItemType contextItemType = contextInfo.getItemType();
+        boolean noWarnings = doneOptimize || (doneTypeCheck && this.staticInfo.getItemType().equals(contextItemType));
         doneTypeCheck = true;
-        if (contextInfo.getItemType() == ErrorType.getInstance()) {
-            XPathException err = new XPathException("Axis step " + toString() +
+        if (contextItemType == ErrorType.getInstance()) {
+            // There is no context item. In principle we could raise XPTY0020 ("Context item is not a node"),
+            // which is a type error and therefore can be thrown statically. But many test cases expect
+            // XPDY0002 ("Context item absent") which for inexplicable reasons is a dynamic error rather than
+            // a type error, and therefore cannot be raised until execution time.
+            XPathException err = new XPathException("Axis step " + this +
                                                             " cannot be used here: the context item is absent");
-            err.setIsTypeError(true);
+            //err.setIsTypeError(true);
             err.setErrorCode("XPDY0002");
             err.setLocation(getLocation());
             throw err;
@@ -122,37 +125,44 @@ public final class AxisExpression extends Expression {
             staticInfo = contextInfo;
         }
         Configuration config = visitor.getConfiguration();
-        TypeHierarchy th = config.getTypeHierarchy();
-        int relation = th.relationship(contextInfo.getItemType(), AnyNodeTest.getInstance());
 
-        if (relation == TypeHierarchy.DISJOINT) {
-            XPathException err = new XPathException("Axis step " + toString() +
-                                                            " cannot be used here: the context item is not a node");
-            err.setIsTypeError(true);
-            err.setErrorCode("XPTY0020");
-            err.setLocation(getLocation());
-            throw err;
-        } else if (relation == TypeHierarchy.OVERLAPS || relation == TypeHierarchy.SUBSUMES) {
-            // need to insert a dynamic check of the context item type
-            Expression thisExp = checkPlausibility(visitor, contextInfo, !noWarnings);
-            if (Literal.isEmptySequence(thisExp)) {
-                return thisExp;
+        if (contextItemType.getGenre() != Genre.NODE) {
+            TypeHierarchy th = config.getTypeHierarchy();
+            int relation = th.relationship(contextItemType, AnyNodeTest.getInstance());
+            if (relation == TypeHierarchy.DISJOINT) {
+                XPathException err = new XPathException("Axis step " + this +
+                                                                " cannot be used here: the context item is not a node");
+                err.setIsTypeError(true);
+                err.setErrorCode("XPTY0020");
+                err.setLocation(getLocation());
+                throw err;
+            } else if (relation == TypeHierarchy.OVERLAPS || relation == TypeHierarchy.SUBSUMES) {
+                // need to insert a dynamic check of the context item type
+                Expression thisExp = checkPlausibility(visitor, contextInfo, !noWarnings);
+                if (Literal.isEmptySequence(thisExp)) {
+                    return thisExp;
+                }
+                ContextItemExpression exp = new ContextItemExpression();
+                ExpressionTool.copyLocationInfo(this, exp);
+                RoleDiagnostic role = new RoleDiagnostic(RoleDiagnostic.AXIS_STEP, "", axis);
+                role.setErrorCode("XPTY0020");
+                ItemChecker checker = new ItemChecker(exp, AnyNodeTest.getInstance(), role);
+                ExpressionTool.copyLocationInfo(this, checker);
+                SimpleStepExpression step = new SimpleStepExpression(checker, thisExp);
+                ExpressionTool.copyLocationInfo(this, step);
+                return step;
             }
-            ContextItemExpression exp = new ContextItemExpression();
-            ExpressionTool.copyLocationInfo(this, exp);
-            RoleDiagnostic role = new RoleDiagnostic(RoleDiagnostic.AXIS_STEP, "", axis);
-            role.setErrorCode("XPTY0020");
-            ItemChecker checker = new ItemChecker(exp, AnyNodeTest.getInstance(), role);
-            ExpressionTool.copyLocationInfo(this, checker);
-            SimpleStepExpression step = new SimpleStepExpression(checker, thisExp);
-            ExpressionTool.copyLocationInfo(this, step);
-            return step;
         }
 
-        return checkPlausibility(visitor, contextInfo, !noWarnings);
+        if (visitor.getStaticContext().getOptimizerOptions().isSet(OptimizerOptions.VOID_EXPRESSIONS)) {
+            return checkPlausibility(visitor, contextInfo, !noWarnings);
+        } else {
+            return this;
+        }
     }
 
-    private Expression checkPlausibility(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo, boolean warnings) throws XPathException {
+    private Expression checkPlausibility(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo, boolean warnings)
+            throws XPathException  {
         StaticContext env = visitor.getStaticContext();
         Configuration config = env.getConfiguration();
         ItemType contextType = contextInfo.getItemType();
@@ -225,7 +235,6 @@ public final class AxisExpression extends Expression {
             itemType = new CombinedNodeTest((NodeTest) target, Token.INTERSECT, test);
         }
 
-        // Old code
         int origin = contextType.getPrimitiveType();
 
         if (test != null) {
@@ -234,14 +243,14 @@ public final class AxisExpression extends Expression {
 
             if (contextType instanceof DocumentNodeTest && kind.equals(UType.ELEMENT)) {
                 NodeTest elementTest = ((DocumentNodeTest) contextType).getElementTest();
-                IntSet outermostElementNames = elementTest.getRequiredNodeNames();
-                if (outermostElementNames != null) {
-                    IntSet selectedElementNames = test.getRequiredNodeNames();
-                    if (selectedElementNames != null) {
+                Optional<IntSet> outermostElementNames = elementTest.getRequiredNodeNames();
+                if (outermostElementNames.isPresent()) {
+                    Optional<IntSet> selectedElementNames = test.getRequiredNodeNames();
+                    if (selectedElementNames.isPresent()) {
                         if (axis == AxisInfo.CHILD) {
                             // check that the name appearing in the step is one of the names allowed by the nodetest
 
-                            if (selectedElementNames.intersect(outermostElementNames).isEmpty()) {
+                            if (selectedElementNames.get().intersect(outermostElementNames.get()).isEmpty()) {
                                 if (warnings) {
                                     visitor.issueWarning(
                                             "Starting at a document node, the step is selecting an element whose name " +
@@ -253,8 +262,8 @@ public final class AxisExpression extends Expression {
 
                             if (env.getPackageData().isSchemaAware() &&
                                     elementTest instanceof SchemaNodeTest &&
-                                    outermostElementNames.size() == 1) {
-                                IntIterator oeni = outermostElementNames.iterator();
+                                    outermostElementNames.get().size() == 1) {
+                                IntIterator oeni = outermostElementNames.get().iterator();
                                 int outermostElementName = oeni.hasNext() ? oeni.next() : -1;
                                 SchemaDeclaration decl = config.getElementDeclaration(outermostElementName);
                                 if (decl == null) {
@@ -276,11 +285,11 @@ public final class AxisExpression extends Expression {
 
                         } else if (axis == AxisInfo.DESCENDANT) {
                             // check that the name appearing in the step is one of the names allowed by the nodetest
-                            boolean canMatchOutermost = !selectedElementNames.intersect(outermostElementNames).isEmpty();
+                            boolean canMatchOutermost = !selectedElementNames.get().intersect(outermostElementNames.get()).isEmpty();
                             if (!canMatchOutermost) {
                                 // The expression /descendant::x starting at the document node doesn't match the outermost
                                 // element, so replace it by child::*/descendant::x, and check that
-                                Expression path = ExpressionTool.makePathExpression(new AxisExpression(AxisInfo.CHILD, elementTest), new AxisExpression(AxisInfo.DESCENDANT, test), false);
+                                Expression path = ExpressionTool.makePathExpression(new AxisExpression(AxisInfo.CHILD, elementTest), new AxisExpression(AxisInfo.DESCENDANT, test));
                                 ExpressionTool.copyLocationInfo(this, path);
                                 return path.typeCheck(visitor, contextInfo);
                             }
@@ -374,7 +383,7 @@ public final class AxisExpression extends Expression {
                 // can never be defined as an extension of a type with simple content
                 if (warnings) {
                     visitor.issueWarning("The " + AxisInfo.axisName[axis] + " axis will never select any " +
-                                                 kind.toString() +
+                                                 kind +
                                                  " nodes when starting at " +
                                                  getStartingNodeDescription(contentType) +
                                                  ", as this type requires simple content", getLocation());
@@ -439,13 +448,13 @@ public final class AxisExpression extends Expression {
                 }
             } else if (axis == AxisInfo.CHILD && kind.equals(UType.ELEMENT)) {
                 try {
-                    StructuredQName childElement = targetName;
+                    int childfp = targetfp;
                     if (targetName == null) {
                         // select="child::*"
                         if (((ComplexType) contentType).containsElementWildcard()) {
                             return this;
                         }
-                        HashSet<StructuredQName> children = new HashSet<StructuredQName>();
+                        IntHashSet children = new IntHashSet();
                         ((ComplexType) contentType).gatherAllPermittedChildren(children, false);
                         if (children.isEmpty()) {
                             if (warnings) {
@@ -460,23 +469,27 @@ public final class AxisExpression extends Expression {
 //                                return this;
 //                            }
                         if (children.size() == 1) {
-                            Iterator<StructuredQName> iter = children.iterator();
+                            IntIterator iter = children.iterator();
                             if (iter.hasNext()) {
-                                childElement = iter.next();
+                                childfp = iter.next();
                             }
                         } else {
                             return this;
                         }
                     }
-                    SchemaType schemaType = ((ComplexType) contentType).getElementParticleType(childElement, true);
+                    SchemaType schemaType = ((ComplexType) contentType).getElementParticleType(childfp, true);
                     if (schemaType == null) {
                         if (warnings) {
+                            StructuredQName childElement = getConfiguration().getNamePool().getStructuredQName(childfp);
                             String message = "The complex type " + contentType.getDescription() +
                                     " does not allow a child element named " + getDiagnosticName(childElement, env);
-                            HashSet<StructuredQName> permitted = new HashSet<StructuredQName>();
+                            IntHashSet permitted = new IntHashSet();
                             ((ComplexType) contentType).gatherAllPermittedChildren(permitted, false);
-                            for (StructuredQName sq : permitted) {
-                                if (sq.getLocalPart().equals(childElement.getLocalPart()) && !sq.equals(childElement)) {
+                            IntIterator kids = permitted.iterator();
+                            while (kids.hasNext()) {
+                                int kid = kids.next();
+                                StructuredQName sq = getConfiguration().getNamePool().getStructuredQName(kid);
+                                if (sq.getLocalPart().equals(childElement.getLocalPart()) && kid != childfp) {
                                     message += ". Perhaps the namespace is " +
                                             (childElement.hasURI("") ? "missing" : "wrong") +
                                             ", and " + sq.getEQName() + " was intended?";
@@ -491,11 +504,12 @@ public final class AxisExpression extends Expression {
                                 test,
                                 Token.INTERSECT,
                                 new ContentTypeTest(Type.ELEMENT, schemaType, config, true));
-                        computedCardinality = ((ComplexType) contentType).getElementParticleCardinality(childElement, true);
+                        computedCardinality = ((ComplexType) contentType).getElementParticleCardinality(childfp, true);
                         ExpressionTool.resetStaticProperties(this);
                         if (computedCardinality == StaticProperty.ALLOWS_ZERO) {
                             // this shouldn't happen, because we've already checked for this a different way.
                             // but it's worth being safe (there was a bug involving an incorrect inference here)
+                            StructuredQName childElement = getConfiguration().getNamePool().getStructuredQName(childfp);
                             visitor.issueWarning(
                                     "The complex type " + contentType.getDescription() +
                                             " appears not to allow a child element named " +
@@ -518,19 +532,21 @@ public final class AxisExpression extends Expression {
                 // when searching for a specific element on the descendant axis, try to produce a more
                 // specific path that avoids searching branches of the tree where the element cannot occur
                 try {
-                    HashSet<StructuredQName> descendants = new HashSet<StructuredQName>();
+                    IntHashSet descendants = new IntHashSet();
                     ((ComplexType) contentType).gatherAllPermittedDescendants(descendants);
-                    if (descendants.contains(StandardNames.getStructuredQName(StandardNames.XS_INVALID_NAME))) {
+                    if (descendants.contains(-1)) {
                         return this;
                     }
-                    if (descendants.contains(targetName)) {
-                        HashSet<StructuredQName> children = new HashSet<StructuredQName>();
+                    if (descendants.contains(targetfp)) {
+                        IntHashSet children = new IntHashSet();
                         ((ComplexType) contentType).gatherAllPermittedChildren(children, false);
-                        HashSet<StructuredQName> usefulChildren = new HashSet<StructuredQName>();
+                        IntHashSet usefulChildren = new IntHashSet();
                         boolean considerSelf = false;
                         boolean considerDescendants = false;
-                        for (StructuredQName c : children) {
-                            if (c.equals(targetName)) {
+                        IntIterator kids = children.iterator();
+                        while (kids.hasNext()) {
+                            int c = kids.next();
+                            if (c == targetfp) {
                                 usefulChildren.add(c);
                                 considerSelf = true;
                             }
@@ -539,9 +555,9 @@ public final class AxisExpression extends Expression {
                                 throw new AssertionError("Can't find type for child element " + c);
                             }
                             if (st instanceof ComplexType) {
-                                HashSet<StructuredQName> subDescendants = new HashSet<StructuredQName>();
+                                IntHashSet subDescendants = new IntHashSet();
                                 ((ComplexType) st).gatherAllPermittedDescendants(subDescendants);
-                                if (subDescendants.contains(targetName)) {
+                                if (subDescendants.contains(targetfp)) {
                                     usefulChildren.add(c);
                                     considerDescendants = true;
                                 }
@@ -549,7 +565,7 @@ public final class AxisExpression extends Expression {
                         }
                         itemType = test;
                         if (considerDescendants) {
-                            SchemaType st = ((ComplexType) contentType).getDescendantElementType(targetName);
+                            SchemaType st = ((ComplexType) contentType).getDescendantElementType(targetfp);
                             if (st != AnyType.getInstance()) {
                                 itemType = new CombinedNodeTest(
                                         test, Token.INTERSECT,
@@ -569,7 +585,7 @@ public final class AxisExpression extends Expression {
                             }
                             AxisExpression next = new AxisExpression(nextAxis, (NodeTest) itemType);
                             ExpressionTool.copyLocationInfo(this, next);
-                            Expression path = ExpressionTool.makePathExpression(first, next, false);
+                            Expression path = ExpressionTool.makePathExpression(first, next);
                             ExpressionTool.copyLocationInfo(this, path);
                             return path.typeCheck(visitor, contextInfo);
                         }
@@ -638,10 +654,12 @@ public final class AxisExpression extends Expression {
      * in this set
      */
 
-    private NodeTest makeUnionNodeTest(HashSet<StructuredQName> elements, NamePool pool) {
+    private NodeTest makeUnionNodeTest(IntHashSet elements, NamePool pool) {
         NodeTest test = null;
-        for (StructuredQName fp : elements) {
-            NodeTest nextTest = new NameTest(Type.ELEMENT, fp.getURI(), fp.getLocalPart(), pool);
+        IntIterator iter = elements.iterator();
+        while (iter.hasNext()) {
+            int fp = iter.next();
+            NodeTest nextTest = new NameTest(Type.ELEMENT, fp, pool);
             if (test == null) {
                 test = nextTest;
             } else {
@@ -664,7 +682,6 @@ public final class AxisExpression extends Expression {
 
     /**
      * Perform optimisation of an expression and its subexpressions.
-     * <p/>
      * <p>This method is called after all references to functions and variables have been resolved
      * to the declaration of the function or variable, and after all type checking has been done.</p>
      *
@@ -715,23 +732,16 @@ public final class AxisExpression extends Expression {
      */
 
     public boolean equals(Object other) {
-        if (!(other instanceof AxisExpression)) {
-            return false;
-        }
-        if (axis != ((AxisExpression) other).axis) {
-            return false;
-        }
-        if (test == null) {
-            return ((AxisExpression) other).test == null;
-        }
-        return test.toString().equals(((AxisExpression) other).test.toString());
+        return other instanceof AxisExpression &&
+                axis == ((AxisExpression) other).axis &&
+                Objects.equals(test, ((AxisExpression) other).test);
     }
 
     /**
      * get HashCode for comparing two expressions
      */
 
-    public int hashCode() {
+    public int computeHashCode() {
         // generate an arbitrary hash code that depends on the axis and the node test
         int h = 9375162 + axis << 20;
         if (test != null) {
@@ -741,17 +751,6 @@ public final class AxisExpression extends Expression {
         return h;
     }
 
-    /**
-     * Determine which aspects of the context the expression depends on. The result is
-     * a bitwise-or'ed value composed from constants such as XPathContext.VARIABLES and
-     * XPathContext.CURRENT_NODE
-     */
-
-//    public int getIntrinsicDependencies() {
-//	    return StaticProperty.DEPENDS_ON_CONTEXT_ITEM;
-//    }
-
-
     /**
      * Copy an expression. This makes a deep copy.
      *
@@ -782,7 +781,7 @@ public final class AxisExpression extends Expression {
     public int computeSpecialProperties() {
         return StaticProperty.CONTEXT_DOCUMENT_NODESET |
                 StaticProperty.SINGLE_DOCUMENT_NODESET |
-                StaticProperty.NON_CREATIVE |
+                StaticProperty.NO_NODES_NEWLY_CREATED |
                 (AxisInfo.isForwards[axis] ? StaticProperty.ORDERED_NODESET : StaticProperty.REVERSE_DOCUMENT_ORDER) |
                 (AxisInfo.isPeerAxis[axis] || isPeerNodeTest(test) ? StaticProperty.PEER_NODESET : 0) |
                 (AxisInfo.isSubtreeAxis[axis] ? StaticProperty.SUBTREE_NODESET : 0) |
@@ -910,7 +909,7 @@ public final class AxisExpression extends Expression {
             SchemaType contentType = originNodeType.getContentType();
             if (contentType instanceof ComplexType) {
                 try {
-                    return ((ComplexType) contentType).getDescendantElementCardinality(nodeTest.getMatchingNodeName());
+                    return ((ComplexType) contentType).getDescendantElementCardinality(nodeTest.getFingerprint());
                 } catch (SchemaException err) {
                     // shouldn't happen; play safe
                     return StaticProperty.ALLOWS_ZERO_OR_MORE;
@@ -998,12 +997,11 @@ public final class AxisExpression extends Expression {
      * Convert this expression to an equivalent XSLT pattern
      *
      * @param config the Saxon configuration
-     * @param is30   true if this is XSLT 3.0
      * @return the equivalent pattern
      * @throws net.sf.saxon.trans.XPathException if conversion is not possible
      */
     @Override
-    public Pattern toPattern(Configuration config, boolean is30) throws XPathException {
+    public Pattern toPattern(Configuration config) throws XPathException {
 
         NodeTest test = getNodeTest();
         Pattern pat;
@@ -1062,12 +1060,12 @@ public final class AxisExpression extends Expression {
      */
 
     /*@NotNull*/
-    public AxisIterator iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<? extends Item> iterate(XPathContext context) throws XPathException {
         Item item = context.getContextItem();
         if (item == null) {
             // Might as well do the test anyway, whether or not contextMaybeUndefined is set
             XPathException err = new XPathException("The context item for axis step " +
-                                                            toString() + " is absent");
+                                                            this + " is absent");
             err.setErrorCode("XPDY0002");
             err.setXPathContext(context);
             err.setLocation(getLocation());
@@ -1082,7 +1080,7 @@ public final class AxisExpression extends Expression {
             }
         } catch (ClassCastException cce) {
             XPathException err = new XPathException("The context item for axis step " +
-                                                            toString() + " is not a node");
+                                                            this + " is not a node");
             err.setErrorCode("XPTY0020");
             err.setXPathContext(context);
             err.setLocation(getLocation());
@@ -1126,14 +1124,14 @@ public final class AxisExpression extends Expression {
         destination.startElement("axis", this);
         destination.emitAttribute("name", AxisInfo.axisName[axis]);
         destination.emitAttribute("nodeTest", test == null ? "node()" : test.toString());
-        if ("JS".equals(destination.getOption("target"))) {
+        if ("JS".equals(((ExpressionPresenter.ExportOptions) destination.getOptions()).target)) {
             NodeTest known = AnyNodeTest.getInstance();
             if (axis == AxisInfo.ATTRIBUTE) {
                 known = NodeKindTest.makeNodeKindTest(Type.ATTRIBUTE);
             } else if (axis == AxisInfo.NAMESPACE) {
                 known = NodeKindTest.makeNodeKindTest(Type.NAMESPACE);
             }
-            int targetVersion = destination.getIntOption("targetVersion", 1);
+            int targetVersion = ((ExpressionPresenter.ExportOptions) destination.getOptions()).targetVersion;
             try {
                 destination.emitAttribute("jsTest", test == null ? "return true;" : test.generateJavaScriptItemTypeTest(known, targetVersion));
             } catch (XPathException e) {
@@ -1197,7 +1195,7 @@ public final class AxisExpression extends Expression {
      * @return A set of conditions, or null if none have been computed
      */
     public Set<Expression> getPreconditions() {
-        HashSet<Expression> pre = new HashSet<Expression>(1);
+        HashSet<Expression> pre = new HashSet<>(1);
         /*Expression args[] = new Expression[1];
         args[0] = this.copy();
         pre.add(SystemFunctionCall.makeSystemFunction(
