@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,7 +7,7 @@
 
 package net.sf.saxon.expr;
 
-import net.sf.saxon.event.SequenceReceiver;
+import net.sf.saxon.event.Receiver;
 import net.sf.saxon.event.TypeCheckingFilter;
 import net.sf.saxon.expr.parser.*;
 import net.sf.saxon.om.Item;
@@ -96,7 +96,7 @@ public final class CardinalityChecker extends UnaryExpression {
 
     /*@NotNull*/
     public Expression typeCheck(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
-        typeCheckChildren(visitor, contextInfo);
+        getOperand().typeCheck(visitor, contextInfo);
         Expression base = getBaseExpression();
         if (requiredCardinality == StaticProperty.ALLOWS_ZERO_OR_MORE ||
                 Cardinality.subsumes(requiredCardinality, base.getCardinality())) {
@@ -107,7 +107,6 @@ public final class CardinalityChecker extends UnaryExpression {
 
     /**
      * Perform optimisation of an expression and its subexpressions.
-     * <p/>
      * <p>This method is called after all references to functions and variables have been resolved
      * to the declaration of the function or variable, and after all type checking has been done.</p>
      *
@@ -123,7 +122,7 @@ public final class CardinalityChecker extends UnaryExpression {
 
     /*@NotNull*/
     public Expression optimize(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
-        optimizeChildren(visitor, contextInfo);
+        getOperand().optimize(visitor, contextInfo);
         Expression base = getBaseExpression();
         if (requiredCardinality == StaticProperty.ALLOWS_ZERO_OR_MORE ||
                 Cardinality.subsumes(requiredCardinality, base.getCardinality())) {
@@ -185,9 +184,6 @@ public final class CardinalityChecker extends UnaryExpression {
         return m;
     }
 
-
-
-
     /**
      * For an expression that returns an integer or a sequence of integers, get
      * a lower and upper bound on the values of the integers that may be returned, from
@@ -211,8 +207,8 @@ public final class CardinalityChecker extends UnaryExpression {
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
-        SequenceIterator base = getBaseExpression().iterate(context);
+    public SequenceIterator<? extends Item> iterate(XPathContext context) throws XPathException {
+        SequenceIterator<? extends Item> base = getBaseExpression().iterate(context);
 
         // If the base iterator knows how many items there are, then check it now rather than wasting time
 
@@ -234,7 +230,7 @@ public final class CardinalityChecker extends UnaryExpression {
 
         // Otherwise return an iterator that does the checking on the fly
 
-        return new CardinalityCheckingIterator(base, requiredCardinality, role, getLocation());
+        return new CardinalityCheckingIterator<>(base, requiredCardinality, role, getLocation());
 
     }
 
@@ -295,7 +291,7 @@ public final class CardinalityChecker extends UnaryExpression {
             if (second != null) {
                 Item[] leaders = new Item[]{first, second};
                 typeError("A sequence of more than one item is not allowed as the " +
-                    role.getMessage() + depictSequenceStart(new ArrayIterator(leaders), 2), role.getErrorCode(), context);
+                    role.getMessage() + depictSequenceStart(new ArrayIterator<Item>(leaders), 2), role.getErrorCode(), context);
                 return null;
             }
         }
@@ -317,7 +313,7 @@ public final class CardinalityChecker extends UnaryExpression {
             next = ((ItemChecker) next).getBaseExpression();
         }
         if ((next.getImplementationMethod() & PROCESS_METHOD) != 0 && !(type instanceof DocumentNodeTest)) {
-            SequenceReceiver out = context.getReceiver();
+            Receiver out = context.getReceiver();
             TypeCheckingFilter filter = new TypeCheckingFilter(out);
             filter.setRequiredType(type, requiredCardinality, role, getLocation());
             context.setReceiver(filter);
@@ -330,6 +326,7 @@ public final class CardinalityChecker extends UnaryExpression {
             }
             context.setReceiver(out);
         } else {
+            // Force pull-mode evaluation
             super.process(context);
         }
     }
@@ -368,7 +365,7 @@ public final class CardinalityChecker extends UnaryExpression {
      * Copy an expression. This makes a deep copy.
      *
      * @return the copy of the original expression
-     * @param rebindings
+     * @param rebindings variable bindings that need to be changed
      */
 
     /*@NotNull*/
@@ -393,8 +390,8 @@ public final class CardinalityChecker extends UnaryExpression {
      */
 
     @Override
-    public int hashCode() {
-        return super.hashCode() ^ requiredCardinality;
+    public int computeHashCode() {
+        return super.computeHashCode() ^ requiredCardinality;
     }
 
     /**
@@ -424,15 +421,15 @@ public final class CardinalityChecker extends UnaryExpression {
         Expression operand = getBaseExpression();
         switch (requiredCardinality) {
             case StaticProperty.ALLOWS_ONE:
-                return "exactly-one(" + operand.toString() + ")";
+                return "exactly-one(" + operand + ")";
             case StaticProperty.ALLOWS_ZERO_OR_ONE:
-                return "zero-or-one(" + operand.toString() + ")";
+                return "zero-or-one(" + operand + ")";
             case StaticProperty.ALLOWS_ONE_OR_MORE:
-                return "one-or-more(" + operand.toString() + ")";
+                return "one-or-more(" + operand + ")";
             case StaticProperty.EMPTY:
-                return "must-be-empty(" + operand.toString() + ")";
+                return "must-be-empty(" + operand + ")";
             default:
-                return "check(" + operand.toString() + ")";
+                return "check(" + operand + ")";
         }
     }
 
