@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -24,14 +24,12 @@ import net.sf.saxon.value.Cardinality;
  * the checks; after that we can simply return the items from the base sequence.
  */
 
-public final class CardinalityCheckingIterator implements SequenceIterator {
+public final class CardinalityCheckingIterator<T extends Item> implements SequenceIterator<T> {
 
-    private SequenceIterator base;
-    private int requiredCardinality;
-    private RoleDiagnostic role;
+    private SequenceIterator<T> base;
     private Location locator;
-    /*@Nullable*/ private Item first = null;
-    private Item second = null;
+    /*@Nullable*/ private T first = null;
+    private T second = null;
     private int position = 0;
 
     /**
@@ -42,12 +40,10 @@ public final class CardinalityCheckingIterator implements SequenceIterator {
      * @param requiredCardinality the required Cardinality
      */
 
-    public CardinalityCheckingIterator(SequenceIterator base, int requiredCardinality,
+    public CardinalityCheckingIterator(SequenceIterator<T> base, int requiredCardinality,
                                        RoleDiagnostic role, Location locator)
             throws XPathException {
         this.base = base;
-        this.requiredCardinality = requiredCardinality;
-        this.role = role;
         this.locator = locator;
         first = base.next();
         if (first == null) {
@@ -62,7 +58,7 @@ public final class CardinalityCheckingIterator implements SequenceIterator {
             }
             second = base.next();
             if (second != null && !Cardinality.allowsMany(requiredCardinality)) {
-                Item[] leaders = new Item[]{first, second};
+                T[] leaders = (T[])new Item[]{first, second};
                 typeError("A sequence of more than one item is not allowed as the " +
                         role.getMessage() + CardinalityChecker.depictSequenceStart(new ArrayIterator(leaders), 2),
                         role.getErrorCode());
@@ -70,14 +66,14 @@ public final class CardinalityCheckingIterator implements SequenceIterator {
         }
     }
 
-    public Item next() throws XPathException {
+    public T next() throws XPathException {
         if (position < 2) {
             if (position == 0) {
-                Item current = first;
+                T current = first;
                 position = first == null ? -1 : 1;
                 return current;
             } else if (position == 1) {
-                Item current = second;
+                T current = second;
                 position = second == null ? -1 : 2;
                 return current;
             } else {
@@ -85,7 +81,7 @@ public final class CardinalityCheckingIterator implements SequenceIterator {
                 return null;
             }
         }
-        Item current = base.next();
+        T current = base.next();
         if (current == null) {
             position = -1;
         } else {
