@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -24,8 +24,7 @@ import net.sf.saxon.value.*;
  * Cast Expression: implements "cast as data-type ( expression )". It also allows an internal
  * cast, which has the same semantics as a user-requested cast, but maps an empty sequence to
  * an empty sequence.
- * <p/>
- * This expression class does not handle casting to a list or union type.
+ * <p>This expression class does not handle casting to a list or union type.</p>
  */
 
 public class CastExpression extends CastingExpression implements Callable {
@@ -49,7 +48,7 @@ public class CastExpression extends CastingExpression implements Callable {
 
     /*@NotNull*/
     public Expression typeCheck(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
-        typeCheckChildren(visitor, contextInfo);
+        getOperand().typeCheck(visitor, contextInfo);
         SequenceType atomicType = SequenceType.makeSequenceType(BuiltInAtomicType.ANY_ATOMIC, getCardinality());
 
         Configuration config = visitor.getConfiguration();
@@ -114,7 +113,6 @@ public class CastExpression extends CastingExpression implements Callable {
 
     /**
      * Perform optimisation of an expression and its subexpressions.
-     * <p/>
      * <p>This method is called after all references to functions and variables have been resolved
      * to the declaration of the function or variable, and after all type checking has been done.</p>
      *
@@ -228,9 +226,7 @@ public class CastExpression extends CastingExpression implements Callable {
                 }
                 throw new XPathException(err.getMessage(), code, this.getLocation());
             } else {
-                Expression lit = Literal.makeLiteral((AtomicValue) result);
-                ExpressionTool.copyLocationInfo(this, lit);
-                return lit;
+                return Literal.makeLiteral((AtomicValue) result, this);
             }
         }
         if (literalOperand.getLength() == 0) {
@@ -256,10 +252,9 @@ public class CastExpression extends CastingExpression implements Callable {
 
     /**
      * Get the static type of the expression
-     * <p/>
-     * <p/>
-     * /*@NotNull
      */
+
+    /*@NotNull*/
     public ItemType getItemType() {
         return getTargetType();
     }
@@ -306,7 +301,7 @@ public class CastExpression extends CastingExpression implements Callable {
     /*@Nullable*/
     @Override
     public IntegerValue[] getIntegerBounds() {
-        if (converter == Converter.BOOLEAN_TO_INTEGER) {
+        if (converter == Converter.BooleanToInteger.INSTANCE) {
             return new IntegerValue[]{Int64Value.ZERO, Int64Value.PLUS_ONE};
         } else {
             return null;
@@ -408,9 +403,9 @@ public class CastExpression extends CastingExpression implements Callable {
 
     public boolean equals(Object other) {
         return other instanceof CastExpression &&
-                getBaseExpression().equals(((CastExpression) other).getBaseExpression()) &&
+                getBaseExpression().isEqual(((CastExpression) other).getBaseExpression()) &&
                 getTargetType() == ((CastExpression) other).getTargetType() &&
-                getTargetType() == ((CastExpression) other).getTargetType();
+                allowsEmpty() == ((CastExpression) other).allowsEmpty();
     }
 
     /**
@@ -419,8 +414,8 @@ public class CastExpression extends CastingExpression implements Callable {
      */
 
     @Override
-    public int hashCode() {
-        return super.hashCode() ^ getTargetType().hashCode();
+    public int computeHashCode() {
+        return super.computeHashCode() ^ getTargetType().hashCode();
     }
 
     /**
@@ -446,7 +441,7 @@ public class CastExpression extends CastingExpression implements Callable {
 
     public void export(ExpressionPresenter out) throws XPathException {
         out.startElement("cast", this);
-        out.emitAttribute("as", getTargetType().getTypeName());
+        out.emitAttribute("as", getTargetType().toExportString());
         out.emitAttribute("emptiable", allowsEmpty() ? "1" : "0");
         getBaseExpression().export(out);
         out.endElement();
