@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -93,6 +93,7 @@ public class ContextItemExpression extends Expression {
         if (contextInfo.getItemType() == ErrorType.getInstance()) {
             visitor.issueWarning("Evaluation will always fail: there is no context item", getLocation());
             ErrorExpression ee = new ErrorExpression("There is no context item", getErrorCodeForUndefinedContext(), true);
+            ee.setOriginalExpression(this);
             ExpressionTool.copyLocationInfo(this, ee);
             return ee;
         } else {
@@ -103,7 +104,6 @@ public class ContextItemExpression extends Expression {
 
     /**
      * Perform optimisation of an expression and its subexpressions.
-     * <p/>
      * <p>This method is called after all references to functions and variables have been resolved
      * to the declaration of the function or variable, and after all type checking has been done.</p>
      *
@@ -174,12 +174,12 @@ public class ContextItemExpression extends Expression {
     /**
      * Determine the special properties of this expression
      *
-     * @return the value {@link StaticProperty#NON_CREATIVE}
+     * @return the value {@link StaticProperty#NO_NODES_NEWLY_CREATED}
      */
 
     public int computeSpecialProperties() {
         int p = super.computeSpecialProperties();
-        return p | StaticProperty.NON_CREATIVE | StaticProperty.CONTEXT_DOCUMENT_NODESET;
+        return p | StaticProperty.NO_NODES_NEWLY_CREATED | StaticProperty.CONTEXT_DOCUMENT_NODESET;
     }
 
     @Override
@@ -192,14 +192,14 @@ public class ContextItemExpression extends Expression {
      */
 
     public boolean equals(Object other) {
-        return (other instanceof ContextItemExpression);
+        return other instanceof ContextItemExpression;
     }
 
     /**
      * get HashCode for comparing two expressions
      */
 
-    public int hashCode() {
+    public int computeHashCode() {
         return "ContextItemExpression".hashCode();
     }
 
@@ -211,7 +211,6 @@ public class ContextItemExpression extends Expression {
     /**
      * Add a representation of this expression to a PathMap. The PathMap captures a map of the nodes visited
      * by an expression in a source tree.
-     * <p/>
      * <p>The default implementation of this method assumes that an expression does no navigation other than
      * the navigation done by evaluating its subexpressions, and that the subexpressions are evaluated in the
      * same context as the containing expression. The method must be overridden for any expression
@@ -258,13 +257,12 @@ public class ContextItemExpression extends Expression {
      * Convert this expression to an equivalent XSLT pattern
      *
      * @param config the Saxon configuration
-     * @param is30   true if this is XSLT 3.0
      * @return the equivalent pattern
      * @throws net.sf.saxon.trans.XPathException
      *          if conversion is not possible
      */
     @Override
-    public Pattern toPattern(Configuration config, boolean is30) throws XPathException {
+    public Pattern toPattern(Configuration config) throws XPathException {
         return AnchorPattern.getInstance();
     }
 
@@ -284,7 +282,7 @@ public class ContextItemExpression extends Expression {
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<? extends Item> iterate(XPathContext context) throws XPathException {
         Item item = context.getContextItem();
         if (item == null) {
             dynamicError("The context item is absent", getErrorCodeForUndefinedContext(), context);
