@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,7 +9,7 @@ package net.sf.saxon.expr;
 
 import net.sf.saxon.Configuration;
 import net.sf.saxon.Controller;
-import net.sf.saxon.event.SequenceReceiver;
+import net.sf.saxon.event.Receiver;
 import net.sf.saxon.expr.instruct.ParameterSet;
 import net.sf.saxon.expr.sort.GroupIterator;
 import net.sf.saxon.lib.UnfailingErrorListener;
@@ -17,8 +17,8 @@ import net.sf.saxon.om.*;
 import net.sf.saxon.regex.RegexIterator;
 import net.sf.saxon.trace.InstructionInfo;
 import net.sf.saxon.trans.NoDynamicContextException;
-import net.sf.saxon.trans.rules.Rule;
 import net.sf.saxon.trans.XPathException;
+import net.sf.saxon.trans.rules.Rule;
 import net.sf.saxon.value.CalendarValue;
 import net.sf.saxon.value.DateTimeValue;
 
@@ -98,14 +98,6 @@ public class EarlyEvaluationContext implements XPathContext {
         return null;
     }
 
-    /**
-     * Get a named collation
-     */
-
-//    public StringCollator getCollation(String name) throws XPathException {
-//        return collationMap.getNamedCollation(name);
-//    }
-
     /**
      * Get the Configuration
      */
@@ -165,7 +157,7 @@ public class EarlyEvaluationContext implements XPathContext {
      *         (which means the context item, position, and size are undefined).
      */
 
-    public FocusTrackingIterator getCurrentIterator() {
+    public FocusTrackingIterator<? extends Item> getCurrentIterator() {
         return null;
     }
 
@@ -255,7 +247,7 @@ public class EarlyEvaluationContext implements XPathContext {
      *
      * @return the current Receiver
      */
-    public SequenceReceiver getReceiver() {
+    public Receiver getReceiver() {
         notAllowed();
         return null;
     }
@@ -340,10 +332,16 @@ public class EarlyEvaluationContext implements XPathContext {
      * Set a new sequence iterator.
      */
 
-    public void setCurrentIterator(FocusIterator iter) {
+    public void setCurrentIterator(FocusIterator<? extends Item> iter) {
         notAllowed();
     }
 
+    @Override
+    public <T extends Item> FocusIterator<T> trackFocus(SequenceIterator<T> iter) {
+        notAllowed();
+        return null;
+    }
+
     /**
      * Set the value of a local variable, identified by its slot number
      */
@@ -356,7 +354,7 @@ public class EarlyEvaluationContext implements XPathContext {
      * Change the Receiver to which output is written
      */
 
-    public void setReceiver(SequenceReceiver receiver) {
+    public void setReceiver(Receiver receiver) {
         notAllowed();
     }
 
@@ -365,8 +363,8 @@ public class EarlyEvaluationContext implements XPathContext {
      * If a parameter of the relevant name was supplied, it is bound to the xsl:param element.
      * Otherwise the method returns false, so the xsl:param default will be evaluated
      *
-     * @param parameterId
-     * @param slotNumber
+     * @param parameterId Globally-unique parameter identifier
+     * @param slotNumber  Slot number of the parameter within the stack frame of the called template
      * @param isTunnel    True if a tunnel parameter is required, else false  @return true if a parameter of this name was supplied, false if not
      */
 
