@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,7 +8,8 @@
 package net.sf.saxon.expr;
 
 import net.sf.saxon.Configuration;
-import net.sf.saxon.event.SequenceReceiver;
+import net.sf.saxon.event.Receiver;
+import net.sf.saxon.event.ReceiverOptions;
 import net.sf.saxon.expr.parser.*;
 import net.sf.saxon.functions.KeyFn;
 import net.sf.saxon.functions.SuperId;
@@ -22,7 +23,9 @@ import net.sf.saxon.pattern.Pattern;
 import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trace.LocationKind;
 import net.sf.saxon.trans.XPathException;
+import net.sf.saxon.tree.iter.EmptyIterator;
 import net.sf.saxon.tree.iter.SingletonIterator;
+import net.sf.saxon.tree.jiter.MonoIterator;
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.type.AtomicType;
 import net.sf.saxon.type.ItemType;
@@ -94,7 +97,7 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
      */
 
     public String getExpressionName() {
-        return getClass().getName();
+        return getClass().getSimpleName();
     }
 
     /**
@@ -180,7 +183,7 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
      */
 
     protected List<Operand> operandSparseList(Operand... a) {
-        List<Operand> operanda = new ArrayList<Operand>();
+        List<Operand> operanda = new ArrayList<>();
         for (Operand o : a) {
             if (o != null) {
                 operanda.add(o);
@@ -214,6 +217,7 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
     /**
      * Verify that parent pointers are correct throughout the subtree rooted at this expression
      * @throws IllegalStateException if invalid parent pointers are found
+     * @return the expression
      */
 
     public Expression verifyParentPointers() throws IllegalStateException {
@@ -231,25 +235,11 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
                 throw new IllegalStateException("Incorrect parent pointer in " + parent.toShortString() +
                                                         " subexpression " + o.getChildExpression().toShortString());
             }
-            if (o.getChildExpression().getRetainedStaticContext() == null) {
-                throw new IllegalStateException("No retained static context in " + parent.toShortString() +
-                                                        " subexpression " + o.getChildExpression().toShortString());
-            }
             o.getChildExpression().verifyParentPointers();
         }
         return this;
     }
 
-    public void verifyParentPointers(String message) {
-        //System.err.println("Verify: " + message);
-        try {
-            verifyParentPointers();
-        } catch (IllegalStateException err) {
-            System.err.println("Verify failed at: " + message + "(" + err.getMessage() + ")");
-            //throw err;
-        }
-    }
-
     /**
      * Restore parent pointers for the subtree rooted at this expression
      */
@@ -299,15 +289,16 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
     /**
      * Ask whether the expression can be lifted out of a loop, assuming it has no dependencies
      * on the controlling variable/focus of the loop
-     * @param forStreaming true if the expression is being optimized for streaming
+     * @param forStreaming true if we are optimizing for streamed evaluation
+     * @return true if the expression can be loop lifted
      */
 
     public boolean isLiftable(boolean forStreaming) {
         int p = getSpecialProperties();
         int d = getDependencies();
-        return (p & StaticProperty.NON_CREATIVE) != 0 && (p & StaticProperty.HAS_SIDE_EFFECTS) == 0
+        return (p & StaticProperty.NO_NODES_NEWLY_CREATED) != 0 && (p & StaticProperty.HAS_SIDE_EFFECTS) == 0
                 && ((d & StaticProperty.DEPENDS_ON_ASSIGNABLE_GLOBALS) == 0)
-                && ((d & StaticProperty.DEPENDS_ON_POSITION) == 0)
+                && ((d & StaticProperty.DEPENDS_ON_POSITION) == 0)   // bug 3758
                 && ((d & StaticProperty.DEPENDS_ON_LAST) == 0);
     }
 
@@ -449,6 +440,7 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
     /**
      * Set the parts of the static context that might be needed by the function, without
      * passing them on to subexpressions. Used for dynamic function calls only.
+     * @param rsc the retained static context
      */
 
     public void setRetainedStaticContextLocally(RetainedStaticContext rsc) {
@@ -465,8 +457,9 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
     public final RetainedStaticContext getRetainedStaticContext() {
         if (retainedStaticContext == null) {
             Expression parent = getParentExpression();
-            return parent == null ? null : parent.getRetainedStaticContext();
-            //setRetainedStaticContextLocally(parent == null ? null : parent.getRetainedStaticContext());
+            assert parent != null;
+            retainedStaticContext = parent.getRetainedStaticContext();
+            assert retainedStaticContext != null;
         }
         return retainedStaticContext;
     }
@@ -489,6 +482,7 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
      * Get the saved static base URI as a URI
      *
      * @return the static base URI as a URI
+     * @throws XPathException if the static base URI is not a valid URI
      */
 
     public URI getStaticBaseURI() throws XPathException {
@@ -498,34 +492,23 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
     /**
      * Ask whether this expression is a call on a particular function
      * @param function the implementation class of the function in question
+     * @return true if the expression is a call on the function
      */
 
     public boolean isCallOn(Class<? extends SystemFunction> function) {
         return false;
     }
 
-    /**
-     * Ask whether this expression is a call on a particular function
-     * @param localName the name of the function in question, in the standard fn namespace
-     */
-
-    public boolean isCallOnSystemFunction(String localName) {
-        return false;
-    }
-
     /**
      * Perform type checking of an expression and its subexpressions. This is the second phase of
      * static optimization.
-     * <p/>
      * <p>This checks statically that the operands of the expression have
      * the correct type; if necessary it generates code to do run-time type checking or type
      * conversion. A static type error is reported only if execution cannot possibly succeed, that
      * is, if a run-time type error is inevitable. The call may return a modified form of the expression.</p>
-     * <p/>
      * <p>This method is called after all references to functions and variables have been resolved
      * to the declaration of the function or variable. However, the types of such functions and
      * variables may not be accurately known if they have not been explicitly declared.</p>
-     * <p/>
      *
      *
      * @param visitor         an expression visitor
@@ -591,7 +574,6 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
     /**
      * Perform optimisation of an expression and its subexpressions. This is the third and final
      * phase of static optimization.
-     * <p/>
      * <p>This method is called after all references to functions and variables have been resolved
      * to the declaration of the function or variable, and after all type checking has been done.</p>
      *
@@ -636,6 +618,10 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
         }
     }
 
+    public void prepareForStreaming() throws XPathException {
+        // no action by default
+    }
+
     /**
      * Return the estimated cost of evaluating an expression. This is a very crude measure based
      * on the syntactic form of the expression (we have no knowledge of data values). We take
@@ -701,6 +687,16 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
         return staticProperties & StaticProperty.SPECIAL_PROPERTY_MASK;
     }
 
+    /**
+     * Ask whether a particular "special property" is present for this expression
+     * @param property the property in question, as an integer code in the {@link StaticProperty} class
+     * @return true if the given property is present
+     */
+
+    public boolean hasSpecialProperty(int property) {
+        return (getSpecialProperties() & property) != 0;
+    }
+
     /**
      * Determine the static cardinality of the expression. This establishes how many items
      * there will be in the result of the expression, at compile time (i.e., without
@@ -725,7 +721,6 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
      * sequences, in general; this method determines the type of the items within the
      * sequence, assuming that (a) this is known in advance, and (b) it is the same for
      * all items in the sequence.
-     * <p/>
      * <p>This method should always return a result, though it may be the best approximation
      * that is available at the time.</p>
      *
@@ -737,7 +732,9 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
     public abstract ItemType getItemType();
 
     /**
-     * Get the static type of the expression as a SequenceType
+     * Get the static type of the expression as a SequenceType: this is the combination of the
+     * item type and the cardinality.
+     * @return the static type of the expression
      */
 
     public SequenceType getStaticType() {
@@ -865,9 +862,9 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<? extends Item> iterate(XPathContext context) throws XPathException {
         Item value = evaluateItem(context);
-        return SingletonIterator.makeIterator(value);
+        return value == null ? EmptyIterator.emptyIterator() : SingletonIterator.rawIterator(value);
     }
 
     /**
@@ -883,7 +880,13 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
      */
 
     public boolean effectiveBooleanValue(XPathContext context) throws XPathException {
-        return ExpressionTool.effectiveBooleanValue(iterate(context));
+        try {
+            return ExpressionTool.effectiveBooleanValue(iterate(context));
+        } catch (XPathException e) {
+            e.maybeSetFailingExpression(this);
+            e.maybeSetContext(context);
+            throw e;
+        }
     }
 
     /**
@@ -928,18 +931,14 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
         if ((m & EVALUATE_METHOD) != 0) {
             Item item = evaluateItem(context);
             if (item != null) {
-                context.getReceiver().append(item, getLocation(), NodeInfo.ALL_NAMESPACES);
+                context.getReceiver().append(item, getLocation(), ReceiverOptions.ALL_NAMESPACES);
             }
 
         } else if ((m & ITERATE_METHOD) != 0) {
 
-            SequenceIterator iter = iterate(context);
-            SequenceReceiver out = context.getReceiver();
+            Receiver out = context.getReceiver();
             try {
-                Item it;
-                while ((it = iter.next()) != null) {
-                    out.append(it, getLocation(), NodeInfo.ALL_NAMESPACES);
-                }
+                iterate(context).forEachOrFail(it -> out.append(it, getLocation(), ReceiverOptions.ALL_NAMESPACES));
             } catch (XPathException e) {
                 e.maybeSetLocation(getLocation());
                 e.maybeSetContext(context);
@@ -974,13 +973,11 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
     /**
      * <p>The toString() method for an expression attempts to give a representation of the expression
      * in an XPath-like form.</p>
-     * <p/>
      * <p>For subclasses of Expression that represent XPath expressions, the result should always be a string that
      * parses as an XPath 3.0 expression. The expression produced should be equivalent to the original making certain
      * assumptions about the static context. In general the expansion will make no assumptions about namespace bindings,
      * except that (a) the prefix "xs" is used to refer to the XML Schema namespace, and (b) the default function namespace
      * is assumed to be the "fn" namespace.</p>
-     * <p/>
      * <p>In the case of XSLT instructions and XQuery expressions, the toString() method gives an abstracted view of the syntax
      * that is not designed in general to be parseable.</p>
      *
@@ -1075,13 +1072,12 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
 
     /**
      * Set up a parent-child relationship between this expression and a given child expression.
-     * <p/>
-     * Note: many calls on this method are now redundant, but are kept in place for "belt-and-braces"
+     * <p>Note: many calls on this method are now redundant, but are kept in place for "belt-and-braces"
      * reasons. The rule is that an implementation of simplify(), typeCheck(), or optimize() that returns
      * a value other than "this" is required to set the location information and parent pointer in the new
      * child expression. However, in the past this was often left to the caller, which did it by calling
      * this method, either unconditionally on return from one of these methods, or after testing that the
-     * returned object was not the same as the original.
+     * returned object was not the same as the original.</p>
      *
      * @param child the child expression
      */
@@ -1131,8 +1127,7 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
 
     public final Location getLocation() {
         if ((location == null || location == ExplicitLocation.UNKNOWN_LOCATION) && getParentExpression() != null) {
-            // Only go up one level to avoid risk of infinite recursion: see bug 3703#1
-            return getParentExpression().location;
+            return getParentExpression().location;  // Only go up one level to avoid infinite recursion: bug 3703#1
         } else {
             return location;
         }
@@ -1198,6 +1193,7 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
 
     public void resetLocalStaticProperties() {
         staticProperties = -1;
+        cachedHashCode = -1;
     }
 
     /**
@@ -1214,7 +1210,9 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
      *
      * @return the computed cardinality, as one of the values {@link StaticProperty#ALLOWS_ZERO_OR_ONE},
      *         {@link StaticProperty#EXACTLY_ONE}, {@link StaticProperty#ALLOWS_ONE_OR_MORE},
-     *         {@link StaticProperty#ALLOWS_ZERO_OR_MORE}
+     *         {@link StaticProperty#ALLOWS_ZERO_OR_MORE}. May also return {@link StaticProperty#ALLOWS_ZERO} if
+     *         the result is known to be an empty sequence, or {@link StaticProperty#ALLOWS_MANY} if
+     *         if is known to return a sequence of length two or more.
      */
 
     protected abstract int computeCardinality();
@@ -1360,7 +1358,7 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
      * @param arity the arity (number of parameters) of the function
      * @return {@link UserFunctionCall#NOT_TAIL_CALL} if no tail call was found;
      *         {@link UserFunctionCall#FOREIGN_TAIL_CALL} if a tail call on a different function was found;
-     * @link UserFunctionCall#SELF_TAIL_CALL} if a tail recursive call was found and if this call accounts for the whole of the value.
+     *         {@link UserFunctionCall#SELF_TAIL_CALL} if a tail recursive call was found and if this call accounts for the whole of the value.
      */
 
     public int markTailFunctionCalls(StructuredQName qName, int arity) {
@@ -1371,12 +1369,11 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
      * Convert this expression to an equivalent XSLT pattern
      *
      * @param config the Saxon configuration
-     * @param is30   true if this is XSLT 3.0
      * @return the equivalent pattern
      * @throws XPathException if conversion is not possible
      */
 
-    public Pattern toPattern(Configuration config, boolean is30) throws XPathException {
+    public Pattern toPattern(Configuration config) throws XPathException {
         ItemType type = getItemType();
         if (((getDependencies() & StaticProperty.DEPENDS_ON_NON_DOCUMENT_FOCUS) == 0) &&
                 (type instanceof NodeTest || this instanceof VariableReference)) {
@@ -1385,7 +1382,7 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
         if (isCallOn(KeyFn.class) || isCallOn(SuperId.class)) {
             return new NodeSetPattern(Expression.this);
         }
-        throw new XPathException("Cannot convert the expression {" + toString() + "} to a pattern");
+        throw new XPathException("Cannot convert the expression {" + this + "} to a pattern");
     }
 
     /**
@@ -1439,6 +1436,7 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
     protected void dynamicError(String message, String code, XPathContext context) throws XPathException {
         XPathException err = new XPathException(message, code, getLocation());
         err.setXPathContext(context);
+        err.setFailingExpression(this);
         throw err;
     }
 
@@ -1455,6 +1453,7 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
         XPathException e = new XPathException(message, errorCode, getLocation());
         e.setIsTypeError(true);
         e.setXPathContext(context);
+        e.setFailingExpression(this);
         throw e;
     }
 
@@ -1487,16 +1486,17 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
      * will be of type String, and each string can be supplied as input to the getProperty()
      * method to retrieve the value of the property. The iterator may return properties whose
      * value is null.
+     *
+     * @return an iterator over the properties
      */
 
     public Iterator<String> getProperties() {
-        return new MonoIterator<String>("expression");
+        return new MonoIterator<>("expression");
     }
 
     /**
      * Add a representation of this expression to a PathMap. The PathMap captures a map of the nodes visited
      * by an expression in a source tree.
-     * <p/>
      * <p>The default implementation of this method assumes that an expression does no navigation other than
      * the navigation done by evaluating its subexpressions, and that the subexpressions are evaluated in the
      * same context as the containing expression. The method must be overridden for any expression
@@ -1582,7 +1582,82 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
         return evaluationMethod;
     }
 
+    /**
+     * The equals() test for expressions returns true if it can be determined that two expressions (given
+     * their static context) will return the same result in all circumstances. The value false is returned
+     * if this is not the case or if it cannot be determined to be the case.
+     * <p>During the various phases of compilation, expression objects are mutable. Changing an expression
+     * may change its hashCode, and may change the result of equals() comparisons. Expressions should therefore
+     * not be held in data structures such as maps and sets that depend on equality comparison unless they
+     * are no longer subject to such mutation.</p>
+     *
+     * @param obj the other operand; the result is false if this is not an Expression
+     * @return true if the other operand is an expression and if it can be determined that the two
+     * expressions are equivalent, in the sense that they will always return the same result.
+     */
 
+    @Override
+    public boolean equals(Object obj) {
+        return super.equals(obj);
+    }
+
+    /**
+     * Test if this is expression is equivalent to another. This method first compares the hash codes,
+     * allowing a quick exit if the expressions are not equal (except that computing the hash code may
+     * itself be expensive...).
+     * @param other the other expression
+     * @return true if the two expressions are equal in the sense of the equals() method
+     */
+
+    public final boolean isEqual(Expression other) {
+        return this == other || (hashCode() == other.hashCode() && equals(other));
+    }
+
+    /**
+     * The hashCode() for an expression has semantics corresponding to equals(). Because computing
+     * a hashCode may be expensive, it is computed lazily on first reference and saved with the
+     * expression.
+     *
+     * @return a hashCode which is the same for two expressions that are equal().
+     */
+
+    public final int hashCode() {
+        if (cachedHashCode == -1) {
+            cachedHashCode = computeHashCode();
+        }
+        return cachedHashCode;
+    }
+
+    /**
+     * Ask whether the static context of this expression is compatible with the static context
+     * of another expression. The static contexts are considered compatible if either (a)
+     * neither expression depends on the static context, or (b) both expressions depend on the static
+     * context and the two static contexts compare equal.
+     * @param other the other expression
+     * @return true if the two expressions have equivalent static contexts
+     */
+
+    protected boolean hasCompatibleStaticContext(Expression other) {
+        boolean d1 = (getIntrinsicDependencies() & StaticProperty.DEPENDS_ON_STATIC_CONTEXT) != 0;
+        boolean d2 = (other.getIntrinsicDependencies() & StaticProperty.DEPENDS_ON_STATIC_CONTEXT) != 0;
+        if (d1 != d2) {
+            return false;
+        }
+        if (d1) {
+            return getRetainedStaticContext().equals(other.getRetainedStaticContext());
+        }
+        return true;
+    }
+
+
+    /**
+     * Compute a hash code, which will then be cached for later use
+     * @return a computed hash code
+     */
+
+    protected int computeHashCode() {
+        return super.hashCode();
+    }
 
     /**
      * Determine whether two IdentityComparable objects are identical. This is a stronger
@@ -1606,22 +1681,6 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
         return System.identityHashCode(Expression.this.getLocation());
     }
 
-
-    public int hashCode() {
-        if (cachedHashCode == -1) {
-            cachedHashCode = computeHashCode();
-        }
-        return cachedHashCode;
-    }
-
-    public int computeHashCode() {
-        return super.hashCode();
-    }
-
-    public boolean isSameExpression(Expression other) {
-        return hashCode() == other.hashCode() && equals(other);
-    }
-
     /**
      * Set an extra property on the expression. (Used for Saxon-EE properties, e.g. properties
      * relating to streaming).
@@ -1634,7 +1693,7 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
             if (value == null) {
                 return;
             }
-            extraProperties = new HashMap<String, Object>(4);
+            extraProperties = new HashMap<>(4);
         }
         if (value == null) {
             extraProperties.remove(name);
