@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -14,7 +14,7 @@ import net.sf.saxon.expr.parser.*;
 import net.sf.saxon.functions.PositionAndLast;
 import net.sf.saxon.functions.SystemFunction;
 import net.sf.saxon.functions.registry.VendorFunctionSetHE;
-import net.sf.saxon.lib.FeatureKeys;
+import net.sf.saxon.lib.Feature;
 import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.om.*;
 import net.sf.saxon.pattern.*;
@@ -23,7 +23,6 @@ import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.iter.EmptyIterator;
 import net.sf.saxon.type.*;
 import net.sf.saxon.value.*;
-import net.sf.saxon.value.StringValue;
 
 import java.math.BigInteger;
 
@@ -171,7 +170,6 @@ public final class FilterExpression extends BinaryExpression implements ContextS
 
     /**
      * Test if the filter always returns a singleton boolean.
-     * <p/>
      * <p>This information is available only after typeCheck() has been called.</p>
      *
      * @return true if the filter is a simple boolean expression
@@ -183,7 +181,6 @@ public final class FilterExpression extends BinaryExpression implements ContextS
 
     /**
      * Determine whether the filter is independent of the context item and position
-     * <p/>
      * <p>This information is available only after typeCheck() has been called.</p>
      *
      * @return true if the filter is a numeric value that does not depend on the context item or position
@@ -237,10 +234,8 @@ public final class FilterExpression extends BinaryExpression implements ContextS
 
     /**
      * Type-check the expression
-     * <p/>
-     * <p/>
-     * param visitor         the expression visitor
      *
+     * @param visitor         the expression visitor
      * @param contextInfo return the expression after type-checking (potentially modified to add run-time
      *                    checks and/or conversions)
      */
@@ -292,7 +287,6 @@ public final class FilterExpression extends BinaryExpression implements ContextS
 
     /**
      * Perform optimisation of an expression and its subexpressions.
-     * <p/>
      * <p>This method is called after all references to functions and variables have been resolved
      * to the declaration of the function or variable, and after all type checking has been done.</p>
      *
@@ -309,8 +303,8 @@ public final class FilterExpression extends BinaryExpression implements ContextS
     /*@NotNull*/
     public Expression optimize(ExpressionVisitor visitor, ContextItemStaticInfo contextItemType) throws XPathException {
         final Configuration config = visitor.getConfiguration();
-        final Optimizer opt = config.obtainOptimizer();
-        final boolean tracing = config.getBooleanProperty(FeatureKeys.TRACE_OPTIMIZER_DECISIONS);
+        final Optimizer opt = visitor.obtainOptimizer();
+        final boolean tracing = config.getBooleanProperty(Feature.TRACE_OPTIMIZER_DECISIONS);
         final TypeHierarchy th = config.getTypeHierarchy();
 
         getLhs().optimize(visitor, contextItemType);
@@ -429,7 +423,7 @@ public final class FilterExpression extends BinaryExpression implements ContextS
                 ((IsLastExpression) getFilter()).getCondition()) {
 
             if (getBase() instanceof Literal) {
-                setFilter(Literal.makeLiteral(new Int64Value(((Literal) getBase()).getValue().getLength())));
+                setFilter(Literal.makeLiteral(new Int64Value(((Literal) getBase()).getValue().getLength()), this));
             } else {
                 return new LastItemExpression(getBase());
             }
@@ -452,25 +446,16 @@ public final class FilterExpression extends BinaryExpression implements ContextS
 
         if (!filterIsPositional && getLhsExpression() instanceof FilterExpression &&
                 !((FilterExpression) getLhsExpression()).isFilterIsPositional()) {
-            Expression base = ((FilterExpression) getLhsExpression()).getLhsExpression();
-            Expression predicate1 = ((FilterExpression) getLhsExpression()).getRhsExpression();
-            Expression predicate2 = getRhsExpression();
-            if (predicate1.getCost() > 2 * predicate2.getCost()) {
-                FilterExpression fe1 = new FilterExpression(base, predicate2);
-                FilterExpression fe2 = new FilterExpression(fe1, predicate1);
-                ExpressionTool.copyLocationInfo(this, fe2);
-                if (tracing) {
-                    opt.trace("Reordered predicates in filter expression", fe2);
-                }
-                return fe2.optimize(visitor, contextItemType);
+            Expression f2 = opt.reorderPredicates(this, visitor, contextItemType);
+            if (f2 != this) {
+                return f2;
             }
-
         }
 
         final Sequence sequence = tryEarlyEvaluation(visitor);
         if (sequence != null) {
-            GroundedValue value = SequenceTool.toGroundedValue(sequence);
-            return Literal.makeLiteral(value);
+            GroundedValue value = sequence.materialize();
+            return Literal.makeLiteral(value, this);
         }
 
         return this;
@@ -531,7 +516,7 @@ public final class FilterExpression extends BinaryExpression implements ContextS
                     !ExpressionTool.refersToVariableOrFunction(getFilter()) &&
                     (getFilter().getDependencies() & ~StaticProperty.DEPENDS_ON_FOCUS) == 0) {
                 XPathContext context = visitor.getStaticContext().makeEarlyEvaluationContext();
-                return SequenceExtent.makeSequenceExtent(iterate(context));
+                return ((SequenceIterator<Item>) iterate(context)).materialize();
             }
         } catch (XPathException e) {
             // can happen for a variety of reasons, for example the filter references a global parameter,
@@ -650,7 +635,7 @@ public final class FilterExpression extends BinaryExpression implements ContextS
             if (Cardinality.allowsZero(card)) {
                 LetExpression let = new LetExpression();
                 let.setRequiredType(SequenceType.makeSequenceType(comparand.getItemType(), card));
-                let.setVariableQName(new StructuredQName("pp", NamespaceConstant.SAXON, "pp" + let.computeHashCode()));
+                let.setVariableQName(new StructuredQName("pp", NamespaceConstant.SAXON, "pp" + let.hashCode()));
                 let.setSequence(comparand);
                 comparand = new LocalVariableReference(let);
                 LocalVariableReference existsArg = new LocalVariableReference(let);
@@ -693,12 +678,12 @@ public final class FilterExpression extends BinaryExpression implements ContextS
 
             LetExpression let = new LetExpression();
             let.setRequiredType(SequenceType.SINGLE_INTEGER);
-            let.setVariableQName(new StructuredQName("nn", NamespaceConstant.SAXON, "nn" + let.computeHashCode()));
+            let.setVariableQName(new StructuredQName("nn", NamespaceConstant.SAXON, "nn" + let.hashCode()));
             let.setSequence(min);
             min = new LocalVariableReference(let);
             LocalVariableReference min2 = new LocalVariableReference(let);
             Expression minMinusOne = new ArithmeticExpression(
-                    min2, Token.MINUS, Literal.makeLiteral(Int64Value.makeIntegerValue(1)));
+                    min2, Token.MINUS, Literal.makeLiteral(Int64Value.makeIntegerValue(1), this));
             Expression length = new ArithmeticExpression(max, Token.MINUS, minMinusOne);
             Expression subs = SystemFunction.makeCall("subsequence", getRetainedStaticContext(), getBase(), min, length);
             let.setAction(subs);
@@ -731,13 +716,13 @@ public final class FilterExpression extends BinaryExpression implements ContextS
 
                     Expression[] args = new Expression[3];
                     args[0] = start;
-                    args[1] = Literal.makeLiteral(Int64Value.makeIntegerValue(1));
+                    args[1] = Literal.makeLiteral(Int64Value.makeIntegerValue(1), start);
                     if (Literal.isAtomic(comparand)) {
                         long n = ((NumericValue) ((Literal) comparand).getValue()).longValue();
-                        args[2] = Literal.makeLiteral(Int64Value.makeIntegerValue(n - 1));
+                        args[2] = Literal.makeLiteral(Int64Value.makeIntegerValue(n - 1), start);
                     } else {
                         ArithmeticExpression decrement = new ArithmeticExpression(
-                                comparand, Token.MINUS, Literal.makeLiteral(Int64Value.makeIntegerValue(1)));
+                                comparand, Token.MINUS, Literal.makeLiteral(Int64Value.makeIntegerValue(1), start));
                         decrement.setCalculator(Calculator.getCalculator(      // bug 2704
                                                                                StandardNames.XS_INTEGER, StandardNames.XS_INTEGER, Calculator.MINUS, true));
                         args[2] = decrement;
@@ -747,7 +732,7 @@ public final class FilterExpression extends BinaryExpression implements ContextS
                 case Token.FLE: {
                     Expression[] args = new Expression[3];
                     args[0] = start;
-                    args[1] = Literal.makeLiteral(Int64Value.makeIntegerValue(1));
+                    args[1] = Literal.makeLiteral(Int64Value.makeIntegerValue(1), start);
                     args[2] = comparand;
                     return SystemFunction.makeCall("subsequence", start.getRetainedStaticContext(), args);
                 }
@@ -759,10 +744,10 @@ public final class FilterExpression extends BinaryExpression implements ContextS
                     args[0] = start;
                     if (Literal.isAtomic(comparand)) {
                         long n = ((NumericValue) ((Literal) comparand).getValue()).longValue();
-                        args[1] = Literal.makeLiteral(Int64Value.makeIntegerValue(n + 1));
+                        args[1] = Literal.makeLiteral(Int64Value.makeIntegerValue(n + 1), start);
                     } else {
                         args[1] = new ArithmeticExpression(
-                                comparand, Token.PLUS, Literal.makeLiteral(Int64Value.makeIntegerValue(1)));
+                                comparand, Token.PLUS, Literal.makeLiteral(Int64Value.makeIntegerValue(1), start));
                     }
                     return SystemFunction.makeCall("subsequence", start.getRetainedStaticContext(), args);
                 }
@@ -785,18 +770,18 @@ public final class FilterExpression extends BinaryExpression implements ContextS
                     LetExpression let = new LetExpression();
                     let.setRequiredType(SequenceType.makeSequenceType(
                             comparand.getItemType(), StaticProperty.ALLOWS_ONE));
-                    let.setVariableQName(new StructuredQName("pp", NamespaceConstant.SAXON, "pp" + let.computeHashCode()));
+                    let.setVariableQName(new StructuredQName("pp", NamespaceConstant.SAXON, "pp" + let.hashCode()));
                     let.setSequence(comparand);
                     LocalVariableReference isWholeArg = new LocalVariableReference(let);
                     LocalVariableReference arithArg = new LocalVariableReference(let);
                     LocalVariableReference floorArg = new LocalVariableReference(let);
                     Expression isWhole = VendorFunctionSetHE.getInstance().makeFunction("is-whole-number", 1).makeFunctionCall(isWholeArg);
                     Expression minusOne = new ArithmeticExpression(
-                            arithArg, Token.MINUS, Literal.makeLiteral(Int64Value.makeIntegerValue(1)));
+                            arithArg, Token.MINUS, Literal.makeLiteral(Int64Value.makeIntegerValue(1), start));
                     Expression floor = SystemFunction.makeCall("floor", start.getRetainedStaticContext(), floorArg);
                     Expression choice = Choose.makeConditional(isWhole, minusOne, floor);
                     Expression subs = SystemFunction.makeCall(
-                            "subsequence", start.getRetainedStaticContext(), start, Literal.makeLiteral(Int64Value.makeIntegerValue(1)), choice);
+                            "subsequence", start.getRetainedStaticContext(), start, Literal.makeLiteral(Int64Value.makeIntegerValue(1), start), choice);
                     let.setAction(subs);
                     //decl.fixupReferences(let);
                     return let;
@@ -804,7 +789,7 @@ public final class FilterExpression extends BinaryExpression implements ContextS
                 case Token.FLE: {
                     Expression floor = SystemFunction.makeCall("floor", start.getRetainedStaticContext(), comparand);
                     return SystemFunction.makeCall(
-                            "subsequence", start.getRetainedStaticContext(), start, Literal.makeLiteral(Int64Value.makeIntegerValue(1)), floor);
+                            "subsequence", start.getRetainedStaticContext(), start, Literal.makeLiteral(Int64Value.makeIntegerValue(1), start), floor);
                 }
                 case Token.FNE: {
                     // rewrite SEQ[position() ne V] as
@@ -813,7 +798,7 @@ public final class FilterExpression extends BinaryExpression implements ContextS
                     ExpressionTool.copyLocationInfo(start, let);
                     let.setRequiredType(SequenceType.makeSequenceType(
                             comparand.getItemType(), StaticProperty.ALLOWS_ONE));
-                    let.setVariableQName(new StructuredQName("pp", NamespaceConstant.SAXON, "pp" + let.computeHashCode()));
+                    let.setVariableQName(new StructuredQName("pp", NamespaceConstant.SAXON, "pp" + let.hashCode()));
                     let.setSequence(comparand);
                     LocalVariableReference isWholeArg = new LocalVariableReference(let);
                     LocalVariableReference castArg = new LocalVariableReference(let);
@@ -822,7 +807,7 @@ public final class FilterExpression extends BinaryExpression implements ContextS
                     Expression cast = new CastExpression(castArg, BuiltInAtomicType.INTEGER, false);
                     ExpressionTool.copyLocationInfo(start, cast);
                     Expression choice = Choose.makeConditional(
-                            isWhole, cast, Literal.makeLiteral(Int64Value.makeIntegerValue(0)));
+                            isWhole, cast, Literal.makeLiteral(Int64Value.makeIntegerValue(0), start));
                     Expression rem = SystemFunction.makeCall("remove", start.getRetainedStaticContext(), start, choice);
                     let.setAction(rem);
                     return let;
@@ -833,14 +818,14 @@ public final class FilterExpression extends BinaryExpression implements ContextS
                     LetExpression let = new LetExpression();
                     let.setRequiredType(SequenceType.makeSequenceType(
                             comparand.getItemType(), StaticProperty.ALLOWS_ONE));
-                    let.setVariableQName(new StructuredQName("pp", NamespaceConstant.SAXON, "pp" + let.computeHashCode()));
+                    let.setVariableQName(new StructuredQName("pp", NamespaceConstant.SAXON, "pp" + let.hashCode()));
                     let.setSequence(comparand);
                     LocalVariableReference isWholeArg = new LocalVariableReference(let);
                     LocalVariableReference arithArg = new LocalVariableReference(let);
                     LocalVariableReference ceilingArg = new LocalVariableReference(let);
                     Expression isWhole = VendorFunctionSetHE.getInstance().makeFunction("is-whole-number", 1).makeFunctionCall(isWholeArg);
                     Expression plusOne = new ArithmeticExpression(
-                            arithArg, Token.PLUS, Literal.makeLiteral(Int64Value.makeIntegerValue(1)));
+                            arithArg, Token.PLUS, Literal.makeLiteral(Int64Value.makeIntegerValue(1), start));
                     Expression ceiling = SystemFunction.makeCall("ceiling", start.getRetainedStaticContext(), ceilingArg);
                     Expression choice = Choose.makeConditional(isWhole, plusOne, ceiling);
                     Expression subs = SystemFunction.makeCall("subsequence", start.getRetainedStaticContext(), start, choice);
@@ -975,7 +960,7 @@ public final class FilterExpression extends BinaryExpression implements ContextS
             }
         }
         if (getFilter() instanceof IsLastExpression && ((IsLastExpression) getFilter()).getCondition()) {
-            return getBase().getCardinality() & ~StaticProperty.ALLOWS_MANY;
+            return StaticProperty.ALLOWS_ZERO_OR_ONE;
         }
         if (!Cardinality.allowsMany(getBase().getCardinality())) {
             return StaticProperty.ALLOWS_ZERO_OR_ONE;
@@ -1006,8 +991,8 @@ public final class FilterExpression extends BinaryExpression implements ContextS
     public boolean equals(Object other) {
         if (other instanceof FilterExpression) {
             FilterExpression f = (FilterExpression) other;
-            return getBase().equals(f.getBase()) &&
-                    getFilter().equals(f.getFilter());
+            return getBase().isEqual(f.getBase()) &&
+                    getFilter().isEqual(f.getFilter());
         }
         return false;
     }
@@ -1019,23 +1004,22 @@ public final class FilterExpression extends BinaryExpression implements ContextS
      */
 
     public int computeHashCode() {
-        return "FilterExpression".hashCode() + getBase().computeHashCode() + getFilter().computeHashCode();
+        return "FilterExpression".hashCode() + getBase().hashCode() + getFilter().hashCode();
     }
 
     /**
      * Convert this expression to an equivalent XSLT pattern
      *
      * @param config the Saxon configuration
-     * @param is30   true if this is XSLT 3.0
      * @return the equivalent pattern
      * @throws net.sf.saxon.trans.XPathException if conversion is not possible
      */
     @Override
-    public Pattern toPattern(Configuration config, boolean is30) throws XPathException {
+    public Pattern toPattern(Configuration config) throws XPathException {
         Expression base = getSelectExpression();
         Expression filter = getFilter();
         TypeHierarchy th = config.getTypeHierarchy();
-        Pattern basePattern = base.toPattern(config, is30);
+        Pattern basePattern = base.toPattern(config);
         if (!isPositional(th)) {
             return new BasePatternWithPredicate(basePattern, filter);
         } else if (basePattern instanceof NodeTestPattern &&
@@ -1066,7 +1050,7 @@ public final class FilterExpression extends BinaryExpression implements ContextS
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<? extends Item> iterate(XPathContext context) throws XPathException {
 
         // Fast path where the filter value is independent of the focus
 
@@ -1079,7 +1063,7 @@ public final class FilterExpression extends BinaryExpression implements ContextS
                 }
                 if (first instanceof NumericValue) {
                     if (it.next() != null) {
-                        ExpressionTool.ebvError("sequence of two or more items starting with a numeric value");
+                        ExpressionTool.ebvError("sequence of two or more items starting with a numeric value", getFilter());
                     } else {
                         // Filter is a constant number
                         int pos = ((NumericValue)first).asSubscript();
@@ -1090,7 +1074,7 @@ public final class FilterExpression extends BinaryExpression implements ContextS
                                     Item m = ((MemoClosure) baseVal).itemAt(pos - 1);
                                     return m == null ? EmptyIterator.emptyIterator() : m.iterate();
                                 } else {
-                                    Item m = SequenceTool.toGroundedValue(baseVal).itemAt(pos - 1);
+                                    Item m = baseVal.materialize().itemAt(pos - 1);
                                     return m == null ? EmptyIterator.emptyIterator() : m.iterate();
                                 }
                             } else if (getBase() instanceof Literal) {
@@ -1112,15 +1096,15 @@ public final class FilterExpression extends BinaryExpression implements ContextS
                     } else if (first instanceof BooleanValue) {
                         ebv = ((BooleanValue) first).getBooleanValue();
                         if (it.next() != null) {
-                            ExpressionTool.ebvError("sequence of two or more items starting with a boolean value");
+                            ExpressionTool.ebvError("sequence of two or more items starting with a boolean value", getFilter());
                         }
                     } else if (first instanceof StringValue) {
                         ebv = !((StringValue) first).isZeroLength();
                         if (it.next() != null) {
-                            ExpressionTool.ebvError("sequence of two or more items starting with a boolean value");
+                            ExpressionTool.ebvError("sequence of two or more items starting with a boolean value", getFilter());
                         }
                     } else {
-                        ExpressionTool.ebvError("sequence starting with an atomic value other than a boolean, number, or string");
+                        ExpressionTool.ebvError("sequence starting with an atomic value other than a boolean, number, or string", getFilter());
                     }
                     if (ebv) {
                         return getBase().iterate(context);
@@ -1190,7 +1174,7 @@ public final class FilterExpression extends BinaryExpression implements ContextS
      */
 
     public String toString() {
-        return ExpressionTool.parenthesize(getBase()) + "[" + getFilter().toString() + "]";
+        return ExpressionTool.parenthesize(getBase()) + "[" + getFilter() + "]";
     }
 
     @Override
