@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2015 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,16 +7,11 @@
 
 package net.sf.saxon.expr;
 
-import com.saxonica.ee.bytecode.ExpressionCompiler;
-import com.saxonica.ee.bytecode.ForExpressionCompiler;
-import com.saxonica.ee.stream.adjunct.ForExpressionAdjunct;
-import com.saxonica.ee.stream.adjunct.StreamingAdjunct;
-import net.sf.saxon.evpull.EventIterator;
-import net.sf.saxon.evpull.EventMappingFunction;
-import net.sf.saxon.evpull.EventMappingIterator;
+import net.sf.saxon.Configuration;
+import net.sf.saxon.expr.flwor.OuterForExpression;
 import net.sf.saxon.expr.instruct.Choose;
 import net.sf.saxon.expr.parser.*;
-import net.sf.saxon.lib.FeatureKeys;
+import net.sf.saxon.lib.Feature;
 import net.sf.saxon.om.Item;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.om.SequenceIterator;
@@ -27,13 +22,14 @@ import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.type.ItemType;
 import net.sf.saxon.type.SchemaType;
 import net.sf.saxon.type.TypeHierarchy;
+import net.sf.saxon.type.UType;
 import net.sf.saxon.value.Cardinality;
 import net.sf.saxon.value.IntegerValue;
 import net.sf.saxon.value.SequenceType;
 
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.List;
-import java.util.Map;
 
 /**
  * A ForExpression maps an expression over a sequence.
@@ -57,17 +53,14 @@ public class ForExpression extends Assignation {
      * Get a name identifying the kind of expression, in terms meaningful to a user.
      *
      * @return a name identifying the kind of expression, in terms meaningful to a user.
-     *         The name will always be in the form of a lexical XML QName, and should match the name used
-     *         in explain() output displaying the expression.
+     * The name will always be in the form of a lexical XML QName, and should match the name used
+     * in explain() output displaying the expression.
      */
 
     public String getExpressionName() {
         return "for";
     }
 
-    public boolean hasVariableBinding(Binding binding) {
-        return this == binding;
-    }
 
     /**
      * Type-check the expression
@@ -81,13 +74,13 @@ public class ForExpression extends Assignation {
         // which in turn is required when type-checking the action part.
 
         getSequenceOp().typeCheck(visitor, contextInfo);
-        if (Literal.isEmptySequence(getSequence())) {
+        if (Literal.isEmptySequence(getSequence()) && !(this instanceof OuterForExpression)) {
             return getSequence();
         }
 
         if (requiredType != null) {
             // if declaration is null, we've already done the type checking in a previous pass
-            final TypeHierarchy th = getConfiguration().getTypeHierarchy();
+            final TypeHierarchy th = visitor.getConfiguration().getTypeHierarchy();
             SequenceType decl = requiredType;
             SequenceType sequenceType = SequenceType.makeSequenceType(
                     decl.getPrimaryType(), StaticProperty.ALLOWS_ZERO_OR_MORE);
@@ -113,7 +106,7 @@ public class ForExpression extends Assignation {
      * Get the cardinality of the range variable
      *
      * @return the cardinality of the range variable (StaticProperty.EXACTLY_ONE). Can be overridden
-     *         in a subclass
+     * in a subclass
      */
 
     protected int getRangeVariableCardinality() {
@@ -126,8 +119,9 @@ public class ForExpression extends Assignation {
 
     /*@NotNull*/
     public Expression optimize(ExpressionVisitor visitor, ContextItemStaticInfo contextItemType) throws XPathException {
-        Optimizer opt = getConfiguration().obtainOptimizer();
-        boolean debug = opt.getConfiguration().getBooleanProperty(FeatureKeys.TRACE_OPTIMIZER_DECISIONS);
+        Configuration config = visitor.getConfiguration();
+        Optimizer opt = visitor.obtainOptimizer();
+        boolean debug = config.getBooleanProperty(Feature.TRACE_OPTIMIZER_DECISIONS);
 
         // Try to promote any WHERE clause appearing immediately within the FOR expression
 
@@ -143,18 +137,6 @@ public class ForExpression extends Assignation {
             return p.optimize(visitor, contextItemType);
         }
 
-        // See if there is a simple "where" condition that can be turned into a predicate
-
-//        Expression pred = convertWhereToPredicate(visitor, contextItemType);
-//        if (pred != null) {
-//            if (debug) {
-//                opt.trace("Converted where clause in for $" + getVariableName() + " to predicate", pred);
-//            }
-//            if (pred != this) {
-//                return visitor.optimize(pred, contextItemType);
-//            }
-//        }
-
         Expression seq0 = getSequence();
         getSequenceOp().optimize(visitor, contextItemType);
         if (seq0 != getSequence()) {
@@ -162,7 +144,7 @@ public class ForExpression extends Assignation {
             return optimize(visitor, contextItemType);
         }
 
-        if (Literal.isEmptySequence(getSequence())) {
+        if (Literal.isEmptySequence(getSequence()) && !(this instanceof OuterForExpression)) {
             return getSequence();
         }
 
@@ -177,14 +159,6 @@ public class ForExpression extends Assignation {
             return getAction();
         }
 
-        Expression e2 = extractLoopInvariants(visitor, contextItemType);
-        if (e2 != null && e2 != this) {
-            if (debug) {
-                opt.trace("Extracted invariant in 'for $" + getVariableName() + "' loop", e2);
-            }
-            return e2.optimize(visitor, contextItemType);
-        }
-
         // Simplify an expression of the form "for $b in a/b/c return $b/d".
         // (XQuery users seem to write these a lot!)
 
@@ -203,7 +177,7 @@ public class ForExpression extends Assignation {
                     // see test qxmp299, where this condition isn't satisfied
                     if (debug) {
                         opt.trace("Collapsed return clause of for $" + getVariableName() +
-                                " into path expression", newPath);
+                                          " into path expression", newPath);
                     }
                     return newPath.optimize(visitor, contextItemType);
                 }
@@ -231,18 +205,11 @@ public class ForExpression extends Assignation {
             let.setSequence(getSequence());
             let.setAction(getAction());
             let.setSlotNumber(slotNumber);
+            let.setRetainedStaticContextLocally(getRetainedStaticContext());
             ExpressionTool.rebindVariableReferences(getAction(), this, let);
             return let.typeCheck(visitor, contextItemType).optimize(visitor, contextItemType);
         }
 
-//        if (visitor.isOptimizeForStreaming()) {
-//            Expression e3 = getConfiguration().obtainOptimizer().optimizeForExpressionForStreaming(this);
-//            if (e3 != this) {
-//                return visitor.optimize(e3, contextItemType);
-//            }
-//        }
-
-        //declaration = null;     // let the garbage collector take it
         return this;
     }
 
@@ -252,7 +219,7 @@ public class ForExpression extends Assignation {
      *
      * @param retainAllNodes set to true if the result must contain exactly the same nodes as the
      *                       original; set to false if the result can eliminate (or introduce) duplicates.
-     * @param forStreaming  set to true if optimizing for streaming
+     * @param forStreaming   set to true if optimizing for streaming
      */
     @Override
     public Expression unordered(boolean retainAllNodes, boolean forStreaming) throws XPathException {
@@ -272,7 +239,7 @@ public class ForExpression extends Assignation {
      * are used to indicate values limited by the size of a string or the size of a sequence.
      *
      * @return the lower and upper bounds of integer values in the result, or null to indicate
-     *         unknown or not applicable.
+     * unknown or not applicable.
      */
     /*@Nullable*/
     @Override
@@ -280,41 +247,9 @@ public class ForExpression extends Assignation {
         return getAction().getIntegerBounds();
     }
 
-    /**
-     * Extract subexpressions in the action part that don't depend on the range variable
-     *
-     * @param visitor         the expression visitor
-     * @param contextItemType the item type of the context item
-     * @return the optimized expression if it has changed, or null if no optimization was possible
-     */
-
-    private Expression extractLoopInvariants(ExpressionVisitor visitor, ContextItemStaticInfo contextItemType) throws XPathException {
-        // Extract subexpressions that don't depend on the range variable or the position variable
-        // If a subexpression is (or might be) creative, this is, if it creates new nodes, we don't
-        // extract it from the loop, but we do extract its non-creative subexpressions
-
-        //if (positionVariable == null) {
-        PromotionOffer offer = new PromotionOffer(getConfiguration().obtainOptimizer());
-        offer.containingExpression = this;
-        offer.action = PromotionOffer.RANGE_INDEPENDENT;
-        offer.bindingList = new Binding[]{this};
-        setAction(doPromotion(getAction(), offer));
-        if (offer.containingExpression instanceof LetExpression) {
-            // a subexpression has been promoted
-            //offer.containingExpression.setParentExpression(container);
-            // try again: there may be further subexpressions to promote
-            offer.containingExpression = offer.containingExpression.optimize(visitor, contextItemType);
-        }
-        return offer.containingExpression;
-        //}
-        //return null;
-
-    }
-
     /**
      * Promote a WHERE clause whose condition doesn't depend on the variable being bound.
      * This rewrites an expression of the form
-     * <p/>
      * <p>let $i := SEQ return if (C) then R else ()</p>
      * <p>to the form:</p>
      * <p>if (C) then (let $i := SEQ return R) else ()</p>
@@ -368,8 +303,8 @@ public class ForExpression extends Assignation {
     /**
      * Copy an expression. This makes a deep copy.
      *
-     * @return the copy of the original expression
      * @param rebindings
+     * @return the copy of the original expression
      */
 
     /*@NotNull*/
@@ -379,10 +314,12 @@ public class ForExpression extends Assignation {
         forExp.setRequiredType(requiredType);
         forExp.setVariableQName(variableName);
         forExp.setSequence(getSequence().copy(rebindings));
+        //rebindings.put(this, forExp);
         Expression newAction = getAction().copy(rebindings);
         forExp.setAction(newAction);
         forExp.variableName = variableName;
         forExp.slotNumber = slotNumber;
+        // TODO: should be able to do this by adding a mapping to rebindings as above. But -s:app-Walmsley -t:d1e41271 crashes.
         ExpressionTool.rebindVariableReferences(newAction, this, forExp);
         return forExp;
     }
@@ -437,7 +374,7 @@ public class ForExpression extends Assignation {
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<? extends Item> iterate(XPathContext context) throws XPathException {
 
         // First create an iteration of the base sequence.
 
@@ -458,44 +395,17 @@ public class ForExpression extends Assignation {
         }
     }
 
-    /**
-     * Deliver the result of the expression as a sequence of events.
-     *
-     * @param context The dynamic evaluation context
-     * @return the result of the expression as an iterator over a sequence of PullEvent objects
-     * @throws XPathException if a dynamic error occurs during expression evaluation
-     */
-
-    /*@Nullable*/
-    public EventIterator iterateEvents(XPathContext context) throws XPathException {
-
-        // First create an iteration of the base sequence.
-
-        // Then create an EventMappingIterator which applies a mapping function to each
-        // item in the base sequence. The mapping function is essentially the "return"
-        // expression, wrapped in an EventMappingAction object that is responsible also for
-        // setting the range variable at each step.
-
-        SequenceIterator base = getSequence().iterate(context);
-        EventMappingFunction map = new EventMappingAction(context, getLocalSlotNumber(), getAction());
-        return new EventMappingIterator(base, map);
-    }
-
     /**
      * Process this expression as an instruction, writing results to the current
      * outputter
      */
 
     public void process(XPathContext context) throws XPathException {
-        SequenceIterator iter = getSequence().iterate(context);
-        int position = 1;
         int slot = getLocalSlotNumber();
-        while (true) {
-            Item item = iter.next();
-            if (item == null) break;
+        getSequence().iterate(context).forEachOrFail(item -> {
             context.setLocalVariable(slot, item);
             getAction().process(context);
-        }
+        });
     }
 
 
@@ -509,22 +419,18 @@ public class ForExpression extends Assignation {
      */
 
     public void evaluatePendingUpdates(XPathContext context, PendingUpdateList pul) throws XPathException {
-        SequenceIterator iter = getSequence().iterate(context);
-        int position = 1;
         int slot = getLocalSlotNumber();
-        while (true) {
-            Item item = iter.next();
-            if (item == null) break;
+        getSequence().iterate(context).forEachOrFail(item -> {
             context.setLocalVariable(slot, item);
             getAction().evaluatePendingUpdates(context, pul);
-        }
+        });
     }
 
     /**
      * Determine the data type of the items returned by the expression, if possible
      *
      * @return one of the values Type.STRING, Type.BOOLEAN, Type.NUMBER, Type.NODE,
-     *         or Type.ITEM (meaning not known in advance)
+     * or Type.ITEM (meaning not known in advance)
      */
 
     /*@NotNull*/
@@ -532,6 +438,18 @@ public class ForExpression extends Assignation {
         return getAction().getItemType();
     }
 
+    /**
+     * Get the static type of the expression as a UType, following precisely the type
+     * inference rules defined in the XSLT 3.0 specification.
+     *
+     * @param contextItemType
+     * @return the static item type of the expression according to the XSLT 3.0 defined rules
+     */
+    @Override
+    public UType getStaticUType(UType contextItemType) {
+        return getAction().getStaticUType(contextItemType);
+    }
+
     /**
      * Determine the static cardinality of the expression
      */
@@ -556,6 +474,13 @@ public class ForExpression extends Assignation {
                 " return " + (getAction() == null ? "(...)" : ExpressionTool.parenthesize(getAction()));
     }
 
+    @Override
+    public String toShortString() {
+        return "for $" + getVariableQName().getDisplayName() +
+                " in " + (getSequence() == null ? "(...)" : getSequence().toShortString()) +
+                " return " + (getAction() == null ? "(...)" : getAction().toShortString());
+    }
+
     /**
      * Diagnostic print of expression structure. The abstract expression tree
      * is written to the supplied output destination.
@@ -564,9 +489,9 @@ public class ForExpression extends Assignation {
     public void export(ExpressionPresenter out) throws XPathException {
         out.startElement("for", this);
         explainSpecializedAttributes(out);
-        out.emitAttribute("var", getVariableName());
-        out.emitAttribute("as", getSequence().getItemType().toString());
-        out.emitAttribute("slot", ""+getLocalSlotNumber());
+        out.emitAttribute("var", getVariableQName());
+        out.emitAttribute("as", getSequence().getItemType().toExportString());
+        out.emitAttribute("slot", "" + getLocalSlotNumber());
         out.setChildRole("in");
         getSequence().export(out);
         out.setChildRole("return");
@@ -615,55 +540,12 @@ public class ForExpression extends Assignation {
             XPathContextMajor c2 = context.newContext();
             StackFrame oldstack = context.getStackFrame();
             Sequence[] vars = oldstack.getStackFrameValues();
-            Sequence[] newvars = new Sequence[vars.length];
-            System.arraycopy(vars, 0, newvars, 0, vars.length);
+            Sequence[] newvars = Arrays.copyOf(vars, vars.length);
             c2.setStackFrame(oldstack.getStackFrameMap(), newvars);
             return new MappingAction(c2, slotNumber, action);
         }
     }
 
-    /**
-     * The EventMappingAction represents the action to be taken for each item in the
-     * source sequence. It acts as the EventMappingFunction for the mapping iterator, and
-     * also provides the Binding of the position variable (at $n) in XQuery, if used.
-     */
-
-    protected static class EventMappingAction implements EventMappingFunction {
-
-        private XPathContext context;
-        private int slotNumber;
-        private Expression action;
-
-        public EventMappingAction(XPathContext context,
-                                  int slotNumber,
-                                  Expression action) {
-            this.context = context;
-            this.slotNumber = slotNumber;
-            this.action = action;
-        }
-
-        /*@Nullable*/
-        public EventIterator map(Item item) throws XPathException {
-            context.setLocalVariable(slotNumber, item);
-            return action.iterateEvents(context);
-        }
-
-    }
-
-//#ifdefined BYTECODE
-
-    /**
-     * Return the compiler of the For expression
-     *
-     * @return the relevant ExpressionCompiler
-     */
-    @Override
-    public ExpressionCompiler getExpressionCompiler() {
-        return new ForExpressionCompiler();
-    }
-//#endif
-
-
     /**
      * Get the type of this expression for use in tracing and diagnostics
      *
@@ -674,19 +556,11 @@ public class ForExpression extends Assignation {
         return LocationKind.FOR_EXPRESSION;
     }
 
-    //#ifdefined STREAM
-
-    /**
-     * Get a class that supports streamed evaluation of this expression
-     *
-     * @return the relevant StreamingAdjunct, or null if none is available
-     */
     @Override
-    protected StreamingAdjunct getStreamingAdjunct() {
-        return new ForExpressionAdjunct();
+    public String getStreamerName() {
+        return "ForExpression";
     }
 
-    //#endif
 
 }
 
