@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -10,16 +10,12 @@ package net.sf.saxon.expr;
 import net.sf.saxon.expr.oper.OperandArray;
 import net.sf.saxon.expr.parser.*;
 import net.sf.saxon.lib.NamespaceConstant;
-import net.sf.saxon.om.Function;
-import net.sf.saxon.om.Sequence;
-import net.sf.saxon.om.SequenceIterator;
-import net.sf.saxon.om.StructuredQName;
+import net.sf.saxon.om.*;
 import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.NoDynamicContextException;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.type.JavaExternalObjectType;
-import net.sf.saxon.value.SequenceExtent;
 import net.sf.saxon.value.SequenceType;
 
 import java.util.Collections;
@@ -206,19 +202,12 @@ public abstract class FunctionCall extends Expression {
 
     public void checkFunctionCall(Function target,
                                   ExpressionVisitor visitor) throws XPathException {
-//        boolean isXSLT = target instanceof UserFunction &&
-//            ((UserFunction) target).getPackageData().getHostLanguage() == Configuration.XSLT;
         TypeChecker tc = visitor.getConfiguration().getTypeChecker(visitor.getStaticContext().isInBackwardsCompatibleMode());
         SequenceType[] argTypes = target.getFunctionItemType().getArgumentTypes();
         int n = target.getArity();
         for (int i = 0; i < n; i++) {
             String name = getFunctionName() == null ? "" : getFunctionName().getDisplayName();
             RoleDiagnostic role = new RoleDiagnostic(RoleDiagnostic.FUNCTION, name, i);
-            // See bug 28371, which drops the XSLT-specific error code. In theory we should still use the old
-            // code in XSLT 2.0 mode, but error codes in XSLT 2.0 are not a conformance issue.
-//            if (isXSLT) {
-//                role.setErrorCode("XTTE0790");
-//            }
             setArg(i, tc.staticTypeCheck(
                 getArg(i),
                 argTypes[i],
@@ -229,7 +218,6 @@ public abstract class FunctionCall extends Expression {
 
     /**
      * Perform optimisation of an expression and its subexpressions.
-     * <p/>
      * <p>This method is called after all references to functions and variables have been resolved
      * to the declaration of the function or variable, and after all type checking has been done.</p>
      *
@@ -286,9 +274,7 @@ public abstract class FunctionCall extends Expression {
             return this;
         }
         try {
-            Literal lit = Literal.makeLiteral(SequenceExtent.makeSequenceExtent(
-                    iterate(visitor.getStaticContext().makeEarlyEvaluationContext())));
-            ExpressionTool.copyLocationInfo(this, lit);
+            Literal lit = Literal.makeLiteral(iterate(visitor.getStaticContext().makeEarlyEvaluationContext()).materialize(), this);
             Optimizer.trace(visitor.getConfiguration(), "Pre-evaluated function call " + toShortString(), lit);
             return lit;
         } catch (NoDynamicContextException e) {
@@ -373,7 +359,6 @@ public abstract class FunctionCall extends Expression {
     /**
      * Add a representation of this expression to a PathMap. The PathMap captures a map of the nodes visited
      * by an expression in a source tree.
-     * <p/>
      * <p>The default implementation of this method assumes that an expression does no navigation other than
      * the navigation done by evaluating its subexpressions, and that the subexpressions are evaluated in the
      * same context as the containing expression. The method must be overridden for any expression
@@ -504,7 +489,7 @@ public abstract class FunctionCall extends Expression {
             return false;
         }
         for (int i = 0; i < getArity(); i++) {
-            if (!getArg(i).equals(f.getArg(i))) {
+            if (!getArg(i).isEqual(f.getArg(i))) {
                 return false;
             }
         }
@@ -521,7 +506,7 @@ public abstract class FunctionCall extends Expression {
         }
         int h = getFunctionName().hashCode();
         for (int i = 0; i < getArity(); i++) {
-            h ^= getArg(i).computeHashCode();
+            h ^= getArg(i).hashCode();
         }
         return h;
     }
@@ -540,7 +525,7 @@ public abstract class FunctionCall extends Expression {
      *                                           expression
      */
     @Override
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<? extends Item> iterate(XPathContext context) throws XPathException {
         Function target = getTargetFunction(context);
         Sequence[] actualArgs = evaluateArguments(context);
         try {
@@ -548,6 +533,7 @@ public abstract class FunctionCall extends Expression {
         } catch (XPathException e) {
             e.maybeSetLocation(getLocation());
             e.maybeSetContext(context);
+            e.maybeSetFailingExpression(this);
             throw e;
         }
     }
@@ -556,7 +542,7 @@ public abstract class FunctionCall extends Expression {
         int numArgs = getArity();
         Sequence[] actualArgs = new Sequence[numArgs];
         for (int i = 0; i < numArgs; i++) {
-            actualArgs[i] = ExpressionTool.lazyEvaluate(getArg(i), context, 1);
+            actualArgs[i] = ExpressionTool.lazyEvaluate(getArg(i), context, false);
         }
         return actualArgs;
     }
