@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -31,7 +31,7 @@ import java.util.List;
 /**
  * GeneralComparison: a boolean expression that compares two expressions
  * for equals, not-equals, greater-than or less-than. This implements the operators
- * =, !=, <, >, etc. This implementation is not used when in backwards-compatible mode
+ * =, !=, &lt;, &gt;, etc. This implementation is not used when in backwards-compatible mode
  */
 
 public abstract class GeneralComparison extends BinaryExpression implements ComparisonExpression, Callable {
@@ -113,28 +113,6 @@ public abstract class GeneralComparison extends BinaryExpression implements Comp
         this.comparer = comparer;
     }
 
-    /*@NotNull*/
-    @Override
-//    public Expression simplify() throws XPathException {
-//        Expression e = super.simplify();
-//        if (e != this) {
-//            ExpressionTool.copyLocationInfo(this, e);
-//            return e;
-//        } else if (getRetainedStaticContext().isBackwardsCompatibility()) {
-//            GeneralComparison10 gc10 = new GeneralComparison10(getLhsExpression(), getOperator(), getRhsExpression());
-//            gc10.setRetainedStaticContext(getRetainedStaticContext());
-//            gc10.setAtomicComparer(getAtomicComparer());
-//            ExpressionTool.copyLocationInfo(this, gc10);
-//            return gc10;
-//        } else {
-//            GeneralComparison20 gc20 = new GeneralComparison20(getLhsExpression(), getOperator(), getRhsExpression());
-//            gc20.setRetainedStaticContext(getRetainedStaticContext());
-//            gc20.setAtomicComparer(getAtomicComparer());
-//            ExpressionTool.copyLocationInfo(this, gc20);
-//            return gc20;
-//        }
-//    }
-
     /**
      * Get a name identifying the kind of expression, in terms meaningful to a user.
      *
@@ -207,7 +185,8 @@ public abstract class GeneralComparison extends BinaryExpression implements Comp
         Expression oldOp0 = getLhsExpression();
         Expression oldOp1 = getRhsExpression();
 
-        typeCheckChildren(visitor, contextInfo);
+        getLhs().typeCheck(visitor, contextInfo);
+        getRhs().typeCheck(visitor, contextInfo);
 
         // If either operand is statically empty, return false
 
@@ -269,7 +248,7 @@ public abstract class GeneralComparison extends BinaryExpression implements Comp
             // then no static type checking is possible
         } else {
             if (!Type.isPossiblyComparable(pt0, pt1, Token.isOrderedOperator(singletonOperator))) {
-                String message = "In {" + toShortString() + "}: cannot compare " + t0.toString() + " to " + t1.toString();
+                String message = "In {" + toShortString() + "}: cannot compare " + t0 + " to " + t1;
                 if (Cardinality.allowsZero(c0) || Cardinality.allowsZero(c1)) {
                     if (!doneWarnings) { // avoid duplicate warnings
                         doneWarnings = true;
@@ -309,9 +288,9 @@ public abstract class GeneralComparison extends BinaryExpression implements Comp
             
             if (t0.equals(BuiltInAtomicType.UNTYPED_ATOMIC)) {
                 if (t1.equals(BuiltInAtomicType.UNTYPED_ATOMIC)) {
-                    e0 = new CastExpression(getLhsExpression(), BuiltInAtomicType.STRING, false);
+                    e0 = new CastExpression(getLhsExpression(), BuiltInAtomicType.STRING, Cardinality.allowsZero(c0));
                     adoptChildExpression(e0);
-                    e1 = new CastExpression(getRhsExpression(), BuiltInAtomicType.STRING, false);
+                    e1 = new CastExpression(getRhsExpression(), BuiltInAtomicType.STRING, Cardinality.allowsZero(c1));
                     adoptChildExpression(e1);
                 } else if (th.isSubType(t1, NumericType.getInstance())) {
 //                    e0 = new CastExpression(getLhsExpression(), BuiltInAtomicType.DOUBLE, false);
@@ -319,7 +298,7 @@ public abstract class GeneralComparison extends BinaryExpression implements Comp
                     Expression vun = makeCompareUntypedToNumeric(getLhsExpression(), getRhsExpression(), singletonOperator);
                     return vun.typeCheck(visitor, contextInfo);
                 } else {
-                    e0 = new CastExpression(getLhsExpression(), pt1, false);
+                    e0 = new CastExpression(getLhsExpression(), pt1, Cardinality.allowsZero(c0));
                     adoptChildExpression(e0);
                 }
             } else if (t1.equals(BuiltInAtomicType.UNTYPED_ATOMIC)) {
@@ -329,13 +308,14 @@ public abstract class GeneralComparison extends BinaryExpression implements Comp
                     Expression vun = makeCompareUntypedToNumeric(getRhsExpression(), getLhsExpression(), Token.inverse(singletonOperator));
                     return vun.typeCheck(visitor, contextInfo);
                 } else {
-                    e1 = new CastExpression(getRhsExpression(), pt0, false);
+                    e1 = new CastExpression(getRhsExpression(), pt0, Cardinality.allowsZero(c1));
                     adoptChildExpression(e1);
                 }
             }
 
             ValueComparison vc = new ValueComparison(e0, singletonOperator, e1);
             vc.setAtomicComparer(comparer);
+            vc.setResultWhenEmpty(BooleanValue.FALSE);
             ExpressionTool.copyLocationInfo(this, vc);
             Optimizer.trace(config, "Replaced general comparison by value comparison", vc);
             return vc.typeCheck(visitor, contextInfo);
@@ -365,14 +345,15 @@ public abstract class GeneralComparison extends BinaryExpression implements Comp
         ValueComparison vc = new ValueComparison(lhs, operator, rhs);
         vc.setAtomicComparer(new UntypedNumericComparer());
         ExpressionTool.copyLocationInfo(this, vc);
-        Optimizer.trace(getConfiguration(), "Replaced general comparison by value comparison", vc);
+        Optimizer.trace(getConfiguration(), "Replaced general comparison by untyped-numeric value comparison", vc);
         return vc;
     }
 
 
-    private static Expression makeMinOrMax(Expression exp, String function) throws XPathException {
+    private static Expression makeMinOrMax(Expression exp, String function) {
         if (Cardinality.allowsMany(exp.getCardinality())) {
             Expression fn = SystemFunction.makeCall(function, exp.getRetainedStaticContext(), exp);
+            assert fn != null;
             ((Minimax) ((SystemFunctionCall) fn).getTargetFunction()).setIgnoreNaN(true);
             return fn;
         } else {
@@ -380,6 +361,34 @@ public abstract class GeneralComparison extends BinaryExpression implements Comp
         }
     }
 
+    @Override
+    public int getIntrinsicDependencies() {
+        // The expression is dependent on the static namespace context if one operand might deliver
+        // untypedAtomic and the other might deliver a QName
+        TypeHierarchy th = getConfiguration().getTypeHierarchy();
+        if (mayInvolveCastToQName(th, getLhsExpression(), getRhsExpression()) ||
+                mayInvolveCastToQName(th, getRhsExpression(), getLhsExpression())) {
+            return StaticProperty.DEPENDS_ON_STATIC_CONTEXT;
+        } else {
+            return 0;
+        }
+    }
+
+    private boolean mayInvolveCastToQName(TypeHierarchy th, Expression e1, Expression e2) {
+        SimpleType s1 = (SimpleType) e1.getItemType().getAtomizedItemType();
+        return (s1 == BuiltInAtomicType.ANY_ATOMIC || s1.isNamespaceSensitive()) &&
+                th.relationship(e2.getItemType().getAtomizedItemType(), BuiltInAtomicType.UNTYPED_ATOMIC) != TypeHierarchy.DISJOINT &&
+                (e2.getSpecialProperties() & StaticProperty.NOT_UNTYPED_ATOMIC) == 0;
+    }
+
+    @Override
+    public boolean equals(Object other) {
+        return other instanceof GeneralComparison &&
+                super.equals(other) &&
+                comparer.equals(((GeneralComparison) other).comparer);
+    }
+
+
     /**
      * Optimize the expression
      *
@@ -392,7 +401,8 @@ public abstract class GeneralComparison extends BinaryExpression implements Comp
         final TypeHierarchy th = visitor.getConfiguration().getTypeHierarchy();
         final StaticContext env = visitor.getStaticContext();
 
-        optimizeChildren(visitor, contextInfo);
+        getLhs().optimize(visitor, contextInfo);
+        getRhs().optimize(visitor, contextInfo);
 
         // If either operand is statically empty, return false
 
@@ -407,7 +417,7 @@ public abstract class GeneralComparison extends BinaryExpression implements Comp
 
         if (getLhsExpression() instanceof Literal && getRhsExpression() instanceof Literal) {
             return Literal.makeLiteral(
-                    SequenceTool.toGroundedValue(evaluateItem(visitor.getStaticContext().makeEarlyEvaluationContext())), this
+                    evaluateItem(visitor.getStaticContext().makeEarlyEvaluationContext()).materialize(), this
             );
         }
 
@@ -543,7 +553,7 @@ public abstract class GeneralComparison extends BinaryExpression implements Comp
         }
 
         // Finally, convert to use the GeneralComparisonEE algorithm if in Saxon-EE
-        return visitor.getConfiguration().obtainOptimizer()
+        return visitor.obtainOptimizer()
                 .optimizeGeneralComparison(visitor, this, false, contextInfo);
     }
 
@@ -558,29 +568,10 @@ public abstract class GeneralComparison extends BinaryExpression implements Comp
                 }
             }
             return true;
-        };
+        }
         return false;
     }
 
-    /**
-     * Copy an expression. This makes a deep copy.
-     *
-     * @return the copy of the original expression
-     * @param rebindings
-     */
-
-    /*@NotNull*/
-//    public abstract Expression copy(RebindingMap rebindings) {
-//        GeneralComparison gc = new GeneralComparison(getLhsExpression().copy(rebindings), operator, getRhsExpression().copy(rebindings));
-//        ExpressionTool.copyLocationInfo(this, gc);
-//        gc.setRetainedStaticContext(getRetainedStaticContext());
-//        gc.comparer = comparer;
-//        gc.singletonOperator = singletonOperator;
-//        gc.needsRuntimeCheck = needsRuntimeCheck;
-//        gc.comparisonCardinality = comparisonCardinality;
-//        return gc;
-//    }
-
     /**
      * Evaluate the expression in a given context
      *
@@ -619,7 +610,7 @@ public abstract class GeneralComparison extends BinaryExpression implements Comp
      * @throws XPathException if a dynamic error occurs during the evaluation of the expression
      */
 
-    public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
+    public BooleanValue call(XPathContext context, Sequence[] arguments) throws XPathException {
         switch (comparisonCardinality) {
             case ONE_TO_ONE: {
                 AtomicValue value0 = (AtomicValue) arguments[0].head();
@@ -681,7 +672,8 @@ public abstract class GeneralComparison extends BinaryExpression implements Comp
     private boolean evaluateOneToOne(AtomicValue value0, AtomicValue value1, XPathContext context) throws XPathException {
         try {
             return !(value0 == null || value1 == null) &&
-                    compare(value0, singletonOperator, value1, comparer, needsRuntimeCheck, context, getRetainedStaticContext());
+                    compare(value0, singletonOperator, value1, comparer.provideContext(context),
+                            needsRuntimeCheck, context, getRetainedStaticContext());
         } catch (XPathException e) {
             // re-throw the exception with location information added
             e.maybeSetLocation(getLocation());
@@ -707,8 +699,9 @@ public abstract class GeneralComparison extends BinaryExpression implements Comp
                 return false;
             }
             AtomicValue item0;
+            AtomicComparer boundComparer = comparer.provideContext(context);
             while ((item0 = (AtomicValue) iter0.next()) != null) {
-                if (compare(item0, singletonOperator, value1, comparer, needsRuntimeCheck, context, getRetainedStaticContext())) {
+                if (compare(item0, singletonOperator, value1, boundComparer, needsRuntimeCheck, context, getRetainedStaticContext())) {
                     iter0.close();
                     return true;
                 }
@@ -738,8 +731,10 @@ public abstract class GeneralComparison extends BinaryExpression implements Comp
             boolean exhausted0 = false;
             boolean exhausted1 = false;
 
-            List<AtomicValue> value0 = new ArrayList<AtomicValue>();
-            List<AtomicValue> value1 = new ArrayList<AtomicValue>();
+            List<AtomicValue> value0 = new ArrayList<>();
+            List<AtomicValue> value1 = new ArrayList<>();
+
+            AtomicComparer boundComparer = comparer.provideContext(context);
 
             // Read items from the two sequences alternately, in each case comparing the item to
             // all items that have previously been read from the other sequence. In the worst case
@@ -757,7 +752,7 @@ public abstract class GeneralComparison extends BinaryExpression implements Comp
                         exhausted0 = true;
                     } else {
                         for (AtomicValue item1 : value1) {
-                            if (compare(item0, singletonOperator, item1, comparer, needsRuntimeCheck, context, getRetainedStaticContext())) {
+                            if (compare(item0, singletonOperator, item1, boundComparer, needsRuntimeCheck, context, getRetainedStaticContext())) {
                                 iter0.close();
                                 iter1.close();
                                 return true;
@@ -777,7 +772,7 @@ public abstract class GeneralComparison extends BinaryExpression implements Comp
                         exhausted1 = true;
                     } else {
                         for (AtomicValue item0 : value0) {
-                            if (compare(item0, singletonOperator, item1, comparer, needsRuntimeCheck, context, getRetainedStaticContext())) {
+                            if (compare(item0, singletonOperator, item1, boundComparer, needsRuntimeCheck, context, getRetainedStaticContext())) {
                                 iter0.close();
                                 iter1.close();
                                 return true;
@@ -806,10 +801,11 @@ public abstract class GeneralComparison extends BinaryExpression implements Comp
      * @param operator   the singleton version of the comparison operator,
      *                   for example {@link net.sf.saxon.expr.parser.Token#FEQ}
      * @param a1         the second value
-     * @param comparer   the comparer to be used to perform the comparison
+     * @param comparer   the comparer to be used to perform the comparison. If the comparer is context-sensitive
+     *                   then the context must already have been bound using comparer.provideContext().
      * @param checkTypes set to true if the operand types need to be checked for comparability at runtime
      * @param context    the XPath evaluation context
-     * @param nsResolver
+     * @param nsResolver namespace resolver
      * @return true if the comparison succeeds
      * @throws XPathException if a dynamic error occurs during the comparison
      */
@@ -822,11 +818,11 @@ public abstract class GeneralComparison extends BinaryExpression implements Comp
                                   XPathContext context,
                                   NamespaceResolver nsResolver) throws XPathException {
 
-        final ConversionRules rules = context.getConfiguration().getConversionRules();
         boolean u0 = a0 instanceof UntypedAtomicValue;
         boolean u1 = a1 instanceof UntypedAtomicValue;
         if (u0 != u1) {
             // one value untyped, the other not
+            final ConversionRules rules = context.getConfiguration().getConversionRules();
             if (u0) {
                 // a0 is untyped atomic
                 if (a1 instanceof NumericValue) {
@@ -836,9 +832,12 @@ public abstract class GeneralComparison extends BinaryExpression implements Comp
                 } else {
                     StringConverter sc = a1.getItemType().getPrimitiveItemType().getStringConverter(rules);
                     if (a1 instanceof QualifiedNameValue) {
-                        sc = (StringConverter)sc.setNamespaceResolver(nsResolver);
+                        sc = (StringConverter) sc.setNamespaceResolver(nsResolver);
+                        // avoid using cached results - bug 3862
+                        a0 = sc.convertString(a0.getStringValueCS()).asAtomic();
+                    } else {
+                        a0 = ((UntypedAtomicValue) a0).obtainConversionResult(a1.getItemType().getPrimitiveType(), sc).asAtomic();
                     }
-                    a0 = ((UntypedAtomicValue)a0).obtainConversionResult(a1.getItemType().getPrimitiveType(), sc).asAtomic();
                 }
             } else {
                 // a1 is untyped atomic
@@ -849,14 +848,17 @@ public abstract class GeneralComparison extends BinaryExpression implements Comp
                 } else {
                     StringConverter sc = a0.getItemType().getPrimitiveItemType().getStringConverter(rules);
                     if (a0 instanceof QualifiedNameValue) {
-                        sc = (StringConverter)sc.setNamespaceResolver(nsResolver);
+                        sc = (StringConverter) sc.setNamespaceResolver(nsResolver);
+                        // avoid using cached results - bug 3862
+                        a1 = sc.convertString(a1.getStringValueCS()).asAtomic();
+                    } else {
+                        a1 = ((UntypedAtomicValue) a1).obtainConversionResult(a0.getItemType().getPrimitiveType(), sc).asAtomic();
                     }
-                    a1 = ((UntypedAtomicValue) a1).obtainConversionResult(a0.getItemType().getPrimitiveType(), sc).asAtomic();
                 }
             }
             checkTypes = false; // No further checking needed if conversion succeeded
         }
-        return ValueComparison.compare(a0, operator, a1, comparer.provideContext(context), checkTypes);
+        return ValueComparison.compare(a0, operator, a1, comparer, checkTypes);
 
     }
 
@@ -876,7 +878,7 @@ public abstract class GeneralComparison extends BinaryExpression implements Comp
      * inference rules defined in the XSLT 3.0 specification.
      *
      * @return the static item type of the expression according to the XSLT 3.0 defined rules
-     * @param contextItemType
+     * @param contextItemType static information about the context item
      */
     @Override
     public UType getStaticUType(UType contextItemType) {
@@ -958,19 +960,5 @@ public abstract class GeneralComparison extends BinaryExpression implements Comp
         out.emitAttribute("comp", comparer.save());
     }
 
-    /**
-     * Find any necessary preconditions for the satisfaction of this expression
-     * as a set of boolean expressions to be evaluated on the context node
-     *
-     * @return A set of conditions, or null if none have been computed
-     *//*
-    public Set<Expression> getPreconditions() {
-        HashSet<Expression> pre = new HashSet<Expression>();
-        Expression r = RealPreconditions.reducedFunction(this);
-        if(r != null) {
-            pre.add(r);
-        }
-        return pre;
-    }*/
 }
 
