@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,7 +7,7 @@
 
 package net.sf.saxon.expr;
 
-import net.sf.saxon.event.SequenceReceiver;
+import net.sf.saxon.event.Receiver;
 import net.sf.saxon.event.TypeCheckingFilter;
 import net.sf.saxon.expr.instruct.Block;
 import net.sf.saxon.expr.parser.*;
@@ -96,19 +96,19 @@ public final class ItemChecker extends UnaryExpression {
 
     /*@NotNull*/
     public Expression typeCheck(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
-        typeCheckChildren(visitor, contextInfo);
+        getOperand().typeCheck(visitor, contextInfo);
         Expression operand = getBaseExpression();
 
         if (operand instanceof Block) {
             // Do the item-checking on each operand of the block separately (it might not be needed on all items)
             // This is particularly needed for streamability analysis of xsl:map
             Block block = (Block)operand;
-            List<Expression> checkedOperands = new ArrayList<Expression>();
+            List<Expression> checkedOperands = new ArrayList<>();
             for (Operand o : block.operands()) {
                 ItemChecker checkedOp = new ItemChecker(o.getChildExpression(), requiredItemType, role);
                 checkedOperands.add(checkedOp);
             }
-            Block newBlock = new Block(checkedOperands.toArray(new Expression[checkedOperands.size()]));
+            Block newBlock = new Block(checkedOperands.toArray(new Expression[0]));
             ExpressionTool.copyLocationInfo(this, newBlock);
             return newBlock.typeCheck(visitor, contextInfo);
         }
@@ -131,13 +131,13 @@ public final class ItemChecker extends UnaryExpression {
             } else if (Cardinality.allowsZero(card)) {
                 if (!(operand instanceof Literal)) {
                     String message = role.composeErrorMessage(
-                            requiredItemType, operand, operand.getItemType());
+                            requiredItemType, operand, th);
                     visitor.getStaticContext().issueWarning(
                             "The only value that can pass type-checking is an empty sequence. " +
                                                                     message, getLocation());
                 }
             } else {
-                String message = role.composeErrorMessage(requiredItemType, operand, operand.getItemType());
+                String message = role.composeErrorMessage(requiredItemType, operand, th);
                 XPathException err = new XPathException(message);
                 err.setErrorCode(role.getErrorCode());
                 err.setLocation(this.getLocation());
@@ -150,7 +150,6 @@ public final class ItemChecker extends UnaryExpression {
 
     /**
      * Perform optimisation of an expression and its subexpressions.
-     * <p/>
      * <p>This method is called after all references to functions and variables have been resolved
      * to the declaration of the function or variable, and after all type checking has been done.</p>
      *
@@ -165,7 +164,7 @@ public final class ItemChecker extends UnaryExpression {
      */
     @Override
     public Expression optimize(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
-        optimizeChildren(visitor, contextInfo);
+        getOperand().optimize(visitor, contextInfo);
         TypeHierarchy th = visitor.getConfiguration().getTypeHierarchy();
         int rel = th.relationship(requiredItemType, getBaseExpression().getItemType());
         if (rel == TypeHierarchy.SAME_TYPE || rel == TypeHierarchy.SUBSUMES) {
@@ -223,9 +222,9 @@ public final class ItemChecker extends UnaryExpression {
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
-        SequenceIterator base = getBaseExpression().iterate(context);
-        return new ItemMappingIterator(base, getMappingFunction(context), true);
+    public SequenceIterator<? extends Item> iterate(XPathContext context) throws XPathException {
+        SequenceIterator<? extends Item> base = getBaseExpression().iterate(context);
+        return new ItemMappingIterator<>(base, getMappingFunction(context), true);
     }
 
     /**
@@ -237,8 +236,8 @@ public final class ItemChecker extends UnaryExpression {
      *         as the input sequence, unless the dynamic type checking reveals an error.
      */
 
-    public ItemMappingFunction getMappingFunction(XPathContext context) {
-        return new ItemTypeCheckingFunction(requiredItemType, role, getLocation(), context.getConfiguration());
+    public ItemMappingFunction<Item, Item> getMappingFunction(XPathContext context) {
+        return new ItemTypeCheckingFunction(requiredItemType, role, getBaseExpression(), context.getConfiguration());
     }
 
     /**
@@ -256,7 +255,7 @@ public final class ItemChecker extends UnaryExpression {
         } else if (requiredItemType.getUType().subsumes(UType.STRING) && BuiltInAtomicType.ANY_URI.matches(item, th)) {
             return item;
         } else {
-            String message = role.composeErrorMessage(requiredItemType, getBaseExpression(), Type.getItemType(item, th));
+            String message = role.composeErrorMessage(requiredItemType, item, th);
             String errorCode = role.getErrorCode();
             if ("XPDY0050".equals(errorCode)) {
                 // error in "treat as" assertion
@@ -283,7 +282,7 @@ public final class ItemChecker extends UnaryExpression {
             next = ((CardinalityChecker) next).getBaseExpression();
         }
         if ((next.getImplementationMethod() & PROCESS_METHOD) != 0 && !(requiredItemType instanceof DocumentNodeTest)) {
-            SequenceReceiver out = context.getReceiver();
+            Receiver out = context.getReceiver();
             TypeCheckingFilter filter = new TypeCheckingFilter(out);
             filter.setRequiredType(requiredItemType, card, role, getLocation());
             context.setReceiver(filter);
@@ -291,6 +290,7 @@ public final class ItemChecker extends UnaryExpression {
             filter.close();
             context.setReceiver(out);
         } else {
+            // Force pull-mode evaluation
             super.process(context);
         }
     }
@@ -299,7 +299,7 @@ public final class ItemChecker extends UnaryExpression {
      * Copy an expression. This makes a deep copy.
      *
      * @return the copy of the original expression
-     * @param rebindings
+     * @param rebindings variable bindings that need to be changed
      */
 
     /*@NotNull*/
@@ -312,10 +312,9 @@ public final class ItemChecker extends UnaryExpression {
 
     /**
      * Determine the data type of the items returned by the expression
-     * <p/>
-     * <p/>
-     * /*@NotNull
      */
+
+    /*@NotNull*/
     public ItemType getItemType() {
         ItemType operandType = getBaseExpression().getItemType();
         TypeHierarchy th = getConfiguration().getTypeHierarchy();
@@ -344,7 +343,7 @@ public final class ItemChecker extends UnaryExpression {
      * inference rules defined in the XSLT 3.0 specification.
      *
      * @return the static item type of the expression according to the XSLT 3.0 defined rules
-     * @param contextItemType
+     * @param contextItemType the type of the context item
      */
     @Override
     public UType getStaticUType(UType contextItemType) {
@@ -366,8 +365,8 @@ public final class ItemChecker extends UnaryExpression {
      */
 
     @Override
-    public int hashCode() {
-        return super.hashCode() ^ requiredItemType.hashCode();
+    public int computeHashCode() {
+        return super.computeHashCode() ^ requiredItemType.hashCode();
     }
 
 
@@ -379,8 +378,8 @@ public final class ItemChecker extends UnaryExpression {
     public void export(ExpressionPresenter out) throws XPathException {
         out.startElement("treat", this);
         out.emitAttribute("as", requiredItemType.toExportString());
-        if ("JS".equals(out.getOption("target"))) {
-            int targetVersion = out.getIntOption("targetVersion", 1);
+        if ("JS".equals(((ExpressionPresenter.ExportOptions) out.getOptions()).target)) {
+            int targetVersion = ((ExpressionPresenter.ExportOptions) out.getOptions()).targetVersion;
             out.emitAttribute("jsTest", requiredItemType.generateJavaScriptItemTypeTest(getBaseExpression().getItemType(), targetVersion));
         }
         out.emitAttribute("diag", role.save());
@@ -408,7 +407,7 @@ public final class ItemChecker extends UnaryExpression {
 
     public String toString() {
         String typeDesc = requiredItemType.toString();
-        return "(" + getBaseExpression().toString() + ") treat as " + typeDesc;
+        return "(" + getBaseExpression() + ") treat as " + typeDesc;
     }
 
     @Override
