@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -14,12 +14,11 @@ import net.sf.saxon.trans.XPathException;
  * ItemMappingFunction is an interface that must be satisfied by an object passed to a
  * ItemMappingIterator. It represents an object which, given an Item, can return either
  * another Item, or null.
- * <p/>
- * The interface is generic, paramterized by F (from) the input item type, and T (to),
- * the output item type
+ * <p>The interface is generic, paramterized by F (from) the input item type, and T (to),
+ * the output item type</p>
  */
 
-public interface ItemMappingFunction {
+public interface ItemMappingFunction<F extends Item, T extends Item> {
 
     /**
      * Map one item to another item.
@@ -30,7 +29,7 @@ public interface ItemMappingFunction {
      */
 
     /*@Nullable*/
-    Item mapItem(Item item) throws XPathException;
+    T mapItem(F item) throws XPathException;
 
 }
 
