@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -36,10 +36,9 @@ import java.util.*;
 
 /**
  * This class together with its embedded subclasses handles conversion from Java values to XPath values.
- * <p/>
- * The general principle is to allocate a specific JPConverter at compile time wherever possible. If there
+ * <p>The general principle is to allocate a specific JPConverter at compile time wherever possible. If there
  * is insufficient type information to make this feasible, a general-purpose JPConverter is allocated, which
- * in turn allocates a more specific converter at run-time to do the actual work.
+ * in turn allocates a more specific converter at run-time to do the actual work.</p>
  */
 
 public abstract class JPConverter {
@@ -179,7 +178,7 @@ public abstract class JPConverter {
         if (c != null) {
              return c;
         }
-        if(javaClass.equals(Object.class)) {
+        if (javaClass.equals(Object.class)) {
             return FromObject.INSTANCE;
         }
         if (NodeInfo.class.isAssignableFrom(javaClass)) {
@@ -212,13 +211,12 @@ public abstract class JPConverter {
             return VoidConverter.INSTANCE;
         }
 
-        return new ExternalObjectWrapper(new JavaExternalObjectType(javaClass, config));
+        return new ExternalObjectWrapper(config.getJavaExternalObjectType(javaClass));
     }
 
     /**
      * Convert a Java object to an equivalent XPath value
      *
-     *
      * @param object  the java object to be converted
      * @param context the XPath dynamic evaluation context
      * @return the XPath value resulting from the conversion
@@ -254,7 +252,8 @@ public abstract class JPConverter {
             Class theClass = object.getClass();
             JPConverter instanceConverter = allocate(theClass, null, context.getConfiguration());
             if (instanceConverter instanceof FromObject) {
-                instanceConverter = new ExternalObjectWrapper(new JavaExternalObjectType(theClass));
+                instanceConverter = new ExternalObjectWrapper(
+                        context.getConfiguration().getJavaExternalObjectType(theClass));
             }
             return instanceConverter.convert(object, context);
         }
@@ -272,7 +271,7 @@ public abstract class JPConverter {
         public static final FromSequenceIterator INSTANCE = new FromSequenceIterator();
 
         public Sequence convert(Object object, XPathContext context) throws XPathException {
-            return SequenceExtent.makeSequenceExtent((SequenceIterator) object);
+            return ((SequenceIterator<Item>) (SequenceIterator) object).materialize();
         }
 
         public ItemType getItemType() {
@@ -323,7 +322,7 @@ public abstract class JPConverter {
 
         public Sequence convert(Object object, XPathContext context) throws XPathException {
             return object instanceof Closure ?
-                    SequenceExtent.makeSequenceExtent(((Closure) object).iterate()) :
+                    ((SequenceIterator<Item>) ((Closure) object).iterate()).materialize() :
                     (Sequence) object;
         }
 
@@ -512,19 +511,17 @@ public abstract class JPConverter {
 
     public static class ExternalObjectWrapper extends JPConverter {
 
-        public static final ExternalObjectWrapper INSTANCE = new ExternalObjectWrapper(new JavaExternalObjectType(Object.class));
-
         private JavaExternalObjectType resultType;
 
         public ExternalObjectWrapper(JavaExternalObjectType resultType) {
             this.resultType = resultType;
         }
 
-        public ObjectValue<Object> convert(Object object, XPathContext context) throws XPathException {
+        public ExternalObject<Object> convert(Object object, XPathContext context) throws XPathException {
             if (object == null) {
                 return null;
             } else if (resultType.getJavaClass().isInstance(object)) {
-                return new ObjectValue<Object>(object);
+                return new ObjectValue<>(object);
             } else {
                 throw new XPathException("Java external object of type " + object.getClass().getName() +
                     " is not an instance of the required type " + resultType.getJavaClass().getName(), "XPTY0004");
@@ -685,6 +682,8 @@ public abstract class JPConverter {
 
     public static class FromByteArray extends JPConverter {
 
+        // See bug 3525 - this maps to xs:unsignedByte for legacy/stability reasons
+
         public static final FromByteArray INSTANCE = new FromByteArray();
 
         public Sequence convert(Object object, XPathContext context) throws XPathException {
