@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -10,7 +10,8 @@ package net.sf.saxon.expr;
 
 import com.saxonica.functions.hof.FunctionLiteral;
 import net.sf.saxon.Configuration;
-import net.sf.saxon.event.SequenceReceiver;
+import net.sf.saxon.event.Receiver;
+import net.sf.saxon.event.ReceiverOptions;
 import net.sf.saxon.expr.parser.*;
 import net.sf.saxon.ma.arrays.ArrayItem;
 import net.sf.saxon.ma.map.KeyValuePair;
@@ -55,11 +56,11 @@ public class Literal extends Expression {
      * @return a Literal of the string sequence
      */
     public static Literal makeStringsLiteral(List<String> strings) {
-        List<AtomicValue> values = new ArrayList<AtomicValue>();
+        List<StringValue> values = new ArrayList<>();
         for (String s : strings) {
             values.add(new StringValue(s));
         }
-        GroundedValue gv = SequenceExtent.makeSequenceExtent(values);
+        GroundedValue<StringValue> gv = SequenceExtent.makeSequenceExtent(values);
         return makeLiteral(gv);
     }
 
@@ -167,7 +168,7 @@ public class Literal extends Expression {
                 return StaticProperty.EMPTY;
             } else {
                 if (iter.next() != null) {
-                    return StaticProperty.ALLOWS_ONE_OR_MORE;
+                    return StaticProperty.ALLOWS_MANY;
                 } else {
                     return StaticProperty.EXACTLY_ONE;
                 }
@@ -180,9 +181,9 @@ public class Literal extends Expression {
 
     /**
      * Compute the static properties of this expression (other than its type). For a
-     * Value, the only special property is {@link StaticProperty#NON_CREATIVE}.
+     * Value, the only special property is {@link StaticProperty#NO_NODES_NEWLY_CREATED}.
      *
-     * @return the value {@link StaticProperty#NON_CREATIVE}
+     * @return the value {@link StaticProperty#NO_NODES_NEWLY_CREATED}
      */
 
 
@@ -191,7 +192,7 @@ public class Literal extends Expression {
             // An empty sequence has all special properties except "has side effects".
             return StaticProperty.SPECIAL_PROPERTY_MASK & ~StaticProperty.HAS_SIDE_EFFECTS;
         }
-        return StaticProperty.NON_CREATIVE;
+        return StaticProperty.NO_NODES_NEWLY_CREATED;
     }
 
     /**
@@ -251,16 +252,15 @@ public class Literal extends Expression {
      * Convert this expression to an equivalent XSLT pattern
      *
      * @param config the Saxon configuration
-     * @param is30   true if this is XSLT 3.0
      * @return the equivalent pattern
      * @throws net.sf.saxon.trans.XPathException if conversion is not possible
      */
     @Override
-    public Pattern toPattern(Configuration config, boolean is30) throws XPathException {
+    public Pattern toPattern(Configuration config) throws XPathException {
         if (isEmptySequence(this)) {
             return new NodeTestPattern(ErrorType.getInstance());
         } else {
-            return super.toPattern(config, is30);
+            return super.toPattern(config);
         }
     }
 
@@ -306,7 +306,7 @@ public class Literal extends Expression {
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<? extends Item> iterate(XPathContext context) throws XPathException {
         return value.iterate();
     }
 
@@ -347,15 +347,13 @@ public class Literal extends Expression {
      */
 
     public void process(XPathContext context) throws XPathException {
-        SequenceReceiver out = context.getReceiver();
+        Receiver out = context.getReceiver();
         if (value instanceof Item) {
-            out.append((Item) value, getLocation(), NodeInfo.ALL_NAMESPACES);
+            out.append((Item) value, getLocation(), ReceiverOptions.ALL_NAMESPACES);
         } else {
-            SequenceIterator iter = value.iterate();
-            Item it;
-            while ((it = iter.next()) != null) {
-                out.append(it, getLocation(), NodeInfo.ALL_NAMESPACES);
-            }
+            value.iterate().forEachOrFail(it -> {
+                out.append(it, getLocation(), ReceiverOptions.ALL_NAMESPACES);
+            });
         }
     }
 
@@ -461,7 +459,7 @@ public class Literal extends Expression {
                     return false;
                 }
                 if (n0) {
-                    if (((NodeInfo) m0).isSameNodeInfo((NodeInfo) m1)) {
+                    if (((NodeInfo) m0).equals((NodeInfo) m1)) {
                         continue;
                     } else {
                         return false;
@@ -540,16 +538,16 @@ public class Literal extends Expression {
         } else if (value instanceof MapItem) {
             out.startElement("map");
             out.emitAttribute("size", "" + ((MapItem) value).size());
-            for (KeyValuePair kvp : (MapItem) value) {
+            for (KeyValuePair kvp : ((MapItem) value).keyValuePairs()) {
                 exportAtomicValue(kvp.key, out);
                 exportValue(kvp.value, out);
             }
             out.endElement();
         } else if (value instanceof Function) {
             ((Function) value).export(out);
-        } else if (value instanceof ObjectValue) {
+        } else if (value instanceof ExternalObject) {
             out.startElement("externalObject");
-            out.emitAttribute("class", ((ObjectValue) value).getObject().getClass().getName());
+            out.emitAttribute("class", ((ExternalObject) value).getObject().getClass().getName());
             out.endElement();
         } else {
             out.startElement("literal");
@@ -579,7 +577,7 @@ public class Literal extends Expression {
     }
 
     public static void exportAtomicValue(AtomicValue value, ExpressionPresenter out) throws XPathException {
-        if ("JS".equals(out.getOption("target"))) {
+        if ("JS".equals(((ExpressionPresenter.ExportOptions) out.getOptions()).target)) {
             value.checkValidInJavascript();
         }
         AtomicType type = value.getItemType();
@@ -615,7 +613,7 @@ public class Literal extends Expression {
         } else {
             out.startElement("atomic");
             out.emitAttribute("val", val);
-            out.emitAttribute("type", type.getTypeName());
+            out.emitAttribute("type", type.toExportString());
             out.endElement();
         }
     }
@@ -744,7 +742,7 @@ public class Literal extends Expression {
      * @return the Literal
      */
 
-    public static Literal makeLiteral(GroundedValue value) {
+    public static <T extends Item> Literal makeLiteral(GroundedValue<T> value) {
         value = value.reduce();
         if (value instanceof StringValue) {
             return new StringLiteral((StringValue) value);
@@ -757,17 +755,19 @@ public class Literal extends Expression {
         }
     }
 
+
     /**
      * Make a literal, taking the retained static context and location information from another
      * expression which is being simplified/optimized
-     * @param value the literal value
+     *
+     * @param value  the literal value
      * @param origin the expression whose context and location information is to be retained
      * @return the Literal
      */
 
     public static Literal makeLiteral(GroundedValue value, Expression origin) {
         Literal lit = makeLiteral(value);
-        lit.setRetainedStaticContext(origin.getRetainedStaticContext());
+        //lit.setRetainedStaticContextLocally(origin.getLocalRetainedStaticContext());
         ExpressionTool.copyLocationInfo(origin, lit);
         return lit;
     }
