@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -30,6 +30,8 @@ import java.net.MalformedURLException;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.net.URL;
+import java.time.Instant;
+import java.time.ZonedDateTime;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.HashMap;
@@ -111,12 +113,13 @@ public abstract class PJConverter {
         if (javaClass.isArray()) {
             Class memberClass = javaClass.getComponentType();
             if (memberClass == byte.class) {
-                // special case byte[], see bugs 3525 and 3818
+                // special-case byte[] which maps to xs:unsignedByte* - see bugs 3525, 3818
                 return SequenceType.makeSequenceType(BuiltInAtomicType.UNSIGNED_BYTE, StaticProperty.ALLOWS_ZERO_OR_MORE);
-            }
-            SequenceType memberType = getEquivalentSequenceType(memberClass);
-            if (memberType != null) {
-                return SequenceType.makeSequenceType(memberType.getPrimaryType(), StaticProperty.ALLOWS_ZERO_OR_MORE);
+            } else {
+                SequenceType memberType = getEquivalentSequenceType(memberClass);
+                if (memberType != null) {
+                    return SequenceType.makeSequenceType(memberType.getPrimaryType(), StaticProperty.ALLOWS_ZERO_OR_MORE);
+                }
             }
         }
         return jpmap.get(javaClass);
@@ -142,10 +145,7 @@ public abstract class PJConverter {
                 ItemType itemType = memberType == null ? null : memberType.getPrimaryType();
                 java.lang.reflect.Type collectionType = aType.getRawType();
                 int cardinality = -1;
-                if (collectionType.equals(Sequence.class)) {
-                    // TODO: but Sequence is no longer a generic class...
-                    cardinality = StaticProperty.ALLOWS_ZERO_OR_MORE;
-                } else if (collectionType.equals(ZeroOrOne.class)) {
+                if (collectionType.equals(ZeroOrOne.class)) {
                     cardinality = StaticProperty.ALLOWS_ZERO_OR_ONE;
                 } else if (collectionType.equals(One.class)) {
                     cardinality = StaticProperty.ALLOWS_ONE;
@@ -153,8 +153,6 @@ public abstract class PJConverter {
                     cardinality = StaticProperty.ALLOWS_ONE_OR_MORE;
                 } else if (collectionType.equals(ZeroOrMore.class)) {
                     cardinality = StaticProperty.ALLOWS_ZERO_OR_MORE;
-                } else {
-                    // no action
                 }
                 if (itemType != null && cardinality != -1) {
                     return SequenceType.makeSequenceType(itemType, cardinality);
@@ -267,9 +265,7 @@ public abstract class PJConverter {
                                 public Object convert(Sequence value, Class targetClass, XPathContext context) throws XPathException {
                                     try {
                                         return constructor.newInstance(value.head().getStringValue());
-                                    } catch (InstantiationException e) {
-                                        throw new XPathException(e);
-                                    } catch (IllegalAccessException e) {
+                                    } catch (InstantiationException | IllegalAccessException e) {
                                         throw new XPathException(e);
                                     } catch (InvocationTargetException e) {
                                         throw new XPathException("Cannot convert untypedAtomic to " + targetClass.getName() +
@@ -385,6 +381,10 @@ public abstract class PJConverter {
                         return CalendarValueToDate.INSTANCE;
                     } else if (targetClass == java.util.Calendar.class) {
                         return CalendarValueToCalendar.INSTANCE;
+                    } else if (targetClass == Instant.class) {
+                        return CalendarValueToInstant.INSTANCE;
+                    } else if (targetClass == ZonedDateTime.class) {
+                        return CalendarValueToZonedDateTime.INSTANCE;
                     } else {
                         throw cannotConvert(itemType, targetClass, config);
                     }
@@ -395,6 +395,10 @@ public abstract class PJConverter {
                         return CalendarValueToDate.INSTANCE;
                     } else if (targetClass == java.util.Calendar.class) {
                         return CalendarValueToCalendar.INSTANCE;
+                    } else if (targetClass == Instant.class) {
+                        return CalendarValueToInstant.INSTANCE;
+                    } else if (targetClass == ZonedDateTime.class) {
+                        return CalendarValueToZonedDateTime.INSTANCE;
                     } else {
                         throw cannotConvert(itemType, targetClass, config);
                     }
@@ -405,6 +409,10 @@ public abstract class PJConverter {
                         return CalendarValueToDate.INSTANCE;
                     } else if (targetClass == java.util.Calendar.class) {
                         return CalendarValueToCalendar.INSTANCE;
+                    } else if (targetClass == Instant.class) {
+                        return CalendarValueToInstant.INSTANCE;
+                    } else if (targetClass == ZonedDateTime.class) {
+                        return CalendarValueToZonedDateTime.INSTANCE;
                     } else {
                         throw cannotConvert(itemType, targetClass, config);
                     }
@@ -478,7 +486,7 @@ public abstract class PJConverter {
     }
 
     private static XPathException cannotConvert(ItemType source, Class target, Configuration config) {
-        return new XPathException("Cannot convert from " + source.toString() +
+        return new XPathException("Cannot convert from " + source +
                 " to " + target.getName());
     }
 
@@ -528,7 +536,7 @@ public abstract class PJConverter {
         public static final ToSequenceExtent INSTANCE = new ToSequenceExtent();
 
         public Object convert(Sequence value, Class targetClass, XPathContext context) throws XPathException {
-            return SequenceExtent.makeSequenceExtent(value.iterate());
+            return ((SequenceIterator<Item>) value.iterate()).materialize();
         }
 
     }
@@ -591,15 +599,14 @@ public abstract class PJConverter {
         }
 
         public Object convert(Sequence value, Class targetClass, XPathContext context) throws XPathException {
-            if (value instanceof ObjectValue && targetClass.isAssignableFrom(((ObjectValue) value).getObject().getClass())) {
-                return ((ObjectValue) value).getObject();
+            if (value instanceof ExternalObject && targetClass.isAssignableFrom(((ExternalObject) value).getObject().getClass())) {
+                return ((ExternalObject) value).getObject();
             }
             Class componentClass = targetClass.getComponentType();
             List<Object> list = new ArrayList<Object>(20);
             SequenceIterator iter = value.iterate();
-            while (true) {
-                Item item = iter.next();
-                if (item == null) break;
+            Item item;
+            while ((item = iter.next()) != null) {
                 Object obj = itemConverter.convert(item, componentClass, context);
                 if (obj != null) {
                     list.add(obj);
@@ -699,7 +706,7 @@ public abstract class PJConverter {
             if (targetClass.isAssignableFrom(value.getClass())) {
                 return value;
             } else {
-                GroundedValue gv = SequenceTool.toGroundedValue(value);
+                GroundedValue gv = value.materialize();
                 if (targetClass.isAssignableFrom(gv.getClass())) {
                     return gv;
                 }
@@ -728,15 +735,15 @@ public abstract class PJConverter {
             if (head == null) {
                 return null;
             }
-            if (!(head instanceof ObjectValue)) {
+            if (!(head instanceof ExternalObject)) {
                 if (Sequence.class.isAssignableFrom(targetClass)) {
-                    head = new ObjectValue(value);
+                    head = (ExternalObject<Sequence>) new ObjectValue<>(value);
                 } else {
                     throw new XPathException("Expected external object of class " + targetClass.getName() +
                             ", got " + head.getClass());
                 }
             }
-            Object obj = ((ObjectValue) head).getObject();
+            Object obj = ((ExternalObject) head).getObject();
             if (!targetClass.isAssignableFrom(obj.getClass())) {
                 throw new XPathException("External object has wrong class (is "
                         + obj.getClass().getName() + ", expected " + targetClass.getName() + ")");
@@ -934,7 +941,29 @@ public abstract class PJConverter {
 
         public Object convert(Sequence value, Class targetClass, XPathContext context) throws XPathException {
             QualifiedNameValue qv = (QualifiedNameValue) value.head();
-            return (qv == null ? null : qv.toJaxpQName());
+            return qv == null ? null : qv.toJaxpQName();
+        }
+
+    }
+
+    public static class CalendarValueToInstant extends PJConverter {
+
+        public static final CalendarValueToInstant INSTANCE = new CalendarValueToInstant();
+
+        public Object convert(Sequence value, Class targetClass, XPathContext context) throws XPathException {
+            CalendarValue cv = (CalendarValue) value.head();
+            return cv == null ? null : cv.toDateTime().toJavaInstant();
+        }
+
+    }
+
+    public static class CalendarValueToZonedDateTime extends PJConverter {
+
+        public static final CalendarValueToZonedDateTime INSTANCE = new CalendarValueToZonedDateTime();
+
+        public Object convert(Sequence value, Class targetClass, XPathContext context) throws XPathException {
+            CalendarValue cv = (CalendarValue) value.head();
+            return cv == null ? null : cv.toDateTime().toZonedDateTime();
         }
 
     }
@@ -1030,7 +1059,7 @@ public abstract class PJConverter {
         public Object convert(Sequence value, Class targetClass, XPathContext context)
                 throws XPathException {
             Configuration config = context.getConfiguration();
-            GroundedValue gv = SequenceTool.toGroundedValue(value);
+            GroundedValue gv = value.materialize();
             PJConverter converter = allocate(
                     config, SequenceTool.getItemType(gv, config.getTypeHierarchy()), SequenceTool.getCardinality(gv), targetClass);
             if (converter instanceof General) {
