@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,16 +9,14 @@ package net.sf.saxon.expr;
 
 import net.sf.saxon.expr.parser.*;
 import net.sf.saxon.om.GroundedValue;
+import net.sf.saxon.om.Item;
 import net.sf.saxon.om.SequenceIterator;
 import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.type.BuiltInAtomicType;
 import net.sf.saxon.type.ItemType;
 import net.sf.saxon.type.UType;
-import net.sf.saxon.value.Int64Value;
-import net.sf.saxon.value.IntegerRange;
-import net.sf.saxon.value.IntegerValue;
-import net.sf.saxon.value.SequenceType;
+import net.sf.saxon.value.*;
 
 
 /**
@@ -48,7 +46,8 @@ public class RangeExpression extends BinaryExpression {
 
     /*@NotNull*/
     public Expression typeCheck(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
-        typeCheckChildren(visitor, contextInfo);
+        getLhs().typeCheck(visitor, contextInfo);
+        getRhs().typeCheck(visitor, contextInfo);
 
         boolean backCompat = visitor.getStaticContext().isInBackwardsCompatibleMode();
         TypeChecker tc = visitor.getConfiguration().getTypeChecker(backCompat);
@@ -65,7 +64,6 @@ public class RangeExpression extends BinaryExpression {
 
     /**
      * Perform optimisation of an expression and its subexpressions.
-     * <p/>
      * <p>This method is called after all references to functions and variables have been resolved
      * to the declaration of the function or variable, and after all type checking has been done.</p>
      *
@@ -81,7 +79,8 @@ public class RangeExpression extends BinaryExpression {
 
     /*@NotNull*/
     public Expression optimize(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
-        optimizeChildren(visitor, contextInfo);
+        getLhs().optimize(visitor, contextInfo);
+        getRhs().optimize(visitor, contextInfo);
         return makeConstantRange();
 
     }
@@ -97,12 +96,12 @@ public class RangeExpression extends BinaryExpression {
                 if (i0 > i1) {
                     result = Literal.makeEmptySequence();
                 } else if (i0 == i1) {
-                    result = Literal.makeLiteral(Int64Value.makeIntegerValue(i0));
+                    result = Literal.makeLiteral(Int64Value.makeIntegerValue(i0), this);
                 } else {
                     if (i1 - i0 > Integer.MAX_VALUE) {
                         throw new XPathException("Maximum length of sequence in Saxon is " + Integer.MAX_VALUE, "XPDY0130");
                     }
-                    result = Literal.makeLiteral(new IntegerRange(i0, i1));
+                    result = Literal.makeLiteral(new IntegerRange(i0, i1), this);
                 }
                 ExpressionTool.copyLocationInfo(this, result);
                 return result;
@@ -225,7 +224,7 @@ public class RangeExpression extends BinaryExpression {
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<? extends Item> iterate(XPathContext context) throws XPathException {
         IntegerValue av1 = (IntegerValue) getLhsExpression().evaluateItem(context);
         IntegerValue av2 = (IntegerValue) getRhsExpression().evaluateItem(context);
         return RangeIterator.makeRangeIterator(av1, av2);
