@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -21,6 +21,7 @@ import net.sf.saxon.type.Type;
 import net.sf.saxon.type.UType;
 import net.sf.saxon.value.Cardinality;
 import net.sf.saxon.value.IntegerValue;
+import net.sf.saxon.value.SequenceType;
 
 import java.util.ArrayList;
 import java.util.List;
@@ -35,7 +36,13 @@ import java.util.List;
  */
 
 public class RawSlashExpression extends BinaryExpression
-        implements ContextSwitchingExpression, ContextMappingFunction {
+        implements ContextSwitchingExpression, ContextMappingFunction<Item> {
+
+    // TODO: (2018-03-27) is this class actually needed? It gets created by the ExpressionParser, and
+    // the typecheck() method then turns it into a regular SlashExpression, optionally with sorting
+    // into document order. Presumably the idea is to avoid pessimistically adding a document sorter
+    // to every path expression when in most cases it's not necessary. Perhaps we could use an ordinary
+    // SlashExpression and label it as "raw" (meaning not yet typechecked)?
 
     /**
      * Constructor
@@ -132,7 +139,7 @@ public class RawSlashExpression extends BinaryExpression
      * inference rules defined in the XSLT 3.0 specification.
      *
      * @return the static item type of the expression according to the XSLT 3.0 defined rules
-     * @param contextItemType
+     * @param contextItemType the static item type of the context item
      */
     @Override
     public UType getStaticUType(UType contextItemType) {
@@ -168,6 +175,21 @@ public class RawSlashExpression extends BinaryExpression
         // Check the first operand
         getLhs().typeCheck(visitor, contextInfo);
 
+        // If the first expression is known to be empty, just return empty without checking the step expression.
+        // (Checking the step expression can cause spurious errors, such as "the context item is absent")
+
+        if (Literal.isEmptySequence(getStart())) {
+            return getStart();
+        }
+
+        // The first operand must be of type node()*
+
+        Configuration config = visitor.getConfiguration();
+        TypeChecker tc = config.getTypeChecker(false);
+        RoleDiagnostic role0 = new RoleDiagnostic(RoleDiagnostic.BINARY_EXPR, "/", 0);
+        role0.setErrorCode("XPTY0019");
+        setStart(tc.staticTypeCheck(getStart(), SequenceType.NODE_SEQUENCE, role0, visitor));
+
         // Now check the second operand
         ItemType startType = getStart().getItemType();
         if (startType.equals(ErrorType.getInstance())) {
@@ -186,10 +208,11 @@ public class RawSlashExpression extends BinaryExpression
         if (t.overlaps(UType.ANY_NODE)) {
             if (t.overlaps(UType.ANY_ATOMIC.union(UType.FUNCTION))) {
                 result = new HomogeneityChecker(result);
-            } else if ((e2.getSpecialProperties() & StaticProperty.ORDERED_NODESET) != 0) {
+            } else if (e2.hasSpecialProperty(StaticProperty.ORDERED_NODESET)) {
                 // no action
-            } else if ((e2.getSpecialProperties() & StaticProperty.REVERSE_DOCUMENT_ORDER) != 0) {
+            } else if (e2.hasSpecialProperty(StaticProperty.REVERSE_DOCUMENT_ORDER)) {
                 result = SystemFunction.makeCall("reverse", getRetainedStaticContext(), e2);
+                assert result != null;
             } else {
                 result = new DocumentSorter(e2);
             }
@@ -210,13 +233,13 @@ public class RawSlashExpression extends BinaryExpression
      */
     @Override
     public int getCost() {
-        int lh = getLhsExpression().getCost();
+        int factor = Cardinality.allowsMany(getLhsExpression().getCardinality()) ? 10 : 1;
+        int lh = getLhsExpression().getCost() + 1;
         int rh = getRhsExpression().getCost();
-        double product = (double) lh * (double) rh;
-        return product > 1e7 ? 10000000 : Math.max(lh * rh, 10000000);
+        double product = (double) lh + factor * (double) rh;
+        return product > 1e7 ? 10000001 : Math.max(lh + factor * rh, 10000001);
     }
 
-
     /**
      * Replace this expression by a simpler expression that delivers the results without regard
      * to order.
@@ -238,7 +261,7 @@ public class RawSlashExpression extends BinaryExpression
      * by an expression in a source tree.
      *
      * @param pathMap        the PathMap to which the expression should be added
-     * @param pathMapNodeSet
+     * @param pathMapNodeSet the PathMapNodeSet to which the paths embodied in this expression should be added
      * @return the pathMapNode representing the focus established by this expression, in the case where this
      *         expression is the first operand of a path expression or filter expression
      */
@@ -266,7 +289,7 @@ public class RawSlashExpression extends BinaryExpression
      * Copy an expression. This makes a deep copy.
      *
      * @return the copy of the original expression
-     * @param rebindings
+     * @param rebindings variables that need to be re-bound
      */
 
     /*@NotNull*/
@@ -291,24 +314,23 @@ public class RawSlashExpression extends BinaryExpression
      * Convert this expression to an equivalent XSLT pattern
      *
      * @param config the Saxon configuration
-     * @param is30   true if this is XSLT 3.0
      * @return the equivalent pattern
      * @throws XPathException
      *          if conversion is not possible
      */
     @Override
-    public Pattern toPattern(Configuration config, boolean is30) throws XPathException {
+    public Pattern toPattern(Configuration config) throws XPathException {
         Expression head = getLeadingSteps();
         Expression tail = getLastStep();
         if (head instanceof ItemChecker) {
             // No need to typecheck the context item
             ItemChecker checker = (ItemChecker) head;
             if (checker.getBaseExpression() instanceof ContextItemExpression) {
-                return tail.toPattern(config, is30);
+                return tail.toPattern(config);
             }
         }
 
-        Pattern tailPattern = tail.toPattern(config, is30);
+        Pattern tailPattern = tail.toPattern(config);
         if (tailPattern instanceof NodeTestPattern) {
             if (tailPattern.getItemType() instanceof ErrorType) {
                 return tailPattern;
@@ -326,13 +348,13 @@ public class RawSlashExpression extends BinaryExpression
                 if (mid.getAxis() == AxisInfo.DESCENDANT_OR_SELF &&
                         (mid.getNodeTest() == null || mid.getNodeTest() instanceof AnyNodeTest)) {
                     axis = AxisInfo.ANCESTOR;
-                    headPattern = start.getSelectExpression().toPattern(config, is30);
+                    headPattern = start.getSelectExpression().toPattern(config);
                 }
             }
         }
         if (headPattern == null) {
             axis = PatternMaker.getAxisForPathStep(tail);
-            headPattern = head.toPattern(config, is30);
+            headPattern = head.toPattern(config);
         }
         return new AncestorQualifiedPattern(tailPattern, headPattern, axis);
     }
@@ -346,7 +368,7 @@ public class RawSlashExpression extends BinaryExpression
             return false;
         }
         RawSlashExpression p = (RawSlashExpression) other;
-        return getStart().equals(p.getStart()) && getStep().equals(p.getStep());
+        return getStart().isEqual(p.getStart()) && getStep().isEqual(p.getStep());
     }
 
     /**
@@ -354,7 +376,7 @@ public class RawSlashExpression extends BinaryExpression
      */
 
     public int computeHashCode() {
-        return "SlashExpression".hashCode() + getStart().computeHashCode() + getStep().computeHashCode();
+        return "SlashExpression".hashCode() + getStart().hashCode() + getStep().hashCode();
     }
 
     /**
@@ -364,16 +386,15 @@ public class RawSlashExpression extends BinaryExpression
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(final XPathContext context) throws XPathException {
+    public SequenceIterator<? extends Item> iterate(final XPathContext context) throws XPathException {
 
         // This class delivers the result of the path expression in unsorted order,
         // without removal of duplicates. If sorting and deduplication are needed,
         // this is achieved by wrapping the path expression in a DocumentSorter
 
-        FocusIterator result = new FocusTrackingIterator(getStart().iterate(context));
         XPathContext context2 = context.newMinorContext();
-        context2.setCurrentIterator(result);
-        return new ContextMappingIterator<Item>(this, context2);
+        context2.trackFocus(getStart().iterate(context));
+        return new ContextMappingIterator<>(this, context2);
     }
 
     /**
@@ -381,7 +402,7 @@ public class RawSlashExpression extends BinaryExpression
      * returned by the child.
      */
 
-    public SequenceIterator map(XPathContext context) throws XPathException {
+    public SequenceIterator<? extends Item> map(XPathContext context) throws XPathException {
         return getStep().iterate(context);
     }
 
@@ -441,7 +462,7 @@ public class RawSlashExpression extends BinaryExpression
 
     public Expression getRemainingSteps() {
         if (getStart() instanceof RawSlashExpression) {
-            List<Expression> list = new ArrayList<Expression>(8);
+            List<Expression> list = new ArrayList<>(8);
             gatherSteps(list);
             Expression rem = rebuildSteps(list.subList(1, list.size()));
             ExpressionTool.copyLocationInfo(this, rem);
@@ -506,7 +527,7 @@ public class RawSlashExpression extends BinaryExpression
 
     public Expression getLeadingSteps() {
         if (getStep() instanceof RawSlashExpression) {
-            List<Expression> list = new ArrayList<Expression>(8);
+            List<Expression> list = new ArrayList<>(8);
             gatherSteps(list);
             Expression rem = rebuildSteps(list.subList(0, list.size()-1));
             ExpressionTool.copyLocationInfo(this, rem);
@@ -525,14 +546,7 @@ public class RawSlashExpression extends BinaryExpression
 
     public boolean isAbsolute() {
         Expression first = getFirstStep();
-        if (first.getItemType().getPrimitiveType() == Type.DOCUMENT) {
-            return true;
-        }
-        // This second test allows keys to be built. See XMark q9.
-//        if (first instanceof AxisExpression && ((AxisExpression)first).getContextItemType().getPrimitiveType() == Type.DOCUMENT) {
-//            return true;
-//        };
-        return false;
+        return first.getItemType().getPrimitiveType() == Type.DOCUMENT;
     }
 
 
