@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -39,6 +39,8 @@ import java.util.Stack;
 public class SlashExpression extends BinaryExpression
         implements ContextSwitchingExpression, ContextMappingFunction {
 
+    boolean contextFree;
+
     /**
      * Constructor
      *
@@ -205,16 +207,16 @@ public class SlashExpression extends BinaryExpression
         }
 
         if (getStart() instanceof ContextItemExpression &&
-                ((getStep().getSpecialProperties() & StaticProperty.ORDERED_NODESET) != 0)) {
+                getStep().hasSpecialProperty(StaticProperty.ORDERED_NODESET)) {
             return getStep();
         }
 
         if (getStep() instanceof ContextItemExpression &&
-                ((getStart().getSpecialProperties() & StaticProperty.ORDERED_NODESET) != 0)) {
+                getStart().hasSpecialProperty(StaticProperty.ORDERED_NODESET)) {
             return getStart();
         }
 
-        if (getStep() instanceof AxisExpression && ((AxisExpression)getStep()).getAxis() == AxisInfo.SELF &&
+        if (getStep() instanceof AxisExpression && ((AxisExpression) getStep()).getAxis() == AxisInfo.SELF &&
                 config.getTypeHierarchy().isSubType(startType, getStep().getItemType())) {
             return getStart();
         }
@@ -253,7 +255,7 @@ public class SlashExpression extends BinaryExpression
             }
             ContextItemExpression cie = new ContextItemExpression();
             ExpressionTool.copyLocationInfo(this, cie);
-            st = ExpressionTool.makePathExpression(cie, stax.copy(new RebindingMap()), false);
+            st = ExpressionTool.makePathExpression(cie, stax.copy(new RebindingMap()));
             ExpressionTool.copyLocationInfo(this, st);
         }
 
@@ -311,7 +313,7 @@ public class SlashExpression extends BinaryExpression
             //System.err.println("as this:");
             //    new PathExpression(startPath.start, newStep).display(10);
 
-            Expression newPath = ExpressionTool.makePathExpression(startPath.getStart(), newStep, false);
+            Expression newPath = ExpressionTool.makePathExpression(startPath.getStart(), newStep);
             if (!(newPath instanceof SlashExpression)) {
                 return null;
             }
@@ -326,8 +328,8 @@ public class SlashExpression extends BinaryExpression
             Expression newStep =
                     new AxisExpression(AxisInfo.DESCENDANT_OR_SELF, NodeKindTest.ELEMENT);
             ExpressionTool.copyLocationInfo(this, newStep);
-            Expression e2 = ExpressionTool.makePathExpression(startPath.getStart(), newStep, false);
-            Expression e3 = ExpressionTool.makePathExpression(e2, getStep(), false);
+            Expression e2 = ExpressionTool.makePathExpression(startPath.getStart(), newStep);
+            Expression e3 = ExpressionTool.makePathExpression(e2, getStep());
             if (!(e3 instanceof SlashExpression)) {
                 return null;
             }
@@ -342,7 +344,7 @@ public class SlashExpression extends BinaryExpression
     public Expression optimize(ExpressionVisitor visitor, ContextItemStaticInfo contextItemType) throws XPathException {
         Configuration config = visitor.getConfiguration();
         TypeHierarchy th = config.getTypeHierarchy();
-        Optimizer opt = config.obtainOptimizer();
+        Optimizer opt = visitor.obtainOptimizer();
 
         getLhs().optimize(visitor, contextItemType);
         ContextItemStaticInfo cit = visitor.getConfiguration().makeContextItemStaticInfo(getStart().getItemType(), false);
@@ -355,7 +357,6 @@ public class SlashExpression extends BinaryExpression
 
         if (getStart() instanceof RootExpression && th.isSubType(contextItemType.getItemType(), NodeKindTest.DOCUMENT)) {
             // remove unnecessary leading "/" - helps streaming
-            // TODO: but we added the RootExpression ourselves, in tryToMakeAbsolute()...
             return getStep();
         }
 
@@ -367,7 +368,7 @@ public class SlashExpression extends BinaryExpression
             Expression lastStep = getLastStep();
             if (lastStep instanceof FilterExpression && !((FilterExpression) lastStep).isPositional(th)) {
                 Expression leading = getLeadingSteps();
-                Expression p2 = ExpressionTool.makePathExpression(leading, ((FilterExpression) lastStep).getSelectExpression(), false);
+                Expression p2 = ExpressionTool.makePathExpression(leading, ((FilterExpression) lastStep).getSelectExpression());
                 Expression f2 = new FilterExpression(p2, ((FilterExpression) lastStep).getFilter());
                 ExpressionTool.copyLocationInfo(this, f2);
                 return f2.optimize(visitor, contextItemType);
@@ -392,13 +393,19 @@ public class SlashExpression extends BinaryExpression
         // Replace $x/child::abcd by a SimpleStepExpression, to avoid the need for creating
         // a new dynamic context at run-time.
 
-        if (getStep() instanceof AxisExpression && !Cardinality.allowsMany(getStart().getCardinality())) {
-            SimpleStepExpression sse = new SimpleStepExpression(getStart(), getStep());
-            ExpressionTool.copyLocationInfo(this, sse);
-            sse.setParentExpression(getParentExpression());
-            return sse;
+        if (getStep() instanceof AxisExpression) {
+            if (!Cardinality.allowsMany(getStart().getCardinality())) {
+                SimpleStepExpression sse = new SimpleStepExpression(getStart(), getStep());
+                ExpressionTool.copyLocationInfo(this, sse);
+                sse.setParentExpression(getParentExpression());
+                return sse;
+            } else {
+                contextFree = true;
+            }
         }
 
+
+
         if (getStart() instanceof RootExpression && getStep().isCallOn(KeyFn.class)) {
             // This happens after optimizations to convert filter expressions to key() calls
             SystemFunctionCall keyCall = (SystemFunctionCall)getStep();
@@ -450,7 +457,7 @@ public class SlashExpression extends BinaryExpression
             if (contextItemType != null && contextItemType.getPrimitiveType() == Type.DOCUMENT) {
                 RootExpression root = new RootExpression();
                 ExpressionTool.copyLocationInfo(this, root);
-                Expression path = ExpressionTool.makePathExpression(root, this.copy(new RebindingMap()), false);
+                Expression path = ExpressionTool.makePathExpression(root, this.copy(new RebindingMap()));
                 if (!(path instanceof SlashExpression)) {
                     return null;
                 }
@@ -485,10 +492,11 @@ public class SlashExpression extends BinaryExpression
      */
     @Override
     public int getCost() {
+        int factor = Cardinality.allowsMany(getLhsExpression().getCardinality()) ? 10 : 1;
         int lh = getLhsExpression().getCost() + 1;
         int rh = getRhsExpression().getCost();
-        double product = (double)lh * (double)rh;
-        return product > 1e7 ? 10000000 : Math.max(lh * rh, 10000000);
+        double product = (double) lh  + factor * (double) rh;
+        return product > 1e7 ? 10000000 : Math.max(lh + factor * rh, 10000000);
     }
 
     public Expression tryToMakeSorted(ExpressionVisitor visitor, ContextItemStaticInfo contextItemType) throws XPathException {
@@ -497,7 +505,7 @@ public class SlashExpression extends BinaryExpression
 
         Configuration config = visitor.getConfiguration();
         TypeHierarchy th = config.getTypeHierarchy();
-        Optimizer opt = config.obtainOptimizer();
+        Optimizer opt = visitor.obtainOptimizer();
         Expression s1 = ExpressionTool.unfilteredExpression(getStart(), false);
         if (!(s1 instanceof AxisExpression && ((AxisExpression) s1).getAxis() == AxisInfo.DESCENDANT)) {
             return null;
@@ -527,23 +535,6 @@ public class SlashExpression extends BinaryExpression
             opt.trace("Rewrote descendant::X/child::Y as descendant::Y[parent::X]", k);
         }
         return k;
-
-//        if (start instanceof AxisExpression && ((AxisExpression)start).getAxis() == AxisInfo.DESCENDANT &&
-//                step instanceof AxisExpression && ((AxisExpression)step).getAxis() == AxisInfo.CHILD) {
-//            // TODO: we could be more ambitious and attempt this in the presence of non-positional filters;
-//            Expression k = new FilterExpression(
-//                    new AxisExpression(AxisInfo.DESCENDANT, ((AxisExpression)step).getNodeTest()),
-//                    new AxisExpression(AxisInfo.PARENT, ((AxisExpression)start).getNodeTest()));
-//            // If we're not starting at the root, ensure we go down at least one level
-//            if (!th.isSubType(contextItemType.itemType, NodeKindTest.DOCUMENT)) {
-//                k = new SlashExpression(new AxisExpression(AxisInfo.CHILD, NodeKindTest.ELEMENT), k);
-//                opt.trace("Rewrote descendant::X/child::Y as child::*/descendant::Y[parent::X]", k);
-//            } else {
-//                opt.trace("Rewrote descendant::X/child::Y as descendant::Y[parent::X]", k);
-//            }
-//            return k;
-//        }
-
     }
 
 
@@ -648,7 +639,7 @@ public class SlashExpression extends BinaryExpression
 
     /*@NotNull*/
     public Expression copy(RebindingMap rebindings) {
-        Expression exp = ExpressionTool.makePathExpression(getStart().copy(rebindings), getStep().copy(rebindings), false);
+        Expression exp = ExpressionTool.makePathExpression(getStart().copy(rebindings), getStep().copy(rebindings));
         ExpressionTool.copyLocationInfo(this, exp);
         return exp;
     }
@@ -666,7 +657,7 @@ public class SlashExpression extends BinaryExpression
 
         if ((stepProperties & StaticProperty.ALL_NODES_NEWLY_CREATED) != 0) {
             // Deem copies/snapshots to be in document order
-            return StaticProperty.ORDERED_NODESET | StaticProperty.PEER_NODESET | StaticProperty.NON_CREATIVE;
+            return StaticProperty.ORDERED_NODESET | StaticProperty.PEER_NODESET | StaticProperty.NO_NODES_NEWLY_CREATED;
         }
 
 
@@ -708,8 +699,8 @@ public class SlashExpression extends BinaryExpression
             p |= StaticProperty.REVERSE_DOCUMENT_ORDER;
         }
 
-        if ((startProperties & stepProperties & StaticProperty.NON_CREATIVE) != 0) {
-            p |= StaticProperty.NON_CREATIVE;
+        if ((startProperties & stepProperties & StaticProperty.NO_NODES_NEWLY_CREATED) != 0) {
+            p |= StaticProperty.NO_NODES_NEWLY_CREATED;
         }
 
         return p;
@@ -820,24 +811,23 @@ public class SlashExpression extends BinaryExpression
      * Convert this expression to an equivalent XSLT pattern
      *
      * @param config the Saxon configuration
-     * @param is30   true if this is XSLT 3.0
      * @return the equivalent pattern
      * @throws net.sf.saxon.trans.XPathException
      *          if conversion is not possible
      */
     @Override
-    public Pattern toPattern(Configuration config, boolean is30) throws XPathException {
+    public Pattern toPattern(Configuration config) throws XPathException {
         Expression head = getLeadingSteps();
         Expression tail = getLastStep();
         if (head instanceof ItemChecker) {
             // No need to typecheck the context item
             ItemChecker checker = (ItemChecker) head;
             if (checker.getBaseExpression() instanceof ContextItemExpression) {
-                return tail.toPattern(config, is30);
+                return tail.toPattern(config);
             }
         }
 
-        Pattern tailPattern = tail.toPattern(config, is30);
+        Pattern tailPattern = tail.toPattern(config);
         if (tailPattern instanceof NodeTestPattern) {
             if (tailPattern.getItemType() instanceof ErrorType) {
                 return tailPattern;
@@ -855,17 +845,39 @@ public class SlashExpression extends BinaryExpression
                 if (mid.getAxis() == AxisInfo.DESCENDANT_OR_SELF &&
                         (mid.getNodeTest() == null || mid.getNodeTest() instanceof AnyNodeTest)) {
                     axis = AxisInfo.ANCESTOR;
-                    headPattern = start.getSelectExpression().toPattern(config, is30);
+                    headPattern = start.getSelectExpression().toPattern(config);
                 }
             }
         }
         if (headPattern == null) {
             axis = PatternMaker.getAxisForPathStep(tail);
-            headPattern = head.toPattern(config, is30);
+            headPattern = head.toPattern(config);
         }
         return new AncestorQualifiedPattern(tailPattern, headPattern, axis);
     }
 
+    /**
+     * A SlashExpression is context free if the right-hand argument is an AxisExpression.
+     * This allows evaluation without creating a new XPathContext or FocusTrackingIterator;
+     * the expression can be evaluated as a simple flat-mapping function from nodes to nodes.
+     * @return true if the expression has been assessed as context-free, that is, if the
+     * right-hand operand is an AxisExpression
+     */
+
+    public boolean isContextFree() {
+        return contextFree;
+    }
+
+    /**
+     * Mark this expression as being context free (or not). This is done during reloading from a SEF file.
+     * @param free true if the expression has been assessed as context-free, that is, if the
+     * right-hand operand is an AxisExpression
+     */
+
+    public void setContextFree(boolean free) {
+        this.contextFree = free;
+    }
+
     /**
      * Is this expression the same as another expression?
      */
@@ -875,7 +887,7 @@ public class SlashExpression extends BinaryExpression
             return false;
         }
         SlashExpression p = (SlashExpression) other;
-        return getStart().equals(p.getStart()) && getStep().equals(p.getStep());
+        return getStart().isEqual(p.getStart()) && getStep().isEqual(p.getStep());
     }
 
     /**
@@ -883,7 +895,7 @@ public class SlashExpression extends BinaryExpression
      */
 
     public int computeHashCode() {
-        return "SlashExpression".hashCode() + getStart().computeHashCode() + getStep().computeHashCode();
+        return "SlashExpression".hashCode() + getStart().hashCode() + getStep().hashCode();
     }
 
     /**
@@ -893,15 +905,19 @@ public class SlashExpression extends BinaryExpression
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(final XPathContext context) throws XPathException {
+    public SequenceIterator<? extends Item> iterate(final XPathContext context) throws XPathException {
 
         // This class delivers the result of the path expression in unsorted order,
         // without removal of duplicates. If sorting and deduplication are needed,
         // this is achieved by wrapping the path expression in a DocumentSorter
 
-        FocusIterator result = new FocusTrackingIterator(getStart().iterate(context));
+        if (contextFree) {
+            return new MappingIterator<>((SequenceIterator<? extends NodeInfo>)getStart().iterate(context),
+                                         ((AxisExpression) getRhsExpression())::iterate);
+        }
+
         XPathContext context2 = context.newMinorContext();
-        context2.setCurrentIterator(result);
+        context2.trackFocus(getStart().iterate(context));
         return new ContextMappingIterator<Item>(this, context2);
     }
 
@@ -923,6 +939,8 @@ public class SlashExpression extends BinaryExpression
         destination.startElement("slash", this);
         if (this instanceof SimpleStepExpression) {
             destination.emitAttribute("simple", "1");
+        } else if (isContextFree()) {
+            destination.emitAttribute("simple", "2");
         }
         getStart().export(destination);
         getStep().export(destination);
@@ -1056,15 +1074,7 @@ public class SlashExpression extends BinaryExpression
      */
 
     public boolean isAbsolute() {
-        Expression first = getFirstStep();
-        if (first.getItemType().getPrimitiveType() == Type.DOCUMENT) {
-            return true;
-        }
-        // This second test allows keys to be built. See XMark q9.
-//        if (first instanceof AxisExpression && ((AxisExpression)first).getContextItemType().getPrimitiveType() == Type.DOCUMENT) {
-//            return true;
-//        };
-        return false;
+        return getFirstStep().getItemType().getPrimitiveType() == Type.DOCUMENT;
     }
 
 
