@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -242,16 +242,24 @@ public abstract class StaticProperty {
 
     public static final int ATTRIBUTE_NS_NODESET = 1 << 21;
 
+    /**
+     * Expression property: this bit indicates that the expression will always create new
+     * nodes: more specifically, any node in the result of the expression is guaranteed
+     * to be a newly created node.
+     */
+
+    public static final int ALL_NODES_NEWLY_CREATED = 1 << 22;
+
     /**
      * Expression property: this bit is set in the case of an expression that will
      * never return newly created nodes, nor a value that depends on the identity
      * of newly created nodes (for example generate-id(new-node())). Expressions
-     * that do create new nodes cannot be moved out of loops as this could cause
+     * that are capable of creating new nodes cannot be moved out of loops as this could cause
      * too few nodes to be created: for example if f() creates a new node, then
      * count(for $i in 1 to 5 return f()) must be 5.
      */
 
-    public static final int NON_CREATIVE = 1 << 22;
+    public static final int NO_NODES_NEWLY_CREATED = 1 << 23;
 
     /**
      * Expression property: this bit is set in the case of an expression that delivers
@@ -259,7 +267,7 @@ public abstract class StaticProperty {
      * document as the context node).
      */
 
-    public static final int SINGLE_DOCUMENT_NODESET = 1 << 23;
+    public static final int SINGLE_DOCUMENT_NODESET = 1 << 24;
 
     /**
      * Expression property: this bit indicates that an expression has (or might have)
@@ -267,14 +275,14 @@ public abstract class StaticProperty {
      * certain instructions such as xsl:result-document and xsl:message.
      */
 
-    public static final int HAS_SIDE_EFFECTS = 1 << 24;
+    public static final int HAS_SIDE_EFFECTS = 1 << 25;
 
     /**
      * Expression property: this bit indicates that although the static type of the expression
      * permits untyped atomic values, it is known that the value will not be untyped atomic.
      */
 
-    public static final int NOT_UNTYPED_ATOMIC = 1 << 25;
+    public static final int NOT_UNTYPED_ATOMIC = 1 << 26;
 
     /**
      * Expression property: this bit indicates that in the result of an expression,
@@ -283,15 +291,8 @@ public abstract class StaticProperty {
      * no element children whose type annotation is anything other than xs:untyped
      */
 
-    public static final int ALL_NODES_UNTYPED = 1 << 26;
+    public static final int ALL_NODES_UNTYPED = 1 << 27;
 
-    /**
-     * Expression property: this bit indicates that the expression will always create new
-     * nodes: more specifically, any node in the result of the expression is guaranteed
-     * to be a newly created node.
-     */
-
-    public static final int ALL_NODES_NEWLY_CREATED = 1 << 27;
 
     /**
      * Mask to select all the dependency bits
@@ -324,7 +325,7 @@ public abstract class StaticProperty {
                     SUBTREE_NODESET |
                     ATTRIBUTE_NS_NODESET |
                     SINGLE_DOCUMENT_NODESET |
-                    NON_CREATIVE |
+                    NO_NODES_NEWLY_CREATED |
                     HAS_SIDE_EFFECTS |
                     NOT_UNTYPED_ATOMIC |
                     ALL_NODES_UNTYPED |
@@ -379,6 +380,9 @@ public abstract class StaticProperty {
         if ((props & DEPENDS_ON_RUNTIME_ENVIRONMENT) != 0) {
             s.append("E");
         }
+        if ((props & DEPENDS_ON_STATIC_CONTEXT) != 0) {
+            s.append("S");
+        }
         s.append(") C(");
         boolean m = Cardinality.allowsMany(props);
         boolean z = Cardinality.allowsZero(props);
@@ -395,7 +399,7 @@ public abstract class StaticProperty {
         if ((props & HAS_SIDE_EFFECTS) != 0) {
             s.append("E");
         }
-        if ((props & NON_CREATIVE) != 0) {
+        if ((props & NO_NODES_NEWLY_CREATED) != 0) {
             s.append("N");
         }
         if ((props & NOT_UNTYPED_ATOMIC) != 0) {
