@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -13,7 +13,6 @@ import net.sf.saxon.om.SequenceIterator;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.iter.ArrayIterator;
 import net.sf.saxon.tree.iter.EmptyIterator;
-import net.sf.saxon.tree.iter.GroundedIterator;
 import net.sf.saxon.tree.iter.LookaheadIterator;
 
 /**
@@ -21,10 +20,10 @@ import net.sf.saxon.tree.iter.LookaheadIterator;
  * The base sequence is represented by an iterator which is consumed in the process
  */
 
-public class TailIterator
-        implements SequenceIterator, LastPositionFinder, LookaheadIterator {
+public class TailIterator<T extends Item>
+        implements SequenceIterator<T>, LastPositionFinder, LookaheadIterator<T> {
 
-    private SequenceIterator base;
+    private SequenceIterator<T> base;
     private int start;
 
     /**
@@ -40,7 +39,7 @@ public class TailIterator
      *              when cloning the iterator or when calculating the value of last().
      */
 
-    private TailIterator(SequenceIterator base, int start) {
+    private TailIterator(SequenceIterator<T> base, int start) {
         this.base = base;
         this.start = start;
     }
@@ -49,10 +48,10 @@ public class TailIterator
      * Static factory method. Creates a TailIterator, unless the base Iterator is an
      * ArrayIterator, in which case it optimizes by creating a new ArrayIterator directly over the
      * underlying array. This optimization is important when doing recursion over a node-set using
-     * repeated calls of $nodes[position()>1]
+     * repeated calls of <code>$nodes[position()&gt;1]</code>
      *
      * @param base  An iteration of the items to be filtered
-     * @param start The position of the first item to be included (base 1). If <= 1, the whole of the
+     * @param start The position of the first item to be included (base 1). If &lt;= 1, the whole of the
      *              base sequence is returned
      * @return an iterator over the items in the sequence from the start item to the end of the sequence.
      *         The returned iterator will not necessarily be an instance of this class.
@@ -60,17 +59,17 @@ public class TailIterator
      *          if a dynamic error occurs
      */
 
-    public static SequenceIterator make(SequenceIterator base, int start) throws XPathException {
+    public static <T extends Item> SequenceIterator<T> make(SequenceIterator<T> base, int start) throws XPathException {
         if (start <= 1) {
             return base;
         } else if (base instanceof ArrayIterator) {
-            return ((ArrayIterator) base).makeSliceIterator(start, Integer.MAX_VALUE);
+            return ((ArrayIterator<T>) base).makeSliceIterator(start, Integer.MAX_VALUE);
         } else if ((base.getProperties() & SequenceIterator.GROUNDED) != 0) {
-            GroundedValue value = ((GroundedIterator) base).materialize();
+            GroundedValue<T> value = base.materialize();
             if (start > value.getLength()) {
                 return EmptyIterator.emptyIterator();
             } else {
-                return new ValueTailIterator(value, start - 1);
+                return new ValueTailIterator<>(value, start - 1);
             }
         } else {
             // discard the first n-1 items from the underlying iterator
@@ -80,12 +79,12 @@ public class TailIterator
                     return EmptyIterator.emptyIterator();
                 }
             }
-            return new TailIterator(base, start);
+            return new TailIterator<>(base, start);
         }
     }
 
 
-    public Item next() throws XPathException {
+    public T next() throws XPathException {
         return base.next();
     }
 
