@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -16,10 +16,7 @@ import net.sf.saxon.om.SequenceIterator;
 import net.sf.saxon.om.StructuredQName;
 import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trace.LocationKind;
-import net.sf.saxon.trans.SaxonErrorCode;
-import net.sf.saxon.trans.SymbolicName;
-import net.sf.saxon.trans.Visibility;
-import net.sf.saxon.trans.XPathException;
+import net.sf.saxon.trans.*;
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.type.AnyItemType;
 import net.sf.saxon.type.ItemType;
@@ -56,7 +53,7 @@ public class UserFunctionCall extends FunctionCall implements UserFunctionResolv
     private boolean beingInlined = false;
 
 
-    /*@Nullable*/ private int[] argumentEvaluationModes = null;
+    /*@Nullable*/ private Evaluator[] argumentEvaluators = null;
 
     public static final int NOT_TAIL_CALL = 0;
     public static final int FOREIGN_TAIL_CALL = 1;
@@ -184,16 +181,6 @@ public class UserFunctionCall extends FunctionCall implements UserFunctionResolv
         return function.getDeclaringComponent();
     }
 
-    /**
-     * Get the evaluation modes that have been determined for each of the arguments
-     *
-     * @return an array of integers representing the evaluation modes, one for each argument
-     */
-
-    public int[] getArgumentEvaluationModes() {
-        return argumentEvaluationModes;
-    }
-
     /**
      * Set the argument evaluation modes
      *
@@ -201,9 +188,71 @@ public class UserFunctionCall extends FunctionCall implements UserFunctionResolv
      */
 
     public void setArgumentEvaluationModes(int[] evalModes) {
-        argumentEvaluationModes = evalModes;
+        argumentEvaluators = new Evaluator[evalModes.length];
+        for (int i=0; i<evalModes.length; i++) {
+            argumentEvaluators[i] = Evaluator.evaluatorForCode(evalModes[i]);
+        }
     }
 
+
+    private static final int UNHANDLED_DEPENDENCIES =
+        StaticProperty.DEPENDS_ON_POSITION | StaticProperty.DEPENDS_ON_LAST |
+            StaticProperty.DEPENDS_ON_XSLT_CONTEXT | StaticProperty.DEPENDS_ON_USER_FUNCTIONS;
+
+    public void allocateArgumentEvaluators() {
+        argumentEvaluators = new Evaluator[getArity()];
+        int i=0;
+        for (Operand o : operands()) {
+            Expression arg = o.getChildExpression();
+            SequenceType required = function.getArgumentType(i);
+            int cardinality = required.getCardinality();
+
+            if (i == 0 && function.getDeclaredStreamability().isConsuming()) {
+                argumentEvaluators[i] = Evaluator.STREAMING_ARGUMENT;
+            } else if (function.getParameterDefinitions()[i].isIndexedVariable()) {
+                argumentEvaluators[i] = Evaluator.MAKE_INDEXED_VARIABLE;
+            } else {
+                if (arg instanceof Literal) {
+                    argumentEvaluators[i] = Evaluator.LITERAL;
+                } else if (arg instanceof VariableReference) {
+                    argumentEvaluators[i] = Evaluator.VARIABLE;
+                } else if (cardinality == StaticProperty.EXACTLY_ONE) {
+                    argumentEvaluators[i] = Evaluator.SINGLE_ITEM;
+                } else if ((arg.getDependencies() & UNHANDLED_DEPENDENCIES) != 0) {
+                    // If the argument contains a call to a user-defined function, then it might be a recursive call.
+                    // It's better to evaluate it now, rather than waiting until we are on a new stack frame, as
+                    // that can blow the stack if done repeatedly. (See test func42)
+                    // If the argument contains calls to position(), last(), regex-group(), current-group(),
+                    // current-merge-group(), etc, then in general we can't save the values in a Closure
+                    // so we need to evaluate the argument eagerly. (Tests position-0103, merge-096).
+                    argumentEvaluators[i] = Evaluator.EAGER_SEQUENCE;
+                } else if (!Cardinality.allowsMany(arg.getCardinality()) && arg.getCost() < 20) {
+                    // the argument is cheap to evaluate and doesn't use much memory...
+                    argumentEvaluators[i] = Evaluator.EAGER_SEQUENCE;
+                } else if (cardinality == StaticProperty.ALLOWS_ZERO_OR_ONE) {
+                    argumentEvaluators[i] = Evaluator.OPTIONAL_ITEM;
+                } else if (arg instanceof Block && ((Block) arg).isCandidateForSharedAppend()) {
+                    // If the expression is a Block, that is, it is appending a value to a sequence,
+                    // then we have the opportunity to use a shared list underpinning the old value and
+                    // the new. This takes precedence over lazy evaluation (it would be possible to do this
+                    // lazily, but more difficult). We currently do this for any Block that has a variable
+                    // reference as one of its subexpressions. The most common case is that the first argument is a reference
+                    // to an argument of recursive function, where the recursive function returns the result of
+                    // appending to the sequence.
+                    argumentEvaluators[i] = Evaluator.SHARED_APPEND;
+                } else {
+                    argumentEvaluators[i] = Evaluator.MEMO_CLOSURE;
+                }
+            }
+            i++;
+        }
+    }
+
+    public Evaluator[] getArgumentEvaluators() {
+        return argumentEvaluators;
+    }
+
+
     /**
      * Pre-evaluate a function at compile time. This version of the method suppresses
      * early evaluation by doing nothing.
@@ -274,13 +323,14 @@ public class UserFunctionCall extends FunctionCall implements UserFunctionResolv
     protected int computeSpecialProperties() {
         // Inherit the properties of the function being called if possible. But we have to prevent
         // looping when the function is recursive. For safety, we only consider the properties of the
-        // function body if it contains no further function calls.
+        // function body if it contains no further function calls. Also, we can only do this safely if
+        // the function is private or final
         if (function == null) {
             return super.computeSpecialProperties();
         } else if (function.getBody() != null &&
                 (function.getDeclaredVisibility() == Visibility.PRIVATE || function.getDeclaredVisibility() == Visibility.FINAL)) {
             int props;
-            List<UserFunction> calledFunctions = new ArrayList<UserFunction>();
+            List<UserFunction> calledFunctions = new ArrayList<>();
             ExpressionTool.gatherCalledFunctions(function.getBody(), calledFunctions);
             if (calledFunctions.isEmpty()) {
                 props = function.getBody().getSpecialProperties();
@@ -288,7 +338,7 @@ public class UserFunctionCall extends FunctionCall implements UserFunctionResolv
                 props = super.computeSpecialProperties();
             }
             if (function.getDeterminism() != UserFunction.Determinism.PROACTIVE) {
-                props |= StaticProperty.NON_CREATIVE;
+                props |= StaticProperty.NO_NODES_NEWLY_CREATED;
             }
             return props;
         } else {
@@ -299,7 +349,7 @@ public class UserFunctionCall extends FunctionCall implements UserFunctionResolv
     /**
      * Copy an expression. This makes a deep copy.
      *
-     * @param rebindings
+     * @param rebindings variable bindings that need to be changed
      * @return the copy of the original expression
      */
 
@@ -356,7 +406,7 @@ public class UserFunctionCall extends FunctionCall implements UserFunctionResolv
         //function.setOnFirstCall(new DeferredAction(visitor, contextItemType));
         Expression e = super.optimize(visitor, contextItemType);
         if (e == this && function != null) {
-            Expression e2 = visitor.getConfiguration().obtainOptimizer().tryInlineFunctionCall(
+            Expression e2 = visitor.obtainOptimizer().tryInlineFunctionCall(
                     this, visitor, contextItemType);
             if (e2 != this) {
                 return e2.optimize(visitor, contextItemType);
@@ -373,52 +423,12 @@ public class UserFunctionCall extends FunctionCall implements UserFunctionResolv
     @Override
     public void resetLocalStaticProperties() {
         super.resetLocalStaticProperties();
-        argumentEvaluationModes = null;
+        argumentEvaluators = null;
     }
-
-    /**
-     * Compute the evaluation mode of each argument
-     */
-
-    public synchronized void computeArgumentEvaluationModes() {
-        int numArgs = getArity();
-        argumentEvaluationModes = new int[numArgs];
-        for (int i = 0; i < numArgs; i++) {
-            if (i == 0 && function.getDeclaredStreamability().isConsuming()) {
-                argumentEvaluationModes[i] = ExpressionTool.STREAMING_ARGUMENT;
-            } else if (function.getParameterDefinitions()[i].isIndexedVariable()) {
-                argumentEvaluationModes[i] = ExpressionTool.MAKE_INDEXED_VARIABLE;
-            } else {
-                Expression arg = getArg(i);
-                if ((arg.getDependencies() & StaticProperty.DEPENDS_ON_USER_FUNCTIONS) != 0) {
-                    // if the argument contains a call to a user-defined function, then it might be a recursive call.
-                    // It's better to evaluate it now, rather than waiting until we are on a new stack frame, as
-                    // that can blow the stack if done repeatedly. (See test func42)
-                    argumentEvaluationModes[i] = ExpressionTool.eagerEvaluationMode(arg);
-                } else if (!Cardinality.allowsMany(arg.getCardinality()) && arg.getCost() < 20) {
-                    // the argument is cheap to evaluate and doesn't use much memory...
-                    argumentEvaluationModes[i] = ExpressionTool.eagerEvaluationMode(arg);
-                } else if (arg instanceof Block && ((Block) arg).isCandidateForSharedAppend()) {
-                    // If the expression is a Block, that is, it is appending a value to a sequence,
-                    // then we have the opportunity to use a shared list underpinning the old value and
-                    // the new. This takes precedence over lazy evaluation (it would be possible to do this
-                    // lazily, but more difficult). We currently do this for any Block that has a variable
-                    // reference as one of its subexpressions. The most common case is that the first argument is a reference
-                    // to an argument of recursive function, where the recursive function returns the result of
-                    // appending to the sequence.
-                    argumentEvaluationModes[i] = ExpressionTool.SHARED_APPEND_EXPRESSION;
-                } else {
-                    argumentEvaluationModes[i] = ExpressionTool.MAKE_MEMO_CLOSURE;
-                }
-            }
-        }
-    }
-
-
+    
     /**
      * Add a representation of this expression to a PathMap. The PathMap captures a map of the nodes visited
      * by an expression in a source tree.
-     * <p/>
      * <p>The default implementation of this method assumes that an expression does no navigation other than
      * the navigation done by evaluating its subexpressions, and that the subexpressions are evaluated in the
      * same context as the containing expression. The method must be overridden for any expression
@@ -476,7 +486,7 @@ public class UserFunctionCall extends FunctionCall implements UserFunctionResolv
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(XPathContext c) throws XPathException {
+    public SequenceIterator<? extends Item> iterate(XPathContext c) throws XPathException {
         return callFunction(c).iterate();
     }
 
@@ -497,6 +507,8 @@ public class UserFunctionCall extends FunctionCall implements UserFunctionResolv
         function.callUpdating(actualArgs, c2, pul);
     }
 
+    private static int depth = 0;
+
     /**
      * This is the method that actually does the function call (in pull mode)
      *
@@ -504,7 +516,7 @@ public class UserFunctionCall extends FunctionCall implements UserFunctionResolv
      * @return the result of the function
      * @throws XPathException if dynamic errors occur
      */
-    private Sequence callFunction(XPathContext context) throws XPathException {
+    private Sequence<? extends Item> callFunction(XPathContext context) throws XPathException {
         UserFunction targetFunction;
         Sequence[] actualArgs = evaluateArguments(context);
         XPathContextMajor c2;
@@ -531,16 +543,19 @@ public class UserFunctionCall extends FunctionCall implements UserFunctionResolv
         }
 
         try {
-            Sequence s = targetFunction.call(c2, actualArgs);
-            return s;
+            return targetFunction.call(c2, actualArgs);
+        } catch (UncheckedXPathException e) {
+            XPathException xe = e.getXPathException();
+            xe.maybeSetLocation(getLocation());
+            throw xe;
         } catch (StackOverflowError err) {
-            //err.printStackTrace();
+            err.printStackTrace();
             throw new XPathException("Too many nested function calls. May be due to infinite recursion",
                                      SaxonErrorCode.SXLM0001, getLocation());
         }
     }
 
-    private void requestTailCall(XPathContext context, Sequence[] actualArgs) throws XPathException {
+    private void requestTailCall(XPathContext context, Sequence[] actualArgs) {
         if (bindingSlot >= 0) {
             TailCallLoop.TailCallComponent info = new TailCallLoop.TailCallComponent();
             Component target = getTargetComponent(context);
@@ -560,7 +575,7 @@ public class UserFunctionCall extends FunctionCall implements UserFunctionResolv
      * Process the function call in push mode
      *
      * @param context the XPath dynamic context
-     * @throws XPathException
+     * @throws XPathException if a dynamic error occurs
      */
 
     public void process(XPathContext context) throws XPathException {
@@ -611,28 +626,22 @@ public class UserFunctionCall extends FunctionCall implements UserFunctionResolv
         int numArgs = getArity();
         Sequence[] actualArgs = new Sequence[numArgs];
         synchronized(this) {
-            if (argumentEvaluationModes == null) {
+            if (argumentEvaluators == null) {
                 // should have been done at compile time
-                computeArgumentEvaluationModes();
+                allocateArgumentEvaluators();
             }
         }
         for (int i = 0; i < numArgs; i++) {
 
-            int refs = 10; // was function.getParameterDefinitions()[i].getReferenceCount();
-            int evalMode = argumentEvaluationModes[i];
-            if (evalMode == ExpressionTool.STREAMING_ARGUMENT && !streamed) {
-                evalMode = ExpressionTool.ITERATE_AND_MATERIALIZE;
+            Evaluator eval = argumentEvaluators[i];
+            if (eval == Evaluator.STREAMING_ARGUMENT && !streamed) {
+                eval = Evaluator.EAGER_SEQUENCE;
             }
-            actualArgs[i] = ExpressionTool.evaluate(getArg(i), evalMode, c, refs);
+            actualArgs[i] = eval.evaluate(getArg(i), c);
 
             if (actualArgs[i] == null) {
                 actualArgs[i] = EmptySequence.getInstance();
             }
-//            // If the argument has come in as a (non-memo) closure but there are multiple references to it,
-//            // then we materialize it in memory now. This shouldn't really happen but it does (tour.xq)
-//            if (refs > 1 && actualArgs[i] instanceof Closure) {
-//                actualArgs[i] = ((Closure) actualArgs[i]).reduce();
-//            }
         }
         return actualArgs;
     }
@@ -645,16 +654,15 @@ public class UserFunctionCall extends FunctionCall implements UserFunctionResolv
     public void export(ExpressionPresenter out) throws XPathException {
         out.startElement("ufCall", this);
         if (getFunctionName() != null) {
-            out.emitAttribute("name", getFunctionName().getEQName());
+            out.emitAttribute("name", getFunctionName());
             out.emitAttribute("tailCall",
                               tailCall == NOT_TAIL_CALL ? "false" : tailCall == SELF_TAIL_CALL ? "self" : "foreign");
         }
         out.emitAttribute("bSlot", "" + getBindingSlot());
-        if (getArgumentEvaluationModes() != null && getArity() > 0) {
+        if (argumentEvaluators != null && getArity() > 0) {
             FastStringBuffer fsb = new FastStringBuffer(FastStringBuffer.C64);
-            int[] e = getArgumentEvaluationModes();
-            for (int i : e) {
-                fsb.append(i + " ");
+            for (Evaluator e : argumentEvaluators) {
+                fsb.append(e.getCode() + " ");
             }
             out.emitAttribute("eval", Whitespace.trim(fsb));
         }
