@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -92,7 +92,7 @@ public final class ValueComparison extends BinaryExpression implements Compariso
      */
 
     public boolean convertsUntypedToOther() {
-        return false;
+        return comparer instanceof UntypedNumericComparer;
     }
 
     /**
@@ -134,17 +134,18 @@ public final class ValueComparison extends BinaryExpression implements Compariso
     public Expression typeCheck(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
 
         resetLocalStaticProperties();
-        typeCheckChildren(visitor, contextInfo);
+        getLhs().typeCheck(visitor, contextInfo);
+        getRhs().typeCheck(visitor, contextInfo);
 
         Configuration config = visitor.getConfiguration();
         StaticContext env = visitor.getStaticContext();
 
         if (Literal.isEmptySequence(getLhsExpression())) {
-            return resultWhenEmpty == null ? getLhsExpression() : Literal.makeLiteral(resultWhenEmpty);
+            return resultWhenEmpty == null ? getLhsExpression() : Literal.makeLiteral(resultWhenEmpty, this);
         }
 
         if (Literal.isEmptySequence(getRhsExpression())) {
-            return resultWhenEmpty == null ? getRhsExpression() : Literal.makeLiteral(resultWhenEmpty);
+            return resultWhenEmpty == null ? getRhsExpression() : Literal.makeLiteral(resultWhenEmpty, this);
         }
 
         if (comparer instanceof UntypedNumericComparer) {
@@ -217,20 +218,8 @@ public final class ValueComparison extends BinaryExpression implements Compariso
             }
         }
         if (!(operator == Token.FEQ || operator == Token.FNE)) {
-            if (!p0.isOrdered(true)) {
-                XPathException err = new XPathException("Type " + t0.toString() + " is not an ordered type");
-                err.setErrorCode("XPTY0004");
-                err.setIsTypeError(true);
-                err.setLocation(getLocation());
-                throw err;
-            }
-            if (!p1.isOrdered(true)) {
-                XPathException err = new XPathException("Type " + t1.toString() + " is not an ordered type");
-                err.setErrorCode("XPTY0004");
-                err.setIsTypeError(true);
-                err.setLocation(getLocation());
-                throw err;
-            }
+            mustBeOrdered(t0, p0);
+            mustBeOrdered(t1, p1);
         }
 
         if (comparer == null) {
@@ -246,9 +235,18 @@ public final class ValueComparison extends BinaryExpression implements Compariso
         return this;
     }
 
+    private void mustBeOrdered(PlainType t1, BuiltInAtomicType p1) throws XPathException {
+        if (!p1.isOrdered(true)) {
+            XPathException err = new XPathException("Type " + t1.toString() + " is not an ordered type");
+            err.setErrorCode("XPTY0004");
+            err.setIsTypeError(true);
+            err.setLocation(getLocation());
+            throw err;
+        }
+    }
+
     /**
      * Perform optimisation of an expression and its subexpressions.
-     * <p/>
      * <p>This method is called after all references to functions and variables have been resolved
      * to the declaration of the function or variable, and after all type checking has been done.</p>
      *
@@ -267,7 +265,8 @@ public final class ValueComparison extends BinaryExpression implements Compariso
 
         TypeHierarchy th = visitor.getConfiguration().getTypeHierarchy();
 
-        optimizeChildren(visitor, contextInfo);
+        getLhs().optimize(visitor, contextInfo);
+        getRhs().optimize(visitor, contextInfo);
 
         Sequence value0 = null;
         Sequence value1 = null;
@@ -285,7 +284,7 @@ public final class ValueComparison extends BinaryExpression implements Compariso
         if ((value0 != null) && (value1 != null)) {
             try {
                 AtomicValue r = evaluateItem(visitor.getStaticContext().makeEarlyEvaluationContext());
-                return Literal.makeLiteral(r == null ? EmptySequence.getInstance() : r);
+                return Literal.makeLiteral(r == null ? EmptySequence.getInstance() : r, this);
             } catch (NoDynamicContextException e) {
                 // early evaluation failed, typically because the implicit context isn't available.
                 // Try again at run-time
@@ -297,12 +296,12 @@ public final class ValueComparison extends BinaryExpression implements Compariso
         // TODO: extend this to cases where N is not a literal
 
         if (getLhsExpression().isCallOn(Count.class) && Literal.isAtomic(getRhsExpression())) {
-            Expression e2 = optimizeCount(false);
+            Expression e2 = optimizeCount(visitor, false);
             if (e2 != null) {
                 return e2.optimize(visitor, contextInfo);
             }
         } else if (getRhsExpression().isCallOn(Count.class) && Literal.isAtomic(getLhsExpression())) {
-            Expression e2 = optimizeCount(true);
+            Expression e2 = optimizeCount(visitor, true);
             if (e2 != null) {
                 return e2.optimize(visitor, contextInfo);
             }
@@ -322,9 +321,9 @@ public final class ValueComparison extends BinaryExpression implements Compariso
                 case Token.FGT:
                     return SystemFunction.makeCall("boolean", getRetainedStaticContext(), arg);
                 case Token.FGE:
-                    return Literal.makeLiteral(BooleanValue.TRUE);
+                    return Literal.makeLiteral(BooleanValue.TRUE, this);
                 case Token.FLT:
-                    return Literal.makeLiteral(BooleanValue.FALSE);
+                    return Literal.makeLiteral(BooleanValue.FALSE, this);
             }
         }
 
@@ -341,9 +340,9 @@ public final class ValueComparison extends BinaryExpression implements Compariso
                 case Token.FLT:
                     return SystemFunction.makeCall("boolean", getRetainedStaticContext(), arg);
                 case Token.FLE:
-                    return Literal.makeLiteral(BooleanValue.TRUE);
+                    return Literal.makeLiteral(BooleanValue.TRUE, this);
                 case Token.FGT:
-                    return Literal.makeLiteral(BooleanValue.FALSE);
+                    return Literal.makeLiteral(BooleanValue.FALSE, this);
             }
         }
 
@@ -412,9 +411,9 @@ public final class ValueComparison extends BinaryExpression implements Compariso
                     ExpressionTool.copyLocationInfo(this, ilefalse);
                     return ilefalse;
                 case Token.FGT:
-                    return Literal.makeLiteral(BooleanValue.FALSE);
+                    return Literal.makeLiteral(BooleanValue.FALSE, this);
                 case Token.FLE:
-                    return Literal.makeLiteral(BooleanValue.TRUE);
+                    return Literal.makeLiteral(BooleanValue.TRUE, this);
             }
         }
         if (getLhsExpression().isCallOn(PositionAndLast.Last.class) &&
@@ -431,9 +430,9 @@ public final class ValueComparison extends BinaryExpression implements Compariso
                     ExpressionTool.copyLocationInfo(this, ilefalse);
                     return ilefalse;
                 case Token.FLT:
-                    return Literal.makeLiteral(BooleanValue.FALSE);
+                    return Literal.makeLiteral(BooleanValue.FALSE, this);
                 case Token.FGE:
-                    return Literal.makeLiteral(BooleanValue.TRUE);
+                    return Literal.makeLiteral(BooleanValue.TRUE, this);
             }
         }
 
@@ -464,6 +463,7 @@ public final class ValueComparison extends BinaryExpression implements Compariso
             boolean negate = (operator == Token.FEQ) != ((BooleanValue) literal.getValue()).getBooleanValue();
             if (negate) {
                 Expression fn = SystemFunction.makeCall("not", getRetainedStaticContext(), other);
+                assert fn != null;
                 ExpressionTool.copyLocationInfo(this, fn);
                 return fn.optimize(visitor, contextInfo);
             } else {
@@ -507,11 +507,11 @@ public final class ValueComparison extends BinaryExpression implements Compariso
      */
 
     /*@Nullable*/
-    private Expression optimizeCount(boolean inverted) throws XPathException {
+    private Expression optimizeCount(ExpressionVisitor visitor, boolean inverted) throws XPathException {
         SystemFunctionCall countFn = (SystemFunctionCall) (inverted ? getRhsExpression() : getLhsExpression());
         Expression sequence = countFn.getArg(0);
         sequence = sequence.unordered(false, false);
-        Optimizer opt = getConfiguration().obtainOptimizer();
+        Optimizer opt = visitor.obtainOptimizer();
 
         AtomicValue literalOperand = (AtomicValue) ((Literal) (inverted ? getLhsExpression() : getRhsExpression())).getValue();
         int op = inverted ? Token.inverse(operator) : operator;
@@ -529,23 +529,23 @@ public final class ValueComparison extends BinaryExpression implements Compariso
                 return result;
             } else if (op == Token.FGE) {
                 // rewrite count(x)>=0 as true()
-                return Literal.makeLiteral(BooleanValue.TRUE);
+                return Literal.makeLiteral(BooleanValue.TRUE, this);
             } else {  // singletonOperator == Token.FLT
                 // rewrite count(x)<0 as false()
-                return Literal.makeLiteral(BooleanValue.FALSE);
+                return Literal.makeLiteral(BooleanValue.FALSE, this);
             }
         } else if (literalOperand instanceof NumericValue) {
             long operand;
             if (literalOperand instanceof IntegerValue) {
                 operand = ((IntegerValue) literalOperand).longValue();
             } else if (literalOperand.isNaN()) {
-                return Literal.makeLiteral(BooleanValue.get(op == Token.FNE));
+                return Literal.makeLiteral(BooleanValue.get(op == Token.FNE), this);
             } else if (((NumericValue) literalOperand).isWholeNumber()) {
                 operand = ((NumericValue) literalOperand).longValue();
             } else if (op == Token.FEQ) {
-                return Literal.makeLiteral(BooleanValue.FALSE);
+                return Literal.makeLiteral(BooleanValue.FALSE, this);
             } else if (op == Token.FNE) {
-                return Literal.makeLiteral(BooleanValue.TRUE);
+                return Literal.makeLiteral(BooleanValue.TRUE, this);
             } else if (op == Token.FGT || op == Token.FGE) {
                 operand = ((NumericValue) literalOperand).ceiling().longValue();
                 op = Token.FGE;
@@ -558,9 +558,9 @@ public final class ValueComparison extends BinaryExpression implements Compariso
                     case Token.FEQ:
                     case Token.FLT:
                     case Token.FLE:
-                        return Literal.makeLiteral(BooleanValue.FALSE);
+                        return Literal.makeLiteral(BooleanValue.FALSE, this);
                     default:
-                        return Literal.makeLiteral(BooleanValue.TRUE);
+                        return Literal.makeLiteral(BooleanValue.TRUE, this);
                 }
             }
             if (operand > Integer.MAX_VALUE) {
@@ -568,9 +568,9 @@ public final class ValueComparison extends BinaryExpression implements Compariso
                     case Token.FEQ:
                     case Token.FGT:
                     case Token.FGE:
-                        return Literal.makeLiteral(BooleanValue.FALSE);
+                        return Literal.makeLiteral(BooleanValue.FALSE, this);
                     default:
-                        return Literal.makeLiteral(BooleanValue.TRUE);
+                        return Literal.makeLiteral(BooleanValue.TRUE, this);
                 }
             }
             if (sequence instanceof TailExpression || sequence.isCallOn(Subsequence_2.class)) {
@@ -585,8 +585,8 @@ public final class ValueComparison extends BinaryExpression implements Compariso
                     // rewrite count(E) op N as count(subsequence(E, 1, N+1)) op N
                     Expression ss = SystemFunction.makeCall("subsequence",
                         getRetainedStaticContext(), sequence,
-                        Literal.makeLiteral(IntegerValue.PLUS_ONE),
-                        Literal.makeLiteral(Int64Value.makeIntegerValue(operand + 1)));
+                        Literal.makeLiteral(Int64Value.PLUS_ONE, this),
+                        Literal.makeLiteral(Int64Value.makeIntegerValue(operand + 1), this));
                     Expression ct = SystemFunction.makeCall("count", getRetainedStaticContext(), ss);
                     CompareToIntegerConstant ctic = new CompareToIntegerConstant(ct, op, operand);
                     opt.trace("Rewrite count()~N as:", ctic);
@@ -617,12 +617,12 @@ public final class ValueComparison extends BinaryExpression implements Compariso
 
     public boolean isNegatable(TypeHierarchy th) {
         // Expression is not negatable if it might involve NaN
-        return !maybeNaN(getLhsExpression(), th) && !maybeNaN(getRhsExpression(), th);
+        return isNeverNaN(getLhsExpression(), th) && isNeverNaN(getRhsExpression(), th);
     }
 
-    private boolean maybeNaN(Expression exp, TypeHierarchy th) {
-        return th.relationship(exp.getItemType(), BuiltInAtomicType.DOUBLE) != TypeHierarchy.DISJOINT ||
-                th.relationship(exp.getItemType(), BuiltInAtomicType.FLOAT) != TypeHierarchy.DISJOINT;
+    private boolean isNeverNaN(Expression exp, TypeHierarchy th) {
+        return th.relationship(exp.getItemType(), BuiltInAtomicType.DOUBLE) == TypeHierarchy.DISJOINT &&
+                th.relationship(exp.getItemType(), BuiltInAtomicType.FLOAT) == TypeHierarchy.DISJOINT;
     }
 
     /**
@@ -657,12 +657,18 @@ public final class ValueComparison extends BinaryExpression implements Compariso
         return v instanceof NumericValue && ((NumericValue) v).compareTo(0) == 0;
     }
 
+    @Override
+    public boolean equals(Object other) {
+        return other instanceof ValueComparison &&
+                super.equals(other) &&
+                comparer.equals(((ValueComparison) other).comparer);
+    }
 
     /**
      * Copy an expression. This makes a deep copy.
      *
      * @return the copy of the original expression
-     * @param rebindings
+     * @param rebindings  variables that need to be re-bound
      */
 
     /*@NotNull*/
@@ -708,7 +714,8 @@ public final class ValueComparison extends BinaryExpression implements Compariso
      * @param op         the operator, as defined by constants such as {@link net.sf.saxon.expr.parser.Token#FEQ} or
      *                   {@link net.sf.saxon.expr.parser.Token#FLT}
      * @param v1         the second operand
-     * @param comparer   the Collator to be used when comparing strings
+     * @param comparer   used to compare values. If the comparer is context-sensitive then the context must
+     *                   already have been bound using comparer.provideContext().
      * @param checkTypes set to true if it is necessary to check that the types of the arguments are comparable
      * @return the result of the comparison: -1 for LT, 0 for EQ, +1 for GT
      * @throws XPathException if the values are not comparable
@@ -744,6 +751,8 @@ public final class ValueComparison extends BinaryExpression implements Compariso
                 default:
                     throw new UnsupportedOperationException("Unknown operator " + op);
             }
+        } catch (ComparisonException err) {
+            throw err.getCause();
         } catch (ClassCastException err) {
             XPathException e2 = new XPathException("Cannot compare " + Type.displayTypeName(v0) +
                     " to " + Type.displayTypeName(v1));
@@ -780,30 +789,6 @@ public final class ValueComparison extends BinaryExpression implements Compariso
         }
     }
 
-    /**
-     * Call the Callable.
-     *
-     * @param context   the dynamic evaluation context
-     * @param arguments the values of the arguments, supplied as Sequences.
-     *                  <p>Generally it is advisable, if calling iterate() to process a supplied sequence, to
-     *                  call it only once; if the value is required more than once, it should first be converted
-     *                  to a {@link net.sf.saxon.om.GroundedValue} by calling the utility methd
-     *                  SequenceTool.toGroundedValue().</p>
-     *                  <p>If the expected value is a single item, the item should be obtained by calling
-     *                  Sequence.head(): it cannot be assumed that the item will be passed as an instance of
-     *                  {@link net.sf.saxon.om.Item} or {@link net.sf.saxon.value.AtomicValue}.</p>
-     *                  <p>It is the caller's responsibility to perform any type conversions required
-     *                  to convert arguments to the type expected by the callee. An exception is where
-     *                  this Callable is explicitly an argument-converting wrapper around the original
-     *                  Callable.</p>
-     * @return the result of the evaluation, in the form of a Sequence. It is the responsibility
-     * of the callee to ensure that the type of result conforms to the expected result type.
-     * @throws net.sf.saxon.trans.XPathException if a dynamic error occurs during the evaluation of the expression
-     */
-    public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
-        return null;
-    }
-
     /**
      * Determine the data type of the expression
      *
@@ -820,7 +805,7 @@ public final class ValueComparison extends BinaryExpression implements Compariso
      * inference rules defined in the XSLT 3.0 specification.
      *
      * @return the static item type of the expression according to the XSLT 3.0 defined rules
-     * @param contextItemType
+     * @param contextItemType the static type of the context item
      */
     @Override
     public UType getStaticUType(UType contextItemType) {
