@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,7 +8,8 @@
 package net.sf.saxon.expr;
 
 import com.saxonica.ee.stream.Streamability;
-import net.sf.saxon.event.SequenceReceiver;
+import net.sf.saxon.event.Receiver;
+import net.sf.saxon.event.ReceiverOptions;
 import net.sf.saxon.expr.flwor.LocalVariableBinding;
 import net.sf.saxon.expr.instruct.GlobalParam;
 import net.sf.saxon.expr.instruct.LocalParam;
@@ -133,7 +134,7 @@ public abstract class VariableReference extends Expression implements BindingRef
         int dependencies = getDependencies();
         staticProperties = (properties & ~StaticProperty.CONTEXT_DOCUMENT_NODESET
                         & ~StaticProperty.ALL_NODES_NEWLY_CREATED) |
-                StaticProperty.NON_CREATIVE |
+                StaticProperty.NO_NODES_NEWLY_CREATED |
                 type.getCardinality() |
                 dependencies;
     }
@@ -180,7 +181,7 @@ public abstract class VariableReference extends Expression implements BindingRef
     }
 
     /**
-     * Determine whether this variable reference appears in a loop relative to its declaration.
+     * Ask whether this variable reference appears in a loop relative to its declaration.
      * By default, when in doubt, returns true. This is calculated during type-checking.
      *
      * @return true if this variable reference occurs in a loop, where the variable declaration is
@@ -191,6 +192,17 @@ public abstract class VariableReference extends Expression implements BindingRef
         return inLoop;
     }
 
+    /**
+     * Say whether this variable reference appears in a loop relative to its declaration.
+     *
+     * @param inLoop true if this variable reference occurs in a loop, where the variable declaration is
+     * outside the loop
+     */
+
+    public void setInLoop(boolean inLoop) {
+        this.inLoop = inLoop;
+    }
+
     /**
      * Type-check the expression. At this stage details of the static type must be known.
      * If the variable has a compile-time value, this is substituted for the variable reference
@@ -200,9 +212,7 @@ public abstract class VariableReference extends Expression implements BindingRef
     public Expression typeCheck(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
         if (constantValue != null) {
             binding = null;
-            Literal lit = Literal.makeLiteral(constantValue);
-            ExpressionTool.copyLocationInfo(this, lit);
-            return lit;
+            return Literal.makeLiteral(constantValue, this);
         }
 //        if (staticType == null) {
 //            throw new IllegalStateException("Variable $" + getDisplayName() + " has not been fixed up");
@@ -244,7 +254,7 @@ public abstract class VariableReference extends Expression implements BindingRef
         }
         if (constantValue != null) {
             binding = null;
-            Expression result = Literal.makeLiteral(constantValue);
+            Expression result = Literal.makeLiteral(constantValue, this);
             ExpressionTool.copyLocationInfo(this, result);
             Optimizer.trace(visitor.getConfiguration(), "Replaced variable " + getDisplayName() + " by its value", result);
             return result;
@@ -393,7 +403,7 @@ public abstract class VariableReference extends Expression implements BindingRef
     /**
      * Determine the special properties of this expression
      *
-     * @return {@link StaticProperty#NON_CREATIVE} (unless the variable is assignable using saxon:assign)
+     * @return {@link StaticProperty#NO_NODES_NEWLY_CREATED} (unless the variable is assignable using saxon:assign)
      */
 
     public int computeSpecialProperties() {
@@ -402,7 +412,7 @@ public abstract class VariableReference extends Expression implements BindingRef
             // if the variable reference is assignable, we mustn't move it, or any expression that contains it,
             // out of a loop. The way to achieve this is to treat it as a "creative" expression, because the
             // optimizer recognizes such expressions and handles them with care...
-            p |= StaticProperty.NON_CREATIVE;
+            p |= StaticProperty.NO_NODES_NEWLY_CREATED;
         }
         if (binding instanceof Assignation) {
             Expression exp = ((Assignation) binding).getSequence();
@@ -504,7 +514,6 @@ public abstract class VariableReference extends Expression implements BindingRef
     /**
      * Add a representation of this expression to a PathMap. The PathMap captures a map of the nodes visited
      * by an expression in a source tree.
-     * <p/>
      * <p>The default implementation of this method assumes that an expression does no navigation other than
      * the navigation done by evaluating its subexpressions, and that the subexpressions are evaluated in the
      * same context as the containing expression. The method must be overridden for any expression
@@ -535,9 +544,9 @@ public abstract class VariableReference extends Expression implements BindingRef
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(XPathContext c) throws XPathException {
+    public SequenceIterator<? extends Item> iterate(XPathContext c) throws XPathException {
         try {
-            Sequence actual = evaluateVariable(c);
+            Sequence<? extends Item> actual = evaluateVariable(c);
             assert actual != null;
             return actual.iterate();
         } catch (XPathException err) {
@@ -571,13 +580,10 @@ public abstract class VariableReference extends Expression implements BindingRef
 
     public void process(XPathContext c) throws XPathException {
         try {
-            SequenceIterator iter = evaluateVariable(c).iterate();
-            SequenceReceiver out = c.getReceiver();
+            SequenceIterator<? extends Item> iter = evaluateVariable(c).iterate();
+            Receiver out = c.getReceiver();
             Location loc = getLocation();
-            Item item;
-            while ((item = iter.next()) != null) {
-                out.append(item, loc, NodeInfo.ALL_NAMESPACES);
-            }
+            iter.forEachOrFail(item -> out.append(item, loc, ReceiverOptions.ALL_NAMESPACES));
         } catch (XPathException err) {
             err.maybeSetLocation(getLocation());
             throw err;
@@ -593,7 +599,7 @@ public abstract class VariableReference extends Expression implements BindingRef
      */
 
     /*@NotNull*/
-    public Sequence evaluateVariable(XPathContext c) throws XPathException {
+    public Sequence<? extends Item> evaluateVariable(XPathContext c) throws XPathException {
         try {
             return binding.evaluateVariable(c);
         } catch (NullPointerException err) {
