@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -15,6 +15,8 @@ import net.sf.saxon.expr.sort.GlobalOrderComparer;
 import net.sf.saxon.functions.CurrentGroupCall;
 import net.sf.saxon.functions.SystemFunction;
 import net.sf.saxon.om.AxisInfo;
+import net.sf.saxon.om.Item;
+import net.sf.saxon.om.NodeInfo;
 import net.sf.saxon.om.SequenceIterator;
 import net.sf.saxon.pattern.*;
 import net.sf.saxon.trans.XPathException;
@@ -114,7 +116,7 @@ public class VennExpression extends BinaryExpression {
      * inference rules defined in the XSLT 3.0 specification.
      *
      * @return the static item type of the expression according to the XSLT 3.0 defined rules
-     * @param contextItemType
+     * @param contextItemType the static type of the context item
      */
     @Override
     public UType getStaticUType(UType contextItemType) {
@@ -185,8 +187,8 @@ public class VennExpression extends BinaryExpression {
         if (testSubTree(prop0, prop1)) {
             props |= StaticProperty.SUBTREE_NODESET;
         }
-        if (!testCreative(prop0, prop1)) {
-            props |= StaticProperty.NON_CREATIVE;
+        if (createsNoNewNodes(prop0, prop1)) {
+            props |= StaticProperty.NO_NODES_NEWLY_CREATED;
         }
         return props;
     }
@@ -263,9 +265,9 @@ public class VennExpression extends BinaryExpression {
      * @return true if the expression can create new nodes
      */
 
-    private boolean testCreative(final int prop0, final int prop1) {
-        return !(((prop0 & StaticProperty.NON_CREATIVE) != 0) &&
-                ((prop1 & StaticProperty.NON_CREATIVE) != 0));
+    private boolean createsNoNewNodes(final int prop0, final int prop1) {
+        return (prop0 & StaticProperty.NO_NODES_NEWLY_CREATED) != 0 &&
+                (prop1 & StaticProperty.NO_NODES_NEWLY_CREATED) != 0;
     }
 
     /**
@@ -277,7 +279,8 @@ public class VennExpression extends BinaryExpression {
 
         Configuration config = visitor.getConfiguration();
         TypeChecker tc = config.getTypeChecker(false);
-        typeCheckChildren(visitor, contextInfo);
+        getLhs().typeCheck(visitor, contextInfo);
+        getRhs().typeCheck(visitor, contextInfo);
 
         if (!(getLhsExpression() instanceof Pattern)) {
             final RoleDiagnostic role0 = new RoleDiagnostic(RoleDiagnostic.BINARY_EXPR, Token.tokens[operator], 0);
@@ -298,7 +301,7 @@ public class VennExpression extends BinaryExpression {
                 if (operator == Token.INTERSECT) {
                     return Literal.makeEmptySequence();
                 } else {
-                    if ((getLhsExpression().getSpecialProperties() & StaticProperty.ORDERED_NODESET) != 0) {
+                    if (getLhsExpression().hasSpecialProperty(StaticProperty.ORDERED_NODESET)) {
                         return getLhsExpression();
                     } else {
                         return new DocumentSorter(getLhsExpression());
@@ -313,7 +316,6 @@ public class VennExpression extends BinaryExpression {
 
     /**
      * Perform optimisation of an expression and its subexpressions.
-     * <p/>
      * <p>This method is called after all references to functions and variables have been resolved
      * to the declaration of the function or variable, and after all type checking has been done.</p>
      *
@@ -424,13 +426,13 @@ public class VennExpression extends BinaryExpression {
             final SlashExpression path1 = (SlashExpression) lhs;
             final SlashExpression path2 = (SlashExpression) rhs;
 
-            if (path1.getFirstStep().equals(path2.getFirstStep())) {
+            if (path1.getFirstStep().isEqual(path2.getFirstStep())) {
                 final VennExpression venn = new VennExpression(
                         path1.getRemainingSteps(),
                         operator,
                         path2.getRemainingSteps());
                 ExpressionTool.copyLocationInfo(this, venn);
-                final Expression path = ExpressionTool.makePathExpression(path1.getFirstStep(), venn, false);
+                final Expression path = ExpressionTool.makePathExpression(path1.getFirstStep(), venn);
                 ExpressionTool.copyLocationInfo(this, path);
                 return path.optimize(visitor, contextItemType);
             }
@@ -445,7 +447,7 @@ public class VennExpression extends BinaryExpression {
 
             if (!exp0.isPositional(th) &&
                     !exp1.isPositional(th) &&
-                    exp0.getSelectExpression().equals(exp1.getSelectExpression())) {
+                    exp0.getSelectExpression().isEqual(exp1.getSelectExpression())) {
                 final Expression filter;
                 switch (operator) {
                     case Token.UNION:
@@ -519,7 +521,7 @@ public class VennExpression extends BinaryExpression {
      * @param forStreaming  set to true if optimizing for streaming
      */
      @Override
-     public Expression unordered(boolean retainAllNodes, boolean forStreaming) throws XPathException {
+     public Expression unordered(boolean retainAllNodes, boolean forStreaming) {
          if (operator == Token.UNION && !forStreaming &&
                  operandsAreDisjoint(getConfiguration().getTypeHierarchy())) {
              // replace union operator by comma operator to avoid cost of sorting into document order. See XMark q7
@@ -535,7 +537,7 @@ public class VennExpression extends BinaryExpression {
      * Copy an expression. This makes a deep copy.
      *
      * @return the copy of the original expression
-     * @param rebindings
+     * @param rebindings variables that need to be rebound
      */
 
     /*@NotNull*/
@@ -561,7 +563,7 @@ public class VennExpression extends BinaryExpression {
     /**
      * Get the operand role (applies to both operands)
      * @return the operand role
-     * @param arg
+     * @param arg which argument: 0 for the lhs, 1 for the rhs
      */
 
     protected OperandRole getOperandRole(int arg) {
@@ -580,14 +582,14 @@ public class VennExpression extends BinaryExpression {
             if (operator != b.operator) {
                 return false;
             }
-            if (getLhsExpression().equals(b.getLhsExpression()) && getRhsExpression().equals(b.getRhsExpression())) {
+            if (getLhsExpression().isEqual(b.getLhsExpression()) && getRhsExpression().isEqual(b.getRhsExpression())) {
                 return true;
             }
             if (operator == Token.UNION || operator == Token.INTERSECT) {
                 // These are commutative and associative, so for example (A|B)|C equals B|(A|C)
-                Set<Expression> s0 = new HashSet<Expression>(10);
+                Set<Expression> s0 = new HashSet<>(10);
                 gatherComponents(operator, s0);
-                Set<Expression> s1 = new HashSet<Expression>(10);
+                Set<Expression> s1 = new HashSet<>(10);
                 ((VennExpression) other).gatherComponents(operator, s1);
                 return s0.equals(s1);
             }
@@ -596,20 +598,19 @@ public class VennExpression extends BinaryExpression {
     }
 
     public int computeHashCode() {
-        return getLhsExpression().computeHashCode() ^ getRhsExpression().computeHashCode();
+        return getLhsExpression().hashCode() ^ getRhsExpression().hashCode();
     }
 
     /**
      * Convert this expression to an equivalent XSLT pattern
      *
      * @param config the Saxon configuration
-     * @param is30   true if this is XSLT 3.0
      * @return the equivalent pattern
      * @throws net.sf.saxon.trans.XPathException
      *          if conversion is not possible
      */
     @Override
-    public Pattern toPattern(Configuration config, boolean is30) throws XPathException {
+    public Pattern toPattern(Configuration config) throws XPathException {
         if (isPredicatePattern(getLhsExpression()) || isPredicatePattern(getRhsExpression())) {
             throw new XPathException(
                     "Cannot use a predicate pattern as an operand of a union, intersect, or except operator",
@@ -617,21 +618,19 @@ public class VennExpression extends BinaryExpression {
         }
         if (operator == Token.UNION) {
             return new UnionPattern(
-                    getLhsExpression().toPattern(config, is30),
-                    getRhsExpression().toPattern(config, is30));
-        } else if (is30) {
+                    getLhsExpression().toPattern(config),
+                    getRhsExpression().toPattern(config));
+        } else {
             if (operator == Token.EXCEPT) {
                 return new ExceptPattern(
-                        getLhsExpression().toPattern(config, is30),
-                        getRhsExpression().toPattern(config, is30));
+                        getLhsExpression().toPattern(config),
+                        getRhsExpression().toPattern(config));
             } else {
                 return new IntersectPattern(
-                        getLhsExpression().toPattern(config, is30),
-                        getRhsExpression().toPattern(config, is30));
+                        getLhsExpression().toPattern(config),
+                        getRhsExpression().toPattern(config));
             }
-        } else {
-            throw new XPathException("Cannot use intersect or except in an XSLT 2.0 pattern", "XTSE0340");
-        }
+        } 
     }
 
     private boolean isPredicatePattern(Expression exp) {
@@ -663,9 +662,11 @@ public class VennExpression extends BinaryExpression {
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(final XPathContext c) throws XPathException {
-        SequenceIterator i1 = getLhsExpression().iterate(c);
-        SequenceIterator i2 = getRhsExpression().iterate(c);
+    public SequenceIterator<? extends Item> iterate(final XPathContext c) throws XPathException {
+        SequenceIterator<? extends NodeInfo> i1 =
+                (SequenceIterator<? extends NodeInfo>)getLhsExpression().iterate(c);
+        SequenceIterator<? extends NodeInfo> i2 =
+                (SequenceIterator<? extends NodeInfo>)getRhsExpression().iterate(c);
         switch (operator) {
             case Token.UNION:
                 return new UnionEnumeration(i1, i2,
