@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,7 +9,7 @@ package net.sf.saxon.expr;
 
 import net.sf.saxon.Configuration;
 import net.sf.saxon.Controller;
-import net.sf.saxon.event.SequenceReceiver;
+import net.sf.saxon.event.Receiver;
 import net.sf.saxon.expr.instruct.ParameterSet;
 import net.sf.saxon.expr.sort.GroupIterator;
 import net.sf.saxon.lib.DelegatingErrorListener;
@@ -17,8 +17,8 @@ import net.sf.saxon.lib.UnfailingErrorListener;
 import net.sf.saxon.om.*;
 import net.sf.saxon.regex.RegexIterator;
 import net.sf.saxon.trans.NoDynamicContextException;
-import net.sf.saxon.trans.rules.Rule;
 import net.sf.saxon.trans.XPathException;
+import net.sf.saxon.trans.rules.Rule;
 import net.sf.saxon.value.DateTimeValue;
 
 import javax.xml.transform.URIResolver;
@@ -116,6 +116,15 @@ public interface XPathContext {
 
     XPathContext getCaller();
 
+    /**
+     * Create, set, and return a focus tracking iterator that wraps a supplied sequence iterator.
+     *
+     * @param iter the current iterator. The context item, position, and size are determined by reference
+     *             to the current iterator.
+     */
+
+    <T extends Item> FocusIterator<T> trackFocus(SequenceIterator<T> iter);
+
     /**
      * Set a new sequence iterator.
      *
@@ -123,7 +132,7 @@ public interface XPathContext {
      *             to the current iterator.
      */
 
-    void setCurrentIterator(FocusIterator iter);
+    void setCurrentIterator(FocusIterator<? extends Item> iter);
 
     /**
      * Get the current iterator.
@@ -133,7 +142,7 @@ public interface XPathContext {
      *         (which means the context item, position, and size are undefined).
      */
 
-    FocusIterator getCurrentIterator();
+    FocusIterator<? extends Item> getCurrentIterator();
 
     /**
      * Get the context item
@@ -194,23 +203,6 @@ public interface XPathContext {
 
     Component getCurrentComponent();
 
-
-    /**
-     * Get a named collation.
-     * <p>Note: although collations are defined in the specification as being part of the static
-     * context, Saxon assumes that all available collations have global scope for a transformation
-     * or query, so that a collation URI can be translated into an actual collation without knowing
-     * where in the source code the collation URI was used. The default collation, however, can vary
-     * in different parts of a query or stylesheet, and expressions using the default collation must
-     * therefore get it from the static context.</p>
-     *
-     * @param name the name (URI) of the required collation
-     * @return a StringCollator representing the collation
-     * @throws XPathException if the collation is not recognized
-     */
-
-    //StringCollator getCollation(String name) throws XPathException;
-
     /**
      * Use local parameter. This is called when a local xsl:param element is processed.
      * If a parameter of the relevant name was supplied, it is bound to the xsl:param element.
@@ -292,14 +284,14 @@ public interface XPathContext {
      * @param receiver the SequenceReceiver to be used
      */
 
-    void setReceiver(SequenceReceiver receiver);
+    void setReceiver(Receiver receiver);
 
     /**
      * Get the Receiver to which output is currently being written.
      *
      * @return the current SequenceReceiver
      */
-    SequenceReceiver getReceiver();
+    Receiver getReceiver();
 
     /**
      * Get the current mode.
