@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -115,8 +115,8 @@ public class GroupByClause extends Clause {
 
     @Override
     public void optimize(ExpressionVisitor visitor, ContextItemStaticInfo contextItemType) throws XPathException {
-        LinkedList<LocalVariableBinding> list = new LinkedList<LocalVariableBinding>(Arrays.asList(bindings));
-        LinkedList<LocalVariableReference> retainingExpr = new LinkedList<LocalVariableReference>();
+        LinkedList<LocalVariableBinding> list = new LinkedList<>(Arrays.asList(bindings));
+        LinkedList<LocalVariableReference> retainingExpr = new LinkedList<>();
         for (Operand o : getRetainedTupleExpression().operands()) {
             retainingExpr.add((LocalVariableReference)o.getChildExpression());
         }
@@ -128,7 +128,7 @@ public class GroupByClause extends Clause {
                 retainingExpr.remove(i - groupingSize);
             }
         }
-        bindings = list.toArray(new LocalVariableBinding[list.size()]);
+        bindings = list.toArray(new LocalVariableBinding[0]);
         getRetainedTupleExpression().setVariables(retainingExpr);
     }
 
@@ -243,7 +243,7 @@ public class GroupByClause extends Clause {
      * @param out the expression presenter used to display the structure
      */
     @Override
-    public void explain(ExpressionPresenter out) throws XPathException {
+    public void explain(ExpressionPresenter out) {
         out.startElement("group-by");
         for (Operand o : getRetainedTupleExpression().operands()) {
             LocalVariableReference ref = (LocalVariableReference)o.getChildExpression();
@@ -291,7 +291,7 @@ public class GroupByClause extends Clause {
             context.setLocalVariable(bindings[j].getLocalSlotNumber(), v);
         }
         for (int j = groupingValues.length; j < bindings.length; j++) {
-            List<Item> concatenatedValue = new ArrayList<Item>();
+            List<Item> concatenatedValue = new ArrayList<>();
             for (ObjectToBeGrouped otbg : group) {
                 Sequence val = otbg.retainedValues.getMembers()[j - groupingValues.length];
                 SequenceIterator si = val.iterate();
@@ -300,7 +300,7 @@ public class GroupByClause extends Clause {
                     concatenatedValue.add(it);
                 }
             }
-            SequenceExtent se = new SequenceExtent(concatenatedValue);
+            SequenceExtent<Item> se = new SequenceExtent<>(concatenatedValue);
             context.setLocalVariable(bindings[j].getLocalSlotNumber(), se);
         }
     }
