@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -15,7 +15,6 @@ import net.sf.saxon.om.SequenceTool;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.value.AtomicValue;
 import net.sf.saxon.value.EmptySequence;
-import net.sf.saxon.value.SequenceExtent;
 
 import java.util.ArrayList;
 import java.util.HashMap;
@@ -30,7 +29,7 @@ public class GroupByClausePush extends TuplePush {
 
     private TuplePush destination;
     private GroupByClause groupByClause;
-    private HashMap<Object, List<GroupByClause.ObjectToBeGrouped>> map = new HashMap<Object, List<GroupByClause.ObjectToBeGrouped>>();
+    private HashMap<Object, List<GroupByClause.ObjectToBeGrouped>> map = new HashMap<>();
     private XPathContext context;
     private GenericAtomicComparer[] comparers;
 
@@ -61,30 +60,40 @@ public class GroupByClausePush extends TuplePush {
 
         GroupByClause.ObjectToBeGrouped otbg = new GroupByClause.ObjectToBeGrouped();
         Sequence[] groupingValues = groupingTupleExpr.evaluateItem(context).getMembers();
+        checkGroupingValues(groupingValues);
+        otbg.groupingValues = new Tuple(groupingValues);
+        otbg.retainedValues = retainedTupleExpr.evaluateItem(context);
+        Object key = groupByClause.getComparisonKey(otbg.groupingValues, comparers);
+        List<GroupByClause.ObjectToBeGrouped> group = map.get(key);
+        addToGroup(key, otbg, group, map);
+    }
+
+    protected static void addToGroup(
+            Object key,
+            GroupByClause.ObjectToBeGrouped objectToBeGrouped,
+            List<GroupByClause.ObjectToBeGrouped> group,
+            HashMap<Object, List<GroupByClause.ObjectToBeGrouped>> map) {
+        if (group != null) {
+            group.add(objectToBeGrouped);
+            map.put(key, group);
+        } else {
+            List<GroupByClause.ObjectToBeGrouped> list = new ArrayList<>();
+            list.add(objectToBeGrouped);
+            map.put(key, list);
+        }
+    }
+
+    protected static void checkGroupingValues(Sequence[] groupingValues) throws XPathException {
         for (int i = 0; i < groupingValues.length; i++) {
             Sequence v = groupingValues[i];
             if (!(v instanceof EmptySequence || v instanceof AtomicValue)) {
-                v = SequenceExtent.makeSequenceExtent(Atomizer.getAtomizingIterator(v.iterate(), false));
+                v = Atomizer.getAtomizingIterator(v.iterate(), false).materialize();
                 if (SequenceTool.getLength(v) > 1) {
                     throw new XPathException("Grouping key value cannot be a sequence of more than one item", "XPTY0004");
                 }
                 groupingValues[i] = v;
             }
         }
-        otbg.groupingValues = new Tuple(groupingValues);
-        otbg.retainedValues = retainedTupleExpr.evaluateItem(context);
-        Object key = groupByClause.getComparisonKey(otbg.groupingValues, comparers);
-        List<GroupByClause.ObjectToBeGrouped> group = map.get(key);
-        if (group != null) {
-            group.add(otbg);
-            map.put(key, group);
-        } else {
-            List<GroupByClause.ObjectToBeGrouped> list = new ArrayList<GroupByClause.ObjectToBeGrouped>();
-            list.add(otbg);
-            map.put(key, list);
-        }
-
-
     }
 
 
