@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,18 +7,22 @@
 
 package net.sf.saxon.expr.flwor;
 
-import net.sf.saxon.expr.Expression;
-import net.sf.saxon.expr.Operand;
-import net.sf.saxon.expr.OperandRole;
-import net.sf.saxon.expr.XPathContext;
-import net.sf.saxon.expr.parser.PathMap;
-import net.sf.saxon.expr.parser.RebindingMap;
+import net.sf.saxon.Configuration;
+import net.sf.saxon.expr.*;
+import net.sf.saxon.expr.parser.*;
+import net.sf.saxon.functions.Count;
 import net.sf.saxon.om.Item;
 import net.sf.saxon.om.Sequence;
+import net.sf.saxon.om.SequenceIterator;
 import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.XPathException;
+import net.sf.saxon.tree.iter.ListIterator;
+import net.sf.saxon.type.ItemType;
+import net.sf.saxon.type.TypeHierarchy;
+import net.sf.saxon.value.Cardinality;
 import net.sf.saxon.value.EmptySequence;
 import net.sf.saxon.value.Int64Value;
+import net.sf.saxon.value.SequenceType;
 import net.sf.saxon.z.IntHashMap;
 
 import java.util.Iterator;
@@ -34,7 +38,9 @@ public class WindowClause extends Clause {
     private Operand sequenceOp;
     private Operand startConditionOp;
     private Operand endConditionOp;
-    private IntHashMap<LocalVariableBinding> windowVars = new IntHashMap<LocalVariableBinding>(10);
+    private IntHashMap<LocalVariableBinding> windowVars = new IntHashMap<>(10);
+    private ItemTypeCheckingFunction itemTypeChecker;
+    private boolean windowMustBeSingleton;
 
     public static final int WINDOW_VAR = 0;
     public static final int START_ITEM = 1;
@@ -125,6 +131,61 @@ public class WindowClause extends Clause {
         return windowVars.get(role);
     }
 
+    public ItemTypeCheckingFunction getItemTypeChecker() {
+        return itemTypeChecker;
+    }
+
+    public boolean isWindowMustBeSingleton() {
+        return windowMustBeSingleton;
+    }
+
+    @Override
+    public void typeCheck(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
+        SequenceType requiredType = getVariableBinding(WindowClause.WINDOW_VAR).getRequiredType();
+        ItemType required = requiredType.getPrimaryType();
+        ItemType supplied = getSequence().getItemType();
+        Configuration config = visitor.getConfiguration();
+        TypeHierarchy th = config.getTypeHierarchy();
+        int rel = th.relationship(required, supplied);
+        switch (rel) {
+            case TypeHierarchy.SAME_TYPE:
+            case TypeHierarchy.SUBSUMES:
+                // no action
+                break;
+            case TypeHierarchy.OVERLAPS:
+            case TypeHierarchy.SUBSUMED_BY:
+                RoleDiagnostic role = new RoleDiagnostic(
+                        RoleDiagnostic.VARIABLE,
+                        getVariableBinding(WindowClause.WINDOW_VAR).getVariableQName().getDisplayName(), 0);
+                itemTypeChecker = new ItemTypeCheckingFunction(
+                        required, role, getLocation(), config);
+                break;
+            case TypeHierarchy.DISJOINT:
+                String message = "The items in the window will always be instances of "
+                        + supplied.toString() + ", never of the required type " + required.toString();
+                throw new XPathException(message, "XPTY0004", getLocation());
+        }
+        windowMustBeSingleton = !Cardinality.allowsMany(requiredType.getCardinality());
+
+        if (requiredType.getCardinality() == StaticProperty.ALLOWS_ZERO) {
+            String message = "The value of the window variable can never be an empty sequence";
+            throw new XPathException(message, "XPTY0004", getLocation());
+        }
+    }
+
+    public void checkWindowContents(Window w) throws XPathException {
+        if (windowMustBeSingleton && w.contents.size() > 1) {
+            throw new XPathException("Required type of window allows only a single item; window has length " + w.contents.size(),
+                                     "XPTY0004", getLocation());
+        }
+        ItemTypeCheckingFunction checker = getItemTypeChecker();
+        if (checker != null) {
+            SequenceIterator<? extends Item> check =
+                    new ItemMappingIterator<>(new ListIterator<>(w.contents), checker);
+            Count.count(check); // a convenient way to consume the iterator and thus perform the checking
+        }
+    }
+
     @Override
     public Clause copy(FLWORExpression flwor, RebindingMap rebindings) {
         WindowClause wc = new WindowClause();
@@ -233,7 +294,7 @@ public class WindowClause extends Clause {
      * @param position the position of the current item in the input sequence
      * @param context  the dynamic evaluation context
      * @return true if the current item forms the start of a new window
-     * @throws XPathException
+     * @throws XPathException if a failure occurs evaluating the conditions
      */
 
     protected boolean matchesStart(Item previous, Item current, Item next, int position, XPathContext context) throws XPathException {
@@ -268,7 +329,7 @@ public class WindowClause extends Clause {
      * @param position the position of the current item in the input sequence
      * @param context  the dynamic evaluation context
      * @return true if the current item is the last item in the specified window
-     * @throws XPathException
+     * @throws XPathException if a failure occurs evaluating the conditions
      */
 
     protected boolean matchesEnd(Window window, Item previous, Item current, Item next, int position, XPathContext context) throws XPathException {
@@ -357,4 +418,4 @@ public class WindowClause extends Clause {
     }
 }
 
-// Copyright (c) 2011 Saxonica Limited.
\ No newline at end of file
+// Copyright (c) 2011-2018 Saxonica Limited.
\ No newline at end of file
