@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,16 +8,19 @@
 package net.sf.saxon.expr.instruct;
 
 import net.sf.saxon.Configuration;
-import net.sf.saxon.Version;
 import net.sf.saxon.expr.*;
 import net.sf.saxon.expr.parser.*;
-import net.sf.saxon.om.*;
+import net.sf.saxon.om.FocusIterator;
+import net.sf.saxon.om.Item;
+import net.sf.saxon.om.SequenceIterator;
+import net.sf.saxon.om.StandardNames;
 import net.sf.saxon.regex.RegexIterator;
 import net.sf.saxon.regex.RegularExpression;
 import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.type.*;
 import net.sf.saxon.value.SequenceType;
+import net.sf.saxon.value.StringValue;
 
 import java.util.ArrayList;
 import java.util.List;
@@ -238,15 +241,15 @@ public class AnalyzeString extends Instruction implements ContextOriginator {
                 final CharSequence flagstr = ((StringLiteral) getFlags()).getStringValue();
 
                 String hostLang = "XP30";
-                List<String> warnings = new ArrayList<String>();
-                pattern = Version.platform.compileRegularExpression(config, regex, flagstr.toString(), hostLang, warnings);
+                List<String> warnings = new ArrayList<>();
+                pattern = config.compileRegularExpression(regex, flagstr.toString(), hostLang, warnings);
                 for (String w : warnings) {
                     visitor.getStaticContext().issueWarning(w, getLocation());
                 }
 
                 if ((!isAllow30features() || !useXsltErrorCodes) && pattern.matches("")) {
                     // prevent it being reported more than once
-                    pattern = Version.platform.compileRegularExpression(config, "x", "", "XP20", warnings);
+                    pattern = config.compileRegularExpression("x", "", "XP20", warnings);
                     invalidRegex("The regular expression must not be one that matches a zero-length string",
                             useXsltErrorCodes ? "XTDE1150" : "FORX0003");
                 }
@@ -375,10 +378,9 @@ public class AnalyzeString extends Instruction implements ContextOriginator {
 
     public TailCall processLeavingTail(XPathContext context) throws XPathException {
         RegexIterator iter = getRegexIterator(context);
-        FocusIterator focusIter = new FocusTrackingIterator(iter);
         XPathContextMajor c2 = context.newContext();
         c2.setOrigin(this);
-        c2.setCurrentIterator(focusIter);
+        FocusIterator focusIter = c2.trackFocus(iter);
         c2.setCurrentRegexIterator(iter);
 
         Item it;
@@ -411,11 +413,14 @@ public class AnalyzeString extends Instruction implements ContextOriginator {
 
         RegularExpression re = pattern;
         if (re == null) {
-            CharSequence flagstr = getFlags().evaluateAsString(context);
+            String flagstr = getFlags().evaluateAsString(context).toString();
             String dialect = "XP30";
-            re = Version.platform.compileRegularExpression(
-                    context.getConfiguration(), getRegex().evaluateAsString(context), flagstr.toString(), dialect, null);
-
+            StringValue regexString = (StringValue)getRegex().evaluateItem(context);
+            re = regexString.getPrecompiledRegularExpression();
+            if (re == null || !re.getFlags().equals(flagstr)) {
+                re = context.getConfiguration().compileRegularExpression(
+                        getRegex().evaluateAsString(context), flagstr, dialect, null);
+            }
             if ((dialect.equals("XP20") || !useXsltErrorCodes) && re.matches("")) {
                 dynamicError("The regular expression must not be one that matches a zero-length string",
                         useXsltErrorCodes ? "XTDE1150" : "FORX0003", context);
@@ -425,6 +430,9 @@ public class AnalyzeString extends Instruction implements ContextOriginator {
         return re.analyze(input);
     }
 
+    // TODO: use the new RegexStringValue mechanism for cases where the regex is known statically, to
+    // avoid having two separate mechanism.
+
     /**
      * Return an Iterator to iterate over the values of a sequence. The value of every
      * expression can be regarded as a sequence, so this method is supported for all
@@ -441,12 +449,11 @@ public class AnalyzeString extends Instruction implements ContextOriginator {
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<? extends Item> iterate(XPathContext context) throws XPathException {
         RegexIterator iter = getRegexIterator(context);
-        FocusIterator focus = new FocusTrackingIterator(iter);
         XPathContextMajor c2 = context.newContext();
         c2.setOrigin(this);
-        c2.setCurrentIterator(focus);
+        c2.trackFocus(iter);
         c2.setCurrentRegexIterator(iter);
 
         AnalyzeMappingFunction fn = new AnalyzeMappingFunction(iter, c2, getNonMatching(), getMatching());
