@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -119,7 +119,7 @@ public class ApplyTemplates extends Instruction implements ITemplateCall, Compon
 
     @Override
     public Iterable<Operand> operands() {
-        List<Operand> operanda = new ArrayList<Operand>();
+        List<Operand> operanda = new ArrayList<>();
         operanda.add(selectOp);
         WithParam.gatherOperands(this, getActualParams(), operanda);
         WithParam.gatherOperands(this, getTunnelParams(), operanda);
@@ -220,7 +220,9 @@ public class ApplyTemplates extends Instruction implements ITemplateCall, Compon
      * Copy an expression. This makes a deep copy.
      *
      * @return the copy of the original expression
-     * @param rebindings
+     * @param rebindings a mutable list of (old binding, new binding) pairs
+     *               that is used to update the bindings held in any
+     *               local variable references that are copied.
      */
 
     /*@NotNull*/
@@ -266,7 +268,7 @@ public class ApplyTemplates extends Instruction implements ITemplateCall, Compon
             XPathContextMajor c2 = context.newContext();
             c2.setOrigin(this);
             return new ApplyTemplatesPackage(
-                    ExpressionTool.lazyEvaluate(getSelect(), context, 1),
+                    ExpressionTool.lazyEvaluate(getSelect(), context, false),
                     targetMode, params, tunnels, c2, getLocation());
         }
 
@@ -283,7 +285,7 @@ public class ApplyTemplates extends Instruction implements ITemplateCall, Compon
         // process the selected nodes now
 
         XPathContextMajor c2 = context.newContext();
-        c2.setCurrentIterator(new FocusTrackingIterator(iter));
+        c2.trackFocus(iter);
         c2.setCurrentMode(targetMode);
         c2.setOrigin(this);
         c2.setCurrentComponent(targetMode);
@@ -406,7 +408,6 @@ public class ApplyTemplates extends Instruction implements ITemplateCall, Compon
     /**
      * Add a representation of this expression to a PathMap. The PathMap captures a map of the nodes visited
      * by an expression in a source tree.
-     * <p/>
      * <p>The default implementation of this method assumes that an expression does no navigation other than
      * the navigation done by evaluating its subexpressions, and that the subexpressions are evaluated in the
      * same context as the containing expression. The method must be overridden for any expression
@@ -443,7 +444,7 @@ public class ApplyTemplates extends Instruction implements ITemplateCall, Compon
 
         out.startElement("applyT", this);
         if (mode != null && !mode.isUnnamedMode()) {
-            out.emitAttribute("mode", mode.getModeName().getEQName());
+            out.emitAttribute("mode", mode.getModeName());
         }
         String flags = "";
         if (useCurrentMode) {
@@ -547,7 +548,7 @@ public class ApplyTemplates extends Instruction implements ITemplateCall, Compon
         }
 
         public TailCall processLeavingTail() throws XPathException {
-            evaluationContext.setCurrentIterator(new FocusTrackingIterator(selectedItems.iterate()));
+            evaluationContext.trackFocus(selectedItems.iterate());
             evaluationContext.setCurrentMode(targetMode);
             evaluationContext.setCurrentComponent(targetMode);
             return targetMode.getActor().applyTemplates(params, tunnelParams, evaluationContext, locationId);
