@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,22 +8,15 @@
 package net.sf.saxon.expr.instruct;
 
 import net.sf.saxon.expr.*;
-import net.sf.saxon.expr.parser.ContextItemStaticInfo;
-import net.sf.saxon.expr.parser.ExpressionTool;
-import net.sf.saxon.expr.parser.ExpressionVisitor;
-import net.sf.saxon.expr.parser.RebindingMap;
-import net.sf.saxon.om.AxisInfo;
-import net.sf.saxon.om.Item;
-import net.sf.saxon.om.SequenceIterator;
+import net.sf.saxon.expr.parser.*;
+import net.sf.saxon.om.*;
 import net.sf.saxon.pattern.NodeKindTest;
 import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.iter.EmptyIterator;
+import net.sf.saxon.tree.iter.ListIterator;
 import net.sf.saxon.type.*;
-import net.sf.saxon.value.AtomicValue;
-import net.sf.saxon.value.Cardinality;
-import net.sf.saxon.value.IntegerRange;
-import net.sf.saxon.value.SequenceExtent;
+import net.sf.saxon.value.*;
 
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -39,7 +32,7 @@ public class Block extends Instruction {
 
     // TODO: allow the last expression in a Block to be a tail-call of a function, at least in push mode
 
-    private Operand[] operanda;
+    private final Operand[] operanda;
     private boolean allNodesUntyped;
 
     /**
@@ -134,7 +127,7 @@ public class Block extends Instruction {
             return e1;
         }
         if (e1 instanceof Block || e2 instanceof Block) {
-            List<Expression> list = new ArrayList<Expression>(10);
+            List<Expression> list = new ArrayList<>(10);
             if (e1 instanceof Block) {
                 for (Operand o : e1.operands()) {
                     list.add(o.getChildExpression());
@@ -231,7 +224,7 @@ public class Block extends Instruction {
         if (allAxisExpressions) {
             p |= StaticProperty.CONTEXT_DOCUMENT_NODESET |
                     StaticProperty.SINGLE_DOCUMENT_NODESET |
-                    StaticProperty.NON_CREATIVE;
+                    StaticProperty.NO_NODES_NEWLY_CREATED;
             // if they all use the child axis, then we have a peer node-set
             if (allChildAxis) {
                 p |= StaticProperty.PEER_NODESET;
@@ -249,6 +242,64 @@ public class Block extends Instruction {
         return p;
     }
 
+    public boolean implementsStaticTypeCheck() {
+        return true;
+    }
+
+    /**
+     * Static type checking for let expressions is delegated to the child expressions,
+     * to allow further delegation to the branches of a conditional
+     *
+     * @param req                 the required type
+     * @param backwardsCompatible true if backwards compatibility mode applies
+     * @param role                the role of the expression in relation to the required type
+     * @param visitor             an expression visitor
+     * @return the expression after type checking (perhaps augmented with dynamic type checking code)
+     * @throws XPathException if failures occur, for example if the static type of one branch of the conditional
+     *                        is incompatible with the required type
+     */
+
+    public Expression staticTypeCheck(SequenceType req,
+                                      boolean backwardsCompatible,
+                                      RoleDiagnostic role, ExpressionVisitor visitor)
+            throws XPathException {
+
+        TypeChecker tc = visitor.getConfiguration().getTypeChecker(backwardsCompatible);
+        if (backwardsCompatible && !Cardinality.allowsMany(req.getCardinality())) {
+            Expression first = FirstItemExpression.makeFirstItemExpression(this);
+            return tc.staticTypeCheck(first, req, role, visitor);
+        }
+        Expression[] checked = new Expression[operanda.length];
+        SequenceType subReq = req;
+        if (req.getCardinality() != StaticProperty.ALLOWS_ZERO_OR_MORE) {
+            subReq = SequenceType.makeSequenceType(req.getPrimaryType(), StaticProperty.ALLOWS_ZERO_OR_MORE);
+        }
+        for (int i=0; i<operanda.length; i++) {
+            checked[i] = tc.staticTypeCheck(operanda[i].getChildExpression(), subReq, role, visitor);
+        }
+        Block b2 = new Block(checked);
+        ExpressionTool.copyLocationInfo(this, b2);
+        b2.allNodesUntyped = allNodesUntyped;
+
+        int reqCard = req.getCardinality();
+        int suppliedCard = b2.getCardinality();
+        if (!Cardinality.subsumes(req.getCardinality(), suppliedCard)) {
+            if ((reqCard & suppliedCard) == 0) {
+                XPathException err = new XPathException(
+                        "The required cardinality of the " + role.getMessage() +
+                                " is " + Cardinality.toString(reqCard) +
+                                ", but the supplied cardinality is " + Cardinality.toString(suppliedCard), role.getErrorCode(), getLocation());
+                err.setIsTypeError(true);
+                err.setFailingExpression(this);
+                throw err;
+            } else {
+                return CardinalityChecker.makeCardinalityChecker(b2, reqCard, role);
+            }
+        }
+        return b2;
+    }
+
+
     /**
      * Determine whether the block includes any instructions that might return nodes with a type annotation
      * @param insn the instruction (for example this block)
@@ -256,19 +307,18 @@ public class Block extends Instruction {
      * @return true if any expression in the block can return type-annotated nodes
      */
 
-    public static boolean mayReturnTypedNodes(Instruction insn, TypeHierarchy th) {
+    public static boolean neverReturnsTypedNodes(Instruction insn, TypeHierarchy th) {
         for (Operand o : insn.operands()) {
             Expression exp = o.getChildExpression();
-            if ((exp.getSpecialProperties() & StaticProperty.ALL_NODES_UNTYPED) == 0) {
+            if (!exp.hasSpecialProperty(StaticProperty.ALL_NODES_UNTYPED)) {
                 ItemType it = exp.getItemType();
                 if (th.relationship(it, NodeKindTest.ELEMENT) != TypeHierarchy.DISJOINT ||
-                        th.relationship(it, NodeKindTest.ATTRIBUTE) != TypeHierarchy.DISJOINT ||
                         th.relationship(it, NodeKindTest.ATTRIBUTE) != TypeHierarchy.DISJOINT) {
-                    return true;
+                    return false;
                 }
             }
         }
-        return false;
+        return true;
     }
 
     /**
@@ -290,7 +340,7 @@ public class Block extends Instruction {
             }
         }
         if (hasAdjacentTextNodes) {
-            List<Expression> content = new ArrayList<Expression>(size());
+            List<Expression> content = new ArrayList<>(size());
             String pendingText = null;
             for (int i = 0; i < size(); i++) {
                 if (isLiteralText[i]) {
@@ -320,7 +370,7 @@ public class Block extends Instruction {
      * Copy an expression. This makes a deep copy.
      *
      * @return the copy of the original expression
-     * @param rebindings
+     * @param rebindings Variables that need to be re-bound
      */
 
     /*@NotNull*/
@@ -355,7 +405,7 @@ public class Block extends Instruction {
             Expression child = child(i);
             if (!(child instanceof Message)) {
                 ItemType t = child.getItemType();
-                t1 = (t1 == null ? t : Type.getCommonSuperType(t1, t, th));
+                t1 = t1 == null ? t : Type.getCommonSuperType(t1, t, th);
                 if (t1 instanceof AnyItemType) {
                     return t1;  // no point going any further
                 }
@@ -369,7 +419,7 @@ public class Block extends Instruction {
      * inference rules defined in the XSLT 3.0 specification.
      *
      * @return the static item type of the expression according to the XSLT 3.0 defined rules
-     * @param contextItemType
+     * @param contextItemType the static type of the context item
      */
     @Override
     public UType getStaticUType(UType contextItemType) {
@@ -394,14 +444,14 @@ public class Block extends Instruction {
      * Determine the cardinality of the expression
      */
 
-    public final int getCardinality() {
+    public int computeCardinality() {
         if (size() == 0) {
             return StaticProperty.EMPTY;
         }
         int c1 = child(0).getCardinality();
         for (int i = 1; i < size(); i++) {
             c1 = Cardinality.sum(c1, child(i).getCardinality());
-            if (c1 == StaticProperty.ALLOWS_ZERO_OR_MORE) {
+            if (c1 == StaticProperty.ALLOWS_MANY) {
                 break;
             }
         }
@@ -435,7 +485,7 @@ public class Block extends Instruction {
         boolean nonUpdating = false;
         for (Operand o : operands()) {
             Expression child = o.getChildExpression();
-            if (!ExpressionTool.isAllowedInUpdatingContext(child)) {
+            if (ExpressionTool.isNotAllowedInUpdatingContext(child)) {
                 if (updating) {
                     XPathException err = new XPathException(
                             "If any subexpression is updating, then all must be updating", "XUST0001");
@@ -509,7 +559,7 @@ public class Block extends Instruction {
             result.setParentExpression(getParentExpression());
             return result;
         } else if (nested) {
-            List<Expression> list = new ArrayList<Expression>(size() * 2);
+            List<Expression> list = new ArrayList<>(size() * 2);
             flatten(list);
             Expression[] children = new Expression[list.size()];
             for (int i = 0; i < list.size(); i++) {
@@ -523,8 +573,7 @@ public class Block extends Instruction {
             for (int c = 0; c < size(); c++) {
                 values[c] = (AtomicValue) ((Literal) child(c)).getValue();
             }
-            Expression result = Literal.makeLiteral(new SequenceExtent(values));
-            ExpressionTool.copyLocationInfo(this, result);
+            Expression result = Literal.makeLiteral(new SequenceExtent<>(values), this);
             result.setParentExpression(getParentExpression());
             return result;
         } else {
@@ -554,7 +603,7 @@ public class Block extends Instruction {
             } else if (child instanceof Literal && !(((Literal) child).getValue() instanceof IntegerRange)) {
                 SequenceIterator iterator = ((Literal) child).getValue().iterate();
                 if (currentLiteralList == null) {
-                    currentLiteralList = new ArrayList<Item>(10);
+                    currentLiteralList = new ArrayList<>(10);
                 }
                 Item item;
                 while ((item = iterator.next()) != null) {
@@ -572,9 +621,8 @@ public class Block extends Instruction {
 
     private void flushCurrentLiteralList(List<Item> currentLiteralList, List<Expression> list) throws XPathException {
         if (currentLiteralList != null) {
-            SequenceIterator iter = new net.sf.saxon.tree.iter.ListIterator(currentLiteralList);
-            Literal lit = Literal.makeLiteral(SequenceExtent.makeSequenceExtent(iter));
-            lit.setRetainedStaticContext(getRetainedStaticContext());
+            ListIterator<Item> iter = new ListIterator<>(currentLiteralList);
+            Literal lit = Literal.makeLiteral(iter.materialize(), this);
             list.add(lit);
         }
     }
@@ -598,7 +646,7 @@ public class Block extends Instruction {
     /*@NotNull*/
     public Expression typeCheck(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
         typeCheckChildren(visitor, contextInfo);
-        if (!mayReturnTypedNodes(this, visitor.getConfiguration().getTypeHierarchy())) {
+        if (neverReturnsTypedNodes(this, visitor.getConfiguration().getTypeHierarchy())) {
             resetLocalStaticProperties();
             allNodesUntyped = true;
         }
@@ -628,7 +676,7 @@ public class Block extends Instruction {
             }
         }
         if (canSimplify) {
-            List<Expression> list = new ArrayList<Expression>(size() * 2);
+            List<Expression> list = new ArrayList<>(size() * 2);
             flatten(list);
             Expression result = Block.makeBlock(list);
             result.setRetainedStaticContext(getRetainedStaticContext());
@@ -713,7 +761,7 @@ public class Block extends Instruction {
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<? extends Item> iterate(XPathContext context) throws XPathException {
         if (size() == 0) {
             return EmptyIterator.emptyIterator();
         } else if (size() == 1) {
