@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -34,7 +34,7 @@ import java.util.List;
  * Also used for typeswitch in XQuery.
  */
 
-public class Choose extends Instruction {
+public class Choose extends Instruction implements ConditionalInstruction {
 
     private Operand[] conditionOps;
     private Operand[] actionOps;
@@ -87,7 +87,7 @@ public class Choose extends Instruction {
             Expression[] actions = new Expression[]{thenExp};
             return new Choose(conditions, actions);
         } else {
-            Expression[] conditions = new Expression[]{condition, Literal.makeLiteral(BooleanValue.TRUE)};
+            Expression[] conditions = new Expression[]{condition, Literal.makeLiteral(BooleanValue.TRUE, condition)};
             Expression[] actions = new Expression[]{thenExp, elseExp};
             return new Choose(conditions, actions);
         }
@@ -382,6 +382,7 @@ public class Choose extends Instruction {
             XPathException err = TypeChecker.ebvError(getCondition(i), th);
             if (err != null) {
                 err.setLocator(getCondition(i).getLocation());
+                err.maybeSetFailingExpression(getCondition(i));
                 throw err;
             }
         }
@@ -393,6 +394,7 @@ public class Choose extends Instruction {
                 actionOps[i].typeCheck(visitor, contextInfo);
             } catch (XPathException err) {
                 err.maybeSetLocation(getLocation());
+                err.maybeSetFailingExpression(getAction(i));
                 // mustn't throw the error unless the branch is actually selected, unless its a static or type error
                 if (err.isStaticError()) {
                     throw err;
@@ -412,7 +414,11 @@ public class Choose extends Instruction {
                 }
             }
         }
-        return removeRedundantBranches(visitor);
+        Expression reduced = removeRedundantBranches(visitor);
+        if (reduced != this) {
+            return reduced.typeCheck(visitor, contextInfo);
+        }
+        return reduced;
     }
 
     /**
@@ -456,12 +462,13 @@ public class Choose extends Instruction {
             } catch (XPathException err) {
                 if (err.isStaticError()) {
                     throw err;
-                } else if (err.isTypeError()) {
-                    visitor.issueWarning("Branch " + (i + 1) + " of conditional will fail with a type error if executed. "
-                        + err.getMessage(), getLocation());
                 }
+//                else if (err.isTypeError()) {
+//                    visitor.issueWarning("Branch " + (i + 1) + " of conditional will fail with a type error if executed. "
+//                        + err.getMessage(), getLocation());
+//                }
                 ErrorExpression ee = new ErrorExpression(err);
-                ExpressionTool.copyLocationInfo(getAction(0), ee);
+                ExpressionTool.copyLocationInfo(getAction(i), ee);
                 setAction(i, ee);
             }
         }
@@ -475,7 +482,7 @@ public class Choose extends Instruction {
                 c[i] = getCondition(i);
                 a[i] = getAction(i);
             }
-            c[size] = Literal.makeLiteral(BooleanValue.TRUE);
+            c[size] = Literal.makeLiteral(BooleanValue.TRUE, this);
             String cond = size == 1 ? "The condition is not" : "None of the conditions is";
             String message =
                     "Conditional expression: " + cond + " satisfied, so an empty sequence is returned, " +
@@ -506,7 +513,7 @@ public class Choose extends Instruction {
                     err.setLocation(getLocation());
                     throw err;
                 }
-                setCondition(i, Literal.makeLiteral(BooleanValue.get(b)));
+                setCondition(i, Literal.makeLiteral(BooleanValue.get(b), this));
             }
         }
         for (int i = 0; i < size; i++) {
@@ -517,9 +524,17 @@ public class Choose extends Instruction {
                 if (err.isTypeError()) {
                     throw err;
                 } else {
-                    setAction(i, new ErrorExpression(err));
+                    ErrorExpression ee = new ErrorExpression(err);
+                    ExpressionTool.copyLocationInfo(actionOps[i].getChildExpression(), ee);
+                    setAction(i, ee);
                 }
             }
+            if (getAction(i) instanceof ErrorExpression &&
+                    ((ErrorExpression)getAction(i)).isTypeError() &&
+                    !Literal.isConstantBoolean(getCondition(i), false)) {
+                visitor.issueWarning("Branch " + (i + 1) + " of conditional will fail with a type error if executed. "
+                        + ((ErrorExpression) getAction(i)).getMessage(), getAction(i).getLocation());
+            }
         }
 
         if (size == 0) {
@@ -527,7 +542,7 @@ public class Choose extends Instruction {
         }
         Expression e = removeRedundantBranches(visitor);
         if (e instanceof Choose) {
-            return visitor.getConfiguration().obtainOptimizer().trySwitch((Choose) e, visitor);
+            return visitor.obtainOptimizer().trySwitch((Choose) e, visitor);
         } else {
             return e;
         }
@@ -580,7 +595,7 @@ public class Choose extends Instruction {
         for (Operand o : actions()) {
             Expression act = o.getChildExpression();
             act.checkForUpdatingSubexpressions();
-            if (!ExpressionTool.isAllowedInUpdatingContext(act)) {
+            if (ExpressionTool.isNotAllowedInUpdatingContext(act)) {
                 if (updating) {
                     XPathException err = new XPathException(
                             "If any branch of a conditional is an updating expression, then all must be updating expressions (or vacuous)",
@@ -746,7 +761,7 @@ public class Choose extends Instruction {
     public final boolean mayCreateNewNodes() {
         for (Operand action : actions()) {
             int props = action.getChildExpression().getSpecialProperties();
-            if ((props & StaticProperty.NON_CREATIVE) == 0) {
+            if ((props & StaticProperty.NO_NODES_NEWLY_CREATED) == 0) {
                 return true;
             }
         }
@@ -787,7 +802,6 @@ public class Choose extends Instruction {
     /**
      * Add a representation of this expression to a PathMap. The PathMap captures a map of the nodes visited
      * by an expression in a source tree.
-     * <p/>
      * <p>The default implementation of this method assumes that an expression does no navigation other than
      * the navigation done by evaluating its subexpressions, and that the subexpressions are evaluated in the
      * same context as the containing expression. The method must be overridden for any expression
@@ -900,7 +914,7 @@ public class Choose extends Instruction {
             try {
                 b = getCondition(i).effectiveBooleanValue(context);
             } catch (XPathException e) {
-                e.maybeSetLocation(getCondition(i).getLocation());
+                e.maybeSetFailingExpression(getCondition(i));
                 throw e;
             }
             if (b) {
@@ -936,9 +950,8 @@ public class Choose extends Instruction {
      * expressions. This default implementation relies on the process() method: it
      * "pushes" the results of the instruction to a sequence in memory, and then
      * iterates over this in-memory sequence.
-     * <p/>
-     * In principle instructions should implement a pipelined iterate() method that
-     * avoids the overhead of intermediate storage.
+     * <p>In principle instructions should implement a pipelined iterate() method that
+     * avoids the overhead of intermediate storage.</p>
      *
      * @param context supplies the context for evaluation
      * @return a SequenceIterator that can be used to iterate over the result
@@ -948,7 +961,7 @@ public class Choose extends Instruction {
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<? extends Item> iterate(XPathContext context) throws XPathException {
         int i = choose(context);
         return i < 0 ? EmptyIterator.emptyIterator() : getAction(i).iterate(context);
     }
