@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -319,7 +319,7 @@ public class Copy extends ElementCreator {
                             new InstanceOfExpression(
                                     new ContextItemExpression(),
                                     SequenceType.makeSequenceType(leafTest, StaticProperty.EXACTLY_ONE)),
-                            Literal.makeLiteral(BooleanValue.TRUE)};
+                            Literal.makeLiteral(BooleanValue.TRUE, this)};
                     Expression[] actions = new Expression[]{copyOf, this};
                     Choose choose = new Choose(conditions, actions);
                     ExpressionTool.copyLocationInfo(this, choose);
@@ -393,14 +393,14 @@ public class Copy extends ElementCreator {
 
     public TailCall processLeavingTail(XPathContext context) throws XPathException {
         Controller controller = context.getController();
-        SequenceReceiver out = context.getReceiver();
+        Receiver out = context.getReceiver();
         Item item = context.getContextItem();//select.evaluateItem(context);
         if (item == null) {
             return null;
         }
 
         if (!(item instanceof NodeInfo)) {
-            out.append(item, getLocation(), NodeInfo.ALL_NAMESPACES);
+            out.append(item, getLocation(), ReceiverOptions.ALL_NAMESPACES);
             return null;
         }
         NodeInfo source = (NodeInfo) item;
@@ -453,7 +453,7 @@ public class Copy extends ElementCreator {
                 break;
 
             case Type.DOCUMENT:
-                SequenceReceiver saved = out;
+                Receiver saved = out;
                 boolean pop = false;
                 if (!preservingTypes) {
                     ParseOptions options = new ParseOptions(getValidationOptions());
@@ -485,7 +485,7 @@ public class Copy extends ElementCreator {
         return null;
     }
 
-    public static void copyUnparsedEntities(NodeInfo source, SequenceReceiver out) throws XPathException {
+    public static void copyUnparsedEntities(NodeInfo source, Receiver out) throws XPathException {
         Iterator<String> unparsedEntities = source.getTreeInfo().getUnparsedEntityNames();
         while (unparsedEntities.hasNext()) {
             String n = unparsedEntities.next();
@@ -502,7 +502,7 @@ public class Copy extends ElementCreator {
 
     public Item evaluateItem(XPathContext context) throws XPathException {
         Controller controller = context.getController();
-        SequenceReceiver saved = context.getReceiver();
+        Receiver saved = context.getReceiver();
         SequenceOutputter seq = controller.allocateSequenceOutputter(1);
         seq.getPipelineConfiguration().setHostLanguage(getPackageData().getHostLanguage());
         seq.getPipelineConfiguration().setLocationIsCodeLocation(true);
