@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -83,7 +83,7 @@ public class Doctype extends Instruction {
     public TailCall processLeavingTail(XPathContext context) throws XPathException {
         Controller controller = context.getController();
 
-        SequenceReceiver out = context.getReceiver();
+        Receiver out = context.getReceiver();
         PipelineConfiguration pipe = controller.makePipelineConfiguration();
         pipe.setLocationIsCodeLocation(true);
         pipe.setHostLanguage(getPackageData().getHostLanguage());
