@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2015 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,14 +7,13 @@
 
 package net.sf.saxon.expr.instruct;
 
-import com.saxonica.ee.stream.adjunct.ElementCreatorAdjunct;
-import com.saxonica.ee.stream.adjunct.StreamingAdjunct;
 import net.sf.saxon.Controller;
 import net.sf.saxon.event.*;
-import net.sf.saxon.evpull.*;
 import net.sf.saxon.expr.*;
 import net.sf.saxon.lib.ParseOptions;
 import net.sf.saxon.lib.Validation;
+import net.sf.saxon.ma.arrays.ArrayItemType;
+import net.sf.saxon.ma.map.MapType;
 import net.sf.saxon.om.Item;
 import net.sf.saxon.om.NamespaceBinding;
 import net.sf.saxon.om.NodeInfo;
@@ -79,6 +78,7 @@ public abstract class ElementCreator extends ParentNodeConstructor {
 
     /**
      * Say whether this element causes its children to inherit namespaces
+     *
      * @param inherit true if namespaces are to be inherited by the children
      */
 
@@ -152,64 +152,77 @@ public abstract class ElementCreator extends ParentNodeConstructor {
      * after creating any child nodes
      *
      * @param env the static context
-     * @throws XPathException
+     * @throws XPathException if the content is found to be incorrect
      */
 
     protected void checkContentSequence(StaticContext env) throws XPathException {
+        Operand[] components;
         if (getContentExpression() instanceof Block) {
-            // TODO: why no checking if it isn't a Block? cf DocumentInstr which does such checking
-            Operand[] components = ((Block) getContentExpression()).getOperanda();
-            boolean foundChild = false;
-            boolean foundPossibleChild = false;
-            for (Operand o : components) {
-                 Expression component = o.getChildExpression();
-                ItemType it = component.getItemType();
-                if (it instanceof NodeTest) {
-                    boolean maybeEmpty = Cardinality.allowsZero(component.getCardinality());
-                    UType possibleNodeKinds = it.getUType();
-                    if (possibleNodeKinds.overlaps(UType.TEXT)) {
-                        // the text node might turn out to be zero-length. If that's a possibility,
-                        // then we only issue a warning. Also, we need to completely ignore a known
-                        // zero-length text node, which is included to prevent space-separation
-                        // in an XQuery construct like <a>{@x}{@y}</b>
-                        if (component instanceof ValueOf &&
-                                ((ValueOf) component).getSelect() instanceof StringLiteral) {
-                            String value = ((StringLiteral) ((ValueOf) component).getSelect()).getStringValue();
-                            if (value.isEmpty()) {
-                                // continue;  // not an error
-                            } else {
-                                foundChild = true;
-                            }
-                        } else {
-                            foundPossibleChild = true;
-                        }
-                    } else if (!possibleNodeKinds.overlaps(UType.CHILD_NODE_KINDS)) {
-                        if (maybeEmpty) {
-                            foundPossibleChild = true;
+            components = ((Block) getContentExpression()).getOperanda();
+        } else {
+            components = new Operand[]{contentOp};
+        }
+
+        boolean foundChild = false;
+        boolean foundPossibleChild = false;
+        for (Operand o : components) {
+            Expression component = o.getChildExpression();
+            ItemType it = component.getItemType();
+            if (it.isAtomicType()) {
+                foundChild = true;
+            } else if (it instanceof FunctionItemType && !(it instanceof ArrayItemType)) {
+                String which = it instanceof MapType ? "map" : "function";
+                XPathException de = new XPathException(
+                        "Cannot add a " + which + " as a child of a constructed element");
+                de.setErrorCode(isXSLT() ? "XTDE0450" : "XQTY0105");
+                de.setLocator(component.getLocation());
+                de.setIsTypeError(true);
+                throw de;
+            } else if (it instanceof NodeTest) {
+                boolean maybeEmpty = Cardinality.allowsZero(component.getCardinality());
+                UType possibleNodeKinds = it.getUType();
+                if (possibleNodeKinds.overlaps(UType.TEXT)) {
+                    // the text node might turn out to be zero-length. If that's a possibility,
+                    // then we only issue a warning. Also, we need to completely ignore a known
+                    // zero-length text node, which is included to prevent space-separation
+                    // in an XQuery construct like <a>{@x}{@y}</b>
+                    if (component instanceof ValueOf &&
+                            ((ValueOf) component).getSelect() instanceof StringLiteral) {
+                        String value = ((StringLiteral) ((ValueOf) component).getSelect()).getStringValue();
+                        if (value.isEmpty()) {
+                            // continue;  // not an error
                         } else {
                             foundChild = true;
                         }
-                    } else if (foundChild && possibleNodeKinds == UType.ATTRIBUTE && !maybeEmpty) {
-                        XPathException de = new XPathException(
-                                "Cannot create an attribute node after creating a child of the containing element");
-                        de.setErrorCode(isXSLT() ? "XTDE0410" : "XQTY0024");
-                        de.setLocator(component.getLocation());
-                        throw de;
-                    } else if (foundChild && possibleNodeKinds == UType.NAMESPACE && !maybeEmpty) {
-                        XPathException de = new XPathException(
-                                "Cannot create a namespace node after creating a child of the containing element");
-                        de.setErrorCode(isXSLT() ? "XTDE0410" : "XQTY0024");
-                        de.setLocator(component.getLocation());
-                        throw de;
-                    } else if ((foundChild || foundPossibleChild) && possibleNodeKinds == UType.ATTRIBUTE) {
-                        env.issueWarning(
-                                "Creating an attribute here will fail if previous instructions create any children",
-                                component.getLocation());
-                    } else if ((foundChild || foundPossibleChild) && possibleNodeKinds == UType.NAMESPACE) {
-                        env.issueWarning(
-                                "Creating a namespace node here will fail if previous instructions create any children",
-                                component.getLocation());
+                    } else {
+                        foundPossibleChild = true;
                     }
+                } else if (!possibleNodeKinds.overlaps(UType.CHILD_NODE_KINDS)) {
+                    if (maybeEmpty) {
+                        foundPossibleChild = true;
+                    } else {
+                        foundChild = true;
+                    }
+                } else if (foundChild && possibleNodeKinds == UType.ATTRIBUTE && !maybeEmpty) {
+                    XPathException de = new XPathException(
+                            "Cannot create an attribute node after creating a child of the containing element");
+                    de.setErrorCode(isXSLT() ? "XTDE0410" : "XQTY0024");
+                    de.setLocator(component.getLocation());
+                    throw de;
+                } else if (foundChild && possibleNodeKinds == UType.NAMESPACE && !maybeEmpty) {
+                    XPathException de = new XPathException(
+                            "Cannot create a namespace node after creating a child of the containing element");
+                    de.setErrorCode(isXSLT() ? "XTDE0410" : "XQTY0024");
+                    de.setLocator(component.getLocation());
+                    throw de;
+                } else if ((foundChild || foundPossibleChild) && possibleNodeKinds == UType.ATTRIBUTE) {
+                    env.issueWarning(
+                            "Creating an attribute here will fail if previous instructions create any children",
+                            component.getLocation());
+                } else if ((foundChild || foundPossibleChild) && possibleNodeKinds == UType.NAMESPACE) {
+                    env.issueWarning(
+                            "Creating a namespace node here will fail if previous instructions create any children",
+                            component.getLocation());
                 }
             }
 
@@ -257,8 +270,7 @@ public abstract class ElementCreator extends ParentNodeConstructor {
      * Callback to get a list of the intrinsic namespaces that need to be generated for the element.
      *
      * @return the set of namespace bindings.
-     * @throws net.sf.saxon.trans.XPathException
-     *          if an error occurs
+     * @throws net.sf.saxon.trans.XPathException if an error occurs
      */
 
     public NamespaceBinding[] getActiveNamespaces() throws XPathException {
@@ -274,39 +286,13 @@ public abstract class ElementCreator extends ParentNodeConstructor {
         return Expression.PROCESS_METHOD | Expression.EVALUATE_METHOD;
     }
 
-    public EventIterator iterateEvents(XPathContext context) throws XPathException {
-        return iterateEvents(context, null);
-    }
-
-    protected EventIterator iterateEvents(XPathContext context, /*@Nullable*/ NodeInfo copiedNode) throws XPathException {
-        if (!preservingTypes && getValidationAction() != Validation.STRIP) {
-            // Schema validation can't be done in pull mode
-            return new SingletonEventIterator(evaluateItem(context));
-        }
-
-        final Controller controller = context.getController();
-        assert controller != null;
-        StartElementEvent start = new StartElementEvent(controller.makePipelineConfiguration());
-        start.setElementName(getElementName(context, copiedNode));
-        start.setTypeCode(getValidationAction() == Validation.PRESERVE ? AnyType.getInstance() : Untyped.getInstance());
-        start.setLocalNamespaces(getActiveNamespaces());
-        start.setLocation(getLocation());
-        EventIterator result = new BracketedElementIterator(
-                start, getContentExpression().iterateEvents(context), EndElementEvent.getInstance());
-        if (getValidationAction() == Validation.STRIP && controller.getExecutable().isSchemaAware()) {
-            return new EventAnnotationStripper(result);
-        } else {
-            return result;
-        }
-    }
-
     /**
      * Evaluate the instruction to produce a new element node. This method is typically used when there is
      * a parent element or document in a result tree, to which the new element is added.
      *
      * @param context XPath dynamic evaluation context
      * @return null (this instruction never returns a tail call)
-     * @throws XPathException
+     * @throws XPathException if a dynamic error occurs
      */
     public TailCall processLeavingTail(XPathContext context)
             throws XPathException {
@@ -330,8 +316,8 @@ public abstract class ElementCreator extends ParentNodeConstructor {
             NodeName elemName = getElementName(context, copiedNode);
             SchemaType typeCode = getValidationAction() == Validation.PRESERVE ? AnyType.getInstance() : Untyped.getInstance();
 
-            SequenceReceiver out = context.getReceiver();
-            SequenceReceiver saved = out;
+            Receiver out = context.getReceiver();
+            Receiver saved = out;
             boolean pop = false;
             Receiver elemOut = out;
             if (!preservingTypes) {
@@ -386,13 +372,8 @@ public abstract class ElementCreator extends ParentNodeConstructor {
      */
 
     public Item evaluateItem(XPathContext context) throws XPathException {
-        if (isLazyConstruction() && preservingTypes) {
-            return context.getConfiguration().makeUnconstructedElement(this, context);
-        } else {
-            NodeInfo node = constructElement(context, null);
-            // TODO: recover from validation errors that might have occurred
-            return node;
-        }
+        return constructElement(context, null);
+        // TODO: recover from validation errors that might have occurred
     }
 
     /**
@@ -406,9 +387,9 @@ public abstract class ElementCreator extends ParentNodeConstructor {
     private NodeInfo constructElement(XPathContext context, /*@Nullable*/ NodeInfo copiedNode) throws XPathException {
         try {
             Controller controller = context.getController();
-            SequenceReceiver saved = context.getReceiver();
+            Receiver saved = context.getReceiver();
             SequenceOutputter seq = controller.allocateSequenceOutputter(1);
-            seq.getPipelineConfiguration().setHostLanguage(getHostLanguage());
+            seq.getPipelineConfiguration().setHostLanguage(getPackageData().getHostLanguage());
             seq.getPipelineConfiguration().setLocationIsCodeLocation(true);
 
             NodeName elemName = getElementName(context, copiedNode);
@@ -461,10 +442,6 @@ public abstract class ElementCreator extends ParentNodeConstructor {
             return result;
 
         } catch (XPathException err) {
-            if (err instanceof ValidationException) {
-                ((ValidationException) err).setLocation(getLocation());
-                ((ValidationException) err).setSystemId(getSystemId());
-            }
             err.maybeSetLocation(getLocation());
             err.maybeSetContext(context);
             throw err;
@@ -499,20 +476,15 @@ public abstract class ElementCreator extends ParentNodeConstructor {
         inheritNamespacesToChildren = !flags.contains("C");
     }
 
-//#ifdefined STREAM
-
     /**
-     * Get a class that supports streamed evaluation of this expression
+     * Get the (partial) name of a class that supports streaming of this kind of expression
      *
-     * @return the relevant StreamingAdjunct, or null if none is available
+     * @return the partial name of a class that can be instantiated to provide streaming support in Saxon-EE,
+     * or null if there is no such class
      */
     @Override
-    public StreamingAdjunct getStreamingAdjunct() {
-        return new ElementCreatorAdjunct();
+    public String getStreamerName() {
+        return "ElementCreator";
     }
-
-//#endif
-
-
 }
 
