@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -11,16 +11,18 @@ import net.sf.saxon.Configuration;
 import net.sf.saxon.expr.PackageData;
 import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.expr.parser.ICompilerService;
+import net.sf.saxon.expr.parser.RoleDiagnostic;
 import net.sf.saxon.functions.FunctionLibraryList;
 import net.sf.saxon.om.Item;
 import net.sf.saxon.om.StructuredQName;
 import net.sf.saxon.query.QueryModule;
 import net.sf.saxon.serialize.CharacterMapIndex;
+import net.sf.saxon.serialize.SerializationProperties;
 import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.KeyManager;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.type.ItemType;
-import net.sf.saxon.type.Type;
+import net.sf.saxon.type.TypeHierarchy;
 
 import java.util.*;
 
@@ -44,7 +46,7 @@ public class Executable {
     private PackageData topLevelPackage;
 
     // the set of packages making up this Executable
-    private List<PackageData> packages = new ArrayList<PackageData>();
+    private List<PackageData> packages = new ArrayList<>();
 
     // definitions of keys, including keys created by the optimizer
     private KeyManager keyManager;
@@ -55,9 +57,6 @@ public class Executable {
     // default output properties (for the unnamed output format)
     private Properties defaultOutputProperties;
 
-    // table of named collations defined in the stylesheet/query
-    //private CollationMap collationTable;
-
     // table of character maps indexed by StructuredQName
     private CharacterMapIndex characterMapIndex;
 
@@ -74,7 +73,7 @@ public class Executable {
     private int hostLanguage = Configuration.XSLT;
 
     // a list of required parameters, identified by the structured QName of their names
-    private Map<StructuredQName, GlobalParam> globalParams = new HashMap<StructuredQName, GlobalParam>();
+    private Map<StructuredQName, GlobalParam> globalParams = new HashMap<>();
 
     // Hash table of named (and unnamed) output declarations. This is assembled only
     // if there is a need for it: that is, if there is a call on xsl:result-document
@@ -256,16 +255,20 @@ public class Executable {
     }
 
     /**
-     * Get the default output properties
+     * Get the primary output properties, that is, the output properties for the primary
+     * result destination. In XSLT these are the properties defined on unnamed {@code xsl:output}
+     * declarations in the top-level stylesheet package; in XQuery they are the properties
+     * defined using {@code "declare option output:x = 'value'"}.
      *
      * @return the properties for the unnamed output format
      */
 
-    public Properties getDefaultOutputProperties() {
+    public SerializationProperties getPrimarySerializationProperties() {
         if (defaultOutputProperties == null) {
             defaultOutputProperties = new Properties();
         }
-        return defaultOutputProperties;
+        Properties props = defaultOutputProperties;
+        return new SerializationProperties(props, getCharacterMapIndex());
     }
 
     /**
@@ -277,7 +280,7 @@ public class Executable {
 
     public void setOutputProperties(StructuredQName qName, Properties properties) {
         if (outputDeclarations == null) {
-            outputDeclarations = new HashMap<StructuredQName, Properties>(5);
+            outputDeclarations = new HashMap<>(5);
         }
         outputDeclarations.put(qName, properties);
     }
@@ -310,12 +313,12 @@ public class Executable {
 
     public void addQueryLibraryModule(QueryModule module) {
         if (queryLibraryModules == null) {
-            queryLibraryModules = new HashMap<String, List<QueryModule>>(5);
+            queryLibraryModules = new HashMap<>(5);
         }
         String uri = module.getModuleNamespace();
         List<QueryModule> existing = queryLibraryModules.get(uri);
         if (existing == null) {
-            existing = new ArrayList<QueryModule>(5);
+            existing = new ArrayList<>(5);
             existing.add(module);
             queryLibraryModules.put(uri, existing);
         } else if (!existing.contains(module)){
@@ -374,7 +377,7 @@ public class Executable {
         if (queryLibraryModules == null) {
             return Collections.EMPTY_LIST.iterator();
         } else {
-            List<QueryModule> modules = new ArrayList<QueryModule>();
+            List<QueryModule> modules = new ArrayList<>();
             for (List<QueryModule> queryModules : queryLibraryModules.values()) {
                 modules.addAll(queryModules);
             }
@@ -393,7 +396,7 @@ public class Executable {
 
     public void addQueryLocationHintProcessed(String uri) {
         if (queryLocationHintsProcessed == null) {
-            queryLocationHintsProcessed = new HashSet<String>();
+            queryLocationHintsProcessed = new HashSet<>();
         }
         queryLocationHintsProcessed.add(uri);
     }
@@ -580,6 +583,7 @@ public class Executable {
         if (contextItem != null && globalContextRequirement.isAbsentFocus()) {
             throw new XPathException("The global context item is required to be absent", "XPDY0002");
         }
+        TypeHierarchy th = config.getTypeHierarchy();
         if (contextItem == null) {
             if (!globalContextRequirement.isMayBeOmitted()) {
                 // Bug 30173 allocates an error code
@@ -605,20 +609,19 @@ public class Executable {
                     throw new XPathException("The context item cannot be initialized to an empty sequence", "XPTY0004");
                 }
                 for (ItemType type : globalContextRequirement.getRequiredItemTypes()) {
-                    if (!type.matches(contextItem, config.getTypeHierarchy())) {
-                        throw new XPathException("The defaulted global context item is of type " +
-                                                         Type.getItemType(contextItem, context.getConfiguration().getTypeHierarchy()) +
-                                                         ", which does not match the required type " + type, "XPTY0004");
+                    if (!type.matches(contextItem, th)) {
+                        RoleDiagnostic role = new RoleDiagnostic(RoleDiagnostic.MISC, "defaulted global context item", 0);
+                        String s = role.composeErrorMessage(type, contextItem, th);
+                        throw new XPathException(s, "XPTY0004");
                     }
                 }
             }
         } else {
             for (ItemType type : globalContextRequirement.getRequiredItemTypes()) {
                 if (!type.matches(contextItem, config.getTypeHierarchy())) {
-                    throw new XPathException("The supplied global context item is of type " +
-                                                     Type.getItemType(contextItem, context.getConfiguration().getTypeHierarchy()) +
-                                                     ", which does not match the required type " + type,
-                                             getHostLanguage() == Configuration.XSLT ? "XTTE0590" : "XPTY0004");
+                    RoleDiagnostic role = new RoleDiagnostic(RoleDiagnostic.MISC, "supplied global context item", 0);
+                    String s = role.composeErrorMessage(type, contextItem, th);
+                    throw new XPathException(s, getHostLanguage() == Configuration.XSLT ? "XTTE0590" : "XPTY0004");
                 }
             }
         }
