@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2015 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,10 +7,6 @@
 
 package net.sf.saxon.expr.instruct;
 
-import com.saxonica.ee.bytecode.ExpressionCompiler;
-import com.saxonica.ee.bytecode.ForEachGroupCompiler;
-import com.saxonica.ee.stream.adjunct.ForEachGroupAdjunct;
-import com.saxonica.ee.stream.adjunct.StreamingAdjunct;
 import net.sf.saxon.Controller;
 import net.sf.saxon.expr.*;
 import net.sf.saxon.expr.parser.*;
@@ -39,7 +35,7 @@ import java.net.URISyntaxException;
  */
 
 public class ForEachGroup extends Instruction
-        implements SortKeyEvaluator, ContextMappingFunction, ContextSwitchingExpression {
+        implements SortKeyEvaluator, ContextMappingFunction<Item>, ContextSwitchingExpression {
 
     public static final int GROUP_BY = 0;
     public static final int GROUP_ADJACENT = 1;
@@ -80,7 +76,8 @@ public class ForEachGroup extends Instruction
                         SortKeyDefinitionList sortKeys) {
         selectOp = new Operand(this, select, OperandRole.FOCUS_CONTROLLING_SELECT);
         actionOp = new Operand(this, action, OperandRole.FOCUS_CONTROLLED_ACTION);
-        keyOp = new Operand(this, key, OperandRole.NEW_FOCUS_ATOMIC);
+        OperandRole keyRole = (algorithm == GROUP_ENDING || algorithm == GROUP_STARTING) ? OperandRole.PATTERN : OperandRole.NEW_FOCUS_ATOMIC;
+        keyOp = new Operand(this, key, keyRole);
         if (collationNameExpression != null) {
             collationOp = new Operand(this, collationNameExpression, OperandRole.SINGLE_ATOMIC);
         }
@@ -224,6 +221,19 @@ public class ForEachGroup extends Instruction
         isInFork = inFork;
     }
 
+    /**
+     * Ask whether common subexpressions found in the operands of this expression can
+     * be extracted and evaluated outside the expression itself. The result is irrelevant
+     * in the case of operands evaluated with a different focus, which will never be
+     * extracted in this way, even if they have no focus dependency.
+     *
+     * @return false for this kind of expression
+     */
+    @Override
+    public boolean allowExtractingCommonSubexpressions() {
+        return false;
+    }
+
     /*@NotNull*/
     public Expression typeCheck(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
 
@@ -237,7 +247,12 @@ public class ForEachGroup extends Instruction
             return Literal.makeEmptySequence();
         }
 
-        ContextItemStaticInfo cit = new ContextItemStaticInfo(selectedItemType, false, getSelectExpression());
+        for (Operand o : operands()) {
+            fixupGroupReferences(this, this, selectedItemType, false);
+        }
+
+        ContextItemStaticInfo cit = visitor.getConfiguration().makeContextItemStaticInfo(selectedItemType, false);
+        cit.setContextSettingExpression(getSelectExpression());
 
         actionOp.typeCheck(visitor, cit);
         keyOp.typeCheck(visitor, cit);
@@ -280,9 +295,7 @@ public class ForEachGroup extends Instruction
             }
         }
         keyItemType = getGroupingKey().getItemType().getPrimitiveType();
-        for (Operand o : operands()) {
-            fixupGroupReferences(this, this, selectedItemType, false);
-        }
+
         return this;
     }
 
@@ -294,12 +307,12 @@ public class ForEachGroup extends Instruction
         } else if (exp instanceof ForEachGroup) {
             // a current-group() reference to the outer for-each-group can occur in the select expression
             // or in the AVTs of a contained xsl:sort
-            /// TODO: can also occur in group-by, group-adjacent, etc??
             ForEachGroup feg2 = (ForEachGroup)exp;
             if (feg2 == feg) {
                 fixupGroupReferences(feg2.getActionExpression(), feg, selectedItemType, false);
             } else {
                 fixupGroupReferences(feg2.getSelectExpression(), feg, selectedItemType, isInLoop);
+                fixupGroupReferences(feg2.getGroupingKey(), feg, selectedItemType, isInLoop);
                 if (feg2.getSortKeyDefinitions() != null) {
                     for (SortKeyDefinition skd : feg2.getSortKeyDefinitions()) {
                         fixupGroupReferences(skd.getOrder(), feg, selectedItemType, isInLoop);
@@ -323,7 +336,8 @@ public class ForEachGroup extends Instruction
     public Expression optimize(ExpressionVisitor visitor, ContextItemStaticInfo contextItemType) throws XPathException {
         selectOp.optimize(visitor, contextItemType);
         ItemType selectedItemType = getSelectExpression().getItemType();
-        ContextItemStaticInfo sit = new ContextItemStaticInfo(selectedItemType, false, getSelectExpression());
+        ContextItemStaticInfo sit = visitor.getConfiguration().makeContextItemStaticInfo(selectedItemType, false);
+        sit.setContextSettingExpression(getSelectExpression());
         actionOp.optimize(visitor, sit);
         keyOp.optimize(visitor, sit);
 
@@ -353,7 +367,7 @@ public class ForEachGroup extends Instruction
                     collationURI = getStaticBaseURI().resolve(collationURI);
                     final String collationNameString = collationURI.toString();
                     setCollationNameExpression(new StringLiteral(collationNameString));
-                    collator = getConfiguration().getCollation(collationNameString);
+                    collator = visitor.getConfiguration().getCollation(collationNameString);
                     if (collator == null) {
                         XPathException err = new XPathException("Unknown collation " + Err.wrap(collationURI.toString(), Err.URI));
                         err.setErrorCode("XTDE1110");
@@ -375,7 +389,9 @@ public class ForEachGroup extends Instruction
      * Copy an expression. This makes a deep copy.
      *
      * @return the copy of the original expression
-     * @param rebindings
+     * @param rebindings a mutable list of (old binding, new binding) pairs
+     *                   that is used to update the bindings held in any
+     *                   local variable references that are copied.
      */
 
     /*@NotNull*/
@@ -397,6 +413,7 @@ public class ForEachGroup extends Instruction
                 newKeyDef == null ? null : new SortKeyDefinitionList(newKeyDef));
         ExpressionTool.copyLocationInfo(this, feg);
         feg.setComposite(isComposite());
+        fixupGroupReferences(feg, feg, getSelectExpression().getItemType(), false);
         return feg;
     }
 
@@ -473,49 +490,25 @@ public class ForEachGroup extends Instruction
      * (Nodes created by the condition can't contribute to the result).
      */
 
-    public final boolean createsNewNodes() {
+    public final boolean mayCreateNewNodes() {
         int props = getActionExpression().getSpecialProperties();
-        return (props & StaticProperty.NON_CREATIVE) == 0;
+        return (props & StaticProperty.NO_NODES_NEWLY_CREATED) == 0;
     }
 
     /**
-     * Handle promotion offers, that is, non-local tree rewrites.
+     * Get the (partial) name of a class that supports streaming of this kind of expression
      *
-     * @param offer The type of rewrite being offered
-     * @throws XPathException
-     */
-
-    protected void promoteChildren(PromotionOffer offer) throws XPathException {
-        setSelect(doPromotion(getSelectExpression(), offer));
-        if (offer.action == PromotionOffer.EXTRACT_GLOBAL_VARIABLES) {
-            // Don't pass on other requests
-            setAction(doPromotion(getActionExpression(), offer));
-            setKey(doPromotion(getGroupingKey(), offer));
-        }
-        // TODO: promote expressions in the sort key definitions
-    }
-
-
-
-//#ifdefined STREAM
-
-    /**
-     * Get a class that supports streamed evaluation of this expression
-     *
-     * @return the relevant StreamingAdjunct, or null if none is available
+     * @return the partial name of a class that can be instantiated to provide streaming support in Saxon-EE,
+     * or null if there is no such class
      */
     @Override
-    public StreamingAdjunct getStreamingAdjunct() {
-        return new ForEachGroupAdjunct();
+    public String getStreamerName() {
+        return "ForEachGroup";
     }
 
-    //#endif
-
-
     /**
      * Add a representation of this expression to a PathMap. The PathMap captures a map of the nodes visited
      * by an expression in a source tree.
-     * <p/>
      * <p>The default implementation of this method assumes that an expression does no navigation other than
      * the navigation done by evaluating its subexpressions, and that the subexpressions are evaluated in the
      * same context as the containing expression. The method must be overridden for any expression
@@ -584,10 +577,9 @@ public class ForEachGroup extends Instruction
         assert controller != null;
 
         GroupIterator groupIterator = getGroupIterator(context);
-        FocusIterator focusIterator = new FocusTrackingIterator(groupIterator);
         XPathContextMajor c2 = context.newContext();
         c2.setOrigin(this);
-        c2.setCurrentIterator(focusIterator);
+        FocusIterator focusIterator = c2.trackFocus(groupIterator);
         c2.setCurrentGroupIterator(groupIterator);
         c2.setCurrentTemplateRule(null);
 
@@ -645,21 +637,11 @@ public class ForEachGroup extends Instruction
         }
     }
 
-    public AtomicComparer getAtomicComparer(XPathContext context) throws XPathException {
-        StringCollator coll = collator;
-        if (coll == null) {
-            // The collation is determined at run-time
-            coll = getCollator(context);
-        }
-        return AtomicSortComparer.makeSortComparer(coll, keyItemType, context);
-    }
-
     private GroupIterator getGroupIterator(XPathContext context) throws XPathException {
-        FocusIterator population = new FocusTrackingIterator(getSelectExpression().iterate(context));
-        return getGroupIterator(population, context);
+        return getGroupIterator(getSelectExpression(), context);
     }
 
-    public GroupIterator getGroupIterator(FocusIterator population, XPathContext context) throws XPathException {
+    public GroupIterator getGroupIterator(Expression select, XPathContext context) throws XPathException {
 
         // get an iterator over the groups in "order of first appearance"
 
@@ -672,8 +654,7 @@ public class ForEachGroup extends Instruction
                     coll = getCollator(context);
                 }
                 XPathContext c2 = context.newMinorContext();
-                c2.setCurrentIterator(population);
-                // TODO: how come this needs a new context and group-adjacent doesn't?
+                FocusIterator<? extends Item> population = c2.trackFocus(select.iterate(context));
                 groupIterator = new GroupByIterator(population, getGroupingKey(), c2, coll, composite);
                 break;
             }
@@ -683,14 +664,14 @@ public class ForEachGroup extends Instruction
                     // The collation is determined at run-time
                     coll = getCollator(context);
                 }
-                groupIterator = new GroupAdjacentIterator(population, getGroupingKey(), context, coll, composite);
+                groupIterator = new GroupAdjacentIterator(select, getGroupingKey(), context, coll, composite);
                 break;
             }
             case GROUP_STARTING:
-                groupIterator = new GroupStartingIterator(population, (Pattern) getGroupingKey(), context);
+                groupIterator = new GroupStartingIterator(select, (Pattern) getGroupingKey(), context);
                 break;
             case GROUP_ENDING:
-                groupIterator = new GroupEndingIterator(population, (Pattern) getGroupingKey(), context);
+                groupIterator = new GroupEndingIterator(select, (Pattern) getGroupingKey(), context);
                 break;
             default:
                 throw new AssertionError("Unknown grouping algorithm");
@@ -720,9 +701,8 @@ public class ForEachGroup extends Instruction
      * expressions. This default implementation relies on the process() method: it
      * "pushes" the results of the instruction to a sequence in memory, and then
      * iterates over this in-memory sequence.
-     * <p/>
-     * In principle instructions should implement a pipelined iterate() method that
-     * avoids the overhead of intermediate storage.
+     * <p>In principle instructions should implement a pipelined iterate() method that
+     * avoids the overhead of intermediate storage.</p>
      *
      * @param context supplies the context for evaluation
      * @return a SequenceIterator that can be used to iterate over the result
@@ -732,15 +712,14 @@ public class ForEachGroup extends Instruction
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<? extends Item> iterate(XPathContext context) throws XPathException {
         GroupIterator master = getGroupIterator(context);
-        FocusIterator focus = new FocusTrackingIterator(master);
         XPathContextMajor c2 = context.newContext();
         c2.setOrigin(this);
-        c2.setCurrentIterator(focus);
+        c2.trackFocus(master);
         c2.setCurrentGroupIterator(master);
         c2.setCurrentTemplateRule(null);
-        return new ContextMappingIterator(this, c2);
+        return new ContextMappingIterator<>(this, c2);
     }
 
     /**
@@ -753,7 +732,7 @@ public class ForEachGroup extends Instruction
      *         sequence.
      */
 
-    public SequenceIterator map(XPathContext context) throws XPathException {
+    public SequenceIterator<? extends Item> map(XPathContext context) throws XPathException {
         return getActionExpression().iterate(context);
     }
 
@@ -765,20 +744,11 @@ public class ForEachGroup extends Instruction
         return (AtomicValue) getSortKeyDefinitions().getSortKeyDefinition(n).getSortKey().evaluateItem(c);
     }
 
-//#ifdefined BYTECODE
-
-    /**
-     * Return the compiler of the ForEachGroup expression
-     *
-     * @return the relevant ExpressionCompiler
-     */
-    @Override
-    public ExpressionCompiler getExpressionCompiler() {
-        return new ForEachGroupCompiler();
-    }
-//#endif
 
     public SortKeyDefinitionList getSortKeyDefinitionList() {
+        if (sortKeysOp == null) {
+            return null;
+        }
         return (SortKeyDefinitionList) sortKeysOp.getChildExpression();
     }
 
@@ -790,8 +760,15 @@ public class ForEachGroup extends Instruction
     public void export(ExpressionPresenter out) throws XPathException {
         out.startElement("forEachGroup", this);
         out.emitAttribute("algorithm", getAlgorithmName(algorithm));
+        String flags = "";
         if (composite) {
-            out.emitAttribute("flags", "c");
+            flags = "c";
+        }
+        if (isInFork()) {
+            flags += "k";
+        }
+        if (!flags.isEmpty()) {
+            out.emitAttribute("flags", flags);
         }
         out.setChildRole("select");
         getSelectExpression().export(out);
