@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -12,8 +12,7 @@ import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.expr.parser.ExplicitLocation;
 import net.sf.saxon.expr.parser.RoleDiagnostic;
 import net.sf.saxon.expr.parser.TypeChecker;
-import net.sf.saxon.om.Sequence;
-import net.sf.saxon.om.StructuredQName;
+import net.sf.saxon.om.*;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.value.SequenceType;
 
@@ -29,8 +28,8 @@ import java.util.HashMap;
  */
 
 public class GlobalParameterSet {
-    private HashMap<StructuredQName, Sequence> params =
-            new HashMap<StructuredQName, Sequence>(10);
+    private HashMap<StructuredQName, GroundedValue> params =
+            new HashMap<>(10);
 
     /**
      * Create an empty parameter set
@@ -45,7 +44,7 @@ public class GlobalParameterSet {
      */
 
     public GlobalParameterSet(GlobalParameterSet parameterSet) {
-        this.params = new HashMap<StructuredQName, Sequence>(parameterSet.params);
+        this.params = new HashMap<>(parameterSet.params);
     }
 
     /**
@@ -55,7 +54,7 @@ public class GlobalParameterSet {
      * @param value The value of the parameter, or null if the parameter is to be removed
      */
 
-    public void put(StructuredQName qName, /*@Nullable*/ Sequence value) {
+    public void put(StructuredQName qName, /*@Nullable*/ GroundedValue value) {
         if (value == null) {
             params.remove(qName);
         } else {
@@ -70,7 +69,7 @@ public class GlobalParameterSet {
      * @return The value of the parameter, or null if not defined
      */
 
-    public Sequence get(StructuredQName qName) {
+    public GroundedValue get(StructuredQName qName) {
         return params.get(qName);
     }
 
@@ -91,7 +90,7 @@ public class GlobalParameterSet {
      * @throws XPathException if the value is of the wrong type
      */
 
-    public Sequence convertParameterValue(
+    public GroundedValue convertParameterValue(
             StructuredQName qName, SequenceType requiredType, boolean convert, XPathContext context)
             throws XPathException {
         Sequence val = get(qName);
@@ -112,7 +111,7 @@ public class GlobalParameterSet {
                 }
             }
         }
-        return val;
+        return val.materialize();
     }
 
 
