@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -25,11 +25,7 @@ import net.sf.saxon.tree.iter.ManualIterator;
 import net.sf.saxon.type.AnyItemType;
 import net.sf.saxon.type.ItemType;
 import net.sf.saxon.type.TypeHierarchy;
-import net.sf.saxon.value.IntegerValue;
-import net.sf.saxon.value.SequenceExtent;
-import net.sf.saxon.value.SequenceType;
-import net.sf.saxon.value.SingletonClosure;
-import net.sf.saxon.value.Cardinality;
+import net.sf.saxon.value.*;
 
 import java.util.*;
 
@@ -40,7 +36,7 @@ import java.util.*;
 public class GlobalVariable extends Actor
     implements Binding, net.sf.saxon.query.Declaration, InstructionInfo, ContextOriginator {
 
-    protected List<BindingReference> references = new ArrayList<BindingReference>(10);
+    protected List<BindingReference> references = new ArrayList<>(10);
     // Note that variableReferences on this list might be dormant;
     // that is, they might be disconnected from the live expression tree.
     // References are maintained in XQuery but not in XSLT (where they are handled at the level of the
@@ -437,7 +433,7 @@ public class GlobalVariable extends Actor
             if (select != null) {
                 properties = select.getSpecialProperties();
             }
-            properties |= StaticProperty.NON_CREATIVE;
+            properties |= StaticProperty.NO_NODES_NEWLY_CREATED;
             // a variable reference is non-creative even if its initializer is creative
             ref.setStaticType(type, constantValue, properties);
         }
@@ -520,26 +516,29 @@ public class GlobalVariable extends Actor
         if (referees.contains(this)) {
             int s = referees.indexOf(this);
             referees.push(this);
-            String message = "Circular definition of global variable: $" + getVariableQName().getDisplayName();
 
+            StringBuilder messageBuilder = new StringBuilder("Circular definition of global variable: $" + getVariableQName().getDisplayName());
             for (int i = s; i < referees.size() - 1; i++) {
                 if (i != s) {
-                    message += ", which";
+                    messageBuilder.append(", which");
                 }
                 if (referees.get(i + 1) instanceof GlobalVariable) {
                     GlobalVariable next = (GlobalVariable) referees.get(i + 1);
-                    message += " uses $" + next.getVariableQName().getDisplayName();
+                    messageBuilder.append(" uses $").append(next.getVariableQName().getDisplayName());
                 } else if (referees.get(i + 1) instanceof XQueryFunction) {
                     XQueryFunction next = (XQueryFunction) referees.get(i + 1);
-                    message += " calls " + next.getFunctionName().getDisplayName() +
-                            "#" + next.getNumberOfArguments() + "()";
+                    messageBuilder.append(" calls ").append(next.getFunctionName().getDisplayName()).append("#").append(next.getNumberOfArguments()).append("()");
                 }
             }
+            String message = messageBuilder.toString();
             message += '.';
             XPathException err = new XPathException(message);
             String errorCode;
             if (getPackageData().getHostLanguage() == Configuration.XSLT) {
                 errorCode = "XTDE0640";
+            } else if (s == 0 && referees.size() == 2) {
+                // Simple self-reference, treated specially in XQuery
+                errorCode = "XPST0008";
             } else {
                 errorCode = "XQDY0054";
             }
@@ -550,14 +549,14 @@ public class GlobalVariable extends Actor
         }
         if (select != null) {
             referees.push(this);
-            List<Binding> list = new ArrayList<Binding>(10);
+            List<Binding> list = new ArrayList<>(10);
             ExpressionTool.gatherReferencedVariables(select, list);
             for (Binding b : list) {
                 if (b instanceof GlobalVariable) {
                     ((GlobalVariable) b).lookForCycles(referees, globalFunctionLibrary);
                 }
             }
-            List<SymbolicName> flist = new ArrayList<SymbolicName>();
+            List<SymbolicName> flist = new ArrayList<>();
             ExpressionTool.gatherCalledFunctionNames(select, flist);
             for (SymbolicName s : flist) {
                 XQueryFunction f = globalFunctionLibrary.getDeclarationByKey(s);
@@ -583,7 +582,7 @@ public class GlobalVariable extends Actor
             XQueryFunction f, Stack<Object> referees, XQueryFunctionLibrary globalFunctionLibrary) throws XPathException {
         Expression body = f.getBody();
         referees.push(f);
-        List<Binding> list = new ArrayList<Binding>(10);
+        List<Binding> list = new ArrayList<>(10);
         ExpressionTool.gatherReferencedVariables(body, list);
         for (Binding b : list) {
             if (b instanceof GlobalVariable) {
@@ -612,7 +611,7 @@ public class GlobalVariable extends Actor
      * @throws XPathException if a dynamic error occurs during evaluation
      */
 
-    public Sequence getSelectValue(XPathContext context, Component target) throws XPathException {
+    public GroundedValue getSelectValue(XPathContext context, Component target) throws XPathException {
         if (select == null) {
             throw new AssertionError("*** No select expression for global variable $" +
                     getVariableQName().getDisplayName() + "!!");
@@ -636,11 +635,11 @@ public class GlobalVariable extends Actor
                 int savedOutputState = c2.getTemporaryOutputState();
                 c2.setTemporaryOutputState(StandardNames.XSL_VARIABLE);
                 c2.setCurrentOutputUri(null);
-                Sequence result;
+                GroundedValue result;
                 if (indexed) {
                     result = c2.getConfiguration().makeSequenceExtent(select, FilterExpression.FILTERED, c2);
                 } else {
-                    result = SequenceExtent.makeSequenceExtent(select.iterate(c2));
+                    result = select.iterate(c2).materialize();
                 }
                 c2.setTemporaryOutputState(savedOutputState);
                 return result;
@@ -657,12 +656,12 @@ public class GlobalVariable extends Actor
      * Evaluate the variable
      */
 
-    public Sequence evaluateVariable(XPathContext context) throws XPathException {
+    public GroundedValue<? extends Item> evaluateVariable(XPathContext context) throws XPathException {
         final Controller controller = context.getController();
         assert controller != null;
         final Bindery b = controller.getBindery(getPackageData());
 
-        final Sequence v = b.getGlobalVariable(getBinderySlotNumber());
+        final GroundedValue<? extends Item> v = b.getGlobalVariable(getBinderySlotNumber());
 
         if (v != null) {
             return v;
@@ -675,7 +674,7 @@ public class GlobalVariable extends Actor
      * Evaluate the variable
      */
 
-    public Sequence evaluateVariable(XPathContext context, Component target) throws XPathException {
+    public GroundedValue evaluateVariable(XPathContext context, Component target) throws XPathException {
         final Controller controller = context.getController();
         assert controller != null;
         Bindery b = controller.getBindery(getPackageData());
@@ -684,9 +683,12 @@ public class GlobalVariable extends Actor
             throw new AssertionError();
             //b = controller.getTopLevelBindery();
         }
-        final Sequence v = b.getGlobalVariable(getBinderySlotNumber());
+        final GroundedValue v = b.getGlobalVariable(getBinderySlotNumber());
 
         if (v != null) {
+            if (v instanceof Bindery.FailureValue) {
+                throw ((Bindery.FailureValue)v).getObject();
+            }
             return v;
         } else {
             return actuallyEvaluate(context, target);
@@ -702,7 +704,7 @@ public class GlobalVariable extends Actor
      * @throws XPathException if evaluation fails
      */
 
-    protected Sequence actuallyEvaluate(XPathContext context, Component target) throws XPathException {
+    protected GroundedValue<? extends Item> actuallyEvaluate(XPathContext context, Component target) throws XPathException {
         final Controller controller = context.getController();
         assert controller != null;
         final Bindery b = controller.getBindery(getPackageData());
@@ -721,7 +723,7 @@ public class GlobalVariable extends Actor
                 return b.getGlobalVariable(getBinderySlotNumber());
             }
 
-            Sequence value = getSelectValue(context, target);
+            GroundedValue value = getSelectValue(context, target);
             if (indexed) {
                 value = controller.getConfiguration().obtainOptimizer().makeIndexedValue(value.iterate());
             }
@@ -738,9 +740,9 @@ public class GlobalVariable extends Actor
                 }
                 err.setErrorCode(errorCode);
                 err.setXPathContext(context);
+                err.setIsGlobalError(true);
                 // Detect it more quickly the next time (in a pattern, the error is recoverable)
-                SingletonClosure closure = new SingletonClosure(new ErrorExpression(err), context);
-                b.setGlobalVariable(this, closure);
+                b.setGlobalVariable(this, new Bindery.FailureValue(err));
                 err.setLocation(getLocation());
                 throw err;
             } else {
@@ -828,7 +830,7 @@ public class GlobalVariable extends Actor
     /**
      * Register a variable reference that refers to the variable bound in this expression
      *
-     * @param ref
+     * @param ref the variable reference to be registered
      * @param isLoopingReference - true if the reference occurs within a loop, such as the predicate
      *                           of a filter expression
      */
@@ -854,7 +856,7 @@ public class GlobalVariable extends Actor
     @Override
     public void export(ExpressionPresenter presenter) throws XPathException {
         presenter.startElement(this instanceof GlobalParam ? "globalParam" : "globalVariable");
-        presenter.emitAttribute("name", getVariableQName().getEQName());
+        presenter.emitAttribute("name", getVariableQName());
         presenter.emitAttribute("type", getRequiredType().toExportString());
         presenter.emitAttribute("line", getLineNumber() + "");
         presenter.emitAttribute("module", getSystemId());
@@ -872,8 +874,8 @@ public class GlobalVariable extends Actor
             presenter.emitAttribute("flags", flags);
         }
 
-        if ("JS".equals(presenter.getOption("target"))) {
-            int targetVersion = presenter.getIntOption("targetVersion", 1);
+        if ("JS".equals(((ExpressionPresenter.ExportOptions) presenter.getOptions()).target)) {
+            int targetVersion = ((ExpressionPresenter.ExportOptions) presenter.getOptions()).targetVersion;
             presenter.emitAttribute("jsAcceptor", getRequiredType().getPrimaryType().generateJavaScriptItemTypeAcceptor("XTTE0590", targetVersion));
             presenter.emitAttribute("jsCardCheck", Cardinality.generateJavaScriptChecker(getRequiredType().getCardinality()));
         }
