@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,10 +8,7 @@
 package net.sf.saxon.expr.instruct;
 
 import net.sf.saxon.Configuration;
-import net.sf.saxon.expr.Expression;
-import net.sf.saxon.expr.Operand;
-import net.sf.saxon.expr.StaticProperty;
-import net.sf.saxon.expr.XPathContext;
+import net.sf.saxon.expr.*;
 import net.sf.saxon.expr.parser.ExpressionTool;
 import net.sf.saxon.expr.parser.Location;
 import net.sf.saxon.om.Item;
@@ -42,7 +39,7 @@ public abstract class Instruction extends Expression implements TailCallReturner
 
     public Instruction() {
     }
-
+    
     /**
      * An implementation of Expression must provide at least one of the methods evaluateItem(), iterate(), or process().
      * This method indicates which of these methods is prefered. For instructions this is the process() method.
@@ -68,19 +65,6 @@ public abstract class Instruction extends Expression implements TailCallReturner
         return -1;
     }
 
-    /**
-     * Get a name identifying the kind of instruction, in terms meaningful to a user. This method
-     * is not used in the case where the instruction name code is a standard name (<1024).
-     *
-     * @return a name identifying the kind of instruction, in terms meaningful to a user.
-     *         The name will always be in the form of a lexical XML QName, and should match the name used
-     *         in explain() output displaying the instruction.
-     */
-
-    public String getInstructionName() {
-        return getClass().getName();
-    }
-
     /**
      * Get a name identifying the kind of expression, in terms meaningful to a user.
      *
@@ -91,13 +75,10 @@ public abstract class Instruction extends Expression implements TailCallReturner
 
     public String getExpressionName() {
         int code = getInstructionNameCode();
-        if (code < 0) {
-            return getClass().getName();
-        }
-        if (code < 1024) {
+        if (code >= 0 & code < 1024) {
             return StandardNames.getDisplayName(code);
         } else {
-            return getInstructionName();
+            return super.getExpressionName();
         }
     }
 
@@ -153,7 +134,7 @@ public abstract class Instruction extends Expression implements TailCallReturner
                 tc = tc.processLeavingTail();
             }
         } catch (XPathException err) {
-            err.maybeSetLocation(getLocation());
+            err.maybeSetFailingExpression(this);
             err.maybeSetContext(context);
             throw err;
         }
@@ -263,7 +244,7 @@ public abstract class Instruction extends Expression implements TailCallReturner
         if (mayCreateNewNodes()) {
             return p;
         } else {
-            return p | StaticProperty.NON_CREATIVE;
+            return p | StaticProperty.NO_NODES_NEWLY_CREATED;
         }
     }
 
@@ -314,7 +295,7 @@ public abstract class Instruction extends Expression implements TailCallReturner
         for (Operand o : operands()) {
             Expression child = o.getChildExpression();
             int props = child.getSpecialProperties();
-            if ((props & StaticProperty.NON_CREATIVE) == 0) {
+            if ((props & StaticProperty.NO_NODES_NEWLY_CREATED) == 0) {
                 return true;
             }
         }
@@ -364,7 +345,7 @@ public abstract class Instruction extends Expression implements TailCallReturner
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<? extends Item> iterate(XPathContext context) throws XPathException {
         int m = getImplementationMethod();
         if ((m & EVALUATE_METHOD) != 0) {
             Item item = evaluateItem(context);
