@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -204,7 +204,7 @@ public final class IterateInstr extends Instruction implements ContextSwitchingE
      */
 
     public int allocateParameterSlots(int nextFree) {
-        HashMap<StructuredQName, Integer> slotMap = new HashMap<StructuredQName, Integer>();
+        HashMap<StructuredQName, Integer> slotMap = new HashMap<>();
         for (Operand o : getInitiallyExp().operands()) {
             LocalParam b = (LocalParam)o.getChildExpression();
             if (b.getLocalSlotNumber() >= 0) {
@@ -257,13 +257,12 @@ public final class IterateInstr extends Instruction implements ContextSwitchingE
     public final boolean mayCreateNewNodes() {
         return (getActionExpression().getSpecialProperties() &
                 getOnCompletion().getSpecialProperties() &
-                StaticProperty.NON_CREATIVE) == 0;
+                StaticProperty.NO_NODES_NEWLY_CREATED) == 0;
     }
 
     /**
      * Add a representation of this expression to a PathMap. The PathMap captures a map of the nodes visited
      * by an expression in a source tree.
-     * <p/>
      * <p>The default implementation of this method assumes that an expression does no navigation other than
      * the navigation done by evaluating its subexpressions, and that the subexpressions are evaluated in the
      * same context as the containing expression. The method must be overridden for any expression
@@ -287,28 +286,6 @@ public final class IterateInstr extends Instruction implements ContextSwitchingE
         return getActionExpression().addToPathMap(pathMap, target);
     }
 
-    /**
-     * Compute the dependencies of an expression, as the union of the
-     * dependencies of its subexpressions. (This is overridden for path expressions
-     * and filter expressions, where the dependencies of a subexpression are not all
-     * propogated). This method should be called only once, to compute the dependencies;
-     * after that, getDependencies should be used.
-     *
-     * @return the depencies, as a bit-mask
-     */
-
-//    public int computeDependencies() {
-//        // Some of the dependencies aren't relevant. Note that the sort keys are absorbed into the select
-//        // expression.
-//        int dependencies = 0;
-//        dependencies |= getSelect().getDependencies();
-//        dependencies |= getInitiallyExp().getDependencies();
-//        dependencies |= getAction().getDependencies() & ~StaticProperty.DEPENDS_ON_FOCUS;
-//        dependencies |= getOnCompletion().getDependencies() & ~StaticProperty.DEPENDS_ON_FOCUS;
-//        return dependencies;
-//    }
-
-
     /**
      * Ask whether this expression is, or contains, the binding of a given variable
      *
@@ -367,7 +344,9 @@ public final class IterateInstr extends Instruction implements ContextSwitchingE
      * Copy an expression. This makes a deep copy.
      *
      * @return the copy of the original expression
-     * @param rebindings
+     * @param rebindings a mutable list of (old binding, new binding) pairs
+     *                   that is used to update the bindings held in any
+     *                   local variable references that are copied.
      */
 
     /*@NotNull*/
@@ -383,11 +362,9 @@ public final class IterateInstr extends Instruction implements ContextSwitchingE
 
 
     public TailCall processLeavingTail(XPathContext context) throws XPathException {
-        FocusIterator iter = new FocusTrackingIterator(getSelectExpression().iterate(context));
-
         XPathContextMajor c2 = context.newContext();
         c2.setOrigin(this);
-        c2.setCurrentIterator(iter);
+        FocusIterator iter = c2.trackFocus(getSelectExpression().iterate(context));
         c2.setCurrentTemplateRule(null);
 
         boolean tracing = context.getController().isTracing();
