@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -19,19 +19,13 @@ import net.sf.saxon.type.ItemType;
 import net.sf.saxon.value.IntegerValue;
 import net.sf.saxon.value.SequenceType;
 
-import java.util.ArrayList;
-import java.util.List;
-
 /**
  * The compiled form of an xsl:param element within a template in an XSLT stylesheet.
- * <p/>
  * <p>The xsl:param element in XSLT has mandatory attribute name and optional attribute select. It can also
  * be specified as required="yes" or required="no".</p>
- * <p/>
  * <p>This is used only for parameters to XSLT templates. For function calls, the caller of the function
  * places supplied arguments onto the callee's stackframe and the callee does not need to do anything.
  * Global parameters (XQuery external variables) are handled using {@link GlobalParam}.</p>
- * <p/>
  * <p>The LocalParam class is also used to represent parameters with the saxon:iterate instruction</p>
  *
  * <p>Changed in Saxon 9.8 to combine the previously-separate LocalParamSetter and LocalParam classes into one</p>
@@ -40,7 +34,7 @@ import java.util.List;
 public final class LocalParam extends Instruction implements LocalBinding {
 
     private Operand conversionOp = null;
-    private int conversionEvaluationMode = ExpressionTool.UNDECIDED;
+    private Evaluator conversionEvaluator = null;
 
     private static final int REQUIRED = 4;
     private static final int TUNNEL = 8;
@@ -53,7 +47,7 @@ public final class LocalParam extends Instruction implements LocalBinding {
     private SequenceType requiredType;
     protected int slotNumber = -999;
     protected int referenceCount = 10;
-    protected int evaluationMode = ExpressionTool.UNDECIDED;
+    protected Evaluator evaluator = null;
 
 
 
@@ -73,7 +67,7 @@ public final class LocalParam extends Instruction implements LocalBinding {
         } else {
             selectOp = null;
         }
-        evaluationMode = ExpressionTool.UNDECIDED;
+        evaluator = null;
     }
 
     /**
@@ -159,22 +153,22 @@ public final class LocalParam extends Instruction implements LocalBinding {
         referenceCount = refCount;
     }
 
-    /**
-     * Get the evaluation mode of the variable
-     *
-     * @return the evaluation mode (a constant in {@link ExpressionTool}
-     */
-
-    public int getEvaluationMode() {
-        if (evaluationMode == ExpressionTool.UNDECIDED) {
-            if (referenceCount == FilterExpression.FILTERED) {
-                evaluationMode = ExpressionTool.MAKE_INDEXED_VARIABLE;
-            } else {
-                evaluationMode = ExpressionTool.lazyEvaluationMode(getSelectExpression());
-            }
-        }
-        return evaluationMode;
-    }
+//    /**
+//     * Get the evaluation mode of the variable
+//     *
+//     * @return the evaluation mode (a constant in {@link ExpressionTool}
+//     */
+//
+//    public int getEvaluationMode() {
+//        if (evaluationMode == ExpressionTool.UNDECIDED) {
+//            if (referenceCount == FilterExpression.FILTERED) {
+//                evaluationMode = ExpressionTool.MAKE_INDEXED_VARIABLE;
+//            } else {
+//                evaluationMode = ExpressionTool.lazyEvaluationMode(getSelectExpression());
+//            }
+//        }
+//        return evaluationMode;
+//    }
 
     /**
      * Get the cardinality of the result of this instruction. An xsl:variable instruction returns nothing, so the
@@ -250,17 +244,19 @@ public final class LocalParam extends Instruction implements LocalBinding {
         if (e2 != this) {
             return e2;
         }
-        if (selectOp != null) {
-            computeEvaluationMode();
-        }
+//        if (selectOp != null) {
+//            computeEvaluationMode();
+//        }
         return this;
     }
 
     public void computeEvaluationMode() {
-        if (referenceCount == FilterExpression.FILTERED) {
-            evaluationMode = ExpressionTool.MAKE_INDEXED_VARIABLE;
-        } else {
-            evaluationMode = ExpressionTool.lazyEvaluationMode(getSelectExpression());
+        if (getSelectExpression() != null) {
+            if (referenceCount == FilterExpression.FILTERED) {
+                evaluator = Evaluator.MAKE_INDEXED_VARIABLE;
+            } else {
+                evaluator = ExpressionTool.lazyEvaluator(getSelectExpression(), referenceCount > 1);
+            }
         }
     }
 
@@ -276,18 +272,20 @@ public final class LocalParam extends Instruction implements LocalBinding {
     public LocalParam copy(RebindingMap rebindings) {
         LocalParam p2 = new LocalParam();
         if (conversionOp != null) {
+            assert getConversion() != null;
             p2.setConversion(getConversion().copy(rebindings));
         }
-        p2.conversionEvaluationMode = conversionEvaluationMode;
+        p2.conversionEvaluator = conversionEvaluator;
         p2.properties = properties;
         if (selectOp != null) {
+            assert getSelectExpression() != null;
             p2.setSelectExpression(getSelectExpression().copy(rebindings));
         }
         p2.variableQName = variableQName;
         p2.requiredType = requiredType;
         p2.slotNumber = slotNumber;
         p2.referenceCount = referenceCount;
-        p2.evaluationMode = evaluationMode;
+        p2.evaluator = evaluator;
         return p2;
     }
 
@@ -340,7 +338,7 @@ public final class LocalParam extends Instruction implements LocalBinding {
             // which will already contain any code to force conversion to the required type.
             int savedOutputState = context.getTemporaryOutputState();
             context.setTemporaryOutputState(StandardNames.XSL_WITH_PARAM);
-            Sequence result = ExpressionTool.evaluate(select, evaluationMode, context, referenceCount);
+            Sequence result = evaluator.evaluate(select, context);
             context.setTemporaryOutputState(savedOutputState);
             return result;
         }
@@ -401,7 +399,7 @@ public final class LocalParam extends Instruction implements LocalBinding {
             if (conversionOp == null) {
                 conversionOp = new Operand(this, convertor, OperandRole.SINGLE_ATOMIC);
             }
-            conversionEvaluationMode = ExpressionTool.eagerEvaluationMode(convertor);
+            conversionEvaluator = ExpressionTool.eagerEvaluator(convertor);
         } else {
             conversionOp = null;
         }
@@ -420,7 +418,7 @@ public final class LocalParam extends Instruction implements LocalBinding {
     }
 
     public int getConversionEvaluationMode() {
-        return conversionEvaluationMode;
+        return conversionEvaluator.getCode();
     }
 
     /**
@@ -442,17 +440,9 @@ public final class LocalParam extends Instruction implements LocalBinding {
 
     @Override
     public Iterable<Operand> operands() {
-        List<Operand> list = new ArrayList<Operand>();
-        if (selectOp != null) {
-            list.add(selectOp);
-        }
-        if (conversionOp != null) {
-            list.add(conversionOp);
-        }
-        return list;
+        return operandSparseList(selectOp, conversionOp);
     }
 
-
     /**
      * Process the local parameter declaration
      *
@@ -474,7 +464,7 @@ public final class LocalParam extends Instruction implements LocalBinding {
                 // then we may need to convert it to the type required
                 if (conversionOp != null) {
                     context.setLocalVariable(slotNumber,
-                            ExpressionTool.evaluate(getConversion(), conversionEvaluationMode, context, 10));
+                            conversionEvaluator.evaluate(getConversion(), context));
                     // We do an eager evaluation here for safety, because the result of the
                     // type conversion overwrites the slot where the actual supplied parameter
                     // is contained.
@@ -485,17 +475,22 @@ public final class LocalParam extends Instruction implements LocalBinding {
             // evaluated by virtue of a forwards reference
 
             case ParameterSet.NOT_SUPPLIED:
-                if (isImplicitlyRequiredParam()) {
+                if (isRequiredParam() || isImplicitlyRequiredParam()) {
                     String name = "$" + getVariableQName().getDisplayName();
-                    XPathException e = new XPathException("A value must be supplied for parameter "
-                            + name + " because " +
-                            "the default value is not a valid instance of the required type");
-                    e.setXPathContext(context);
-                    e.setErrorCode("XTDE0700");
-                    throw e;
-                } else if (isRequiredParam()) {
-                    String name = "$" + getVariableQName().getDisplayName();
-                    XPathException e = new XPathException("No value supplied for required parameter " + name);
+                    int suppliedAsTunnel = context.useLocalParameter(variableQName, slotNumber, !isTunnelParam());
+                    String message = "No value supplied for required parameter " + name;
+                    if (isImplicitlyRequiredParam()) {
+                        message += ". A value is required because " +
+                                "the default value is not a valid instance of the required type";
+                    }
+                    if (suppliedAsTunnel != ParameterSet.NOT_SUPPLIED) {
+                        if (isTunnelParam()) {
+                            message += ". A non-tunnel parameter with this name was supplied, but a tunnel parameter is required";
+                        } else {
+                            message += ". A tunnel parameter with this name was supplied, but a non-tunnel parameter is required";
+                        }
+                    }
+                    XPathException e = new XPathException(message);
                     e.setXPathContext(context);
                     e.setErrorCode("XTDE0700");
                     throw e;
@@ -530,22 +525,15 @@ public final class LocalParam extends Instruction implements LocalBinding {
      * @return result of the compatibility check
      */
     public boolean isCompatible(LocalParam other) {
-        if (!getVariableQName().equals(other.getVariableQName())) {
-            return false;
-        }
-        if (!getRequiredType().equals(other.getRequiredType())) {
-            return false;
-        }
-        if (isTunnelParam() != other.isTunnelParam()) {
-            return false;
-        }
-        return true;
+        return getVariableQName().equals(other.getVariableQName()) &&
+                getRequiredType().equals(other.getRequiredType()) &&
+                isTunnelParam() == other.isTunnelParam();
     }
 
     /**
      * Ask whether the expression can be lifted out of a loop, assuming it has no dependencies
      * on the controlling variable/focus of the loop
-     * @param forStreaming
+     * @param forStreaming true if compiling streamable code
      */
     @Override
     public boolean isLiftable(boolean forStreaming) {
@@ -621,6 +609,16 @@ public final class LocalParam extends Instruction implements LocalBinding {
         return "param";
     }
 
+    /**
+     * Produce a short string identifying the expression for use in error messages
+     *
+     * @return a short string, sufficient to identify the expression
+     */
+    @Override
+    public String toShortString() {
+        return "$" + getVariableQName().getDisplayName();
+    }
+
     /**
      * Diagnostic print of expression structure. The abstract expression tree
      * is written to the supplied output destination.
@@ -628,7 +626,7 @@ public final class LocalParam extends Instruction implements LocalBinding {
 
     public void export(ExpressionPresenter out) throws XPathException {
         out.startElement("param", this);
-        out.emitAttribute("name", getVariableQName().getEQName());
+        out.emitAttribute("name", getVariableQName());
         out.emitAttribute("slot", "" + getSlotNumber());
         String flags = getFlags();
         if (!flags.isEmpty()) {
