@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,22 +9,23 @@ package net.sf.saxon.expr.instruct;
 
 import net.sf.saxon.Configuration;
 import net.sf.saxon.Controller;
-import net.sf.saxon.event.ComplexContentOutputter;
-import net.sf.saxon.event.PipelineConfiguration;
-import net.sf.saxon.event.Receiver;
-import net.sf.saxon.event.SequenceReceiver;
+import net.sf.saxon.event.*;
 import net.sf.saxon.expr.*;
 import net.sf.saxon.expr.parser.*;
 import net.sf.saxon.functions.IriToUri;
+import net.sf.saxon.functions.ResolveURI;
 import net.sf.saxon.lib.*;
 import net.sf.saxon.om.*;
 import net.sf.saxon.serialize.CharacterMapIndex;
-import net.sf.saxon.serialize.ReconfigurableSerializer;
+import net.sf.saxon.serialize.PrincipalOutputGatekeeper;
+import net.sf.saxon.serialize.SerializationProperties;
 import net.sf.saxon.style.StylesheetPackage;
 import net.sf.saxon.style.XSLResultDocument;
 import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.Err;
 import net.sf.saxon.trans.XPathException;
+import net.sf.saxon.trans.XsltController;
+import net.sf.saxon.tree.iter.AxisIterator;
 import net.sf.saxon.type.ErrorType;
 import net.sf.saxon.type.ItemType;
 import net.sf.saxon.type.SchemaType;
@@ -38,7 +39,6 @@ import javax.xml.transform.sax.SAXResult;
 import javax.xml.transform.stream.StreamResult;
 import java.io.IOException;
 import java.io.StringWriter;
-import java.net.URI;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Map;
@@ -47,15 +47,13 @@ import java.util.Properties;
 
 /**
  * The compiled form of an xsl:result-document element in the stylesheet.
- * <p/>
- * The xsl:result-document element takes an attribute href="filename". The filename will
+ * <p>The xsl:result-document element takes an attribute href="filename". The filename will
  * often contain parameters, e.g. {position()} to ensure that a different file is produced
- * for each element instance.
- * <p/>
- * There is a further attribute "format" which determines the format of the
+ * for each element instance.</p>
+ * <p>There is a further attribute "format" which determines the format of the
  * output file, it identifies the name of an xsl:output element containing the output
  * format details. In addition, individual serialization properties may be specified as attributes.
- * These are attribute value templates, so they may need to be computed at run-time.
+ * These are attribute value templates, so they may need to be computed at run-time.</p>
  */
 
 public class ResultDocument extends Instruction
@@ -65,12 +63,12 @@ public class ResultDocument extends Instruction
     private Operand formatOp;    // null if format was known at compile time
     private Operand contentOp;
     private boolean async = false;
-    protected Properties globalProperties;
-    protected Properties localProperties;
+    protected final Properties globalProperties;
+    protected final Properties localProperties;
     protected ParseOptions validationOptions;
-    protected Map<StructuredQName, Operand> serializationAttributes;
+    protected final Map<StructuredQName, Operand> serializationAttributes;
     protected boolean resolveAgainstStaticBase = false;        // used with fn:put()
-    protected CharacterMapIndex characterMapIndex;
+    protected final CharacterMapIndex characterMapIndex;
 
 
     /**
@@ -102,7 +100,7 @@ public class ResultDocument extends Instruction
             formatOp = new Operand(this, formatExpression, OperandRole.SINGLE_ATOMIC);
         }
         setValidationAction(validationAction, schemaType);
-        this.serializationAttributes = new HashMap<StructuredQName, Operand>(serializationAttributes.size());
+        this.serializationAttributes = new HashMap<>(serializationAttributes.size());
         for (Map.Entry<StructuredQName, Expression> entry : serializationAttributes.entrySet()) {
             this.serializationAttributes.put(entry.getKey(), new Operand(this, entry.getValue(), OperandRole.SINGLE_ATOMIC));
         }
@@ -232,7 +230,7 @@ public class ResultDocument extends Instruction
     @Override
     public boolean isMultiThreaded(Configuration config) {
         return isAsynchronous() && config.isLicensedFeature(Configuration.LicenseFeature.SCHEMA_VALIDATION)
-                && config.getBooleanProperty(FeatureKeys.ALLOW_MULTITHREADING);
+                && config.getBooleanProperty(Feature.ALLOW_MULTITHREADING);
     }
 
     /*@NotNull*/
@@ -278,15 +276,15 @@ public class ResultDocument extends Instruction
     /**
      * Copy an expression. This makes a deep copy.
      *
+     * @param rebindings variable bindings that need to be changed
      * @return the copy of the original expression
-     * @param rebindings
      */
 
     /*@NotNull*/
     public Expression copy(RebindingMap rebindings) {
-        Map<StructuredQName, Expression> map = new HashMap<StructuredQName, Expression>();
+        Map<StructuredQName, Expression> map = new HashMap<>();
         for (Map.Entry<StructuredQName, Operand> entry : serializationAttributes.entrySet()) {
-            map.put(entry.getKey(), entry.getValue().getChildExpression());
+            map.put(entry.getKey(), entry.getValue().getChildExpression().copy(rebindings));
         }
         ResultDocument r = new ResultDocument(
                 globalProperties,
@@ -327,7 +325,7 @@ public class ResultDocument extends Instruction
 
     @Override
     public Iterable<Operand> operands() {
-        ArrayList<Operand> list = new ArrayList<Operand>(6);
+        ArrayList<Operand> list = new ArrayList<>(6);
         list.add(contentOp);
         if (hrefOp != null) {
             list.add(hrefOp);
@@ -335,16 +333,13 @@ public class ResultDocument extends Instruction
         if (formatOp != null) {
             list.add(formatOp);
         }
-        for (Operand e : serializationAttributes.values()) {
-            list.add(e);
-        }
+        list.addAll(serializationAttributes.values());
         return list;
     }
 
     /**
      * Add a representation of this expression to a PathMap. The PathMap captures a map of the nodes visited
      * by an expression in a source tree.
-     * <p/>
      * <p>The default implementation of this method assumes that an expression does no navigation other than
      * the navigation done by evaluating its subexpressions, and that the subexpressions are evaluated in the
      * same context as the containing expression. The method must be overridden for any expression
@@ -375,15 +370,7 @@ public class ResultDocument extends Instruction
     }
 
     public void process(Expression content, XPathContext context) throws XPathException {
-        if (context.getTemporaryOutputState() != 0) {
-            // Check this before starting a new thread (because in temporary output state the context is sometimes
-            // not sufficiently shared to allow the error to be passed back
-            XPathException err = new XPathException("Cannot execute xsl:result-document while evaluating xsl:" +
-                                                            context.getNamePool().getLocalName(context.getTemporaryOutputState()));
-            err.setErrorCode("XTDE1480");
-            err.setLocation(getLocation());
-            throw err;
-        }
+        checkNotTemporaryOutputState(context);
         context.getConfiguration().processResultDocument(this, content, context);
     }
 
@@ -397,91 +384,16 @@ public class ResultDocument extends Instruction
      */
 
     public void processInstruction(Expression content, XPathContext context) throws XPathException {
-        final Controller controller = context.getController();
+        final XsltController controller = (XsltController) context.getController();
         assert controller != null;
-        SequenceReceiver saved = context.getReceiver();
+        final Configuration config = controller.getConfiguration();
+        Receiver saved = context.getReceiver();
+        PipelineConfiguration pipe = saved.getPipelineConfiguration();
+
         String savedOutputUri = context.getCurrentOutputUri();
-        if (context.getTemporaryOutputState() != 0) {
-            XPathException err = new XPathException("Cannot execute xsl:result-document while evaluating xsl:" +
-                                                            context.getNamePool().getLocalName(context.getTemporaryOutputState()));
-            err.setErrorCode("XTDE1480");
-            err.setLocation(getLocation());
-            throw err;
-        }
-
-        Properties computedLocalProps = gatherOutputProperties(context);
-        if (getStaticBaseURIString() != null) {
-            computedLocalProps.setProperty(SaxonOutputKeys.PARAMETER_DOCUMENT_BASE_URI, getStaticBaseURIString());
-        }
-        String nextInChain = computedLocalProps.getProperty(SaxonOutputKeys.NEXT_IN_CHAIN);
-        //TODO: reinstate this code
-//        if (nextInChain != null && nextInChain.length() > 0) {
-//            try {
-//                result = sf.prepareNextStylesheet(controller, nextInChain, baseURI, result);
-//            } catch (TransformerException e) {
-//                throw XPathException.makeXPathException(e);
-//            }
-//        }
-
-        Receiver out;
-        OutputURIResolver resolver = null;
-        CloseableResult closeableResult = null;
-        Result result = null;
-        boolean buildTree = SaxonOutputKeys.isBuildTree(computedLocalProps);
-        if (getHref() == null) {
-            if (!computedLocalProps.propertyNames().hasMoreElements()) {
-                out = controller.getPrincipalResult();
-            } else if (controller.getPrincipalResult() instanceof ReconfigurableSerializer) {
-                ReconfigurableSerializer rs = (ReconfigurableSerializer) controller.getPrincipalResult();
-                rs.reconfigure(computedLocalProps, characterMapIndex);
-                out = rs;
-            } else {
-                // assume we're not serializing
-                out = controller.getPrincipalResult();
-                //throw new IllegalStateException();
-            }
-            String resultURI = controller.getBaseOutputURI();
-            if (resultURI == null) {
-                resultURI = Controller.ANONYMOUS_PRINCIPAL_OUTPUT_URI;
-            }
-            checkAcceptableUri(context, resultURI);
-            if (buildTree) {
-                out = ComplexContentOutputter.makeComplexContentReceiver(out, getValidationOptions());
-            }
-            context.setReceiver((SequenceReceiver) out);
-            context.setCurrentOutputUri(resultURI);
-        } else {
-            resolver = controller.getOutputURIResolver().newInstance();
-
-            try {
-                closeableResult = getResult(getHref(), getStaticBaseURIString(), context, resolver, resolveAgainstStaticBase);
-                result = closeableResult.result;
-            } catch (XPathException e) {
-                e.maybeSetLocation(getLocation());
-                throw e;
-            }
-            SerializerFactory sf = context.getConfiguration().getSerializerFactory();
-
-            PipelineConfiguration pipe = controller.makePipelineConfiguration();
-            pipe.setLocationIsCodeLocation(true);
-            pipe.setHostLanguage(Configuration.XSLT);
-            out = sf.getReceiver(result, pipe, computedLocalProps, characterMapIndex);
-            if (buildTree) {
-                out = ComplexContentOutputter.makeComplexContentReceiver(out, getValidationOptions());
-            }
-            context.setReceiver((SequenceReceiver) out);
-            context.setCurrentOutputUri(result.getSystemId());
-        }
-
-        out.open();
+        Receiver out = processLeft(context);
         try {
-            if (buildTree) {
-                out.startDocument(0);
-                content.process(context);
-                out.endDocument();
-            } else {
-                content.process(context);
-            }
+            content.process(context);
         } catch (XPathException err) {
             err.setXPathContext(context);
             err.maybeSetLocation(getLocation());
@@ -491,93 +403,117 @@ public class ResultDocument extends Instruction
         }
         context.setReceiver(saved);
         context.setCurrentOutputUri(savedOutputUri);
-        if (closeableResult != null) {
-            try {
-                closeableResult.close();
-            } catch (TransformerException e) {
-                throw XPathException.makeXPathException(e);
+
+    }
+
+    public Receiver processLeft(XPathContext context) throws XPathException {
+        XsltController controller = (XsltController)context.getController();
+        Configuration config = controller.getConfiguration();
+        checkNotTemporaryOutputState(context);
+
+        Properties computedLocalProps = gatherOutputProperties(context);
+        if (getStaticBaseURIString() != null) {
+            computedLocalProps.setProperty(SaxonOutputKeys.PARAMETER_DOCUMENT_BASE_URI, getStaticBaseURIString());
+        }
+        SerializationProperties serParams = new SerializationProperties(computedLocalProps, characterMapIndex);
+
+        Receiver out = null;
+        ResultDocumentResolver resolver = null;
+        String hrefValue = "";
+        if (getHref() != null) {
+            hrefValue = IriToUri.iriToUri(getHref().evaluateAsString(context)).toString();
+        }
+        if (hrefValue.isEmpty() || hrefValue.equals(controller.getBaseOutputURI())) {
+            PrincipalOutputGatekeeper rs = controller.getGatekeeper();
+            if (rs != null) {
+                rs.useAsSecondary();
             }
         }
+        resolver = controller.getResultDocumentResolver();
+
+        // If validation was requested, create a function that instantiates a validator
+        // which can then be injected at an appropriate point into the output pipeline
+
+        if (validationOptions != null && validationOptions.getSchemaValidationMode() != Validation.PRESERVE) {
+            serParams.setValidationFactory(
+                    output -> {
+                        // Validation can add redundant namespace declarations so we
+                        // need to follow it with a namespace reducer
+                        NamespaceReducer nr = new NamespaceReducer(output);
+                        return config.getDocumentValidator(
+                            nr, output.getSystemId(), validationOptions, getLocation());
+                    }
+            );
+        }
+
+        try {
+            out = makeReceiver(hrefValue, getStaticBaseURIString(), context,
+                               resolver, serParams, resolveAgainstStaticBase);
+        } catch (XPathException e) {
+            e.maybeSetLocation(getLocation());
+            e.maybeSetContext(context);
+            throw e;
+        }
+        out.getPipelineConfiguration().setController(controller);
+        String systemId = out.getSystemId();
+
+        NamespaceReducer nr = new NamespaceReducer(out);
+        out = new ComplexContentOutputter(nr);
+
+        context.setReceiver(out);
+        context.setCurrentOutputUri(systemId);
+
+        out.open();
+        return out;
+    }
+
+    public CharacterMapIndex getCharacterMapIndex() {
+        return characterMapIndex;
+    }
+
+    private void checkNotTemporaryOutputState(XPathContext context) throws XPathException {
+        if (context.getTemporaryOutputState() != 0) {
+            XPathException err = new XPathException("Cannot execute xsl:result-document while evaluating xsl:" +
+                                                            context.getNamePool().getLocalName(context.getTemporaryOutputState()));
+            err.setErrorCode("XTDE1480");
+            err.setLocation(getLocation());
+            throw err;
+        }
     }
 
-    public static class CloseableResult {
-        public Result result;
-        public void close() throws TransformerException {};
-    }
-
-    public static CloseableResult getResult(Expression href, String baseURI,
-                                   XPathContext context, OutputURIResolver resolver,
-                                   boolean resolveAgainstStaticBase) throws XPathException {
-        String resultURI;
-        Result result;
-        CloseableResult closeable;
-        boolean resultFromResolver = false;
+    public static Receiver makeReceiver(String hrefValue, String baseURI,
+                                        XPathContext context, ResultDocumentResolver resolver,
+                                        SerializationProperties params,
+                                        boolean resolveAgainstStaticBase) throws XPathException {
+        String resultURI = null;
         Controller controller = context.getController();
-        if (href == null) {
-            result = controller.getPrincipalResult();
-            resultURI = controller.getBaseOutputURI();
-            if (resultURI == null) {
-                resultURI = Controller.ANONYMOUS_PRINCIPAL_OUTPUT_URI;
+        try {
+            String base;
+            if (resolveAgainstStaticBase) {
+                base = baseURI;
+            } else {
+                base = controller.getBaseOutputURI();
             }
-            closeable = new CloseableResult();
-            closeable.result = result;
-        } else {
-            try {
-                String base;
-                if (resolveAgainstStaticBase) {
-                    base = baseURI;
-                } else {
-                    base = controller.getCookedBaseOutputURI();
-                }
 
-                String hrefValue = IriToUri.iriToUri(href.evaluateAsString(context)).toString();
-                if (hrefValue.equals("")) {
-                    result = controller.getPrincipalResult();
-                    resultURI = controller.getBaseOutputURI();
-                    if (resultURI == null) {
-                        resultURI = Controller.ANONYMOUS_PRINCIPAL_OUTPUT_URI;
-                    }
-                    closeable = new CloseableResult();
-                    closeable.result = result;
-                } else {
-                    try {
-                        result = resolver == null ? null : resolver.resolve(hrefValue, base);
-                        //System.err.println("Resolver returned " + result);
-                    } catch (TransformerException err) {
-                        throw XPathException.makeXPathException(err);
-                    } catch (Exception err) {
-                        err.printStackTrace();
-                        throw new XPathException("Exception thrown by OutputURIResolver", err);
-                    }
-                    if (result == null) {
-                        resolver = StandardOutputResolver.getInstance();
-                        result = resolver.resolve(hrefValue, base);
-                    }
-                    final OutputURIResolver finalResolver = resolver;
-                    closeable = new CloseableResult() {
-                        @Override
-                        public void close() throws TransformerException {
-                            finalResolver.close(result);
-                        }
-                    };
-                    closeable.result = result;
-                    resultURI = result.getSystemId();
-                    if (resultURI == null) {
-                        try {
-                            resultURI = new URI(base).resolve(hrefValue).toString();
-                            result.setSystemId(resultURI);
-                        } catch (Exception err) {
-                            // no action
-                        }
-                    }
+            try {
+                Receiver out = resolver.resolve(context, hrefValue, base, params);
+                String systemId = out.getSystemId();
+                if (systemId == null) {
+                    systemId = ResolveURI.makeAbsolute(hrefValue, base).toASCIIString();
+                    out.setSystemId(systemId);
                 }
-            } catch (TransformerException e) {
-                throw XPathException.makeXPathException(e);
+                checkAcceptableUri(context, systemId);
+                return out;
+            } catch (XPathException e) {
+                throw e;
+            } catch (Exception err) {
+                err.printStackTrace();
+                throw new XPathException("Exception thrown by output resolver", err);
             }
+        } catch (TransformerException e) {
+            throw XPathException.makeXPathException(e);
         }
-        checkAcceptableUri(context, resultURI);
-        traceDestination(context, result);
-        return closeable;
+
     }
 
     public static void traceDestination(XPathContext context, Result result) {
@@ -601,9 +537,8 @@ public class ResultDocument extends Instruction
     }
 
     public static void checkAcceptableUri(XPathContext context, String uri) throws XPathException {
-        Controller controller = context.getController();
+        XsltController controller = (XsltController) context.getController();
         assert controller != null;
-        //String uri = result.getSystemId();
         if (uri != null) {
             if (controller.getDocumentPool().find(uri) != null) {
                 XPathException err = new XPathException("Cannot write to a URI that has already been read: " +
@@ -627,7 +562,6 @@ public class ResultDocument extends Instruction
                 }
             }
         }
-        controller.setThereHasBeenAnExplicitResultDocument();
     }
 
     /**
@@ -645,6 +579,7 @@ public class ResultDocument extends Instruction
         Configuration config = context.getConfiguration();
         Properties computedGlobalProps = globalProperties;
         NamespaceResolver nsResolver = getRetainedStaticContext();
+        assert nsResolver != null;
 
         if (getFormatExpression() != null) {
             // format was an AVT and now needs to be computed
@@ -668,7 +603,7 @@ public class ResultDocument extends Instruction
                 throw err;
             }
             StructuredQName qName = new StructuredQName(parts[0], uri, parts[1]);
-            computedGlobalProps = ((StylesheetPackage)getRetainedStaticContext().getPackageData()).getNamedOutputProperties(qName);
+            computedGlobalProps = ((StylesheetPackage) getRetainedStaticContext().getPackageData()).getNamedOutputProperties(qName);
             if (computedGlobalProps == null) {
                 XPathException err = new XPathException("There is no xsl:output format named " + format);
                 err.setErrorCode("XTDE1460");
@@ -699,7 +634,7 @@ public class ResultDocument extends Instruction
 
         // Now add the properties that were specified as AVTs
 
-        if (serializationAttributes.size() > 0) {
+        if (!serializationAttributes.isEmpty()) {
             for (Map.Entry<StructuredQName, Operand> entry : serializationAttributes.entrySet()) {
                 String value = entry.getValue().getChildExpression().evaluateAsString(context).toString();
                 String lname = entry.getKey().getLocalPart();
@@ -766,89 +701,97 @@ public class ResultDocument extends Instruction
         SerializerFactory sf = config.getSerializerFactory();
         String clarkName = lname;
         if (!uri.isEmpty()) {
-            clarkName = '{' + uri + '}' + lname;
+            clarkName = "{" + uri + "}" + lname;
         }
         if (uri.isEmpty() || NamespaceConstant.SAXON.equals(uri)) {
-            if (clarkName.equals("method")) {
-                value = Whitespace.trim(value);
-                if (value.startsWith("Q{}") && value.length() > 3) {
-                    value = value.substring(3);
-                }
-                if (value.equals("xml") || value.equals("html") ||
-                        value.equals("text") || value.equals("xhtml") ||
-                        value.equals("json") || value.equals("adaptive") ||
-                        prevalidated || value.startsWith("{")) {
-                    details.setProperty(OutputKeys.METHOD, value);
-                } else if (value.startsWith("Q{")) {
-                    details.setProperty(OutputKeys.METHOD, value.substring(1));
-                } else {
-                    String[] parts;
-                    try {
-                        parts = NameChecker.getQNameParts(value);
-                        String prefix = parts[0];
-                        if (prefix.isEmpty()) {
-                            XPathException err = new XPathException("method must be xml, html, xhtml, text, json, adaptive, or a prefixed name");
-                            err.setErrorCode("SEPM0016");
-                            err.setIsStaticError(true);
-                            throw err;
-                        } else if (nsResolver != null) {
-                            String muri = nsResolver.getURIForPrefix(prefix, false);
-                            if (muri == null) {
-                                XPathException err = new XPathException("Namespace prefix '" + prefix + "' has not been declared");
+            switch (clarkName) {
+                case "method":
+                    value = Whitespace.trim(value);
+                    if (value.startsWith("Q{}") && value.length() > 3) {
+                        value = value.substring(3);
+                    }
+                    if (value.equals("xml") || value.equals("html") ||
+                            value.equals("text") || value.equals("xhtml") ||
+                            value.equals("json") || value.equals("adaptive") ||
+                            prevalidated || value.startsWith("{")) {
+                        details.setProperty(OutputKeys.METHOD, value);
+                    } else if (value.startsWith("Q{")) {
+                        details.setProperty(OutputKeys.METHOD, value.substring(1));
+                    } else {
+                        String[] parts;
+                        try {
+                            parts = NameChecker.getQNameParts(value);
+                            String prefix = parts[0];
+                            if (prefix.isEmpty()) {
+                                XPathException err = new XPathException("method must be xml, html, xhtml, text, json, adaptive, or a prefixed name");
                                 err.setErrorCode("SEPM0016");
                                 err.setIsStaticError(true);
                                 throw err;
+                            } else if (nsResolver != null) {
+                                String muri = nsResolver.getURIForPrefix(prefix, false);
+                                if (muri == null) {
+                                    XPathException err = new XPathException("Namespace prefix '" + prefix + "' has not been declared");
+                                    err.setErrorCode("SEPM0016");
+                                    err.setIsStaticError(true);
+                                    throw err;
+                                }
+                                details.setProperty(OutputKeys.METHOD, '{' + muri + '}' + parts[1]);
+                            } else {
+                                details.setProperty(OutputKeys.METHOD, value);
                             }
-                            details.setProperty(OutputKeys.METHOD, '{' + muri + '}' + parts[1]);
-                        } else {
-                            details.setProperty(OutputKeys.METHOD, value);
+                        } catch (QNameException e) {
+                            XPathException err = new XPathException("Invalid method name. " + e.getMessage());
+                            err.setErrorCode("SEPM0016");
+                            err.setIsStaticError(true);
+                            throw err;
                         }
-                    } catch (QNameException e) {
-                        XPathException err = new XPathException("Invalid method name. " + e.getMessage());
-                        err.setErrorCode("SEPM0016");
-                        err.setIsStaticError(true);
-                        throw err;
                     }
-                }
-            } else if (clarkName.equals("use-character-maps")) {
-                // The use-character-maps attribute is always turned into a Clark-format name at compile time
-                String existing = details.getProperty(SaxonOutputKeys.USE_CHARACTER_MAPS);
-                if (existing == null) {
-                    existing = "";
-                }
-                details.setProperty(SaxonOutputKeys.USE_CHARACTER_MAPS, existing + value);
-            } else if (clarkName.equals("cdata-section-elements")) {
-                processListOfNodeNames(details, OutputKeys.CDATA_SECTION_ELEMENTS, value, nsResolver, true, prevalidated);
-            } else if (clarkName.equals("suppress-indentation")) {
-                processListOfNodeNames(details, SaxonOutputKeys.SUPPRESS_INDENTATION, value, nsResolver, true, prevalidated);
-            } else if (clarkName.equals(SaxonOutputKeys.DOUBLE_SPACE)) {
-                processListOfNodeNames(details, SaxonOutputKeys.DOUBLE_SPACE, value, nsResolver, true, prevalidated);
-            } else if (clarkName.equals(SaxonOutputKeys.ATTRIBUTE_ORDER)) {
-                processListOfNodeNames(details, SaxonOutputKeys.ATTRIBUTE_ORDER, value, nsResolver, false, prevalidated);
-            } else if (clarkName.equals(SaxonOutputKeys.NEXT_IN_CHAIN)) {
+                    break;
+                case "use-character-maps":
+                    // The use-character-maps attribute is always turned into a Clark-format name at compile time
+                    String existing = details.getProperty(SaxonOutputKeys.USE_CHARACTER_MAPS);
+                    if (existing == null) {
+                        existing = "";
+                    }
+                    details.setProperty(SaxonOutputKeys.USE_CHARACTER_MAPS, existing + value);
+                    break;
+                case "cdata-section-elements":
+                    processListOfNodeNames(details, clarkName, value, nsResolver, true, prevalidated, false);
+                    break;
+                case "suppress-indentation":
+                    processListOfNodeNames(details, clarkName, value, nsResolver, true, prevalidated, false);
+                    break;
+                case SaxonOutputKeys.DOUBLE_SPACE:
+                    processListOfNodeNames(details, clarkName, value, nsResolver, true, prevalidated, false);
+                    break;
+                case SaxonOutputKeys.ATTRIBUTE_ORDER:
+                    processListOfNodeNames(details, clarkName, value, nsResolver, false, prevalidated, true);
+                    break;
+                case SaxonOutputKeys.NEXT_IN_CHAIN:
 //                XPathException e = new XPathException("saxon:next-in-chain property is available only on xsl:output");
 //                e.setErrorCodeQName(
 //                        new StructuredQName("saxon", NamespaceConstant.SAXON, SaxonErrorCode.SXWN9004));
 //                throw e;
-            } else {
-                // all other properties in the default or Saxon namespaces
-                if (clarkName.equals("output-version")) {
-                    clarkName = "version";
-                }
-
-                if (!prevalidated) {
-                    try {
-                        if (!SaxonOutputKeys.ITEM_SEPARATOR.equals(clarkName)) {
-                            // TODO: whitespace rules seem to vary for different interfaces
-                            value = Whitespace.trim(value);
-                        }
-                        value = sf.checkOutputProperty(clarkName, value);
-                    } catch (XPathException err) {
-                        err.maybeSetErrorCode("SEPM0016");
-                        throw err;
+                    break;
+                default:
+                    // all other properties in the default or Saxon namespaces
+                    if (clarkName.equals("output-version")) {
+                        clarkName = "version";
                     }
-                }
-                details.setProperty(clarkName, value);
+                    if (!prevalidated) {
+                        try {
+                            if (!SaxonOutputKeys.isUnstrippedProperty(clarkName)) {
+                                // TODO: whitespace rules seem to vary for different interfaces
+                                value = Whitespace.trim(value);
+                            }
+                            value = sf.checkOutputProperty(clarkName, value);
+                        } catch (XPathException err) {
+                            err.maybeSetErrorCode("SEPM0016");
+                            throw err;
+                        }
+                    }
+                    details.setProperty(clarkName, value);
+                    break;
             }
         } else {
             // properties in user-defined namespaces
@@ -858,12 +801,13 @@ public class ResultDocument extends Instruction
     }
 
     private static void processListOfNodeNames(Properties details, String key, String value,
-                                               NamespaceResolver nsResolver, boolean useDefaultNS, boolean prevalidated) throws XPathException {
+                                               NamespaceResolver nsResolver, boolean useDefaultNS,
+                                               boolean prevalidated, boolean allowStar) throws XPathException {
         String existing = details.getProperty(key);
         if (existing == null) {
             existing = "";
         }
-        String s = SaxonOutputKeys.parseListOfNodeNames(value, nsResolver, useDefaultNS, prevalidated, "SEPM0016");
+        String s = SaxonOutputKeys.parseListOfNodeNames(value, nsResolver, useDefaultNS, prevalidated, allowStar, "SEPM0016");
         details.setProperty(key, existing + s);
     }
 
@@ -881,7 +825,7 @@ public class ResultDocument extends Instruction
         }
         final SchemaType schemaType = getSchemaType();
         if (schemaType != null) {
-            out.emitAttribute("type", schemaType.getEQName());
+            out.emitAttribute("type", schemaType.getStructuredQName());
         }
         if (getHref() != null) {
             out.setChildRole("href");
@@ -923,21 +867,16 @@ public class ResultDocument extends Instruction
      */
 
     public static void processXslOutputElement(NodeInfo element, Properties props, XPathContext c) throws XPathException {
-        SequenceIterator iter = element.iterateAxis(AxisInfo.ATTRIBUTE);
+        AxisIterator iter = element.iterateAxis(AxisInfo.ATTRIBUTE);
         NamespaceResolver resolver = new InscopeNamespaceResolver(element);
-        while (true) {
-            NodeInfo att = (NodeInfo) iter.next();
-            if (att == null) {
-                break;
-            }
+        iter.forEachOrFail(att -> {
             String uri = att.getURI();
             String local = att.getLocalPart();
             String val = Whitespace.trim(att.getStringValueCS());
             setSerializationProperty(props, uri, local, val, resolver, false, c.getConfiguration());
-        }
+        });
     }
 
-
     /**
      * Get the (partial) name of a class that supports streaming of this kind of expression
      *
