@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -26,22 +26,20 @@ import net.sf.saxon.trans.Visibility;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.iter.SingletonIterator;
 import net.sf.saxon.tree.iter.UnfailingIterator;
-import net.sf.saxon.type.AnyItemType;
-import net.sf.saxon.type.FunctionItemType;
-import net.sf.saxon.type.ItemType;
-import net.sf.saxon.type.SpecificFunctionType;
+import net.sf.saxon.type.*;
 import net.sf.saxon.value.EmptySequence;
 import net.sf.saxon.value.SequenceType;
 
+import java.util.Objects;
+
 /**
  * This object represents the compiled form of a user-written function
  * (the source can be either an XSLT stylesheet function or an XQuery function).
- * <p/>
  * <p>It is assumed that type-checking, of both the arguments and the results,
  * has been handled at compile time. That is, the expression supplied as the body
  * of the function must be wrapped in code to check or convert the result to the
  * required type, and calls on the function must be wrapped at compile time to check or
- * convert the supplied arguments.
+ * convert the supplied arguments.</p>
  */
 
 public class UserFunction extends Actor implements Function, ContextOriginator {
@@ -56,7 +54,7 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
     private UserFunctionParameter[] parameterDefinitions;
     private SequenceType resultType;
     private SequenceType declaredResultType;
-    protected int evaluationMode = ExpressionTool.UNDECIDED;
+    protected Evaluator evaluator = null;
     private boolean isUpdating = false;
     private int inlineable = -1; // 0:no 1:yes -1:don't know
     private boolean overrideExtensionFunction = true;
@@ -209,7 +207,7 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
      */
 
     public void setAnnotations(AnnotationList list) {
-        this.annotations = list;
+        this.annotations = Objects.requireNonNull(list);
     }
 
     /**
@@ -250,11 +248,11 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
             // If this function contains tail calls, we evaluate it eagerly, because
             // the caller needs to know whether a tail call was returned or not: if we
             // return a Closure, the tail call escapes into the wild and can reappear anywhere...
-            evaluationMode = ExpressionTool.eagerEvaluationMode(getBody());
+            evaluator = ExpressionTool.eagerEvaluator(getBody());
 //        } else if ((getBody().getSpecialProperties() & StaticProperty.ALL_NODES_NEWLY_CREATED) != 0) {
 //            evaluationMode = ExpressionTool.PROCESS;
         } else {
-            evaluationMode = ExpressionTool.lazyEvaluationMode(getBody());
+            evaluator = ExpressionTool.lazyEvaluator(getBody(), true);
         }
     }
 
@@ -273,7 +271,7 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
             // bug 2226
             return null;
         }
-        if ((body.getSpecialProperties() & StaticProperty.HAS_SIDE_EFFECTS) != 0 || tailCalls) {
+        if (body.hasSpecialProperty(StaticProperty.HAS_SIDE_EFFECTS) || tailCalls) {
             // This is mainly to handle current-output-uri()
             return false;
         }
@@ -473,11 +471,11 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
      * @return the computed evaluation mode
      */
 
-    public int getEvaluationMode() {
-        if (evaluationMode == ExpressionTool.UNDECIDED) {
+    public Evaluator getEvaluator() {
+        if (evaluator == null) {
             computeEvaluationMode();
         }
-        return evaluationMode;
+        return evaluator;
     }
 
     /**
@@ -487,7 +485,7 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
      */
 
     public void setEvaluationMode(int mode) {
-        evaluationMode = mode;
+        evaluator = Evaluator.evaluatorForCode(mode);
     }
 
     /**
@@ -512,6 +510,14 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
 
     public void typeCheck(ExpressionVisitor visitor) throws XPathException {
         Expression exp = getBody();
+        if (exp instanceof ValueOf
+                && ((ValueOf)exp).getSelect().getItemType().isAtomicType()
+                && declaredResultType.getPrimaryType().isAtomicType()
+                && declaredResultType.getPrimaryType() != BuiltInAtomicType.STRING) {
+            visitor.getStaticContext().issueWarning(
+                    "A function that computes atomic values should use xsl:sequence rather than xsl:value-of",
+                    getLocation());
+        }
         ExpressionTool.resetPropertiesWithinSubtree(exp);
         Expression exp2 = exp;
         try {
@@ -522,7 +528,7 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
             if (resultType != null) {
                 RoleDiagnostic role =
                         new RoleDiagnostic(RoleDiagnostic.FUNCTION_RESULT,
-                                           functionName == null ? "" : functionName.getDisplayName(), 0);
+                                           functionName == null ? "" : functionName.getDisplayName() + "#" + getArity(), 0);
                 role.setErrorCode(getPackageData().getHostLanguage() == Configuration.XSLT && getFunctionName() != null ? "XTTE0780" : "XPTY0004");
                 exp2 = visitor.getConfiguration().getTypeChecker(false).staticTypeCheck(exp2, resultType, role, visitor);
             }
@@ -543,8 +549,7 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
      */
 
     public XPathContextMajor makeNewContext(XPathContext oldContext) {
-        XPathContextMajor c2;
-        c2 = oldContext.newCleanContext();
+        XPathContextMajor c2 = oldContext.newCleanContext();
         c2.setOrigin(this);
         c2.setReceiver(oldContext.getReceiver());
         c2.setTemporaryOutputState(StandardNames.XSL_FUNCTION);
@@ -553,8 +558,6 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
         return c2;
     }
 
-    //private static int pullCalls = 0, pushCalls = 0;
-
 
     /**
      * Call this function to return a value.
@@ -570,35 +573,21 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
      * @throws net.sf.saxon.trans.XPathException if a dynamic error occurs while evaluating the function
      */
 
-    public Sequence call(XPathContext context, Sequence[] actualArgs)
+    public Sequence<? extends Item> call(XPathContext context, Sequence[] actualArgs)
             throws XPathException {
-//        if (pullCalls++ % 1000 == 0) {
-//            System.err.println(pullCalls + " pull function calls");
-//        }
-//        if(onFirstCall != null) {
-//                if(callsUntilOptimize-- <=0) {
-//                    synchronized (this){
-//                        body = body.optimize(null, null); //TODO action to be performed as a Closure - holds the deferred ExpressionVisitor visitor, ContextItemStaticInfo cit
-//                        optimized = true;
-//                    }
-//                }
-//
-//
-//        }
-        if (evaluationMode == ExpressionTool.UNDECIDED) {
+        if (evaluator == null) {
             // should have been done at compile time
             computeEvaluationMode();
         }
 
-        // Otherwise evaluate the function
-
         XPathContextMajor c2 = (XPathContextMajor) context;
         c2.setStackFrame(getStackFrameMap(), actualArgs);
         Sequence result;
         try {
-            result = ExpressionTool.evaluate(getBody(), evaluationMode, c2, 1);
+            result = evaluator.evaluate(getBody(), c2);
         } catch (XPathException err) {
             err.maybeSetLocation(getLocation());
+            err.maybeSetContext(c2);
             throw err;
         } catch (Exception err2) {
             String message = "Internal error evaluating function "
@@ -626,9 +615,6 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
 
     public void process(Sequence[] actualArgs, XPathContextMajor context)
             throws XPathException {
-//        if (pushCalls++ % 1000 == 0) {
-//            System.err.println(pushCalls + " push function calls");
-//        }
         context.setStackFrame(getStackFrameMap(), actualArgs);
         getBody().process(context);
     }
@@ -687,10 +673,10 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
     public void export(ExpressionPresenter presenter) throws XPathException {
         presenter.startElement("function");
         if (getFunctionName() != null) {
-            presenter.emitAttribute("name", getFunctionName().getEQName());
+            presenter.emitAttribute("name", getFunctionName());
             presenter.emitAttribute("line", getLineNumber() + "");
             presenter.emitAttribute("module", getSystemId());
-            presenter.emitAttribute("eval", getEvaluationMode() + "");
+            presenter.emitAttribute("eval", getEvaluator().getCode() + "");
         }
         String flags = "";
         if (determinism == Determinism.PROACTIVE) {
@@ -700,7 +686,7 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
         } else {
             flags += "d";
         }
-        if (isMemoFunction() && !"JS".equals(presenter.getOption("target"))) {
+        if (isMemoFunction()) {
             flags += "m";
         }
         switch (declaredStreamability) {
@@ -880,9 +866,8 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
      * node as defined in the XPath 2.0 data model, except that all nodes are treated as being
      * untyped: it is not an error to get the string value of a node with a complex type.
      * For atomic values, the method returns the result of casting the atomic value to a string.
-     * <p/>
-     * If the calling code can handle any CharSequence, the method {@link #getStringValueCS} should
-     * be used. If the caller requires a string, this method is preferred.
+     * <p>If the calling code can handle any CharSequence, the method {@link #getStringValueCS} should
+     * be used. If the caller requires a string, this method is preferred.</p>
      *
      * @return the string value of the item
      * @throws UnsupportedOperationException if the item is a function item (an unchecked exception
@@ -900,12 +885,10 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
      * the version of the method that returns a String. The method satisfies the rule that
      * <code>X.getStringValueCS().toString()</code> returns a string that is equal to
      * <code>X.getStringValue()</code>.
-     * <p/>
-     * Note that two CharSequence values of different types should not be compared using equals(), and
-     * for the same reason they should not be used as a key in a hash table.
-     * <p/>
-     * If the calling code can handle any CharSequence, this method should
-     * be used. If the caller requires a string, the {@link #getStringValue} method is preferred.
+     * <p>Note that two CharSequence values of different types should not be compared using equals(), and
+     * for the same reason they should not be used as a key in a hash table.</p>
+     * <p>If the calling code can handle any CharSequence, this method should
+     * be used. If the caller requires a string, the {@link #getStringValue} method is preferred.</p>
      *
      * @return the string value of the item
      * @throws UnsupportedOperationException if the item is a function item (an unchecked exception
