@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -32,7 +32,7 @@ public class WithParam  {
     private int slotNumber = -1;
     private SequenceType requiredType;
     private StructuredQName variableQName;
-    private int evaluationMode = ExpressionTool.UNDECIDED;
+    private Evaluator evaluator = null;
 
     public WithParam() {
     }
@@ -180,7 +180,7 @@ public class WithParam  {
         if (params != null) {
             for (WithParam param : params) {
                 param.selectOp.optimize(visitor, contextItemType);
-                param.computeEvaluationMode();
+                param.computeEvaluator();
             }
         }
     }
@@ -192,14 +192,16 @@ public class WithParam  {
      */
 
     public int getEvaluationMode() {
-        if (evaluationMode == ExpressionTool.UNDECIDED) {
-            computeEvaluationMode();
+        if (evaluator == null) {
+            computeEvaluator();
         }
-        return evaluationMode;
+        return evaluator.getCode();
     }
 
-    private void computeEvaluationMode() {
-        evaluationMode = ExpressionTool.lazyEvaluationMode(selectOp.getChildExpression());
+
+
+    private void computeEvaluator() {
+        evaluator = ExpressionTool.lazyEvaluator(selectOp.getChildExpression(), true);
     }
 
 
@@ -256,7 +258,7 @@ public class WithParam  {
         if (params != null) {
             for (WithParam param : params) {
                 out.startElement("withParam");
-                out.emitAttribute("name", param.variableQName.getEQName());
+                out.emitAttribute("name", param.variableQName);
                 String flags = "";
                 if (tunnel) {
                     flags += "t";
@@ -294,9 +296,12 @@ public class WithParam  {
     public Sequence getSelectValue(XPathContext context) throws XPathException {
         // There is a select attribute: do a lazy evaluation of the expression,
         // which will already contain any code to force conversion to the required type.
+        if (evaluator == null) {
+            computeEvaluator();
+        }
         int savedOutputState = context.getTemporaryOutputState();
         context.setTemporaryOutputState(StandardNames.XSL_WITH_PARAM);
-        Sequence result = ExpressionTool.evaluate(selectOp.getChildExpression(), evaluationMode, context, 10);
+        Sequence result = evaluator.evaluate(selectOp.getChildExpression(), context);
         context.setTemporaryOutputState(savedOutputState);
         return result;
     }
