@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -11,21 +11,22 @@ import net.sf.saxon.Configuration;
 import net.sf.saxon.expr.*;
 import net.sf.saxon.expr.instruct.*;
 import net.sf.saxon.expr.sort.DocumentSorter;
-import net.sf.saxon.lib.FeatureKeys;
+import net.sf.saxon.lib.Feature;
 import net.sf.saxon.lib.Logger;
+import net.sf.saxon.om.GroundedValue;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.om.SequenceIterator;
 import net.sf.saxon.pattern.Pattern;
 import net.sf.saxon.style.XSLFunction;
 import net.sf.saxon.style.XSLTemplate;
 import net.sf.saxon.trans.GlobalVariableManager;
-import net.sf.saxon.trans.rules.RuleTarget;
 import net.sf.saxon.trans.XPathException;
+import net.sf.saxon.trans.rules.RuleTarget;
 import net.sf.saxon.type.TypeHierarchy;
 
 /**
  * This class performs optimizations that vary between different versions of the Saxon product.
- * The optimizer is obtained from the Saxon Configuration. This class is the version used in Saxon-B,
+ * The optimizer is obtained from the Saxon Configuration. This class is the version used in Saxon-HE,
  * which in most cases does no optimization at all: the methods are provided so that they can be
  * overridden in Saxon-EE.
  */
@@ -46,7 +47,7 @@ public class Optimizer {
 
     public Optimizer(Configuration config) {
         this.config = config;
-        this.tracing = config.getBooleanProperty(FeatureKeys.TRACE_OPTIMIZER_DECISIONS);
+        this.tracing = config.getBooleanProperty(Feature.TRACE_OPTIMIZER_DECISIONS);
     }
 
     /**
@@ -151,6 +152,21 @@ public class Optimizer {
         return f;
     }
 
+    /**
+     * Consider reordering the predicates in a filter expression based on cost estimates
+     *
+     * @param f       the filter expression (which is known to have more than one predicate)
+     * @param visitor expression visitor
+     * @param cisi    information about the context item type
+     * @return either the original expression, or an optimized replacement
+     * @throws XPathException if things go wrong
+     */
+
+    public Expression reorderPredicates(FilterExpression f, ExpressionVisitor visitor, ContextItemStaticInfo cisi)
+            throws XPathException {
+        return f;
+    }
+
     /**
      * Convert a path expression such as a/b/c[predicate] into a filter expression
      * of the form (a/b/c)[predicate]. This is possible whenever the predicate is non-positional.
@@ -184,13 +200,22 @@ public class Optimizer {
      *
      * @param iter the iterator that delivers the sequence of values to be indexed
      * @return the indexed value
-     * @throws UnsupportedOperationException: this method should not be called in Saxon-B
+     * @throws UnsupportedOperationException: this method should not be called in Saxon-HE
      */
 
-    public Sequence makeIndexedValue(SequenceIterator iter) throws XPathException {
+    public GroundedValue makeIndexedValue(SequenceIterator iter) throws XPathException {
         throw new UnsupportedOperationException("Indexing requires Saxon-EE");
     }
 
+    /**
+     * Prepare an expression for streaming
+     * @param exp the expression to be prepared
+     */
+
+    public void prepareForStreaming(Expression exp) throws XPathException {
+        // No action except for Saxon-EE
+    }
+
     /**
      * Evaluate the streaming (first) argument of a streamable stylesheet function
      *
@@ -243,7 +268,7 @@ public class Optimizer {
      * @param sorter the document sorter
      * @param path   the path expression
      * @return the original sorter unchanged when no optimization is possible, which is always the
-     * case in Saxon-B
+     * case in Saxon-HE
      */
 
     public Expression makeConditionalDocumentSorter(DocumentSorter sorter, SlashExpression path) throws XPathException {
@@ -258,7 +283,7 @@ public class Optimizer {
      * @param visitor         the expression visitor
      * @param contextItemType the context item type
      * @return either the original expression unchanged, or an expression that consists of the inlined
-     * function body, with all function parameters bound as required. In Saxon-B, function inlining is
+     * function body, with all function parameters bound as required. In Saxon-HE, function inlining is
      * not supported, so the original functionCall is always returned unchanged
      */
 
@@ -298,7 +323,6 @@ public class Optimizer {
      * Try to convert a Choose expression into a switch
      *
      * @param choose the Choose expression
-     * @param visitor
      * @return the result of optimizing this (the original expression if no optimization was possible)
      */
 
@@ -458,7 +482,7 @@ public class Optimizer {
     }
 
     public static void trace(Configuration config, String message, Expression exp) {
-        if (config.getBooleanProperty(FeatureKeys.TRACE_OPTIMIZER_DECISIONS)) {
+        if (config.getBooleanProperty(Feature.TRACE_OPTIMIZER_DECISIONS)) {
             Logger err = config.getLogger();
             err.info("OPT : At line " + exp.getLocation().getLineNumber() + " of " + exp.getLocation().getSystemId());
             err.info("OPT : " + message);
