@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -10,12 +10,14 @@ package net.sf.saxon.expr.parser;
 import net.sf.saxon.Configuration;
 import net.sf.saxon.expr.*;
 import net.sf.saxon.lib.ConversionRules;
+import net.sf.saxon.ma.map.MapType;
+import net.sf.saxon.ma.map.TupleType;
 import net.sf.saxon.om.*;
+import net.sf.saxon.pattern.NodeTest;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.type.*;
 import net.sf.saxon.value.AtomicValue;
 import net.sf.saxon.value.Cardinality;
-import net.sf.saxon.value.SequenceExtent;
 import net.sf.saxon.value.SequenceType;
 
 /**
@@ -31,7 +33,6 @@ public class TypeChecker {
 
     /**
      * Check an expression against a required type, modifying it if necessary.
-     * <p/>
      * <p>This method takes the supplied expression and checks to see whether it is
      * known statically to conform to the specified type. There are three possible
      * outcomes. If the static type of the expression is a subtype of the required
@@ -43,13 +44,11 @@ public class TypeChecker {
      * is constructed that evaluates the original expression and checks the dynamic
      * type of the result; this new expression is returned as the result of the
      * method.</p>
-     * <p/>
      * <p>The rules applied are those for function calling in XPath, that is, the rules
      * that the argument of a function call must obey in relation to the signature of
      * the function. Some contexts require slightly different rules (for example,
      * operands of polymorphic operators such as "+"). In such cases this method cannot
      * be used.</p>
-     * <p/>
      * <p>Note that this method does <b>not</b> do recursive type-checking of the
      * sub-expressions.</p>
      *
@@ -110,16 +109,11 @@ public class TypeChecker {
         // NOTE: we don't currently do any static inference regarding the content type
         if (!itemTypeOK) {
             suppliedItemType = exp.getItemType();
-//            if (suppliedItemType instanceof ErrorType) {
-//                // supplied type is empty-sequence(): this can violate a cardinality constraint but not an item type constraint
-//                itemTypeOK = true;
-//            } else {
             if (reqItemType == null || suppliedItemType == null) {
                 throw new NullPointerException();
             }
             int relation = th.relationship(reqItemType, suppliedItemType);
             itemTypeOK = relation == TypeHierarchy.SAME_TYPE || relation == TypeHierarchy.SUBSUMES;
-//            }
         }
 
 
@@ -131,15 +125,28 @@ public class TypeChecker {
                 // rule 1: Atomize
                 if (!suppliedItemType.isPlainType() &&
                         !(suppliedCard == StaticProperty.EMPTY)) {
-                    if (!suppliedItemType.isAtomizable()) {
+                    if (!suppliedItemType.isAtomizable(th)) {
+                        String shortItemType;
+                        if (suppliedItemType instanceof TupleType) {
+                            shortItemType = "a tuple type";
+                        } else if (suppliedItemType instanceof MapType) {
+                            shortItemType = "a map type";
+                        } else if (suppliedItemType instanceof FunctionItemType) {
+                            shortItemType = "a function type";
+                        } else if (suppliedItemType instanceof NodeTest) {
+                            shortItemType = "an element type with element-only content";
+                        } else {
+                            shortItemType = suppliedItemType.toString();
+                        }
                         XPathException err = new XPathException(
                                 "An atomic value is required for the " + role.getMessage() +
-                                        ", but the supplied value cannot be atomized", "FOTY0013", supplied.getLocation());
+                                        ", but the supplied type is " + shortItemType + ", which cannot be atomized", "FOTY0013", supplied.getLocation());
                         err.setIsTypeError(true);
+                        err.setFailingExpression(supplied);
                         throw err;
                     }
 
-                    if (exp instanceof ContextItemExpression && exp.getRetainedStaticContext() == null) {
+                    if (exp.getRetainedStaticContext() == null) {
                         exp.setRetainedStaticContextLocally(env.makeRetainedStaticContext());
                     }
                     Expression cexp = Atomizer.makeAtomizer(exp);
@@ -154,7 +161,6 @@ public class TypeChecker {
                 }
 
                 // rule 2: convert untypedAtomic to the required type
-                //TODO: merge common code between 2a & 2b
 
                 //   2a: all supplied values are untyped atomic. Convert if necessary, and we're finished.
 
@@ -167,18 +173,20 @@ public class TypeChecker {
                         err.setIsTypeError(true);
                         throw err;
                     }
-                    Expression cexp = UntypedSequenceConverter.makeUntypedSequenceConverter(config, exp, (PlainType) reqItemType);
+                    UntypedSequenceConverter cexp = UntypedSequenceConverter.makeUntypedSequenceConverter(config, exp, (PlainType) reqItemType);
+                    cexp.setRoleDiagnostic(role);
                     ExpressionTool.copyLocationInfo(exp, cexp);
                     try {
                         if (exp instanceof Literal) {
                             exp = Literal.makeLiteral(
-                                    SequenceExtent.makeSequenceExtent(cexp.iterate(visitor.makeDynamicContext())));
+                                    cexp.iterate(visitor.makeDynamicContext()).materialize(), exp);
                             ExpressionTool.copyLocationInfo(cexp, exp);
                         } else {
                             exp = cexp;
                         }
                     } catch (XPathException err) {
                         err.maybeSetLocation(exp.getLocation());
+                        err.setFailingExpression(supplied);
                         err.setErrorCode(role.getErrorCode());
                         err.setIsStaticError(true);
                         throw err;
@@ -192,19 +200,21 @@ public class TypeChecker {
 
                 if (suppliedItemType.equals(BuiltInAtomicType.ANY_ATOMIC)
                         && !(reqItemType.equals(BuiltInAtomicType.UNTYPED_ATOMIC) || reqItemType.equals(BuiltInAtomicType.ANY_ATOMIC))
-                        && (exp.getSpecialProperties() & StaticProperty.NOT_UNTYPED_ATOMIC) == 0) {
+                        && !exp.hasSpecialProperty(StaticProperty.NOT_UNTYPED_ATOMIC)) {
 
                     Expression conversion;
                     if (((SimpleType) reqItemType).isNamespaceSensitive()) {
                         conversion = UntypedSequenceConverter.makeUntypedSequenceRejector(config, exp, (PlainType) reqItemType);
                     } else {
-                        conversion = UntypedSequenceConverter.makeUntypedSequenceConverter(config, exp, (PlainType) reqItemType);
+                        UntypedSequenceConverter usc = UntypedSequenceConverter.makeUntypedSequenceConverter(config, exp, (PlainType) reqItemType);
+                        usc.setRoleDiagnostic(role);
+                        conversion = usc;
                     }
                     ExpressionTool.copyLocationInfo(exp, conversion);
                     try {
                         if (exp instanceof Literal) {
                             exp = Literal.makeLiteral(
-                                    SequenceExtent.makeSequenceExtent(conversion.iterate(visitor.makeDynamicContext())));
+                                    conversion.iterate(visitor.makeDynamicContext()).materialize(), exp);
                             ExpressionTool.copyLocationInfo(supplied, exp);
                         } else {
                             exp = conversion;
@@ -224,6 +234,9 @@ public class TypeChecker {
                     if (rt == StandardNames.XS_DOUBLE &&
                             th.relationship(suppliedItemType, NumericType.getInstance()) != TypeHierarchy.DISJOINT) {
                         Expression cexp = makePromoterToDouble(exp);
+                        if (cexp instanceof AtomicSequenceConverter) {
+                            ((AtomicSequenceConverter) cexp).setRoleDiagnostic(role);
+                        }
                         ExpressionTool.copyLocationInfo(exp, cexp);
                         exp = cexp;
                         try {
@@ -240,12 +253,16 @@ public class TypeChecker {
                             th.relationship(suppliedItemType, NumericType.getInstance()) != TypeHierarchy.DISJOINT &&
                             !th.isSubType(suppliedItemType, BuiltInAtomicType.DOUBLE)) {
                         Expression cexp = makePromoterToFloat(exp);
+                        if (cexp instanceof AtomicSequenceConverter) {
+                            ((AtomicSequenceConverter)cexp).setRoleDiagnostic(role);
+                        }
                         ExpressionTool.copyLocationInfo(exp, cexp);
                         exp = cexp;
                         try {
                             exp = exp.simplify().typeCheck(visitor, defaultContextInfo);
                         } catch (XPathException err) {
                             err.maybeSetLocation(exp.getLocation());
+                            err.setFailingExpression(supplied);
                             err.setIsStaticError(true);
                             throw err;
                         }
@@ -276,6 +293,7 @@ public class TypeChecker {
                     exp = makeFunctionSequenceCoercer(exp, (FunctionItemType) reqItemType, visitor, role);
                     itemTypeOK = true;
                 }
+
             } else if (reqItemType instanceof JavaExternalObjectType &&
                     /*Sequence.class.isAssignableFrom(((JavaExternalObjectType) reqItemType).getJavaClass()) &&  */
                     reqCard == StaticProperty.EXACTLY_ONE) {
@@ -321,6 +339,7 @@ public class TypeChecker {
         if (suppliedCard == StaticProperty.EMPTY && ((reqCard & StaticProperty.ALLOWS_ZERO) == 0)) {
             XPathException err = new XPathException("An empty sequence is not allowed as the " + role.getMessage(), role.getErrorCode(), supplied.getLocation());
             err.setIsTypeError(role.isTypeError());
+            err.setFailingExpression(supplied);
             throw err;
         }
 
@@ -339,14 +358,15 @@ public class TypeChecker {
             if (Cardinality.allowsZero(suppliedCard) &&
                     Cardinality.allowsZero(reqCard)) {
                 if (suppliedCard != StaticProperty.EMPTY) {
-                    String msg = role.composeErrorMessage(reqItemType, supplied, suppliedItemType);
+                    String msg = role.composeErrorMessage(reqItemType, supplied, th);
                     msg += ". The expression can succeed only if the supplied value is an empty sequence.";
                     visitor.issueWarning(msg, supplied.getLocation());
                 }
             } else {
-                String msg = role.composeErrorMessage(reqItemType, supplied, suppliedItemType);
+                String msg = role.composeErrorMessage(reqItemType, supplied, th);
                 XPathException err = new XPathException(msg, role.getErrorCode(), supplied.getLocation());
                 err.setIsTypeError(role.isTypeError());
+                err.setFailingExpression(supplied);
                 throw err;
             }
         }
@@ -361,7 +381,7 @@ public class TypeChecker {
                 if (req.matches(((Literal) exp).getValue(), th)) {
                     return exp;
                 }
-                String msg = role.composeErrorMessage(reqItemType, supplied, suppliedItemType);
+                String msg = role.composeErrorMessage(reqItemType, supplied, th);
                 XPathException err = new XPathException(msg, role.getErrorCode(), supplied.getLocation());
                 err.setIsTypeError(role.isTypeError());
                 throw err;
@@ -508,14 +528,14 @@ public class TypeChecker {
                     Cardinality.allowsZero(reqCard)) {
                 if (suppliedCard != StaticProperty.EMPTY) {
                     String msg = "Required item type of " + role.getMessage() +
-                            " is " + reqItemType.toString() +
+                            " is " + reqItemType +
                             "; supplied value (" + supplied.toShortString() + ") has item type " +
                             suppliedItemType.toString() +
                             ". The expression can succeed only if the supplied value is an empty sequence.";
                     env.issueWarning(msg, supplied.getLocation());
                 }
             } else {
-                String msg = role.composeErrorMessage(reqItemType, supplied, suppliedItemType);
+                String msg = role.composeErrorMessage(reqItemType, supplied, th);
                 XPathException err = new XPathException(msg, role.getErrorCode(), supplied.getLocation());
                 err.setIsTypeError(role.isTypeError());
                 throw err;
@@ -573,7 +593,7 @@ public class TypeChecker {
             count++;
             if (!reqItemType.matches(item, context.getConfiguration().getTypeHierarchy())) {
                 XPathException err = new XPathException("Required type is " + reqItemType +
-                                                                "; supplied value has type " + UType.getUType(SequenceTool.toGroundedValue(val)));
+                                                                "; supplied value has type " + UType.getUType(val.materialize()));
                 err.setIsTypeError(true);
                 err.setErrorCode("XPTY0004");
                 return err;
@@ -650,7 +670,7 @@ public class TypeChecker {
         if (exp instanceof Literal && ((Literal) exp).getValue() instanceof AtomicValue) {
             ConversionResult result = converter.convert((AtomicValue) ((Literal) exp).getValue());
             if (result instanceof AtomicValue) {
-                Literal converted = Literal.makeLiteral((AtomicValue) result);
+                Literal converted = Literal.makeLiteral((AtomicValue) result, exp);
                 ExpressionTool.copyLocationInfo(exp, converted);
                 return converted;
             }
