@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -11,10 +11,13 @@ import net.sf.saxon.Configuration;
 import net.sf.saxon.expr.*;
 import net.sf.saxon.expr.parser.*;
 import net.sf.saxon.om.AxisInfo;
+import net.sf.saxon.om.Item;
 import net.sf.saxon.om.SequenceIterator;
+import net.sf.saxon.pattern.AnyNodeTest;
 import net.sf.saxon.pattern.Pattern;
 import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.XPathException;
+import net.sf.saxon.type.TypeHierarchy;
 import net.sf.saxon.value.Cardinality;
 import net.sf.saxon.value.SequenceType;
 
@@ -70,7 +73,7 @@ public class DocumentSorter extends UnaryExpression {
     /*@NotNull*/
     public Expression simplify() throws XPathException {
         Expression operand = getBaseExpression().simplify();
-        if ((operand.getSpecialProperties() & StaticProperty.ORDERED_NODESET) != 0) {
+        if (operand.hasSpecialProperty(StaticProperty.ORDERED_NODESET)) {
             // this can happen as a result of further simplification
             return operand;
         }
@@ -83,7 +86,11 @@ public class DocumentSorter extends UnaryExpression {
         if (e2 != this) {
             return e2;
         }
-        RoleDiagnostic role = new RoleDiagnostic(RoleDiagnostic.DOCUMENT_ORDER, "", 0);
+        TypeHierarchy th = visitor.getConfiguration().getTypeHierarchy();
+        if (th.relationship(getBaseExpression().getItemType(), AnyNodeTest.getInstance()) == TypeHierarchy.DISJOINT) {
+            return getBaseExpression();
+        }
+        RoleDiagnostic role = new RoleDiagnostic(RoleDiagnostic.MISC, "document-order sorter", 0);
         Expression operand = visitor.getConfiguration().getTypeChecker(false).staticTypeCheck(
                 getBaseExpression(), SequenceType.NODE_SEQUENCE, role, visitor);
         setBaseExpression(operand);
@@ -93,40 +100,60 @@ public class DocumentSorter extends UnaryExpression {
     /*@NotNull*/
     public Expression optimize(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
         getOperand().optimize(visitor, contextInfo);
-        Expression operand = getBaseExpression();
-        boolean tryHarder = operand.isStaticPropertiesKnown();
+        Expression sortable = getBaseExpression();
+        boolean tryHarder = sortable.isStaticPropertiesKnown();
         while (true) {
-            if ((operand.getSpecialProperties() & StaticProperty.ORDERED_NODESET) != 0) {
+            if (sortable.hasSpecialProperty(StaticProperty.ORDERED_NODESET)) {
                 // this can happen as a result of further simplification
-                return operand;
+                return sortable;
             }
-            if (!Cardinality.allowsMany(operand.getCardinality())) {
-                return operand;
+            if (!Cardinality.allowsMany(sortable.getCardinality())) {
+                return sortable;
             }
-            // Bug 3389: try to rewrite sort(conditionalSort($var/child::x) / child::y)
-            // as conditionalSort($var, (child::x/child::y))
-            if (operand instanceof SlashExpression) {
-                SlashExpression slash = (SlashExpression)operand;
-                if (slash.getLhsExpression() instanceof ConditionalSorter &&
-                        (slash.getRhsExpression().getSpecialProperties() & StaticProperty.PEER_NODESET) != 0) {
-                    ConditionalSorter c = (ConditionalSorter)slash.getLhsExpression();
+
+            if (sortable instanceof SlashExpression) {
+                SlashExpression slash = (SlashExpression)sortable;
+
+                // Bug 3389: try to rewrite sort(conditionalSort($var/child::x) / child::y)
+                // as conditionalSort($var, (child::x/child::y))
+                Expression lhs = slash.getLhsExpression();
+                Expression rhs = slash.getRhsExpression();
+                if (lhs instanceof ConditionalSorter &&
+                        slash.getRhsExpression().hasSpecialProperty(StaticProperty.PEER_NODESET)) {
+                    ConditionalSorter c = (ConditionalSorter) lhs;
                     DocumentSorter d = c.getDocumentSorter();
                     Expression condition = c.getCondition();
-                    SlashExpression s = new SlashExpression(d.getBaseExpression(), slash.getRhsExpression());
+                    SlashExpression s = new SlashExpression(d.getBaseExpression(), rhs);
                     return new ConditionalSorter(condition, new DocumentSorter(s));
                 }
+                // docOrder(docOrder(A)/B) can be rewritten as docOrder(A/B). However, this may not always
+                // be wise, because the inner docOrder might eliminate many duplicates, reducing the cost
+                // of the /B operation. We therefore do it only if B is a low-cost operation.
+                if (lhs instanceof DocumentSorter && rhs instanceof AxisExpression && ((AxisExpression)rhs).getAxis() == AxisInfo.CHILD) {
+                    SlashExpression s1 = new SlashExpression(((DocumentSorter)lhs).getBaseExpression(), rhs);
+                    ExpressionTool.copyLocationInfo(this, s1);
+                    return new DocumentSorter(s1).optimize(visitor, contextInfo);
+                }
+                // docOrder(A/B) can be rewritten as docOrder(B) in the case where B returns nodes
+                // and is independent of the focus. We already know it returns nodes otherwise we wouldn't be here.
+                if (!ExpressionTool.dependsOnFocus(rhs) &&
+                        !rhs.hasSpecialProperty(StaticProperty.HAS_SIDE_EFFECTS) &&
+                        rhs.hasSpecialProperty(StaticProperty.NO_NODES_NEWLY_CREATED)) {
+                    setBaseExpression(slash.getRhsExpression());
+                    return optimize(visitor, contextInfo);
+                }
             }
             // Try once more after recomputing the static properties of the expression
             if (tryHarder) {
-                operand.resetLocalStaticProperties();
+                sortable.resetLocalStaticProperties();
                 tryHarder = false;
             } else {
                 break;
             }
         }
-        if (operand instanceof SlashExpression && !visitor.isOptimizeForStreaming()) {
-            return visitor.getConfiguration().obtainOptimizer().makeConditionalDocumentSorter(
-                    this, (SlashExpression) operand);
+        if (sortable instanceof SlashExpression && !visitor.isOptimizeForStreaming()) {
+            return visitor.obtainOptimizer().makeConditionalDocumentSorter(
+                    this, (SlashExpression) sortable);
         }
         return this;
     }
@@ -155,7 +182,7 @@ public class DocumentSorter extends UnaryExpression {
     @Override
     public Expression unordered(boolean retainAllNodes, boolean forStreaming) throws XPathException {
         Expression operand = getBaseExpression().unordered(retainAllNodes, forStreaming);
-        if ((operand.getSpecialProperties() & StaticProperty.ORDERED_NODESET) != 0) {
+        if (operand.hasSpecialProperty(StaticProperty.ORDERED_NODESET)) {
             return operand;
         }
         if (!retainAllNodes) {
@@ -202,14 +229,13 @@ public class DocumentSorter extends UnaryExpression {
      * Convert this expression to an equivalent XSLT pattern
      *
      * @param config the Saxon configuration
-     * @param is30   true if this is XSLT 3.0
      * @return the equivalent pattern
      * @throws net.sf.saxon.trans.XPathException
      *          if conversion is not possible
      */
     @Override
-    public Pattern toPattern(Configuration config, boolean is30) throws XPathException {
-        return getBaseExpression().toPattern(config, is30);
+    public Pattern toPattern(Configuration config) throws XPathException {
+        return getBaseExpression().toPattern(config);
     }
 
     /**
@@ -226,7 +252,7 @@ public class DocumentSorter extends UnaryExpression {
     }
 
     /*@NotNull*/
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<? extends Item> iterate(XPathContext context) throws XPathException {
         return new DocumentOrderIterator(getBaseExpression().iterate(context), comparer);
     }
 
