@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -28,7 +28,6 @@ import java.util.List;
  * order of first appearance. Note that an item can appear in several groups;
  * indeed, an item may be the leading item of more than one group, which means
  * that knowing the leading item is not enough to know the current group.
- * <p/>
  * <p>The GroupByIterator acts as a SequenceIterator, where successive calls of
  * next() return the leading item of each group in turn. The current item of
  * the iterator is therefore the leading item of the current group. To get access
@@ -37,7 +36,7 @@ import java.util.List;
  * current group is available via the getCurrentGroupingKey() method.</p>
  */
 
-public class GroupByIterator implements GroupIterator, LastPositionFinder, LookaheadIterator {
+public class GroupByIterator implements GroupIterator, LastPositionFinder, LookaheadIterator<Item> {
 
     // The implementation of group-by is not pipelined. All the items in the population
     // are read at the start, their grouping keys are calculated, and the groups are formed
@@ -46,7 +45,7 @@ public class GroupByIterator implements GroupIterator, LastPositionFinder, Looka
     // as a list of items in population order), a list of grouping keys, and a list of
     // the initial items of the groups.
 
-    private SequenceIterator population;
+    private SequenceIterator<? extends Item> population;
     protected Expression keyExpression;
     private StringCollator collator;
     private XPathContext keyContext;
@@ -55,11 +54,11 @@ public class GroupByIterator implements GroupIterator, LastPositionFinder, Looka
     // Main data structure holds one entry for each group. The entry is also an ArrayList,
     // which contains the Items that are members of the group, in population order.
     // The groups are arranged in order of first appearance within the population.
-    protected List<List<Item>> groups = new ArrayList<List<Item>>(40);
+    protected List<List<Item>> groups = new ArrayList<>(40);
 
     // This parallel structure identifies the grouping key for each group. The list
     // corresponds one-to-one with the list of groups.
-    protected List<AtomicSequence> groupKeys = new ArrayList<AtomicSequence>(40);
+    protected List<AtomicSequence> groupKeys = new ArrayList<>(40);
 
     // An AtomicComparer is used to do the comparisons of individual atomic values
     //protected AtomicComparer comparer;
@@ -77,7 +76,7 @@ public class GroupByIterator implements GroupIterator, LastPositionFinder, Looka
      * @throws XPathException if an error occurs
      */
 
-    public GroupByIterator(SequenceIterator population, Expression keyExpression,
+    public GroupByIterator(SequenceIterator<? extends Item> population, Expression keyExpression,
                            XPathContext keyContext, StringCollator collator, boolean composite)
             throws XPathException {
         this.population = population;
@@ -104,10 +103,9 @@ public class GroupByIterator implements GroupIterator, LastPositionFinder, Looka
      */
 
     private void buildIndexedGroups() throws XPathException {
-        HashMap<AtomicMatchKey, List<Item>> index = new HashMap<AtomicMatchKey, List<Item>>(40);
+        HashMap<AtomicMatchKey, List<Item>> index = new HashMap<>(40);
         XPathContext c2 = keyContext.newMinorContext();
-        FocusIterator focus = new FocusTrackingIterator(population);
-        c2.setCurrentIterator(focus);
+        FocusIterator focus = c2.trackFocus(population);
         int implicitTimezone = c2.getImplicitTimezone();
         Item item;
         while ((item = focus.next()) != null) {
@@ -120,13 +118,13 @@ public class GroupByIterator implements GroupIterator, LastPositionFinder, Looka
                 }
                 AtomicMatchKey comparisonKey;
                 if (key.isNaN()) {
-                    comparisonKey = AtomicValue.NaN_MATCH_KEY;
+                    comparisonKey = AtomicMatchKey.NaN_MATCH_KEY;
                 } else {
                     comparisonKey = key.getXPathComparable(false, collator, implicitTimezone);
                 }
                 List<Item> g = index.get(comparisonKey);
                 if (g == null) {
-                    List<Item> newGroup = new ArrayList<Item>(20);
+                    List<Item> newGroup = new ArrayList<>(20);
                     newGroup.add(item);
                     groups.add(newGroup);
                     groupKeys.add(key);
@@ -158,16 +156,15 @@ public class GroupByIterator implements GroupIterator, LastPositionFinder, Looka
      */
 
     private void buildIndexedGroupsComposite() throws XPathException {
-        HashMap<List<AtomicMatchKey>, List<Item>> index = new HashMap<List<AtomicMatchKey>, List<Item>>(40);
+        HashMap<List<AtomicMatchKey>, List<Item>> index = new HashMap<>(40);
         XPathContext c2 = keyContext.newMinorContext();
-        FocusIterator focus = new FocusTrackingIterator(population);
-        c2.setCurrentIterator(focus);
+        FocusIterator focus = c2.trackFocus(population);
         int implicitTimezone = c2.getImplicitTimezone();
         Item item;
         while ((item = focus.next()) != null) {
             SequenceIterator keys = keyExpression.iterate(c2);
-            List<AtomicMatchKey> ckList = new ArrayList<AtomicMatchKey>();
-            List<AtomicValue> compositeKey = new ArrayList<AtomicValue>();
+            List<AtomicMatchKey> ckList = new ArrayList<>();
+            List<AtomicValue> compositeKey = new ArrayList<>();
             while (true) {
                 AtomicValue key = (AtomicValue) keys.next();
                 if (key == null) {
@@ -176,7 +173,7 @@ public class GroupByIterator implements GroupIterator, LastPositionFinder, Looka
                 compositeKey.add(key);
                 AtomicMatchKey comparisonKey;
                 if (key.isNaN()) {
-                    comparisonKey = AtomicValue.NaN_MATCH_KEY;
+                    comparisonKey = AtomicMatchKey.NaN_MATCH_KEY;
                 } else {
                     comparisonKey = key.getXPathComparable(false, collator, implicitTimezone);
                 }
@@ -185,7 +182,7 @@ public class GroupByIterator implements GroupIterator, LastPositionFinder, Looka
 
             List<Item> g = index.get(ckList);
             if (g == null) {
-                List<Item> newGroup = new ArrayList<Item>(20);
+                List<Item> newGroup = new ArrayList<>(20);
                 newGroup.add(item);
                 groups.add(newGroup);
                 groupKeys.add(new AtomicArray(compositeKey));
@@ -218,8 +215,8 @@ public class GroupByIterator implements GroupIterator, LastPositionFinder, Looka
      * @return the iterator
      */
 
-    public SequenceIterator iterateCurrentGroup() {
-        return new ListIterator(groups.get(position - 1));
+    public SequenceIterator<? extends Item> iterateCurrentGroup() {
+        return new ListIterator<>(groups.get(position - 1));
     }
 
     /**
@@ -282,7 +279,7 @@ public class GroupByIterator implements GroupIterator, LastPositionFinder, Looka
 
 //#if EE==true
 
-    public ManualGroupIterator getSnapShot(XPathContext context) throws XPathException {
+    public ManualGroupIterator getSnapShot(XPathContext context) {
         return new ManualGroupByIterator();
     }
 
@@ -293,15 +290,11 @@ public class GroupByIterator implements GroupIterator, LastPositionFinder, Looka
 
         public ManualGroupByIterator() {
             super(groups.get(position - 1).get(0), position);
-            setLastPositionFinder(new LastPositionFinder() {
-                public int getLength() throws XPathException {
-                    return groups.size();
-                }
-            });
+            setLastPositionFinder(() -> groups.size());
         }
 
-        public SequenceIterator iterateCurrentGroup() throws XPathException {
-            return new ListIterator(currentGroup);
+        public SequenceIterator<Item> iterateCurrentGroup()  {
+            return new ListIterator<>(currentGroup);
         }
 
         public AtomicSequence getCurrentGroupingKey() {
