@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -10,6 +10,7 @@ package net.sf.saxon.expr.sort;
 import com.saxonica.ee.stream.ManualGroupIterator;
 import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.om.AtomicSequence;
+import net.sf.saxon.om.Item;
 import net.sf.saxon.om.SequenceIterator;
 import net.sf.saxon.trans.XPathException;
 
@@ -21,7 +22,7 @@ import net.sf.saxon.trans.XPathException;
  * members of the current group.
  */
 
-public interface GroupIterator extends SequenceIterator {
+public interface GroupIterator extends SequenceIterator<Item> {
 
     /**
      * Get the grouping key of the current group
@@ -31,7 +32,7 @@ public interface GroupIterator extends SequenceIterator {
      */
 
     /*@Nullable*/
-    public AtomicSequence getCurrentGroupingKey();
+    AtomicSequence getCurrentGroupingKey();
 
     /**
      * Get an iterator over the members of the current group, in population
@@ -44,14 +45,14 @@ public interface GroupIterator extends SequenceIterator {
      *          if a dynamic error occurs
      */
 
-    public SequenceIterator iterateCurrentGroup() throws XPathException;
+    SequenceIterator<? extends Item> iterateCurrentGroup() throws XPathException;
 
     //#if EE==true
 
     /**
      * Get a sequence which is a snapshot of this sequence at the current position
      */
-    public ManualGroupIterator getSnapShot(XPathContext context) throws XPathException;
+    ManualGroupIterator getSnapShot(XPathContext context) throws XPathException;
 
 //#endif
 }
