@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -26,7 +26,7 @@ import java.util.List;
  * A GroupMatchingIterator contains code shared between GroupStartingIterator and GroupEndingIterator
  */
 
-public abstract class GroupMatchingIterator implements LookaheadIterator, LastPositionFinder, GroupIterator {
+public abstract class GroupMatchingIterator implements LookaheadIterator<Item>, LastPositionFinder, GroupIterator {
 
     protected Expression select;
     protected FocusIterator population;
@@ -45,8 +45,8 @@ public abstract class GroupMatchingIterator implements LookaheadIterator, LastPo
         return null;
     }
 
-    public SequenceIterator iterateCurrentGroup() {
-        return new ListIterator(currentMembers);
+    public SequenceIterator<Item> iterateCurrentGroup() {
+        return new ListIterator<>(currentMembers);
     }
 
     public boolean hasNext() {
@@ -76,7 +76,7 @@ public abstract class GroupMatchingIterator implements LookaheadIterator, LastPo
 
 //#if EE==true
 
-    public ManualGroupIterator getSnapShot(XPathContext context) throws XPathException {
+    public ManualGroupIterator getSnapShot(XPathContext context) {
         return new ManualGroupMatchingIterator();
     }
 
@@ -86,19 +86,15 @@ public abstract class GroupMatchingIterator implements LookaheadIterator, LastPo
 
         ManualGroupMatchingIterator() {
             super(current, position);
-            setLastPositionFinder(new LastPositionFinder() {
-                public int getLength() throws XPathException {
-                    return savedcontext.getLast();
-                }
-            });
+            setLastPositionFinder(() -> savedcontext.getLast());
         }
 
         public AtomicSequence getCurrentGroupingKey() {
             return null;
         }
 
-        public SequenceIterator iterateCurrentGroup() throws XPathException {
-            return new ListIterator(cMembers);
+        public SequenceIterator<Item> iterateCurrentGroup() {
+            return new ListIterator<>(cMembers);
         }
 
 
