@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,17 +9,16 @@ package net.sf.saxon.expr.sort;
 
 import com.saxonica.ee.stream.adjunct.MergeInstrAdjunct;
 import net.sf.saxon.Configuration;
-import net.sf.saxon.event.SequenceReceiver;
+import net.sf.saxon.event.Receiver;
+import net.sf.saxon.event.ReceiverOptions;
 import net.sf.saxon.expr.*;
 import net.sf.saxon.expr.accum.Accumulator;
 import net.sf.saxon.expr.accum.AccumulatorManager;
 import net.sf.saxon.expr.instruct.Instruction;
 import net.sf.saxon.expr.instruct.TailCall;
 import net.sf.saxon.expr.parser.*;
-import net.sf.saxon.functions.Count;
-import net.sf.saxon.functions.CurrentMergeGroup;
-import net.sf.saxon.functions.CurrentMergeKey;
-import net.sf.saxon.functions.DocumentFn;
+import net.sf.saxon.functions.*;
+import net.sf.saxon.lib.Feature;
 import net.sf.saxon.lib.ParseOptions;
 import net.sf.saxon.lib.Validation;
 import net.sf.saxon.om.*;
@@ -27,6 +26,7 @@ import net.sf.saxon.pattern.NodeKindTest;
 import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.NoDynamicContextException;
 import net.sf.saxon.trans.XPathException;
+import net.sf.saxon.trans.XsltController;
 import net.sf.saxon.tree.iter.EmptyIterator;
 import net.sf.saxon.tree.iter.ManualIterator;
 import net.sf.saxon.tree.iter.SingletonIterator;
@@ -35,9 +35,9 @@ import net.sf.saxon.type.ItemType;
 import net.sf.saxon.type.SchemaType;
 import net.sf.saxon.type.Type;
 import net.sf.saxon.type.TypeHierarchy;
+import net.sf.saxon.value.ExternalObject;
 import net.sf.saxon.value.ObjectValue;
 import net.sf.saxon.value.SequenceType;
-import org.jetbrains.annotations.NotNull;
 
 import java.util.ArrayList;
 import java.util.List;
@@ -114,6 +114,16 @@ public class MergeInstr extends Instruction {
             rowSelectOp = new Operand(instruction, rowSelect, ROW_SELECT);
         }
 
+        public void setStreamable(boolean streamable) {
+            this.streamable = streamable;
+            if (streamable && instruction.getConfiguration().getBooleanProperty(Feature.STREAMING_FALLBACK)) {
+                this.streamable = false;
+                Expression select = rowSelectOp.getChildExpression();
+                rowSelectOp.setChildExpression(
+                        SystemFunction.makeCall("snapshot", select.getRetainedStaticContext(), select));
+            }
+        }
+
         public MergeSource copyMergeSource(MergeInstr newInstr, RebindingMap rebindings) {
             SortKeyDefinition[] newKeyDef = new SortKeyDefinition[mergeKeyDefinitions.size()];
 
@@ -177,7 +187,7 @@ public class MergeInstr extends Instruction {
             if (streamable) {
                 MergeInstrAdjunct.checkStreamability(this, instruction.getConfiguration());
             }
-//#endif
+            //#endif
 
         }
     }
@@ -288,7 +298,7 @@ public class MergeInstr extends Instruction {
             } else if (mergeSource.getForEachSource() != null) {
                 mergeSource.forEachStreamOp.typeCheck(visitor, contextInfo);
 
-                RoleDiagnostic role = new RoleDiagnostic(RoleDiagnostic.INSTRUCTION, "xsl:merge/for-each-stream", 0);
+                RoleDiagnostic role = new RoleDiagnostic(RoleDiagnostic.INSTRUCTION, "xsl:merge/for-each-source", 0);
                 mergeSource.setForEachStream(tc.staticTypeCheck(
                         mergeSource.getForEachSource(), SequenceType.STRING_SEQUENCE, role, visitor));
 
@@ -358,7 +368,7 @@ public class MergeInstr extends Instruction {
         fixupGroupReferences(this, this, false);
     }
 
-    private static void fixupGroupReferences(Expression exp, MergeInstr instr, boolean isInLoop) throws XPathException {
+    private static void fixupGroupReferences(Expression exp, MergeInstr instr, boolean isInLoop) {
         if (exp == null) {
             // no action
         } else if (exp.isCallOn(CurrentMergeGroup.class)) {
@@ -411,7 +421,7 @@ public class MergeInstr extends Instruction {
 
     public final boolean mayCreateNewNodes() {
         int props = getAction().getSpecialProperties();
-        return (props & StaticProperty.NON_CREATIVE) == 0;
+        return (props & StaticProperty.NO_NODES_NEWLY_CREATED) == 0;
     }
 
 
@@ -437,7 +447,7 @@ public class MergeInstr extends Instruction {
             } else {
                 inputType = Type.getCommonSuperType(inputType, rowItemType, th);
             }
-            mergeSource.prepareForStreaming();
+            //mergeSource.prepareForStreaming();
         }
 
         ContextItemStaticInfo cit = config.makeContextItemStaticInfo(inputType, false);
@@ -453,6 +463,13 @@ public class MergeInstr extends Instruction {
         return this;
     }
 
+    @Override
+    public void prepareForStreaming() throws XPathException {
+        for (MergeSource mergeSource : mergeSources) {
+            mergeSource.prepareForStreaming();
+        }
+    }
+
     /**
      * Check that the sort key definitions are all compatible
      *
@@ -462,7 +479,7 @@ public class MergeInstr extends Instruction {
 
     private void checkMergeAtt(SortKeyDefinition[] sortKeyDefs) throws XPathException {
         for (int i = 1; i < sortKeyDefs.length; i++) {
-            if (!sortKeyDefs[0].equals(sortKeyDefs[i])) {
+            if (!sortKeyDefs[0].isEqual(sortKeyDefs[i])) {
                 throw new XPathException("Corresponding xsl:merge-key attributes in different xsl:merge-source elements " +
                         "do not have the same effective values", "XTDE2210");
             }
@@ -470,7 +487,7 @@ public class MergeInstr extends Instruction {
     }
 
     /**
-     * A call on last() while merging is unavoidedly expensive. If it happens, we evaluate the whole expression
+     * A call on last() while merging is unavoidably expensive. If it happens, we evaluate the whole expression
      * from scratch to count how many groups there are going to be. This method returns a function to perform
      * this computation; the resulting function is made known to the MergeGroupIterator, which thereby becomes
      * capable of computing its own length on demand, without actually disrupting its operation.
@@ -506,7 +523,7 @@ public class MergeInstr extends Instruction {
 
 
     /*@NotNull*/
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<? extends Item> iterate(XPathContext context) throws XPathException {
 
         try {
             AtomicComparer[] comps = getComparators(context);
@@ -521,14 +538,9 @@ public class MergeInstr extends Instruction {
 
             // and apply the merging action to each group of duplicate items within this sequence
             c1.setCurrentMergeGroupIterator((GroupIterator) inputIterator);
-            ContextMappingFunction applyAction = new ContextMappingFunction() {
-                public SequenceIterator map(XPathContext context) throws XPathException {
-                    return getAction().iterate(context);
-                }
-            };
             XPathContext c3 = c1.newMinorContext();
-            c3.setCurrentIterator(new FocusTrackingIterator(inputIterator));
-            return new ContextMappingIterator(applyAction, c3);
+            c3.trackFocus(inputIterator);
+            return new ContextMappingIterator<>(cxt -> getAction().iterate(cxt), c3);
         } catch (XPathException e) {
             e.maybeSetLocation(getLocation());
             throw e;
@@ -555,9 +567,9 @@ public class MergeInstr extends Instruction {
 
             if (ms.streamable && ms.getForEachSource() != null) {
 //#ifdefined STREAM
-                List<ContextMappingIterator<ObjectValue<ItemWithMergeKeys>>> msIters =
+                List<ContextMappingIterator<ExternalObject<ItemWithMergeKeys>>> msIters =
                         MergeInstrAdjunct.makeSourceIterators(ms, getPackageData(), c1);
-                for (ContextMappingIterator<ObjectValue<ItemWithMergeKeys>> iter : msIters) {
+                for (ContextMappingIterator<ExternalObject<ItemWithMergeKeys>> iter : msIters) {
                     inputIterator = makeMergeIterator(inputIterator, comps, ms, iter);
                 }
 //#endif
@@ -567,53 +579,34 @@ public class MergeInstr extends Instruction {
                 options.setTopLevelType(ms.schemaType);
                 options.setApplicableAccumulators(ms.accumulators);
                 SequenceIterator uriIter = ms.getForEachSource().iterate(c1);
-                final AccumulatorManager accumulatorManager = context.getController().getAccumulatorManager();
-                ItemMappingIterator mappedIter = new ItemMappingIterator(uriIter, new ItemMappingFunction() {
-                    public Item mapItem(Item baseItem) throws XPathException {
-                        String uri = baseItem.getStringValue();
-                        NodeInfo node = DocumentFn.makeDoc(uri, getRetainedStaticContext().getStaticBaseUriString(),
-                                                           getPackageData(), options, c1, getLocation(), true);
-                        accumulatorManager.setApplicableAccumulators(node.getTreeInfo(), ms.accumulators);
-                        return node;
-                    };
+                XsltController controller = (XsltController)context.getController();
+                final AccumulatorManager accumulatorManager = controller.getAccumulatorManager();
+                anchorsIter = new ItemMappingIterator(uriIter, baseItem -> {
+                    String uri = baseItem.getStringValue();
+                    NodeInfo node = DocumentFn.makeDoc(uri, getRetainedStaticContext().getStaticBaseUriString(),
+                                                       getPackageData(), options, c1, getLocation(), true);
+                    accumulatorManager.setApplicableAccumulators(node.getTreeInfo(), ms.accumulators);
+                    return node;
                 });
-                anchorsIter = mappedIter;
                 XPathContext c2 = c1.newMinorContext();
-                FocusIterator anchorsIterFocus = new FocusTrackingIterator(anchorsIter);
-                c2.setCurrentIterator(anchorsIterFocus);
+                FocusIterator anchorsIterFocus = c2.trackFocus(anchorsIter);
                 while (anchorsIterFocus.next() != null) {
-                    FocusIterator rowIntr = new FocusTrackingIterator(ms.getRowSelect().iterate(c2));
                     XPathContext c4 = c2.newMinorContext();
-                    c4.setCurrentIterator(rowIntr);
+                    FocusIterator rowIntr = c4.trackFocus(ms.getRowSelect().iterate(c2));
                     MergeKeyMappingFunction addMergeKeys = new MergeKeyMappingFunction(c4, ms);
-                    ContextMappingIterator<ObjectValue<ItemWithMergeKeys>> contextMapKeysItr =
-                            new ContextMappingIterator<ObjectValue<ItemWithMergeKeys>>(addMergeKeys, c4);
+                    ContextMappingIterator<ExternalObject<ItemWithMergeKeys>> contextMapKeysItr =
+                            new ContextMappingIterator<>(addMergeKeys, c4);
                     inputIterator = makeMergeIterator(inputIterator, comps, ms, contextMapKeysItr);
                 }
             } else if (ms.getForEachItem() != null) {
                 anchorsIter = ms.getForEachItem().iterate(c1);
                 XPathContext c2 = c1.newMinorContext();
-                FocusIterator anchorsIterFocus = new FocusTrackingIterator(anchorsIter);
-                c2.setCurrentIterator(anchorsIterFocus);
+                FocusIterator anchorsIterFocus = c2.trackFocus(anchorsIter);
                 while (anchorsIterFocus.next() != null) {
-                    FocusIterator rowIntr = new FocusTrackingIterator(ms.getRowSelect().iterate(c2));
-                    XPathContext c4 = c2.newMinorContext();
-                    c4.setTemporaryOutputState(StandardNames.XSL_MERGE_KEY);
-                    c4.setCurrentIterator(rowIntr);
-                    MergeKeyMappingFunction addMergeKeys = new MergeKeyMappingFunction(c4, ms);
-                    ContextMappingIterator<ObjectValue<ItemWithMergeKeys>> contextMapKeysItr =
-                            new ContextMappingIterator<ObjectValue<ItemWithMergeKeys>>(addMergeKeys, c4);
-                    inputIterator = makeMergeIterator(inputIterator, comps, ms, contextMapKeysItr);
+                    inputIterator = getInputIterator(comps, inputIterator, ms, c2);
                 }
             } else {
-                FocusIterator rowIntr = new FocusTrackingIterator(ms.getRowSelect().iterate(c1));
-                XPathContext c4 = c1.newMinorContext();
-                c4.setTemporaryOutputState(StandardNames.XSL_MERGE_KEY);
-                c4.setCurrentIterator(rowIntr);
-                MergeKeyMappingFunction addMergeKeys = new MergeKeyMappingFunction(c4, ms);
-                ContextMappingIterator<ObjectValue<ItemWithMergeKeys>> contextMapKeysItr =
-                        new ContextMappingIterator<ObjectValue<ItemWithMergeKeys>>(addMergeKeys, c4);
-                inputIterator = makeMergeIterator(inputIterator, comps, ms, contextMapKeysItr);
+                inputIterator = getInputIterator(comps, inputIterator, ms, c1);
 
             }
 
@@ -621,6 +614,17 @@ public class MergeInstr extends Instruction {
         return inputIterator;
     }
 
+    private SequenceIterator getInputIterator(AtomicComparer[] comps, SequenceIterator inputIterator, MergeSource ms, XPathContext c2) throws XPathException {
+        XPathContext c4 = c2.newMinorContext();
+        c4.setTemporaryOutputState(StandardNames.XSL_MERGE_KEY);
+        FocusIterator rowIntr = c4.trackFocus(ms.getRowSelect().iterate(c2));
+        MergeKeyMappingFunction addMergeKeys = new MergeKeyMappingFunction(c4, ms);
+        ContextMappingIterator<ExternalObject<ItemWithMergeKeys>> contextMapKeysItr =
+                new ContextMappingIterator<>(addMergeKeys, c4);
+        inputIterator = makeMergeIterator(inputIterator, comps, ms, contextMapKeysItr);
+        return inputIterator;
+    }
+
     /**
      * Get an array of comparers to be used for comparing items according to their merge keys. Ideally this
      * will have been done at compile time, in which case the compile-time result is simply returned.
@@ -630,7 +634,6 @@ public class MergeInstr extends Instruction {
      * of the merge key definitions
      */
 
-    @NotNull
     private AtomicComparer[] getComparators(XPathContext context) throws XPathException {
         // First establish an array of comparators to be used for comparing items according to their
         // merge keys. Ideally this will have been done at compile time.
@@ -669,7 +672,7 @@ public class MergeInstr extends Instruction {
 
     private SequenceIterator makeMergeIterator(
             SequenceIterator result, AtomicComparer[] comps, MergeSource ms,
-            ContextMappingIterator<ObjectValue<ItemWithMergeKeys>> contextMapKeysItr) throws XPathException {
+            ContextMappingIterator<ExternalObject<ItemWithMergeKeys>> contextMapKeysItr) throws XPathException {
         if (result == null || result instanceof EmptyIterator) {
             result = contextMapKeysItr;
         } else {
@@ -723,29 +726,25 @@ public class MergeInstr extends Instruction {
     }
 
     public ItemOrderComparer getComparer(final SortKeyDefinitionList sKeys, final AtomicComparer[] comps) {
-        return new ItemOrderComparer() {
-            public int compare(Item a, Item b) {
-                ObjectValue aObj = (ObjectValue) a;
-                ObjectValue bObj = (ObjectValue) b;
-                ItemWithMergeKeys aItem = (ItemWithMergeKeys) aObj.getObject();
-                ItemWithMergeKeys bItem = (ItemWithMergeKeys) bObj.getObject();
+        return (a, b) -> {
+            ObjectValue aObj = (ObjectValue) a;
+            ObjectValue bObj = (ObjectValue) b;
+            ItemWithMergeKeys aItem = (ItemWithMergeKeys) aObj.getObject();
+            ItemWithMergeKeys bItem = (ItemWithMergeKeys) bObj.getObject();
 
-
-                for (int i = 0; i < sKeys.size(); i++) {
-                    int val;
-                    try {
-                        val = comps[i].compareAtomicValues(aItem.sortKeyValues.get(i), bItem.sortKeyValues.get(i));
-                    } catch (NoDynamicContextException e) {
-                        throw new IllegalStateException(e);
-                    }
-
-                    if (val != 0) {
-                        return val;
-                    }
+            for (int i = 0; i < sKeys.size(); i++) {
+                int val;
+                try {
+                    val = comps[i].compareAtomicValues(aItem.sortKeyValues.get(i), bItem.sortKeyValues.get(i));
+                } catch (NoDynamicContextException e) {
+                    throw new IllegalStateException(e);
                 }
-                return 0;
-            }
 
+                if (val != 0) {
+                    return val;
+                }
+            }
+            return 0;
         };
     }
 
@@ -753,7 +752,7 @@ public class MergeInstr extends Instruction {
      * Copy an expression. This makes a deep copy.
      *
      * @return the copy of the original expression
-     * @param rebindings
+     * @param rebindings variables that need to be re-bound
      */
 
     /*@NotNull*/
@@ -825,22 +824,14 @@ public class MergeInstr extends Instruction {
     public TailCall processLeavingTail(XPathContext context)
             throws XPathException {
 
-        SequenceIterator iter = iterate(context);
-        SequenceReceiver out = context.getReceiver();
-        try {
-            Item it;
-            while ((it = iter.next()) != null) {
-                out.append(it, getLocation(), NodeInfo.ALL_NAMESPACES);
-            }
+        Receiver out = context.getReceiver();
+        try (SequenceIterator<? extends Item> iter = iterate(context)) {
+            iter.forEachOrFail(it -> out.append(it, getLocation(), ReceiverOptions.ALL_NAMESPACES));
         } catch (XPathException e) {
             e.maybeSetLocation(getLocation());
             e.maybeSetContext(context);
             throw e;
-        }  finally {
-            iter.close();
         }
-
-
         return null;
     }
 
@@ -855,6 +846,11 @@ public class MergeInstr extends Instruction {
         return "MergeInstr";
     }
 
+    /**
+     * Mapping function for items encountered during the merge; the mapping function wraps the merged
+     * item and its merge keys into a single composite object
+     */
+
     public static class MergeKeyMappingFunction implements ContextMappingFunction {
         private MergeSource ms;
         private XPathContext baseContext;
@@ -875,7 +871,7 @@ public class MergeInstr extends Instruction {
             Item currentItem = context.getContextItem();
             manualIterator.setContextItem(currentItem);
             ItemWithMergeKeys newItem = new ItemWithMergeKeys(currentItem, ms.mergeKeyDefinitions, ms.sourceName, keyContext);
-            return SingletonIterator.makeIterator(new ObjectValue<ItemWithMergeKeys>(newItem));
+            return SingletonIterator.makeIterator(new ObjectValue<>(newItem));
 
         };
     }
