@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,6 +9,7 @@ package net.sf.saxon.expr.sort;
 
 import net.sf.saxon.expr.*;
 import net.sf.saxon.expr.parser.*;
+import net.sf.saxon.om.Item;
 import net.sf.saxon.om.SequenceIterator;
 import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.XPathException;
@@ -108,7 +109,6 @@ public class SortExpression extends Expression
     /**
      * Add a representation of this expression to a PathMap. The PathMap captures a map of the nodes visited
      * by an expression in a source tree.
-     * <p/>
      * <p>The default implementation of this method assumes that an expression does no navigation other than
      * the navigation done by evaluating its subexpressions, and that the subexpressions are evaluated in the
      * same context as the containing expression. The method must be overridden for any expression
@@ -231,7 +231,6 @@ public class SortExpression extends Expression
 
     /**
      * Perform optimisation of an expression and its subexpressions.
-     * <p/>
      * <p>This method is called after all references to functions and variables have been resolved
      * to the declaration of the function or variable, and after all type checking has been done.</p>
      *
@@ -273,7 +272,7 @@ public class SortExpression extends Expression
      * Copy an expression. This makes a deep copy.
      *
      * @return the copy of the original expression
-     * @param rebindings
+     * @param rebindings variables that need to be re-bound
      */
 
     /*@NotNull*/
@@ -334,14 +333,14 @@ public class SortExpression extends Expression
 
     public int computeSpecialProperties() {
         int props = 0;
-        if ((getSelect().getSpecialProperties() & StaticProperty.CONTEXT_DOCUMENT_NODESET) != 0) {
+        if (getSelect().hasSpecialProperty(StaticProperty.CONTEXT_DOCUMENT_NODESET)) {
             props |= StaticProperty.CONTEXT_DOCUMENT_NODESET;
         }
-        if ((getSelect().getSpecialProperties() & StaticProperty.SINGLE_DOCUMENT_NODESET) != 0) {
+        if (getSelect().hasSpecialProperty(StaticProperty.SINGLE_DOCUMENT_NODESET)) {
             props |= StaticProperty.SINGLE_DOCUMENT_NODESET;
         }
-        if ((getSelect().getSpecialProperties() & StaticProperty.NON_CREATIVE) != 0) {
-            props |= StaticProperty.NON_CREATIVE;
+        if (getSelect().hasSpecialProperty(StaticProperty.NO_NODES_NEWLY_CREATED)) {
+            props |= StaticProperty.NO_NODES_NEWLY_CREATED;
         }
         return props;
     }
@@ -364,9 +363,9 @@ public class SortExpression extends Expression
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<? extends Item> iterate(XPathContext context) throws XPathException {
 
-        SequenceIterator iter = getSelect().iterate(context);
+        SequenceIterator<? extends Item> iter = getSelect().iterate(context);
         if (iter instanceof EmptyIterator) {
             return iter;
         }
