@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -39,7 +39,7 @@ public class SortedGroupIterator extends SortedIterator implements GroupIterator
     /**
      * Override the method that builds the array of values and sort keys.
      *
-     * @throws XPathException
+     * @throws XPathException in the event of a dynamic error
      */
 
     protected void buildArray() throws XPathException {
@@ -54,7 +54,7 @@ public class SortedGroupIterator extends SortedIterator implements GroupIterator
         count = 0;
 
         XPathContextMajor c2 = context.newContext();
-        c2.setCurrentIterator((FocusIterator)base);
+        c2.setCurrentIterator((FocusIterator<Item>)base);
         GroupIterator groupIter = (GroupIterator)((FocusTrackingIterator)base).getUnderlyingIterator();
         c2.setCurrentGroupIterator(groupIter);
 
