@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -23,7 +23,7 @@ import net.sf.saxon.tree.iter.LookaheadIterator;
  * Class to do a sorted iteration
  */
 
-public class SortedIterator implements SequenceIterator, LastPositionFinder, LookaheadIterator, Sortable {
+public class SortedIterator implements SequenceIterator<Item>, LastPositionFinder, LookaheadIterator<Item>, Sortable {
 
     // the items to be sorted
     protected SequenceIterator base;
@@ -65,13 +65,12 @@ public class SortedIterator implements SequenceIterator, LastPositionFinder, Loo
      * @param createNewContext true if sort keys are computed relative to the item being sorted (as in XSLT but not XQuery)
      */
 
-    public SortedIterator(XPathContext context, SequenceIterator base,
+    public SortedIterator(XPathContext context, SequenceIterator<? extends Item> base,
                           SortKeyEvaluator sortKeyEvaluator, AtomicComparer[] comparators, boolean createNewContext) {
 
         if (createNewContext) {
             this.context = context.newMinorContext();
-            this.base = new FocusTrackingIterator(base);
-            this.context.setCurrentIterator((FocusTrackingIterator)this.base);
+            this.base = this.context.trackFocus(base);
             this.context.setTemporaryOutputState(StandardNames.XSL_SORT);
         } else {
             this.base = base;
@@ -103,9 +102,8 @@ public class SortedIterator implements SequenceIterator, LastPositionFinder, Loo
      * is stateless and it is not necessary (or usual) to call it before calling
      * next(). It is used only when there is an explicit need to tell if we
      * are at the last element.
-     * <p/>
-     * This method must not be called unless the result of getProperties() on the iterator
-     * includes the bit setting {@link net.sf.saxon.om.SequenceIterator#LOOKAHEAD}
+     * <p>This method must not be called unless the result of getProperties() on the iterator
+     * includes the bit setting {@link net.sf.saxon.om.SequenceIterator#LOOKAHEAD}</p>
      *
      * @return true if there are more items in the sequence
      */
@@ -127,7 +125,7 @@ public class SortedIterator implements SequenceIterator, LastPositionFinder, Loo
                     // (a) it wouldn't have failed unless there was something to sort, and
                     // (b) it's going to fail again when next() is called
                     count = -1;
-                    base = new FocusTrackingIterator(new ErrorIterator(err));
+                    base = new FocusTrackingIterator<>(new ErrorIterator(err));
                     return true;
                 }
             }
@@ -149,7 +147,7 @@ public class SortedIterator implements SequenceIterator, LastPositionFinder, Loo
             doSort();
         }
         if (position < count) {
-            return (Item) values[(position++)].value;
+            return (Item) values[position++].value;
         } else {
             position = -1;
             return null;
@@ -255,7 +253,7 @@ public class SortedIterator implements SequenceIterator, LastPositionFinder, Loo
      * Compare two items in sorted sequence
      * (needed to implement the Sortable interface)
      *
-     * @return <0 if obj[a]<obj[b], 0 if obj[a]=obj[b], >0 if obj[a]>obj[b]
+     * @return &lt;0 if obj[a]&lt;obj[b], 0 if obj[a]=obj[b], &gt;0 if obj[a]&gt;obj[b]
      */
 
     public int compare(int a, int b) {
