@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -14,7 +14,7 @@ import net.sf.saxon.expr.Expression;
 import net.sf.saxon.expr.SystemFunctionCall;
 import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.expr.parser.ExpressionVisitor;
-import net.sf.saxon.lib.FeatureKeys;
+import net.sf.saxon.lib.Feature;
 import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.trans.XPathException;
@@ -36,7 +36,7 @@ public class AvailableSystemProperties extends SystemFunction {
      *          if a dynamic error occurs during the evaluation of the expression
      */
     public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
-        List<QNameValue> myList = new ArrayList<QNameValue>();
+        List<QNameValue> myList = new ArrayList<>();
         myList.add(new QNameValue("xsl", NamespaceConstant.XSLT, "version"));
         myList.add(new QNameValue("xsl", NamespaceConstant.XSLT, "vendor"));
         myList.add(new QNameValue("xsl", NamespaceConstant.XSLT, "vendor-url"));
@@ -57,7 +57,7 @@ public class AvailableSystemProperties extends SystemFunction {
         myList.add(new QNameValue("archive", Archive.NAMESPACE, "version"));
 //#endif
 
-        if (context.getConfiguration().getBooleanProperty(FeatureKeys.ALLOW_EXTERNAL_FUNCTIONS)) {
+        if (context.getConfiguration().getBooleanProperty(Feature.ALLOW_EXTERNAL_FUNCTIONS)) {
             for (Object s : System.getProperties().keySet()) {
                  myList.add(new QNameValue("", "", s.toString()));
             }
@@ -78,4 +78,4 @@ public class AvailableSystemProperties extends SystemFunction {
     }
 }
 
-//Copyright (c) 2017 Saxonica Limited.
\ No newline at end of file
+//Copyright (c) 2018 Saxonica Limited.
\ No newline at end of file
