@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -11,17 +11,21 @@ import net.sf.saxon.Configuration;
 import net.sf.saxon.Controller;
 import net.sf.saxon.expr.*;
 import net.sf.saxon.lib.CollectionFinder;
-import net.sf.saxon.lib.FeatureKeys;
+import net.sf.saxon.lib.Feature;
 import net.sf.saxon.lib.Resource;
 import net.sf.saxon.lib.ResourceCollection;
 import net.sf.saxon.om.*;
 import net.sf.saxon.resource.AbstractResourceCollection;
 import net.sf.saxon.style.StylesheetPackage;
+import net.sf.saxon.trans.UncheckedXPathException;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.wrapper.SpaceStrippedDocument;
 import net.sf.saxon.type.Type;
+import net.sf.saxon.value.ExternalObject;
 import net.sf.saxon.value.ObjectValue;
 
+import java.io.Closeable;
+import java.io.IOException;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.util.ArrayList;
@@ -61,11 +65,11 @@ public class CollectionFn extends SystemFunction implements Callable {
             return collectionUri;
         }
 
-        public Iterator<String> getResourceURIs(XPathContext context) throws XPathException {
+        public Iterator<String> getResourceURIs(XPathContext context) {
             return new ArrayList<String>().iterator();
         }
 
-        public Iterator<Resource> getResources(XPathContext context) throws XPathException {
+        public Iterator<Resource> getResources(XPathContext context) {
             return new ArrayList<Resource>().iterator();
         }
 
@@ -81,7 +85,7 @@ public class CollectionFn extends SystemFunction implements Callable {
     public int getSpecialProperties(Expression[] arguments) {
         // See redmine bug 1652. We cannot assume that the nodes will be in document order because we can't assume
         // they will all be "new" documents. We can't even assume that they will be distinct.
-        return (super.getSpecialProperties(arguments) & ~StaticProperty.NON_CREATIVE) | StaticProperty.PEER_NODESET;
+        return (super.getSpecialProperties(arguments) & ~StaticProperty.NO_NODES_NEWLY_CREATED) | StaticProperty.PEER_NODESET;
     }
 
     private String getAbsoluteCollectionURI(String href, XPathContext context) throws XPathException {
@@ -114,21 +118,6 @@ public class CollectionFn extends SystemFunction implements Callable {
             } catch (URISyntaxException e) {
                 throw new XPathException(e.getMessage(), "FODC0004");
             }
-//            try {
-//                URI uri = new URI(href);
-//                if (uri.isAbsolute()) {
-//                    absoluteURI = uri.toString();
-//                } else {
-//                    String base = getRetainedStaticContext().getStaticBaseUriString();
-//                    if (base != null) {
-//                        absoluteURI = new URI(base).resolve(href).toString();
-//                    } else {
-//                        throw new XPathException("Relative collection URI cannot be resolved: no base URI available", "FODC0002");
-//                    }
-//                }
-//            } catch (URISyntaxException e) {
-//                throw new XPathException(e.getMessage(), "FODC0004");
-//            }
         }
         return absoluteURI;
     }
@@ -139,7 +128,7 @@ public class CollectionFn extends SystemFunction implements Callable {
      * @param collection the resource collection
      * @param context the XPath dynamic context
      * @return a SequenceIterator delivering ObjectValue&lt;Resource> items
-     * @throws XPathException
+     * @throws XPathException if a dynamic error occurs
      */
 
     private SequenceIterator getSequenceIterator(final ResourceCollection collection, final XPathContext context) throws XPathException {
@@ -157,7 +146,13 @@ public class CollectionFn extends SystemFunction implements Callable {
             }
 
             public void close() {
-                // no action;
+                if (sources instanceof Closeable) {
+                    try {
+                        ((Closeable)sources).close();
+                    } catch (IOException e) {
+                        throw new UncheckedXPathException(new XPathException(e));
+                    }
+                }
             }
 
             public int getProperties() {
@@ -173,10 +168,10 @@ public class CollectionFn extends SystemFunction implements Callable {
      * @param context   the dynamic evaluation context
      * @param arguments the values of the arguments, supplied as Sequences.
      * @return the sequence of nodes forming the collection
-     * @throws XPathException
+     * @throws XPathException if a dynamic error occurs
      */
 
-    public Sequence call(final XPathContext context, Sequence[] arguments) throws XPathException {
+    public Sequence<? extends Item> call(final XPathContext context, Sequence[] arguments) throws XPathException {
         String href;
         if (getArity() == 0) {
             // No arguments supplied: this gets the default collection
@@ -211,7 +206,7 @@ public class CollectionFn extends SystemFunction implements Callable {
             }
         }
 
-        GroundedValue cachedCollection = (GroundedValue)context.getController().getUserData("saxon:collections", collectionKey);
+        GroundedValue<? extends Item> cachedCollection = (GroundedValue<? extends Item>)context.getController().getUserData("saxon:collections", collectionKey);
         if (cachedCollection != null) {
             return cachedCollection;
         }
@@ -239,35 +234,30 @@ public class CollectionFn extends SystemFunction implements Callable {
         SequenceIterator sourceSeq = getSequenceIterator(collection, context);
 
         // Get an iterator over the items representing the resources
-        SequenceIterator result = context.getConfiguration()
+        SequenceIterator<? extends Item> result = context.getConfiguration()
                 .getMultithreadedItemMappingIterator(sourceSeq,
-                             new ItemMappingFunction() {
-                                 public Item mapItem(Item item1) throws XPathException {
-                                     return ((ObjectValue<Resource>)item1).getObject().getItem(context);
-                                 }
-                             });
+                                                     item1 -> ((ExternalObject<Resource>)item1).getObject().getItem(context));
 
         // In XSLT, apply space-stripping to document nodes in the collection
         if (whitespaceRule != null) {
             final SpaceStrippingRule rule = whitespaceRule;
-            ItemMappingFunction stripper = new ItemMappingFunction() {
-                public Item mapItem(Item item) throws XPathException {
-                    if (item instanceof NodeInfo && ((NodeInfo) item).getNodeKind() == Type.DOCUMENT) {
-                        SpaceStrippedDocument ssd = new SpaceStrippedDocument(((NodeInfo) item).getTreeInfo(), rule);
-                        return ssd.getRootNode();
-                    } else {
-                        return item;
+            ItemMappingFunction stripper = item -> {
+                if (item instanceof NodeInfo && ((NodeInfo) item).getNodeKind() == Type.DOCUMENT) {
+                    TreeInfo treeInfo = ((NodeInfo) item).getTreeInfo();
+                    if (treeInfo.getSpaceStrippingRule() != rule) {
+                        return new SpaceStrippedDocument(treeInfo, rule).getRootNode();
                     }
                 }
+                return item;
             };
             result = new ItemMappingIterator(result, stripper);
         }
 
         // If the collection is stable, cache the result
-        if (collection.isStable(context) || context.getConfiguration().getBooleanProperty(FeatureKeys.STABLE_COLLECTION_URI)) {
+        if (collection.isStable(context) || context.getConfiguration().getBooleanProperty(Feature.STABLE_COLLECTION_URI)) {
             Controller controller = context.getController();
             DocumentPool docPool = controller.getDocumentPool();
-            cachedCollection = SequenceTool.toGroundedValue(result);
+            cachedCollection = result.materialize();
             SequenceIterator iter = cachedCollection.iterate();
             Item item;
             while ((item = iter.next()) != null) {
