@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,10 +7,7 @@
 
 package net.sf.saxon.functions;
 
-import net.sf.saxon.expr.Callable;
-import net.sf.saxon.expr.Expression;
-import net.sf.saxon.expr.StaticProperty;
-import net.sf.saxon.expr.XPathContext;
+import net.sf.saxon.expr.*;
 import net.sf.saxon.expr.instruct.ForEachGroup;
 import net.sf.saxon.expr.parser.RebindingMap;
 import net.sf.saxon.expr.sort.GroupIterator;
@@ -166,7 +163,7 @@ public class CurrentGroupCall extends Expression implements Callable {
      * Determine the special properties of this expression. The properties such as document-ordering are the same as
      * the properties of the grouping population as a whole.
      *
-     * @return {@link net.sf.saxon.expr.StaticProperty#NON_CREATIVE} (unless the variable is assignable using saxon:assign)
+     * @return {@link net.sf.saxon.expr.StaticProperty#NO_NODES_NEWLY_CREATED} (unless the variable is assignable using saxon:assign)
      */
     @Override
     public int computeSpecialProperties() {
@@ -191,7 +188,7 @@ public class CurrentGroupCall extends Expression implements Callable {
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(XPathContext c) throws XPathException {
+    public SequenceIterator<? extends Item> iterate(XPathContext c) throws XPathException {
         GroupIterator gi = c.getCurrentGroupIterator();
         if (gi == null) {
             XPathException err = new XPathException("There is no current group", "XTDE1061");
@@ -221,20 +218,18 @@ public class CurrentGroupCall extends Expression implements Callable {
      * of the callee to ensure that the type of result conforms to the expected result type.
      * @throws XPathException if a dynamic error occurs during the evaluation of the expression
      */
-    public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
+    public Sequence<? extends Item> call(XPathContext context, Sequence[] arguments) throws XPathException {
         return SequenceTool.toLazySequence(iterate(context));
     }
 
     /**
      * <p>The toString() method for an expression attempts to give a representation of the expression
      * in an XPath-like form.</p>
-     * <p/>
      * <p>For subclasses of Expression that represent XPath expressions, the result should always be a string that
      * parses as an XPath 3.0 expression. The expression produced should be equivalent to the original making certain
      * assumptions about the static context. In general the expansion will make no assumptions about namespace bindings,
      * except that (a) the prefix "xs" is used to refer to the XML Schema namespace, and (b) the default funcion namespace
      * is assumed to be the "fn" namespace.</p>
-     * <p/>
      * <p>In the case of XSLT instructions and XQuery expressions, the toString() method gives an abstracted view of the syntax
      * that is not designed in general to be parseable.</p>
      *
