@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -18,7 +18,7 @@ import net.sf.saxon.expr.parser.Location;
 import net.sf.saxon.expr.parser.PathMap;
 import net.sf.saxon.expr.sort.DocumentOrderIterator;
 import net.sf.saxon.expr.sort.GlobalOrderComparer;
-import net.sf.saxon.lib.FeatureKeys;
+import net.sf.saxon.lib.Feature;
 import net.sf.saxon.lib.ParseOptions;
 import net.sf.saxon.lib.RelativeURIResolver;
 import net.sf.saxon.lib.StandardErrorHandler;
@@ -27,6 +27,7 @@ import net.sf.saxon.style.StylesheetPackage;
 import net.sf.saxon.trans.Err;
 import net.sf.saxon.trans.NonDelegatingURIResolver;
 import net.sf.saxon.trans.XPathException;
+import net.sf.saxon.trans.XsltController;
 import net.sf.saxon.tree.tiny.Statistics;
 import net.sf.saxon.tree.tiny.TinyBuilder;
 import net.sf.saxon.value.Cardinality;
@@ -47,7 +48,7 @@ import java.net.URISyntaxException;
 
 public class DocumentFn extends SystemFunction implements Callable {
 
-    private Location location = null;
+    private Location location;
 
     /**
      * Determine the static cardinality
@@ -67,13 +68,13 @@ public class DocumentFn extends SystemFunction implements Callable {
      * Get the static properties of this expression (other than its type). The result is
      * bit-signficant. These properties are used for optimizations. In general, if
      * property bit is set, it is true, but if it is unset, the value is unknown.
-     * @param arguments
+     * @param arguments the expressions supplied as arguments to the function call
      */
 
     public int getSpecialProperties(Expression[] arguments) {
         return StaticProperty.ORDERED_NODESET |
                 StaticProperty.PEER_NODESET |
-                StaticProperty.NON_CREATIVE;
+                StaticProperty.NO_NODES_NEWLY_CREATED;
         // Declaring it as a peer node-set expression avoids sorting of expressions such as
         // document(XXX)/a/b/c
         // The document() function might appear to be creative: but it isn't, because multiple calls
@@ -256,7 +257,8 @@ public class DocumentFn extends SystemFunction implements Callable {
 
             // check that the document was not written by this transformation
 
-            if (!controller.checkUniqueOutputDestination(documentKey)) {
+            if (controller instanceof XsltController &&
+                    !((XsltController)controller).checkUniqueOutputDestination(documentKey)) {
                 pool.markUnavailable(documentKey);
                 XPathException err = new XPathException(
                         "Cannot read a document that was written during the same transformation: " + documentKey);
@@ -329,6 +331,11 @@ public class DocumentFn extends SystemFunction implements Callable {
                     Sender.send(source, s, options);
                     newdoc = b.getCurrentRoot().getTreeInfo();
                     b.reset();
+                } catch (XPathException err) {
+                    if (err.getErrorCodeLocalPart() == null || err.getErrorCodeLocalPart().equals("SXXP0003")) {
+                        err.setErrorCode("FODC0002");
+                    }
+                    throw err;
                 } finally {
                     if (options.isPleaseCloseAfterUse()) {
                         ParseOptions.close(source);
@@ -347,7 +354,9 @@ public class DocumentFn extends SystemFunction implements Callable {
                     return getFragment(doc, fragmentId, c, locator);
                 }
                 controller.registerDocument(newdoc, documentKey);
-                controller.addUnavailableOutputDestination(documentKey);
+                if (controller instanceof XsltController) {
+                    ((XsltController) controller).addUnavailableOutputDestination(documentKey);
+                }
             }
             return getFragment(newdoc, fragmentId, c, locator);
 
@@ -398,7 +407,7 @@ public class DocumentFn extends SystemFunction implements Callable {
         } catch (Exception ex) {
             //ex.printStackTrace();
             XPathException de = new XPathException("Exception thrown by URIResolver", ex);
-            if (context.getConfiguration().getBooleanProperty(FeatureKeys.TRACE_EXTERNAL_FUNCTIONS)) {
+            if (context.getConfiguration().getBooleanProperty(Feature.TRACE_EXTERNAL_FUNCTIONS)) {
                 ex.printStackTrace();
             }
             throw de;
@@ -483,9 +492,7 @@ public class DocumentFn extends SystemFunction implements Callable {
                 try {
                     URI uri = new URI(baseURI).resolve(href);
                     documentKey = uri.toString();
-                } catch (URISyntaxException err) {
-                    documentKey = baseURI + "/../" + href;
-                } catch (IllegalArgumentException err) {
+                } catch (URISyntaxException | IllegalArgumentException err) {
                     documentKey = baseURI + "/../" + href;
                 }
             }
@@ -572,7 +579,7 @@ public class DocumentFn extends SystemFunction implements Callable {
                     source = r.resolve(href, baseURI);
                 } catch (Exception ex) {
                     XPathException de = new XPathException("Exception thrown by URIResolver", ex);
-                    if (config.getBooleanProperty(FeatureKeys.TRACE_EXTERNAL_FUNCTIONS)) {
+                    if (config.getBooleanProperty(Feature.TRACE_EXTERNAL_FUNCTIONS)) {
                         ex.printStackTrace();
                     }
                     de.setLocator(locator);
