@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -18,9 +18,9 @@ import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.tiny.CharSlice;
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.value.*;
-import net.sf.saxon.value.StringValue;
 
 import java.math.BigDecimal;
+import java.math.RoundingMode;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
@@ -265,13 +265,12 @@ public class FormatNumber extends SystemFunction implements Callable {
         String suffix = "";
         int[] wholePartGroupingPositions = null;
         int[] fractionalPartGroupingPositions = null;
-        boolean is30 = false;
+        boolean regular;
         boolean is31 = false;
 
         public SubPicture(int[] pic, DecimalSymbols dfs) throws XPathException {
 
-            is30 = dfs.getLanguageLevel() >= 30;
-            is31 = dfs.getLanguageLevel() >= 31;
+            is31 = true;
 
             final int percentSign = dfs.getPercent();
             final int perMilleSign = dfs.getPerMille();
@@ -289,7 +288,7 @@ public class FormatNumber extends SystemFunction implements Callable {
             boolean foundExponentSeparator = false;
             boolean foundExponentSeparator2 = false;
             for (int ch : pic) {
-                if (ch == digitSign || ch == zeroDigit || (is30 && isInDigitFamily(ch, zeroDigit))) {
+                if (ch == digitSign || ch == zeroDigit || isInDigitFamily(ch, zeroDigit)) {
                     foundDigit = true;
                     break;
                 }
@@ -299,321 +298,197 @@ public class FormatNumber extends SystemFunction implements Callable {
             }
 
             int phase = 0;
-            if (is31) {
-                // phase = 0: passive characters at start
-                // phase = 1: digit signs in whole part
-                // phase = 2: zero-digit signs in whole part
-                // phase = 3: zero-digit signs in fractional part
-                // phase = 4: digit signs in fractional part
-                // phase = 5: zero-digit signs in exponent part
-                // phase = 6: passive characters at end
 
-                for (int c : pic) {
-                    if (c == percentSign || c == perMilleSign) {
-                        if (isPercent || isPerMille) {
-                            grumble("Cannot have more than one percent or per-mille character in a sub-picture");
-                        }
-                        isPercent = c == percentSign;
-                        isPerMille = c == perMilleSign;
-                        switch (phase) {
-                            case 0:
-                                prefix += unicodeChar(c);
-                                break;
-                            case 1:
-                            case 2:
-                            case 3:
-                            case 4:
-                            case 5:
-                                if (foundExponentSeparator) {
-                                    grumble("Cannot have exponent-separator as well as percent or per-mille character in a sub-picture");
-                                }
-                            case 6:
-                                phase = 6;
-                                suffix += unicodeChar(c);
-                                break;
-                        }
-                    } else if (c == digitSign) {
-                        switch (phase) {
-                            case 0:
-                            case 1:
-                                phase = 1;
-                                maxWholePartSize++;
-                                break;
-                            case 2:
-                                grumble("Digit sign must not appear after a zero-digit sign in the integer part of a sub-picture");
-                                break;
-                            case 3:
-                            case 4:
-                                phase = 4;
-                                maxFractionPartSize++;
-                                break;
-                            case 5:
-                                grumble("Digit sign must not appear in the exponent part of a sub-picture");
-                                break;
-                            case 6:
-                                if (foundExponentSeparator2) {
-                                    grumble("There must only be one exponent separator in a sub-picture");
-                                }
-                                else grumble("Passive character must not appear between active characters in a sub-picture");
-                                break;
-                        }
-                    } else if (c == zeroDigit || (is30 && isInDigitFamily(c, zeroDigit))) {
-                        switch (phase) {
-                            case 0:
-                            case 1:
-                            case 2:
-                                phase = 2;
-                                minWholePartSize++;
-                                maxWholePartSize++;
-                                break;
-                            case 3:
-                                minFractionPartSize++;
-                                maxFractionPartSize++;
-                                break;
-                            case 4:
-                                grumble("Zero digit sign must not appear after a digit sign in the fractional part of a sub-picture");
-                                break;
-                            case 5:
-                                minExponentSize++;
-                                break;
-                            case 6:
-                                if (foundExponentSeparator2) {
-                                    grumble("There must only be one exponent separator in a sub-picture");
-                                }
-                                else grumble("Passive character must not appear between active characters in a sub-picture");
-                                break;
-                        }
-                    } else if (c == decimalSeparator) {
-                        if (foundDecimalSeparator) {
-                            grumble("There must only be one decimal separator in a sub-picture");
-                        }
-                        switch (phase) {
-                            case 0:
-                            case 1:
-                            case 2:
-                                phase = 3;
-                                foundDecimalSeparator = true;
-                                break;
-                            case 3:
-                            case 4:
-                            case 5:
-                                if (foundExponentSeparator) {
-                                    grumble("Decimal separator must not appear in the exponent part of a sub-picture");
-                                }
-                                break;
-                            case 6:
-                                grumble("Decimal separator cannot come after a character in the suffix");
-                                break;
-                        }
-                    } else if (c == groupingSeparator) {
-                        switch (phase) {
-                            case 0:
-                            case 1:
-                            case 2:
-                                if (wholePartPositions == null) {
-                                    wholePartPositions = new ArrayList<Integer>(3);
-                                }
-                                if (wholePartPositions.contains(maxWholePartSize)){
-                                    grumble("Sub-picture cannot contain adjacent grouping separators");
-                                }
-                                wholePartPositions.add(maxWholePartSize);
-                                // note these are positions from a false offset, they will be corrected later
-                                break;
-                            case 3:
-                            case 4:
-                                if (maxFractionPartSize == 0) {
-                                    grumble("Grouping separator cannot be adjacent to decimal separator");
-                                }
-                                if (fractionalPartPositions == null) {
-                                    fractionalPartPositions = new ArrayList<Integer>(3);
-                                }
-                                if (fractionalPartPositions.contains(maxFractionPartSize)){
-                                    grumble("Sub-picture cannot contain adjacent grouping separators");
-                                }
-                                fractionalPartPositions.add(maxFractionPartSize);
-                                break;
-                            case 5:
-                                if (foundExponentSeparator) {
-                                    grumble("Grouping separator must not appear in the exponent part of a sub-picture");
-                                }
-                                break;
-                            case 6:
-                                grumble("Grouping separator found in suffix of sub-picture");
-                                break;
-                        }
-                    } else if (c == exponentSeparator) {
-                        switch (phase) {
-                            case 0:
-                                prefix += unicodeChar(c);
-                                break;
-                            case 1:
-                            case 2:
-                            case 3:
-                            case 4:
-                                phase = 5;
-                                foundExponentSeparator = true;
-                                break;
-                            case 5:
-                                if (foundExponentSeparator) {
-                                    foundExponentSeparator2 = true;
-                                    phase = 6;
-                                    suffix += unicodeChar(exponentSeparator);
-                                }
-                                break;
-                            case 6:
-                                suffix += unicodeChar(c);
-                                break;
-                        }
-                    } else {    // passive character found
-                        switch (phase) {
-                            case 0:
-                                prefix += unicodeChar(c);
-                                break;
-                            case 1:
-                            case 2:
-                            case 3:
-                            case 4:
-                            case 5:
-                                if (minExponentSize == 0 && foundExponentSeparator) {
-                                    phase = 6;
-                                    suffix += unicodeChar(exponentSeparator);
-                                    suffix += unicodeChar(c);
-                                    break;
-                                }
-                            case 6:
-                                phase = 6;
-                                suffix += unicodeChar(c);
-                                break;
-                        }
+            // phase = 0: passive characters at start
+            // phase = 1: digit signs in whole part
+            // phase = 2: zero-digit signs in whole part
+            // phase = 3: zero-digit signs in fractional part
+            // phase = 4: digit signs in fractional part
+            // phase = 5: zero-digit signs in exponent part
+            // phase = 6: passive characters at end
+
+            for (int c : pic) {
+                if (c == percentSign || c == perMilleSign) {
+                    if (isPercent || isPerMille) {
+                        grumble("Cannot have more than one percent or per-mille character in a sub-picture");
                     }
-                }
-            } else { // TODO - if XPath 3.0 is no longer supported, this branch could go
-                // phase = 0: passive characters at start
-                // phase = 1: digit signs in whole part
-                // phase = 2: zero-digit signs in whole part
-                // phase = 3: zero-digit signs in fractional part
-                // phase = 4: digit signs in fractional part
-                // phase = 5: passive characters at end
-
-                for (int c : pic) {
-                    if (c == percentSign || c == perMilleSign) {
-                        if (isPercent || isPerMille) {
-                            grumble("Cannot have more than one percent or per-mille character in a sub-picture");
-                        }
-                        isPercent = c == percentSign;
-                        isPerMille = c == perMilleSign;
-                        switch (phase) {
-                            case 0:
-                                prefix += unicodeChar(c);
-                                break;
-                            case 1:
-                            case 2:
-                            case 3:
-                            case 4:
-                            case 5:
-                                phase = 5;
+                    isPercent = c == percentSign;
+                    isPerMille = c == perMilleSign;
+                    switch (phase) {
+                        case 0:
+                            prefix += unicodeChar(c);
+                            break;
+                        case 1:
+                        case 2:
+                        case 3:
+                        case 4:
+                        case 5:
+                            if (foundExponentSeparator) {
+                                grumble("Cannot have exponent-separator as well as percent or per-mille character in a sub-picture");
+                            }
+                        case 6:
+                            phase = 6;
+                            suffix += unicodeChar(c);
+                            break;
+                    }
+                } else if (c == digitSign) {
+                    switch (phase) {
+                        case 0:
+                        case 1:
+                            phase = 1;
+                            maxWholePartSize++;
+                            break;
+                        case 2:
+                            grumble("Digit sign must not appear after a zero-digit sign in the integer part of a sub-picture");
+                            break;
+                        case 3:
+                        case 4:
+                            phase = 4;
+                            maxFractionPartSize++;
+                            break;
+                        case 5:
+                            grumble("Digit sign must not appear in the exponent part of a sub-picture");
+                            break;
+                        case 6:
+                            if (foundExponentSeparator2) {
+                                grumble("There must only be one exponent separator in a sub-picture");
+                            } else {
+                                grumble("Passive character must not appear between active characters in a sub-picture");
+                            }
+                            break;
+                    }
+                } else if (c == zeroDigit || isInDigitFamily(c, zeroDigit)) {
+                    switch (phase) {
+                        case 0:
+                        case 1:
+                        case 2:
+                            phase = 2;
+                            minWholePartSize++;
+                            maxWholePartSize++;
+                            break;
+                        case 3:
+                            minFractionPartSize++;
+                            maxFractionPartSize++;
+                            break;
+                        case 4:
+                            grumble("Zero digit sign must not appear after a digit sign in the fractional part of a sub-picture");
+                            break;
+                        case 5:
+                            minExponentSize++;
+                            break;
+                        case 6:
+                            if (foundExponentSeparator2) {
+                                grumble("There must only be one exponent separator in a sub-picture");
+                            } else {
+                                grumble("Passive character must not appear between active characters in a sub-picture");
+                            }
+                            break;
+                    }
+                } else if (c == decimalSeparator) {
+                    if (foundDecimalSeparator) {
+                        grumble("There must only be one decimal separator in a sub-picture");
+                    }
+                    switch (phase) {
+                        case 0:
+                        case 1:
+                        case 2:
+                            phase = 3;
+                            foundDecimalSeparator = true;
+                            break;
+                        case 3:
+                        case 4:
+                        case 5:
+                            if (foundExponentSeparator) {
+                                grumble("Decimal separator must not appear in the exponent part of a sub-picture");
+                            }
+                            break;
+                        case 6:
+                            grumble("Decimal separator cannot come after a character in the suffix");
+                            break;
+                    }
+                } else if (c == groupingSeparator) {
+                    switch (phase) {
+                        case 0:
+                        case 1:
+                        case 2:
+                            if (wholePartPositions == null) {
+                                wholePartPositions = new ArrayList<>(3);
+                            }
+                            if (wholePartPositions.contains(maxWholePartSize)) {
+                                grumble("Sub-picture cannot contain adjacent grouping separators");
+                            }
+                            wholePartPositions.add(maxWholePartSize);
+                            // note these are positions from a false offset, they will be corrected later
+                            break;
+                        case 3:
+                        case 4:
+                            if (maxFractionPartSize == 0) {
+                                grumble("Grouping separator cannot be adjacent to decimal separator");
+                            }
+                            if (fractionalPartPositions == null) {
+                                fractionalPartPositions = new ArrayList<Integer>(3);
+                            }
+                            if (fractionalPartPositions.contains(maxFractionPartSize)) {
+                                grumble("Sub-picture cannot contain adjacent grouping separators");
+                            }
+                            fractionalPartPositions.add(maxFractionPartSize);
+                            break;
+                        case 5:
+                            if (foundExponentSeparator) {
+                                grumble("Grouping separator must not appear in the exponent part of a sub-picture");
+                            }
+                            break;
+                        case 6:
+                            grumble("Grouping separator found in suffix of sub-picture");
+                            break;
+                    }
+                } else if (c == exponentSeparator) {
+                    switch (phase) {
+                        case 0:
+                            prefix += unicodeChar(c);
+                            break;
+                        case 1:
+                        case 2:
+                        case 3:
+                        case 4:
+                            phase = 5;
+                            foundExponentSeparator = true;
+                            break;
+                        case 5:
+                            if (foundExponentSeparator) {
+                                foundExponentSeparator2 = true;
+                                phase = 6;
+                                suffix += unicodeChar(exponentSeparator);
+                            }
+                            break;
+                        case 6:
+                            suffix += unicodeChar(c);
+                            break;
+                    }
+                } else {    // passive character found
+                    switch (phase) {
+                        case 0:
+                            prefix += unicodeChar(c);
+                            break;
+                        case 1:
+                        case 2:
+                        case 3:
+                        case 4:
+                        case 5:
+                            if (minExponentSize == 0 && foundExponentSeparator) {
+                                phase = 6;
+                                suffix += unicodeChar(exponentSeparator);
                                 suffix += unicodeChar(c);
                                 break;
-                        }
-                    } else if (c == digitSign) {
-                        switch (phase) {
-                            case 0:
-                            case 1:
-                                phase = 1;
-                                maxWholePartSize++;
-                                break;
-                            case 2:
-                                grumble("Digit sign must not appear after a zero-digit sign in the integer part of a sub-picture");
-                                break;
-                            case 3:
-                            case 4:
-                                phase = 4;
-                                maxFractionPartSize++;
-                                break;
-                            case 5:
-                                grumble("Passive character must not appear between active characters in a sub-picture");
-                                break;
-                        }
-                    } else if (c == zeroDigit || (is30 && isInDigitFamily(c, zeroDigit))) {
-                        switch (phase) {
-                            case 0:
-                            case 1:
-                            case 2:
-                                phase = 2;
-                                minWholePartSize++;
-                                maxWholePartSize++;
-                                break;
-                            case 3:
-                                minFractionPartSize++;
-                                maxFractionPartSize++;
-                                break;
-                            case 4:
-                                grumble("Zero digit sign must not appear after a digit sign in the fractional part of a sub-picture");
-                                break;
-                            case 5:
-                                grumble("Passive character must not appear between active characters in a sub-picture");
-                                break;
-                        }
-                    } else if (c == decimalSeparator) {
-                        switch (phase) {
-                            case 0:
-                            case 1:
-                            case 2:
-                                phase = 3;
-                                foundDecimalSeparator = true;
-                                break;
-                            case 3:
-                            case 4:
-                            case 5:
-                                if (foundDecimalSeparator) {
-                                    grumble("There must only be one decimal separator in a sub-picture");
-                                } else {
-                                    grumble("Decimal separator cannot come after a character in the suffix");
-                                }
-                                break;
-                        }
-                    } else if (c == groupingSeparator) {
-                        switch (phase) {
-                            case 0:
-                            case 1:
-                            case 2:
-                                if (wholePartPositions == null) {
-                                    wholePartPositions = new ArrayList<Integer>(3);
-                                }
-                                wholePartPositions.add(maxWholePartSize);
-                                // note these are positions from a false offset, they will be corrected later
-                                break;
-                            case 3:
-                            case 4:
-                                if (maxFractionPartSize == 0) {
-                                    grumble("Grouping separator cannot be adjacent to decimal separator");
-                                }
-                                if (fractionalPartPositions == null) {
-                                    fractionalPartPositions = new ArrayList<Integer>(3);
-                                }
-                                fractionalPartPositions.add(maxFractionPartSize);
-                                break;
-                            case 5:
-                                grumble("Grouping separator found in suffix of sub-picture");
-                                break;
-                        }
-                    } else {    // passive character found
-                        switch (phase) {
-                            case 0:
-                                prefix += unicodeChar(c);
-                                break;
-                            case 1:
-                            case 2:
-                            case 3:
-                            case 4:
-                            case 5:
-                                phase = 5;
-                                suffix += unicodeChar(c);
-                                break;
-                        }
+                            }
+                        case 6:
+                            phase = 6;
+                            suffix += unicodeChar(c);
+                            break;
                     }
                 }
             }
+
             /* 4.7.4 Rule 3 */
             scalingFactor = minWholePartSize;
 
@@ -623,7 +498,7 @@ public class FormatNumber extends SystemFunction implements Callable {
 
             /* 4.7.4 Rule 8 */
             if (minWholePartSize == 0 && maxFractionPartSize == 0) { //minWholePartSize == 0 && !foundDecimalSeparator
-                if (minExponentSize != 0){
+                if (minExponentSize != 0) {
                     minFractionPartSize = 1;
                     maxFractionPartSize = 1;
                 } else {
@@ -631,11 +506,11 @@ public class FormatNumber extends SystemFunction implements Callable {
                 }
             }
             /* 4.7.4 Rule 9 */
-            if (minExponentSize != 0 && minWholePartSize == 0 && maxWholePartSize != 0){
+            if (minExponentSize != 0 && minWholePartSize == 0 && maxWholePartSize != 0) {
                 minWholePartSize = 1;
             }
             /* 4.7.4 Rule 10 */
-            if (minWholePartSize == 0 && minFractionPartSize == 0){
+            if (minWholePartSize == 0 && minFractionPartSize == 0) {
                 minFractionPartSize = 1;
             }
 
@@ -654,16 +529,18 @@ public class FormatNumber extends SystemFunction implements Callable {
                     wholePartGroupingPositions[i] =
                             maxWholePartSize - wholePartPositions.get(n - i - 1);
                 }
-                if (n > 1) {
-                    boolean regular = true;
+                if (n == 1) {
+                    regular = wholePartGroupingPositions[0] * 2 >= maxWholePartSize;
+                } else if (n > 1) {
+                    regular = true;
                     int first = wholePartGroupingPositions[0];
                     for (int i = 1; i < n; i++) {
-                        if (wholePartGroupingPositions[i] != (i+1) * first) {
+                        if (wholePartGroupingPositions[i] != (i + 1) * first) {
                             regular = false;
                             break;
                         }
                     }
-                    if (regular && (maxWholePartSize - wholePartGroupingPositions[n-1] > first)) {
+                    if (regular && (maxWholePartSize - wholePartGroupingPositions[n - 1] > first)) {
                         regular = false;
                     }
                     if (regular) {
@@ -713,7 +590,7 @@ public class FormatNumber extends SystemFunction implements Callable {
             if (multiplier != 1) {
                 try {
                     value = (NumericValue) ArithmeticExpression.compute(
-                        value, Calculator.TIMES, new Int64Value(multiplier), null);
+                            value, Calculator.TIMES, new Int64Value(multiplier), null);
                 } catch (XPathException e) {
                     value = new DoubleValue(Double.POSITIVE_INFINITY);
                 }
@@ -725,7 +602,6 @@ public class FormatNumber extends SystemFunction implements Callable {
             }
 
 
-
             FastStringBuffer sb = new FastStringBuffer(FastStringBuffer.C16);
             if (value instanceof DoubleValue || value instanceof FloatValue) {
                 BigDecimal dec = adjustToDecimal(value.getDoubleValue(), 2);
@@ -734,7 +610,7 @@ public class FormatNumber extends SystemFunction implements Callable {
 
             } else if (value instanceof IntegerValue) {
                 if (minExponentSize != 0) {
-                    formatDecimal(((IntegerValue)value).getDecimalValue(), sb);
+                    formatDecimal(((IntegerValue) value).getDecimalValue(), sb);
                 } else {
                     formatInteger(value, sb);
                 }
@@ -786,7 +662,7 @@ public class FormatNumber extends SystemFunction implements Callable {
             // Add the whole-part grouping separators
 
             if (wholePartGroupingPositions != null) {
-                if (wholePartGroupingPositions.length == 1) {
+                if (regular) {
                     // grouping separators are at regular positions
                     int g = wholePartGroupingPositions[0];
                     int p = point - g;
@@ -850,11 +726,11 @@ public class FormatNumber extends SystemFunction implements Callable {
                 minWholePartSize = 1;
             }*/ // this bit of logic is now included in the SubPicture class
             if (minExponentSize == 0) {
-                dval = dval.setScale(maxFractionPartSize, BigDecimal.ROUND_HALF_EVEN);
+                dval = dval.setScale(maxFractionPartSize, RoundingMode.HALF_EVEN);
             } else {
                 exponent = dval.precision() - dval.scale() - scalingFactor;
                 dval = dval.movePointLeft(exponent);
-                dval = dval.setScale(maxFractionPartSize, BigDecimal.ROUND_HALF_EVEN);
+                dval = dval.setScale(maxFractionPartSize, RoundingMode.HALF_EVEN);
             }
             BigDecimalValue.decimalToString(dval, fsb);
 
@@ -1043,7 +919,7 @@ public class FormatNumber extends SystemFunction implements Callable {
         StructuredQName qName;
         try {
             qName = StructuredQName.fromLexicalQName(lexicalName, false,
-                    true, getRetainedStaticContext());
+                                                     true, getRetainedStaticContext());
         } catch (XPathException e) {
             XPathException err = new XPathException("Invalid decimal format name. " + e.getMessage());
             err.setErrorCode("FODF1280");
@@ -1073,6 +949,7 @@ public class FormatNumber extends SystemFunction implements Callable {
 
     /**
      * Format a double as required by the adaptive serialization method
+     *
      * @param value the value to be formatted
      * @return the formatted value
      */
