@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -12,7 +12,6 @@ import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.expr.parser.ContextItemStaticInfo;
 import net.sf.saxon.expr.parser.ExpressionVisitor;
 import net.sf.saxon.expr.sort.AtomicComparer;
-import net.sf.saxon.om.Item;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.om.SequenceIterator;
 import net.sf.saxon.om.SequenceTool;
@@ -53,15 +52,15 @@ public class IndexOf extends CollatingFunctionFixed  {
     }
 
     @Override
-    public void supplyTypeInformation(ExpressionVisitor visitor, ContextItemStaticInfo contextItemType, Expression[] arguments) throws XPathException {
+    public void supplyTypeInformation(ExpressionVisitor visitor, ContextItemStaticInfo contextItemType, Expression[] arguments) {
         ItemType type0 = arguments[0].getItemType();
         ItemType type1 = arguments[1].getItemType();
         if (type0 instanceof AtomicType && type1 instanceof AtomicType) {
-            preAllocateComparer((AtomicType) type0, (AtomicType) type1, visitor.getStaticContext(), false);
+            preAllocateComparer((AtomicType) type0, (AtomicType) type1, visitor.getStaticContext());
         }
     }
 
-    private static class IndexIterator implements SequenceIterator {
+    private static class IndexIterator implements SequenceIterator<Int64Value> {
         private int index = 0;
         private SequenceIterator base;
         private BuiltInAtomicType searchType;
@@ -80,7 +79,6 @@ public class IndexOf extends CollatingFunctionFixed  {
          * does not require any more items to be delivered by the iterator. This may enable the
          * supplier to release resources. After calling close(), no further calls on the
          * iterator should be made; if further calls are made, the effect of such calls is undefined.
-         * <p/>
          * <p>(Currently, closing an iterator is important only when the data is being "pushed" in
          * another thread. Closing the iterator terminates that thread and means that it needs to do
          * no additional work. Indeed, failing to close the iterator may cause the push thread to hang
@@ -104,7 +102,7 @@ public class IndexOf extends CollatingFunctionFixed  {
          * @throws net.sf.saxon.trans.XPathException if an error occurs retrieving the next item
          * @since 8.4
          */
-        public Item next() throws XPathException {
+        public Int64Value next() throws XPathException {
             AtomicValue baseItem;
             while ((baseItem = (AtomicValue) base.next()) != null) {
                 index++;
