@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -45,13 +45,13 @@ public class Innermost extends SystemFunction {
      * @throws net.sf.saxon.trans.XPathException
      *          if a dynamic error occurs during the evaluation of the expression
      */
-    public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
+    public Sequence<? extends NodeInfo> call(XPathContext context, Sequence[] arguments) throws XPathException {
         return SequenceTool.toLazySequence(innermost(arguments[0].iterate()));
     }
 
-    public SequenceIterator innermost(SequenceIterator in) throws XPathException {
+    public SequenceIterator<? extends NodeInfo> innermost(SequenceIterator<? extends NodeInfo> in) throws XPathException {
         if (!presorted) {
-            in = new DocumentOrderIterator(in, GlobalOrderComparer.getInstance());
+            in = new DocumentOrderIterator<>(in, GlobalOrderComparer.getInstance());
         }
         return new InnermostIterator(in);
     }
@@ -77,18 +77,17 @@ public class Innermost extends SystemFunction {
 
     /**
      * Inner class implementing the logic in the form of an iterator.
-     * <p/>
-     * The principle behind the algorithm is the assertion that in a sorted input
+     * <p>The principle behind the algorithm is the assertion that in a sorted input
      * sequence, if a node A is immediately followed by a descendant B, then we can
      * discard A; if it is immediately followed by a node B that is not a descendant,
      * then we can emit A. So there is a single-node lookahead. The last node in the input
      * sequence is always emitted. (Note, "descendant" is used loosely here to
-     * include attributes and namespaces.)
+     * include attributes and namespaces.)</p>
      */
 
-    private class InnermostIterator implements SequenceIterator {
+    private class InnermostIterator implements SequenceIterator<NodeInfo> {
 
-        SequenceIterator in;
+        SequenceIterator<? extends NodeInfo> in;
         NodeInfo pending = null;
         int position = 0;
 
@@ -100,9 +99,9 @@ public class Innermost extends SystemFunction {
          * @throws XPathException if an error occurs evaluating the input iterator
          */
 
-        public InnermostIterator(SequenceIterator in) throws XPathException {
+        public InnermostIterator(SequenceIterator<? extends NodeInfo> in) throws XPathException {
             this.in = in;
-            pending = (NodeInfo)in.next();
+            pending = in.next();
         }
 
         public NodeInfo next() throws XPathException {
@@ -112,7 +111,7 @@ public class Innermost extends SystemFunction {
                 return null;
             } else {
                 while (true) {
-                    NodeInfo next = (NodeInfo)in.next();
+                    NodeInfo next = in.next();
                     if (next == null) {
                         NodeInfo current = pending;
                         position++;
