@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -17,7 +17,6 @@ import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.type.AnyItemType;
 import net.sf.saxon.type.ItemType;
-import net.sf.saxon.type.Type;
 import net.sf.saxon.type.TypeHierarchy;
 import net.sf.saxon.value.SequenceType;
 
@@ -60,11 +59,10 @@ public class IntegratedFunctionCall extends FunctionCall implements Callable {
      * Get the target function to be called
      *
      * @param context the dynamic evaluation context
-     * @return the target function
-     * @throws net.sf.saxon.trans.XPathException if the target function cannot be determined
+     * @return the target function, or null if unknown
      */
     @Override
-    public Function getTargetFunction(XPathContext context) throws XPathException {
+    public Function getTargetFunction(XPathContext context) {
         return null;
     }
 
@@ -142,15 +140,14 @@ public class IntegratedFunctionCall extends FunctionCall implements Callable {
     }
 
     /**
-     * Pre-evaluate a function at compile time. Functions that do not allow
-     * pre-evaluation, or that need access to context information, can override this method.
+     * Pre-evaluate a function at compile time. This function does not allow
+     * pre-evaluation, so the method returns null.
      *
      * @param visitor an expression visitor
-     * @return the result of the early evaluation, or the original expression, or potentially
-     *         a simplified expression
+     * @return for this class: always null
      */
 
-    public Expression preEvaluate(ExpressionVisitor visitor) throws XPathException {
+    public Expression preEvaluate(ExpressionVisitor visitor) {
         return this;
     }
 
@@ -159,7 +156,6 @@ public class IntegratedFunctionCall extends FunctionCall implements Callable {
      * sequences, in general; this method determines the type of the items within the
      * sequence, assuming that (a) this is known in advance, and (b) it is the same for
      * all items in the sequence.
-     * <p/>
      * <p>This method should always return a result, though it may be the best approximation
      * that is available at the time.</p>
      *
@@ -210,14 +206,14 @@ public class IntegratedFunctionCall extends FunctionCall implements Callable {
 
     protected int computeSpecialProperties() {
         ExtensionFunctionDefinition definition = function.getDefinition();
-        return definition.hasSideEffects() ? StaticProperty.HAS_SIDE_EFFECTS : StaticProperty.NON_CREATIVE;
+        return definition.hasSideEffects() ? StaticProperty.HAS_SIDE_EFFECTS : StaticProperty.NO_NODES_NEWLY_CREATED;
     }
 
     /**
      * Copy an expression. This makes a deep copy.
      *
      * @return the copy of the original expression
-     * @param rebindings
+     * @param rebindings Variables that need to be re-bound
      */
 
     /*@NotNull*/
@@ -244,7 +240,7 @@ public class IntegratedFunctionCall extends FunctionCall implements Callable {
 
     public void export(ExpressionPresenter out) throws XPathException {
         out.startElement("ifCall", this);
-        out.emitAttribute("name", getFunctionName().getEQName());
+        out.emitAttribute("name", getFunctionName());
         out.emitAttribute("type", resultType.toExportString());
         for (Operand o : operands()) {
             o.getChildExpression().export(out);
@@ -269,7 +265,7 @@ public class IntegratedFunctionCall extends FunctionCall implements Callable {
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(final XPathContext context) throws XPathException {
+    public SequenceIterator<? extends Item> iterate(final XPathContext context) throws XPathException {
         ExtensionFunctionDefinition definition = function.getDefinition();
         Sequence[] argValues = new Sequence[getArity()];
         for (int i = 0; i < argValues.length; i++) {
@@ -279,7 +275,7 @@ public class IntegratedFunctionCall extends FunctionCall implements Callable {
         final Configuration config = context.getConfiguration();
         final TypeHierarchy th = config.getTypeHierarchy();
 
-        SequenceIterator result;
+        SequenceIterator<? extends Item> result;
         try {
             result = function.call(context, argValues).iterate();
         } catch (XPathException e) {
@@ -289,31 +285,20 @@ public class IntegratedFunctionCall extends FunctionCall implements Callable {
         if (!definition.trustResultType()) {
             int card = resultType.getCardinality();
             if (card != StaticProperty.ALLOWS_ZERO_OR_MORE) {
-                result = new CardinalityCheckingIterator(result,
-                        card,
-                        role,
-                        getLocation());
+                result = new CardinalityCheckingIterator<>(result, card, role, getLocation());
             }
             final ItemType type = resultType.getPrimaryType();
             if (type != AnyItemType.getInstance()) {
-                result = new ItemMappingIterator(result,
-                        new ItemMappingFunction() {
-                            public Item mapItem(Item item) throws XPathException {
-                                if (!type.matches(item, th)) {
-                                    String msg = "Item returned by integrated function " +
-                                            getFunctionName().getDisplayName() +
-                                            "() is not of declared item type. Actual type is " +
-                                            Type.getItemType(item, config.getTypeHierarchy()).toString() +
-                                            "; expected type is " + type.toString();
-                                    XPathException err = new XPathException(
-                                            msg);
-                                    err.setErrorCode("XPTY0004");
-                                    err.setLocation(getLocation());
-                                    throw err;
-                                }
-                                return item;
-                            }
-                        }, true);
+                result = new ItemMappingIterator<>(result,
+                                                   (ItemMappingFunction) item -> {
+                                                       if (!type.matches(item, th)) {
+                                                           String msg = role.composeErrorMessage(type, item, th);
+                                                           XPathException err = new XPathException(msg, "XPTY0004");
+                                                           err.setLocation(getLocation());
+                                                           throw err;
+                                                       }
+                                                       return item;
+                                                   }, true);
             }
         }
         return result;
@@ -344,7 +329,7 @@ public class IntegratedFunctionCall extends FunctionCall implements Callable {
         }
     }
 
-    public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
+    public Sequence<? extends Item> call(XPathContext context, Sequence[] arguments) throws XPathException {
         return function.call(context, arguments);
     }
 
