@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -66,19 +66,19 @@ public class Remove extends SystemFunction {
      * at a specified position.
      */
 
-    public static class RemoveIterator implements SequenceIterator, LastPositionFinder {
+    public static class RemoveIterator<T extends Item> implements SequenceIterator<T>, LastPositionFinder {
 
-        SequenceIterator base;
+        SequenceIterator<T> base;
         int removePosition;
         int basePosition = 0;
-        Item current = null;
+        T current = null;
 
-        public RemoveIterator(SequenceIterator base, int removePosition) {
+        public RemoveIterator(SequenceIterator<T> base, int removePosition) {
             this.base = base;
             this.removePosition = removePosition;
         }
 
-        public Item next() throws XPathException {
+        public T next() throws XPathException {
             current = base.next();
             basePosition++;
             if (current != null && basePosition == removePosition) {
