@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -12,6 +12,7 @@ import net.sf.saxon.expr.StaticProperty;
 import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.expr.parser.ContextItemStaticInfo;
 import net.sf.saxon.expr.parser.ExpressionVisitor;
+import net.sf.saxon.om.Item;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.om.SequenceIterator;
 import net.sf.saxon.om.SequenceTool;
@@ -42,7 +43,6 @@ public class Reverse extends SystemFunction {
 
 //    /**
 //     * Perform optimisation of an expression and its subexpressions.
-//     * <p/>
 //     * <p>This method is called after all references to functions and variables have been resolved
 //     * to the declaration of the function or variable, and after all type checking has been done.</p>
 //     *
@@ -70,11 +70,11 @@ public class Reverse extends SystemFunction {
 //    }
 
 
-    public static SequenceIterator getReverseIterator(SequenceIterator forwards) throws XPathException {
+    public static <T extends Item> SequenceIterator<T> getReverseIterator(SequenceIterator<T> forwards) throws XPathException {
         if (forwards instanceof ReversibleIterator) {
-            return ((ReversibleIterator) forwards).getReverseIterator();
+            return ((ReversibleIterator<T>) forwards).getReverseIterator();
         } else {
-            SequenceExtent extent = new SequenceExtent(forwards);
+            SequenceExtent<T> extent = new SequenceExtent<>(forwards);
             return extent.reverseIterate();
         }
     }
