@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,7 +8,8 @@
 package net.sf.saxon.functions;
 
 import net.sf.saxon.event.PipelineConfiguration;
-import net.sf.saxon.event.SequenceReceiver;
+import net.sf.saxon.event.Receiver;
+import net.sf.saxon.event.SequenceCopier;
 import net.sf.saxon.expr.Callable;
 import net.sf.saxon.expr.StaticProperty;
 import net.sf.saxon.expr.XPathContext;
@@ -25,6 +26,7 @@ import net.sf.saxon.om.*;
 import net.sf.saxon.regex.UnicodeString;
 import net.sf.saxon.serialize.CharacterMap;
 import net.sf.saxon.serialize.CharacterMapIndex;
+import net.sf.saxon.serialize.SerializationProperties;
 import net.sf.saxon.serialize.SerializationParamsHandler;
 import net.sf.saxon.trans.Err;
 import net.sf.saxon.trans.XPathException;
@@ -87,6 +89,7 @@ public class Serialize extends SystemFunction implements Callable {
         op.addAllowedOption(sx("indent-spaces"), SequenceType.SINGLE_INTEGER); //integer
         op.addAllowedOption(sx("line-length"), SequenceType.SINGLE_INTEGER); //integer
         //requiredTypes.put("next-in-chain", SequenceType.SINGLE_STRING); //uri
+        op.addAllowedOption(sx("property-order"), listOfQNames); 
         op.addAllowedOption(sx("recognize-binary"), SequenceType.SINGLE_BOOLEAN); //boolean
         op.addAllowedOption(sx("require-well-formed"), SequenceType.SINGLE_BOOLEAN); //boolean
         op.addAllowedOption(sx("single-quotes"), SequenceType.SINGLE_BOOLEAN); //boolean
@@ -116,7 +119,7 @@ public class Serialize extends SystemFunction implements Callable {
 
     private String[] paramNamesSaxon = new String[]{
             "attribute-order", "character-representation", "double-space", "indent-spaces", "line-length",
-            /*"next-in-chain",*/ "recognize-binary", "require-well-formed", "single-quotes",
+            /*"next-in-chain",*/ "property-order", "recognize-binary", "require-well-formed", "single-quotes",
             "supply-source-locator", "suppress-indentation"
     };
 
@@ -129,7 +132,7 @@ public class Serialize extends SystemFunction implements Callable {
         return false;
     }
 
-    private final static Map<String, SequenceType> requiredTypes = new HashMap<String, SequenceType>(40);
+    private final static Map<String, SequenceType> requiredTypes = new HashMap<>(40);
 
     static {
         requiredTypes.put("allow-duplicate-names", SequenceType.SINGLE_BOOLEAN); //yes-no-param-type
@@ -161,7 +164,7 @@ public class Serialize extends SystemFunction implements Callable {
         requiredTypes.put("version", SequenceType.SINGLE_STRING); //string-param-type
     }
 
-    private final static Map<String, SequenceType> requiredTypesSaxon = new HashMap<String, SequenceType>(20);
+    private final static Map<String, SequenceType> requiredTypesSaxon = new HashMap<>(20);
 
     static {
         requiredTypesSaxon.put("attribute-order", BuiltInAtomicType.QNAME.zeroOrMore());
@@ -202,8 +205,7 @@ public class Serialize extends SystemFunction implements Callable {
                     // parameter, this results in the fn:serialize call failing with a dynamic error.
                     Sequence converted = th.applyFunctionConversionRules(
                         map.get(key), requiredTypes.get(keyName), role, ExplicitLocation.UNKNOWN_LOCATION);
-                    converted = SequenceTool.toGroundedValue(converted);
-                    result = result.addEntry(key, converted);
+                    result = result.addEntry(key, converted.materialize());
                 }
             } else if (key instanceof QNameValue) {
                 if (key.getComponent(AccessorFn.Component.NAMESPACE).getStringValue().equals("")) {
@@ -215,8 +217,7 @@ public class Serialize extends SystemFunction implements Callable {
                         RoleDiagnostic role = new RoleDiagnostic(RoleDiagnostic.OPTION, keyName, 0);
                         Sequence converted = th.applyFunctionConversionRules(
                                 map.get(key), requiredTypesSaxon.get(keyName), role, ExplicitLocation.UNKNOWN_LOCATION);
-                        converted = SequenceTool.toGroundedValue(converted);
-                        result = result.addEntry(key, converted);
+                        result = result.addEntry(key, converted.materialize());
                     }
                 }
                 // Implementation-defined serialization parameters in an unrecognised namespace are ignored.
@@ -258,12 +259,15 @@ public class Serialize extends SystemFunction implements Callable {
     private String toQNamesTypeString(Sequence seqVal) throws XPathException {
         SequenceIterator iterator = seqVal.iterate();
         Item item;
-        String stringVal = "";
+        StringBuilder stringVal = new StringBuilder();
         while ((item = iterator.next()) != null) {
             QNameValue qNameValue = (QNameValue) item;
-            stringVal = stringVal + " {" + qNameValue.getComponent(AccessorFn.Component.NAMESPACE).getStringValue() + '}' + qNameValue.getComponent(AccessorFn.Component.LOCALNAME).getStringValue();
+            stringVal.append(" {")
+                    .append(qNameValue.getComponent(AccessorFn.Component.NAMESPACE).getStringValue())
+                    .append('}')
+                    .append(qNameValue.getComponent(AccessorFn.Component.LOCALNAME).getStringValue());
         }
-        return stringVal;
+        return stringVal.toString();
     }
 
     // Convert a QName or string value to a method-type (or json-node-output-method-type) string.
@@ -288,10 +292,14 @@ public class Serialize extends SystemFunction implements Callable {
 
     private static MapItem checkCharacterMapOptions(MapItem map, XPathContext context) throws XPathException {
         TypeHierarchy th = context.getConfiguration().getTypeHierarchy();
-        for (KeyValuePair pair : map) {
+        for (KeyValuePair pair : map.keyValuePairs()) {
             AtomicValue key = pair.key;
-            if (!(key instanceof StringValue && ((StringValue) key).getStringLength() == 1)) {
-                throw new XPathException("Keys in a character map must all be one-character strings. Found " + Err.wrap(key.toString()), "XPTY0004");
+            if (!(key instanceof StringValue)) {
+                throw new XPathException(
+                        "Keys in a character map must all be strings. Found a value of type " + key.getItemType(), "XPTY0004");
+            }
+            if (((StringValue) key).getStringLength() != 1) {
+                throw new XPathException("Keys in a character map must all be one-character strings. Found " + Err.wrap(key.toString()), "SEPM0016");
             }
             if (!SequenceType.SINGLE_STRING.matches(pair.value, th)) {
                 throw new XPathException("Values in a character map must all be single strings. Found " + Err.wrap(key.toString()), "XPTY0004");
@@ -310,7 +318,7 @@ public class Serialize extends SystemFunction implements Callable {
     public static CharacterMap toCharacterMap(MapItem charMap) throws XPathException {
         AtomicIterator iterator = charMap.keys();
         AtomicValue charKey;
-        IntHashMap<String> intHashMap = new IntHashMap<String>();
+        IntHashMap<String> intHashMap = new IntHashMap<>();
         while ((charKey = iterator.next()) != null) {
             String ch = charKey.getStringValue();
             String str = charMap.get(charKey).head().getStringValue();
@@ -327,19 +335,14 @@ public class Serialize extends SystemFunction implements Callable {
             }
         }
         StructuredQName name = new StructuredQName("output", NamespaceConstant.OUTPUT, "serialization-parameters");
-        CharacterMap characterMap = new CharacterMap(name, intHashMap);
-        return characterMap;
+        return new CharacterMap(name, intHashMap);
     }
 
-    // Just set properties using props.setProperty("allow-duplicate-names", stringVal); as below?
-    // or should be using something similar to
-    // ResultDocument.setSerializationProperty(properties, uri, lname, value,
-    // new InscopeNamespaceResolver(child), false, node.getConfiguration());
-    // for validation?
-
-    private Properties setSerializationParams(Properties props, Map<String, Sequence> map,
-                                              CharacterMapIndex charMapIndex, XPathContext context) throws XPathException {
+    private SerializationProperties serializationParamsFromMap(
+            Map<String, Sequence> map, XPathContext context) throws XPathException {
         Sequence seqVal;
+        Properties props = new Properties();
+        CharacterMapIndex charMapIndex = new CharacterMapIndex();
         if ((seqVal = map.get("allow-duplicate-names")) != null) {
             props.setProperty("allow-duplicate-names", toYesNoTypeString(seqVal));
         }
@@ -409,6 +412,9 @@ public class Serialize extends SystemFunction implements Callable {
         if ((seqVal = map.get(sx("attribute-order"))) != null) {
             props.setProperty(SaxonOutputKeys.ATTRIBUTE_ORDER, toQNamesTypeString(seqVal));
         }
+        if ((seqVal = map.get(sx("canonical"))) != null) {
+            props.setProperty(SaxonOutputKeys.CANONICAL, toYesNoTypeString(seqVal));
+        }
         if ((seqVal = map.get(sx("character-representation"))) != null) {
             props.setProperty(SaxonOutputKeys.CHARACTER_REPRESENTATION, seqVal.head().getStringValue());
         }
@@ -421,6 +427,9 @@ public class Serialize extends SystemFunction implements Callable {
         if ((seqVal = map.get(sx("line-length"))) != null) {
             props.setProperty(SaxonOutputKeys.LINE_LENGTH, seqVal.head().getStringValue());
         }
+        if ((seqVal = map.get(sx("property-order"))) != null) {
+            props.setProperty(SaxonOutputKeys.PROPERTY_ORDER, toQNamesTypeString(seqVal));
+        }
         if ((seqVal = map.get(sx("recognize-binary"))) != null) {
             props.setProperty(SaxonOutputKeys.RECOGNIZE_BINARY, toYesNoTypeString(seqVal));
         }
@@ -433,7 +442,7 @@ public class Serialize extends SystemFunction implements Callable {
         if ((seqVal = map.get(sx("supply-source-locator"))) != null) {
             props.setProperty(SaxonOutputKeys.SUPPLY_SOURCE_LOCATOR, toYesNoTypeString(seqVal));
         }
-        return props;
+        return new SerializationProperties(props, charMapIndex);
     }
 
 
@@ -442,10 +451,9 @@ public class Serialize extends SystemFunction implements Callable {
             arguments.length == 1 ? null : arguments[1].head(), context);
     }
 
-    private StringValue evalSerialize(SequenceIterator iter, /*@Nullable*/ Item param, XPathContext context) throws XPathException {
+    private StringValue evalSerialize(SequenceIterator<? extends Item> iter, Item param, XPathContext context) throws XPathException {
 
-        Properties props = new Properties();
-        CharacterMapIndex charMapIndex = new CharacterMapIndex();
+        SerializationProperties params = null;
 
         // The default parameter values are implementation-defined when an output:serialization-parameters
         // element is used (or when the argument is omitted), but are fixed by this specification in the
@@ -460,16 +468,9 @@ public class Serialize extends SystemFunction implements Callable {
                         + NamespaceConstant.OUTPUT + "}serialization-parameters", "XPTY0004");
                 }
                 SerializationParamsHandler sph = new SerializationParamsHandler();
-                //sph.setLocation(getLocation());
                 sph.setSerializationParams(paramNode);
-                props = sph.getSerializationProperties();
-                CharacterMap charMap = sph.getCharacterMap();
-                if (charMap != null) {
-                    // convoluted code here because the SerializerFactory expects an index of character maps
-                    props.setProperty(SaxonOutputKeys.USE_CHARACTER_MAPS, "charMap");
-                    charMapIndex = new CharacterMapIndex();
-                    charMapIndex.putCharacterMap(new StructuredQName("", "", "charMap"), charMap);
-                }
+                params = sph.getSerializationProperties();
+
             } else if (param instanceof MapItem) {
                 // If any parameters are supplied as QNames in the Saxon namespace, convert them to EQName strings
                 MapItem paramMap = (MapItem)param;
@@ -482,46 +483,37 @@ public class Serialize extends SystemFunction implements Callable {
                     }
                 }
                 Map<String, Sequence> checkedOptions = getDetails().optionDetails.processSuppliedOptions(paramMap, context);
-                props = setSerializationParams(props, checkedOptions, charMapIndex, context);
+                params = serializationParamsFromMap(checkedOptions, context);
             } else {
                 throw new XPathException("Second argument to fn:serialize() must either be an element named {"
                     + NamespaceConstant.OUTPUT + "}serialization-parameters, or a map (if using XPath 3.1)", "XPTY0004");
             }
 
 
+        } else {
+            params = new SerializationProperties(new Properties());
         }
 
+        Properties props = params.getProperties();
         if (props.getProperty(OutputKeys.METHOD) == null) {
             props.setProperty(OutputKeys.METHOD, "xml");
         }
         if (props.getProperty(OutputKeys.OMIT_XML_DECLARATION) == null) {
             props.setProperty(OutputKeys.OMIT_XML_DECLARATION, "true");
         }
-
+                                                                                  
         // TODO add more spec-defined defaults here (for both cases)
         try {
             StringWriter result = new StringWriter();
 
             SerializerFactory sf = context.getConfiguration().getSerializerFactory();
             PipelineConfiguration pipe = context.getConfiguration().makePipelineConfiguration();
-            pipe.setController(context.getController());
-            SequenceReceiver out = sf.getReceiver(new StreamResult(result), pipe, props, charMapIndex);
-            out.open();
-            Item item;
-            while ((item = iter.next()) != null) {
-                out.append(item);
-            }
-            out.close();
+            Receiver out = sf.getReceiver(new StreamResult(result), params, pipe);
+            SequenceCopier.copySequence(iter, out);
             return new StringValue(result.toString());
-        } catch (Exception e) {
-            if (e instanceof XPathException) {
-                ((XPathException) e).maybeSetErrorCode("SENR0001");
-                throw (XPathException) e;
-            } else {
-                XPathException se = new XPathException("Serialization unsuccessful", e);
-                se.setErrorCode("SEPM0016");
-                throw se;
-            }
+        } catch (XPathException e) {
+            e.maybeSetErrorCode("SENR0001");
+            throw e;
         }
 
 
