@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -13,6 +13,7 @@ import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.expr.parser.ContextItemStaticInfo;
 import net.sf.saxon.expr.parser.ExpressionVisitor;
 import net.sf.saxon.om.Item;
+import net.sf.saxon.om.One;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.om.ZeroOrOne;
 import net.sf.saxon.trans.XPathException;
@@ -140,14 +141,12 @@ public class StringJoin extends FoldingFunction {
         /**
          * Compute the final result of the function, when all the input has been processed
          * @return the result of the function
-         * @throws net.sf.saxon.trans.XPathException
-         *          if a dynamic error occurs
          */
-        public ZeroOrOne<StringValue> result() throws XPathException {
+        public ZeroOrOne<StringValue> result()  {
             if (position == 0 && returnEmptyIfEmpty) {
                 return ZeroOrOne.empty();
             } else {
-                return new ZeroOrOne<StringValue>(new StringValue(data));
+                return One.string(data.toString());
             }
         }
     }
