@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,6 +8,7 @@
 package net.sf.saxon.functions;
 
 import net.sf.saxon.expr.*;
+import net.sf.saxon.om.Item;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.om.SequenceIterator;
 import net.sf.saxon.om.SequenceTool;
@@ -50,9 +51,9 @@ public class Subsequence_3 extends SystemFunction implements Callable {
      *                  this Callable is explicitly an argument-converting wrapper around the original
      *                  Callable.</p>
      * @return the result of the function call
-     * @throws XPathException
+     * @throws XPathException if evaluation of the arguments fails
      */
-    public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
+    public Sequence<? extends Item> call(XPathContext context, Sequence[] arguments) throws XPathException {
         return SequenceTool.toLazySequence(subSequence(
                 arguments[0].iterate(),
                 (NumericValue) arguments[1].head(),
@@ -61,7 +62,7 @@ public class Subsequence_3 extends SystemFunction implements Callable {
     }
 
 
-    public static SequenceIterator subSequence(
+    public static SequenceIterator<? extends Item> subSequence(
             SequenceIterator seq, NumericValue startVal, NumericValue lengthVal, XPathContext context)
             throws XPathException {
 
