@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -73,7 +73,7 @@ public abstract class SystemFunction extends AbstractFunction {
         SystemFunction fn = rsc.getConfiguration().makeSystemFunction(name, arity);
         if (fn == null) {
             rsc.getConfiguration().makeSystemFunction(name, arity);
-            throw new IllegalStateException(name + '#' + arity);
+            throw new IllegalStateException(name);
         }
         fn.setRetainedStaticContext(rsc);
         return fn;
@@ -138,7 +138,7 @@ public abstract class SystemFunction extends AbstractFunction {
         // Check if any arguments are known to be empty, with a declared result for that case
         for (int i = 0; i < getArity(); i++) {
             if (Literal.isEmptySequence(arguments[i]) && resultIfEmpty(i) != null) {
-                return Literal.makeLiteral(SequenceTool.toGroundedValue(details.resultIfEmpty[i]));
+                return Literal.makeLiteral(details.resultIfEmpty[i].materialize());
             }
         }
         return null;
@@ -291,9 +291,7 @@ public abstract class SystemFunction extends AbstractFunction {
      */
     @Override
     public boolean equals(Object o) {
-        return (o != null) &&
-                (o instanceof SystemFunction) &&
-                super.equals(o);
+        return (o instanceof SystemFunction) && super.equals(o);
     }
 
     /**
@@ -407,7 +405,7 @@ public abstract class SystemFunction extends AbstractFunction {
         if ((details.properties & BuiltInFunctionSet.NEW) != 0) {
             return StaticProperty.ALL_NODES_NEWLY_CREATED;
         }
-        int p = StaticProperty.NON_CREATIVE;
+        int p = StaticProperty.NO_NODES_NEWLY_CREATED;
         if ((details.properties & BuiltInFunctionSet.SIDE) != 0) {
              p |= StaticProperty.HAS_SIDE_EFFECTS;
         }
@@ -448,7 +446,8 @@ public abstract class SystemFunction extends AbstractFunction {
      */
 
     public static Sequence dynamicCall(Function f, XPathContext context, Sequence[] args) throws XPathException {
-        return f.call(f.makeNewContext(context), args);
+        context = f.makeNewContext(context);
+        return f.call(context, args);
     }
 
 
@@ -536,5 +535,21 @@ public abstract class SystemFunction extends AbstractFunction {
         return null;
     }
 
+    /**
+     * Provide a short string showing the contents of the item, suitable
+     * for use in error messages
+     *
+     * @return a depiction of the item suitable for use in error messages
+     */
+    @Override
+    public String toShortString() {
+        return getFunctionName().getDisplayName() + '#' + getArity();
+    }
+
+    public String toString() {
+        return getFunctionName().getDisplayName() + '#' + getArity();
+    }
+
+    
 }
 
