@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -19,12 +19,12 @@ import net.sf.saxon.ma.arrays.ArrayItem;
 import net.sf.saxon.ma.map.MapItem;
 import net.sf.saxon.om.*;
 import net.sf.saxon.trace.LocationKind;
+import net.sf.saxon.trans.Err;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.util.Navigator;
 import net.sf.saxon.type.Type;
 import net.sf.saxon.value.AtomicValue;
 import net.sf.saxon.value.ObjectValue;
-import net.sf.saxon.value.SequenceExtent;
 
 import javax.xml.transform.SourceLocator;
 
@@ -81,14 +81,19 @@ public class Trace extends SystemFunction {
             if (val instanceof NodeInfo) {
                 out.info(label + ": " + Type.displayTypeName(val) + ": "
                     + Navigator.getPath((NodeInfo) val));
-            } else if (val instanceof AtomicValue || val instanceof ObjectValue) {
+            } else if (val instanceof AtomicValue) {
                 out.info(label + ": " + Type.displayTypeName(val) + ": "
                         + val.getStringValue());
             } else if (val instanceof ArrayItem || val instanceof MapItem) {
-                out.info(label + ": " + val.toString());
+                out.info(label + ": " + val.toShortString());
             } else if (val instanceof Function) {
                 StructuredQName name = ((Function)val).getFunctionName();
                 out.info(label + ": function " + (name==null ? "(anon)" : name.getDisplayName()) + "#" + ((Function)val).getArity());
+            } else if (val instanceof ObjectValue) {
+                Object obj = ((ObjectValue)val).getObject();
+                out.info(label + ": " + obj.getClass().getName() + " = " + Err.truncate30(obj.toString()));
+            } else {
+                out.info(label + ": " + val.toShortString());
             }
         }
     }
@@ -106,7 +111,7 @@ public class Trace extends SystemFunction {
         Controller controller = context.getController();
         String label = arguments.length == 1 ? "*" : arguments[1].head().getStringValue();
         if (controller.isTracing()) {
-            Sequence value = SequenceExtent.makeSequenceExtent(arguments[0].iterate());
+            Sequence value = ((SequenceIterator<Item>) arguments[0].iterate()).materialize();
             notifyListener(label, value, location, context);
             return value;
         } else {
