@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -13,9 +13,12 @@ import net.sf.saxon.om.Item;
 import net.sf.saxon.om.LazySequence;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.om.SequenceIterator;
+import net.sf.saxon.trans.UncheckedXPathException;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.value.AnyURIValue;
 
+import java.io.Closeable;
+import java.io.IOException;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.util.Iterator;
@@ -50,12 +53,15 @@ public class UriCollection extends SystemFunction {
             }
 
             public void close() {
-                // no action;
+                if (sources instanceof Closeable) {
+                    try {
+                        ((Closeable) sources).close();
+                    } catch (IOException e) {
+                        throw new UncheckedXPathException(new XPathException(e));
+                    }
+                }
             }
 
-            public int getProperties() {
-                return 0;
-            }
         };
     }
 
