@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -28,6 +28,7 @@ import net.sf.saxon.value.EmptySequence;
 import net.sf.saxon.value.SequenceType;
 
 import java.util.HashMap;
+import java.util.List;
 
 /**
  * This class is used to contain information about a set of built-in functions.
@@ -163,13 +164,15 @@ public abstract class BuiltInFunctionSet implements FunctionLibrary {
      *                     that the static type of the arguments (obtainable via getItemType() and getCardinality()) may
      *                     be used as part of the binding algorithm. In some cases it may be possible for the function
      *                     to be pre-evaluated at compile time, for example if these expressions are all constant values.
-     *                     <p/>
-     *                     The conventions of the XPath language demand that the results of a function depend only on the
+     *                     <p>The conventions of the XPath language demand that the results of a function depend only on the
      *                     values of the expressions supplied as arguments, and not on the form of those expressions. For
      *                     example, the result of f(4) is expected to be the same as f(2+2). The actual expression is supplied
      *                     here to enable the binding mechanism to select the most efficient possible implementation (including
-     *                     compile-time pre-evaluation where appropriate).
+     *                     compile-time pre-evaluation where appropriate).</p>
      * @param env          The static context of the function call
+     * @param reasons      If no matching function is found by the function library, it may add
+     *                     a diagnostic explanation to this list explaining why none of the available
+     *                     functions could be used.
      * @return An expression equivalent to a call on the specified function, if one is found;
      * null if no function was found matching the required name and arity.
      * @throws XPathException if a function is found with the required name and arity, but
@@ -177,17 +180,22 @@ public abstract class BuiltInFunctionSet implements FunctionLibrary {
      *                        while searching for the function.
      */
     @Override
-    public Expression bind(SymbolicName.F symbolicName, Expression[] staticArgs, StaticContext env) throws XPathException {
+    public Expression bind(SymbolicName.F symbolicName, Expression[] staticArgs, StaticContext env, List<String> reasons) {
         StructuredQName functionName = symbolicName.getComponentName();
         int arity = symbolicName.getArity();
         String localName = functionName.getLocalPart();
         if (functionName.hasURI(getNamespace()) && getFunctionDetails(localName, arity) != null) {
             RetainedStaticContext rsc = new RetainedStaticContext(env);
-            SystemFunction fn = makeFunction(localName, arity);
-            fn.setRetainedStaticContext(rsc);
-            Expression f = fn.makeFunctionCall(staticArgs);
-            f.setRetainedStaticContext(rsc);
-            return f;
+            try {
+                SystemFunction fn = makeFunction(localName, arity);
+                fn.setRetainedStaticContext(rsc);
+                Expression f = fn.makeFunctionCall(staticArgs);
+                f.setRetainedStaticContext(rsc);
+                return f;
+            } catch (XPathException e) {
+                reasons.add(e.getMessage());
+                return null;
+            }
         } else {
             return null;
         }
@@ -273,7 +281,6 @@ public abstract class BuiltInFunctionSet implements FunctionLibrary {
     /**
      * Test whether a function with a given name and arity is available; if so, return a function
      * item that can be dynamically called.
-     * <p/>
      * <p>This supports the function-lookup() function in XPath 3.0.</p>
      *
      * @param symbolicName  the qualified name of the function being called
