@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,8 +7,12 @@
 
 package net.sf.saxon.lib;
 
-import net.sf.saxon.expr.*;
+import net.sf.saxon.expr.Callable;
+import net.sf.saxon.expr.Expression;
+import net.sf.saxon.expr.StaticContext;
+import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.expr.parser.ExpressionTool;
+import net.sf.saxon.om.Item;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.trans.XPathException;
 
@@ -17,18 +21,14 @@ import net.sf.saxon.trans.XPathException;
  * with the full capabilities of functions that are an intrinsic part of the Saxon product.
  * In particular, the class has the opportunity to save data from the static context and
  * to optimize itself at compile time.
- * <p/>
  * <p>Instances of this class are created by calling the method makeCallExpression() on the
  * {@link ExtensionFunctionDefinition} object that represents the definition of the function.</p>
- * <p/>
  * <p>The compiler will create one instance of this class for each function call appearing in the
  * expression being compiled. The class must therefore have a public zero-argument constructor.</p>
- * <p/>
  * <p>The compiler will ensure that the supplied arguments in the extension function call are converted
  * if necessary to the declared argument types, by applying the standard conversion rules. The result
  * returned by the function is checked against the declared return type, but no conversion takes place:
  * the returned value must strictly conform to the declared type.</p>
- * <p/>
  * <p>Note that an <code>ExtensionFunctionCall</code> is trusted; calls are allowed even if the configuration option
  * {@link net.sf.saxon.lib.FeatureKeys#ALLOW_EXTERNAL_FUNCTIONS} is false. In cases where an <code>ExtensionFunctionCall</code>
  * is used to load and execute untrusted code, it should check this configuration option before doing so.</p>
@@ -91,7 +91,6 @@ public abstract class ExtensionFunctionCall implements Callable {
      * Rewrite the function call. This method is called at compile time. It gives the implementation
      * an opportunity to replace itself with an optimized implementation that returns the same result.
      * This includes the ability to pre-evaluate the function and return its result as a literal value.
-     * <p/>
      * <p>There is also a further opportunity to perform static checking at this stage and to throw an error
      * if the arguments are invalid.</p>
      *
@@ -119,11 +118,9 @@ public abstract class ExtensionFunctionCall implements Callable {
      * Copy local data from one copy of the function to another. This method must be implemented
      * in any implementation that maintains local data retained from the static context; the job of the
      * method is to copy this local data to the supplied destination function.
-     * <p/>
      * <p>This method is called if a call to the extension function needs to be copied during
      * the process of optimization. For example, this occurs if the function containing the call
      * to the extension function is inlined.</p>
-     * <p/>
      * <p>If any objects held as local data for the function call are mutable then deep copies must
      * be made.</p>
      *
@@ -144,12 +141,10 @@ public abstract class ExtensionFunctionCall implements Callable {
      *                  to the declared types of the arguments will already have been performed.
      *                  <p>If the argument is always a singleton, then the single item may be obtained by calling
      *                  <code>arguments[i].head()</code>.</p>
-     *                  <p/>
      *                  <p>The implementation is not obliged to read all the items in each <code>SequenceIterator</code>
      *                  if they are not required to compute the result; but if any <code>SequenceIterator</code> is not read
      *                  to completion, it is good practice to call its <code>close()</code> method.</p>
      * @return the results of the function as a <code>Sequence</code>.
-     *         <p/>
      *         <p>The implementation is responsible for ensuring that the result is a valid instance of the declared
      *         result type. Saxon will check that this is the case if the {@link net.sf.saxon.lib.ExtensionFunctionDefinition#trustResultType()}
      *         method returns false, but it will never convert the supplied result value to the declared result type.</p>
@@ -164,11 +159,10 @@ public abstract class ExtensionFunctionCall implements Callable {
      *                        code will add information about the error location.
      */
 
-    public abstract Sequence call(XPathContext context, Sequence[] arguments) throws XPathException;
+    public abstract Sequence<? extends Item> call(XPathContext context, Sequence[] arguments) throws XPathException;
 
     /**
      * Compute the effective boolean value of the result.
-     * <p/>
      * <p>Implementations can override this method but are not required to do so. If it is overridden,
      * the result must be consistent with the rules for calculating effective boolean value. The method
      * should be implemented in cases where computing the effective boolean value is significantly cheaper
@@ -180,10 +174,9 @@ public abstract class ExtensionFunctionCall implements Callable {
      *                  to the declared types of the arguments will already have been performed.
      *                  <p>If the argument is always a singleton, then the single item may be obtained by calling
      *                  <code>arguments[i].head()</code>.</p>
-     *                  <p/>
-     *                  <p>The implementation is not obliged to read all the items in each <code>SequenceIterator</code>
-     *                  if they are not required to compute the result; but if any <code>SequenceIterator</code> is not read
-     *                  to completion, it is good practice to call its <code>close()</code> method.</p>
+     *                  <p>The implementation is not obliged to read all the items in each <code>Sequence</code>
+     *                  if they are not required to compute the result; but if any <code>Sequence</code> is not read
+     *                  to completion, it is good practice to call <code>close()</code> on the iterator.</p>
      * @return the effective boolean value of the result
      * @throws XPathException if a dynamic error occurs during evaluation of the function. The Saxon run-time
      *                        code will add information about the error location.
