@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -168,11 +168,12 @@ public class ArrayFunctionSet extends BuiltInFunctionSet {
      * Check that a number proposed for use as a subscript is greater than zero and less than
      * the maximum subscript allowed by the implementation (2^31-1), returning the value
      * as a Java int
+     *
      * @param subscript the proposed subscript
      * @return the proposed subscript as an int, if it is in range
      * @throws XPathException if the subscript is 0, negative, or outside the permitted range
      */
-    private static int checkSubscript(IntegerValue subscript) throws XPathException {
+    public static int checkSubscript(IntegerValue subscript) throws XPathException {
         int index = subscript.asSubscript();
         if (index <= 0) {
             throw new XPathException("Array subscript " + subscript.getStringValue() + " is out of range", "FOAY0001");
@@ -182,16 +183,19 @@ public class ArrayFunctionSet extends BuiltInFunctionSet {
 
 
     /**
-     * Implementation of the extension function array:append(array, item()*) => array
+     * Implementation of the function array:append(array, item()*) =&gt; array
      */
     public static class ArrayAppend extends SystemFunction {
 
         public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
             ArrayItem array = (ArrayItem) arguments[0].head();
             assert array != null;
-            Sequence other = SequenceTool.makeRepeatable(arguments[1]);
-            List<Sequence> list = new ArrayList<Sequence>(1);
-            list.add(other);
+            return append(array, arguments[1]);
+        }
+
+        public static ArrayItem append(ArrayItem array, Sequence member) throws XPathException {
+            List<GroundedValue<? extends Item>> list = new ArrayList<>(1);
+            list.add(member.materialize());
             SimpleArrayItem otherArray = new SimpleArrayItem(list);
             return array.concat(otherArray);
         }
@@ -199,7 +203,7 @@ public class ArrayFunctionSet extends BuiltInFunctionSet {
     }
 
     /**
-     * Implementation of the extension function array:filter(array, function) => array
+     * Implementation of the function array:filter(array, function) =&gt; array
      */
     public static class ArrayFilter extends SystemFunction {
 
@@ -208,7 +212,7 @@ public class ArrayFunctionSet extends BuiltInFunctionSet {
             ArrayItem array = (ArrayItem) arguments[0].head();
             assert array != null;
             Function fn = (Function) arguments[1].head();
-            List<Sequence> list = new ArrayList<Sequence>(1);
+            List<GroundedValue<? extends Item>> list = new ArrayList<>(1);
             int i;
             for (i=0; i < array.arrayLength(); i++) {
                 if (((BooleanValue) dynamicCall(fn, context, new Sequence[]{array.get(i)}).head()).getBooleanValue()) {
@@ -220,22 +224,20 @@ public class ArrayFunctionSet extends BuiltInFunctionSet {
     }
 
     /**
-     * Implementation of the extension function array:flatten => item()*
+     * Implementation of the function array:flatten =&gt; item()*
      */
     public static class ArrayFlatten extends SystemFunction {
 
         private void flatten(Sequence arg, List<Item> out) throws XPathException {
-            SequenceIterator iter = arg.iterate();
-            Item item;
-            while ((item = iter.next()) != null) {
+            arg.iterate().forEachOrFail(item -> {
                 if (item instanceof ArrayItem) {
-                    for (Sequence member : (ArrayItem)item) {
+                    for (Sequence member : ((ArrayItem) item).members()) {
                         flatten(member, out);
                     }
                 } else {
                     out.add(item);
                 }
-            }
+            });
         }
 
         public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
@@ -246,7 +248,7 @@ public class ArrayFunctionSet extends BuiltInFunctionSet {
     }
 
     /**
-     * Implementation of the extension function array:fold-left(array, item()*, function) => array
+     * Implementation of the function array:fold-left(array, item()*, function) =&gt; array
      */
     public static class ArrayFoldLeft extends SystemFunction {
 
@@ -266,7 +268,7 @@ public class ArrayFunctionSet extends BuiltInFunctionSet {
     }
 
     /**
-     * Implementation of the extension function array:fold-left(array, item()*, function) => array
+     * Implementation of the function array:fold-left(array, item()*, function) =&gt; array
      */
     public static class ArrayFoldRight extends SystemFunction {
 
@@ -285,7 +287,7 @@ public class ArrayFunctionSet extends BuiltInFunctionSet {
     }
 
     /**
-     * Implementation of the extension function array:for-each(array, function) => array
+     * Implementation of the function array:for-each(array, function) =&gt; array
      */
     public static class ArrayForEach extends SystemFunction {
 
@@ -294,10 +296,10 @@ public class ArrayFunctionSet extends BuiltInFunctionSet {
             ArrayItem array = (ArrayItem) arguments[0].head();
             assert array != null;
             Function fn = (Function) arguments[1].head();
-            List<Sequence> list = new ArrayList<Sequence>(1);
+            List<GroundedValue<? extends Item>> list = new ArrayList<>(1);
             int i;
             for (i=0; i < array.arrayLength(); i++) {
-                list.add(dynamicCall(fn, context, new Sequence[]{array.get(i)}));
+                list.add(dynamicCall(fn, context, new GroundedValue[]{array.get(i)}).materialize());
             }
             return new SimpleArrayItem(list);
         }
@@ -305,7 +307,7 @@ public class ArrayFunctionSet extends BuiltInFunctionSet {
     }
 
     /**
-     * Implementation of the extension function array:for-each-pair(array, array, function) => array
+     * Implementation of the function array:for-each-pair(array, array, function) =&gt; array
      */
     public static class ArrayForEachPair extends SystemFunction {
 
@@ -316,17 +318,17 @@ public class ArrayFunctionSet extends BuiltInFunctionSet {
             ArrayItem array2 = (ArrayItem) arguments[1].head();
             assert array2 != null;
             Function fn = (Function) arguments[2].head();
-            List<Sequence> list = new ArrayList<Sequence>(1);
+            List<GroundedValue<? extends Item>> list = new ArrayList<>(1);
             int i;
             for (i=0; i < array1.arrayLength() && i < array2.arrayLength(); i++) {
-                list.add(dynamicCall(fn, context, new Sequence[]{array1.get(i), array2.get(i)}));
+                list.add(dynamicCall(fn, context, new Sequence[]{array1.get(i), array2.get(i)}).materialize());
             }
             return new SimpleArrayItem(list);
         }
     }
 
     /**
-     * Implementation of the extension function array:get(array, xs:integer) => item()*
+     * Implementation of the function array:get(array, xs:integer) =&gt; item()*
      */
     public static class ArrayGet extends SystemFunction {
 
@@ -339,7 +341,7 @@ public class ArrayFunctionSet extends BuiltInFunctionSet {
     }
 
     /**
-     * Implementation of the extension function array:head(array) => item()*
+     * Implementation of the function array:head(array) =&gt; item()*
      */
     public static class ArrayHead extends SystemFunction {
 
@@ -355,7 +357,7 @@ public class ArrayFunctionSet extends BuiltInFunctionSet {
     }
 
     /**
-     * Implementation of the extension function array:insert-before(array, xs:integer, item()*) => array
+     * Implementation of the function array:insert-before(array, xs:integer, item()*) =&gt; array
      */
     public static class ArrayInsertBefore extends SystemFunction {
 
@@ -365,25 +367,16 @@ public class ArrayFunctionSet extends BuiltInFunctionSet {
             assert array != null;
             int index = checkSubscript((IntegerValue) arguments[1].head()) - 1;
             if (index < 0 || index > array.arrayLength()){
-                throw new XPathException("Insertion position " + (index+1) + " is not in range","FOAY0001");
+                throw new XPathException("Specified position is not in range","FOAY0001");
             }
-            Sequence newMember = arguments[2];
-            List<Sequence> list = new ArrayList<Sequence>(1);
-            int i;
-            for (i=0; i < index; i++) {
-                list.add(array.get(i));
-            }
-            list.add(newMember);
-            for (i = index; i < array.arrayLength(); i++) {
-                list.add(array.get(i));
-            }
-            return new SimpleArrayItem(list);
+            Sequence<? extends Item> newMember = arguments[2];
+            return array.insert(index, newMember.materialize());
         }
 
     }
 
     /**
-     * Implementation of the extension function array:join(arrays) => array
+     * Implementation of the function array:join(arrays) =&gt; array
      */
     public static class ArrayJoin extends SystemFunction {
 
@@ -401,7 +394,7 @@ public class ArrayFunctionSet extends BuiltInFunctionSet {
     }
 
     /**
-     * Implementation of the extension function array:put(arrays, index, newValue) => array
+     * Implementation of the function array:put(arrays, index, newValue) =&gt; array
      */
     public static class ArrayPut extends SystemFunction {
 
@@ -409,14 +402,14 @@ public class ArrayFunctionSet extends BuiltInFunctionSet {
         public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
             ArrayItem array = (ArrayItem) arguments[0].head();
             int index = checkSubscript((IntegerValue) arguments[1].head()) - 1;
-            GroundedValue newVal = SequenceExtent.makeSequenceExtent(arguments[2].iterate());
+            GroundedValue newVal = arguments[2].materialize();
             return array.put(index, newVal);
         }
 
     }
 
     /**
-     * Implementation of the extension function array:remove(array, xs:integer) => array
+     * Implementation of the function array:remove(array, xs:integer) =&gt; array
      */
     public static class ArrayRemove extends SystemFunction {
 
@@ -425,35 +418,34 @@ public class ArrayFunctionSet extends BuiltInFunctionSet {
             ArrayItem array = (ArrayItem) arguments[0].head();
             if (arguments[1] instanceof IntegerValue) {
                 int index = checkSubscript((IntegerValue) arguments[1]) - 1;
-                if (index < 0 || index >= array.arrayLength()) {
-                    throw new XPathException("Position " + (index+1) + " is not in range", "FOAY0001");
+                if (index >= array.arrayLength()) {
+                    throw new XPathException("Position " + index + " is not in range", "FOAY0001");
                 }
                 return array.remove(index);
             }
-            SequenceIterator iter = arguments[1].iterate();
-            IntegerValue pos;
             IntSet positions = new IntHashSet();
-            while ((pos = (IntegerValue) iter.next()) != null) {
+            SequenceIterator<IntegerValue> arg1 = (SequenceIterator<IntegerValue>)arguments[1].iterate();
+            arg1.forEachOrFail(pos -> {
                 int index = checkSubscript(pos) - 1;
-                if (index < 0 || index >= array.arrayLength()) {
-                    throw new XPathException("Position " + (index+1) + " is not in range", "FOAY0001");
+                if (index >= array.arrayLength()) {
+                    throw new XPathException("Position " + index + " is not in range", "FOAY0001");
                 }
                 positions.add(index);
-            }
+            });
             return array.removeSeveral(positions);
         }
 
     }
 
     /**
-     * Implementation of the extension function array:reverse(array, xs:integer, xs:integer) => array
+     * Implementation of the function array:reverse(array, xs:integer, xs:integer) =&gt; array
      */
     public static class ArrayReverse extends SystemFunction {
 
         public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
             ArrayItem array = (ArrayItem) arguments[0].head();
             assert array != null;
-            List<Sequence> list = new ArrayList<Sequence>(1);
+            List<GroundedValue<? extends Item>> list = new ArrayList<>(1);
             int i;
             for (i=0; i < array.arrayLength(); i++) {
                 list.add(array.get(array.arrayLength()-i-1));
@@ -464,7 +456,7 @@ public class ArrayFunctionSet extends BuiltInFunctionSet {
     }
 
     /**
-     * Implementation of the extension function array:size(array) => integer
+     * Implementation of the function array:size(array) =&gt; integer
      */
     public static class ArraySize extends SystemFunction {
 
@@ -477,7 +469,7 @@ public class ArrayFunctionSet extends BuiltInFunctionSet {
     }
 
     /**
-     * Implementation of the extension function array:subarray(array, xs:integer, xs:integer) => array
+     * Implementation of the function array:subarray(array, xs:integer, xs:integer) =&gt; array
      */
     public static class ArraySubarray extends SystemFunction {
 
@@ -488,7 +480,7 @@ public class ArrayFunctionSet extends BuiltInFunctionSet {
             int start = checkSubscript((IntegerValue) arguments[1].head());
             int length;
             if (arguments.length == 3) {
-                IntegerValue len = ((IntegerValue) arguments[2].head());
+                IntegerValue len = (IntegerValue) arguments[2].head();
                 int signum = len.signum();
                 if (signum < 0) {
                     throw new XPathException("Specified length of subarray is less than zero", "FOAY0002");
@@ -507,18 +499,13 @@ public class ArrayFunctionSet extends BuiltInFunctionSet {
             if (start + length > array.arrayLength() + 1) {
                 throw new XPathException("Specified length of subarray is too great for start position given","FOAY0001");
             }
-            List<Sequence> list = new ArrayList<Sequence>(1);
-            int i;
-            for (i=0; i < length; i++) {
-                list.add(array.get(start - 1 + i));
-            }
-            return new SimpleArrayItem(list);
+            return array.subArray(start-1, start+length-1);
         }
 
     }
 
     /**
-     * Implementation of the extension function array:tail(array) => item()*
+     * Implementation of the function array:tail(array) =&gt; item()*
      */
     public static class ArrayTail extends SystemFunction {
 
@@ -533,31 +520,27 @@ public class ArrayFunctionSet extends BuiltInFunctionSet {
     }
 
     /**
-     * Implementation of the extension function array:_to-sequence(array) => item()* which
+     * Implementation of the function array:_to-sequence(array) =&gt; item()* which
      * is used internally for the implementation of array?*
      */
 
     public static class ArrayToSequence extends SystemFunction {
         public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
             ArrayItem array = (ArrayItem) arguments[0].head();
-            List<GroundedValue> results = new ArrayList<GroundedValue>();
-            for (int i = 0; i < array.arrayLength(); i++) {
-                results.add(SequenceTool.toGroundedValue(array.get(i)));
-            }
-            return new Chain(results);
+            return toSequence(array);
         }
 
         public static Sequence toSequence(ArrayItem array) throws XPathException {
             List<GroundedValue> results = new ArrayList<GroundedValue>();
-            for (int i = 0; i < array.arrayLength(); i++) {
-                results.add(SequenceTool.toGroundedValue(array.get(i)));
+            for (Sequence seq : array.members()) {
+                results.add(((Sequence<Item>) seq).materialize());
             }
             return new Chain(results);
         }
     }
 
     /**
-     * Implementation of the extension function array:_from-sequence(item()*) => array(*) which
+     * Implementation of the function array:_from-sequence(item()*) =&gt; array(*) which
      * is used internally for the implementation of array{} and of the saxon:array extension
      */
 
@@ -572,12 +555,12 @@ public class ArrayFunctionSet extends BuiltInFunctionSet {
          * @param context             the dynamic evaluation context
          * @param additionalArguments the values of all arguments other than the first.
          * @return the Fold object used to compute the function
-         * @throws XPathException if a dynamic error occurs
          */
         @Override
-        public Fold getFold(XPathContext context, Sequence... additionalArguments) throws XPathException {
+        public Fold getFold(XPathContext context, Sequence... additionalArguments) {
             return new Fold() {
-                List<Sequence> members = new ArrayList<Sequence>();
+                List<GroundedValue<? extends Item>> members = new ArrayList<>();
+
                 /**
                  * Process one item in the input sequence, returning a new copy of the working data
                  *
