@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -18,7 +18,8 @@ import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.type.SpecificFunctionType;
 import net.sf.saxon.value.SequenceType;
 import net.sf.saxon.value.StringValue;
-import net.sf.saxon.z.IntPredicate;
+
+import java.util.function.IntPredicate;
 
 import java.util.Map;
 
@@ -117,10 +118,10 @@ public class JsonHandler {
         CharSequence escaped;
         if (escape) {
             escaped = JsonReceiver.escape(val, true, new IntPredicate() {
-                public boolean matches(int value) {
+                public boolean test(int value) {
                     return (value >= 0 && value <= 0x1F) ||
                             (value >= 0x7F && value <= 0x9F) ||
-                            !charChecker.matches(value) ||
+                            !charChecker.test(value) ||
                             (value == 0x5C);
                 }
             });
@@ -165,12 +166,12 @@ public class JsonHandler {
                         substitute(buffer, i, 1, context);
                     } else {
                         int pair = UTF16CharacterSet.combinePair(buffer.charAt(i - 1), ch);
-                        if (!charChecker.matches(pair)) {
+                        if (!charChecker.test(pair)) {
                             substitute(buffer, i - 1, 2, context);
                         }
                     }
                 } else {
-                    if (!charChecker.matches(ch)) {
+                    if (!charChecker.test(ch)) {
                         substitute(buffer, i, 1, context);
                     }
                 }
@@ -260,4 +261,4 @@ public class JsonHandler {
     }
 }
 
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
