@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -41,6 +41,7 @@ public class JsonParser {
     private static final String ERR_DUPLICATE = "FOJS0003";
     private static final String ERR_SCHEMA = "FOJS0004";
     private static final String ERR_OPTIONS = "FOJS0005";
+    private static final String ERR_LIMITS = "FOJS0001";  // No specific code in spec
 
     /**
      * Create a JSON parser
@@ -52,9 +53,8 @@ public class JsonParser {
     /**
      * Parse the JSON string according to supplied options
      *
-     *
      * @param input   JSON input string
-     * @param flags options for the conversion as a map of xs:string : value pairs
+     * @param flags   options for the conversion as a map of xs:string : value pairs
      * @param handler event handler to which parsing events are notified
      * @param context XPath evaluation context
      * @throws XPathException if the syntax of the input is incorrect
@@ -152,8 +152,9 @@ public class JsonParser {
                 break;
 
             case JsonTokenizer.NUMERIC_LITERAL:
-                double d = parseNumericLiteral(tokenizer.currentTokenValue.toString(), flags);
-                handler.writeNumeric(tokenizer.currentTokenValue.toString(), d);
+                String lexical = tokenizer.currentTokenValue.toString();
+                double d = parseNumericLiteral(lexical, flags);
+                handler.writeNumeric(lexical, d);
                 break;
 
             case JsonTokenizer.TRUE:
@@ -208,13 +209,17 @@ public class JsonParser {
             if (duplicate && ((flags & DUPLICATES_REJECTED) != 0)) {
                 invalidJSON("Duplicate key value \"" + Err.wrap(key) + "\"", ERR_DUPLICATE);
             }
-            if (!duplicate || ((flags & (DUPLICATES_LAST|DUPLICATES_RETAINED)) != 0)) {
-                parseConstruct(handler, tokenizer, flags, context);
-            } else {
-                // retain first: parse the duplicate value but discard it
-                JsonHandler h2 = new JsonHandler();
-                h2.setContext(context);
-                parseConstruct(h2, tokenizer, flags, context);
+            try {
+                if (!duplicate || ((flags & (DUPLICATES_LAST | DUPLICATES_RETAINED)) != 0)) {
+                    parseConstruct(handler, tokenizer, flags, context);
+                } else {
+                    // retain first: parse the duplicate value but discard it
+                    JsonHandler h2 = new JsonHandler();
+                    h2.setContext(context);
+                    parseConstruct(h2, tokenizer, flags, context);
+                }
+            } catch (StackOverflowError e) {
+                invalidJSON("Objects are too deeply nested", ERR_LIMITS);
             }
             tok = tokenizer.next();
             if (tok == JsonTokenizer.COMMA) {
@@ -254,7 +259,11 @@ public class JsonParser {
             return;
         }
         while (true) {
-            parseConstruct(handler, tokenizer, flags, context);
+            try {
+                parseConstruct(handler, tokenizer, flags, context);
+            } catch (StackOverflowError e) {
+                invalidJSON("Arrays are too deeply nested", ERR_LIMITS);
+            }
             tok = tokenizer.next();
             if (tok == JsonTokenizer.COMMA) {
                 tok = tokenizer.next();
@@ -268,7 +277,8 @@ public class JsonParser {
             } else if (tok == JsonTokenizer.RSQB) {
                 break;
             } else {
-                invalidJSON("Unexpected token after entry in array", ERR_GRAMMAR);
+                invalidJSON("Unexpected token (" + toString(tok, tokenizer.currentTokenValue.toString()) +
+                                    ") after entry in array", ERR_GRAMMAR);
             }
         }
         handler.endArray();
@@ -295,7 +305,7 @@ public class JsonParser {
                         t = t.substring(1);
                     }
                     if (t.startsWith("0") &&
-                        !(t.equals("0") || t.startsWith("0.") || t.startsWith("0e") || t.startsWith("0E"))) {
+                            !(t.equals("0") || t.startsWith("0.") || t.startsWith("0e") || t.startsWith("0E"))) {
                         invalidJSON("Redundant leading zeroes not allowed: " + token, ERR_GRAMMAR);
                     }
                     if (t.endsWith(".") || t.contains(".e") || t.contains(".E")) {
@@ -316,8 +326,8 @@ public class JsonParser {
     /**
      * Unescape a JSON string literal,
      *
-     * @param literal the string literal to be processed
-     * @param flags   parsing options
+     * @param literal   the string literal to be processed
+     * @param flags     parsing options
      * @param errorCode Error code
      * @return the result of parsing and conversion to XDM
      * @throws net.sf.saxon.trans.XPathException if a dynamic error occurs (such as invalid JSON input)
@@ -378,7 +388,9 @@ public class JsonParser {
                         if (liberal) {
                             buffer.append(literal.charAt(i));
                         } else {
-                            throw new XPathException("Unknown escape sequence \\" + literal.charAt(i), errorCode);
+                            char next = literal.charAt(i);
+                            String xx = next < 256 ? next + "" : "x" + Integer.toHexString(next);
+                            throw new XPathException("Unknown escape sequence \\" + xx, errorCode);
                         }
                 }
             } else {
@@ -426,8 +438,8 @@ public class JsonParser {
         public int currentToken;
         public FastStringBuffer currentTokenValue = new FastStringBuffer(FastStringBuffer.C64);
 
-        private final static int LSQB = 1;
-        private final static int RSQB = 2;
+        public final static int LSQB = 1;
+        public final static int RSQB = 2;
         public static final int LCURLY = 3;
         public static final int RCURLY = 4;
         public static final int STRING_LITERAL = 5;
@@ -437,14 +449,14 @@ public class JsonParser {
         public static final int NULL = 9;
         public static final int COLON = 10;
         public static final int COMMA = 11;
-        public static final int UNQUOTED_STRING = 12; // for "liberal" parsing only
+        public static final int UNQUOTED_STRING = 12;  // for "liberal" parsing only
         public static final int EOF = 999;
 
         public JsonTokenizer(String input) {
             this.input = input;
             this.position = 0;
             // Ignore a leading BOM
-            if (input.length() > 0 && input.charAt(0) == 65279) {
+            if (!input.isEmpty() && input.charAt(0) == 65279) {
                 position++;
             }
         }
@@ -477,6 +489,9 @@ public class JsonParser {
                     currentTokenValue.setLength(0);
                     boolean afterBackslash = false;
                     while (true) {
+                        if (position >= input.length()) {
+                            invalidJSON("Unclosed quotes in string literal", ERR_GRAMMAR);
+                        }
                         char c = input.charAt(position++);
                         if (c < 32) {
                             invalidJSON("Unescaped control character (x" + Integer.toHexString(c) + ")", ERR_GRAMMAR);
@@ -496,9 +511,6 @@ public class JsonParser {
                             currentTokenValue.append(c);
                             afterBackslash = c == '\\' && !afterBackslash;
                         }
-                        if (position >= input.length()) {
-                            invalidJSON("Unclosed quotes in string literal", ERR_GRAMMAR);
-                        }
                     }
                     return STRING_LITERAL;
                 case ':':
@@ -534,7 +546,7 @@ public class JsonParser {
                         }
                     }
                     return NUMERIC_LITERAL;
-                default:
+                default: {
                     // Allow unquoted strings in liberal mode
                     if (NameChecker.isNCNameChar(ch)) {
                         currentTokenValue.setLength(0);
@@ -549,14 +561,15 @@ public class JsonParser {
                             }
                         }
                         String val = currentTokenValue.toString();
-                        if (val.equals("true")) {
-                            return TRUE;
-                        } else if (val.equals("false")) {
-                            return FALSE;
-                        } else if (val.equals("null")) {
-                            return NULL;
-                        } else {
-                            return UNQUOTED_STRING;
+                        switch (val) {
+                            case "true":
+                                return TRUE;
+                            case "false":
+                                return FALSE;
+                            case "null":
+                                return NULL;
+                            default:
+                                return UNQUOTED_STRING;
                         }
                     } else {
                         char c = input.charAt(--position);
@@ -564,10 +577,44 @@ public class JsonParser {
                                             Integer.toHexString(c) + ") at position " + position, ERR_GRAMMAR);
                         return -1;
                     }
+                }
             }
         }
     }
 
+
+    public static String toString(int token, String currentTokenValue) {
+        switch (token) {
+            case JsonTokenizer.LSQB:
+                return "[";
+            case JsonTokenizer.RSQB:
+                return "]";
+            case JsonTokenizer.LCURLY:
+                return "{";
+            case JsonTokenizer.RCURLY:
+                return "}";
+            case JsonTokenizer.STRING_LITERAL:
+                return "string (\"" + currentTokenValue + "\")";
+            case JsonTokenizer.NUMERIC_LITERAL:
+                return "number (" + currentTokenValue + ")";
+            case JsonTokenizer.TRUE:
+                return "true";
+            case JsonTokenizer.FALSE:
+                return "false";
+            case JsonTokenizer.NULL:
+                return "null";
+            case JsonTokenizer.COLON:
+                return ":";
+            case JsonTokenizer.COMMA:
+                return ",";
+            case JsonTokenizer.EOF:
+                return "<eof>";
+            default:
+                return "<" + token + ">";
+        }
+    }
+
+
 }
 
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
