@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,35 +7,29 @@
 
 package net.sf.saxon.ma.map;
 
-import net.sf.saxon.expr.OperandRole;
 import net.sf.saxon.expr.StaticProperty;
-import net.sf.saxon.expr.XPathContext;
-import net.sf.saxon.expr.sort.AtomicComparer;
 import net.sf.saxon.expr.sort.AtomicMatchKey;
-import net.sf.saxon.functions.DeepEqual;
 import net.sf.saxon.ma.trie.ImmutableHashTrieMap;
 import net.sf.saxon.ma.trie.ImmutableMap;
 import net.sf.saxon.ma.trie.Tuple2;
-import net.sf.saxon.om.*;
-import net.sf.saxon.pattern.NodeKindTest;
-import net.sf.saxon.pattern.NodeTest;
-import net.sf.saxon.query.AnnotationList;
-import net.sf.saxon.trace.ExpressionPresenter;
+import net.sf.saxon.om.GroundedValue;
+import net.sf.saxon.om.Sequence;
+import net.sf.saxon.om.SequenceTool;
+import net.sf.saxon.trans.Err;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.iter.AtomicIterator;
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.type.*;
 import net.sf.saxon.value.AtomicValue;
 import net.sf.saxon.value.Cardinality;
-import net.sf.saxon.value.EmptySequence;
 import net.sf.saxon.value.SequenceType;
 
 import java.util.Iterator;
 
 /**
- * An immutable map. This is the Saxon 9.6 implementation, which uses a hash trie
+ * An immutable map. This implementation, which uses a hash trie, was introduced in Saxon 9.6
  */
-public class HashTrieMap extends AbstractItem implements MapItem, GroundedValue {
+public class HashTrieMap implements MapItem {
 
     public final static SequenceType SINGLE_MAP_TYPE = SequenceType.makeSequenceType(
             MapType.ANY_MAP_TYPE, StaticProperty.EXACTLY_ONE);
@@ -89,12 +83,13 @@ public class HashTrieMap extends AbstractItem implements MapItem, GroundedValue
      * Create a singleton map with a single key and value
      * @param key the key value
      * @param value the associated value
-     * @param context dynamic evaluation context. Gives access to type information.
      * @return a singleton map
-     * @throws XPathException if map mixes timezoned and timezoneless values
      */
 
-    public static HashTrieMap singleton(AtomicValue key, Sequence value, XPathContext context) throws XPathException {
+    public static HashTrieMap singleton(AtomicValue key, GroundedValue value) {
+//        if (Instrumentation.ACTIVE) {
+//            Instrumentation.count("singletons");
+//        }
         return new HashTrieMap().addEntry(key, value);
     }
 
@@ -104,6 +99,9 @@ public class HashTrieMap extends AbstractItem implements MapItem, GroundedValue
      */
 
     public HashTrieMap(ImmutableMap<AtomicMatchKey, KeyValuePair> imap) {
+//        if (Instrumentation.ACTIVE) {
+//            Instrumentation.count("new maps");
+//        }
         this.imap = imap;
         entries = -1;
     }
@@ -116,56 +114,19 @@ public class HashTrieMap extends AbstractItem implements MapItem, GroundedValue
      */
 
     public static HashTrieMap copy(MapItem map) {
+//        if (Instrumentation.ACTIVE) {
+//            Instrumentation.count("copies");
+//        }
         if (map instanceof HashTrieMap) {
             return (HashTrieMap)map;
         }
         HashTrieMap m2 = new HashTrieMap();
-        for (KeyValuePair pair : map) {
+        for (KeyValuePair pair : map.keyValuePairs()) {
             m2 = m2.addEntry(pair.key, pair.value);
         }
         return m2;
     }
 
-    /**
-     * Ask whether this function item is an array
-     *
-     * @return false (it is not an array)
-     */
-    public boolean isArray() {
-        return false;
-    }
-
-    /**
-     * Ask whether this function item is a map
-     *
-     * @return true (it is a map)
-     */
-    public boolean isMap() {
-        return true;
-    }
-
-    /**
-     * Get the function annotations (as defined in XQuery). Returns an empty
-     * list if there are no function annotations.
-     *
-     * @return the function annotations
-     */
-
-    @Override
-    public AnnotationList getAnnotations() {
-        return AnnotationList.EMPTY;
-    }
-
-    /**
-     * Atomize the item.
-     *
-     * @return the result of atomization
-     * @throws XPathException if atomization is not allowed for this kind of item
-     */
-    public AtomicSequence atomize() throws XPathException {
-        throw new XPathException("Maps cannot be atomized", "FOTY0013");
-    }
-
     /**
      * After adding an entry to the map, update the cached type information
      * @param key the new key
@@ -174,11 +135,14 @@ public class HashTrieMap extends AbstractItem implements MapItem, GroundedValue
      */
 
     private void updateTypeInformation(AtomicValue key, Sequence val, boolean wasEmpty) {
+//        if (Instrumentation.ACTIVE) {
+//            Instrumentation.count("updateTypeInformation");
+//        }
         if (wasEmpty) {
             keyUType = key.getUType();
             valueUType = SequenceTool.getUType(val);
             keyAtomicType = key.getItemType();
-            valueItemType = getItemTypeOfSequence(val);
+            valueItemType = MapItem.getItemTypeOfSequence(val);
             valueCardinality = SequenceTool.getCardinality(val);
         } else {
             keyUType = keyUType.union(key.getUType());
@@ -187,99 +151,26 @@ public class HashTrieMap extends AbstractItem implements MapItem, GroundedValue
             if (key.getItemType() != keyAtomicType) {
                 keyAtomicType = null;
             }
-            if (!isKnownToConform(val, valueItemType)) {
+            if (!MapItem.isKnownToConform(val, valueItemType)) {
                 valueItemType = null;
             }
         }
     }
 
-    /**
-     * Ask whether all the items in a sequence are known to conform to a given item type
-     * @param value the sequence
-     * @param itemType the given item type
-     * @return true if all the items conform; false if not, or if the information cannot
-     * be efficiently determined
-     */
-
-    private static boolean isKnownToConform(Sequence value, ItemType itemType) {
-        // Problem is we don't have access to a TypeHierarchy object...
-        if (itemType == AnyItemType.getInstance()) {
-            return true;
-        }
-        try {
-            SequenceIterator iter = value.iterate();
-            Item item;
-            while ((item = iter.next()) != null) {
-                if (item instanceof AtomicValue) {
-                    if (itemType instanceof AtomicType) {
-                        if (!Type.isSubType(((AtomicValue)item).getItemType(), (AtomicType)itemType)) {
-                            return false;
-                        }
-                    } else {
-                        return false;
-                    }
-                } else if (item instanceof NodeInfo) {
-                    if (itemType instanceof NodeTest) {
-                        if (!((NodeTest)itemType).matchesNode((NodeInfo)item)) {
-                            return false;
-                        }
-                    } else {
-                        return false;
-                    }
-                } else {
-                    // functions, maps, arrays: give up (this is only an optimization)
-                    return false;
-                }
-            }
-            return true;
-        } catch (XPathException e) {
-            return false;
-        }
-    }
-
-    /**
-     * Get an item type to which all the values in a sequence are known to conform
-     * @param val the sequence
-     * @return the type of the first item in the sequence, provided that all subsequent
-     * values in the sequence are known to conform to this type; otherwise item().
-     */
-
-    private ItemType getItemTypeOfSequence(Sequence val) {
-        try {
-            Item first = val.head();
-            if (first == null) {
-                return AnyItemType.getInstance();
-            } else {
-                ItemType type;
-                if (first instanceof AtomicValue) {
-                    type = ((AtomicValue)first).getItemType();
-                } else if (first instanceof NodeInfo) {
-                    type = NodeKindTest.makeNodeKindTest(((NodeInfo)first).getNodeKind());
-                } else {
-                    type = AnyFunctionType.getInstance();
-                }
-                if (isKnownToConform(val, type)) {
-                    return type;
-                } else {
-                    return AnyItemType.getInstance();
-                }
-            }
-        } catch (XPathException e) {
-            return AnyItemType.getInstance();
-        }
-    }
-
     /**
      * Get the size of the map
      */
 
     public int size() {
+//        if (Instrumentation.ACTIVE) {
+//            Instrumentation.count("size");
+//        }
         if (entries >= 0) {
             return entries;
         }
         int count = 0;
         //noinspection UnusedDeclaration
-        for (KeyValuePair entry: this) {
+        for (KeyValuePair entry: keyValuePairs()) {
             count++;
         }
         return entries = count;
@@ -291,6 +182,9 @@ public class HashTrieMap extends AbstractItem implements MapItem, GroundedValue
      * @return true if and only if the size of the map is zero
      */
     public boolean isEmpty() {
+//        if (Instrumentation.ACTIVE) {
+//            Instrumentation.count("isEmpty");
+//        }
         return entries == 0 || !imap.iterator().hasNext();
     }
 
@@ -304,6 +198,9 @@ public class HashTrieMap extends AbstractItem implements MapItem, GroundedValue
      */
     @Override
     public boolean conforms(AtomicType requiredKeyType, SequenceType requiredValueType, TypeHierarchy th) {
+//        if (Instrumentation.ACTIVE) {
+//            Instrumentation.count("conforms");
+//        }
         if (isEmpty()) {
             return true;
         }
@@ -373,6 +270,9 @@ public class HashTrieMap extends AbstractItem implements MapItem, GroundedValue
      */
     @Override
     public MapType getItemType(TypeHierarchy th) {
+//        if (Instrumentation.ACTIVE) {
+//            Instrumentation.count("getItemType");
+//        }
         AtomicType keyType = null;
         ItemType valueType = null;
         int valueCard = 0;
@@ -415,15 +315,6 @@ public class HashTrieMap extends AbstractItem implements MapItem, GroundedValue
         return keyUType;
     }
 
-    /**
-     * Get the roles of the arguments, for the purposes of streaming
-     *
-     * @return an array of OperandRole objects, one for each argument
-     */
-    public OperandRole[] getOperandRoles() {
-        return new OperandRole[]{OperandRole.SINGLE_ATOMIC};
-    }
-
     /**
      * Create a new map containing the existing entries in the map plus an additional entry,
      * without modifying the original. If there is already an entry with the specified key,
@@ -434,7 +325,10 @@ public class HashTrieMap extends AbstractItem implements MapItem, GroundedValue
      * @return the new map containing the additional entry
      */
 
-    public HashTrieMap addEntry(AtomicValue key, Sequence value) {
+    public HashTrieMap addEntry(AtomicValue key, GroundedValue value) {
+//        if (Instrumentation.ACTIVE) {
+//            Instrumentation.count("addEntry");
+//        }
         boolean empty = isEmpty();
         ImmutableMap<AtomicMatchKey, KeyValuePair> imap2 = imap.put(makeKey(key), new KeyValuePair(key, value));
         HashTrieMap t2 = new HashTrieMap(imap2);
@@ -457,7 +351,10 @@ public class HashTrieMap extends AbstractItem implements MapItem, GroundedValue
      * @return true if an existing entry with the same key was replaced
      */
 
-    public boolean initialPut(AtomicValue key, Sequence value) {
+    public boolean initialPut(AtomicValue key, GroundedValue value) {
+//        if (Instrumentation.ACTIVE) {
+//            Instrumentation.count("initialPut");
+//        }
         boolean empty = isEmpty();
         boolean exists = get(key) != null;
         imap = imap.put(makeKey(key), new KeyValuePair(key, value));
@@ -480,6 +377,9 @@ public class HashTrieMap extends AbstractItem implements MapItem, GroundedValue
      */
 
     public HashTrieMap remove(AtomicValue key) {
+//        if (Instrumentation.ACTIVE) {
+//            Instrumentation.count("remove");
+//        }
         ImmutableMap<AtomicMatchKey, KeyValuePair> m2 = imap.remove(makeKey(key));
         if (m2 == imap) {
             // The key is not present; the map is unchanged
@@ -500,7 +400,10 @@ public class HashTrieMap extends AbstractItem implements MapItem, GroundedValue
      * @return the value associated with the given key, or null if the key is not present in the map
      */
 
-    public Sequence get(AtomicValue key)  {
+    public GroundedValue get(AtomicValue key)  {
+//        if (Instrumentation.ACTIVE) {
+//            Instrumentation.count("get");
+//        }
         KeyValuePair o = imap.get(makeKey(key));
         return o==null ? null : o.value;
     }
@@ -513,6 +416,9 @@ public class HashTrieMap extends AbstractItem implements MapItem, GroundedValue
      */
 
     public KeyValuePair getKeyValuePair(AtomicValue key) {
+//        if (Instrumentation.ACTIVE) {
+//            Instrumentation.count("getKVP");
+//        }
         return imap.get(makeKey(key));
     }
 
@@ -522,6 +428,9 @@ public class HashTrieMap extends AbstractItem implements MapItem, GroundedValue
      */
 
     public AtomicIterator keys() {
+//        if (Instrumentation.ACTIVE) {
+//            Instrumentation.count("keys");
+//        }
         return new AtomicIterator() {
 
             Iterator<Tuple2<AtomicMatchKey, KeyValuePair>> base = imap.iterator();
@@ -533,13 +442,6 @@ public class HashTrieMap extends AbstractItem implements MapItem, GroundedValue
                     return null;
                 }
             }
-
-            public void close() {
-            }
-
-            public int getProperties() {
-                return 0;
-            }
         };
 
     }
@@ -549,9 +451,11 @@ public class HashTrieMap extends AbstractItem implements MapItem, GroundedValue
      *
      * @return an iterator over the key-value pairs
      */
-    public Iterator<KeyValuePair> iterator() {
-        return new Iterator<KeyValuePair>() {
-
+    public Iterable<KeyValuePair> keyValuePairs() {
+//        if (Instrumentation.ACTIVE) {
+//            Instrumentation.count("keyValuePairs");
+//        }
+        return () -> new Iterator<KeyValuePair>() {
             Iterator<Tuple2<AtomicMatchKey, KeyValuePair>> base = imap.iterator();
 
             public boolean hasNext() {
@@ -567,204 +471,9 @@ public class HashTrieMap extends AbstractItem implements MapItem, GroundedValue
             }
         };
 
+
     }
 
-    /**
-     * Get the item type of this item as a function item. Note that this returns the generic function
-     * type for maps, not a type related to this specific map.
-     *
-     * @return the function item's type
-     */
-    public FunctionItemType getFunctionItemType(/*@Nullable*/) {
-        //return new MapType(getKeyType(), getValueType());
-        return MapType.ANY_MAP_TYPE;
-    }
-
-    /**
-     * Get the name of the function, or null if it is anonymous
-     *
-     * @return the function name, or null for an anonymous inline function
-     */
-    public StructuredQName getFunctionName() {
-        return null;
-    }
-
-    /**
-     * Get a description of this function for use in error messages. For named functions, the description
-     * is the function name (as a lexical QName). For others, it might be, for example, "inline function",
-     * or "partially-applied ends-with function".
-     *
-     * @return a description of the function for use in error messages
-     */
-    public String getDescription() {
-        return "map";
-    }
-
-    /**
-     * Get the arity of the function
-     *
-     * @return the number of arguments in the function signature
-     */
-    public int getArity() {
-        return 1;
-    }
-
-    /**
-     * Invoke the function
-     *
-     * @param context the XPath dynamic evaluation context
-     * @param args    the actual arguments to be supplied
-     * @return the result of invoking the function
-     * @throws net.sf.saxon.trans.XPathException if an error occurs evaluating
-     * the supplied argument
-     *
-     */
-    public Sequence call(XPathContext context, Sequence[] args) throws XPathException {
-        AtomicValue key = (AtomicValue) args[0].head();
-        Sequence value = get(key);
-        if (value == null) {
-            return EmptySequence.getInstance();
-        } else {
-            return value;
-        }
-    }
-
-    /**
-     * Get the value of the item as a string. For nodes, this is the string value of the
-     * node as defined in the XPath 2.0 data model, except that all nodes are treated as being
-     * untyped: it is not an error to get the string value of a node with a complex type.
-     * For atomic values, the method returns the result of casting the atomic value to a string.
-     * <p/>
-     * If the calling code can handle any CharSequence, the method {@link #getStringValueCS} should
-     * be used. If the caller requires a string, this method is preferred.
-     *
-     * @return the string value of the item
-     * @throws UnsupportedOperationException if the item is a function item (an unchecked exception
-     *                                       is used here to avoid introducing exception handling to a large number of paths where it is not
-     *                                       needed)
-     * @see #getStringValueCS
-     * @since 8.4
-     */
-    public String getStringValue() {
-        throw new UnsupportedOperationException("A map has no string value");
-    }
-
-    /**
-     * Get the string value of the item as a CharSequence. This is in some cases more efficient than
-     * the version of the method that returns a String. The method satisfies the rule that
-     * <code>X.getStringValueCS().toString()</code> returns a string that is equal to
-     * <code>X.getStringValue()</code>.
-     * <p/>
-     * Note that two CharSequence values of different types should not be compared using equals(), and
-     * for the same reason they should not be used as a key in a hash table.
-     * <p/>
-     * If the calling code can handle any CharSequence, this method should
-     * be used. If the caller requires a string, the {@link #getStringValue} method is preferred.
-     *
-     * @return the string value of the item
-     * @throws UnsupportedOperationException if the item is a function item (an unchecked exception
-     *                                       is used here to avoid introducing exception handling to a large number of paths where it is not
-     *                                       needed)
-     * @see #getStringValue
-     * @since 8.4
-     */
-    public CharSequence getStringValueCS() {
-        throw new UnsupportedOperationException("A map has no string value");
-    }
-
-    /**
-     * Get the typed value of the item.
-     * <p/>
-     * For a node, this is the typed value as defined in the XPath 2.0 data model. Since a node
-     * may have a list-valued data type, the typed value is in general a sequence, and it is returned
-     * in the form of a SequenceIterator.
-     * <p/>
-     * If the node has not been validated against a schema, the typed value
-     * will be the same as the string value, either as an instance of xs:string or as an instance
-     * of xs:untypedAtomic, depending on the node kind.
-     * <p/>
-     * For an atomic value, this method returns an iterator over a singleton sequence containing
-     * the atomic value itself.
-     *
-     * @return an iterator over the items in the typed value of the node or atomic value. The
-     *         items returned by this iterator will always be atomic values.
-     * @throws net.sf.saxon.trans.XPathException
-     *          where no typed value is available, for example in the case of
-     *          an element with complex content
-     * @since 8.4
-     */
-    public SequenceIterator getTypedValue() throws XPathException {
-        throw new XPathException("A map has no typed value");
-    }
-
-    /**
-     * Test whether this FunctionItem is deep-equal to another function item,
-     * under the rules of the deep-equal function
-     *
-     * @param other the other function item
-     */
-    public boolean deepEquals(Function other, XPathContext context, AtomicComparer comparer, int flags) throws XPathException {
-        if (other instanceof MapItem &&
-                ((MapItem) other).size() == size()) {
-            AtomicIterator keys = keys();
-            AtomicValue key;
-            while ((key = keys.next()) != null) {
-                Sequence thisValue = get(key);
-                Sequence otherValue = ((MapItem) other).get(key);
-                if (otherValue == null) {
-                    return false;
-                }
-                if (!DeepEqual.deepEqual(otherValue.iterate(),
-                    thisValue.iterate(), comparer, context, flags)) {
-                    return false;
-                }
-            }
-            return true;
-        }
-        return false;
-    }
-
-    /*@Nullable*/
-    public MapItem itemAt(int n) {
-        return n == 0 ? this : null;
-    }
-
-    public boolean effectiveBooleanValue() throws XPathException {
-        throw new XPathException("A map item has no effective boolean value");
-    }
-
-    /**
-     * Returns a string representation of the object.
-     * @return a string representation of the object.
-     */
-    @Override
-    public String toString() {
-        FastStringBuffer buffer = new FastStringBuffer(256);
-        buffer.append("map{");
-        for (KeyValuePair pair : this) {
-            if (buffer.length() > 4) {
-                buffer.append(",");
-            }
-            buffer.append(pair.key.toString());
-            buffer.append(":");
-            buffer.append(pair.value.toString());
-        }
-        buffer.append("}");
-        return buffer.toString();
-    }
-
-    /**
-     * Output information about this function item to the diagnostic explain() output
-     */
-    public void export(ExpressionPresenter out) throws XPathException {
-        out.startElement("map");
-        out.emitAttribute("size", size() + "");
-        out.endElement();
-    }
-
-    public boolean isTrustedResultType() {
-        return true;
-    }
 
     public void diagnosticDump() {
         System.err.println("Map details:");
@@ -779,9 +488,42 @@ public class HashTrieMap extends AbstractItem implements MapItem, GroundedValue
                                        " = (" + k2.getClass() + " " + k2 + " : " + v + ")");
         }
     }
+
+    public String toShortString() {
+        int size = size();
+        if (size == 0) {
+            return "map{}";
+        } else if (size > 5) {
+            return "map{(:size " + size + ":)}";
+        } else {
+            FastStringBuffer buff = new FastStringBuffer(256);
+            buff.append("map{");
+            Iterator<Tuple2<AtomicMatchKey, KeyValuePair>> iter = imap.iterator();
+            while (iter.hasNext()) {
+                Tuple2<AtomicMatchKey, KeyValuePair> entry = iter.next();
+                AtomicMatchKey k1 = entry._1;
+                AtomicValue k2 = entry._2.key;
+                Sequence v = entry._2.value;
+                buff.append(k2.toShortString());
+                buff.append(':');
+                buff.append(Err.depictSequence(v).toString().trim());
+                buff.append(", ");
+            }
+            if (size == 1) {
+                buff.append("}");
+            } else {
+                buff.setCharAt(buff.length() - 2, '}');
+            }
+            return buff.toString().trim();
+        }
+    }
+
+    public String toString() {
+        return MapItem.mapToString(this);
+    }
 }
 
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 
 
 
