@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,8 +7,8 @@
 
 package net.sf.saxon.ma.map;
 
-import net.sf.saxon.om.Sequence;
-import net.sf.saxon.om.SequenceTool;
+import net.sf.saxon.om.GroundedValue;
+import net.sf.saxon.om.Item;
 import net.sf.saxon.value.AtomicValue;
 
 /**
@@ -17,15 +17,12 @@ import net.sf.saxon.value.AtomicValue;
 
 public class KeyValuePair {
     public AtomicValue key;
-    public Sequence value;
+    public GroundedValue<? extends Item> value;
 
-    public KeyValuePair(AtomicValue key, Sequence value) {
+    public KeyValuePair(AtomicValue key, GroundedValue<? extends Item> value) {
         this.key = key;
         this.value = value;
-        if (SequenceTool.isUnrepeatable(value)) {
-            throw new IllegalArgumentException();
-        }
     }
 }
 
-// Copyright (c) 2010 Saxonica Limited.
+// Copyright (c) 2010-2018 Saxonica Limited.
