@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -35,7 +35,7 @@ public class MapCreate extends SystemFunction {
             }
             MapItem next;
             while ((next = (MapItem) iter.next()) != null) {
-                for (KeyValuePair pair : next) {
+                for (KeyValuePair pair : next.keyValuePairs()) {
                     if (baseMap.get(pair.key) != null) {
                         throw new XPathException("Duplicate key value (" + pair.key + ") in map", "XQDY0137");
                     } else {
