@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -17,7 +17,7 @@ import net.sf.saxon.trans.XPathException;
  * without actually instantiating the nodes themselves.</p>
  */
 
-public interface AtomizedValueIterator extends SequenceIterator {
+public interface AtomizedValueIterator<T extends Item> extends SequenceIterator<T> {
 
     /**
      * Deliver the atomic value that is next in the atomized result
@@ -25,7 +25,7 @@ public interface AtomizedValueIterator extends SequenceIterator {
      * @throws XPathException if a failure occurs reading or atomizing the next value
      */
 
-    public AtomicSequence nextAtomizedValue() throws XPathException;
+    AtomicSequence nextAtomizedValue() throws XPathException;
 
 }
 
