@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -13,7 +13,7 @@ import net.sf.saxon.trans.XPathException;
  * An iterator that maintains the values of position() and current(),
  * typically implemented as a wrapper over an iterator which does not maintain these values itself.
  */
-public interface FocusIterator extends SequenceIterator {
+public interface FocusIterator<T extends Item> extends SequenceIterator<T> {
 
     /**
      * Get the current value in the sequence (the one returned by the
@@ -25,7 +25,7 @@ public interface FocusIterator extends SequenceIterator {
      *         of the sequence has been reached.
      * @since 8.4
      */
-    public Item current();
+    T current();
 
     /**
      * Get the current position. This will usually be zero before the first call
@@ -35,18 +35,17 @@ public interface FocusIterator extends SequenceIterator {
      * implementations follow this practice. (In particular, the EmptyIterator
      * is stateless, and always returns 0 as the value of position(), whether
      * or not next() has been called.)
-     * <p/>
-     * This method does not change the state of the iterator.
+     * <p>This method does not change the state of the iterator.</p>
      *
      * @return the current position, the position of the item returned by the
      *         most recent call of next(). This is 1 after next() has been successfully
      *         called once, 2 after it has been called twice, and so on. If next() has
      *         never been called, the method returns zero. If the end of the sequence
-     *         has been reached, the value returned will always be <= 0; the preferred
+     *         has been reached, the value returned will always be &lt;= 0; the preferred
      *         value is -1.
      * @since 8.4
      */
-    public int position();
+    int position();
 
     /**
      * Get the position of the last item in the sequence
@@ -54,6 +53,6 @@ public interface FocusIterator extends SequenceIterator {
      * @throws XPathException if a failure occurs reading the sequence
      */
 
-    public int getLength() throws XPathException;
+    int getLength() throws XPathException;
 
 }
\ No newline at end of file
