@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,14 +8,13 @@
 package net.sf.saxon.om;
 
 import net.sf.saxon.expr.LastPositionFinder;
-import net.sf.saxon.expr.sort.GroupIterator;
 import net.sf.saxon.pattern.AnyNodeTest;
 import net.sf.saxon.pattern.NodeTest;
-import net.sf.saxon.regex.RegexIterator;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.iter.AxisIterator;
 import net.sf.saxon.tree.iter.GroundedIterator;
 import net.sf.saxon.tree.iter.LookaheadIterator;
+import net.sf.saxon.tree.iter.UnfailingIterator;
 import net.sf.saxon.tree.wrapper.SiblingCountingNode;
 import net.sf.saxon.value.SequenceExtent;
 
@@ -31,32 +30,17 @@ import net.sf.saxon.value.SequenceExtent;
  *
  * @since 9.6
  */
-public class FocusTrackingIterator implements FocusIterator, LookaheadIterator, GroundedIterator, LastPositionFinder {
+public class FocusTrackingIterator<T extends Item>
+        implements FocusIterator<T>, LookaheadIterator<T>, GroundedIterator<T>, LastPositionFinder {
 
-    // The ability to memoize the focus sequence is introduced by issue 3636. To use the facility, it must be enabled
-    // first by compiling the Saxon code with MEMOIZABLE set to true, and then by setting the mutable static variable
-    // MEMOIZE to true. This ensures that the facility has no run-time overhead if not enabled.
-
-    public final static boolean MEMOIZABLE = false;
-    public static boolean MEMOIZE = false;
-
-    private SequenceIterator base;
-    private Item curr;
+    private SequenceIterator<T> base;
+    private T curr;
     private int pos = 0;
     private int last = -1;
-    private SiblingMemory siblingMemory = null;
+    private SiblingMemory siblingMemory;
 
-    public FocusTrackingIterator(SequenceIterator base) {
-        if (MEMOIZABLE && MEMOIZE && (base.getProperties() & GROUNDED) == 0 &&
-                !(base instanceof GroupIterator) && !(base instanceof RegexIterator)) {
-            try {
-                this.base = new MemoSequence(base).iterate();
-            } catch (XPathException e) {
-                this.base = base;
-            }
-        } else {
-            this.base = base;
-        }
+    public FocusTrackingIterator(SequenceIterator<T> base) {
+        this.base = base;
     }
 
     /**
@@ -64,7 +48,7 @@ public class FocusTrackingIterator implements FocusIterator, LookaheadIterator,
      * @return the iterator underlying this FocusIterator
      */
 
-    public SequenceIterator getUnderlyingIterator() {
+    public SequenceIterator<T> getUnderlyingIterator() {
         return base;
     }
 
@@ -81,7 +65,7 @@ public class FocusTrackingIterator implements FocusIterator, LookaheadIterator,
      *          if an error occurs retrieving the next item
      * @since 8.4
      */
-    public Item next() throws XPathException {
+    public T next() throws XPathException {
         curr = base.next();
         if (curr == null) {
             pos = -1;
@@ -101,7 +85,7 @@ public class FocusTrackingIterator implements FocusIterator, LookaheadIterator,
      *         of the sequence has been reached.
      * @since 8.4
      */
-    public Item current() {
+    public T current() {
         return curr;
     }
 
@@ -113,14 +97,13 @@ public class FocusTrackingIterator implements FocusIterator, LookaheadIterator,
      * implementations follow this practice. (In particular, the EmptyIterator
      * is stateless, and always returns 0 as the value of position(), whether
      * or not next() has been called.)
-     * <p/>
-     * This method does not change the state of the iterator.
+     * <p>This method does not change the state of the iterator.</p>
      *
      * @return the current position, the position of the item returned by the
      *         most recent call of next(). This is 1 after next() has been successfully
      *         called once, 2 after it has been called twice, and so on. If next() has
      *         never been called, the method returns zero. If the end of the sequence
-     *         has been reached, the value returned will always be <= 0; the preferred
+     *         has been reached, the value returned will always be &lt;= 0; the preferred
      *         value is -1.
      * @since 8.4
      */
@@ -145,7 +128,7 @@ public class FocusTrackingIterator implements FocusIterator, LookaheadIterator,
             if (last == -1) {
                 GroundedValue residue = SequenceExtent.makeResidue(base);
                 last = pos + residue.getLength();
-                base = residue.iterate();
+                base = (UnfailingIterator<T>)residue.iterate();
             }
         }
         return last;
@@ -156,9 +139,8 @@ public class FocusTrackingIterator implements FocusIterator, LookaheadIterator,
      * is stateless and it is not necessary (or usual) to call it before calling
      * next(). It is used only when there is an explicit need to tell if we
      * are at the last element.
-     * <p/>
-     * This method must not be called unless the result of getProperties() on the iterator
-     * includes the bit setting {@link net.sf.saxon.om.SequenceIterator#LOOKAHEAD}
+     * <p>This method must not be called unless the result of getProperties() on the iterator
+     * includes the bit setting {@link net.sf.saxon.om.SequenceIterator#LOOKAHEAD}</p>
      *
      * @return true if there are more items in the sequence
      * @throws ClassCastException if the base iterator does not support lookahead processing
@@ -169,29 +151,29 @@ public class FocusTrackingIterator implements FocusIterator, LookaheadIterator,
 
     /**
      * Return a GroundedValue containing all the items in the sequence returned by this
-     * SequenceIterator. This should be an "in-memory" value, not a Closure. This method
-     * must not consume the iterator.
+     * SequenceIterator. This should be an "in-memory" value, not a Closure.
      *
      * @return the corresponding Value
      * @throws XPathException in the cases of subclasses (such as the iterator over a MemoClosure)
      *                        which cause evaluation of expressions while materializing the value.
+     * @throws ClassCastException if the iterator does not have the {@link #GROUNDED} property.
      */
     @Override
-    public GroundedValue materialize() throws XPathException {
-        return ((GroundedIterator)base).materialize();
+    public GroundedValue<T> materialize() throws XPathException {
+        return base.materialize();
     }
 
     /**
      * Return a GroundedValue containing all the remaining items in the sequence returned by this
-     * SequenceIterator, starting at the current position. This should be an "in-memory" value, not a
-     * Closure.  This method must not consume the iterator.
+     * SequenceIterator, starting at the current position. This should be an "in-memory" value, not a Closure.
      *
      * @return the corresponding Value
      * @throws XPathException in the cases of subclasses (such as the iterator over a MemoClosure)
      *                        which cause evaluation of expressions while materializing the value.
+     * @throws ClassCastException if the iterator does not have the {@link #GROUNDED} property.
      */
     @Override
-    public GroundedValue getResidue() throws XPathException {
+    public GroundedValue<T> getResidue() throws XPathException {
         return ((GroundedIterator) base).getResidue();
     }
 
@@ -200,7 +182,6 @@ public class FocusTrackingIterator implements FocusIterator, LookaheadIterator,
      * does not require any more items to be delivered by the iterator. This may enable the
      * supplier to release resources. After calling close(), no further calls on the
      * iterator should be made; if further calls are made, the effect of such calls is undefined.
-     * <p/>
      * <p>(Currently, closing an iterator is important only when the data is being "pushed" in
      * another thread. Closing the iterator terminates that thread and means that it needs to do
      * no additional work. Indeed, failing to close the iterator may cause the push thread to hang
@@ -261,7 +242,7 @@ public class FocusTrackingIterator implements FocusIterator, LookaheadIterator,
         }
         if (siblingMemory == null) {
             siblingMemory = new SiblingMemory();
-        } else if (siblingMemory.mostRecentNodeTest.equals(nodeTest) && node.isSameNodeInfo(siblingMemory.mostRecentNode)) {
+        } else if (siblingMemory.mostRecentNodeTest.equals(nodeTest) && node.equals(siblingMemory.mostRecentNode)) {
             return siblingMemory.mostRecentPosition;
         }
         SiblingMemory s = siblingMemory;
@@ -269,7 +250,7 @@ public class FocusTrackingIterator implements FocusIterator, LookaheadIterator,
         NodeInfo prior;
         int count = 1;
         while ((prior = prev.next()) != null) {
-            if (s.mostRecentNode != null && prior.isSameNodeInfo(s.mostRecentNode) && nodeTest.equals(s.mostRecentNodeTest)) {
+            if (s.mostRecentNode != null && prior.equals(s.mostRecentNode) && nodeTest.equals(s.mostRecentNodeTest)) {
                 int result = count + s.mostRecentPosition;
                 s.mostRecentNode = node;
                 s.mostRecentPosition = result;
