@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -36,11 +36,7 @@ public class OneOrMore<T extends Item> extends SequenceExtent {
 
     public static OneOrMore makeOneOrMore(Sequence sequence) throws XPathException {
         List<Item> content = new ArrayList<Item>();
-        SequenceIterator iter = sequence.iterate();
-        Item item;
-        while ((item = iter.next()) != null) {
-            content.add(item);
-        }
+        sequence.iterate().forEachOrFail(content::add);
         if (content.isEmpty()) {
             throw new IllegalArgumentException();
         }
