@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,7 +7,8 @@
 
 package net.sf.saxon.om;
 
-import net.sf.saxon.event.SequenceReceiver;
+import net.sf.saxon.event.Receiver;
+import net.sf.saxon.event.ReceiverOptions;
 import net.sf.saxon.expr.*;
 import net.sf.saxon.expr.parser.Location;
 import net.sf.saxon.functions.Count;
@@ -31,23 +32,6 @@ public class SequenceTool {
 
     public static final int INDETERMINATE_ORDERING = Integer.MIN_VALUE;
 
-    /**
-     * Produce a GroundedValue containing the same values as a supplied sequence.
-     *
-     * @param sequence the supplied sequence
-     * @return a GroundedValue containing the same items
-     * @throws XPathException for example if the value is a closure that needs to be evaluated,
-     *                        and evaluation fails
-     */
-
-    public static GroundedValue toGroundedValue(Sequence sequence) throws XPathException {
-        if (sequence instanceof GroundedValue) {
-            return (GroundedValue) sequence;
-        } else {
-            return SequenceExtent.makeSequenceExtent(sequence.iterate());
-        }
-    }
-
     /**
      * Produce a GroundedValue containing the same values as a supplied sequence.
      *
@@ -57,31 +41,26 @@ public class SequenceTool {
      * @throws XPathException if a failure occurs reading the input iterator
      */
 
-    public static GroundedValue toGroundedValue(SequenceIterator iterator) throws XPathException {
-        if ((iterator.getProperties() & SequenceIterator.GROUNDED) != 0) {
-            return ((GroundedIterator) iterator).materialize();
-        } else {
-            return SequenceExtent.makeSequenceExtent(iterator);
-        }
+    public static <T extends Item> GroundedValue<T> toGroundedValue(SequenceIterator<T> iterator) throws XPathException {
+        return iterator.materialize();
     }
 
     /**
      * Produce a Sequence containing the same values as a supplied sequence; the input is
      * read progressively as required, and saved in a buffer as it is read in case it is needed
-     * again. But if the iterator is already backed by a grounded value, we return an iterator
-     * over that value.
+     * again. But if the iterator is already backed by a grounded value, we return that value.
      *
      * @param iterator the supplied sequence. The iterator may or may not be consumed as a result of
      *                 passing it to this method.
-     * @return a GroundedValue containing the same items
+     * @return a Sequence containing the same items
      * @throws XPathException if a failure occurs reading the input iterator
      */
 
-    public static Sequence toMemoSequence(SequenceIterator iterator) throws XPathException {
+    public static <T extends Item> Sequence<T> toMemoSequence(SequenceIterator<T> iterator) throws XPathException {
         if ((iterator.getProperties() & SequenceIterator.GROUNDED) != 0) {
-            return ((GroundedIterator) iterator).materialize();
+            return ((GroundedIterator<T>) iterator).materialize();
         } else {
-            return new MemoSequence(iterator);
+            return new MemoSequence<>(iterator);
         }
     }
 
@@ -97,13 +76,13 @@ public class SequenceTool {
      * @throws XPathException if a failure occurs reading the input iterator
      */
 
-    public static Sequence toLazySequence(SequenceIterator iterator) throws XPathException {
+    public static <T extends Item> Sequence<T> toLazySequence(SequenceIterator<T> iterator) throws XPathException {
         if ((iterator.getProperties() & SequenceIterator.GROUNDED) != 0 &&
                 !(iterator instanceof RangeIterator) &&
                 !(iterator instanceof ReverseRangeIterator)) {
-            return ((GroundedIterator) iterator).materialize();
+            return iterator.materialize();
         } else {
-            return new LazySequence(iterator);
+            return new LazySequence<>(iterator);
         }
     }
 
@@ -112,14 +91,6 @@ public class SequenceTool {
                 (seq instanceof Closure && !(seq instanceof MemoClosure || seq instanceof SingletonClosure));
     }
 
-    public static Sequence makeRepeatable(Sequence seq) throws XPathException {
-        if (isUnrepeatable(seq)) {
-            return toMemoSequence(seq.iterate());
-        } else {
-            return seq;
-        }
-    }
-
     /**
      * Get the length of a sequence (the number of items it contains)
      *
@@ -201,11 +172,11 @@ public class SequenceTool {
      *                        evaluated, and evaluation fails
      */
 
-    public static Item itemAt(Sequence sequence, int index) throws XPathException {
+    public static <T extends Item> T itemAt(Sequence<T> sequence, int index) throws XPathException {
         if (sequence instanceof Item && index == 0) {
-            return (Item) sequence;
+            return (T) sequence;
         }
-        return toGroundedValue(sequence).itemAt(index);
+        return sequence.materialize().itemAt(index);
     }
 
     /**
@@ -218,12 +189,12 @@ public class SequenceTool {
      */
 
     /*@Nullable*/
-    public static Item asItem(/*@NotNull*/ Sequence sequence) throws XPathException {
+    public static <T extends Item> T asItem(Sequence<T> sequence) throws XPathException {
         if (sequence instanceof Item) {
-            return (Item) sequence;
+            return (T) sequence;
         }
-        SequenceIterator iter = sequence.iterate();
-        Item first = iter.next();
+        SequenceIterator<T> iter = sequence.iterate();
+        T first = iter.next();
         if (first == null) {
             return null;
         }
@@ -256,8 +227,8 @@ public class SequenceTool {
             return node;
         } else if (item instanceof Function) {
             return item;
-        } else if (item instanceof ObjectValue) {
-            return ((ObjectValue) item).getObject();
+        } else if (item instanceof ExternalObject) {
+            return ((ExternalObject) item).getObject();
         } else {
             AtomicValue value = (AtomicValue) item;
             switch (value.getItemType().getPrimitiveType()) {
@@ -304,19 +275,14 @@ public class SequenceTool {
      *                        for example a function item
      */
 
-    public static String getStringValue(Sequence sequence) throws XPathException {
+    public static <T extends Item> String getStringValue(Sequence<T> sequence) throws XPathException {
         FastStringBuffer fsb = new FastStringBuffer(FastStringBuffer.C64);
-        SequenceIterator iter = sequence.iterate();
-        boolean first = true;
-        Item item;
-        while ((item = iter.next()) != null) {
-            if (first) {
-                first = false;
-            } else {
+        sequence.iterate().forEachOrFail(item -> {
+            if (!fsb.isEmpty()) {
                 fsb.append(' ');
             }
             fsb.append(item.getStringValueCS());
-        }
+        });
         return fsb.toString();
     }
 
@@ -395,7 +361,7 @@ public class SequenceTool {
      * a sequence with more than one item)
      */
 
-    public static int getCardinality(Sequence sequence) {
+    public static int getCardinality(Sequence<? extends Item> sequence) {
         if (sequence instanceof Item) {
             return StaticProperty.EXACTLY_ONE;
         }
@@ -411,7 +377,7 @@ public class SequenceTool {
             }
         }
         try {
-            SequenceIterator iter = sequence.iterate();
+            SequenceIterator<? extends Item> iter = sequence.iterate();
             Item item = iter.next();
             if (item == null) {
                 return StaticProperty.ALLOWS_ZERO;
@@ -423,6 +389,18 @@ public class SequenceTool {
         }
     }
 
+    /**
+     * Get a sequence type to which a given value conforms
+     * @param value the given value
+     * @param th the type hierarchy cache
+     * @return the best available sequence type to which the supplied value conforms
+     */
+
+    public static SequenceType getSequenceType(Sequence<? extends Item> value, TypeHierarchy th) {
+        return SequenceType.makeSequenceType(getItemType(value, th), getCardinality(value));
+    }
+
+
     /**
      * Process a supplied value by copying it to the current output destination
      *
@@ -436,13 +414,9 @@ public class SequenceTool {
      *                                           a closure that needs to be evaluated)
      */
 
-    public static void process(Sequence value, XPathContext context, Location locationId) throws XPathException {
-        SequenceIterator iter = value.iterate();
-        SequenceReceiver out = context.getReceiver();
-        Item it;
-        while ((it = iter.next()) != null) {
-            out.append(it, locationId, NodeInfo.ALL_NAMESPACES);
-        }
+    public static void process(Sequence<? extends Item> value, XPathContext context, Location locationId) throws XPathException {
+        Receiver out = context.getReceiver();
+        value.iterate().forEachOrFail(it -> out.append(it, locationId, ReceiverOptions.ALL_NAMESPACES));
     }
 }
 
