@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2015 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -17,7 +17,6 @@ import java.util.HashMap;
  * fingerprints in the range 0-1023, to avoid clashing with codes allocated
  * in a NamePool. We use the top three bits for the namespace, and the bottom
  * seven bits for the local name.
- * <p/>
  * <p>Codes in the range 0-100 are used for standard node kinds such as ELEMENT,
  * DOCUMENT, etc, and for built-in types such as ITEM and EMPTY.</p>
  */
@@ -106,7 +105,6 @@ public abstract class StandardNames {
     public static final int XSL_SEQUENCE = XSL + 66;
     public static final int XSL_SORT = XSL + 67;
     public static final int XSL_SOURCE_DOCUMENT = XSL + 68;
-    public static final int XSL_STREAM = XSL + 69;  // retained for compatibility
     public static final int XSL_STRIP_SPACE = XSL + 70;
     public static final int XSL_STYLESHEET = XSL + 71;
     public static final int XSL_TEMPLATE = XSL + 72;
@@ -123,24 +121,25 @@ public abstract class StandardNames {
 
     public static final int XSL_DEFAULT_COLLATION = XSL + 100;
     public static final int XSL_DEFAULT_MODE = XSL + 101;
-    public static final int XSL_EXCLUDE_RESULT_PREFIXES = XSL + 102;
-    public static final int XSL_EXPAND_TEXT = XSL + 103;
-    public static final int XSL_EXTENSION_ELEMENT_PREFIXES = XSL + 104;
-    public static final int XSL_INHERIT_NAMESPACES = XSL + 105;
-    public static final int XSL_TYPE = XSL + 106;
-    public static final int XSL_USE_ATTRIBUTE_SETS = XSL + 107;
-    public static final int XSL_USE_WHEN = XSL + 108;
-    public static final int XSL_VALIDATION = XSL + 109;
-    public static final int XSL_VERSION = XSL + 110;
-    public static final int XSL_XPATH_DEFAULT_NAMESPACE = XSL + 111;
+    public static final int XSL_DEFAULT_VALIDATION = XSL + 102;
+    public static final int XSL_EXCLUDE_RESULT_PREFIXES = XSL + 103;
+    public static final int XSL_EXPAND_TEXT = XSL + 104;
+    public static final int XSL_EXTENSION_ELEMENT_PREFIXES = XSL + 105;
+    public static final int XSL_INHERIT_NAMESPACES = XSL + 106;
+    public static final int XSL_TYPE = XSL + 107;
+    public static final int XSL_USE_ATTRIBUTE_SETS = XSL + 108;
+    public static final int XSL_USE_WHEN = XSL + 109;
+    public static final int XSL_VALIDATION = XSL + 110;
+    public static final int XSL_VERSION = XSL + 111;
+    public static final int XSL_XPATH_DEFAULT_NAMESPACE = XSL + 112;
 
 
     public static final int SAXON_ASSIGN = SAXON + 1;
-    public static final int SAXON_COLLATION = SAXON + 5;
+    public static final int SAXON_DEEP_UPDATE = SAXON + 3;
+    public static final int SAXON_DO = SAXON + 6;
     public static final int SAXON_DOCTYPE = SAXON + 7;
     public static final int SAXON_ENTITY_REF = SAXON + 8;
-    public static final int SAXON_IMPORT_QUERY = SAXON + 10;
-    public static final int SAXON_SCRIPT = SAXON + 13;
+    public static final int SAXON_TABULATE_MAPS = SAXON + 9;
     public static final int SAXON_WHILE = SAXON + 15;
 
     // Schema extension elements
@@ -149,17 +148,8 @@ public abstract class StandardNames {
 
     private static final String SAXON_B = '{' + NamespaceConstant.SAXON + '}';
     public static final String SAXON_ASSIGNABLE = SAXON_B + "assignable";
-    public static final String SAXON_ATTRIBUTE_ORDER = SAXON_B + "attribute-order";
-    public static final String SAXON_CHARACTER_REPRESENTATION = SAXON_B + "character-representation";
-    public static final String SAXON_DOUBLE_SPACE = SAXON_B + "double-space";
     public static final String SAXON_EXPLAIN = SAXON_B + "explain";
     public static final String SAXON_READ_ONCE = SAXON_B + "read-once";
-    public static final String SAXON_INDENT_SPACES = SAXON_B + "indent-spaces";
-    public static final String SAXON_LINE_LENGTH = SAXON_B + "line-length";
-    public static final String SAXON_NEXT_IN_CHAIN = SAXON_B + "next-in-chain";
-    public static final String SAXON_RECOGNIZE_BINARY = SAXON_B + "recognize-binary";
-    public static final String SAXON_REQUIRE_WELL_FORMED = SAXON_B + "require-well-formed";
-    public static final String SAXON_SUPPRESS_INDENTATION = SAXON_B + "suppress-indentation";
     public static final String SAXON_MEMO_FUNCTION = SAXON_B + "memo-function";
     public static final String SAXON_THREADS = SAXON_B + "threads";
 
@@ -422,7 +412,6 @@ public abstract class StandardNames {
         bindXSLTName(XSL_SEQUENCE, "sequence");
         bindXSLTName(XSL_SORT, "sort");
         bindXSLTName(XSL_SOURCE_DOCUMENT, "source-document");
-        bindXSLTName(XSL_STREAM, "stream");
         bindXSLTName(XSL_STRIP_SPACE, "strip-space");
         bindXSLTName(XSL_STYLESHEET, "stylesheet");
         bindXSLTName(XSL_TEMPLATE, "template");
@@ -439,7 +428,7 @@ public abstract class StandardNames {
 
         bindXSLTName(XSL_DEFAULT_COLLATION, "default-collation");
         bindXSLTName(XSL_DEFAULT_MODE, "default-mode");
-        bindXSLTName(XSL_XPATH_DEFAULT_NAMESPACE, "xpath-default-namespace");
+        bindXSLTName(XSL_DEFAULT_VALIDATION, "default-validation");
         bindXSLTName(XSL_EXPAND_TEXT, "expand-text");
         bindXSLTName(XSL_EXCLUDE_RESULT_PREFIXES, "exclude-result-prefixes");
         bindXSLTName(XSL_EXTENSION_ELEMENT_PREFIXES, "extension-element-prefixes");
@@ -449,13 +438,14 @@ public abstract class StandardNames {
         bindXSLTName(XSL_USE_WHEN, "use-when");
         bindXSLTName(XSL_VALIDATION, "validation");
         bindXSLTName(XSL_VERSION, "version");
+        bindXSLTName(XSL_XPATH_DEFAULT_NAMESPACE, "xpath-default-namespace");
+
 
         bindSaxonName(SAXON_ASSIGN, "assign");
-        bindSaxonName(SAXON_COLLATION, "collation");
+        bindSaxonName(SAXON_DEEP_UPDATE, "deep-update");
+        bindSaxonName(SAXON_DO, "do");
         bindSaxonName(SAXON_DOCTYPE, "doctype");
         bindSaxonName(SAXON_ENTITY_REF, "entity-ref");
-        bindSaxonName(SAXON_IMPORT_QUERY, "import-query");
-        bindSaxonName(SAXON_SCRIPT, "script");
         bindSaxonName(SAXON_WHILE, "while");
         bindSaxonName(SAXON_PARAM, "param");
         bindSaxonName(SAXON_PREPROCESS, "preprocess");
@@ -576,7 +566,6 @@ public abstract class StandardNames {
         bindXSName(XS_ANY_ATOMIC_TYPE, "anyAtomicType");
         bindXSName(XS_YEAR_MONTH_DURATION, "yearMonthDuration");
         bindXSName(XS_DAY_TIME_DURATION, "dayTimeDuration");
-        bindXSName(XS_NUMERIC, "_numeric_");
 
         bindXSIName(XSI_TYPE, "type");
         bindXSIName(XSI_NIL, "nil");
@@ -711,8 +700,7 @@ public abstract class StandardNames {
      * Get a StructuredQName representing a system-defined name
      *
      * @param fingerprint the fingerprint of the name
-     * @return a StructuredQName representing the system-defined name. The result will include
-     * the conventional prefix associated with the namespace, e.g. "xs" for the XSD namespace
+     * @return a StructuredQName representing the system-defined name
      */
 
     public static StructuredQName getStructuredQName(int fingerprint) {
@@ -720,16 +708,10 @@ public abstract class StandardNames {
     }
 
     /**
-     * Get an unprefixed StructuredQName representing a system-defined name
-     *
-     * @param fingerprint the fingerprint of the name
-     * @return a StructuredQName representing the system-defined name. The result will include
-     * no prefix
+     * A commonly-used name held in static:
      */
 
-    public static StructuredQName getUnprefixedStructuredQName(int fingerprint) {
-        return new StructuredQName("", getURI(fingerprint), getLocalName(fingerprint));
-    }
+    public final static StructuredQName SQ_XS_INVALID_NAME = getStructuredQName(XS_INVALID_NAME);
 
 }
 
