@@ -1,8 +1,8 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
-// This Source Code Form is “Incompatible With Secondary Licenses”, as defined by the Mozilla Public License, v. 2.0.
+// This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 package net.sf.saxon.om;
@@ -24,7 +24,7 @@ import java.util.List;
  * checking in the same way as for a native XSLT/XQuery function declaring the type as xs:integer*.
  */
 
-public class ZeroOrMore<T extends Item> implements GroundedValue {
+public class ZeroOrMore<T extends Item> implements GroundedValue<T> {
 
     private List<T> content;
 
@@ -42,20 +42,17 @@ public class ZeroOrMore<T extends Item> implements GroundedValue {
         this.content = content;
     }
 
-    public ZeroOrMore(SequenceIterator iter) throws XPathException {
+    public ZeroOrMore(SequenceIterator<? extends T> iter) throws XPathException {
         content = new ArrayList<T>();
-        Item item;
-        while ((item = iter.next()) != null) {
-            content.add((T)item);
-        }
+        iter.forEachOrFail(content::add);
     }
 
     public T head() {
         return content.isEmpty() ? null : content.get(0);
     }
 
-    public UnfailingIterator iterate() {
-        return new ListIterator(content);
+    public UnfailingIterator<T> iterate() {
+        return new ListIterator<>(content);
     }
 
     /**
@@ -64,7 +61,7 @@ public class ZeroOrMore<T extends Item> implements GroundedValue {
      * @param n the index of the required item, with 0 representing the first item in the sequence
      * @return the n'th item if it exists, or null otherwise
      */
-    public Item itemAt(int n) {
+    public T itemAt(int n) {
         if (n >= 0 && n < content.size()) {
             return content.get(n);
         } else {
