@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -17,11 +17,11 @@ import net.sf.saxon.value.EmptySequence;
  * A value that is a sequence containing zero or one items.
  */
 
-public class ZeroOrOne<T extends Item> implements GroundedValue {
+public class ZeroOrOne<T extends Item> implements GroundedValue<T> {
 
     private T item; // may be null, to represent an empty sequence
 
-    private static ZeroOrOne<Item> EMPTY = new ZeroOrOne<Item>(null);
+    private static ZeroOrOne<Item> EMPTY = new ZeroOrOne<>(null);
 
     /**
      * Return the instance of ZeroOrOne that represents the empty sequence
@@ -45,15 +45,12 @@ public class ZeroOrOne<T extends Item> implements GroundedValue {
 
     /**
      * Get the string value of this sequence. The string value of an item is the result of applying the string()
-     * function. The string value of a sequence is the space-separated result of applying the string-join() function
-     * using a single space as the separator
+     * function. The string value of an empty sequence is the zero-length string.
      *
      * @return the string value of the sequence.
-     * @throws XPathException if the sequence contains items that have no string value (for example, function items)
      */
 
-
-    public CharSequence getStringValueCS() throws XPathException {
+    public CharSequence getStringValueCS() {
         return item == null ? "" : item.getStringValueCS();
     }
 
@@ -119,7 +116,7 @@ public class ZeroOrOne<T extends Item> implements GroundedValue {
      */
 
     /*@NotNull*/
-    public GroundedValue subsequence(int start, int length) {
+    public GroundedValue<T> subsequence(int start, int length) {
         if (item != null && start <= 0 && start + length > 0) {
             return this;
         } else {
@@ -132,7 +129,7 @@ public class ZeroOrOne<T extends Item> implements GroundedValue {
      */
 
     /*@NotNull*/
-    public UnfailingIterator iterate() {
+    public UnfailingIterator<T> iterate() {
         return SingletonIterator.makeIterator(item); // handles null properly
     }
 
@@ -162,7 +159,7 @@ public class ZeroOrOne<T extends Item> implements GroundedValue {
      *
      * @return the simplified sequence
      */
-    public GroundedValue reduce() {
+    public GroundedValue<T> reduce() {
         if (item == null) {
             return EmptySequence.getInstance();
         }
