@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -122,9 +122,9 @@ public class ExceptPattern extends VennPattern {
 
     public boolean equals(/*@NotNull*/ Object other) {
         if (other instanceof ExceptPattern) {
-            Set s0 = new HashSet(10);
+            Set<Pattern> s0 = new HashSet<>(10);
             gatherComponentPatterns(s0);
-            Set s1 = new HashSet(10);
+            Set<Pattern> s1 = new HashSet<>(10);
             ((ExceptPattern) other).gatherComponentPatterns(s1);
             return s0.equals(s1);
         } else {
@@ -154,7 +154,7 @@ public class ExceptPattern extends VennPattern {
      * Copy a pattern. This makes a deep copy.
      *
      * @return the copy of the original pattern
-     * @param rebindings
+     * @param rebindings variables that need to be re-bound
      */
 
     /*@NotNull*/
