@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -32,9 +32,8 @@ import net.sf.saxon.value.NumericValue;
 /**
  * A GeneralPositionalPattern is a pattern of the form A[P] where A is an axis expression using the child axis
  * and P is an expression that depends on the position.
- * <p/>
- * This class handles cases where the predicate P is arbitrarily complex. Simple comparisons of position() against
- * an integer value are handled by the class SimplePositionalPattern.
+ * <p>This class handles cases where the predicate P is arbitrarily complex. Simple comparisons of position() against
+ * an integer value are handled by the class SimplePositionalPattern.</p>
  */
 public class GeneralPositionalPattern extends Pattern {
 
@@ -58,7 +57,6 @@ public class GeneralPositionalPattern extends Pattern {
      * Get the immediate sub-expressions of this expression, with information about the relationship
      * of each expression to its parent expression. Default implementation
      * works off the results of iterateSubExpressions()
-     * <p/>
      * <p>If the expression is a Callable, then it is required that the order of the operands
      * returned by this function is the same as the order of arguments supplied to the corresponding
      * call() method.</p>
@@ -90,6 +88,10 @@ public class GeneralPositionalPattern extends Pattern {
         return nodeTest;
     }
 
+    public void setUsesPosition(boolean usesPosition) {
+        this.usesPosition = usesPosition;
+    }
+
     /**
      * Simplify the pattern: perform any context-independent optimisations
      *
@@ -124,7 +126,6 @@ public class GeneralPositionalPattern extends Pattern {
     /**
      * Perform optimisation of an expression and its subexpressions. This is the third and final
      * phase of static optimization.
-     * <p/>
      * <p>This method is called after all references to functions and variables have been resolved
      * to the declaration of the function or variable, and after all type checking has been done.</p>
      *
@@ -171,13 +172,17 @@ public class GeneralPositionalPattern extends Pattern {
     }
 
     /**
-     * Get the dependencies of the pattern. The only possible dependency for a pattern is
-     * on local variables. This is analyzed in those patterns where local variables may appear.
+     * Get the dependencies of the pattern. The only interesting dependencies for a pattern are
+     * dependencies on local variables or on user-defined functions. These are analyzed in those
+     * patterns containing predicates.
+     *
+     * @return the dependencies, as a bit-significant mask
      */
 
     public int getDependencies() {
         // the only dependency that's interesting is a dependency on local variables
-        return positionExpr.getDependencies() & (StaticProperty.DEPENDS_ON_LOCAL_VARIABLES | StaticProperty.DEPENDS_ON_USER_FUNCTIONS);
+        return positionExpr.getDependencies() &
+                (StaticProperty.DEPENDS_ON_LOCAL_VARIABLES | StaticProperty.DEPENDS_ON_USER_FUNCTIONS);
     }
 
     /**
@@ -227,7 +232,7 @@ public class GeneralPositionalPattern extends Pattern {
         }
 
         XPathContext c2 = context.newMinorContext();
-        ManualIterator iter = new ManualIterator(node);
+        ManualIterator<NodeInfo> iter = new ManualIterator<>(node);
         c2.setCurrentIterator(iter);
 
         try {
@@ -235,7 +240,7 @@ public class GeneralPositionalPattern extends Pattern {
             int actualPosition = -1;
             if (usesPosition) {
                 actualPosition = getActualPosition(node, Integer.MAX_VALUE, context.getCurrentIterator());
-                ManualIterator man = new ManualIterator(node, actualPosition);
+                ManualIterator<NodeInfo> man = new ManualIterator<>(node, actualPosition);
                 XPathContext c3 = c2.newMinorContext();
                 c3.setCurrentIterator(man);
                 c = c3;
@@ -307,7 +312,7 @@ public class GeneralPositionalPattern extends Pattern {
     public boolean equals(Object other) {
         if (other instanceof GeneralPositionalPattern) {
             GeneralPositionalPattern fp = (GeneralPositionalPattern) other;
-            return nodeTest.equals(fp.nodeTest) && positionExpr.equals(fp.positionExpr);
+            return nodeTest.equals(fp.nodeTest) && positionExpr.isEqual(fp.positionExpr);
         } else {
             return false;
         }
@@ -344,7 +349,7 @@ public class GeneralPositionalPattern extends Pattern {
      * Copy a pattern. This makes a deep copy.
      *
      * @return the copy of the original pattern
-     * @param rebindings
+     * @param rebindings variables that need to be re-bound
      */
 
     /*@NotNull*/
@@ -361,15 +366,19 @@ public class GeneralPositionalPattern extends Pattern {
      */
     @Override
     public String toString() {
-        return nodeTest.toString() + "[" + positionExpr.toString() + "]";
+        return nodeTest + "[" + positionExpr + "]";
     }
 
     public void export(ExpressionPresenter presenter) throws XPathException {
         presenter.startElement("p.genPos");
         presenter.emitAttribute("type", nodeTest.toString());
-        if ("JS".equals(presenter.getOption("target"))) {
+        if (!usesPosition) {
+            // flag is this way around for backwards compatibility with 9.8
+            presenter.emitAttribute("flags", "P");
+        }
+        if ("JS".equals(((ExpressionPresenter.ExportOptions) presenter.getOptions()).target)) {
             try {
-                int targetVersion = presenter.getIntOption("targetVersion", 1);
+                int targetVersion = ((ExpressionPresenter.ExportOptions) presenter.getOptions()).targetVersion;
                 presenter.emitAttribute("jsTest", nodeTest.generateJavaScriptItemTypeTest(AnyItemType.getInstance(), targetVersion));
             } catch (XPathException e) {
                 e.maybeSetLocation(getLocation());
