@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -15,6 +15,7 @@ import net.sf.saxon.expr.parser.ExpressionVisitor;
 import net.sf.saxon.expr.parser.RebindingMap;
 import net.sf.saxon.expr.parser.XPathParser;
 import net.sf.saxon.functions.Current;
+import net.sf.saxon.lib.Feature;
 import net.sf.saxon.om.*;
 import net.sf.saxon.style.ExpressionContext;
 import net.sf.saxon.trace.ExpressionPresenter;
@@ -24,8 +25,8 @@ import net.sf.saxon.type.ItemType;
 import net.sf.saxon.type.UType;
 
 /**
- * A Pattern represents the result of parsing an XSLT pattern string. <br>
- * Patterns are created by calling the static method Pattern.make(string). <br>
+ * A Pattern represents the result of parsing an XSLT pattern string. <p>
+ * Patterns are created by calling the static method Pattern.make(string). <p>
  * The pattern is used to test a particular node by calling match().
  */
 
@@ -46,15 +47,16 @@ public abstract class Pattern extends PseudoExpression {
 
     public static Pattern make(String pattern, StaticContext env, PackageData packageData) throws XPathException {
 
-        int languageLevel = 30;
+        int languageLevel = env.getConfiguration().getConfigurationProperty(Feature.XPATH_VERSION_FOR_XSLT);
+        if (languageLevel == 30) {
+            languageLevel = 305; // XPath 3.0 + XSLT extensions
+        }
         int lineNumber = env instanceof ExpressionContext ? ((ExpressionContext) env).getStyleElement().getLineNumber() : -1;
-        PatternParser parser = (PatternParser) env.getConfiguration().newExpressionParser("PATTERN", false, 30);
+        PatternParser parser = (PatternParser) env.getConfiguration().newExpressionParser("PATTERN", false, languageLevel);
         ((XPathParser) parser).setLanguage(XPathParser.XSLT_PATTERN, 30);
-        Pattern pat = parser.parsePattern(pattern, env, lineNumber);
+        Pattern pat = parser.parsePattern(pattern, env);
         pat.setRetainedStaticContext(env.makeRetainedStaticContext());
         // System.err.println("Simplified [" + pattern + "] to " + pat.getClass() + " default prio = " + pat.getDefaultPriority());
-        // set the pattern text for use in diagnostics
-        pat.setOriginalText(pattern);
         pat = pat.simplify();
         return pat;
     }
@@ -92,7 +94,7 @@ public abstract class Pattern extends PseudoExpression {
     /**
      * Ask whether the expression can be lifted out of a loop, assuming it has no dependencies
      * on the controlling variable/focus of the loop
-     * @param forStreaming
+     * @param forStreaming true if we are generating code for streaming
      */
 
     public boolean isLiftable(boolean forStreaming) {
@@ -128,15 +130,38 @@ public abstract class Pattern extends PseudoExpression {
         /*originalText = text;*/
     }
 
-
+    /**
+     * Ask whether dynamic errors in evaluating the pattern should be recovered. This is the default
+     * for normal XSLT patterns, but patterns used internally to represent scannable streaming expressions
+     * are non-recoverable
+     * @return true if dynamic errors in pattern evaluation should be treated as recoverable (if an error
+     * occurs, the pattern is treated as non-matching; a warning is sent to the ErrorListener).
+     */
     public boolean isRecoverable() {
         return recoverable;
     }
 
+    /**
+     * Aay whether dynamic errors in evaluating the pattern should be recovered. This is the default
+     * for normal XSLT patterns, but patterns used internally to represent scannable streaming expressions
+     * are non-recoverable
+     *
+     * @param recoverable true if dynamic errors in pattern evaluation should be treated as recoverable (if an error
+     * occurs, the pattern is treated as non-matching; a warning is sent to the ErrorListener).
+     */
+
     public void setRecoverable(boolean recoverable) {
         this.recoverable = recoverable;
     }
 
+    /**
+     * Handle a dynamic error occurring in the course of pattern evaluation. The effect depends on (a) whether
+     * the error is a circularity error, and (b) whether the pattern is marked as recoverable or not
+     * @param ex  the exception
+     * @param context the evaluation context
+     * @throws XPathException if the error is found to be non-recoverable
+     */
+
     protected void handleDynamicError(XPathException ex, XPathContext context) throws XPathException {
         if ("XTDE0640".equals(ex.getErrorCodeLocalPart())) {
             // Treat circularity error as fatal (test error213)
@@ -146,7 +171,7 @@ public abstract class Pattern extends PseudoExpression {
             // Typically happens when this is a pseudo-pattern used for scannable expressions when streaming
             throw ex;
         }
-        XPathException err = new XPathException("An error occurred matching pattern {" + toString() + "}: ", ex);
+        XPathException err = new XPathException("An error occurred matching pattern {" + this + "}: ", ex);
         err.setXPathContext(context);
         err.setErrorCodeQName(ex.getErrorCodeQName());
         err.setLocation(getLocation());
@@ -180,8 +205,9 @@ public abstract class Pattern extends PseudoExpression {
 
     /**
      * Get the dependencies of the pattern. The only interesting dependencies for a pattern are
-     * on local variables and user-defined functions. This is analyzed in those patterns where
-     * local variables may appear.
+     * dependencies on local variables or on user-defined functions. These are analyzed in those
+     * patterns containing predicates.
+     *
      * @return the dependencies, as a bit-significant mask
      */
 
@@ -265,92 +291,57 @@ public abstract class Pattern extends PseudoExpression {
      * @return an iterator over the selected nodes in the document.
      */
 
-    public SequenceIterator selectNodes(TreeInfo document, final XPathContext context) throws XPathException {
+    public SequenceIterator<? extends NodeInfo> selectNodes(TreeInfo document, final XPathContext context) throws XPathException {
         NodeInfo doc = document.getRootNode();
         final UType uType = getUType();
         if (UType.DOCUMENT.subsumes(uType)) {
             if (matches(doc, context)) {
                 return SingletonIterator.makeIterator(doc);
             } else {
-                return EmptyIterator.getInstance();
+                return new EmptyIterator.OfNodes();
             }
         } else if (UType.ATTRIBUTE.subsumes(uType)) {
             AxisIterator allElements = doc.iterateAxis(AxisInfo.DESCENDANT, NodeKindTest.ELEMENT);
-            MappingFunction atts = new MappingFunction() {
-                public SequenceIterator map(Item item) {
-                    return ((NodeInfo)item).iterateAxis(AxisInfo.ATTRIBUTE);
-                }
-            };
-            SequenceIterator allAttributes =
-                    new MappingIterator(allElements, atts);
-            ItemMappingFunction test = new ItemMappingFunction() {
-                public Item mapItem(Item item) throws XPathException {
-                    if (matches(item, context)) {
-                        return item;
-                    } else {
-                        return null;
-                    }
-                }
-            };
-            return new ItemMappingIterator(allAttributes, test);
+            MappingFunction<NodeInfo, NodeInfo> atts = item -> item.iterateAxis(AxisInfo.ATTRIBUTE);
+            SequenceIterator<NodeInfo> allAttributes = new MappingIterator<>(allElements, atts);
+            return new ItemMappingIterator<>(allAttributes, item -> matches(item, context) ? item : null);
         } else if (UType.NAMESPACE.subsumes(uType)) {
             AxisIterator allElements = doc.iterateAxis(AxisInfo.DESCENDANT, NodeKindTest.ELEMENT);
-            MappingFunction atts = new MappingFunction() {
-                public SequenceIterator map(Item item) {
-                    return ((NodeInfo)item).iterateAxis(AxisInfo.NAMESPACE);
-                }
-            };
-            SequenceIterator allNamespaces =
-                    new MappingIterator(allElements, atts);
-            ItemMappingFunction test = new ItemMappingFunction() {
-                public Item mapItem(Item item) throws XPathException {
-                    if (matches(item, context)) {
-                        return item;
-                    } else {
-                        return null;
-                    }
-                }
-            };
-            return new ItemMappingIterator(allNamespaces, test);
+            MappingFunction<NodeInfo, NodeInfo> atts = item -> item.iterateAxis(AxisInfo.NAMESPACE);
+            SequenceIterator<NodeInfo> allNamespaces = new MappingIterator<>(allElements, atts);
+            return new ItemMappingIterator<>(allNamespaces, item -> matches(item, context) ? item : null);
 
         } else if (UType.CHILD_NODE_KINDS.subsumes(uType)) {
-            AxisIterator allChildren = doc.iterateAxis(AxisInfo.DESCENDANT, new MultipleNodeKindTest(uType));
-            ItemMappingFunction test = new ItemMappingFunction() {
-                public Item mapItem(Item item) throws XPathException {
-                    if (matches(item, context)) {
-                        return item;
-                    } else {
-                        return null;
-                    }
-                }
-            };
-            return new ItemMappingIterator(allChildren, test);
+            NodeTest nodeTest;
+            if (uType.equals(UType.ELEMENT)) {
+                nodeTest = NodeKindTest.ELEMENT; // common case, enables use of getAllElements()
+            } else {
+                nodeTest = new MultipleNodeKindTest(uType);
+            }
+            AxisIterator allChildren = doc.iterateAxis(AxisInfo.DESCENDANT, nodeTest);
+            return new ItemMappingIterator<>(allChildren, item -> matches(item, context) ? item : null);
         } else {
             byte axis = uType.subsumes(UType.DOCUMENT) ? AxisInfo.DESCENDANT_OR_SELF : AxisInfo.DESCENDANT;
             AxisIterator allChildren = doc.iterateAxis(axis);
-            MappingFunction processElement = new MappingFunction() {
-                public SequenceIterator map(Item item) {
-                    AxisIterator mapper = SingleNodeIterator.makeIterator((NodeInfo)item);
-                    if (uType.subsumes(UType.NAMESPACE)) {
-                        mapper = new ConcatenatingAxisIterator(mapper, ((NodeInfo) item).iterateAxis(AxisInfo.NAMESPACE));
-                    }
-                    if (uType.subsumes(UType.ATTRIBUTE)) {
-                        mapper = new ConcatenatingAxisIterator(mapper, ((NodeInfo) item).iterateAxis(AxisInfo.ATTRIBUTE));
-                    }
-                    return mapper;
+            MappingFunction processElement = item -> {
+                AxisIterator mapper = SingleNodeIterator.makeIterator((NodeInfo)item);
+                if (uType.subsumes(UType.NAMESPACE)) {
+                    mapper = new ConcatenatingAxisIterator(mapper, ((NodeInfo) item).iterateAxis(AxisInfo.NAMESPACE));
+                }
+                if (uType.subsumes(UType.ATTRIBUTE)) {
+                    mapper = new ConcatenatingAxisIterator(mapper, ((NodeInfo) item).iterateAxis(AxisInfo.ATTRIBUTE));
+                }
+                return mapper;
+            };
+            SequenceIterator<NodeInfo> attributesOrSelf = new MappingIterator(allChildren, processElement);
+            ItemMappingFunction<NodeInfo, NodeInfo> test = item -> {
+                if (matches(item, context)) {
+                    return item;
+                } else {
+                    return null;
                 }
             };
-            SequenceIterator attributesOrSelf = new MappingIterator(allChildren, processElement);
-            ItemMappingFunction test = new ItemMappingFunction() {
-                public Item mapItem(Item item) throws XPathException {
-                    if (matches(item, context)) {
-                        return item;
-                    } else {
-                        return null;
-                    }
-                }
-            };
-            return new ItemMappingIterator(attributesOrSelf, test);
+            return new ItemMappingIterator<>(attributesOrSelf, test);
 
         }
     }
@@ -413,7 +404,7 @@ public abstract class Pattern extends PseudoExpression {
      */
 
     public String toString() {
-        return "pattern matching " + getItemType().toString();
+        return "pattern matching " + getItemType();
     }
 
     /**
@@ -465,7 +456,6 @@ public abstract class Pattern extends PseudoExpression {
     /**
      * Perform optimisation of an expression and its subexpressions. This is the third and final
      * phase of static optimization.
-     * <p/>
      * <p>This method is called after all references to functions and variables have been resolved
      * to the declaration of the function or variable, and after all type checking has been done.</p>
      *
@@ -483,6 +473,7 @@ public abstract class Pattern extends PseudoExpression {
         return this;
     }
 
+
     @Override
     public String toShortString() {
         return toString();
