@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -35,13 +35,6 @@ public class PatternThatSetsCurrent extends Pattern {
 
     public PatternThatSetsCurrent(Pattern wrappedPattern) {
         this(wrappedPattern, new LocalVariableBinding(Current.FN_CURRENT, SequenceType.SINGLE_ITEM));
-//        this.wrappedPattern = wrappedPattern;
-//        this.binding = new LocalVariableBinding(Current.FN_CURRENT, SequenceType.SINGLE_ITEM);
-//        setPackageData(wrappedPattern.getPackageData());
-//        setPriority(wrappedPattern.getDefaultPriority());
-//        setOriginalText(wrappedPattern.toString());
-//        setLineNumber(wrappedPattern.getLineNumber());
-//        setSystemId(wrappedPattern.getSystemId());
     }
 
     public PatternThatSetsCurrent(Pattern wrappedPattern, LocalVariableBinding binding) {
@@ -50,14 +43,12 @@ public class PatternThatSetsCurrent extends Pattern {
         binding.setRequiredType(SequenceType.makeSequenceType(wrappedPattern.getItemType(), StaticProperty.EXACTLY_ONE));
         adoptChildExpression(wrappedPattern);
         setPriority(wrappedPattern.getDefaultPriority());
-        setOriginalText(wrappedPattern.toString());
     }
 
     /**
      * Get the immediate sub-expressions of this expression, with information about the relationship
      * of each expression to its parent expression. Default implementation
      * works off the results of iterateSubExpressions()
-     * <p/>
      * <p>If the expression is a Callable, then it is required that the order of the operands
      * returned by this function is the same as the order of arguments supplied to the corresponding
      * call() method.</p>
@@ -248,7 +239,7 @@ public class PatternThatSetsCurrent extends Pattern {
      * Copy a pattern. This makes a deep copy.
      *
      * @return the copy of the original pattern
-     * @param rebindings
+     * @param rebindings variables that need to be re-bound
      */
 
     /*@NotNull*/
@@ -271,15 +262,5 @@ public class PatternThatSetsCurrent extends Pattern {
         return wrappedPattern;
     }
 
-    @Override
-    public int computeHashCode() {
-        return wrappedPattern.hashCode() ^ 0x77332211;
-    }
-
-    @Override
-    public boolean equals(Object obj) {
-        return obj instanceof PatternThatSetsCurrent && wrappedPattern.equals(((PatternThatSetsCurrent)obj).wrappedPattern);
-    }
-
 }
 
