@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -46,7 +46,7 @@ public class StaxBridge implements PullProvider {
     private StaxAttributes attributes = new StaxAttributes();
     private PipelineConfiguration pipe;
     private NamePool namePool;
-    private HashMap<String, NodeName> nameCache = new HashMap<String, NodeName>();
+    private HashMap<String, NodeName> nameCache = new HashMap<>();
     private List unparsedEntities = null;
     int currentEvent = START_OF_INPUT;
     int depth = 0;
@@ -257,7 +257,6 @@ public class StaxBridge implements PullProvider {
      * of the returned AttributeCollection are guaranteed to remain unchanged
      * until the next START_ELEMENT event, but may be modified thereafter. The object
      * should not be modified by the client.
-     * <p/>
      * <p>Attributes may be read before or after reading the namespaces of an element,
      * but must not be read after the first child node has been read, or after calling
      * one of the methods skipToEnd(), getStringValue(), or getTypedValue().</p>
@@ -279,15 +278,12 @@ public class StaxBridge implements PullProvider {
      * node. In the case of a non-top-level element, the NamespaceDeclarations will contain
      * a set of namespace declarations and undeclarations, representing the differences between
      * this element and its parent.
-     * <p/>
      * <p>It is permissible for this method to return namespace declarations that are redundant.</p>
-     * <p/>
      * <p>The NamespaceDeclarations object is guaranteed to remain unchanged until the next START_ELEMENT
      * event, but may then be overwritten. The object should not be modified by the client.</p>
-     * <p/>
      * <p>Namespaces may be read before or after reading the attributes of an element,
      * but must not be read after the first child node has been read, or after calling
-     * one of the methods skipToEnd(), getStringValue(), or getTypedValue().</p>*
+     * one of the methods skipToEnd(), getStringValue(), or getTypedValue().</p>
      */
 
     public NamespaceBinding[] getNamespaceDeclarations() throws XPathException {
@@ -409,9 +405,7 @@ public class StaxBridge implements PullProvider {
     /**
      * Get the string value of the current element, text node, processing-instruction,
      * or top-level attribute or namespace node, or atomic value.
-     * <p/>
      * <p>In other situations the result is undefined and may result in an IllegalStateException.</p>
-     * <p/>
      * <p>If the most recent event was a {@link #START_ELEMENT}, this method causes the content
      * of the element to be read. The current event on completion of this method will be the
      * corresponding {@link #END_ELEMENT}. The next call of next() will return the event following
@@ -598,7 +592,6 @@ public class StaxBridge implements PullProvider {
 
         /**
          * Get the systemId part of the location of an attribute, at a given index.
-         * <p/>
          * <p>Attribute location information is not available from a SAX parser, so this method
          * is not useful for getting the location of an attribute in a source document. However,
          * in a Saxon result document, the location information represents the location in the
@@ -615,7 +608,6 @@ public class StaxBridge implements PullProvider {
 
         /**
          * Get the line number part of the location of an attribute, at a given index.
-         * <p/>
          * <p>Attribute location information is not available from a SAX parser, so this method
          * is not useful for getting the location of an attribute in a source document. However,
          * in a Saxon result document, the location information represents the location in the
@@ -787,7 +779,7 @@ public class StaxBridge implements PullProvider {
         if (unparsedEntities == null) {
             return null;
         }
-        List<UnparsedEntity> list = new ArrayList<UnparsedEntity>(unparsedEntities.size());
+        List<UnparsedEntity> list = new ArrayList<>(unparsedEntities.size());
         for (Object ent : unparsedEntities) {
             String name = null;
             String systemId = null;
@@ -814,11 +806,7 @@ public class StaxBridge implements PullProvider {
                     publicId = (String) method.invoke(ent, noArgs);
                     method = woodstoxClass.getMethod("getBaseURI", noArgClasses);
                     baseURI = (String) method.invoke(ent, noArgs);
-                } catch (NoSuchMethodException e) {
-                    //
-                } catch (IllegalAccessException e) {
-                    //
-                } catch (InvocationTargetException e) {
+                } catch (NoSuchMethodException | IllegalAccessException | InvocationTargetException e) {
                     //
                 }
             }
