@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -17,6 +17,7 @@ import net.sf.saxon.trans.SymbolicName;
 import net.sf.saxon.trans.XPathException;
 
 import java.util.HashSet;
+import java.util.List;
 
 /**
  * This implementation of FunctionLibrary contains all the functions imported into a Query Module.
@@ -28,7 +29,7 @@ public class ImportedFunctionLibrary implements FunctionLibrary, XQueryFunctionB
 
     private transient QueryModule importingModule;
     private XQueryFunctionLibrary baseLibrary;
-    private HashSet<String> namespaces = new HashSet<String>(5);
+    private HashSet<String> namespaces = new HashSet<>(5);
 
     /**
      * Create an imported function library
@@ -63,40 +64,31 @@ public class ImportedFunctionLibrary implements FunctionLibrary, XQueryFunctionB
      *                     that the static type of the arguments (obtainable via getItemType() and getCardinality()) may
      *                     be used as part of the binding algorithm. In some cases it may be possible for the function
      *                     to be pre-evaluated at compile time, for example if these expressions are all constant values.
-     *                     <p/>
-     *                     The conventions of the XPath language demand that the results of a function depend only on the
+     *                     <p>The conventions of the XPath language demand that the results of a function depend only on the
      *                     values of the expressions supplied as arguments, and not on the form of those expressions. For
      *                     example, the result of f(4) is expected to be the same as f(2+2). The actual expression is supplied
      *                     here to enable the binding mechanism to select the most efficient possible implementation (including
-     *                     compile-time pre-evaluation where appropriate).
-     * @param env
+     *                     compile-time pre-evaluation where appropriate).</p>
+     * @param env          The static context of the function call
+     * @param reasons      If no matching function is found by the function library, it may add
+     *                     a diagnostic explanation to this list explaining why none of the available
+     *                     functions could be used.
      * @return An object representing the function to be called, if one is found;
      *         null if no function was found matching the required name and arity.
-     * @throws net.sf.saxon.trans.XPathException
-     *          if a function is found with the required name and arity, but
-     *          the implementation of the function cannot be loaded or used; or if an error occurs
-     *          while searching for the function.
      */
 
     /*@Nullable*/
-    public Expression bind(/*@NotNull*/ SymbolicName.F symbolicName, Expression[] staticArgs, StaticContext env) throws XPathException {
+    public Expression bind(/*@NotNull*/ SymbolicName.F symbolicName, Expression[] staticArgs, StaticContext env, List<String> reasons) {
         final StructuredQName functionName = symbolicName.getComponentName();
         final String uri = functionName.getURI();
         RetainedStaticContext rsc = new RetainedStaticContext(env);
         for (Expression arg : staticArgs) {
-            if (arg.getRetainedStaticContext() == null) {
+            if (arg.getLocalRetainedStaticContext() == null) {
                 arg.setRetainedStaticContext(rsc);
             }
         }
         if (namespaces.contains(uri)) {
-            Expression call = baseLibrary.bind(symbolicName, staticArgs, env);
-            if (call != null) {
-                // Check that the result type and all the argument types are in the static context of the
-                // calling module
-                XQueryFunction def = baseLibrary.getDeclaration(functionName, staticArgs.length);
-                importingModule.checkImportedFunctionSignature(def);
-            }
-            return call;
+            return baseLibrary.bind(symbolicName, staticArgs, env, reasons);
         } else {
             return null;
         }
@@ -150,7 +142,6 @@ public class ImportedFunctionLibrary implements FunctionLibrary, XQueryFunctionB
     /**
      * Test whether a function with a given name and arity is available; if so, return a function
      * item that can be dynamically called.
-     * <p/>
      * <p>This supports the function-lookup() function in XPath 3.0.</p>
      *
      * @param functionName  the qualified name of the function being called
