@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -12,6 +12,7 @@ import net.sf.saxon.event.*;
 import net.sf.saxon.expr.parser.ExplicitLocation;
 import net.sf.saxon.lib.SerializerFactory;
 import net.sf.saxon.om.*;
+import net.sf.saxon.serialize.SerializationProperties;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.iter.SingletonIterator;
 import net.sf.saxon.tree.tiny.Statistics;
@@ -75,8 +76,7 @@ public class QueryResult {
         TinyBuilder builder = new TinyBuilder(pipe);
         builder.setStatistics(Statistics.SOURCE_DOCUMENT_STATISTICS);
         NamespaceReducer reducer = new NamespaceReducer(builder);
-        ComplexContentOutputter outputter = new ComplexContentOutputter(pipe);
-        outputter.setReceiver(reducer);
+        ComplexContentOutputter outputter = new ComplexContentOutputter(reducer);
         sendWrappedSequence(iterator, outputter);
         return new DocumentInfo(builder.getCurrentRoot());
     }
@@ -129,7 +129,7 @@ public class QueryResult {
      */
 
     public static void serializeSequence(
-            /*@NotNull*/ SequenceIterator iterator, /*@NotNull*/ Configuration config, /*@NotNull*/ OutputStream destination, /*@NotNull*/ Properties outputProps)
+            SequenceIterator<? extends Item> iterator, Configuration config, OutputStream destination, Properties outputProps)
             throws XPathException {
         serializeSequence(iterator, config, new StreamResult(destination), outputProps);
         try {
@@ -152,7 +152,7 @@ public class QueryResult {
      */
 
     public static void serializeSequence(
-            /*@NotNull*/ SequenceIterator iterator, /*@NotNull*/ Configuration config, /*@NotNull*/ Writer writer, /*@NotNull*/ Properties outputProps)
+            SequenceIterator<? extends Item> iterator, Configuration config, Writer writer, Properties outputProps)
             throws XPathException {
         serializeSequence(iterator, config, new StreamResult(writer), outputProps);
         try {
@@ -175,17 +175,11 @@ public class QueryResult {
      */
 
     public static void serializeSequence(
-            /*@NotNull*/ SequenceIterator iterator, /*@NotNull*/ Configuration config, /*@NotNull*/ Result result, /*@NotNull*/ Properties outputProperties)
+            /*@NotNull*/ SequenceIterator<? extends Item> iterator, Configuration config, Result result, Properties outputProperties)
             throws XPathException {
         SerializerFactory sf = config.getSerializerFactory();
-        PipelineConfiguration pipe = config.makePipelineConfiguration();
-        SequenceReceiver tr = sf.getReceiver(result, pipe, outputProperties);
-        Item item;
-        tr.open();
-        while ((item = iterator.next()) != null) {
-            tr.append(item, ExplicitLocation.UNKNOWN_LOCATION, NodeInfo.ALL_NAMESPACES);
-        }
-        tr.close();
+        Receiver tr = sf.getReceiver(result, new SerializationProperties(outputProperties));
+        SequenceCopier.copySequence(iterator, tr);
     }
 
 
@@ -244,6 +238,7 @@ public class QueryResult {
                 log.println("Creating file " + existingFile);
             }
             try {
+                //noinspection ResultOfMethodCallIgnored
                 existingFile.createNewFile();
             } catch (IOException e) {
                 throw new XPathException("Failed to create new file " + existingFile);
@@ -254,9 +249,8 @@ public class QueryResult {
             }
         }
         Configuration config = doc.getConfiguration();
-        PipelineConfiguration pipe = config.makePipelineConfiguration();
         SerializerFactory factory = config.getSerializerFactory();
-        Receiver r = factory.getReceiver(new StreamResult(existingFile), pipe, outputProperties);
+        Receiver r = factory.getReceiver(new StreamResult(existingFile), new SerializationProperties(outputProperties));
         doc.copy(r, CopyOptions.ALL_NAMESPACES, ExplicitLocation.UNKNOWN_LOCATION);
         r.close();
     }
