@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -14,6 +14,7 @@ import net.sf.saxon.expr.instruct.GlobalParam;
 import net.sf.saxon.expr.instruct.GlobalVariable;
 import net.sf.saxon.expr.parser.CodeInjector;
 import net.sf.saxon.expr.parser.Location;
+import net.sf.saxon.expr.parser.OptimizerOptions;
 import net.sf.saxon.expr.sort.SimpleCollation;
 import net.sf.saxon.functions.FunctionLibrary;
 import net.sf.saxon.lib.*;
@@ -34,11 +35,9 @@ import java.util.*;
 /**
  * StaticQueryContext contains information used to build a StaticContext for use when processing XQuery
  * expressions.
- * <p/>
  * <p>Despite its name, <code>StaticQueryContext</code> no longer implements the <code>StaticContext</code>
  * interface, which means it cannot be used directly by Saxon when parsing a query. Instead it is first copied
- * to create a <code>QueryModule</code> object, which does implement the <code>StaticContext</code> interface.
- * <p/>
+ * to create a <code>QueryModule</code> object, which does implement the <code>StaticContext</code> interface.</p>
  * <p>The application constructs a StaticQueryContext
  * and initializes it with information about the context, for example, default namespaces, base URI, and so on.
  * When a query is compiled using this StaticQueryContext, the query parser makes a copy of the StaticQueryContext
@@ -46,12 +45,10 @@ import java.util.*;
  * such as namespace and variable declarations that can occur at any point in the query. The query parser does
  * not modify the original StaticQueryContext supplied by the calling application, which may therefore be used
  * for compiling multiple queries, serially or even in multiple threads.</p>
- * <p/>
  * <p>This class forms part of Saxon's published XQuery API. Methods that
  * are considered stable are labelled with the JavaDoc "since" tag.
  * The value 8.4 indicates a method introduced at or before Saxon 8.4; other
  * values indicate the version at which the method was introduced.</p>
- * <p/>
  * <p>In the longer term, this entire API may at some stage be superseded by a proposed
  * standard Java API for XQuery.</p>
  *
@@ -69,7 +66,7 @@ public class StaticQueryContext {
     private boolean preserveNamespaces = true;
     private int constructionMode = Validation.PRESERVE;
     /*@Nullable*/ private NamespaceResolver externalNamespaceResolver = null;
-    private String defaultFunctionNamespace;
+    private String defaultFunctionNamespace = NamespaceConstant.FN;
     /*@Nullable*/ private String defaultElementNamespace = NamespaceConstant.NULL;
     private ItemType requiredContextItemType = AnyItemType.getInstance();
     private boolean preserveSpace = false;
@@ -81,6 +78,7 @@ public class StaticQueryContext {
     private int languageVersion = 30;
     private String defaultCollationName;
     private Location moduleLocation;
+    private OptimizerOptions optimizerOptions;
 
 
 
@@ -93,38 +91,43 @@ public class StaticQueryContext {
 
     /**
      * Create a StaticQueryContext using a given Configuration. This creates a StaticQueryContext for a main module
-     * (that is, a module that is not a library module).
+     * (that is, a module that is not a library module). The static query context is initialized from the
+     * default static query context held in the configuration.
      *
      * @param config the Saxon Configuration
      * @since 8.4
-     * @deprecated since 9.2. Use config.newStaticQueryContext(). This will create a StaticQueryContext with
-     *             capabilities appropriate to the configuration (for example, offering XQuery 1.1 support).
      */
 
-    public StaticQueryContext(/*@NotNull*/ Configuration config) {
-        //copyFrom(config.getDefaultStaticQueryContext());
-        this.config = config;
-        namePool = config.getNamePool();
-        errorListener = config.getErrorListener();
-        if (errorListener instanceof StandardErrorListener) {
-            errorListener = ((StandardErrorListener) errorListener).makeAnother(Configuration.XQUERY);
-            ((StandardErrorListener) errorListener).setRecoveryPolicy(Configuration.DO_NOT_RECOVER);
-        }
-        defaultCollationName = config.getDefaultCollationName();
-        reset();
+    public StaticQueryContext(Configuration config) {
+        this(config, true);
     }
 
     /**
      * Create a StaticQueryContext using a given Configuration. This creates a StaticQueryContext for a main module
-     * (that is, a module that is not a library module).
-     * <p/>
-     * <p>This method is primarily for internal use. The recommended way to create a StaticQueryContext is by
-     * using the factory method Configuration.newStaticQueryContext().</p>
+     * (that is, a module that is not a library module). The static query context is initialized from the
+     * default static query context held in the configuration only if the {@code initialize} argument is set to true.
      *
-     * @param config  the Saxon Configuration
-     * @param initial if set, this is the StaticQueryContext owned by the Configuration
+     * @param config the Saxon Configuration
+     * @param initialize specifies whether the static context should be initialized using defaults held in the
+     *                   configuration. Setting this to true has the same effect as calling the single-argument
+     *                   constructor. Setting it to false is typically used only when creating the default
+     *                   static query context held by the configuration.
+     * @since 9.9
      */
 
+    public StaticQueryContext(Configuration config, boolean initialize) {
+        this.config = config;
+        this.namePool = config.getNamePool();
+        if (initialize) {
+            copyFrom(config.getDefaultStaticQueryContext());
+        } else {
+            userDeclaredNamespaces = new HashMap<>();
+            userDeclaredVariables = new HashSet<>();
+            optimizerOptions = config.getOptimizerOptions();
+            clearNamespaces();
+        }
+    }
+
     /**
      * Create a copy of a supplied StaticQueryContext
      *
@@ -135,16 +138,33 @@ public class StaticQueryContext {
         copyFrom(c);
     }
 
+    /**
+     * Static method used to create the default static context: intended for internal use only
+     */
+
+    public static StaticQueryContext makeDefaultStaticQueryContext(Configuration config) {
+        StaticQueryContext sqc = new StaticQueryContext();
+        sqc.config = config;
+        sqc.namePool = config.getNamePool();
+        sqc.reset();
+        return sqc;
+    }
+
+    /**
+     * Copy details from another StaticQueryContext
+     * @param c the other StaticQueryContext
+     */
+
     public void copyFrom(/*@NotNull*/ StaticQueryContext c) {
         config = c.config;
         namePool = c.namePool;
         baseURI = c.baseURI;
         moduleURIResolver = c.moduleURIResolver;
         if (c.userDeclaredNamespaces != null) {
-            userDeclaredNamespaces = new HashMap<String, String>(c.userDeclaredNamespaces);
+            userDeclaredNamespaces = new HashMap<>(c.userDeclaredNamespaces);
         }
         if (c.userDeclaredVariables != null) {
-            userDeclaredVariables = new HashSet<GlobalVariable>(c.userDeclaredVariables);
+            userDeclaredVariables = new HashSet<>(c.userDeclaredVariables);
         }
         inheritNamespaces = c.inheritNamespaces;
         preserveNamespaces = c.preserveNamespaces;
@@ -159,6 +179,7 @@ public class StaticQueryContext {
         errorListener = c.errorListener;
         codeInjector = c.codeInjector;
         isUpdating = c.isUpdating;
+        optimizerOptions = c.optimizerOptions;
     }
 
 
@@ -169,7 +190,7 @@ public class StaticQueryContext {
      */
 
     public void reset() {
-        userDeclaredNamespaces = new HashMap<String, String>(10);
+        userDeclaredNamespaces = new HashMap<>(10);
         externalNamespaceResolver = null;
         errorListener = config.getErrorListener();
         if (errorListener instanceof StandardErrorListener) {
@@ -184,8 +205,10 @@ public class StaticQueryContext {
         defaultFunctionNamespace = NamespaceConstant.FN;
         defaultElementNamespace = NamespaceConstant.NULL;
         moduleURIResolver = null;
+        defaultCollationName = config.getDefaultCollationName();
         clearNamespaces();
         isUpdating = false;
+        optimizerOptions = config.getOptimizerOptions();
 
     }
 
@@ -303,7 +326,6 @@ public class StaticQueryContext {
      * Convenience method for building Saxon's internal representation of a source XML
      * document. The document will be built using Configuration (and NamePool) associated
      * with this StaticQueryContext.
-     * <p/>
      * <p>This method is retained for backwards compatibility; however, it is merely a wrapper
      * around the method {@link Configuration#buildDocumentTree}, which should be called in preference.</p>
      *
@@ -522,11 +544,33 @@ public class StaticQueryContext {
         return moduleLocation;
     }
 
+    /**
+     * Set the optimizer options to be used for compiling queries that use this static context.
+     * By default the optimizer options set in the {@link Configuration} are used.
+     *
+     * @param options the optimizer options to be used
+     */
+
+    public void setOptimizerOptions(OptimizerOptions options) {
+        this.optimizerOptions = options;
+    }
+
+    /**
+     * Get the optimizer options being used for compiling queries that use this static context.
+     * By default the optimizer options set in the {@link Configuration} are used.
+     *
+     * @return the optimizer options being used
+     */
+
+    public OptimizerOptions getOptimizerOptions() {
+        return this.optimizerOptions;
+    }
+
+
     /**
      * Prepare an XQuery query for subsequent evaluation. The source text of the query
      * is supplied as a String. The base URI of the query is taken from the static context,
      * and defaults to the current working directory.
-     * <p/>
      * <p>Note that this interface makes the caller responsible for decoding the query and
      * presenting it as a string of characters. This means it is likely that any encoding
      * specified in the query prolog will be ignored.</p>
@@ -550,7 +594,6 @@ public class StaticQueryContext {
         }
         qp.setStreaming(isStreaming());
         QueryModule mainModule = new QueryModule(this);
-        qp.setDisableCycleChecks(true);
         return qp.makeXQueryExpression(query, mainModule, config);
     }
 
@@ -558,7 +601,6 @@ public class StaticQueryContext {
      * Prepare an XQuery query for subsequent evaluation. The Query is supplied
      * in the form of a Reader. The base URI of the query is taken from the static context,
      * and defaults to the current working directory.
-     * <p/>
      * <p>Note that this interface makes the Reader responsible for decoding the query and
      * presenting it as a stream of characters. This means it is likely that any encoding
      * specified in the query prolog will be ignored. Also, some implementations of Reader
@@ -574,7 +616,7 @@ public class StaticQueryContext {
      */
 
     /*@Nullable*/
-    public XQueryExpression compileQuery(/*@NotNull*/ Reader source)
+    public synchronized XQueryExpression compileQuery(/*@NotNull*/ Reader source)
             throws XPathException, IOException {
         char[] buffer = new char[4096];
         StringBuilder sb = new StringBuilder(4096);
@@ -607,12 +649,11 @@ public class StaticQueryContext {
      * @throws net.sf.saxon.trans.XPathException
      *                             if the syntax of the expression is wrong, or if it references namespaces,
      *                             variables, or functions that have not been declared, or any other static error is reported.
-     * @throws java.io.IOException if a failure occurs reading the supplied input.
      * @since 8.5
      */
     /*@Nullable*/
-    public XQueryExpression compileQuery(/*@NotNull*/ InputStream source, /*@Nullable*/ String encoding)
-            throws XPathException, IOException {
+    public synchronized XQueryExpression compileQuery(/*@NotNull*/ InputStream source, /*@Nullable*/ String encoding)
+            throws XPathException {
         String query = QueryReader.readInputStream(source, encoding, config.getValidCharacterChecker());
         return compileQuery(query);
     }
@@ -788,7 +829,6 @@ public class StaticQueryContext {
      * Set an external namespace resolver. If a namespace prefix cannot be resolved using any
      * other mechanism, then as a last resort the external namespace resolver is called to
      * obtain a URI for the given prefix.
-     * <p/>
      * <p><i>Changed in Saxon 9.0 so that the namespaces resolved by the external namespace resolver
      * are available at run-time, just like namespaces declared in the query prolog. In consequence,
      * the supplied NamespaceResolver must now implement the
@@ -892,10 +932,10 @@ public class StaticQueryContext {
         var.setVariableQName(qName);
         var.setRequiredType(type);
         if (value != null) {
-            var.setSelectExpression(Literal.makeLiteral(SequenceTool.toGroundedValue(value)));
+            var.setSelectExpression(Literal.makeLiteral(((Sequence<Item>) value).materialize()));
         }
         if (userDeclaredVariables == null) {
-            userDeclaredVariables = new HashSet<GlobalVariable>();
+            userDeclaredVariables = new HashSet<>();
         }
         userDeclaredVariables.add(var);
     }
@@ -1171,10 +1211,8 @@ public class StaticQueryContext {
     /**
      * Set the ErrorListener to be used to report compile-time errors in a query. This will also
      * be the default for the run-time error listener used to report dynamic errors.
-     * <p/>
      * <p>If the supplied listener is a StandardErrorListener, then a new copy will be made
      * using {@link StandardErrorListener#makeAnother(int)}</p>
-     * <p/>
      * <p>If the supplied listener is not an {@link UnfailingErrorListener}, then it will be wrapped
      * in a {@link DelegatingErrorListener}.</p>
      *
