@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,9 +9,7 @@ package net.sf.saxon.query;
 
 import net.sf.saxon.Configuration;
 import net.sf.saxon.Controller;
-import net.sf.saxon.event.PipelineConfiguration;
-import net.sf.saxon.event.Receiver;
-import net.sf.saxon.event.SequenceReceiver;
+import net.sf.saxon.event.*;
 import net.sf.saxon.expr.Expression;
 import net.sf.saxon.expr.ExpressionOwner;
 import net.sf.saxon.expr.PackageData;
@@ -20,8 +18,11 @@ import net.sf.saxon.expr.instruct.Executable;
 import net.sf.saxon.expr.instruct.GlobalVariable;
 import net.sf.saxon.expr.instruct.SlotManager;
 import net.sf.saxon.expr.parser.*;
-import net.sf.saxon.lib.*;
+import net.sf.saxon.lib.SerializerFactory;
+import net.sf.saxon.lib.TraceListener;
+import net.sf.saxon.lib.UnfailingErrorListener;
 import net.sf.saxon.om.*;
+import net.sf.saxon.serialize.SerializationProperties;
 import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.SaxonErrorCode;
 import net.sf.saxon.trans.XPathException;
@@ -38,7 +39,6 @@ import java.util.*;
  * XQueryExpression represents a compiled query. This object is immutable and thread-safe,
  * the same compiled query may be executed many times in series or in parallel. The object
  * can be created only by using the compileQuery method of the QueryProcessor class.
- * <p/>
  * <p>Various methods are provided for evaluating the query, with different options for
  * delivery of the results.</p>
  */
@@ -67,9 +67,9 @@ public class XQueryExpression implements Location, ExpressionOwner {
         executable = exec;
         this.mainModule = mainModule;
         exp.setRetainedStaticContext(mainModule.makeRetainedStaticContext());
-        Optimizer optimizer = config.obtainOptimizer();
         try {
             ExpressionVisitor visitor = ExpressionVisitor.make(mainModule);
+            Optimizer optimizer = visitor.obtainOptimizer();
             visitor.setOptimizeForStreaming(streaming);
             exp = exp.simplify();
             exp.checkForUpdatingSubexpressions();
@@ -77,13 +77,24 @@ public class XQueryExpression implements Location, ExpressionOwner {
             Expression e2 = exp.typeCheck(visitor, cit);
             if (e2 != exp) {
                 e2.setRetainedStaticContext(exp.getRetainedStaticContext());
+                e2.setParentExpression(null);
                 exp = e2;
             }
-            if (optimizer.getOptimizerOptions().isSet(OptimizerOptions.MISCELLANEOUS)) {
-                exp = exp.optimize(visitor, cit);
+            if (optimizer.isOptionSet(OptimizerOptions.MISCELLANEOUS)) {
+                e2 = exp.optimize(visitor, cit);
+                if (e2 != exp) {
+                    e2.setRetainedStaticContext(exp.getRetainedStaticContext());
+                    e2.setParentExpression(null);
+                    exp = e2;
+                }
             }
-            if (optimizer.getOptimizerOptions().isSet(OptimizerOptions.LOOP_LIFTING)) {
-                exp = LoopLifter.process(exp, visitor, cit);
+            if (optimizer.isOptionSet(OptimizerOptions.LOOP_LIFTING)) {
+                e2 = LoopLifter.process(exp, visitor, cit);
+                if (e2 != exp) {
+                    e2.setRetainedStaticContext(exp.getRetainedStaticContext());
+                    e2.setParentExpression(null);
+                    exp = e2;
+                }
             }
         } catch (XPathException err) {
             //err.printStackTrace();
@@ -199,7 +210,6 @@ public class XQueryExpression implements Location, ExpressionOwner {
 
     /**
      * Get a list containing the names of the external variables in the query.
-     * <p/>
      * <p><i>Changed in Saxon 9.0 to return an array of StructuredQName objects rather than
      * integer fingerprints.</i></p>
      *
@@ -218,7 +228,9 @@ public class XQueryExpression implements Location, ExpressionOwner {
     }
 
     /**
-     * Execute a the compiled Query, returning the results as a List.
+     * Execute a the compiled Query, returning the results as a List of objects
+     * obtained by converting the items in the query result to the nearest appropriate
+     * Java type
      *
      * @param env Provides the dynamic query evaluation context
      * @return The results of the expression, as a List. The List represents the sequence
@@ -235,12 +247,8 @@ public class XQueryExpression implements Location, ExpressionOwner {
         if (isUpdateQuery()) {
             throw new XPathException("Cannot call evaluate() on an updating query");
         }
-        SequenceIterator iterator = iterator(env);
-        ArrayList<Object> list = new ArrayList<Object>(100);
-        Item item;
-        while ((item = iterator.next()) != null) {
-            list.add(SequenceTool.convertToJava(item));
-        }
+        ArrayList<Object> list = new ArrayList<>(100);
+        iterator(env).forEachOrFail(item -> list.add(SequenceTool.convertToJava(item)));
         return list;
     }
 
@@ -275,12 +283,9 @@ public class XQueryExpression implements Location, ExpressionOwner {
      * any conversion of the returned items to "native" Java classes. The iterator will
      * deliver a sequence of Item objects, each item being either a NodeInfo (representing
      * a node) or an AtomicValue (representing an atomic value).
-     * <p/>
      * <p>To get the results of the query in the form of an XML document in which each
      * item is wrapped by an element indicating its type, use:</p>
-     * <p/>
      * <p><code>QueryResult.wrap(iterator(env))</code></p>
-     * <p/>
      * <p>To serialize the results to a file, use the QueryResult.serialize() method.</p>
      *
      * @param env Provides the dynamic query evaluation context
@@ -293,7 +298,7 @@ public class XQueryExpression implements Location, ExpressionOwner {
      */
 
     /*@NotNull*/
-    public SequenceIterator iterator(/*@NotNull*/ DynamicQueryContext env) throws XPathException {
+    public SequenceIterator<? extends Item> iterator(/*@NotNull*/ DynamicQueryContext env) throws XPathException {
         if (isUpdateQuery()) {
             throw new XPathException("Cannot call iterator() on an updating query");
         }
@@ -317,11 +322,11 @@ public class XQueryExpression implements Location, ExpressionOwner {
 
             context.openStackFrame(stackFrameMap);
 
-            SequenceIterator iterator = expression.iterate(context);
+            SequenceIterator<? extends Item> iterator = expression.iterate(context);
             if ((iterator.getProperties() & SequenceIterator.GROUNDED) != 0) {
                 return iterator;
             } else {
-                return new ErrorReportingIterator(iterator, controller.getErrorListener());
+                return new ErrorReportingIterator<>(iterator, controller.getErrorListener());
             }
         } catch (XPathException err) {
             TransformerException terr = err;
@@ -356,7 +361,7 @@ public class XQueryExpression implements Location, ExpressionOwner {
      * @throws XPathException if the query fails.
      */
 
-    public void run(/*@NotNull*/ DynamicQueryContext env, /*@NotNull*/ Result result, Properties outputProperties) throws XPathException {
+    public void run(/*@NotNull*/ DynamicQueryContext env, Result result, Properties outputProperties) throws XPathException {
         if (isUpdateQuery()) {
             throw new XPathException("Cannot call run() on an updating query");
         }
@@ -388,14 +393,14 @@ public class XQueryExpression implements Location, ExpressionOwner {
         context.openStackFrame(stackFrameMap);
 
         boolean mustClose = result instanceof StreamResult && ((StreamResult) result).getOutputStream() == null;
-        SequenceReceiver out;
-        if (result instanceof SequenceReceiver) {
-            out = (SequenceReceiver)result;
+        Receiver out;
+        if (result instanceof Receiver) {
+            out = (Receiver)result;
         } else {
             SerializerFactory sf = context.getConfiguration().getSerializerFactory();
             PipelineConfiguration pipe = controller.makePipelineConfiguration();
             pipe.setHostLanguage(Configuration.XQUERY);
-            out = sf.getReceiver(result, pipe, actualProperties);
+            out = sf.getReceiver(result, new SerializationProperties(actualProperties), pipe);
 
 //            String itemSeparator = actualProperties.getProperty(SaxonOutputKeys.ITEM_SEPARATOR);
 //            if (out instanceof ComplexContentOutputter && itemSeparator != null) {
@@ -403,6 +408,10 @@ public class XQueryExpression implements Location, ExpressionOwner {
 //                context.setReceiver(out);
 //            }
         }
+
+        NamespaceReducer nr = new NamespaceReducer(out);
+        out = new ComplexContentOutputter(nr);
+
         context.setReceiver(out);
         out.open();
 
@@ -413,13 +422,23 @@ public class XQueryExpression implements Location, ExpressionOwner {
             controller.reportFatalError(err);
             throw err;
         } finally {
-            if (tracer != null) {
-                tracer.close();
+            try {
+                if (tracer != null) {
+                    tracer.close();
+                }
+                out.close();
+            } catch (XPathException e) {
+                e.printStackTrace();
             }
-            out.close();
         }
+        if (result instanceof StreamResult) {
+            closeStreamIfNecessary((StreamResult) result, mustClose);
+        }
+    }
+
+    protected void closeStreamIfNecessary(StreamResult result, boolean mustClose) throws XPathException {
         if (mustClose) {
-            OutputStream os = ((StreamResult) result).getOutputStream();
+            OutputStream os = result.getOutputStream();
             if (os != null) {
                 try {
                     os.close();
@@ -444,12 +463,17 @@ public class XQueryExpression implements Location, ExpressionOwner {
         throw new XPathException("Streaming requires Saxon-EE");
     }
 
+    /**
+     * Validate the serialization properties requested; any problems result in warnings being output
+     * @param controller The query controller
+     * @param outputProperties the requested serialization properties (may be null)
+     * @return the properties after validation and normalization
+     */
 
-    /*@NotNull*/
-    protected Properties validateOutputProperties(/*@NotNull*/ Controller controller, /*@Nullable*/ Properties outputProperties) throws XPathException {
+    protected Properties validateOutputProperties(Controller controller, /*@Nullable*/ Properties outputProperties) {
         // Validate the serialization properties requested
 
-        Properties baseProperties = controller.getExecutable().getDefaultOutputProperties();
+        Properties baseProperties = controller.getExecutable().getPrimarySerializationProperties().getProperties();
         SerializerFactory sf = controller.getConfiguration().getSerializerFactory();
         if (outputProperties != null) {
             Enumeration iter = outputProperties.propertyNames();
@@ -521,18 +545,15 @@ public class XQueryExpression implements Location, ExpressionOwner {
 
     /**
      * Run an updating query, writing back eligible updated node to persistent storage.
-     * <p/>
      * <p>A node is eligible to be written back to disk if it is present in the document pool,
      * which generally means that it was originally read using the doc() or collection() function.</p>
-     * <p/>
      * <p>On completion of this method it is generally unsafe to rely on the contents or relationships
      * of NodeInfo objects that were obtained before the updating query was run. Such objects may or may not
      * reflect the results of the update operations. This is especially true in the case of nodes that
      * are part of a subtree that has been deleted (detached from its parent). Instead, the new updated
      * tree should be accessed by navigation from the root nodes returned by this method.</p>
-     * <p/>
      * <p>If one or more eligible updated nodes cannot be written back to disk, perhaps because the URI
-     * identifies a non-updatable location, then an exception is thrown. In this case it is undefined
+     * identifies a non-updatable location, then an exception is thrown. In this case it is undefined</p>
      *
      * @param dynamicEnv the dynamic context for query execution
      * @param agent      a callback class that is called to process each document updated by the query
@@ -550,22 +571,7 @@ public class XQueryExpression implements Location, ExpressionOwner {
         XPathContextMajor context = controller.newXPathContext();
 
         if (contextItem != null) {
-//            // Check the type of the externally-supplied context item against the API-defined required type
-//            if (!mainModule.getUserQueryContext().getRequiredContextItemType().matches(contextItem, getConfiguration().getTypeHierarchy())) {
-//                throw new XPathException("The supplied context item does not match the required context item type", "XPTY0004");
-//            }
-//            GlobalContextRequirement req = controller.getExecutable().getGlobalContextRequirement();
-//            if (req != null) {
-//                if (!req.mayBeSupplied) {
-//                    controller.setGlobalContextItem(null);
-//                    controller.warning("The query does not allow a global context item so the supplied value is ignored", "SXWN9000");
-//                    return context;
-//                }
-//                if (!req.requiredItemType.matches(contextItem, controller.getConfiguration().getTypeHierarchy())) {
-//                    throw new XPathException("The supplied context item does not match its declared type", "XPTY0004");
-//                }
-//            }
-            ManualIterator single = new ManualIterator(contextItem);
+            ManualIterator<Item> single = new ManualIterator<Item>(contextItem);
             context.setCurrentIterator(single);
             controller.setGlobalContextItem(contextItem);
         }
@@ -644,7 +650,6 @@ public class XQueryExpression implements Location, ExpressionOwner {
 
     /**
      * Return the public identifier for the current document event.
-     * <p/>
      * <p>The return value is the public identifier of the document
      * entity or of the external parsed entity in which the markup that
      * triggered the event appears.</p>
@@ -660,11 +665,9 @@ public class XQueryExpression implements Location, ExpressionOwner {
 
     /**
      * Return the system identifier for the current document event.
-     * <p/>
      * <p>The return value is the system identifier of the document
      * entity or of the external parsed entity in which the markup that
      * triggered the event appears.</p>
-     * <p/>
      * <p>If the system identifier is a URL, the parser must resolve it
      * fully before passing it to the application.</p>
      *
@@ -679,12 +682,10 @@ public class XQueryExpression implements Location, ExpressionOwner {
 
     /**
      * Return the line number where the current document event ends.
-     * <p/>
      * <p><strong>Warning:</strong> The return value from the method
      * is intended only as an approximation for the sake of error
      * reporting; it is not intended to provide sufficient information
      * to edit the character content of the original XML document.</p>
-     * <p/>
      * <p>The return value is an approximation of the line number
      * in the document entity or external parsed entity where the
      * markup that triggered the event appears.</p>
@@ -698,12 +699,10 @@ public class XQueryExpression implements Location, ExpressionOwner {
 
     /**
      * Return the character position where the current document event ends.
-     * <p/>
      * <p><strong>Warning:</strong> The return value from the method
      * is intended only as an approximation for the sake of error
      * reporting; it is not intended to provide sufficient information
      * to edit the character content of the original XML document.</p>
-     * <p/>
      * <p>The return value is an approximation of the column number
      * in the document entity or external parsed entity where the
      * markup that triggered the event appears.</p>
@@ -743,17 +742,17 @@ public class XQueryExpression implements Location, ExpressionOwner {
      * any exceptions that are raised to the ErrorListener
      */
 
-    private class ErrorReportingIterator implements SequenceIterator {
-        private SequenceIterator base;
+    private class ErrorReportingIterator<T extends Item> implements SequenceIterator<T> {
+        private SequenceIterator<T> base;
         private UnfailingErrorListener listener;
 
-        public ErrorReportingIterator(SequenceIterator base, UnfailingErrorListener listener) {
+        public ErrorReportingIterator(SequenceIterator<T> base, UnfailingErrorListener listener) {
             this.base = base;
             this.listener = listener;
         }
 
         /*@Nullable*/
-        public Item next() throws XPathException {
+        public T next() throws XPathException {
             try {
                 return base.next();
             } catch (XPathException e1) {
