@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -21,6 +21,7 @@ import net.sf.saxon.trans.FunctionStreamability;
 import net.sf.saxon.trans.GlobalVariableManager;
 import net.sf.saxon.trans.SymbolicName;
 import net.sf.saxon.trans.XPathException;
+import net.sf.saxon.tree.jiter.PairIterator;
 import net.sf.saxon.value.SequenceType;
 
 import java.util.*;
@@ -34,7 +35,7 @@ public class XQueryFunction implements InstructionInfo, Declaration {
     private List<UserFunctionParameter> arguments;
     private SequenceType resultType;
     private Expression body = null;
-    private List<UserFunctionResolvable> references = new ArrayList<UserFunctionResolvable>(10);
+    private List<UserFunctionResolvable> references = new ArrayList<>(10);
     private Location location;
     /*@Nullable*/ private UserFunction compiledFunction = null;
     private boolean memoFunction;
@@ -48,7 +49,7 @@ public class XQueryFunction implements InstructionInfo, Declaration {
      */
 
     public XQueryFunction() {
-        arguments = new ArrayList<UserFunctionParameter>(8);
+        arguments = new ArrayList<>(8);
     }
 
     /**
@@ -356,28 +357,40 @@ public class XQueryFunction implements InstructionInfo, Declaration {
 
                 // type-check the body of the function
 
-                RetainedStaticContext rsc = getStaticContext().makeRetainedStaticContext();
-                body.setRetainedStaticContext(rsc);
+                RetainedStaticContext rsc = null;
+                try {
+                    rsc = getStaticContext().makeRetainedStaticContext();
+                    body.setRetainedStaticContext(rsc);
 
-                ExpressionVisitor visitor = ExpressionVisitor.make(staticContext);
-                body = body.simplify().typeCheck(visitor, ContextItemStaticInfo.ABSENT);
+                    ExpressionVisitor visitor = ExpressionVisitor.make(staticContext);
+                    body = body.simplify().typeCheck(visitor, ContextItemStaticInfo.ABSENT);
 
-                // Try to extract new global variables from the body of the function
-                //body = config.getOptimizer().promoteExpressionsToGlobal(body, visitor);
+                    // Try to extract new global variables from the body of the function
+                    //body = config.getOptimizer().promoteExpressionsToGlobal(body, visitor);
 
-                //body.setContainer(this);
-                RoleDiagnostic role =
-                        new RoleDiagnostic(RoleDiagnostic.FUNCTION_RESULT, functionName.getDisplayName(), 0);
-                //role.setSourceLocator(this);
-                body = config.getTypeChecker(false).staticTypeCheck(body, resultType, role, visitor);
-                if (staticContext.getUserQueryContext().isCompileWithTracing()) {
-                    namespaceResolver = staticContext.getNamespaceResolver();
-                    TraceExpression trace = new TraceExpression(body);
-                    trace.setConstructType(StandardNames.XSL_FUNCTION);
-                    trace.setObjectName(functionName);
-                    trace.setNamespaceResolver(staticContext.getNamespaceResolver());
-                    trace.setLocation(location);
-                    body = trace;
+                    //body.setContainer(this);
+                    RoleDiagnostic role =
+                            new RoleDiagnostic(RoleDiagnostic.FUNCTION_RESULT, functionName.getDisplayName(), 0);
+                    //role.setSourceLocator(this);
+                    body = config.getTypeChecker(false).staticTypeCheck(body, resultType, role, visitor);
+                    if (staticContext.getUserQueryContext().isCompileWithTracing()) {
+                        namespaceResolver = staticContext.getNamespaceResolver();
+                        TraceExpression trace = new TraceExpression(body);
+                        trace.setConstructType(StandardNames.XSL_FUNCTION);
+                        trace.setObjectName(functionName);
+                        trace.setNamespaceResolver(staticContext.getNamespaceResolver());
+                        trace.setLocation(location);
+                        body = trace;
+                    }
+                } catch (XPathException e) {
+                    e.maybeSetLocation(this);
+                    if (e.isReportableStatically()) {
+                        throw e;
+                    } else {
+                        Expression newBody = new ErrorExpression(e);
+                        ExpressionTool.copyLocationInfo(body, newBody);
+                        body = newBody;
+                    }
                 }
 
                 compiledFunction = config.newUserFunction(memoFunction, FunctionStreamability.UNCLASSIFIED);
@@ -409,13 +422,13 @@ public class XQueryFunction implements InstructionInfo, Declaration {
      *
      * @throws net.sf.saxon.trans.XPathException
      *          if execution fails, for example because the function is updating
-     *          and contains constructs not allowed in an updating function, or vice-versa.
+     *                                           and contains constructs not allowed in an updating function, or vice-versa.
      */
 
     public void optimize() throws XPathException {
         body.checkForUpdatingSubexpressions();
         if (isUpdating) {
-            if (!ExpressionTool.isAllowedInUpdatingContext(body)) {
+            if (ExpressionTool.isNotAllowedInUpdatingContext(body)) {
                 XPathException err = new XPathException(
                         "The body of an updating function must be an updating expression", "XUST0002");
                 err.setLocator(body.getLocation());
@@ -432,13 +445,16 @@ public class XQueryFunction implements InstructionInfo, Declaration {
         }
         ExpressionVisitor visitor = ExpressionVisitor.make(staticContext);
         Configuration config = staticContext.getConfiguration();
-        Optimizer opt = config.obtainOptimizer();
+        Optimizer opt = visitor.obtainOptimizer();
         int arity = arguments.size();
-        if (opt.getOptimizerOptions().isSet(OptimizerOptions.MISCELLANEOUS)) {
+        if (opt.isOptionSet(OptimizerOptions.MISCELLANEOUS)) {
             body = body.optimize(visitor, ContextItemStaticInfo.ABSENT);
-            if (opt.isOptionSet(OptimizerOptions.LOOP_LIFTING)) {
-                body = LoopLifter.process(body, visitor, ContextItemStaticInfo.ABSENT);
-            }
+        }
+        body.setParentExpression(null);
+        if (opt.isOptionSet(OptimizerOptions.LOOP_LIFTING)) {
+            body = LoopLifter.process(body, visitor, ContextItemStaticInfo.ABSENT);
+        }
+        if (opt.isOptionSet(OptimizerOptions.EXTRACT_GLOBALS)) {
             final Executable exec = ((QueryModule)getStaticContext()).getExecutable();
             GlobalVariableManager manager = new GlobalVariableManager() {
                 public void addGlobalVariable(GlobalVariable variable) throws XPathException {
@@ -462,19 +478,19 @@ public class XQueryFunction implements InstructionInfo, Declaration {
             if (b2 != null) {
                 body = body.optimize(visitor, ContextItemStaticInfo.ABSENT);
             }
-
-            // mark tail calls within the function body
-
-            if (!isUpdating) {
-                int tailCalls = ExpressionTool.markTailFunctionCalls(body, functionName, arity);
-                if (tailCalls != 0) {
-                    compiledFunction.setBody(body);
-                    compiledFunction.setTailRecursive(tailCalls > 0, tailCalls > 1);
-                    body = new TailCallLoop(compiledFunction, body);
-                }
-            }
-            compiledFunction.setBody(body);
         }
+        
+        // mark tail calls within the function body
+        if (opt.getOptimizerOptions().isSet(OptimizerOptions.MISCELLANEOUS) && !isUpdating) {
+            int tailCalls = ExpressionTool.markTailFunctionCalls(body, functionName, arity);
+            if (tailCalls != 0) {
+                compiledFunction.setBody(body);
+                compiledFunction.setTailRecursive(tailCalls > 0, tailCalls > 1);
+                body = new TailCallLoop(compiledFunction, body);
+            }
+        }
+        compiledFunction.setBody(body);
+
         compiledFunction.computeEvaluationMode();
         ExpressionTool.allocateSlots(body, arity, compiledFunction.getStackFrameMap());
         if (config.isGenerateByteCode(Configuration.XQUERY)) {
@@ -493,7 +509,7 @@ public class XQueryFunction implements InstructionInfo, Declaration {
      * Fix up references to this function
      */
 
-    public void fixupReferences() throws XPathException {
+    public void fixupReferences() {
         for (UserFunctionResolvable ufc : references) {
             ufc.setFunction(compiledFunction);
         }
@@ -515,7 +531,7 @@ public class XQueryFunction implements InstructionInfo, Declaration {
         }
 
         // clear the list of references, so that more can be added in another module
-        references = new ArrayList<UserFunctionResolvable>(0);
+        references = new ArrayList<>(0);
 
     }
 
@@ -595,7 +611,7 @@ public class XQueryFunction implements InstructionInfo, Declaration {
      * Return the public identifier for the current document event.
      *
      * @return A string containing the public identifier, or
-     *         null if none is available.
+     * null if none is available.
      * @see #getSystemId
      */
     /*@Nullable*/
@@ -660,7 +676,7 @@ public class XQueryFunction implements InstructionInfo, Declaration {
 
     /*@NotNull*/
     public Iterator<String> getProperties() {
-        return new PairIterator<String>("name", "as");
+        return new PairIterator<>("name", "as");
     }
 
     /**
