@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -54,29 +54,27 @@ public class CollectionURIResolverWrapper implements CollectionFinder {
 
             public Iterator<String> getResourceURIs(XPathContext context) throws XPathException {
                 List<String> uris = new ArrayList<String>();
-                SequenceIterator iter = collectionURIResolver.resolve(collectionURI, "", context);
-                Item item;
-                while ((item = iter.next()) != null) {
+                SequenceIterator<? extends Item> iter = collectionURIResolver.resolve(collectionURI, "", context);
+                iter.forEachOrFail(item -> {
                     if (item instanceof AnyURIValue) {
                         uris.add(item.getStringValue());
                     } else if (item instanceof NodeInfo) {
-                        uris.add(((NodeInfo)item).getSystemId());
+                        uris.add(((NodeInfo) item).getSystemId());
                     } else {
                         throw new XPathException("Result of CollectionURIResolver must consist of xs:anyURI values and/or nodes");
                     }
-                }
+                });
                 return uris.iterator();
             }
 
             public Iterator<XmlResource> getResources(XPathContext context) throws XPathException {
                 List<XmlResource> resources = new ArrayList<XmlResource>();
-                SequenceIterator iter = collectionURIResolver.resolve(collectionURI, "", context);
+                SequenceIterator<? extends Item> iter = collectionURIResolver.resolve(collectionURI, "", context);
                 Configuration config = context.getConfiguration();
                 URIResolver uriResolver = config.getURIResolver();
                 ParseOptions options = new ParseOptions(config.getParseOptions());
                 options.setSpaceStrippingRule(whitespaceRules);
-                Item item;
-                while ((item = iter.next()) != null) {
+                iter.forEachOrFail(item -> {
                     if (item instanceof AnyURIValue) {
                         Source source;
                         try {
@@ -87,12 +85,11 @@ public class CollectionURIResolverWrapper implements CollectionFinder {
                         TreeInfo treeInfo = config.buildDocumentTree(source, options);
                         resources.add(new XmlResource(treeInfo.getRootNode()));
                     } else if (item instanceof NodeInfo) {
-                        resources.add(new XmlResource((NodeInfo)item));
+                        resources.add(new XmlResource((NodeInfo) item));
                     } else {
                         throw new XPathException("Result of CollectionURIResolver must consist of xs:anyURI values and/or nodes");
                     }
-                }
-
+                });
                 return resources.iterator();
             }
 
