@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -12,10 +12,10 @@ import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.functions.URIQueryParameters;
 import net.sf.saxon.lib.ParseOptions;
 import net.sf.saxon.lib.Resource;
-import net.sf.saxon.om.Sequence;
+import net.sf.saxon.om.GroundedValue;
 import net.sf.saxon.om.SpaceStrippingRule;
 import net.sf.saxon.trans.XPathException;
-import net.sf.saxon.tree.iter.MappingJavaIterator;
+import net.sf.saxon.tree.jiter.MappingJavaIterator;
 import net.sf.saxon.value.BooleanValue;
 import net.sf.saxon.value.DateTimeValue;
 import net.sf.saxon.value.Int64Value;
@@ -91,35 +91,32 @@ public class DirectoryCollection extends AbstractResourceCollection {
         final boolean metadata = metadataParam != null && metadataParam;
         Iterator<String> resourceURIs = getResourceURIs(context);
 
-        return new MappingJavaIterator<String, Resource>(resourceURIs,
-                                                          new MappingJavaIterator.Mapper<String, Resource>() {
-            @Override
-            public Resource map(String in) {
-                try {
-                    InputDetails details = getInputDetails(in);
-                    details.parseOptions = options;
-                    Resource resource = makeResource(context.getConfiguration(), details, in);
-                    if (resource != null) {
-                        if (metadata) {
-                            return makeMetadataResource(resource, details);
-                        } else {
-                            return resource;
-                        }
-                    }
-                    return null;
-                } catch (XPathException e) {
-                    int onError = params.getOnError();
-                    if (onError == URIQueryParameters.ON_ERROR_FAIL) {
-                        return new FailedResource(in, e);
-                    } else if (onError == URIQueryParameters.ON_ERROR_WARNING) {
-                        context.getController().warning("collection(): failed to parse " + in + ": " + e.getMessage(), e.getErrorCodeLocalPart(), null);
-                        return null;
-                    } else {
-                        return null;
-                    }
-                }
-            }
-        });
+        return new MappingJavaIterator<>(resourceURIs,
+                                         in -> {
+                                             try {
+                                                 InputDetails details = getInputDetails(in);
+                                                 details.parseOptions = options;
+                                                 Resource resource = makeResource(context.getConfiguration(), details, in);
+                                                 if (resource != null) {
+                                                     if (metadata) {
+                                                         return makeMetadataResource(resource, details);
+                                                     } else {
+                                                         return resource;
+                                                     }
+                                                 }
+                                                 return null;
+                                             } catch (XPathException e) {
+                                                 int onError = params.getOnError();
+                                                 if (onError == URIQueryParameters.ON_ERROR_FAIL) {
+                                                     return new FailedResource(in, e);
+                                                 } else if (onError == URIQueryParameters.ON_ERROR_WARNING) {
+                                                     context.getController().warning("collection(): failed to parse " + in + ": " + e.getMessage(), e.getErrorCodeLocalPart(), null);
+                                                     return null;
+                                                 } else {
+                                                     return null;
+                                                 }
+                                             }
+                                         });
     }
 
     /**
@@ -132,7 +129,7 @@ public class DirectoryCollection extends AbstractResourceCollection {
      */
 
     private MetadataResource makeMetadataResource(Resource resource, InputDetails details) {
-        Map<String, Sequence> properties = new HashMap<String, Sequence>();
+        Map<String, GroundedValue> properties = new HashMap<String, GroundedValue>();
         try {
 
             URI uri = new URI(resource.getResourceURI());
