@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -11,18 +11,15 @@ import net.sf.saxon.Configuration;
 import net.sf.saxon.Version;
 import net.sf.saxon.event.NamespaceReducer;
 import net.sf.saxon.event.Receiver;
+import net.sf.saxon.event.ReceiverOptions;
 import net.sf.saxon.event.TreeReceiver;
 import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.expr.parser.ExplicitLocation;
 import net.sf.saxon.expr.sort.RuleBasedSubstringMatcher;
 import net.sf.saxon.expr.sort.SimpleCollation;
 import net.sf.saxon.lib.*;
-import net.sf.saxon.om.Item;
-import net.sf.saxon.om.NodeInfo;
-import net.sf.saxon.om.Sequence;
-import net.sf.saxon.om.StructuredQName;
+import net.sf.saxon.om.*;
 import net.sf.saxon.trans.XPathException;
-import net.sf.saxon.value.SequenceExtent;
 import net.sf.saxon.value.SequenceType;
 
 import javax.xml.transform.Source;
@@ -31,20 +28,20 @@ import java.io.OutputStream;
 import java.io.Writer;
 import java.text.RuleBasedCollator;
 import java.util.Comparator;
+import java.util.Objects;
 
 /**
  * The <code>Processor</code> class serves three purposes: it allows global Saxon configuration options to be set;
  * it acts as a factory for generating XQuery, XPath, and XSLT compilers; and it owns certain shared
  * resources such as the Saxon NamePool and compiled schemas. This is the first object that a
  * Saxon application should create. Once established, a Processor may be used in multiple threads.
- * <p/>
  * <p>It is possible to run more than one Saxon Processor concurrently, but only when running completely
  * independent workloads. Nothing can be shared between Processor instances. Within a query or transformation,
  * all source documents and schemas must be built using the same Processor, which must also be used to
  * compile the query or stylesheet.</p>
  */
 
-public class Processor {
+public class Processor implements Configuration.ApiProvider {
 
     private Configuration config;
     private SchemaManager schemaManager;
@@ -215,11 +212,10 @@ public class Processor {
     /**
      * Register a simple external/extension function that is to be made available within any stylesheet, query,
      * or XPath expression compiled under the control of this processor.
-     * <p/>
      * <p>This interface provides only for simple extension functions that have no side-effects and no dependencies
-     * on the static or dynamic context.
+     * on the static or dynamic context.</p>
      *
-     * @param function the implementation of the extension function.</p>
+     * @param function the implementation of the extension function.
      * @since 9.4
      */
 
@@ -233,7 +229,6 @@ public class Processor {
      * or XPath expression compiled under the control of this processor. This method
      * registers an extension function implemented as an instance of
      * {@link net.sf.saxon.lib.ExtensionFunctionDefinition}, using an arbitrary name and namespace.
-     * <p/>
      * <p>This interface allows extension functions that have dependencies on the static or dynamic
      * context. It also allows an extension function to declare that it has side-effects, in which
      * case calls to the function will be optimized less aggressively than usual, although the semantics
@@ -299,7 +294,6 @@ public class Processor {
      * output documents will be serialized as XML 1.0. This option also affects
      * the characters permitted to appear in queries and stylesheets, and the characters that can appear
      * in names (for example, in path expressions).
-     * <p/>
      * <p>Note that source documents specifying xml version="1.0" or "1.1" are accepted
      * regardless of this setting.</p>
      *
@@ -308,12 +302,15 @@ public class Processor {
      */
 
     public void setXmlVersion(/*@NotNull*/ String version) {
-        if (version.equals("1.0")) {
-            config.setXMLVersion(Configuration.XML10);
-        } else if (version.equals("1.1")) {
-            config.setXMLVersion(Configuration.XML11);
-        } else {
-            throw new IllegalArgumentException("XmlVersion");
+        switch (version) {
+            case "1.0":
+                config.setXMLVersion(Configuration.XML10);
+                break;
+            case "1.1":
+                config.setXMLVersion(Configuration.XML11);
+                break;
+            default:
+                throw new IllegalArgumentException("XmlVersion");
         }
     }
 
@@ -343,6 +340,7 @@ public class Processor {
      * @param value the value of the option to be set.
      * @throws IllegalArgumentException if the property name is not recognized or if the supplied value
      *                                  is not a valid value for the named property.
+     * @deprecated since 9.9 - use {@link #setConfigurationProperty(Feature, Object)}
      */
 
     public void setConfigurationProperty(/*@NotNull*/ String name, /*@NotNull*/ Object value) {
@@ -361,6 +359,7 @@ public class Processor {
      * @return the value of the property, if one is set; or null if the property is unset and there is
      *         no default.
      * @throws IllegalArgumentException if the property name is not recognized
+     * @deprecated since 9.9 - use {@link #getConfigurationProperty(Feature)}
      */
 
 
@@ -369,6 +368,43 @@ public class Processor {
         return config.getConfigurationProperty(name);
     }
 
+    /**
+     * Set a configuration property
+     *
+     * @param feature the option to be set. The names of the options available are listed
+     *              as constants in class {@link net.sf.saxon.lib.Feature}.
+     * @param value the value of the option to be set (which must be of the appropriate type for the
+     *              particular feature.
+     * @throws IllegalArgumentException if the supplied value is not a valid value for the selected feature.
+     * @since 9.9 introduced to give a faster and type-safe alternative to
+     * {@link #setConfigurationProperty(String, Object)}
+     */
+
+    public <T> void setConfigurationProperty(Feature<T> feature, T value) {
+        if (feature == Feature.CONFIGURATION) {
+            config = (Configuration) value;
+        } else {
+            config.setConfigurationProperty(feature, value);
+        }
+    }
+
+    /**
+     * Get the value of a configuration property
+     *
+     * @param feature the option required. The names of the properties available are listed
+     *             as constants in class {@link net.sf.saxon.lib.Feature}.
+     * @return the value of the property, if one is set; or null if the property is unset and there is
+     * no default.
+     * @since 9.9 introduced to give a faster and type-safe alternative to
+     * {@link #getConfigurationProperty(String)}
+     */
+
+
+    /*@Nullable*/
+    public <T> T getConfigurationProperty(Feature<T> feature) {
+        return config.getConfigurationProperty(feature);
+    }
+
     /**
      * Bind a collation URI to a collation
      *
@@ -427,22 +463,24 @@ public class Processor {
      * @throws SaxonApiException if any failure occurs, for example a serialization error
      */
 
-    public void writeXdmValue(/*@NotNull*/ XdmValue value, /*@NotNull*/ Destination destination) throws SaxonApiException {
+    public void writeXdmValue(XdmValue value, Destination destination) throws SaxonApiException {
+        Objects.requireNonNull(value);
+        Objects.requireNonNull(destination);
         try {
             if (destination instanceof Serializer) {
                 ((Serializer)destination).serializeXdmValue(value);
             } else {
-                Receiver out = destination.getReceiver(config);
+                Receiver out = destination.getReceiver(config.makePipelineConfiguration(), config.obtainDefaultSerializationProperties());
                 out = new NamespaceReducer(out);
                 TreeReceiver tree = new TreeReceiver(out);
                 tree.open();
                 tree.startDocument(0);
                 for (XdmItem item : value) {
-                    tree.append((Item) item.getUnderlyingValue(), ExplicitLocation.UNKNOWN_LOCATION, NodeInfo.ALL_NAMESPACES);
+                    tree.append(item.getUnderlyingValue(), ExplicitLocation.UNKNOWN_LOCATION, ReceiverOptions.ALL_NAMESPACES);
                 }
                 tree.endDocument();
                 tree.close();
-                destination.close();
+                destination.closeAndNotify();
             }
         } catch (XPathException err) {
             throw new SaxonApiException(err);
@@ -587,11 +625,11 @@ public class Processor {
         public ExtensionFunctionCall makeCallExpression() {
             return new ExtensionFunctionCall() {
                 @Override
-                public Sequence call(
+                public Sequence<? extends Item> call(
                         /*@NotNull*/ XPathContext context, Sequence[] arguments) throws XPathException {
                     XdmValue[] args = new XdmValue[arguments.length];
                     for (int i = 0; i < args.length; i++) {
-                        Sequence val = SequenceExtent.makeSequenceExtent(arguments[i].iterate());
+                        Sequence val = arguments[i].materialize();
                         args[i] = XdmValue.wrap(val);
                     }
                     try {
