@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -10,7 +10,11 @@ package net.sf.saxon.s9api;
 import net.sf.saxon.Configuration;
 import net.sf.saxon.Controller;
 import net.sf.saxon.event.Builder;
+import net.sf.saxon.event.PipelineConfiguration;
 import net.sf.saxon.event.Receiver;
+import net.sf.saxon.event.SequenceNormalizer;
+import net.sf.saxon.expr.instruct.Executable;
+import net.sf.saxon.expr.instruct.GlobalContextRequirement;
 import net.sf.saxon.expr.instruct.UserFunction;
 import net.sf.saxon.expr.parser.ExplicitLocation;
 import net.sf.saxon.expr.parser.RoleDiagnostic;
@@ -19,13 +23,16 @@ import net.sf.saxon.lib.TraceListener;
 import net.sf.saxon.om.*;
 import net.sf.saxon.query.DynamicQueryContext;
 import net.sf.saxon.query.XQueryExpression;
+import net.sf.saxon.serialize.SerializationProperties;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.tiny.Statistics;
 import net.sf.saxon.tree.tiny.TinyBuilder;
 import net.sf.saxon.type.TypeHierarchy;
-import net.sf.saxon.value.SequenceExtent;
 
-import javax.xml.transform.*;
+import javax.xml.transform.ErrorListener;
+import javax.xml.transform.Source;
+import javax.xml.transform.TransformerException;
+import javax.xml.transform.URIResolver;
 import javax.xml.transform.dom.DOMSource;
 import java.util.HashSet;
 import java.util.Iterator;
@@ -34,26 +41,24 @@ import java.util.Set;
 /**
  * An <code>XQueryEvaluator</code> represents a compiled and loaded query ready for execution.
  * The <code>XQueryEvaluator</code> holds details of the dynamic evaluation context for the query.
- * <p/>
  * <p>An <code>XQueryEvaluator</code> must not be used concurrently in multiple threads.
  * It is safe, however, to reuse the object within a single thread to run the same
  * query several times. Running the query does not change the context
  * that has been established.</p>
- * <p/>
  * <p>An <code>XQueryEvaluator</code> is always constructed by running the <code>Load</code>
  * method of an {@link net.sf.saxon.s9api.XQueryExecutable}.</p>
- * <p/>
  * <p>An <code>XQueryEvaluator</code> is itself a <code>Iterable</code>. This makes it possible to
  * evaluate the results in a for-each expression.</p>
- * <p/>
  * <p>An <code>XQueryEvaluator</code> is itself a <code>Destination</code>. This means it is possible to use
  * one <code>XQueryEvaluator</code> as the destination to receive the results of another transformation,
  * this providing a simple way for transformations to be chained into a pipeline. When the query is executed
- * this way, {@link #setDestination(Destination)} must be called to provide a destination for the result of this query.
- * Note however that a when the input to a query is supplied in this way, it will always be built as a tree in
- * memory, rather than the transformation being streamed.</p>
+ * this way, {@link #setDestination(Destination)} must be called to provide a destination for the result of this query.</p>
+ * <p>As a {@code Destination}, an {@code XQueryEvaluator} performs <b>sequence normalization</b> as defined
+ * in the Serialization specification, including inserting item separators if required. The input to the
+ * query (the global context item) will therefore always be a single document node. This will always be built
+ * as a tree in memory, it will never be streamed.</p>
  */
-public class XQueryEvaluator implements Iterable<XdmItem>, Destination {
+public class XQueryEvaluator extends AbstractDestination implements Iterable<XdmItem> {
 
     private Processor processor;
     private XQueryExpression expression;
@@ -111,13 +116,10 @@ public class XQueryEvaluator implements Iterable<XdmItem>, Destination {
 
     /**
      * Set the source document for the query.
-     * <p/>
      * <p>If the source is an instance of {@link net.sf.saxon.om.NodeInfo}, the supplied node is used
      * directly as the context node of the query.</p>
-     * <p/>
      * <p>If the source is an instance of {@link javax.xml.transform.dom.DOMSource}, the DOM node identified
      * by the DOMSource is wrapped as a Saxon node, and this is then used as the context item</p>
-     * <p/>
      * <p>In all other cases a new Saxon tree is built, by calling
      * {@link net.sf.saxon.s9api.DocumentBuilder#build(javax.xml.transform.Source)}, and the document
      * node of this tree is then used as the context item for the query.</p>
@@ -139,11 +141,17 @@ public class XQueryEvaluator implements Iterable<XdmItem>, Destination {
      * Set the initial context item for the query
      *
      * @param item the initial context item, or null if there is to be no initial context item
+     * @throws SaxonApiException if the query declares the context item and does not define
+     * it to be external
      */
 
-    public void setContextItem(XdmItem item) {
+    public void setContextItem(XdmItem item) throws SaxonApiException {
         if (item != null) {
-            context.setContextItem((Item) item.getUnderlyingValue());
+            GlobalContextRequirement gcr = expression.getExecutable().getGlobalContextRequirement();
+            if (gcr != null && !gcr.isExternal()) {
+                throw new SaxonApiException("The context item for the query is not defined as external");
+            }
+            context.setContextItem(item.getUnderlyingValue());
         }
     }
 
@@ -168,11 +176,16 @@ public class XQueryEvaluator implements Iterable<XdmItem>, Destination {
      *
      * @param name  the name of the external variable, as a QName
      * @param value the value of the external variable, or null to clear a previously set value
+     * @throws SaxonApiUncheckedException if the value is evaluated lazily, and evaluation fails
      */
 
     public void setExternalVariable(QName name, XdmValue value) {
-        context.setParameter(name.getStructuredQName(),
-                (value == null ? null : value.getUnderlyingValue()));
+        try {
+            context.setParameter(name.getStructuredQName(),
+                                 value == null ? null : ((Sequence<? extends Item>) value.getUnderlyingValue()).materialize());
+        } catch (XPathException e) {
+            throw new SaxonApiUncheckedException(e);
+        }
     }
 
     /**
@@ -187,10 +200,7 @@ public class XQueryEvaluator implements Iterable<XdmItem>, Destination {
         if (oval == null) {
             return null;
         }
-        if (oval instanceof Sequence) {
-            return XdmValue.wrap((Sequence) oval);
-        }
-        throw new IllegalStateException(oval.getClass().getName());
+        return XdmValue.wrap((Sequence) oval);
     }
 
     /**
@@ -298,7 +308,6 @@ public class XQueryEvaluator implements Iterable<XdmItem>, Destination {
 
     /**
      * Perform the query.
-     * <p/>
      * <ul><li>In the case of a non-updating query, the results are sent to the
      * registered Destination.</li>
      * <li>In the case of an updating query, all updated documents will be available after query
@@ -315,25 +324,25 @@ public class XQueryEvaluator implements Iterable<XdmItem>, Destination {
         try {
             if (expression.isUpdateQuery()) {
                 Set<MutableNodeInfo> docs = expression.runUpdate(context);
-                updatedDocuments = new HashSet<XdmNode>();
+                updatedDocuments = new HashSet<>();
                 for (MutableNodeInfo doc : docs) {
-                    updatedDocuments.add((XdmNode) XdmNode.wrapItem(doc));
+                    updatedDocuments.add(XdmNode.wrapItem(doc));
                 }
             } else {
                 if (destination == null) {
                     throw new IllegalStateException("No destination supplied");
                 }
-
-                Result receiver;
-                if (destination instanceof Serializer) {
-                    //receiver = ((Serializer) destination).getResult();
-                    //context.set
-                    receiver = ((Serializer) destination).getReceiver(expression.getExecutable());
-                } else {
-                    receiver = destination.getReceiver(expression.getConfiguration());
-                }
-                expression.run(context, receiver, null);
-                destination.close();
+                run(destination);
+//                Result receiver;
+//                if (destination instanceof Serializer) {
+//                    //receiver = ((Serializer) destination).getResult();
+//                    //context.set
+//                    receiver = ((Serializer) destination).getReceiver(expression.getExecutable());
+//                } else {
+//                    receiver = destination.getReceiver(expression.getConfiguration());
+//                }
+//                expression.run(context, receiver, null);
+//                destination.close();
             }
         } catch (TransformerException e) {
             throw new SaxonApiException(e);
@@ -342,9 +351,7 @@ public class XQueryEvaluator implements Iterable<XdmItem>, Destination {
 
     /**
      * Perform the query, sending the results to a specified destination.
-     * <p/>
      * <p>This method must not be used with an updating query.</p>
-     * <p/>
      * <p>This method is designed for use with a query that produces a single node (typically
      * a document node or element node) as its result. If the query produces multiple nodes,
      * the effect depends on the kind of destination. For example, if the result is an
@@ -361,27 +368,22 @@ public class XQueryEvaluator implements Iterable<XdmItem>, Destination {
             throw new IllegalStateException("Query is updating");
         }
         try {
-            Receiver receiver;
-            if (destination instanceof Serializer) {
-                receiver = ((Serializer) destination).getReceiver(expression.getExecutable());
-            } else {
-                receiver = destination.getReceiver(expression.getExecutable().getConfiguration());
-            }
-            expression.run(context, receiver, null);
+            Receiver out = getDestinationReceiver(destination);
+            expression.run(context, out, null);
+            destination.closeAndNotify();
         } catch (TransformerException e) {
             throw new SaxonApiException(e);
         }
     }
 
     /**
-     * Perform the query, sending the results to a specified destination.
-     * <p/>
+     * Perform the query in streaming mode, sending the results to a specified destination.
      * <p>This method must not be used with an updating query.</p>
-     * <p/>
      * <p>This method is designed for use with a query that produces a single node (typically
      * a document node or element node) as its result. If the query produces multiple nodes,
      * the effect depends on the kind of destination. For example, if the result is an
      * <code>XdmDestination</code>, only the last of the nodes will be accessible.</p>
+     *
      * @param source the input stream whose document node will act as the initial context item.
      * Should be a SAXSource or StreamSource
      * @param destination The destination where the result document will be sent
@@ -403,12 +405,8 @@ public class XQueryEvaluator implements Iterable<XdmItem>, Destination {
             config.getStandardErrorOutput().println("Processing streamed input " + systemId);
         }
         try {
-            Receiver receiver;
-            if (destination instanceof Serializer) {
-                receiver = ((Serializer) destination).getReceiver(expression.getExecutable());
-            } else {
-                receiver = destination.getReceiver(expression.getExecutable().getConfiguration());
-            }
+            SerializationProperties params = expression.getExecutable().getPrimarySerializationProperties();
+            Receiver receiver = destination.getReceiver(config.makePipelineConfiguration(), params);
             expression.runStreamed(context, source, receiver, null);
         } catch (TransformerException e) {
             throw new SaxonApiException(e);
@@ -430,8 +428,8 @@ public class XQueryEvaluator implements Iterable<XdmItem>, Destination {
             throw new IllegalStateException("Query is updating");
         }
         try {
-            SequenceIterator iter = expression.iterator(context);
-            Sequence result = SequenceExtent.makeSequenceExtent(iter);
+            SequenceIterator<? extends Item> iter = expression.iterator(context);
+            Sequence result = iter.materialize();
             return XdmValue.wrap(result);
         } catch (XPathException e) {
             throw new SaxonApiException(e);
@@ -476,39 +474,44 @@ public class XQueryEvaluator implements Iterable<XdmItem>, Destination {
      * described in bug 2016.
      */
 
-    public XdmSequenceIterator iterator() throws SaxonApiUncheckedException {
+    public XdmSequenceIterator<XdmItem> iterator() throws SaxonApiUncheckedException {
         if (expression.isUpdateQuery()) {
             throw new IllegalStateException("Query is updating");
         }
         try {
-            return new XdmSequenceIterator(expression.iterator(context));
+            return new XdmSequenceIterator<>(expression.iterator(context));
         } catch (XPathException e) {
             throw new SaxonApiUncheckedException(e);
         }
     }
 
+    private Receiver getDestinationReceiver(Destination destination) throws SaxonApiException {
+        Executable exec = expression.getExecutable();
+        PipelineConfiguration pipe = expression.getConfiguration().makePipelineConfiguration();
+        return destination.getReceiver(pipe, exec.getPrimarySerializationProperties());
+    }
+
     /**
      * Return a Receiver which can be used to supply the principal source document for the transformation.
      * This method is intended primarily for internal use, though it can also
      * be called by a user application that wishes to feed events into the query engine.
-     * <p/>
      * <p>Saxon calls this method to obtain a Receiver, to which it then sends
      * a sequence of events representing the content of an XML document. This method is provided so that
      * <code>XQueryEvaluator</code> implements <code>Destination</code>, allowing one transformation
      * to receive the results of another in a pipeline.</p>
-     * <p/>
      * <p>Note that when an <code>XQueryEvaluator</code> is used as a <code>Destination</code>, the initial
      * context node set on that <code>XQueryEvaluator</code> (using {@link #setSource(javax.xml.transform.Source)}) is ignored.</p>
      *
-     * @param config The Saxon configuration. This is supplied so that the destination can
+     * @param pipe The Saxon configuration. This is supplied so that the destination can
      *               use information from the configuration (for example, a reference to the name pool)
      *               to construct or configure the returned Receiver.
+     * @param params
      * @return the Receiver to which events are to be sent.
      * @throws SaxonApiException     if the Receiver cannot be created
      * @throws IllegalStateException if no Destination has been supplied
      */
 
-    public Receiver getReceiver(Configuration config) throws SaxonApiException {
+    public Receiver getReceiver(PipelineConfiguration pipe, SerializationProperties params) throws SaxonApiException {
         if (destination == null) {
             throw new IllegalStateException("No destination has been supplied");
         }
@@ -525,7 +528,20 @@ public class XQueryEvaluator implements Iterable<XdmItem>, Destination {
         if (sourceTreeBuilder instanceof TinyBuilder) {
             ((TinyBuilder) sourceTreeBuilder).setStatistics(Statistics.SOURCE_DOCUMENT_STATISTICS);
         }
-        return controller.makeStripper(sourceTreeBuilder);
+        Receiver out = controller.makeStripper(sourceTreeBuilder);
+        SequenceNormalizer sn = params.makeSequenceNormalizer(out);
+        sn.onClose(() -> {
+            NodeInfo doc = sourceTreeBuilder.getCurrentRoot();
+            doc.getTreeInfo().setSpaceStrippingRule(controller.getSpaceStrippingRule());
+            setSource(doc);
+            sourceTreeBuilder = null;
+            if (doc == null) {
+                throw new SaxonApiException("No source document has been built by the previous pipeline stage");
+            }
+            run(destination);
+            destination.closeAndNotify();
+        });
+        return sn;
     }
 
     /**
@@ -535,23 +551,15 @@ public class XQueryEvaluator implements Iterable<XdmItem>, Destination {
      */
 
     public void close() throws SaxonApiException {
-        if (sourceTreeBuilder != null) {
-            NodeInfo doc = sourceTreeBuilder.getCurrentRoot();
-            setSource(doc);
-            sourceTreeBuilder = null;
-            if (doc == null) {
-                throw new SaxonApiException("No source document has been built by the previous pipeline stage");
-            }
-            run(destination);
-            destination.close();
-        }
+//        if (sourceTreeBuilder != null) {
+//
+//        }
     }
 
 
     /**
      * After executing an updating query using the {@link #run()} method, iterate over the root
      * nodes of the documents updated by the query.
-     * <p/>
      * <p>The results remain available until a new query is executed. This method returns the results
      * of the most recently executed query. It does not consume the results.</p>
      *
@@ -565,11 +573,10 @@ public class XQueryEvaluator implements Iterable<XdmItem>, Destination {
 
     /**
      * Call a global user-defined function in the compiled query.
-     * <p/>
-     * If this is called more than once (to evaluate the same function repeatedly with different arguments,
+     * <p>If this is called more than once (to evaluate the same function repeatedly with different arguments,
      * or to evaluate different functions) then the sequence of evaluations uses the same values of global
      * variables including external variables (query parameters); the effect of any changes made to query parameters
-     * between calls is undefined.
+     * between calls is undefined.</p>
      *
      * @param function  The name of the function to be called
      * @param arguments The values of the arguments to be supplied to the function. If necessary
