@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,6 +8,7 @@
 package net.sf.saxon.s9api;
 
 import net.sf.saxon.om.Item;
+import net.sf.saxon.om.NodeInfo;
 import net.sf.saxon.value.AtomicValue;
 
 import java.util.Map;
@@ -15,15 +16,12 @@ import java.util.Map;
 /**
  * The class XdmItem represents an item in a sequence, as defined by the XDM data model.
  * An item is either an atomic value or a node.
- * <p/>
  * <p>An item is a member of a sequence, but it can also be considered as a sequence
  * (of length one) in its own right. <tt>XdmItem</tt> is a subtype of <tt>XdmValue</tt> because every
  * Item in the XDM data model is also a value.</p>
- * <p/>
  * <p>It cannot be assumed that every sequence of length one will be represented by
  * an <tt>XdmItem</tt>. It is quite possible for an <tt>XdmValue</tt> that is not an <tt>XdmItem</tt> to hold
  * a singleton sequence.</p>
- * <p/>
  * <p>Saxon provides two concrete subclasses of <code>XdmItem</code>, namely
  * {@link XdmNode} and {@link XdmAtomicValue}. Users must not attempt to create
  * additional subclasses.</p>
@@ -46,7 +44,8 @@ public abstract class XdmItem extends XdmValue {
      */
 
     public XdmItem(Item item) {
-        super(item);
+        super();
+        setValue(item);
     }
 
     // internal factory mathod to wrap an Item
@@ -56,6 +55,14 @@ public abstract class XdmItem extends XdmValue {
         return item == null ? null : (XdmItem) XdmValue.wrap(item);
     }
 
+    protected static XdmNode wrapItem(NodeInfo item) {
+        return item == null ? null : (XdmNode) XdmValue.wrap(item);
+    }
+
+    protected static XdmAtomicValue wrapItem(AtomicValue item) {
+        return item == null ? null : (XdmAtomicValue) XdmValue.wrap(item);
+    }
+
     /**
      * Get the underlying implementation object representing the value. This method allows
      * access to lower-level Saxon functionality, including classes and methods that offer
@@ -74,7 +81,6 @@ public abstract class XdmItem extends XdmValue {
      * of the node. For an atomic value, it has the same effect as casting the value
      * to a string. In all cases the result is the same as applying the XPath string()
      * function.
-     * <p/>
      * <p>For atomic values, the result is the same as the result of calling
      * <code>toString</code>. This is not the case for nodes, where <code>toString</code>
      * returns an XML serialization of the node.</p>
