@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -11,9 +11,8 @@ import net.sf.saxon.ma.map.HashTrieMap;
 import net.sf.saxon.ma.map.KeyValuePair;
 import net.sf.saxon.ma.map.MapItem;
 import net.sf.saxon.om.Sequence;
-import net.sf.saxon.tree.iter.AtomicIterator;
+import net.sf.saxon.tree.jiter.MappingJavaIterator;
 import net.sf.saxon.value.AtomicValue;
-import org.jetbrains.annotations.NotNull;
 
 import java.util.*;
 
@@ -21,14 +20,13 @@ import java.util.*;
  * A map in the XDM data model. A map is a list of zero or more entries, each of which
  * is a pair comprising a key (which is an atomic value) and a value (which is an arbitrary value).
  * The map itself is an XDM item.
- * <p/>
- * An XdmMap is immutable.
- * <p/>
- * As originally issued in Saxon 9.8, this class implemented the {@link java.util.Map} interface.
+ * <p>An XdmMap is immutable.</p>
+ * <p>As originally issued in Saxon 9.8, this class implemented the {@link java.util.Map} interface.
  * It no longer does so, because it was found that the methods {@link #size()}, {@link #put(XdmAtomicValue, XdmValue)},
  * and {@link #remove} did not adhere to the semantic contract of that interface. Instead, it is now
  * possible to obtain a view of this object as an immutable Java {@link java.util.Map} by calling the
- * method {@link #asImmutableMap()}. See bug 3824.
+ * method {@link #asImmutableMap()}. See bug 3824.</p>
+ *
  * @since 9.8
  */
 
@@ -48,7 +46,7 @@ public class XdmMap extends XdmFunctionItem {
      */
 
     public XdmMap(MapItem map) {
-        super(map);
+        setValue(map);
     }
 
     /**
@@ -118,23 +116,33 @@ public class XdmMap extends XdmFunctionItem {
     }
 
     /**
-     * Get the keys present in the map in the form of a list.
-     * @return a list of the keys present in this map, in arbitrary order.
+     * Get the keys present in the map in the form of an unordered set.
+     * @return an unordered set of the keys present in this map, in arbitrary order.
      */
-    @NotNull
     public Set<XdmAtomicValue> keySet() {
-        Set<XdmAtomicValue> result = new HashSet<XdmAtomicValue>();
-        AtomicIterator iter = getUnderlyingValue().keys();
-        AtomicValue key;
-        while ((key = iter.next()) != null) {
-            result.add((XdmAtomicValue)XdmValue.wrap(key));
-        }
-        return result;
+        return new AbstractSet<XdmAtomicValue>() {
+            @Override
+            public Iterator<XdmAtomicValue> iterator() {
+                return new MappingJavaIterator<>(
+                        getUnderlyingValue().keyValuePairs().iterator(),
+                        kvp -> (XdmAtomicValue) XdmValue.wrap(kvp.key));
+            }
+
+            @Override
+            public int size() {
+                return getUnderlyingValue().size();
+            }
+
+            @Override
+            public boolean contains(Object o) {
+                // Intentionally throws ClassCastException if o is not an XdmAtomicValue
+                return getUnderlyingValue().get(((XdmAtomicValue)o).getUnderlyingValue()) != null;
+            }
+        };
     }
 
     /**
      * Return this map as an immutable instance of {@link java.util.Map}
-     *
      * @return an immutable instance of {@link java.util.Map} backed by this map.
      * Methods such as {@link #remove} and {@link #put} applied to the result will
      * always throw {@code UnsupportedOperationException}.
@@ -142,7 +150,7 @@ public class XdmMap extends XdmFunctionItem {
 
     public Map<XdmAtomicValue, XdmValue> asImmutableMap() {
         // See bug 3824
-        final XdmMap base = this;
+        XdmMap base = this;
         return new AbstractMap<XdmAtomicValue, XdmValue>() {
             @Override
             public Set<Entry<XdmAtomicValue, XdmValue>> entrySet() {
@@ -162,9 +170,9 @@ public class XdmMap extends XdmFunctionItem {
             @Override
             public boolean containsValue(Object value) {
                 for (Entry<XdmAtomicValue, XdmValue> e : entrySet()) {
-                    if (value.equals(e.getValue())) {
-                        return true;
-                    }
+                     if (value.equals(e.getValue())) {
+                         return true;
+                     }
                 }
                 return false;
             }
@@ -220,10 +228,9 @@ public class XdmMap extends XdmFunctionItem {
      */
 
     public Map<XdmAtomicValue, XdmValue> asMap() {
-        return new HashMap<XdmAtomicValue, XdmValue>(asImmutableMap());
+        return new HashMap<>(asImmutableMap());
     }
 
-
     /**
      * Removes all of the mappings from this map (optional operation).
      * The map will be empty after this call returns.
@@ -266,34 +273,9 @@ public class XdmMap extends XdmFunctionItem {
         return getUnderlyingValue().get(k) != null;
     }
 
-    /**
-     * Returns <tt>true</tt> if this map maps one or more keys to the
-     * specified value.  More formally, returns <tt>true</tt> if and only if
-     * this map contains at least one mapping to a value <tt>v</tt> such that
-     * <tt>(value==null ? v==null : value.equals(v))</tt>.  This operation
-     * will probably require time linear in the map size for most
-     * implementations of the <tt>Map</tt> interface.
-     *
-     * <p>This implementation always throws UnsupportedOperationException.</p>
-     *
-     * @param value value whose presence in this map is to be tested
-     * @return <tt>true</tt> if this map maps one or more keys to the
-     * specified value
-     * @throws ClassCastException   if the value is of an inappropriate type for
-     *                              this map
-     *                              (<a href="{@docRoot}/java/util/Collection.html#optional-restrictions">optional</a>)
-     * @throws NullPointerException if the specified value is null and this
-     *                              map does not permit null values
-     *                              (<a href="{@docRoot}/java/util/Collection.html#optional-restrictions">optional</a>)
-     */
-    public boolean containsValue(Object value) {
-        throw new UnsupportedOperationException("Not implemented: cannot compare sequences");
-    }
-
     /**
      * Returns the value to which the specified key is mapped,
      * or {@code null} if this map contains no mapping for the key.
-     * <p/>
      *
      * @param key the key whose associated value is to be returned. If this is
      *            not an XdmAtomicValue, the method attempts to construct an
@@ -323,34 +305,6 @@ public class XdmMap extends XdmFunctionItem {
         return v == null ? null : XdmValue.wrap(v);
     }
 
-    /**
-     * Removes the mapping for a key from this map if it is present
-     * (optional operation). This map implementation is immutable
-     * so the method always throws {@code UnsupportedOperationException}.
-     *
-     * @param key key whose mapping is to be removed from the map
-     * @return the previous value associated with <tt>key</tt>, or
-     * <tt>null</tt> if there was no mapping for <tt>key</tt>.
-     * @throws UnsupportedOperationException if the <tt>remove</tt> operation
-     *                                       is not supported by this map
-     */
-    public XdmValue remove(Object key) {
-        throw new UnsupportedOperationException("XdmMap is immutable");
-    }
-
-    /**
-     * Copies all of the mappings from the specified map to this map
-     * (optional operation).  This map implementation is immutable
-     * so the method always throws {@code UnsupportedOperationException}.
-     *
-     * @param m mappings to be stored in this map
-     * @throws UnsupportedOperationException if the <tt>putAll</tt> operation
-     *                                       is not supported by this map
-     */
-    public void putAll(@NotNull Map<? extends XdmAtomicValue, ? extends XdmValue> m) {
-        throw new UnsupportedOperationException("XdmMap is immutable");
-    }
-
     /**
      * Returns a {@link Collection} view of the values contained in this map.
      * The collection is backed by the map, so changes to the map are
@@ -366,10 +320,9 @@ public class XdmMap extends XdmFunctionItem {
      *
      * @return a collection view of the values contained in this map
      */
-    @NotNull
     public Collection<XdmValue> values() {
-        List<XdmValue> result = new ArrayList<XdmValue>();
-        for (KeyValuePair keyValuePair : getUnderlyingValue()) {
+        List<XdmValue> result = new ArrayList<>();
+        for (KeyValuePair keyValuePair : getUnderlyingValue().keyValuePairs()) {
             result.add(XdmValue.wrap(keyValuePair.value));
         }
         return result;
@@ -380,12 +333,10 @@ public class XdmMap extends XdmFunctionItem {
      *
      * @return a set view of the mappings contained in this map
      */
-    @NotNull
     public Set<Map.Entry<XdmAtomicValue, XdmValue>> entrySet() {
-        Set<Map.Entry<XdmAtomicValue, XdmValue>> result = new HashSet<Map.Entry<XdmAtomicValue, XdmValue>>();
-        for (KeyValuePair keyValuePair : getUnderlyingValue()) {
-            Map.Entry<XdmAtomicValue, XdmValue> entry = new XdmMapEntry(keyValuePair);
-            result.add(entry);
+        Set<Map.Entry<XdmAtomicValue, XdmValue>> result = new HashSet<>();
+        for (KeyValuePair keyValuePair : getUnderlyingValue().keyValuePairs()) {
+            result.add(new XdmMapEntry(keyValuePair));
         }
         return result;
     }
@@ -425,6 +376,9 @@ public class XdmMap extends XdmFunctionItem {
          * Returns the key corresponding to this entry.
          *
          * @return the key corresponding to this entry
+         * @throws IllegalStateException implementations may, but are not
+         *                               required to, throw this exception if the entry has been
+         *                               removed from the backing map.
          */
         @Override
         public XdmAtomicValue getKey() {
@@ -432,9 +386,14 @@ public class XdmMap extends XdmFunctionItem {
         }
 
         /**
-         * Returns the value corresponding to this entry.
+         * Returns the value corresponding to this entry.  If the mapping
+         * has been removed from the backing map (by the iterator's
+         * <tt>remove</tt> operation), the results of this call are undefined.
          *
          * @return the value corresponding to this entry
+         * @throws IllegalStateException implementations may, but are not
+         *                               required to, throw this exception if the entry has been
+         *                               removed from the backing map.
          */
         @Override
         public XdmValue getValue() {
@@ -443,8 +402,23 @@ public class XdmMap extends XdmFunctionItem {
 
         /**
          * Replaces the value corresponding to this entry with the specified
-         * value (optional operation). Always throws {@code UnsupportedOperationException}
-         * @throws UnsupportedOperationException always
+         * value (optional operation).  (Writes through to the map.)  The
+         * behavior of this call is undefined if the mapping has already been
+         * removed from the map (by the iterator's <tt>remove</tt> operation).
+         *
+         * @param value new value to be stored in this entry
+         * @return old value corresponding to the entry
+         * @throws UnsupportedOperationException if the <tt>put</tt> operation
+         *                                       is not supported by the backing map
+         * @throws ClassCastException            if the class of the specified value
+         *                                       prevents it from being stored in the backing map
+         * @throws NullPointerException          if the backing map does not permit
+         *                                       null values, and the specified value is null
+         * @throws IllegalArgumentException      if some property of this value
+         *                                       prevents it from being stored in the backing map
+         * @throws IllegalStateException         implementations may, but are not
+         *                                       required to, throw this exception if the entry has been
+         *                                       removed from the backing map.
          */
         @Override
         public XdmValue setValue(XdmValue value) {
