@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,6 +8,7 @@
 package net.sf.saxon.serialize;
 
 import net.sf.saxon.event.PipelineConfiguration;
+import net.sf.saxon.event.ReceiverWithOutputProperties;
 import net.sf.saxon.event.SequenceWriter;
 import net.sf.saxon.functions.FormatNumber;
 import net.sf.saxon.lib.NamespaceConstant;
@@ -33,7 +34,7 @@ import java.util.Properties;
  * This class implements the Adaptive serialization method defined in XSLT+XQuery Serialization 3.1.
  */
 
-public class AdaptiveEmitter extends SequenceWriter {
+public class AdaptiveEmitter extends SequenceWriter implements ReceiverWithOutputProperties {
 
     private Writer writer;
     private Properties outputProperties;
@@ -47,8 +48,9 @@ public class AdaptiveEmitter extends SequenceWriter {
 
     public void setOutputProperties(Properties props) {
         outputProperties = props;
-        if (props.getProperty(SaxonOutputKeys.ITEM_SEPARATOR) != null) {
-            itemSeparator = props.getProperty(SaxonOutputKeys.ITEM_SEPARATOR);
+        String sep = props.getProperty(SaxonOutputKeys.ITEM_SEPARATOR);
+        if (sep != null && !"#absent".equals(sep)) {
+            itemSeparator = sep;
         }
     }
 
@@ -194,7 +196,7 @@ public class AdaptiveEmitter extends SequenceWriter {
     private void serializeArray(ArrayItem array) throws XPathException {
         emit("[");
         boolean first = true;
-        for (Sequence seq: array) {
+        for (Sequence seq: array.members()) {
             if (first) {
                 first = false;
             } else {
@@ -208,7 +210,7 @@ public class AdaptiveEmitter extends SequenceWriter {
     private void serializeMap(MapItem map) throws XPathException {
         emit("map{");
         boolean first = true;
-        for (KeyValuePair pair : map) {
+        for (KeyValuePair pair : map.keyValuePairs()) {
             if (first) {
                 first = false;
             } else {
