@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -65,18 +65,25 @@ class UnicodeDataParserFromXML {
         AxisIterator iter = doc.getRootNode().iterateAxis(AxisInfo.DESCENDANT, NodeKindTest.ELEMENT);
         NodeInfo item;
         while ((item = iter.next()) != null) {
-            if (item.getLocalPart().equals("CanonicalClassKeys")) {
-                canonicalClassKeys = item;
-            } else if (item.getLocalPart().equals("CanonicalClassValues")) {
-                canonicalClassValues = item;
-            } else if (item.getLocalPart().equals("DecompositionKeys")) {
-                decompositionKeys = item;
-            } else if (item.getLocalPart().equals("DecompositionValues")) {
-                decompositionValues = item;
-            } else if (item.getLocalPart().equals("ExclusionList")) {
-                readExclusionList(item.getStringValue(), isExcluded);
-            } else if (item.getLocalPart().equals("CompatibilityList")) {
-                readCompatibilityList(item.getStringValue(), isCompatibility);
+            switch (item.getLocalPart()) {
+                case "CanonicalClassKeys":
+                    canonicalClassKeys = item;
+                    break;
+                case "CanonicalClassValues":
+                    canonicalClassValues = item;
+                    break;
+                case "DecompositionKeys":
+                    decompositionKeys = item;
+                    break;
+                case "DecompositionValues":
+                    decompositionValues = item;
+                    break;
+                case "ExclusionList":
+                    readExclusionList(item.getStringValue(), isExcluded);
+                    break;
+                case "CompatibilityList":
+                    readCompatibilityList(item.getStringValue(), isCompatibility);
+                    break;
             }
         }
 
