@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -42,17 +42,18 @@ public class Compilation {
     private int errorCount = 0;
     private boolean schemaAware;
     private QNameParser qNameParser;
-    private Map<StructuredQName, ValueAndPrecedence> staticVariables = new HashMap<StructuredQName, ValueAndPrecedence>();
-    private Map<DocumentURI, TreeInfo> stylesheetModules = new HashMap<DocumentURI, TreeInfo>();
-    private Stack<DocumentURI> importStack = new Stack<DocumentURI>(); // handles both include and import
+    private Map<StructuredQName, ValueAndPrecedence> staticVariables = new HashMap<>();
+    private Map<DocumentURI, TreeInfo> stylesheetModules = new HashMap<>();
+    private Stack<DocumentURI> importStack = new Stack<>(); // handles both include and import
     private PackageData packageData;
     private boolean preScan = true;
     private boolean createsSecondaryResultDocuments = false;
     private boolean libraryPackage = false;
     private VersionedPackageName expectedNameAndVersion = null;
-    private List<UsePack> packageDependencies = new ArrayList<UsePack>();
-    private List<VersionedPackageName> usingPackages = new ArrayList<VersionedPackageName>();
+    private List<UsePack> packageDependencies = new ArrayList<>();
+    private List<VersionedPackageName> usingPackages = new ArrayList<>();
     private GlobalParameterSet suppliedParameters;
+    private boolean fallbackToNonStreaming = false;
 
     private static class ValueAndPrecedence {
         public ValueAndPrecedence(GroundedValue v, NestedIntegerValue p, boolean isParam) {
@@ -190,12 +191,12 @@ public class Compilation {
                     }
                     buffer.append("and ");
                     buffer.append(thisPackage.getName());
-                    throw new XPathException("There is a cycle of package dependencies involving " + buffer.toString(), "XTSE3005");
+                    throw new XPathException("There is a cycle of package dependencies involving " + buffer, "XTSE3005");
                 }
             }
             StylesheetPackage used;
             try {
-                List<VersionedPackageName> disallowed = new ArrayList<VersionedPackageName>(usingPackages);
+                List<VersionedPackageName> disallowed = new ArrayList<>(usingPackages);
                 disallowed.add(details.nameAndVersion);
                 used = library.obtainLoadedPackage(details, disallowed);
             } catch (XPathException err) {
@@ -340,6 +341,10 @@ public class Compilation {
         }
 
         psm.getStylesheetPackage().setCreatesSecondaryResultDocuments(createsSecondaryResultDocuments);
+
+        if (isFallbackToNonStreaming()) {
+            psm.getStylesheetPackage().setFallbackToNonStreaming();
+        }
         return psm;
     }
 
@@ -534,13 +539,13 @@ public class Compilation {
             if (i0 instanceof AtomicValue) {
                 return i1 instanceof AtomicValue && ((AtomicValue) i0).isIdentical((AtomicValue) i1);
             } else if (i0 instanceof NodeInfo) {
-                return i1 instanceof NodeInfo && ((NodeInfo) i0).isSameNodeInfo((NodeInfo) i1);
+                return i1 instanceof NodeInfo && i0.equals(i1);
             } else {
                 return i0 == i1;
             }
         } else {
             for (int i = 0; i < val0.getLength(); i++) {
-                if (!valuesAreCompatible((GroundedValue) val0.itemAt(i), (GroundedValue) val1.itemAt(i))) {
+                if (!valuesAreCompatible(val0.itemAt(i), val1.itemAt(i))) {
                     return false;
                 }
             }
@@ -679,13 +684,13 @@ public class Compilation {
      * @param seq  the value of the stylesheet parameter
      */
 
-    public void setParameter(StructuredQName name, Sequence seq) {
+    public void setParameter(StructuredQName name, GroundedValue seq) {
         suppliedParameters.put(name, seq);
     }
 
     /**
      * Get the values of all stylesheet parameters that have been set using the
-     * {@link #setParameter(net.sf.saxon.om.StructuredQName, net.sf.saxon.om.Sequence)}
+     * {@link #setParameter(net.sf.saxon.om.StructuredQName, net.sf.saxon.om.GroundedValue)}
      * method.
      *
      * @return the set of all parameters that have been set.
@@ -697,7 +702,7 @@ public class Compilation {
 
     /**
      * Clear the values of all stylesheet parameters that have been set using the
-     * {@link #setParameter(net.sf.saxon.om.StructuredQName, net.sf.saxon.om.Sequence)}
+     * {@link #setParameter(net.sf.saxon.om.StructuredQName, net.sf.saxon.om.GroundedValue)}
      * method; and decouple the parameters held by this Compilation from those defined
      * in the originating CompilerInfo
      */
@@ -706,5 +711,27 @@ public class Compilation {
         suppliedParameters = new GlobalParameterSet();
     }
 
+    /**
+     * Ask whether a non-streamable construct has been found, forcing the entire stylesheet
+     * to fall back to unstreamed processing
+     * @return true if the stylesheet must fall back to unstreamed processing
+     */
+
+    public boolean isFallbackToNonStreaming() {
+        return fallbackToNonStreaming;
+    }
+
+    /**
+     * This method is called when a non-streamable construct is found, and the configuration option
+     * {@link net.sf.saxon.lib.Feature#STREAMING_FALLBACK} has been set; the effect is to mark the whole
+     * stylesheet as non-streamable
+     * @param fallbackToNonStreaming true if a construct has been found that is declared streamable but
+     *                               not actually streamable, if fallback processing was requested.
+     */
+
+    public void setFallbackToNonStreaming(boolean fallbackToNonStreaming) {
+        this.fallbackToNonStreaming = fallbackToNonStreaming;
+    }
+
 
 }
