@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,12 +8,10 @@
 package net.sf.saxon.testdriver;
 
 import net.sf.saxon.Configuration;
+import net.sf.saxon.event.Receiver;
 import net.sf.saxon.expr.parser.ExplicitLocation;
 import net.sf.saxon.functions.ResolveURI;
-import net.sf.saxon.lib.EnvironmentVariableResolver;
-import net.sf.saxon.lib.FeatureKeys;
-import net.sf.saxon.lib.NamespaceConstant;
-import net.sf.saxon.lib.StandardModuleURIResolver;
+import net.sf.saxon.lib.*;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.om.TreeModel;
 import net.sf.saxon.s9api.*;
@@ -27,13 +25,14 @@ import javax.xml.transform.TransformerException;
 import javax.xml.transform.URIResolver;
 import javax.xml.transform.stream.StreamSource;
 import java.io.File;
+import java.io.IOException;
 import java.io.StringWriter;
 import java.net.URI;
 import java.net.URISyntaxException;
-import java.util.HashMap;
-import java.util.HashSet;
-import java.util.Map;
-import java.util.Set;
+import java.util.*;
+
+import static net.sf.saxon.s9api.streams.Predicates.*;
+import static net.sf.saxon.s9api.streams.Steps.*;
 
 /**
  * Test Driver for the QT3 test suite
@@ -62,22 +61,30 @@ public class QT3TestDriverHE extends TestDriver {
 
     public void processSpec(String specStr) {
 
-        if (specStr.equals("XP20")) {
-            spec = Spec.XP20;
-        } else if (specStr.equals("XP30")) {
-            spec = Spec.XP30;
-        } else if (specStr.equals("XP31")) {
-            spec = Spec.XP31;
-        } else if (specStr.equals("XQ10")) {
-            spec = Spec.XQ10;
-        } else if (specStr.equals("XQ30")) {
-            spec = Spec.XQ30;
-        } else if (specStr.equals("XQ31")) {
-            spec = Spec.XQ31;
-        } else if (specStr.equals("XT30")) { // map function tests
-            spec = Spec.XT30;
-        } else {
-            throw new IllegalArgumentException("The specific language must be one of the following: XP20, XP30, XP31, XQ10, XQ30, XQ31, XT30");
+        switch (specStr) {
+            case "XP20":
+                spec = Spec.XP20;
+                break;
+            case "XP30":
+                spec = Spec.XP30;
+                break;
+            case "XP31":
+                spec = Spec.XP31;
+                break;
+            case "XQ10":
+                spec = Spec.XQ10;
+                break;
+            case "XQ30":
+                spec = Spec.XQ30;
+                break;
+            case "XQ31":
+                spec = Spec.XQ31;
+                break;
+            case "XT30":  // map function tests
+                spec = Spec.XT30;
+                break;
+            default:
+                throw new IllegalArgumentException("The specific language must be one of the following: XP20, XP30, XP31, XQ10, XQ30, XQ31, XT30");
         }
         resultsDoc = new QT3TestReport(this, spec);
     }
@@ -100,7 +107,7 @@ public class QT3TestDriverHE extends TestDriver {
     @Override
     protected void createGlobalEnvironments(XdmNode catalog, XPathCompiler xpc) throws SaxonApiException {
         Environment environment = null;
-        for (XdmItem env : xpc.evaluate("//environment", catalog)) {
+        for (XdmNode env : catalog.select(descendant("environment")).asList()) {
             try {
                 environment = Environment.processEnvironment(
                         this, xpc, env, globalEnvironments, localEnvironments.get("default"));
@@ -127,12 +134,12 @@ public class QT3TestDriverHE extends TestDriver {
      */
 
     public boolean ensureDependencySatisfied(XdmNode dependency, Environment env) {
-        String type = dependency.getAttributeValue(new QName("type"));
-        String value = dependency.getAttributeValue(new QName("value"));
-        boolean inverse = "false".equals(dependency.getAttributeValue(new QName("satisfied")));
+        String type = dependency.attribute("type");
+        String value = dependency.attribute("value");
+        boolean inverse = "false".equals(dependency.attribute("satisfied"));
         if ("saxon:config".equals(type)) {
             // Used in QT3Extra tests
-            final String name = dependency.getAttributeValue(new QName("name"));
+            final String name = dependency.attribute("name");
             final Object oldValue = env.processor.getConfigurationProperty(name);
             env.processor.setConfigurationProperty(name, value);
             env.resetActions.add(new Environment.ResetAction() {
@@ -165,23 +172,23 @@ public class QT3TestDriverHE extends TestDriver {
             }
             return true;
         } else if ("xsd-version".equals(type)) {
-            final String old = (String) env.processor.getConfigurationProperty(FeatureKeys.XSD_VERSION);
+            final String old = env.processor.getConfigurationProperty(Feature.XSD_VERSION);
             if ("1.1".equals(value)) {
                 if (env != null) {
-                    env.processor.setConfigurationProperty(FeatureKeys.XSD_VERSION, inverse ? "1.0" : "1.1");
+                    env.processor.setConfigurationProperty(Feature.XSD_VERSION, inverse ? "1.0" : "1.1");
                 } else {
                     return false;
                 }
             } else if ("1.0".equals(value)) {
                 if (env != null) {
-                    env.processor.setConfigurationProperty(FeatureKeys.XSD_VERSION, inverse ? "1.1" : "1.0");
+                    env.processor.setConfigurationProperty(Feature.XSD_VERSION, inverse ? "1.1" : "1.0");
                 } else {
                     return false;
                 }
             }
             env.resetActions.add(new Environment.ResetAction() {
                 public void reset(Environment env) {
-                    env.processor.setConfigurationProperty(FeatureKeys.XSD_VERSION, old);
+                    env.processor.setConfigurationProperty(Feature.XSD_VERSION, old);
                 }
             });
             return true;
@@ -193,12 +200,12 @@ public class QT3TestDriverHE extends TestDriver {
             // SAXON has a problem here - we don't support stable collections
             return "false".equals(value) != inverse;
         } else if ("default-language".equals(type)) {
-            final String old = (String) env.processor.getConfigurationProperty(FeatureKeys.DEFAULT_LANGUAGE);
+            final String old = (String) env.processor.getConfigurationProperty(Feature.DEFAULT_LANGUAGE);
             if (!value.equals(old)){
-                env.processor.setConfigurationProperty(FeatureKeys.DEFAULT_LANGUAGE, value);
+                env.processor.setConfigurationProperty(Feature.DEFAULT_LANGUAGE, value);
                 env.resetActions.add(new Environment.ResetAction() {
                     public void reset(Environment env) {
-                        env.processor.setConfigurationProperty(FeatureKeys.DEFAULT_LANGUAGE, old);
+                        env.processor.setConfigurationProperty(Feature.DEFAULT_LANGUAGE, old);
                     }
                 });
             }
@@ -213,7 +220,7 @@ public class QT3TestDriverHE extends TestDriver {
         } else if ("format-integer-sequence".equals(type)) {
             return !inverse;
         } else if ("unicode-normalization-form".equals(type)) {
-            return value.equalsIgnoreCase("FULLY-NORMALIZED") ? inverse : !inverse;
+            return value.equalsIgnoreCase("FULLY-NORMALIZED") == inverse;
         } else if ("feature".equals(type)) {
             String edition = env.processor.getSaxonEdition();
             if ("namespace-axis".equals(value)) {
@@ -295,14 +302,18 @@ public class QT3TestDriverHE extends TestDriver {
         String langVersion;
         Spec specOpt = null;
         XPathCompiler xpath = driverProc.newXPathCompiler();
-        String testCaseName = testCase.getAttributeValue(new QName("name"));
-        String testSetName = testCase.getParent().getAttributeValue(new QName("name"));
-        boolean needSerializedResult = ((XdmAtomicValue) xpc.evaluateSingle(
-                "exists(./result//assert-serialization-error) or exists(./result//serialization-matches)", testCase)).getBooleanValue();
+        String testCaseName = testCase.attribute("name");
+        String testSetName = testCase.getParent().attribute("name");
+        XdmNode resultElement = testCase.select(child("result")).asNode();
+        boolean needSerializedResult = resultElement.select(descendant("assert-serialization-error")).exists()
+                || resultElement.select(descendant("serialization-matches")).exists();
         boolean needResultValue = true;
         if (needSerializedResult) {
-            needResultValue = ((XdmAtomicValue) xpc.evaluateSingle(
-                    "exists(./result//*[not(self::serialization-matches or self::assert-serialization-error or self::any-of or self::all-of)])", testCase)).getBooleanValue();
+            needResultValue = resultElement.select(descendant(isElement()).where(not(
+                    hasLocalName("serialization-matches")
+                            .or(hasLocalName("assert-serialization-error"))
+                            .or(hasLocalName("any-of"))
+                            .or(hasLocalName("all-of"))))).exists();
         }
 
         XdmNode alternativeResult = null;
@@ -332,12 +343,14 @@ public class QT3TestDriverHE extends TestDriver {
         env.xpathCompiler.setBackwardsCompatible(false);
         env.processor.setXmlVersion("1.0");
 
-        for (XdmItem dependency : xpc.evaluate("/*/dependency, ./dependency", testCase)) {
-            String type = ((XdmNode) dependency).getAttributeValue(new QName("type"));
+        List<XdmNode> dependencies = testCase.getParent().select(child("dependency")).asList();
+        dependencies.addAll(testCase.select(child("dependency")).asList());
+        for (XdmNode dependency : dependencies) {
+            String type = dependency.attribute("type");
             if (type == null) {
                 throw new IllegalStateException("dependency/@type is missing");
             }
-            String value = ((XdmNode) dependency).getAttributeValue(new QName("value"));
+            String value = dependency.attribute("value");
             if (value == null) {
                 throw new IllegalStateException("dependency/@value is missing");
             }
@@ -386,8 +399,8 @@ public class QT3TestDriverHE extends TestDriver {
                 xpDependency = true;
                 specOpt = Spec.XP30;
             } */
-            if (!ensureDependencySatisfied((XdmNode) dependency, env)) {
-                if ("false".equals(((XdmNode) dependency).getAttributeValue(new QName("", "satisfied")))) {
+            if (!ensureDependencySatisfied(dependency, env)) {
+                if ("false".equals(dependency.attribute("satisfied"))) {
                     type = "!" + type;
                 }
                 println("*** Dependency not satisfied: " + type + ":" + value);
@@ -403,35 +416,32 @@ public class QT3TestDriverHE extends TestDriver {
             exceptionElement = exceptionsMap.get("$" + testSetName);
         }
         if (exceptionElement != null) {
-            XdmItem config = xpath.evaluateSingle("configuration", exceptionElement);
+            Optional<XdmNode> config = exceptionElement.select(child("configuration")).asOptionalNode();
 
-            String runAtt = exceptionElement.getAttributeValue(new QName("run"));
-            String reasonMsg = exceptionElement.axisIterator(Axis.CHILD, new QName("reason")).next().getStringValue();
-            if (reasonMsg == null) {
-                reasonMsg = "no reason given";
-            }
-            String reportAtt = exceptionElement.getAttributeValue(new QName("report"));
+            String runAtt = exceptionElement.attribute("run");
+            Optional<String> reasonMsg = exceptionElement.select(child("reason")).asOptionalString();
+            String reportAtt = exceptionElement.attribute("report");
             if (reportAtt == null) {
                 reportAtt = "";
             }
 
-            if (config != null) {
-                XdmItem paramValue = xpath.evaluateSingle("param[@name='not-unfolded' and @value='yes']/@name", config);
+            if (config.isPresent()) {
+                XdmItem paramValue = xpath.evaluateSingle("param[@name='not-unfolded' and @value='yes']/@name", config.get());
                 if (unfolded && paramValue != null) {
-                    writeTestcaseElement(testCaseName, "notRun", reasonMsg);
+                    writeTestcaseElement(testCaseName, "notRun", reasonMsg.orElse("no reason given"));
                     notrun++;
                     return;
                 }
             }
 
             if ("false".equals(runAtt)) {
-                writeTestcaseElement(testCaseName, reportAtt, reasonMsg);
+                writeTestcaseElement(testCaseName, reportAtt, reasonMsg.orElse("no reason given"));
                 notrun++;
                 return;
             }
 
-            alternativeResult = (XdmNode) xpc.evaluateSingle("result", exceptionElement);
-            optimization = (XdmNode) xpc.evaluateSingle("optimization", exceptionElement);
+            alternativeResult = exceptionElement.select(child("result")).asOptionalNode().orElse(null);
+            optimization = exceptionElement.select(child("optimization")).asOptionalNode().orElse(null);
         }
 
         if (run && (specOpt == null || specOpt == spec)) {
@@ -442,7 +452,11 @@ public class QT3TestDriverHE extends TestDriver {
                 exp = xpc.evaluate("if (test/@file) then unparsed-text(resolve-uri(test/@file, base-uri(.))) else string(test)", testCase).toString();
             } catch (SaxonApiException err) {
                 println("*** Failed to read query: " + err.getMessage());
-                outcome.setException(err);
+                if (err.getErrorCode().getLocalName().equals("FOUT1190")) {
+                    outcome.setException(new SaxonApiException(new XPathException(err.getMessage(), "XPST0003")));
+                } else {
+                    outcome.setException(err);
+                }
             }
 
             if (exp != null && env.processor.getSaxonEdition().equals("HE") && testSetName.equals("app-spec-examples")) {
@@ -469,157 +483,167 @@ public class QT3TestDriverHE extends TestDriver {
 
             //noinspection ThrowableResultOfMethodCallIgnored
             if (outcome.getException() == null) {
-                if (hostLang.equals("XP") || hostLang.equals("XT")) {
-                    XPathCompiler testXpc = env.xpathCompiler;
-                    testXpc.setLanguageVersion(langVersion);
-                    testXpc.declareNamespace("fn", NamespaceConstant.FN);
-                    testXpc.declareNamespace("xs", NamespaceConstant.SCHEMA);
-                    //testXpc.declareNamespace("math", NamespaceConstant.MATH);
-                    testXpc.declareNamespace("map", NamespaceConstant.MAP_FUNCTIONS);
-                    testXpc.declareNamespace("array", NamespaceConstant.ARRAY_FUNCTIONS);
+                switch (hostLang) {
+                    case "XP":
+                    case "XT": {
+                        XPathCompiler testXpc = env.xpathCompiler;
+                        testXpc.setLanguageVersion(langVersion);
+                        testXpc.declareNamespace("fn", NamespaceConstant.FN);
+                        testXpc.declareNamespace("xs", NamespaceConstant.SCHEMA);
+                        //testXpc.declareNamespace("math", NamespaceConstant.MATH);
+                        testXpc.declareNamespace("map", NamespaceConstant.MAP_FUNCTIONS);
+                        testXpc.declareNamespace("array", NamespaceConstant.ARRAY_FUNCTIONS);
 
-                    copySchemaNamespaces(env, testXpc);  // ensure environment has schema namespaces
+                        copySchemaNamespaces(env, testXpc);  // ensure environment has schema namespaces
 
-                    ModuleResolver mr = new ModuleResolver(xpc); //
-                    mr.setTestCase(testCase);
-                    testXpc.getProcessor().getUnderlyingConfiguration().setModuleURIResolver(mr);
 
-                    try {
-                        XPathSelector selector = testXpc.compile(exp).load();
-                        for (QName varName : env.params.keySet()) {
-                            selector.setVariable(varName, env.params.get(varName));
-                        }
-                        if (env.contextItem != null) {
-                            selector.setContextItem(env.contextItem);
-                        }
-                        selector.setURIResolver(new TestURIResolver(env));
-                        selector.setURIResolver(new TestURIResolver(env));
-                        if (env.unparsedTextResolver != null) {
-                            selector.getUnderlyingXPathContext().setUnparsedTextURIResolver(env.unparsedTextResolver);
-                        }
-                        XdmValue result = selector.evaluate();
-                        outcome.setPrincipalResult(result);
-                    } catch (SaxonApiException err) {
-                        println(err.getMessage());
-                        outcome.setException(err);
-                    } catch (Exception err) {
-                        println(err.getMessage());
-                        err.printStackTrace();
-                        writeTestcaseElement(testCaseName, "fail", "*** crashed: " + err.getMessage());
-                        //notrun++;
-                        //return;
-                    }
-                } else if (hostLang.equals("XQ")) {
-                    XQueryCompiler testXqc = env.xqueryCompiler;
-                    testXqc.setLanguageVersion(langVersion);
-                    testXqc.declareNamespace("fn", NamespaceConstant.FN);
-                    testXqc.declareNamespace("xs", NamespaceConstant.SCHEMA);
-                    //testXqc.declareNamespace("math", NamespaceConstant.MATH);
-                    testXqc.declareNamespace("map", NamespaceConstant.MAP_FUNCTIONS);
-                    testXqc.declareNamespace("array", NamespaceConstant.ARRAY_FUNCTIONS);
-                    ErrorCollector errorCollector = new ErrorCollector();
-                    testXqc.setErrorListener(errorCollector);
-                    String decVars = env.paramDecimalDeclarations.toString();
-                    if (decVars.length() != 0) {
-                        int x = exp.indexOf("(:%DECL%:)");
-                        if (x < 0) {
-                            exp = decVars + exp;
-                        } else {
-                            exp = exp.substring(0, x) + decVars + exp.substring(x + 13);
-                        }
-                    }
-                    String vars = env.paramDeclarations.toString();
-                    if (vars.length() != 0) {
-                        int x = exp.indexOf("(:%VARDECL%:)");
-                        if (x < 0) {
-                            exp = vars + exp;
-                        } else {
-                            exp = exp.substring(0, x) + vars + exp.substring(x + 13);
-                        }
-                    }
-                    ModuleResolver mr = new ModuleResolver(xpc);
-                    mr.setTestCase(testCase);
-                    testXqc.getProcessor().getUnderlyingConfiguration().setModuleURIResolver(mr);
-                    testXqc.setModuleURIResolver(mr);
+                        ModuleResolver mr = new ModuleResolver(xpc); //
+
+                        mr.setTestCase(testCase);
+                        testXpc.getProcessor().getUnderlyingConfiguration().setModuleURIResolver(mr);
 
-                    try {
-                        testXqc.getUnderlyingStaticContext().setModuleLocation(
-                            new ExplicitLocation(testXqc.getBaseURI().toString(), testCase.getLineNumber(), -1));
-                        XQueryExecutable q = null;
                         try {
-                            q = testXqc.compile(exp);
-                        } catch (SaxonApiException e) {
-                            println("Static error in query");
-                            throw e;
-                        }
-                        if (optimization != null) {
-                            // Test whether required optimizations have been performed
-                            XdmDestination expDest = new XdmDestination();
-                            Configuration config = driverProc.getUnderlyingConfiguration();
-                            try {
-                                ExpressionPresenter presenter = new ExpressionPresenter(config, expDest.getReceiver(config));
-                                q.getUnderlyingCompiledQuery().explain(presenter);
-                                presenter.close();
-                            } catch (XPathException e) {
-                                e.printStackTrace();
+                            XPathSelector selector = testXpc.compile(exp).load();
+                            for (QName varName : env.params.keySet()) {
+                                selector.setVariable(varName, env.params.get(varName));
                             }
-                            XdmNode explanation = expDest.getXdmNode();
-                            XdmItem optResult = xpc.evaluateSingle(optimization.getAttributeValue(new QName("assert")), explanation);
-                            if (((XdmAtomicValue) optResult).getBooleanValue()) {
-                                println("Optimization result OK");
-                            } else {
-                                println("Failed optimization test");
-                                driverProc.writeXdmValue(explanation, driverProc.newSerializer(System.err));
-                                writeTestcaseElement(testCaseName, "fail", "Failed optimization assertions");
-                                noteFailure(testSetName, testCaseName);
-                                return;
+                            if (env.contextItem != null) {
+                                selector.setContextItem(env.contextItem);
+                            }
+                            selector.setURIResolver(new TestURIResolver(env));
+                            //selector.setURIResolver(new TestURIResolver(env));
+                            if (env.unparsedTextResolver != null) {
+                                selector.getUnderlyingXPathContext().setUnparsedTextURIResolver(env.unparsedTextResolver);
                             }
-
-                        }
-                        XQueryEvaluator selector = q.load();
-                        for (QName varName : env.params.keySet()) {
-                            selector.setExternalVariable(varName, env.params.get(varName));
-                        }
-                        if (env.contextItem != null) {
-                            selector.setContextItem(env.contextItem);
-                        }
-                        selector.setURIResolver(new TestURIResolver(env));
-                        if (env.unparsedTextResolver != null) {
-                            selector.getUnderlyingQueryContext().setUnparsedTextURIResolver(env.unparsedTextResolver);
-                        }
-                        if (needSerializedResult) {
-                            StringWriter sw = new StringWriter();
-                            Serializer serializer = env.processor.newSerializer(sw);
-                            selector.setDestination(serializer);
-                            selector.run();
-                            outcome.setPrincipalSerializedResult(sw.toString());
-                        }
-                        if (needResultValue) {
                             XdmValue result = selector.evaluate();
                             outcome.setPrincipalResult(result);
+                        } catch (SaxonApiException err) {
+                            println(err.getMessage());
+                            outcome.setException(err);
+                        } catch (Exception err) {
+                            println(err.getMessage());
+                            err.printStackTrace();
+                            writeTestcaseElement(testCaseName, "fail", "*** crashed: " + err.getMessage());
+                            //notrun++;
+                            //return;
                         }
-                    } catch (SaxonApiException err) {
-                        println("in TestSet " + testSetName);
-                        println(err.getMessage());
-                        outcome.setException(err);
-                        outcome.setErrorsReported(errorCollector.getErrorCodes());
-                    } catch (LicenseException err) {
-                        // treat this as "facility not available", XQST0075
-                        println("in TestSet " + testSetName);
-                        println(err.getMessage());
-                        XPathException xe = new XPathException(err.getMessage(), "XPST0075");
-                        outcome.setException(new SaxonApiException(xe));
-                        try {
-                            errorCollector.error(xe);
-                        } catch (Exception e3) {
-                            // ignore the exception
-                        }
-                        outcome.setErrorsReported(errorCollector.getErrorCodes());
+                        break;
                     }
-                } else {
-                    writeTestcaseElement(testCaseName, "notRun", "No processor found");
-                    notrun++;
-                    return;
+                    case "XQ": {
+                        XQueryCompiler testXqc = env.xqueryCompiler;
+                        testXqc.setLanguageVersion(langVersion);
+                        testXqc.declareNamespace("fn", NamespaceConstant.FN);
+                        testXqc.declareNamespace("xs", NamespaceConstant.SCHEMA);
+                        //testXqc.declareNamespace("math", NamespaceConstant.MATH);
+                        testXqc.declareNamespace("map", NamespaceConstant.MAP_FUNCTIONS);
+                        testXqc.declareNamespace("array", NamespaceConstant.ARRAY_FUNCTIONS);
+                        ErrorCollector errorCollector = new ErrorCollector();
+                        testXqc.setErrorListener(errorCollector);
+                        String decVars = env.paramDecimalDeclarations.toString();
+                        if (decVars.length() != 0) {
+                            int x = exp.indexOf("(:%DECL%:)");
+                            if (x < 0) {
+                                exp = decVars + exp;
+                            } else {
+                                exp = exp.substring(0, x) + decVars + exp.substring(x + 13);
+                            }
+                        }
+                        String vars = env.paramDeclarations.toString();
+                        if (vars.length() != 0) {
+                            int x = exp.indexOf("(:%VARDECL%:)");
+                            if (x < 0) {
+                                exp = vars + exp;
+                            } else {
+                                exp = exp.substring(0, x) + vars + exp.substring(x + 13);
+                            }
+                        }
+                        ModuleResolver mr = new ModuleResolver(xpc);
+                        mr.setTestCase(testCase);
+                        testXqc.getProcessor().getUnderlyingConfiguration().setModuleURIResolver(mr);
+                        testXqc.setModuleURIResolver(mr);
+
+                        try {
+                            testXqc.getUnderlyingStaticContext().setModuleLocation(
+                                    new ExplicitLocation(testXqc.getBaseURI().toString(), testCase.getLineNumber(), -1));
+                            XQueryExecutable q = null;
+                            try {
+                                q = testXqc.compile(exp);
+                            } catch (SaxonApiException e) {
+                                println("Static error in query");
+                                throw e;
+                            }
+                            if (optimization != null) {
+                                // Test whether required optimizations have been performed
+                                XdmDestination expDest = new XdmDestination();
+                                Configuration config = driverProc.getUnderlyingConfiguration();
+                                try {
+                                    Receiver receiver = expDest.getReceiver(config.makePipelineConfiguration(), config.obtainDefaultSerializationProperties());
+                                    ExpressionPresenter presenter = new ExpressionPresenter(config, receiver);
+                                    q.getUnderlyingCompiledQuery().explain(presenter);
+                                    presenter.close();
+                                } catch (XPathException e) {
+                                    e.printStackTrace();
+                                }
+                                XdmNode explanation = expDest.getXdmNode();
+                                XPathCompiler assertXpc = xpc.getProcessor().newXPathCompiler();
+                                XdmItem optResult = assertXpc.evaluateSingle(optimization.attribute("assert"), explanation);
+                                if (((XdmAtomicValue) optResult).getBooleanValue()) {
+                                    println("Optimization result OK");
+                                } else {
+                                    println("Failed optimization test {" + optimization.attribute("assert") + "}");
+                                    driverProc.writeXdmValue(explanation, driverProc.newSerializer(System.err));
+                                    writeTestcaseElement(testCaseName, "fail", "Failed optimization assertions");
+                                    noteFailure(testSetName, testCaseName);
+                                    return;
+                                }
+
+                            }
+                            XQueryEvaluator selector = q.load();
+                            for (QName varName : env.params.keySet()) {
+                                selector.setExternalVariable(varName, env.params.get(varName));
+                            }
+                            if (env.contextItem != null) {
+                                selector.setContextItem(env.contextItem);
+                            }
+                            selector.setURIResolver(new TestURIResolver(env));
+                            if (env.unparsedTextResolver != null) {
+                                selector.getUnderlyingQueryContext().setUnparsedTextURIResolver(env.unparsedTextResolver);
+                            }
+                            if (needSerializedResult) {
+                                StringWriter sw = new StringWriter();
+                                Serializer serializer = env.processor.newSerializer(sw);
+                                selector.setDestination(serializer);
+                                selector.run();
+                                outcome.setPrincipalSerializedResult(sw.toString());
+                            }
+                            if (needResultValue) {
+                                XdmValue result = selector.evaluate();
+                                outcome.setPrincipalResult(result);
+                            }
+                        } catch (SaxonApiException err) {
+                            println("in TestSet " + testSetName);
+                            println(err.getMessage());
+                            outcome.setException(err);
+                            outcome.setErrorsReported(errorCollector.getErrorCodes());
+                        } catch (LicenseException err) {
+                            // treat this as "facility not available", XQST0075
+                            println("in TestSet " + testSetName);
+                            println(err.getMessage());
+                            XPathException xe = new XPathException(err.getMessage(), "XPST0075");
+                            outcome.setException(new SaxonApiException(xe));
+                            try {
+                                errorCollector.error(xe);
+                            } catch (Exception e3) {
+                                // ignore the exception
+                            }
+                            outcome.setErrorsReported(errorCollector.getErrorCodes());
+                        }
+                        break;
+                    }
+                    default:
+                        writeTestcaseElement(testCaseName, "notRun", "No processor found");
+                        notrun++;
+                        return;
                 }
             }
 
@@ -736,7 +760,7 @@ public class QT3TestDriverHE extends TestDriver {
     }
 
 
-    protected void writeResultFilePreamble(Processor processor, XdmNode catalog) throws Exception {
+    protected void writeResultFilePreamble(Processor processor, XdmNode catalog) throws IOException, SaxonApiException, XMLStreamException {
         resultsDoc.writeResultFilePreamble(processor, catalog);
     }
 
@@ -785,7 +809,8 @@ public class QT3TestDriverHE extends TestDriver {
 
         public StreamSource[] resolve(String moduleURI, String baseURI, String[] locations) throws XPathException {
             try {
-                XdmValue files = catXPC.evaluate("./module[@uri='" + moduleURI + "']/@file/string()", testCase);
+                XdmValue files = testCase.select(child("module").where(attributeEq("uri", moduleURI))
+                    .then(attribute("file")).then(atomize())).asXdmValue();
                 if (files.size() == 0) {
                     throw new XPathException("Failed to find module entry for " + moduleURI);
                 }
@@ -795,7 +820,7 @@ public class QT3TestDriverHE extends TestDriver {
                     ss[i] = getQuerySource(uri);
                 }
                 return ss;
-            } catch (SaxonApiException e) {
+            } catch (SaxonApiUncheckedException e) {
                 throw new XPathException(e);
             }
         }
@@ -820,7 +845,7 @@ public class QT3TestDriverHE extends TestDriver {
                 }
                 return node.asSource();
             } catch (URISyntaxException e) {
-                throw new XPathException(e);
+                throw new XPathException("Invalid URI: " + e.getMessage(), "FODC0005");
             }
         }
     }
@@ -864,8 +889,8 @@ public class QT3TestDriverHE extends TestDriver {
         XdmValue result = eval.evaluate();
         for (XdmItem item : result) {
             XdmNode node = (XdmNode) item;
-            String type = node.getAttributeValue(new QName("type"));
-            String value = node.getAttributeValue(new QName("value"));
+            String type = node.attribute("type");
+            String value = node.attribute("value");
             addDependency(type, value, ensureDependencySatisfied(node, env));
         }
 
