@@ -1,8 +1,8 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
-// This Source Code Form is “Incompatible With Secondary Licenses”, as defined by the Mozilla Public License, v. 2.0.
+// This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 package net.sf.saxon.testdriver;
@@ -11,20 +11,29 @@ package net.sf.saxon.testdriver;
 import net.sf.saxon.Configuration;
 import net.sf.saxon.Version;
 import net.sf.saxon.dom.DOMObjectModel;
+import net.sf.saxon.lib.Feature;
 import net.sf.saxon.lib.FeatureKeys;
 import net.sf.saxon.om.TreeModel;
 import net.sf.saxon.s9api.*;
+import net.sf.saxon.s9api.streams.Steps;
 import org.w3c.dom.Document;
 
 import javax.xml.stream.XMLStreamException;
 import java.io.File;
 import java.io.IOException;
 import java.util.*;
+import java.util.function.Predicate;
 import java.util.regex.Pattern;
 
+import static net.sf.saxon.s9api.streams.Predicates.attributeEq;
+import static net.sf.saxon.s9api.streams.Predicates.hasAttribute;
+import static net.sf.saxon.s9api.streams.Steps.*;
+import static net.sf.saxon.s9api.streams.XdmCollectors.asString;
+
 public abstract class TestDriver {
 
     protected String catalogFileName;
+    private XdmNode catalogNode;
     protected String exceptionsFileName = "exceptions.xml";
     protected String resultsDir = null;
     protected TestReport resultsDoc;
@@ -38,12 +47,14 @@ public abstract class TestDriver {
     protected boolean runPostureAndSweepTests = true;
     protected int generateByteCode = 1;
     protected boolean debugByteCode = false;
+    protected String optimizerFlags = "-";
     protected boolean jitFlag = false;
     protected int streaming = 1;
     protected TreeModel treeModel = TreeModel.TINY_TREE;
     protected boolean debug = false;
     protected boolean export = false;
     protected boolean runWithJS = false;
+    protected boolean runWithNodeJS = false;
     protected boolean relocatable = false;
     protected String jsBase = null;
     protected Pattern testPattern = null;
@@ -61,10 +72,10 @@ public abstract class TestDriver {
     protected boolean useXslt30Transformer = true;  // Temporary for controlling test processor
     protected boolean tracing = false;
     protected boolean isAltova = false;
-    protected Map<String, Integer> failSummary = new TreeMap<String, Integer>();
+    protected Map<String, Integer> failSummary = new TreeMap<>();
     protected boolean quiet = false;
 
-    static Set<String> unsharedEnvironments = new HashSet<String>();
+    static Set<String> unsharedEnvironments = new HashSet<>();
     static {
         unsharedEnvironments.add("import-schema-e01");
         unsharedEnvironments.add("merge002");
@@ -99,35 +110,44 @@ public abstract class TestDriver {
         catalog = testSuiteDir+catalog;
 
         for (int i = 2; i < args.length; i++) {
+            int colon = args[i].indexOf(':');
+            String argValue = colon < 0 ? "" : args[i].substring(colon+1);
+
             if (args[i].startsWith("-t:")) {
-                testPattern = Pattern.compile(args[i].substring(3));
+                testPattern = Pattern.compile(argValue);
             }
             if (args[i].startsWith("-s:")) {
-                requestedTestSet = args[i].substring(3);
+                requestedTestSet = argValue;
             }
             if (args[i].equals("-quiet")) {
                 quiet = true;
             }
             if (args[i].startsWith("-o:")) {
-                resultsDir = args[i].substring(3);
+                resultsDir = argValue;
             }
             if (args[i].startsWith("-debug")) {
                 debug = true;
             }
-            if (args[i].startsWith("-export") && !args[i].equals("-export:off")) {
+            if (args[i].startsWith("-export") && !argValue.equals("off")) {
                 export = true;
             }
-            if (args[i].startsWith("-jit") && !args[i].equals("-jit:off")) {
+            if (args[i].startsWith("-jit") && !argValue.equals("off")) {
                 jitFlag = true;
             }
             if (args[i].startsWith("-js")) {
                 export = true;
                 runWithJS = true;
                 treeModel = DOMObjectModel.getInstance();
-                if (args[i].startsWith("-js:")) {
-                    jsBase = args[i].substring(4);
+                if (args[i].startsWith("-js:") || args[i].startsWith("-js2:")) {
+                    jsBase = argValue;
                 }
             }
+            if (args[i].startsWith("-nodeJS")) {
+                export = true;
+                runWithJS = true;
+                runWithNodeJS = true;
+                treeModel = DOMObjectModel.getInstance();
+            }
             if (args[i].equals("-unfolded")) {
                 unfolded = true;
             }
@@ -135,73 +155,70 @@ public abstract class TestDriver {
                 saveResults = true;
             }
             if (args[i].startsWith("-exceptions:")) {
-                exceptionsFileName = args[i].substring(12);
+                exceptionsFileName = argValue;
             }
             if (args[i].equals("-T")) {
                 tracing = true;
             }
             if (args[i].startsWith("-bytecode:")) {
-                String value = args[i].substring(10);
-                if (args[i].substring(10).equals("on")) {
-                    generateByteCode = 0;
-                } else if (args[i].substring(10).equals("debug")) {
-                    debugByteCode = true;
-                    generateByteCode = 0;
-                } else if (args[i].substring(10).equals("off")) {
-                    generateByteCode = -1;
+                switch (argValue) {
+                    case "on":
+                        generateByteCode = 0;
+                        break;
+                    case "debug":
+                        debugByteCode = true;
+                        generateByteCode = 0;
+                        break;
+                    case "off":
+                        generateByteCode = -1;
 
-                } else {
-                    generateByteCode = Integer.parseInt(value);
+                        break;
+                    default:
+                        generateByteCode = Integer.parseInt(argValue);
+                        break;
                 }
             }
             if (args[i].startsWith("-streaming:")) {
-                if (args[i].substring(11).equals("off")) {
-                    streaming = 0;
-                } else if (args[i].substring(11).equals("on")) {
-                    streaming = 1;
-                } else if (args[i].substring(11).equals("strict")) {
-                    streaming = 3;
-                } else {
-                    throw new Exception("-streaming must be off|on|strict");
+                switch (argValue) {
+                    case "off":
+                        streaming = 0;
+                        break;
+                    case "on":
+                        streaming = 1;
+                        break;
+                    case "strict":
+                        streaming = 3;
+                        break;
+                    default:
+                        throw new Exception("-streaming must be off|on|strict");
                 }
             }
             if (args[i].startsWith("-tree")) {
-                String model = args[i].substring(6);
-                treeModel = getTreeModel(model);
+                treeModel = getTreeModel(argValue);
                 if (treeModel == null) {
-                    throw new Exception("The requested TreeModel '" + model + "' does not exist");
+                    throw new Exception("The requested TreeModel '" + argValue + "' does not exist");
                 }
 
             }
             if (args[i].startsWith("-lang:")) {
-                String specStr = null;
-                specStr = args[i].substring(6);
-                lang = specStr;
-                processSpec(specStr);
+                lang = argValue;
+                processSpec(argValue);
             }
             // Temporary for controlling test processor
             if (args[i].startsWith("-xt30:")) {
-                if (args[i].substring(6).equals("on")) {
-                    useXslt30Transformer = true;
-                } else if (args[i].substring(6).equals("off")) {
-                    useXslt30Transformer = false;
-                }
+                useXslt30Transformer = argValue.equals("on");
             }
             // Determine whether Posture/Sweep tests should be run
             if (args[i].startsWith("-ps:")) {
-                if (args[i].substring(4).equals("on")) {
-                    runPostureAndSweepTests = true;
-                } else if (args[i].substring(4).equals("off")) {
-                    runPostureAndSweepTests = false;
-                }
+                runPostureAndSweepTests = argValue.equals("on");
             }
             // Determine whether static base uri should be exported
             if (args[i].startsWith("-relocatable:")) {
-                if (args[i].substring(13).equals("on")) {
-                    relocatable = true;
-                } else if (args[i].substring(13).equals("off")) {
-                    relocatable = false;
-                }
+                relocatable = argValue.equals("on");
+            }
+            // Set optimization flags
+            if (args[i].startsWith("-opt:")) {
+                optimizerFlags = argValue;
             }
         }
         if (resultsDoc == null) {
@@ -224,7 +241,7 @@ public abstract class TestDriver {
         driverSerializer.setOutputProperty(Serializer.Property.METHOD, "adaptive");
         driverSerializer.setOutputProperty(Serializer.Property.OMIT_XML_DECLARATION, "yes");
         driverSerializer.setOutputProperty(Serializer.Property.INDENT, "yes");
-        driverProc.setConfigurationProperty(FeatureKeys.STABLE_UNPARSED_TEXT, true);
+        driverProc.setConfigurationProperty(Feature.STABLE_UNPARSED_TEXT, true);
         processCatalog(new File(catalog));
         printResults(resultsDir + "/results" + Version.getProductVersion() + ".xml");
     }
@@ -281,29 +298,32 @@ public abstract class TestDriver {
     protected void processCatalog(File catalogFile) throws SaxonApiException {
         if (driverProc.getSaxonEdition().equals("EE")) {
             if (generateByteCode == 0 && !debugByteCode) {
-                driverProc.setConfigurationProperty(FeatureKeys.GENERATE_BYTE_CODE, "true");
-                driverProc.setConfigurationProperty(FeatureKeys.DEBUG_BYTE_CODE, "false");
+                driverProc.setConfigurationProperty(Feature.GENERATE_BYTE_CODE, true);
+                driverProc.setConfigurationProperty(Feature.DEBUG_BYTE_CODE, false);
             } else if (generateByteCode >0) {
                 driverProc.setConfigurationProperty(FeatureKeys.GENERATE_BYTE_CODE, ""+generateByteCode);
-                driverProc.setConfigurationProperty(FeatureKeys.DEBUG_BYTE_CODE, "false");
+                driverProc.setConfigurationProperty(Feature.DEBUG_BYTE_CODE, false);
             } else if (generateByteCode == 0 && debugByteCode) {
-                driverProc.setConfigurationProperty(FeatureKeys.GENERATE_BYTE_CODE, "true");
-                driverProc.setConfigurationProperty(FeatureKeys.DEBUG_BYTE_CODE, "true");
+                driverProc.setConfigurationProperty(Feature.GENERATE_BYTE_CODE, true);
+                driverProc.setConfigurationProperty(Feature.DEBUG_BYTE_CODE, true);
             } else {
-                driverProc.setConfigurationProperty(FeatureKeys.GENERATE_BYTE_CODE, "false");
-                driverProc.setConfigurationProperty(FeatureKeys.DEBUG_BYTE_CODE, "false");
+                driverProc.setConfigurationProperty(Feature.GENERATE_BYTE_CODE, false);
+                driverProc.setConfigurationProperty(Feature.DEBUG_BYTE_CODE, false);
             }
         }
         DocumentBuilder catbuilder = driverProc.newDocumentBuilder();
         catbuilder.setTreeModel(treeModel);
         catbuilder.setLineNumbering(true);
         XdmNode catalog = catbuilder.build(catalogFile);
+        catalogNode = catalog;
         XPathCompiler xpc = driverProc.newXPathCompiler();
         xpc.setLanguageVersion("3.1");
         xpc.setCaching(true);
         xpc.declareNamespace("", catalogNamespace());
 
-        createGlobalEnvironments(catalog, xpc);
+        if (requestedTestSet == null) {
+            createGlobalEnvironments(catalog, xpc);
+        }
 
         try {
             writeResultFilePreamble(driverProc, catalog);
@@ -316,17 +336,19 @@ public abstract class TestDriver {
 
         if (requestedTestSet != null) {
             try {
-                XdmNode funcSetNode = (XdmNode) xpc.evaluateSingle("//test-set[@name='" + requestedTestSet + "']", catalog);
-                if (funcSetNode == null) {
+                Predicate<XdmNode> isRequestedTestSet = node -> requestedTestSet.equals(node.attribute("name"));
+                Optional<XdmNode> funcSetNode = catalog.select(
+                        descendant("test-set").where(isRequestedTestSet)).asOptionalNode();
+                if (!funcSetNode.isPresent()) {
                     throw new Exception("Test-set " + requestedTestSet + " not found!");
                 }
-                processTestSet(catbuilder, xpc, funcSetNode);
+                processTestSet(catbuilder, xpc, funcSetNode.get());
             } catch (Exception e1) {
                 e1.printStackTrace();
             }
         } else {
-            for (XdmItem testSet : xpc.evaluate("//test-set", catalog)) {
-                processTestSet(catbuilder, xpc, (XdmNode) testSet);
+            for (XdmNode testSet : catalog.select(descendant("test-set")).asList()) {
+                processTestSet(catbuilder, xpc, testSet);
             }
         }
         try {
@@ -344,7 +366,7 @@ public abstract class TestDriver {
 
     /**
      * Look for an exceptions.xml document with the general format:
-     * <p/>
+     * {@code
      * <exceptions xmlns="...test catalog namespace...">
      * <exception test-set ="testset1" test-case="testcase" run="yes/no/not-unfolded"
      * bug="bug-reference" reason="">
@@ -356,6 +378,7 @@ public abstract class TestDriver {
      * </optimization>
      * </exception>
      * </exceptions>
+     * }
      */
 
     protected void readExceptionsFile() {
@@ -408,7 +431,7 @@ public abstract class TestDriver {
                         XdmSequenceIterator iter2 = entry.axisIterator(Axis.CHILD, new QName("optimization"));
                         if (iter2.hasNext()) {
                             XdmNode optim = (XdmNode) iter2.next();
-                            optimizationAssertions.put(testName, optim.getAttributeValue(new QName("", "assert")));
+                            optimizationAssertions.put(testName, optim.attribute("assert"));
                         }
                     }
                 }
@@ -431,19 +454,21 @@ public abstract class TestDriver {
     }
 
     protected Environment getEnvironment(XdmNode testCase, XPathCompiler xpc) throws SaxonApiException {
-        String testCaseName = testCase.getAttributeValue(new QName("name"));
-        XdmNode environmentNode = (XdmNode) xpc.evaluateSingle("environment", testCase);
+        String testCaseName = testCase.attribute("name");
+        Optional<XdmNode> environmentNode = testCase.select(child("environment")).asOptionalNode();
         Environment env;
-        if (environmentNode == null) {
+        if (!environmentNode.isPresent()) {
             env = localEnvironments.get("default");
         } else {
-            String envName = environmentNode.getAttributeValue(new QName("ref"));
+            String envName = environmentNode.get().attribute("ref");
             if (envName == null || envName.equals("")) {
                 boolean baseUriCheck = false;
                 env = null;
                 try {
-                    env = Environment.processEnvironment(this, xpc, environmentNode, null, localEnvironments.get("default"));
-                    baseUriCheck = ((XdmAtomicValue) xpc.evaluateSingle("static-base-uri/@uri='#UNDEFINED'", environmentNode)).getBooleanValue();
+                    env = Environment.processEnvironment(
+                            this, xpc, environmentNode.get(), null, localEnvironments.get("default"));
+                    baseUriCheck =  environmentNode.get().select(
+                            child("static-base-uri").where(attributeEq("uri", "#UNDEFINED"))).exists();
                 } catch (NullPointerException ex) {
                     ex.printStackTrace();
                     System.err.println("Failure loading environment");
@@ -462,7 +487,7 @@ public abstract class TestDriver {
                 }
                 if (env == null) {
                     try {
-                        for (XdmItem e : xpc.evaluate("//environment[@name='" + envName + "']", testCase)) {
+                        for (XdmNode e : testCase.getParent().select(descendant("environment").where(attributeEq("name", envName))).asList()) {
                             Environment.processEnvironment(this, xpc, e, localEnvironments, localEnvironments.get("default"));
                         }
                         env = localEnvironments.get(envName);
@@ -477,10 +502,16 @@ public abstract class TestDriver {
                         }
                     }
                 }
+                if (env == null) {
+                    // when running a single test set, we no longer pre-build the global environments. So we now
+                    // need to check if the reference is to a global environment
+                    createGlobalEnvironments(catalogNode, xpc);
+                    env = globalEnvironments.get(envName);
+                }
                 if (env == null) {
                     println("*** Unknown environment " + envName);
                     //writeTestcaseElement(testCaseName, "fail", "Environment " + envName + " not found", null);
-                    noteFailure(testCase.getParent().getAttributeValue(new QName("name")), testCaseName);
+                    noteFailure(testCase.getParent().attribute("name"), testCaseName);
                     return null;
                 }
 
@@ -501,9 +532,9 @@ public abstract class TestDriver {
      * Export a stylesheet (used in the XSLT JS driver
      */
 
-    public File exportStylesheet(XsltCompiler compiler, String fileName) {
+    public File exportStylesheet(XsltCompiler compiler, String fileName) throws SaxonApiException {
         return null;
-    };
+    }
 
     /**
      * Inject code into a compiled query
@@ -520,17 +551,12 @@ public abstract class TestDriver {
         if (requestedTestSet != null) {
             failSummary.put(testCase, 1);
         } else {
-            Integer fails = failSummary.get(testSet);
-            if (fails != null) {
-                failSummary.put(testSet, fails + 1);
-            } else {
-                failSummary.put(testSet, 1);
-            }
+            failSummary.merge(testSet, 1, (a, b) -> a + b);
         }
     }
 
     protected void writeResultFilePreamble(Processor processor, XdmNode catalog)
-            throws IOException, SaxonApiException, XMLStreamException, Exception {
+            throws IOException, SaxonApiException, XMLStreamException {
         resultsDoc.writeResultFilePreamble(processor, catalog);
     }
 
@@ -563,16 +589,16 @@ public abstract class TestDriver {
         String testName;
         String testSet;
         startTestSetElement(testSetNode);
-        File testSetFile = new File(testSuiteDir + "/" + testSetNode.getAttributeValue(new QName("file")));
+        File testSetFile = new File(testSuiteDir + "/" + testSetNode.attribute("file"));
         XdmNode testSetDocNode = catbuilder.build(testSetFile);
         createLocalEnvironments(testSetDocNode);
         boolean run = true;
         // pick up any test-set level dependencies in the QT3 catalog
-        if (((XdmAtomicValue) xpc.evaluate("exists(/test-set/dependency)", testSetDocNode).itemAt(0)).getBooleanValue()) {
-            for (XdmItem dependency : xpc.evaluate("/test-set/dependency", testSetDocNode)) {
-                if (!ensureDependencySatisfied((XdmNode) dependency, localEnvironments.get("default"))) {
-                    for (XdmItem testCase : xpc.evaluate("//test-case", testSetDocNode)) {
-                        String testCaseName = ((XdmNode) testCase).getAttributeValue(new QName("name"));
+        if (testSetDocNode.select(child("test-set").then(child("dependency"))).exists()) {
+            for (XdmNode dependency : testSetDocNode.select(child("test-set").then(child("dependency"))).asList()) {
+                if (!ensureDependencySatisfied(dependency, localEnvironments.get("default"))) {
+                    for (XdmNode testCase : testSetDocNode.select(descendant("test-case")).asList()) {
+                        String testCaseName = testCase.attribute("name");
                         resultsDoc.writeTestcaseElement(testCaseName, "n/a", "test-set dependencies not satisfied");
                         notrun++;
                     }
@@ -582,21 +608,21 @@ public abstract class TestDriver {
             }
         }
         // pick up any test-set level dependencies in the XSLT 3.0 catalog
-        if (((XdmAtomicValue) xpc.evaluate("exists(/test-set/dependencies)", testSetDocNode).itemAt(0)).getBooleanValue()) {
-            for (XdmItem dependency : xpc.evaluate("/test-set/dependencies/*", testSetDocNode)) {
-                if (!ensureDependencySatisfied((XdmNode) dependency, localEnvironments.get("default"))) {
-                    for (XdmItem testCase : xpc.evaluate("//test-case", testSetDocNode)) {
-                        String type = ((XdmNode) dependency).getNodeName().getLocalName();
-                        String value = ((XdmNode) dependency).getAttributeValue(new QName("", "value"));
+        if (testSetDocNode.select(child("test-set").then(child("dependencies"))).exists()) {
+            for (XdmNode dependency : testSetDocNode.select(path("test-set", "dependencies", "*")).asListOfNodes()) {
+                if (!ensureDependencySatisfied(dependency, localEnvironments.get("default"))) {
+                    for (XdmNode testCase : testSetDocNode.select(descendant("test-case")).asList()) {
+                        String type = dependency.getNodeName().getLocalName();
+                        String value = dependency.attribute("value");
                         if (value == null) {
                             value = type;
                         } else {
                             value = type + ":" + value;
                         }
-                        if ("false".equals(((XdmNode) dependency).getAttributeValue(new QName("", "satisfied")))) {
+                        if ("false".equals(dependency.attribute("satisfied"))) {
                             value = "!" + value;
                         }
-                        String testCaseName = ((XdmNode) testCase).getAttributeValue(new QName("name"));
+                        String testCaseName = testCase.attribute("name");
                         resultsDoc.writeTestcaseElement(testCaseName, "n/a", "test-set dependencies not satisfied: " + value);
                         notrun++;
                     }
@@ -607,8 +633,8 @@ public abstract class TestDriver {
         }
         if (run) {
             if (testPattern == null) {
-                for (XdmItem env : xpc.evaluate("//environment[@name]", testSetDocNode)) {
-                    String envName = ((XdmNode) env).getAttributeValue(new QName("name"));
+                for (XdmNode env : testSetDocNode.select(descendant("environment").where(hasAttribute("name"))).asList()) {
+                    String envName = env.attribute("name");
                     if (!unsharedEnvironments.contains(envName)) {
                         try {
                             Environment.processEnvironment(this, xpc, env, localEnvironments, localEnvironments.get("default"));
@@ -619,22 +645,20 @@ public abstract class TestDriver {
                     }
                 }
             }
-            testSet = xpc.evaluateSingle("/test-set/@name", testSetDocNode).getStringValue();
-            for (XdmItem testCase : xpc.evaluate("//test-case", testSetDocNode)) {
+            testSet = testSetDocNode.select(Steps.path("/", "test-set", "@name")).collect(asString());
+            testSetDocNode.select(descendant("test-case")).forEach(testCase -> {
+                String name = testCase.attribute("name");
+                if (testPattern == null || testPattern.matcher(name).matches()) {
+                    println("-s:" + testSet + " -t:" + name);
 
-                testName = xpc.evaluateSingle("@name", testCase).getStringValue();
-                if (testPattern != null && !testPattern.matcher(testName).matches()) {
-                    continue;
+                    try {
+                        runTestCase(testCase, xpc);
+                    } catch (SaxonApiException ex) {
+                        ex.printStackTrace();
+                        System.err.println("*** Error in evaluating testcase:" + ex.getMessage());
+                    }
                 }
-                println("-s:" + testSet + " -t:" + testName);
-
-                try {
-                    runTestCase((XdmNode) testCase, xpc);
-                } catch (SaxonApiException ex) {
-                    ex.printStackTrace();
-                    System.err.println("*** Error in evaluating testcase:" + ex.getMessage());
-                }
-            }
+            });
         }
         writeTestSetEndElement();
     }
@@ -681,13 +705,12 @@ public abstract class TestDriver {
 
     public abstract boolean ensureDependencySatisfied(XdmNode dependency, Environment env);
 
+
     protected void copySchemaNamespaces(Environment env, XPathCompiler testXpc) {
         Configuration config = env.xpathCompiler.getProcessor().getUnderlyingConfiguration();
-        //if (config instanceof EnterpriseConfiguration) {
         for (String s : config.getImportedNamespaces()) {
             testXpc.importSchemaNamespace(s);
         }
-        //}
     }
 
 }
