@@ -1,13 +1,12 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
-// This Source Code Form is “Incompatible With Secondary Licenses”, as defined by the Mozilla Public License, v. 2.0.
+// This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 package net.sf.saxon.testdriver;
 
-import net.sf.saxon.Version;
 import net.sf.saxon.expr.ItemMappingFunction;
 import net.sf.saxon.expr.ItemMappingIterator;
 import net.sf.saxon.expr.XPathContext;
@@ -16,7 +15,7 @@ import net.sf.saxon.expr.sort.AtomicSortComparer;
 import net.sf.saxon.expr.sort.CodepointCollator;
 import net.sf.saxon.expr.sort.GenericAtomicComparer;
 import net.sf.saxon.functions.DeepEqual;
-import net.sf.saxon.lib.FeatureKeys;
+import net.sf.saxon.lib.Feature;
 import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.lib.StringCollator;
 import net.sf.saxon.om.*;
@@ -27,7 +26,6 @@ import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.type.Type;
 import net.sf.saxon.value.AtomicValue;
-import net.sf.saxon.value.SequenceExtent;
 import net.sf.saxon.value.StringValue;
 import net.sf.saxon.value.Whitespace;
 
@@ -64,9 +62,10 @@ public class TestOutcome {
     private boolean warningsReported;
     private SaxonApiException exception;
     private String comment;
-    private Set<XdmNode> xslMessages = new HashSet<XdmNode>(4);
-    private Map<URI, SingleResultDoc> xslResultDocuments = new HashMap<URI, SingleResultDoc>(4);
+    private Set<XdmNode> xslMessages = new HashSet<>(4);
+    private Map<URI, SingleResultDoc> xslResultDocuments = new HashMap<>(4);
     private String wrongError;
+    private String resultVar;
 
     public TestOutcome(TestDriver driver) {
         this.driver = driver;
@@ -80,6 +79,10 @@ public class TestOutcome {
         return exception != null;
     }
 
+    public void setResultVar(String variableName) {
+        resultVar = variableName;
+    }
+
     public SaxonApiException getException() {
         return exception;
     }
@@ -146,7 +149,7 @@ public class TestOutcome {
 
     public synchronized void addReportedError(String error) {
         if (errorsReported == null) {
-            errorsReported = new HashSet<String>();
+            errorsReported = new HashSet<>();
         }
         errorsReported.add(error);
     }
@@ -156,7 +159,7 @@ public class TestOutcome {
     }
 
     public String toString() {
-        return (isException() ? "EXCEPTION " + exception.getMessage() : getPrincipalResult().toString());
+        return isException() ? "EXCEPTION " + exception.getMessage() : getPrincipalResult().toString();
     }
 
     public void setPrincipalSerializedResult(String result) {
@@ -222,7 +225,7 @@ public class TestOutcome {
             try {
                 s.serializeXdmValue(value);
             } catch (SaxonApiException err) {
-                return ("SERIALIZATION FAILED: " + err.getMessage());
+                return "SERIALIZATION FAILED: " + err.getMessage();
             }
             String str = sw.toString().trim();
             if (uri == null) {
@@ -265,7 +268,7 @@ public class TestOutcome {
             try {
                 s.serializeXdmValue(value);
             } catch (SaxonApiException err) {
-                return ("SERIALIZATION FAILED: " + err.getMessage());
+                return "SERIALIZATION FAILED: " + err.getMessage();
             }
             String str = sw.toString().trim();
             doc.serialization = str;
@@ -287,8 +290,7 @@ public class TestOutcome {
         return sw.toString().trim();
     }
 
-    public boolean testAssertion(XdmNode assertion, SingleResultDoc result, XPathCompiler assertXpc, XPathCompiler catalogXpc, boolean debug)
-            throws SaxonApiException {
+    public boolean testAssertion(XdmNode assertion, SingleResultDoc result, XPathCompiler assertXpc, XPathCompiler catalogXpc, boolean debug) {
         try {
             String tag = assertion.getNodeName().getLocalName();
             boolean success = testAssertion2(assertion, result, assertXpc, catalogXpc, debug);
@@ -308,7 +310,7 @@ public class TestOutcome {
                             driver.println(wrongError);
                         }
                     } else {
-                        driver.println("Expected exception " + assertion.getAttributeValue(new QName("code")) + "; got success");
+                        driver.println("Expected error " + assertion.attribute("code") + "; got success");
                     }
                 } else if (!success && isException()) {
                     FastStringBuffer fsb = new FastStringBuffer(32);
@@ -338,104 +340,108 @@ public class TestOutcome {
     private boolean testAssertion2(XdmNode assertion, SingleResultDoc result, XPathCompiler assertXpc, XPathCompiler catalogXpc, boolean debug) throws SaxonApiException {
         String tag = assertion.getNodeName().getLocalName();
 
-        if (tag.equals("assert-eq")) {
-            return assertEq(assertion, result, assertXpc);
+        switch (tag) {
+            case "assert-eq":
+                return assertEq(assertion, result, assertXpc);
 
-        } else if (tag.equals("assert-deep-eq")) {
-            return assertDeepEq(assertion, result, assertXpc);
+            case "assert-deep-eq":
+                return assertDeepEq(assertion, result, assertXpc);
 
-        } else if (tag.equals("assert-permutation")) {
-            return assertPermutation(assertion, result, assertXpc);
+            case "assert-permutation":
+                return assertPermutation(assertion, result, assertXpc);
 
-        } else if (tag.equals("assert-xml")) {
-            return assertXml(assertion, result, assertXpc, catalogXpc, debug);
+            case "assert-xml":
+                return assertXml(assertion, result, assertXpc, catalogXpc, debug);
 
-        } else if (tag.equals("serialization-matches")) {
-            return assertSerializationMatches(assertion, result, catalogXpc);
+            case "serialization-matches":
+                return assertSerializationMatches(assertion, result, catalogXpc);
 
-        } else if (tag.equals("assert-serialization-error")) {
-            return assertSerializationError(assertion, result, assertXpc);
+            case "assert-serialization-error":
+                return assertSerializationError(assertion, result, assertXpc);
 
-        } else if (tag.equals("assert-empty")) {
-            return assertEmpty(result.value);
+            case "assert-empty":
+                return assertEmpty(result.value);
 
-        } else if (tag.equals("assert-count")) {
-            return assertCount(assertion, result);
+            case "assert-count":
+                return assertCount(assertion, result);
 
-        } else if (tag.equals("assert")) {
-            return assertXPath(assertion, result, assertXpc, debug);
+            case "assert":
+                return assertXPath(assertion, result, assertXpc, debug);
 
-        } else if (tag.equals("assert-string-value")) {
-            return assertStringValue(assertion, result, debug);
+            case "assert-string-value":
+                return assertStringValue(assertion, result, debug);
 
-        } else if (tag.equals("assert-serialization")) {
-            return assertSerialization(assertion, result, catalogXpc, debug);
+            case "assert-serialization":
+                return assertSerialization(assertion, result, catalogXpc, debug);
 
-        } else if (tag.equals("assert-type")) {
-            return assertType(assertion, result, assertXpc);
+            case "assert-type":
+                return assertType(assertion, result, assertXpc);
 
-        } else if (tag.equals("assert-true")) {
-            return assertTrue(result);
+            case "assert-true":
+                return assertTrue(result);
 
-        } else if (tag.equals("assert-false")) {
-            return assertFalse(result);
-        } else if (tag.equals("assert-warning")) {
-            return assertWarning();
+            case "assert-false":
+                return assertFalse(result);
+            case "assert-warning":
+                return assertWarning();
 
-        } else if (tag.equals("assert-message")) {
-            XdmNode subAssertion = (XdmNode) catalogXpc.evaluateSingle("*", assertion);
-            for (XdmNode message : xslMessages) {
-                if (message.getNodeKind() == XdmNodeKind.PROCESSING_INSTRUCTION && message.getNodeName().getLocalName().equals("trust") && message.getStringValue().equals("me")) {
-                    // In the JS tests, assume message assertions are OK
-                    return true;
+            case "assert-message": {
+                XdmNode subAssertion = (XdmNode) catalogXpc.evaluateSingle("*", assertion);
+                for (XdmNode message : xslMessages) {
+                    if (message.getNodeKind() == XdmNodeKind.PROCESSING_INSTRUCTION && message.getNodeName().getLocalName().equals("trust") && message.getStringValue().equals("me")) {
+                        // In the JS tests, assume message assertions are OK
+                        return true;
+                    }
+                    if (testAssertion2(subAssertion, new SingleResultDoc(message, ""), assertXpc, catalogXpc, debug)) {
+                        return true;
+                    }
                 }
-                if (testAssertion2(subAssertion, new SingleResultDoc(message, ""), assertXpc, catalogXpc, debug)) {
-                    return true;
-                }
-            }
-            return false;
-
-        } else if (tag.equals("assert-result-document")) {
-            XdmNode subAssertion = (XdmNode) catalogXpc.evaluateSingle("*", assertion);
-            URI uri = new File(driver.resultsDir + "/results/output.xml").toURI().resolve(assertion.getAttributeValue(new QName("uri")));
-            SingleResultDoc doc = getSecondaryResult(uri);
-            if (doc == null) {
-                System.err.println("**** No output document found for " + uri);
                 return false;
-            }
-            boolean ok = testAssertion2(subAssertion, doc, assertXpc, catalogXpc, debug);
-            if (!ok) {
-                System.err.println("**** Assertion failed for result-document " + uri);
-            }
-            return ok;
 
-        } else if (tag.equals("error")) {
-            return isException() && compareExpectedError(assertion);
-
-        } else if (tag.equals("all-of")) {
-            for (XdmItem child : catalogXpc.evaluate("*", assertion)) {
-                if (!testAssertion((XdmNode) child, result, assertXpc, catalogXpc, debug)) {
+            }
+            case "assert-result-document": {
+                XdmNode subAssertion = (XdmNode) catalogXpc.evaluateSingle("*", assertion);
+                URI uri = new File(driver.resultsDir + "/results/output.xml").toURI().resolve(assertion.attribute("uri"));
+                SingleResultDoc doc = getSecondaryResult(uri);
+                if (doc == null) {
+                    System.err.println("**** No output document found for " + uri);
                     return false;
                 }
-            }
-            return true;
-
-        } else if (tag.equals("any-of")) {
-            boolean partialSuccess = false;
-            for (XdmItem child : catalogXpc.evaluate("*", assertion)) {
-                if (testAssertion((XdmNode) child, result, assertXpc, catalogXpc, debug)) {
-                    if (wrongError != null) {
-                        partialSuccess = true;
-                        continue;
-                    }
-                    return true;
+                boolean ok = testAssertion2(subAssertion, doc, assertXpc, catalogXpc, debug);
+                if (!ok) {
+                    System.err.println("**** Assertion failed for result-document " + uri);
                 }
-            }
-            return partialSuccess;
+                return ok;
 
-        } else if (tag.equals("not")) {
-            XdmNode subAssertion = (XdmNode) catalogXpc.evaluateSingle("*", assertion);
-            return !testAssertion(subAssertion, result, assertXpc, catalogXpc, debug);
+            }
+            case "error":
+                return isException() && compareExpectedError(assertion);
+
+            case "all-of":
+                for (XdmNode child : assertion.children("*")) {
+                    if (!testAssertion((XdmNode) child, result, assertXpc, catalogXpc, debug)) {
+                        return false;
+                    }
+                }
+                return true;
+
+            case "any-of":
+                boolean partialSuccess = false;
+                for (XdmItem child : catalogXpc.evaluate("*", assertion)) {
+                    if (testAssertion((XdmNode) child, result, assertXpc, catalogXpc, debug)) {
+                        if (wrongError != null) {
+                            partialSuccess = true;
+                            continue;
+                        }
+                        return true;
+                    }
+                }
+                return partialSuccess;
+
+            case "not": {
+                XdmNode subAssertion = (XdmNode) catalogXpc.evaluateSingle("*", assertion);
+                return !testAssertion(subAssertion, result, assertXpc, catalogXpc, debug);
+            }
         }
         throw new IllegalStateException("Unknown assertion element " + tag);
     }
@@ -498,7 +504,7 @@ public class TestOutcome {
                 }
                 resultString = fsb.toString();
             }
-            String normalizeAtt = assertion.getAttributeValue(new QName("normalize-space"));
+            String normalizeAtt = assertion.attribute("normalize-space");
             if (normalizeAtt == null && driver.catalogNamespace().equals("http://www.w3.org/2012/10/xslt-test-catalog")) {
                 // default in XSLT test suite is "true"
                 normalizeAtt = "true";
@@ -519,8 +525,8 @@ public class TestOutcome {
                         if (resultString.charAt(i) != assertionString.charAt(i)) {
                             driver.println("Actual:'" + StringValue.diagnosticDisplay(resultString) + "'");
                             driver.println("Results differ at index " + i +
-                                "(\"" + StringValue.diagnosticDisplay(resultString.substring(i, (i + 10 > len ? len : i + 10))) + "\") vs (\"" +
-                                StringValue.diagnosticDisplay(assertionString.substring(i, (i + 10 > len ? len : i + 10))) + "\")");
+                                "(\"" + StringValue.diagnosticDisplay(resultString.substring(i, i + 10 > len ? len : i + 10)) + "\") vs (\"" +
+                                StringValue.diagnosticDisplay(assertionString.substring(i, i + 10 > len ? len : i + 10)) + "\")");
                             break;
                         }
                     }
@@ -534,7 +540,7 @@ public class TestOutcome {
         if (isException()) {
             return false;
         } else {
-            String method = assertion.getAttributeValue(new QName("method"));
+            String method = assertion.attribute("method");
             if (method == null) {
                 method = "xml";
             }
@@ -563,7 +569,7 @@ public class TestOutcome {
             boolean isHtml = method.equals("html") || method.equals("xhtml");
             boolean normalize = isHtml;
             if (!normalize) {
-                String normalizeAtt = assertion.getAttributeValue(new QName("normalize-space"));
+                String normalizeAtt = assertion.attribute("normalize-space");
                 normalize = normalizeAtt != null && (normalizeAtt.trim().equals("true") || normalizeAtt.trim().equals("1"));
             }
             if (normalize) {
@@ -607,12 +613,6 @@ public class TestOutcome {
         }
     }
 
-    private String resultVar;
-    public void setResultVar(String variableName) {
-        resultVar = variableName;
-    }
-
-
     private boolean assertXPath(XdmNode assertion, SingleResultDoc result, XPathCompiler assertXpc, boolean debug) throws SaxonApiException {
         if (isException()) {
             return false;
@@ -670,63 +670,19 @@ public class TestOutcome {
         }
     }
 
-
-    private boolean assertXPathOLD(XdmNode assertion, SingleResultDoc result, XPathCompiler assertXpc, boolean debug) throws SaxonApiException {
+    private boolean assertCount(XdmNode assertion, SingleResultDoc result) {
         if (isException()) {
             return false;
         } else {
-            XdmSequenceIterator iter = assertion.axisIterator(Axis.NAMESPACE);
-            while (iter.hasNext()) {
-                XdmNode namespace = (XdmNode) iter.next();
-                if (namespace.getNodeName() != null) {
-                    assertXpc.declareNamespace(namespace.getNodeName().getLocalName(), namespace.getStringValue());
-                }
+            int expected = Integer.parseInt(assertion.getStringValue());
+            int actual = result.value.size();
+            if (actual != expected) {
+                driver.println("Expected result size = " + expected + "; actual size = " + actual);
             }
-            driver.println("Testing " + assertion.getStringValue());
-            XPathExecutable exp = assertXpc.compile(assertion.getStringValue());
-            XPathSelector s = exp.load();
-            QName resultVar = new QName("result");
-            if (exp.getRequiredCardinalityForVariable(resultVar) == null) {
-                if (result.value instanceof XdmItem) { // this path used in XSLT tests
-                    s.setContextItem((XdmItem) result.value);
-                }
-            } else {
-                s.setVariable(resultVar, result.value);
-            }
-            boolean b = s.effectiveBooleanValue();
-            if (!b && debug) {
-                driver.println("XPath assertion " + assertion.getStringValue() + " failed");
-                try {
-                    String ass = assertion.getStringValue();
-                    // Try to evaluate the expression on the lhs of an "=" operator in the assertion
-                    int eq = ass.indexOf("=");
-                    if (eq > 0) {
-                        ass = ass.substring(0, eq);
-                        exp = assertXpc.compile(ass);
-                        s = exp.load();
-                        if (exp.getRequiredCardinalityForVariable(resultVar) == null) {
-                            if (result.value instanceof XdmItem) { // this path used in XSLT tests
-                                s.setContextItem((XdmItem) result.value);
-                            }
-                        } else {
-                            s.setVariable(resultVar, result.value);
-                        }
-                        XdmValue val = s.evaluate();
-                        driver.println("Actual result of " + ass + ": " + val.toString());
-                    }
-                } catch (Exception err) {
-                    // Occurs for example with an assertion like /x[a = 2] where what precedes the '=' is not an expression
-                }
-                driver.println("Actual results: " + result.value);
-            }
-            return b;
+            return actual == expected;
         }
     }
 
-    private boolean assertCount(XdmNode assertion, SingleResultDoc result) {
-        return !isException() && result.value.size() == Integer.parseInt(assertion.getStringValue());
-    }
-
     private boolean assertEmpty(XdmValue result) {
         return !isException() && result.size() == 0;
     }
@@ -735,7 +691,7 @@ public class TestOutcome {
         if (isException()) {
             return compareExpectedError(assertion);
         } else {
-            String expectedError = assertion.getAttributeValue(new QName("code"));
+            String expectedError = assertion.attribute("code");
             driver.println("Expected serialization error " + expectedError + "; got success");
             return false;
         }
@@ -762,12 +718,12 @@ public class TestOutcome {
 //        }
     }
 
-    private boolean assertSerializationMatches(XdmNode assertion, SingleResultDoc result, XPathCompiler xpath) throws SaxonApiException {
+    private boolean assertSerializationMatches(XdmNode assertion, SingleResultDoc result, XPathCompiler xpath) {
         if (isException()) {
             return false;
         } else {
             //String testSet = catalogXpc.evaluate("string(/*/@name)", assertion).toString();
-            String flagsAtt = assertion.getAttributeValue(new QName("flags"));
+            String flagsAtt = assertion.attribute("flags");
             if (flagsAtt == null) {
                 flagsAtt = "";
             }
@@ -782,15 +738,15 @@ public class TestOutcome {
                 regex = regex.substring(0, regex.length() - 1);
             }*/
             String regex = assertion.getStringValue();
-            List<String> warnings = new ArrayList<String>(1);
+            List<String> warnings = new ArrayList<>(1);
             try {
                 String principalSerializedResult = result.serialization;
                 if (principalSerializedResult == null) {
                     driver.println("No serialized result available!");
                     return false;
                 }
-                RegularExpression re = Version.platform.compileRegularExpression(
-                        xpath.getProcessor().getUnderlyingConfiguration(), regex, flagsAtt, "XP30", warnings);
+                RegularExpression re = xpath.getProcessor().getUnderlyingConfiguration().compileRegularExpression(
+                        regex, flagsAtt, "XP30", warnings);
                 if (re.containsMatch(principalSerializedResult)) {
                     return true;
                 } else {
@@ -808,11 +764,11 @@ public class TestOutcome {
         if (isException()) {
             return false;
         } else {
-            String normalizeAtt = assertion.getAttributeValue(new QName("normalize-space"));
+            String normalizeAtt = assertion.attribute("normalize-space");
             boolean normalize = normalizeAtt != null && ("true".equals(normalizeAtt.trim()) || "1".equals(normalizeAtt.trim()));
-            String ignoreAtt = assertion.getAttributeValue(new QName("ignore-prefixes"));
+            String ignoreAtt = assertion.attribute("ignore-prefixes");
             boolean ignorePrefixes = ignoreAtt != null && ("true".equals(ignoreAtt.trim()) || "1".equals(ignoreAtt.trim()));
-            String xmlVersion = assertion.getAttributeValue(new QName("xml-version"));
+            String xmlVersion = assertion.attribute("xml-version");
             boolean xml11 = "1.1".equals(xmlVersion);
 
             String comparand = catalogXpc.evaluate("if (@file) then unparsed-text(resolve-uri(@file, base-uri(.))) else string(.)", assertion).toString();
@@ -837,7 +793,7 @@ public class TestOutcome {
 
             DocumentBuilder builder = assertXpc.getProcessor().newDocumentBuilder();
             if (xml11) {
-                assertXpc.getProcessor().setConfigurationProperty(FeatureKeys.XML_VERSION, "1.1");
+                assertXpc.getProcessor().setConfigurationProperty(Feature.XML_VERSION, "1.1");
             }
             StringReader reader = new StringReader((xml11 ? "<?xml version='1.1'?>" : "") + "<z>" + comparand + "</z>");
             /*if (expectedDoctype != null) {
@@ -859,7 +815,7 @@ public class TestOutcome {
                 flag |= DeepEqual.WARNING_IF_FALSE;
             }
             try {
-                SequenceIterator iter0;
+                SequenceIterator<? extends Item> iter0;
                 if (result == null) {
                     System.err.println("Result value is null");
                     return false;
@@ -874,10 +830,10 @@ public class TestOutcome {
                 } else {
                     iter0 = value.getUnderlyingValue().iterate();
                 }
-                GroundedValue val0 = SequenceExtent.makeSequenceExtent(iter0);
-                SequenceIterator iter1 = ((NodeInfo) expected.axisIterator(Axis.CHILD).next()
-                        .getUnderlyingValue()).iterateAxis(AxisInfo.CHILD);
-                GroundedValue val1 = SequenceExtent.makeSequenceExtent(iter1);
+                GroundedValue<? extends Item> val0 = iter0.materialize();
+                SequenceIterator<? extends Item> iter1 = expected.axisIterator(Axis.CHILD).next()
+                        .getUnderlyingValue().iterateAxis(AxisInfo.CHILD);
+                GroundedValue<? extends Item> val1 = iter1.materialize();
                 boolean success = DeepEqual.deepEqual(
                     val0.iterate(), val1.iterate(),
                     new GenericAtomicComparer(CodepointCollator.getInstance(), null),
@@ -921,10 +877,10 @@ public class TestOutcome {
         }
     }
 
-    private class RemoveWhitespace implements ItemMappingFunction {
-        public NodeInfo mapItem(Item item) throws XPathException {
-            boolean isWhite = ((NodeInfo)item).getNodeKind() == Type.TEXT && Whitespace.isWhite(item.getStringValueCS());
-            return isWhite ? null : (NodeInfo) item;
+    private class RemoveWhitespace implements ItemMappingFunction<NodeInfo, NodeInfo> {
+        public NodeInfo mapItem(NodeInfo item) {
+            boolean isWhite = item.getNodeKind() == Type.TEXT && Whitespace.isWhite(item.getStringValueCS());
+            return isWhite ? null : item;
         }
     }
 
@@ -935,7 +891,7 @@ public class TestOutcome {
         } else {
             try {
                 int expectedItems = 0;
-                HashSet expected = new HashSet();
+                HashSet<AtomicMatchKey> expected = new HashSet<>();
                 XPathSelector s = assertXpc.compile("(" + assertion.getStringValue() + ")").load();
                 s.setVariable(new QName("result"), result.value); // not used, but we declared it
                 StringCollator collator = CodepointCollator.getInstance();
@@ -997,7 +953,7 @@ public class TestOutcome {
     }
 
     public boolean compareExpectedError(XdmNode assertion) {
-        String expectedError = assertion.getAttributeValue(new QName("code"));
+        String expectedError = assertion.attribute("code");
         QName expectedErrorQ;
         if (expectedError.equals("*")) {
             expectedErrorQ = null;
