@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -10,10 +10,7 @@ package net.sf.saxon.trans;
 import net.sf.saxon.Configuration;
 import net.sf.saxon.event.Builder;
 import net.sf.saxon.functions.AccessorFn;
-import net.sf.saxon.lib.FeatureKeys;
-import net.sf.saxon.lib.Initializer;
-import net.sf.saxon.lib.NamespaceConstant;
-import net.sf.saxon.lib.Validation;
+import net.sf.saxon.lib.*;
 import net.sf.saxon.s9api.*;
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.type.SchemaException;
@@ -284,16 +281,19 @@ public class CommandLineOptions {
                 try {
                     sourceInput = config.getURIResolver().resolve(value, null);
                 } catch (TransformerException e) {
-
+                    // no action - try the standard URI resolver instead
                 }
                 if (sourceInput == null) {
                     sourceInput = config.getSystemURIResolver().resolve(value, null);
                 }
                 value = sourceInput.getSystemId();
 
-            }  else {
-
-                value = (new File(value)).toURI().toASCIIString();
+            } else {
+                File catalogFile = new File(value);
+                if (!catalogFile.exists()) {
+                    throw new XPathException("Catalog file not found: " + value);
+                }
+                value = catalogFile.toURI().toASCIIString();
             }
 
             try {
@@ -307,20 +307,23 @@ public class CommandLineOptions {
         value = getOptionValue("cr");
         if (value != null) {
             Object resolver = config.getInstance(value, null);
-            processor.setConfigurationProperty(FeatureKeys.COLLECTION_URI_RESOLVER, resolver);
+            if (!(resolver instanceof CollectionURIResolver)) {
+                throw new XPathException("Class specified using -cr is not a CollectionURIResolver");
+            }
+            processor.setConfigurationProperty(Feature.COLLECTION_URI_RESOLVER, (CollectionURIResolver)resolver);
         }
 
         value = getOptionValue("dtd");
         if (value != null) {
             if ("on".equals(value)) {
-                config.setBooleanProperty(FeatureKeys.DTD_VALIDATION, true);
+                config.setBooleanProperty(Feature.DTD_VALIDATION, true);
                 config.getParseOptions().setDTDValidationMode(Validation.STRICT);
             } else if ("off".equals(value)) {
-                config.setBooleanProperty(FeatureKeys.DTD_VALIDATION, false);
+                config.setBooleanProperty(Feature.DTD_VALIDATION, false);
                 config.getParseOptions().setDTDValidationMode(Validation.SKIP);
             } else if ("recover".equals(value)) {
-                config.setBooleanProperty(FeatureKeys.DTD_VALIDATION, true);
-                config.setBooleanProperty(FeatureKeys.DTD_VALIDATION_RECOVERABLE, true);
+                config.setBooleanProperty(Feature.DTD_VALIDATION, true);
+                config.setBooleanProperty(Feature.DTD_VALIDATION_RECOVERABLE, true);
                 config.getParseOptions().setDTDValidationMode(Validation.LAX);
             }
         }
@@ -337,35 +340,39 @@ public class CommandLineOptions {
 
         value = getOptionValue("ext");
         if (value != null) {
-            config.setBooleanProperty(FeatureKeys.ALLOW_EXTERNAL_FUNCTIONS, "on".equals(value));
+            config.setBooleanProperty(Feature.ALLOW_EXTERNAL_FUNCTIONS, "on".equals(value));
         }
 
         value = getOptionValue("l");
         if (value != null) {
-            config.setBooleanProperty(FeatureKeys.LINE_NUMBERING, "on".equals(value));
+            config.setBooleanProperty(Feature.LINE_NUMBERING, "on".equals(value));
         }
 
         value = getOptionValue("m");
         if (value != null) {
-            config.setConfigurationProperty(FeatureKeys.MESSAGE_EMITTER_CLASS, value);
+            config.setConfigurationProperty(Feature.MESSAGE_EMITTER_CLASS, value);
         }
 
         value = getOptionValue("opt");
         if (value != null) {
-            config.setConfigurationProperty(FeatureKeys.OPTIMIZATION_LEVEL, value);
+            config.setConfigurationProperty(Feature.OPTIMIZATION_LEVEL, value);
         }
 
         value = getOptionValue("or");
         if (value != null) {
             Object resolver = config.getInstance(value, null);
-            config.setConfigurationProperty(FeatureKeys.OUTPUT_URI_RESOLVER, resolver);
+            if (resolver instanceof OutputURIResolver) {
+                config.setConfigurationProperty(Feature.OUTPUT_URI_RESOLVER, (OutputURIResolver) resolver);
+            } else {
+                throw new XPathException("Class " + value + " is not an OutputURIResolver");
+            }
         }
 
         value = getOptionValue("outval");
         if (value != null) {
             Boolean isRecover = "recover".equals(value);
-            config.setConfigurationProperty(FeatureKeys.VALIDATION_WARNINGS, isRecover);
-            config.setConfigurationProperty(FeatureKeys.VALIDATION_COMMENTS, isRecover);
+            config.setConfigurationProperty(Feature.VALIDATION_WARNINGS, isRecover);
+            config.setConfigurationProperty(Feature.VALIDATION_COMMENTS, isRecover);
         }
 
         value = getOptionValue("r");
@@ -375,7 +382,7 @@ public class CommandLineOptions {
 
         value = getOptionValue("strip");
         if (value != null) {
-            config.setConfigurationProperty(FeatureKeys.STRIP_WHITESPACE, value);
+            config.setConfigurationProperty(Feature.STRIP_WHITESPACE, value);
         }
 
         value = getOptionValue("T");
@@ -385,7 +392,7 @@ public class CommandLineOptions {
 
         value = getOptionValue("TJ");
         if (value != null) {
-            config.setBooleanProperty(FeatureKeys.TRACE_EXTERNAL_FUNCTIONS,
+            config.setBooleanProperty(Feature.TRACE_EXTERNAL_FUNCTIONS,
                     "on".equals(value));
         }
 
@@ -403,54 +410,54 @@ public class CommandLineOptions {
         value = getOptionValue("val");
         if (value != null) {
             if ("strict".equals(value)) {
-                processor.setConfigurationProperty(FeatureKeys.SCHEMA_VALIDATION, Validation.STRICT);
+                processor.setConfigurationProperty(Feature.SCHEMA_VALIDATION, Validation.STRICT);
             } else if ("lax".equals(value)) {
-                processor.setConfigurationProperty(FeatureKeys.SCHEMA_VALIDATION, Validation.LAX);
+                processor.setConfigurationProperty(Feature.SCHEMA_VALIDATION, Validation.LAX);
             }
         }
 
         value = getOptionValue("warnings");
         if (value != null) {
             if ("silent".equals(value)) {
-                processor.setConfigurationProperty(FeatureKeys.RECOVERY_POLICY,
+                processor.setConfigurationProperty(Feature.RECOVERY_POLICY,
                         Configuration.RECOVER_SILENTLY);
             } else if ("recover".equals(value)) {
-                processor.setConfigurationProperty(FeatureKeys.RECOVERY_POLICY,
+                processor.setConfigurationProperty(Feature.RECOVERY_POLICY,
                         Configuration.RECOVER_WITH_WARNINGS);
             } else if ("fatal".equals(value)) {
-                processor.setConfigurationProperty(FeatureKeys.RECOVERY_POLICY,
+                processor.setConfigurationProperty(Feature.RECOVERY_POLICY,
                         Configuration.DO_NOT_RECOVER);
             }
         }
 
         value = getOptionValue("x");
         if (value != null) {
-            processor.setConfigurationProperty(FeatureKeys.SOURCE_PARSER_CLASS, value);
+            processor.setConfigurationProperty(Feature.SOURCE_PARSER_CLASS, value);
         }
 
         value = getOptionValue("xi");
         if (value != null) {
-            processor.setConfigurationProperty(FeatureKeys.XINCLUDE, "on".equals(value));
+            processor.setConfigurationProperty(Feature.XINCLUDE, "on".equals(value));
         }
 
         value = getOptionValue("xmlversion");
         if (value != null) {
-            processor.setConfigurationProperty(FeatureKeys.XML_VERSION, value);
+            processor.setConfigurationProperty(Feature.XML_VERSION, value);
         }
 
         value = getOptionValue("xsdversion");
         if (value != null) {
-            processor.setConfigurationProperty(FeatureKeys.XSD_VERSION, value);
+            processor.setConfigurationProperty(Feature.XSD_VERSION, value);
         }
 
         value = getOptionValue("xsiloc");
         if (value != null) {
-            processor.setConfigurationProperty(FeatureKeys.USE_XSI_SCHEMA_LOCATION, "on".equals(value));
+            processor.setConfigurationProperty(Feature.USE_XSI_SCHEMA_LOCATION, "on".equals(value));
         }
 
         value = getOptionValue("y");
         if (value != null) {
-            processor.setConfigurationProperty(FeatureKeys.STYLE_PARSER_CLASS, value);
+            processor.setConfigurationProperty(Feature.STYLE_PARSER_CLASS, value);
         }
 
         // The init option must be done last
@@ -491,7 +498,7 @@ public class CommandLineOptions {
         System.err.println("Help for -" + option + " option");
         int prop = recognizedOptions.get(option);
         if ((prop & VALUE_PROHIBITED) == 0) {
-            switch ((prop & 0xff)) {
+            switch (prop & 0xff) {
                 case TYPE_BOOLEAN:
                     System.err.println("Value: on|off");
                     break;
@@ -563,10 +570,10 @@ public class CommandLineOptions {
         for (Enumeration e = paramFiles.propertyNames(); e.hasMoreElements(); ) {
             String name = (String) e.nextElement();
             String value = paramFiles.getProperty(name);
-            List<Source> sourceList = new ArrayList<Source>();
+            List<Source> sourceList = new ArrayList<>();
             loadDocuments(value, useURLs, processor, true, sourceList);
             if (sourceList.size() > 0) {
-                List<XdmNode> nodeList = new ArrayList<XdmNode>(sourceList.size());
+                List<XdmNode> nodeList = new ArrayList<>(sourceList.size());
                 DocumentBuilder builder = processor.newDocumentBuilder();
                 for (Source s : sourceList) {
                     nodeList.add(builder.build(s));
@@ -588,7 +595,14 @@ public class CommandLineOptions {
         }
     }
 
-    public void setSerializationParams(Serializer serializer) {
+    /**
+     * Set any output properties appearing on the command line in the form {@code !indent=yes}
+     * as properties of the supplied {@code Serializer}
+     * @param serializer the supplied {@code Serializer}, whose serialization properties
+     *                   are to be modified.
+     */
+
+    public void setSerializationProperties(Serializer serializer) {
         for (Enumeration e = serializationParams.propertyNames(); e.hasMoreElements(); ) {
             String name = (String) e.nextElement();
             String value = serializationParams.getProperty(name);
