@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2015 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -10,11 +10,19 @@ package net.sf.saxon.trans;
 
 import net.sf.saxon.Configuration;
 import net.sf.saxon.Version;
+import net.sf.saxon.event.ContentHandlerProxy;
+import net.sf.saxon.expr.Expression;
 import net.sf.saxon.expr.instruct.ResultDocument;
 import net.sf.saxon.expr.parser.ExplicitLocation;
+import net.sf.saxon.expr.parser.Token;
+import net.sf.saxon.expr.parser.XPathParser;
 import net.sf.saxon.functions.ResolveURI;
 import net.sf.saxon.lib.*;
-import net.sf.saxon.om.NamespaceResolver;
+import net.sf.saxon.om.*;
+import net.sf.saxon.sxpath.IndependentContext;
+import net.sf.saxon.trans.packages.PackageDetails;
+import net.sf.saxon.trans.packages.PackageLibrary;
+import net.sf.saxon.trans.packages.VersionedPackageName;
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.type.SchemaException;
 import org.xml.sax.*;
@@ -25,6 +33,7 @@ import javax.xml.transform.TransformerException;
 import javax.xml.transform.TransformerFactoryConfigurationError;
 import javax.xml.transform.sax.SAXSource;
 import javax.xml.transform.stream.StreamSource;
+import java.io.File;
 import java.io.IOException;
 import java.net.URI;
 import java.net.URISyntaxException;
@@ -45,6 +54,9 @@ public class ConfigurationReader implements ContentHandler, NamespaceResolver {
     private List<XPathException> errors = new ArrayList<XPathException>();
     private Locator locator;
     private Stack<List<String[]>> namespaceStack = new Stack<List<String[]>>();
+    private PackageLibrary packageLibrary;
+    private PackageDetails currentPackage;
+    private Configuration baseConfiguration;
 
     public ConfigurationReader() {
     }
@@ -60,6 +72,17 @@ public class ConfigurationReader implements ContentHandler, NamespaceResolver {
         this.classLoader = classLoader;
     }
 
+    /**
+     * Set a base Configuration to be used by the new Configuration. The new Configuration
+     * shares a NamePool and document number allocator with the base Configuration
+     *
+     * @param base the base configuration to be used
+     */
+
+    public void setBaseConfiguration(Configuration base) {
+        this.baseConfiguration = base;
+    }
+
     /**
      * Create a Configuration based on the contents of this configuration file
      *
@@ -71,7 +94,35 @@ public class ConfigurationReader implements ContentHandler, NamespaceResolver {
     public Configuration makeConfiguration(Source source) throws XPathException {
         InputSource is;
         XMLReader parser = null;
-        if (source instanceof SAXSource) {
+        if (source instanceof NodeInfo) {
+            ContentHandlerProxy proxy = new ContentHandlerProxy() {
+                @Override
+                public void startDocument(int properties) throws XPathException {
+                    try {
+                        getUnderlyingContentHandler().startDocument();
+                    } catch (SAXException e) {
+                        throw XPathException.makeXPathException(e);
+                    }
+                }
+
+                @Override
+                public void endDocument() throws XPathException {
+                    try {
+                        getUnderlyingContentHandler().endDocument();
+                    } catch (SAXException e) {
+                        throw XPathException.makeXPathException(e);
+                    }
+                }
+            };
+            proxy.setUnderlyingContentHandler(this);
+            proxy.setPipelineConfiguration(((NodeInfo) source).getConfiguration().makePipelineConfiguration());
+            proxy.open();
+            proxy.startDocument(0);
+            ((NodeInfo) source).copy(proxy, CopyOptions.ALL_NAMESPACES, ExplicitLocation.UNKNOWN_LOCATION);
+            proxy.endDocument();
+            proxy.close();
+            return config;
+        } else if (source instanceof SAXSource) {
             parser = ((SAXSource) source).getXMLReader();
             is = ((SAXSource) source).getInputSource();
         } else if (source instanceof StreamSource) {
@@ -79,7 +130,7 @@ public class ConfigurationReader implements ContentHandler, NamespaceResolver {
             is.setCharacterStream(((StreamSource) source).getReader());
             is.setByteStream(((StreamSource) source).getInputStream());
         } else {
-            throw new XPathException("Source for configuration file must be a StreamSource or SAXSource");
+            throw new XPathException("Source for configuration file must be a StreamSource or SAXSource or NodeInfo");
         }
         if (parser == null) {
             // Don't use the parser from the pool, it might be validating
@@ -131,6 +182,7 @@ public class ConfigurationReader implements ContentHandler, NamespaceResolver {
 
     public void endDocument() throws SAXException {
         namespaceStack.pop();
+        config.getDefaultXsltCompilerInfo().setPackageLibrary(packageLibrary);
     }
 
     public void startPrefixMapping(String prefix, String uri) throws SAXException {
@@ -162,6 +214,13 @@ public class ConfigurationReader implements ContentHandler, NamespaceResolver {
                     error("configuration", "edition", edition, "HE|PE|EE");
                     config = new Configuration();
                 }
+
+                if (baseConfiguration != null) {
+                    config.setNamePool(baseConfiguration.getNamePool());
+                    config.setDocumentNumberAllocator(baseConfiguration.getDocumentNumberAllocator());
+                }
+
+                packageLibrary = new PackageLibrary(config.getDefaultXsltCompilerInfo());
                 String licenseLoc = atts.getValue("licenseFileLocation");
                 if (licenseLoc != null && !edition.equals("HE")) {
                     String base = locator.getSystemId();
@@ -188,6 +247,8 @@ public class ConfigurationReader implements ContentHandler, NamespaceResolver {
                     readXQueryElement(atts);
                 } else if ("xslt".equals(localName)) {
                     readXsltElement(atts);
+                } else if ("xsltPackages".equals(localName)) {
+                    // no action until later;
                 } else if ("xsd".equals(localName)) {
                     readXsdElement(atts);
                 } else if ("resources".equals(localName)) {
@@ -221,20 +282,20 @@ public class ConfigurationReader implements ContentHandler, NamespaceResolver {
                 } else if ("xslt".equals(section)) {
                     if ("extensionElement".equals(localName)) {
                         readExtensionElement(atts);
-                    } else if ("patternOptimization".equals(localName)) {
-                        readPatternOptimization(atts);
                     } else {
                         error(localName, null, null, null);
                     }
+                } else if ("xsltPackages".equals(section)) {
+                    if ("package".equals(localName)) {
+                        readXsltPackage(atts);
+                    }
                 }
             } else if (level == 3) {
-                if ("patternOptimization".equals(subsection)) {
-                    if ("precondition".equals(localName)) {
-                        readPatternPrecondition(atts);
-                    } else if ("rewrite".equals(localName)) {
-                        readPatternRewrite(atts);
-                    } else if ("index".equals(localName)) {
-                        readPatternIndex(atts);
+                if ("package".equals(subsection)) {
+                    if ("withParam".equals(localName)) {
+                        readWithParam(atts);
+                    } else {
+                        error(localName, null, null, null);
                     }
                 }
             }
@@ -258,6 +319,7 @@ public class ConfigurationReader implements ContentHandler, NamespaceResolver {
         applyProperty(props, "allowExternalFunctions", FeatureKeys.ALLOW_EXTERNAL_FUNCTIONS);
         applyProperty(props, "allowMultiThreading", FeatureKeys.ALLOW_MULTITHREADING);
         applyProperty(props, "allowOldJavaUriFormat", FeatureKeys.ALLOW_OLD_JAVA_URI_FORMAT);
+        applyProperty(props, "allowSyntaxExtensions", FeatureKeys.ALLOW_SYNTAX_EXTENSIONS);
         applyProperty(props, "collationUriResolver", FeatureKeys.COLLATION_URI_RESOLVER_CLASS);
         applyProperty(props, "collectionUriResolver", FeatureKeys.COLLECTION_URI_RESOLVER_CLASS);
         applyProperty(props, "compileWithTracing", FeatureKeys.COMPILE_WITH_TRACING);
@@ -265,6 +327,7 @@ public class ConfigurationReader implements ContentHandler, NamespaceResolver {
         applyProperty(props, "debugByteCodeDirectory", FeatureKeys.DEBUG_BYTE_CODE_DIR);
         applyProperty(props, "defaultCollation", FeatureKeys.DEFAULT_COLLATION);
         applyProperty(props, "defaultCollection", FeatureKeys.DEFAULT_COLLECTION);
+        applyProperty(props, "defaultRegexEngine", FeatureKeys.DEFAULT_REGEX_ENGINE);
         applyProperty(props, "displayByteCode", FeatureKeys.DISPLAY_BYTE_CODE);
         applyProperty(props, "dtdValidation", FeatureKeys.DTD_VALIDATION);
         applyProperty(props, "dtdValidationRecoverable", FeatureKeys.DTD_VALIDATION_RECOVERABLE);
@@ -275,9 +338,10 @@ public class ConfigurationReader implements ContentHandler, NamespaceResolver {
         applyProperty(props, "expandAttributeDefaults", FeatureKeys.EXPAND_ATTRIBUTE_DEFAULTS);
         applyProperty(props, "generateByteCode", FeatureKeys.GENERATE_BYTE_CODE);
         applyProperty(props, "ignoreSAXSourceParser", FeatureKeys.IGNORE_SAX_SOURCE_PARSER);
-        applyProperty(props, "lazyConstructionMode", FeatureKeys.LAZY_CONSTRUCTION_MODE);
         applyProperty(props, "lineNumbering", FeatureKeys.LINE_NUMBERING);
         applyProperty(props, "markDefaultedAttributes", FeatureKeys.MARK_DEFAULTED_ATTRIBUTES);
+        applyProperty(props, "maxCompiledClasses", FeatureKeys.MAX_COMPILED_CLASSES);
+        applyProperty(props, "monitorHotSpotByteCode", FeatureKeys.MONITOR_HOT_SPOT_BYTE_CODE);
         applyProperty(props, "optimizationLevel", FeatureKeys.OPTIMIZATION_LEVEL);
         applyProperty(props, "parser", FeatureKeys.SOURCE_PARSER_CLASS);
         applyProperty(props, "preEvaluateDoc", FeatureKeys.PRE_EVALUATE_DOC_FUNCTION);
@@ -287,18 +351,22 @@ public class ConfigurationReader implements ContentHandler, NamespaceResolver {
         applyProperty(props, "serializerFactory", FeatureKeys.SERIALIZER_FACTORY_CLASS);
         applyProperty(props, "sourceResolver", FeatureKeys.SOURCE_RESOLVER_CLASS);
         applyProperty(props, "stableCollectionUri", FeatureKeys.STABLE_COLLECTION_URI);
+        applyProperty(props, "stableUnparsedText", FeatureKeys.STABLE_UNPARSED_TEXT);
         applyProperty(props, "standardErrorOutputFile", FeatureKeys.STANDARD_ERROR_OUTPUT_FILE);
         applyProperty(props, "streamability", FeatureKeys.STREAMABILITY);
         applyProperty(props, "streamingFallback", FeatureKeys.STREAMING_FALLBACK);
         applyProperty(props, "stripSpace", FeatureKeys.STRIP_WHITESPACE);
         applyProperty(props, "styleParser", FeatureKeys.STYLE_PARSER_CLASS);
         applyProperty(props, "suppressEvaluationExpiryWarning", FeatureKeys.SUPPRESS_EVALUATION_EXPIRY_WARNING);
+        applyProperty(props, "suppressXPathWarnings", FeatureKeys.SUPPRESS_XPATH_WARNINGS);
         applyProperty(props, "suppressXsltNamespaceCheck", FeatureKeys.SUPPRESS_XSLT_NAMESPACE_CHECK);
         applyProperty(props, "timing", FeatureKeys.TIMING);
         applyProperty(props, "traceExternalFunctions", FeatureKeys.TRACE_EXTERNAL_FUNCTIONS);
         applyProperty(props, "traceListener", FeatureKeys.TRACE_LISTENER_CLASS);
+        applyProperty(props, "traceListenerOutputFile", FeatureKeys.TRACE_LISTENER_OUTPUT_FILE);
         applyProperty(props, "traceOptimizerDecisions", FeatureKeys.TRACE_OPTIMIZER_DECISIONS);
         applyProperty(props, "treeModel", FeatureKeys.TREE_MODEL_NAME);
+        applyProperty(props, "unparsedTextUriResolver", FeatureKeys.UNPARSED_TEXT_URI_RESOLVER_CLASS);
         applyProperty(props, "uriResolver", FeatureKeys.URI_RESOLVER_CLASS);
         applyProperty(props, "usePiDisableOutputEscaping", FeatureKeys.USE_PI_DISABLE_OUTPUT_ESCAPING);
         applyProperty(props, "useTypedValueCache", FeatureKeys.USE_TYPED_VALUE_CACHE);
@@ -424,55 +492,6 @@ public class ConfigurationReader implements ContentHandler, NamespaceResolver {
         config.registerFileExtension(extension, mediaType);
     }
 
-
-    /**
-     * Process details of patternOptimization elements. Overridden in Saxon-PE configuration reader
-     *
-     * @param atts the attributes of the patternOptimization element in the configuration file
-     */
-
-    protected void readPatternOptimization(Attributes atts) {
-        XPathException err = new XPathException("Pattern optimization elements are not available in Saxon-HE");
-        err.setLocator(ExplicitLocation.makeFromSax(locator));
-        errors.add(err);
-    }
-
-    /**
-     * Process details of patternPrecondition elements. Overridden in Saxon-PE configuration reader
-     *
-     * @param atts the attributes of the patternPrecondition element in the configuration file
-     */
-
-    protected void readPatternPrecondition(Attributes atts) {
-        XPathException err = new XPathException("Pattern optimization elements are not available in Saxon-HE");
-        err.setLocator(ExplicitLocation.makeFromSax(locator));
-        errors.add(err);
-    }
-
-    /**
-     * Process details of patternRewrite elements. Overridden in Saxon-PE configuration reader
-     *
-     * @param atts the attributes of the patternRewrite element in the configuration file
-     */
-
-    protected void readPatternRewrite(Attributes atts) {
-        XPathException err = new XPathException("Pattern rewrite elements are not available in Saxon-HE");
-        err.setLocator(ExplicitLocation.makeFromSax(locator));
-        errors.add(err);
-    }
-
-    /**
-     * Process details of patternIndex elements. Overridden in Saxon-PE configuration reader
-     *
-     * @param atts the attributes of the patternIndex element in the configuration file
-     */
-
-    protected void readPatternIndex(Attributes atts) {
-        XPathException err = new XPathException("Pattern index elements are not available in Saxon-HE");
-        err.setLocator(ExplicitLocation.makeFromSax(locator));
-        errors.add(err);
-    }
-
     /**
      * Process details of XSLT extension elements. Overridden in Saxon-PE configuration reader
      *
@@ -485,6 +504,117 @@ public class ConfigurationReader implements ContentHandler, NamespaceResolver {
         errors.add(err);
     }
 
+    protected void readXsltPackage(Attributes atts) {
+        String name = atts.getValue("name");
+        if (name == null) {
+            String attName = "exportLocation";
+            String location = atts.getValue("exportLocation");
+            URI uri = null;
+            if (location == null) {
+                attName = "sourceLocation";
+                location = atts.getValue("sourceLocation");
+            }
+            if (location == null) {
+                error("package", attName, null, null);
+            }
+            try {
+                uri = ResolveURI.makeAbsolute(location, locator.getSystemId());
+            } catch (URISyntaxException e) {
+                error("package", attName, location, "Requires a valid URI.");
+            }
+            File file = new File(uri);
+            try {
+                packageLibrary.addPackage(file);
+            } catch (XPathException e) {
+                error(e);
+            }
+        } else {
+            String version = atts.getValue("version");
+            if (version == null) {
+                version = "1";
+            }
+            VersionedPackageName vpn = null;
+            PackageDetails details = new PackageDetails();
+            try {
+                vpn = new VersionedPackageName(name, version);
+            } catch (XPathException err) {
+                error("package", "version", version, null);
+            }
+            details.nameAndVersion = vpn;
+            currentPackage = details;
+            String sourceLoc = atts.getValue("sourceLocation");
+            StreamSource source = null;
+            if (sourceLoc != null) {
+                try {
+                    source = new StreamSource(
+                            ResolveURI.makeAbsolute(sourceLoc, locator.getSystemId()).toString());
+                } catch (URISyntaxException e) {
+                    error("package", "sourceLocation", sourceLoc, "Requires a valid URI.");
+                }
+                details.sourceLocation = source;
+            }
+            String exportLoc = atts.getValue("exportLocation");
+            if (exportLoc != null) {
+                try {
+                    source = new StreamSource(
+                            ResolveURI.makeAbsolute(exportLoc, locator.getSystemId()).toString());
+                } catch (URISyntaxException e) {
+                    error("package", "exportLocation", exportLoc, "Requires a valid URI.");
+                }
+                details.exportLocation = source;
+            }
+            String priority = atts.getValue("priority");
+            if (priority != null) {
+                try {
+                    details.priority = Integer.parseInt(priority);
+                } catch (NumberFormatException err) {
+                    error("package", "priority", priority, "Requires an integer.");
+                }
+            }
+            details.baseName = atts.getValue("base");
+            details.shortName = atts.getValue("shortName");
+
+            packageLibrary.addPackage(details);
+        }
+    }
+
+    protected void readWithParam(Attributes atts) {
+        if (currentPackage.exportLocation != null) {
+            error("withParam", null, null, "Not allowed when @exportLocation exists");
+        }
+        String name = atts.getValue("name");
+        if (name == null) {
+            error("withParam", "name", null, null);
+        }
+        QNameParser qp = new QNameParser(this);
+        qp.setAcceptEQName(true);
+        StructuredQName qName = null;
+        try {
+            qName = qp.parse(name);
+        } catch (XPathException e) {
+            error("withParam", "name", name, "Requires valid QName");
+        }
+        String select = atts.getValue("select");
+        if (select == null) {
+            error("withParam", "select", null, null);
+        }
+        IndependentContext env = new IndependentContext(config);
+        env.setNamespaceResolver(this);
+        XPathParser parser = new XPathParser();
+        GroundedValue value = null;
+        try {
+            Expression exp = parser.parse(select, 0, Token.EOF, env);
+            value = exp.iterate(env.makeEarlyEvaluationContext()).materialize();
+        } catch (XPathException e) {
+            error(e);
+        }
+        if (currentPackage.staticParams == null) {
+            currentPackage.staticParams = new HashMap<StructuredQName, GroundedValue>();
+        }
+        currentPackage.staticParams.put(qName, value);
+    }
+
+
     private void readXQueryElement(Attributes atts) {
         Properties props = new Properties();
         for (int i = 0; i < atts.getLength(); i++) {
@@ -520,18 +650,21 @@ public class ConfigurationReader implements ContentHandler, NamespaceResolver {
             }
         }
         props.put("#element", "xslt");
+        applyProperty(props, "disableXslEvaluate", FeatureKeys.DISABLE_XSL_EVALUATE);
         applyProperty(props, "enableAssertions", FeatureKeys.XSLT_ENABLE_ASSERTIONS);
         applyProperty(props, "initialMode", FeatureKeys.XSLT_INITIAL_MODE);
         applyProperty(props, "initialTemplate", FeatureKeys.XSLT_INITIAL_TEMPLATE);
         applyProperty(props, "messageEmitter", FeatureKeys.MESSAGE_EMITTER_CLASS);
         applyProperty(props, "outputUriResolver", FeatureKeys.OUTPUT_URI_RESOLVER_CLASS);
         applyProperty(props, "recoveryPolicy", FeatureKeys.RECOVERY_POLICY_NAME);
+        applyProperty(props, "resultDocumentThreads", FeatureKeys.RESULT_DOCUMENT_THREADS);
         applyProperty(props, "schemaAware", FeatureKeys.XSLT_SCHEMA_AWARE);
         applyProperty(props, "staticErrorListener", FeatureKeys.XSLT_STATIC_ERROR_LISTENER_CLASS);
         applyProperty(props, "staticUriResolver", FeatureKeys.XSLT_STATIC_URI_RESOLVER_CLASS);
+        applyProperty(props, "strictStreamability", FeatureKeys.STRICT_STREAMABILITY);
         applyProperty(props, "styleParser", FeatureKeys.STYLE_PARSER_CLASS);
         applyProperty(props, "version", FeatureKeys.XSLT_VERSION);
-        applyProperty(props, "versionWarning", FeatureKeys.VERSION_WARNING);
+        //applyProperty(props, "versionWarning", FeatureKeys.VERSION_WARNING);
     }
 
     private void readXsdElement(Attributes atts) {
@@ -549,6 +682,7 @@ public class ConfigurationReader implements ContentHandler, NamespaceResolver {
         applyProperty(props, "multipleSchemaImports", FeatureKeys.MULTIPLE_SCHEMA_IMPORTS);
         applyProperty(props, "occurrenceLimits", FeatureKeys.OCCURRENCE_LIMITS);
         applyProperty(props, "schemaUriResolver", FeatureKeys.SCHEMA_URI_RESOLVER_CLASS);
+        applyProperty(props, "thresholdForCompilingTypes", FeatureKeys.THRESHOLD_FOR_COMPILING_TYPES);
         applyProperty(props, "useXsiSchemaLocation", FeatureKeys.USE_XSI_SCHEMA_LOCATION);
         applyProperty(props, "version", FeatureKeys.XSD_VERSION);
     }
@@ -693,11 +827,15 @@ public class ConfigurationReader implements ContentHandler, NamespaceResolver {
      * Get an iterator over all the prefixes declared in this namespace context. This will include
      * the default namespace (prefix="") and the XML namespace where appropriate
      */
-
-    /*@Nullable*/
+    
     public Iterator<String> iteratePrefixes() {
-        // Not used
-        return null;
+        Set<String> prefixes = new HashSet<String>();
+        for (List<String[]> list : namespaceStack) {
+            for (String[] pair : list) {
+                prefixes.add(pair[0]);
+            }
+        }
+        return prefixes.iterator();
     }
 }
 
