@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -24,8 +24,7 @@ import net.sf.saxon.lib.TraceListener;
 import net.sf.saxon.lib.TraceListener2;
 import net.sf.saxon.om.*;
 import net.sf.saxon.trace.ExpressionPresenter;
-import net.sf.saxon.trans.rules.BuiltInRuleSet;
-import net.sf.saxon.trans.rules.Rule;
+import net.sf.saxon.trans.rules.*;
 import net.sf.saxon.tree.iter.LookaheadIterator;
 import net.sf.saxon.type.BuiltInAtomicType;
 import net.sf.saxon.type.SchemaType;
@@ -150,7 +149,7 @@ public abstract class Mode extends Actor {
      * @return the list of accumulators applicable when this is the initial mode
      */
     public Set<? extends Accumulator> getAccumulators() {
-        return accumulators == null ? Collections.<Accumulator>emptySet() : accumulators;
+        return accumulators == null ? Collections.emptySet() : accumulators;
     }
 
     /**
@@ -192,7 +191,7 @@ public abstract class Mode extends Actor {
     public abstract boolean isEmpty();
 
     /**
-     * Set the policy for handling recoverable errrors. Note that for some errors the decision can be
+     * Set the policy for handling recoverable errors. Note that for some errors the decision can be
      * made at run-time, but for the "ambiguous template match" error, the decision is (since 9.2)
      * fixed at compile time.
      *
@@ -274,20 +273,10 @@ public abstract class Mode extends Actor {
         XPathContextMajor c2 = context.newContext();
         c2.setOrigin(context.getController());   // WHY?
         c2.openStackFrame(getStackFrameSlotsNeeded());
-        return c2;
-    }
-
-    /**
-     * Expand the context to handle stack frame - this should not be called repeatedly on the same context
-     *
-     * @param context The existing XPath context
-     * @return a new XPath context
-     */
-    protected XPathContext makeNewContextIfNeeded(XPathContext context) {
-        if (getStackFrameSlotsNeeded() > 0) {
-            context = makeNewContext(context);
+        if (!(context.getCurrentComponent().getActor() instanceof Accumulator)) {
+            c2.setCurrentComponent(context.getCurrentMode()); // bug 3706
         }
-        return context;
+        return c2;
     }
 
     /**
@@ -328,11 +317,9 @@ public abstract class Mode extends Actor {
      */
 
     public Rule getRule(Item item, final int min, final int max, XPathContext context) throws XPathException {
-        RuleFilter filter = new RuleFilter() {
-            public boolean testRule(Rule r) {
-                int p = r.getPrecedence();
-                return p >= min && p <= max;
-            }
+        RuleFilter filter = r -> {
+            int p = r.getPrecedence();
+            return p >= min && p <= max;
         };
         return getRule(item, context, filter);
     }
@@ -349,11 +336,9 @@ public abstract class Mode extends Actor {
      */
 
     public Rule getNextMatchRule(Item item, final Rule currentRule, XPathContext context) throws XPathException {
-        SimpleMode.RuleFilter filter = new SimpleMode.RuleFilter() {
-            public boolean testRule(Rule r) {
-                int comp = r.compareRank(currentRule);
-                return comp < 0 || (comp == 0 && r.getSequence() < currentRule.getSequence());
-            }
+        SimpleMode.RuleFilter filter = r -> {
+            int comp = r.compareRank(currentRule);
+            return comp < 0 || (comp == 0 && r.getSequence() < currentRule.getSequence());
         };
         return getRule(item, context, filter);
     }
@@ -519,31 +504,58 @@ public abstract class Mode extends Actor {
      * @return a simple string code or "???" if the ruleset is unknown
      */
     public String getCodeForBuiltInRuleSet(BuiltInRuleSet builtInRuleSet) {
-        if (builtInRuleSet instanceof TextOnlyCopyRuleSet) {
+        if (builtInRuleSet instanceof ShallowCopyRuleSet) {
+            return "SC";
+        } else if (builtInRuleSet instanceof ShallowSkipRuleSet) {
+            return "SS";
+        } else if (builtInRuleSet instanceof DeepCopyRuleSet) {
+            return "DC";
+        } else if (builtInRuleSet instanceof DeepSkipRuleSet) {
+            return "DS";
+        } else if (builtInRuleSet instanceof FailRuleSet) {
+            return "FF";
+        } else if (builtInRuleSet instanceof TextOnlyCopyRuleSet) {
             return "TC";
+        } else if (builtInRuleSet instanceof RuleSetWithWarnings) {
+            return getCodeForBuiltInRuleSet(((RuleSetWithWarnings) builtInRuleSet).getBaseRuleSet()) + "+W";
         } else {
             return "???";
         }
     }
 
     /**
-     * Return a built-in rule set for a code string. This can be specialised in subclasses for PE/EE
+     * Return a built-in rule set for a code string. 
      *
      * @param code the code used in export
      * @return a suitable ruleset
      */
-    public BuiltInRuleSet getBuiltInRuleSetForCode(String code) throws XPathException {
-        if (code.startsWith("TC")) {
-            return TextOnlyCopyRuleSet.getInstance();
+    public BuiltInRuleSet getBuiltInRuleSetForCode(String code) {
+        BuiltInRuleSet base;
+        if (code.startsWith("SC")) {
+            base = ShallowCopyRuleSet.getInstance();
+        } else if (code.startsWith("SS")) {
+            base = ShallowSkipRuleSet.getInstance();
+        } else if (code.startsWith("DC")) {
+            base = DeepCopyRuleSet.getInstance();
+        } else if (code.startsWith("DS")) {
+            base = DeepSkipRuleSet.getInstance();
+        } else if (code.startsWith("FF")) {
+            base = FailRuleSet.getInstance();
+        } else if (code.startsWith("TC")) {
+            base = TextOnlyCopyRuleSet.getInstance();
         } else {
-            throw new AssertionError();
+            throw new IllegalArgumentException(code);
         }
+        if (code.endsWith("+W")) {
+            base = new RuleSetWithWarnings(base);
+        }
+        return base;
     }
 
     public final void export(ExpressionPresenter presenter) throws XPathException {
         int s = presenter.startElement("mode");
         if (!isUnnamedMode()) {
-            presenter.emitAttribute("name", getModeName().getEQName());
+            presenter.emitAttribute("name", getModeName());
         }
         presenter.emitAttribute("onNo", getCodeForBuiltInRuleSet(getBuiltInRuleSet()));
         String flags = "";
@@ -579,7 +591,7 @@ public abstract class Mode extends Actor {
         }
     }
 
-    protected void exportUseAccumulators(ExpressionPresenter presenter){};
+    protected void exportUseAccumulators(ExpressionPresenter presenter){}
 
     public boolean isMustBeTyped() {
         return mustBeTyped;
