@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -18,7 +18,7 @@ import net.sf.saxon.om.NodeInfo;
  * user of the class and is not enforced.)
  */
 
-public interface AxisIterator extends UnfailingIterator {
+public interface AxisIterator extends UnfailingIterator<NodeInfo> {
 
     NodeInfo next();
 
