@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,6 +7,7 @@
 
 package net.sf.saxon.tree.iter;
 
+import net.sf.saxon.om.Item;
 import net.sf.saxon.om.SequenceIterator;
 
 
@@ -16,16 +17,15 @@ import net.sf.saxon.om.SequenceIterator;
  * after the current node.
  */
 
-public interface LookaheadIterator extends SequenceIterator {
+public interface LookaheadIterator<T extends Item> extends SequenceIterator<T> {
 
     /**
      * Determine whether there are more items to come. Note that this operation
      * is stateless and it is not necessary (or usual) to call it before calling
      * next(). It is used only when there is an explicit need to tell if we
      * are at the last element.
-     * <p/>
-     * This method must not be called unless the result of getProperties() on the iterator
-     * includes the bit setting {@link SequenceIterator#LOOKAHEAD}
+     * <p>This method must not be called unless the result of getProperties() on the iterator
+     * includes the bit setting {@link SequenceIterator#LOOKAHEAD} </p>
      *
      * @return true if there are more items in the sequence
      */
