@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,10 +8,7 @@
 package net.sf.saxon.tree.iter;
 
 import net.sf.saxon.expr.LastPositionFinder;
-import net.sf.saxon.om.FocusIterator;
-import net.sf.saxon.om.GroundedValue;
-import net.sf.saxon.om.Item;
-import net.sf.saxon.om.One;
+import net.sf.saxon.om.*;
 import net.sf.saxon.trans.XPathException;
 
 
@@ -20,10 +17,10 @@ import net.sf.saxon.trans.XPathException;
  * which are set manually. Calling last() is an error. Calling next() always returns null.
  */
 
-public class ManualIterator implements FocusIterator, UnfailingIterator,
-        ReversibleIterator, LastPositionFinder, GroundedIterator, LookaheadIterator {
+public class ManualIterator<T extends Item> implements FocusIterator<T>, UnfailingIterator<T>,
+        ReversibleIterator<T>, LastPositionFinder, GroundedIterator<T>, LookaheadIterator<T> {
 
-    private Item item;
+    private T item;
     private int position;
     private LastPositionFinder lastPositionFinder;
 
@@ -44,7 +41,7 @@ public class ManualIterator implements FocusIterator, UnfailingIterator,
      * @param position the context position
      */
 
-    public ManualIterator(Item value, int position) {
+    public ManualIterator(T value, int position) {
         this.item = value;
         this.position = position;
     }
@@ -55,17 +52,13 @@ public class ManualIterator implements FocusIterator, UnfailingIterator,
      * @param value the context item
      */
 
-    public ManualIterator(Item value) {
+    public ManualIterator(T value) {
         this.item = value;
         this.position = 1;
-        this.lastPositionFinder = new LastPositionFinder() {
-            public int getLength() {
-                return 1;
-            }
-        };
+        this.lastPositionFinder = () -> 1;
     }
 
-    public void setContextItem(Item value) {
+    public void setContextItem(T value) {
         this.item = value;
     }
 
@@ -98,11 +91,11 @@ public class ManualIterator implements FocusIterator, UnfailingIterator,
         }
     }
 
-    public Item next() {
+    public T next() {
         return null;
     }
 
-    public Item current() {
+    public T current() {
         return item;
     }
 
@@ -127,8 +120,8 @@ public class ManualIterator implements FocusIterator, UnfailingIterator,
     public void close() {
     }
 
-    public ManualIterator getReverseIterator() {
-        return new ManualIterator(item);
+    public ManualIterator<T> getReverseIterator() {
+        return new ManualIterator<>(item);
     }
 
     /**
@@ -140,16 +133,16 @@ public class ManualIterator implements FocusIterator, UnfailingIterator,
      */
 
     /*@Nullable*/
-    public GroundedValue materialize() {
+    public GroundedValue<T> materialize() {
         if (item instanceof GroundedValue) {
-            return (GroundedValue) item;
+            return (GroundedValue<T>) item;
         } else {
-            return new One<Item>(item);
+            return ZeroOrOne.empty();
         }
     }
 
     @Override
-    public GroundedValue getResidue() throws XPathException {
+    public GroundedValue<T> getResidue() {
         return materialize();
     }
 
