@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,6 +7,7 @@
 
 package net.sf.saxon.tree.iter;
 
+import net.sf.saxon.om.Item;
 import net.sf.saxon.om.SequenceIterator;
 
 
@@ -16,7 +17,7 @@ import net.sf.saxon.om.SequenceIterator;
  * do so).
  */
 
-public interface ReversibleIterator extends SequenceIterator {
+public interface ReversibleIterator<T extends Item> extends SequenceIterator<T> {
 
     /**
      * Get a new SequenceIterator that returns the same items in reverse order.
@@ -26,7 +27,7 @@ public interface ReversibleIterator extends SequenceIterator {
      * @return an iterator over the items in reverse order
      */
 
-    public SequenceIterator getReverseIterator();
+    SequenceIterator<T> getReverseIterator();
 
 }
 
