@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -30,7 +30,6 @@ import java.util.*;
 /**
  * A node in the XML parse tree representing the Document itself (or equivalently, the root
  * node of the Document).
- * <p/>
  * <p>A DocumentImpl object may either represent a real document node, or it may represent an imaginary
  * container for a parentless element.</p>
  *
@@ -49,13 +48,15 @@ public final class DocumentImpl extends ParentNodeImpl implements TreeInfo, Muta
     private String baseURI;
     private HashMap<String, String[]> entityTable;
     private Set<ElementImpl> nilledElements;
-
+    private Set<ElementImpl> topWithinEntityElements;
     /*@Nullable*/ private IntHashMap<List<NodeImpl>> elementList;
     private HashMap<String, Object> userData;
     private Configuration config;
     private LineNumberMap lineNumberMap;
     private SystemIdMap systemIdMap = new SystemIdMap();
     private boolean imaginary;
+    private SpaceStrippingRule spaceStrippingRule = NoElementsSpaceStrippingRule.getInstance();
+
 
     /**
      * Create a DocumentImpl
@@ -326,6 +327,16 @@ public final class DocumentImpl extends ParentNodeImpl implements TreeInfo, Muta
         return nilledElements != null && nilledElements.contains(element);
     }
 
+    public void markTopWithinEntity(ElementImpl element) {
+        if (topWithinEntityElements == null) {
+            topWithinEntityElements = new HashSet<>();
+        }
+        topWithinEntityElements.add(element);
+    }
+
+    public boolean isTopWithinEntity(ElementImpl element) {
+        return topWithinEntityElements != null && topWithinEntityElements.contains(element);
+    }
 
     /**
      * Get the line number of this root node.
@@ -424,12 +435,12 @@ public final class DocumentImpl extends ParentNodeImpl implements TreeInfo, Muta
     /*@NotNull*/
     AxisIterator getAllElements(int fingerprint) {
         if (elementList == null) {
-            elementList = new IntHashMap<List<NodeImpl>>(500);
+            elementList = new IntHashMap<>(500);
         }
         IntHashMap<List<NodeImpl>> eList = elementList;
         List<NodeImpl> list = eList.get(fingerprint);
         if (list == null) {
-            list = new ArrayList<NodeImpl>(500);
+            list = new ArrayList<>(500);
             NodeImpl next = getNextInDocument(this);
             while (next != null) {
                 if (next.getNodeKind() == Type.ELEMENT &&
@@ -440,7 +451,7 @@ public final class DocumentImpl extends ParentNodeImpl implements TreeInfo, Muta
             }
             eList.put(fingerprint, list);
         }
-        return new ListIterator.OfNodes(list);
+        return new ListIterator.OfNodes((List)list);
     }
 
     /**
@@ -672,6 +683,33 @@ public final class DocumentImpl extends ParentNodeImpl implements TreeInfo, Muta
         elementList = null;
     }
 
+    /**
+     * Set details of space stripping action that was applied to this document during
+     * construction. This ensures that space stripping is not applied twice to the same
+     * document.
+     *
+     * @param rule details of the space stripping rules that have been applied to this
+     *             document during its construction.
+     * @since 9.9
+     */
+    public void setSpaceStrippingRule(SpaceStrippingRule rule) {
+        this.spaceStrippingRule = rule;
+    }
+
+    /**
+     * Get details of space stripping action that was applied to this document during
+     * construction. This ensures that space stripping is not applied twice to the same
+     * document.
+     *
+     * @return details of the space stripping rules that have been applied to this
+     * document during its construction. By default, returns {@link NoElementsSpaceStrippingRule},
+     * indicating that no space stripping has been applied
+     * @since 9.9
+     */
+    public SpaceStrippingRule getSpaceStrippingRule() {
+        return spaceStrippingRule;
+    }
+
     /**
      * Set user data on the document node. The user data can be retrieved subsequently
      * using {@link #getUserData}
