@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -19,6 +19,7 @@ import net.sf.saxon.tree.NamespaceNode;
 import net.sf.saxon.tree.iter.AxisIterator;
 import net.sf.saxon.tree.iter.EmptyIterator;
 import net.sf.saxon.tree.iter.SingletonIterator;
+import net.sf.saxon.tree.iter.UnfailingIterator;
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.tree.util.Navigator;
 import net.sf.saxon.tree.util.SteppingNavigator;
@@ -67,7 +68,7 @@ public abstract class NodeImpl
      * @return a singleton iterator that returns this item
      */
 
-    public SequenceIterator iterate() {
+    public UnfailingIterator iterate() {
         return SingletonIterator.makeIterator(this);
     }
 
@@ -113,21 +114,11 @@ public abstract class NodeImpl
         }
     }
 
-    /**
-     * Get the public identifier of the document entity containing this node.
-     * The default implementation returns null, meaning unknown
-     */
-
-    /*@Nullable*/
-    public String getPublicId() {
-        return null;
-    }
-
     /**
      * Get the index position of this node among its siblings (starting from 0)
      *
      * @return 0 for the first child, 1 for the second child, etc. Returns -1 for a node
-     *         that has been deleted.
+     * that has been deleted.
      */
     public final int getSiblingPosition() {
         return index;
@@ -147,8 +138,8 @@ public abstract class NodeImpl
      * Get the typed value.
      *
      * @return the typed value. If requireSingleton is set to true, the result will always be an
-     *         AtomicValue. In other cases it may be a Value representing a sequence whose items are atomic
-     *         values.
+     * AtomicValue. In other cases it may be a Value representing a sequence whose items are atomic
+     * values.
      * @since 8.5
      */
 
@@ -179,33 +170,17 @@ public abstract class NodeImpl
      * Determine whether this is the same node as another node
      *
      * @return true if this Node object and the supplied Node object represent the
-     *         same node in the tree.
+     * same node in the tree.
      */
 
-    public boolean isSameNodeInfo(NodeInfo other) {
+    public boolean equals(NodeInfo other) {
         // default implementation: differs for attribute and namespace nodes
         return this == other;
     }
 
-    /**
-     * The equals() method compares nodes for identity. It is defined to give the same result
-     * as isSameNodeInfo().
-     *
-     * @param other the node to be compared with this node
-     * @return true if this NodeInfo object and the supplied NodeInfo object represent
-     *         the same node in the tree.
-     * @since 8.7 Previously, the effect of the equals() method was not defined. Callers
-     *        should therefore be aware that third party implementations of the NodeInfo interface may
-     *        not implement the correct semantics. It is safer to use isSameNodeInfo() for this reason.
-     *        The equals() method has been defined because it is useful in contexts such as a Java Set or HashMap.
-     */
-
-    public boolean equals(Object other) {
-        return other instanceof NodeInfo && isSameNodeInfo((NodeInfo) other);
-    }
-
     /**
      * Get the name of the node. Returns null for an unnamed node
+     *
      * @return the name of the node
      */
 
@@ -215,9 +190,9 @@ public abstract class NodeImpl
 
     /**
      * Ask whether this NodeInfo implementation holds a fingerprint identifying the name of the
-     * node in the NamePool. If the answer is true, then the {link #getFingerprint} method must
-     * return the fingerprint of the node. If the answer is false, then the {link #getFingerprint}
-     * method should throw an {@link UnsupportedOperationException}. In the case of unnamed nodes
+     * node in the NamePool. If the answer is true, then the {@link #getFingerprint} method must
+     * return the fingerprint of the node. If the answer is false, then the {@link #getFingerprint}
+     * method should throw an {@code UnsupportedOperationException}. In the case of unnamed nodes
      * such as text nodes, the result can be either true (in which case getFingerprint() should
      * return -1) or false (in which case getFingerprint may throw an exception).
      *
@@ -293,7 +268,7 @@ public abstract class NodeImpl
         for (int i = 0; ; i++) {
             if (prev instanceof ParentNodeImpl) {
                 long prevseq = prev.getSequenceNumber();
-                return (prevseq == -1L ? prevseq : prevseq + 0x10000 + i);
+                return prevseq == -1L ? prevseq : prevseq + 0x10000 + i;
                 // note the 0x10000 is to leave room for namespace and attribute nodes.
             }
             assert prev != null;
@@ -308,8 +283,8 @@ public abstract class NodeImpl
      *
      * @param other The other node, whose position is to be compared with this node
      * @return -1 if this node precedes the other node, +1 if it follows the other
-     *         node, or 0 if they are the same node. (In this case, isSameNode() will always
-     *         return true, and the two nodes will produce the same result for generateId())
+     * node, or 0 if they are the same node. (In this case, isSameNode() will always
+     * return true, and the two nodes will produce the same result for generateId())
      */
 
     public final int compareOrder(/*@NotNull*/ NodeInfo other) {
@@ -320,37 +295,9 @@ public abstract class NodeImpl
         long b = ((NodeImpl) other).getSequenceNumber();
         if (a == -1L || b == -1L) {
             // Nodes added by XQuery Update do not have sequence numbers
-            return Navigator.compareOrder(this, ((NodeImpl) other));
+            return Navigator.compareOrder(this, (NodeImpl) other);
         }
-        if (a < b) {
-            return -1;
-        }
-        if (a > b) {
-            return +1;
-        }
-        return 0;
-    }
-
-    /**
-     * Determine the relative position of this node and another node, in document order,
-     * distinguishing whether the first node is a preceding, following, descendant, ancestor,
-     * or the same node as the second.
-     * <p/>
-     * The other node must always be in the same tree; the effect of calling this method
-     * when the two nodes are in different trees is undefined. If either node is a namespace
-     * or attribute node, the method should throw UnsupportedOperationException.
-     *
-     * @param other The other node, whose position is to be compared with this
-     *              node
-     * @return {@link net.sf.saxon.om.AxisInfo#PRECEDING} if this node is on the preceding axis of the other node;
-     *         {@link net.sf.saxon.om.AxisInfo#FOLLOWING} if it is on the following axis; {@link net.sf.saxon.om.AxisInfo#ANCESTOR} if the first node is an
-     *         ancestor of the second; {@link net.sf.saxon.om.AxisInfo#DESCENDANT} if the first is a descendant of the second;
-     *         {@link net.sf.saxon.om.AxisInfo#SELF} if they are the same node.
-     * @throws UnsupportedOperationException if either node is an attribute or namespace
-     * @since 9.5
-     */
-    public int comparePosition(NodeInfo other) {
-        return Navigator.comparePosition(this, other);
+        return Long.compare(a, b);
     }
 
     /**
@@ -385,7 +332,7 @@ public abstract class NodeImpl
      * prefix, or the URI of the default namespace if appropriate.
      *
      * @return The URI of the namespace of this node. For the null namespace, return an
-     *         empty string. For an unnamed node, return the empty string.
+     * empty string. For an unnamed node, return the empty string.
      */
 
     public String getURI() {
@@ -398,7 +345,7 @@ public abstract class NodeImpl
      * For unnamed nodes, it is an empty string.
      *
      * @return The display name of this node.
-     *         For a node with no name, return an empty string.
+     * For a node with no name, return an empty string.
      */
 
     public String getDisplayName() {
@@ -410,7 +357,7 @@ public abstract class NodeImpl
      * Get the local name of this node.
      *
      * @return The local name of this node.
-     *         For a node with no name, return "",.
+     * For a node with no name, return "",.
      */
 
     public String getLocalPart() {
@@ -456,7 +403,7 @@ public abstract class NodeImpl
      * document node which is not properly part of the tree.
      *
      * @return either the real parent of this node, or the "imaginary" parent present in the tree
-     *         implementation to provide a root object for the tree
+     * implementation to provide a root object for the tree
      */
 
     /*@Nullable*/
@@ -479,7 +426,7 @@ public abstract class NodeImpl
      * Get the previous sibling of the node
      *
      * @return The previous sibling node. Returns null if the current node is the first
-     *         child of its parent.
+     * child of its parent.
      */
 
     /*@Nullable*/
@@ -495,7 +442,7 @@ public abstract class NodeImpl
      * Get next sibling node
      *
      * @return The next sibling node of the required type. Returns null if the current node is the last
-     *         child of its parent.
+     * child of its parent.
      */
 
     /*@Nullable*/
@@ -649,7 +596,7 @@ public abstract class NodeImpl
      * Get the root node
      *
      * @return the NodeInfo representing the logical root of the tree. For this tree implementation the
-     *         root will either be a document node or an element node.
+     * root will either be a document node or an element node.
      */
 
     public NodeInfo getRoot() {
@@ -666,7 +613,7 @@ public abstract class NodeImpl
      * should be used only when control information held at the physical root is required
      *
      * @return the document node, which may be imaginary. In the case of a node that has been detached
-     *         from the tree by means of a delete() operation, this method returns null.
+     * from the tree by means of a delete() operation, this method returns null.
      */
 
     /*@Nullable*/
@@ -720,11 +667,11 @@ public abstract class NodeImpl
     }
 
     public NodeImpl getSuccessorElement(NodeImpl anchor, String uri, String local) {
-        NodeImpl next = getNextInDocument((NodeImpl) anchor);
+        NodeImpl next = getNextInDocument(anchor);
         while (next != null && !(next.getNodeKind() == Type.ELEMENT &&
-                (uri == null || uri.equals(next.getURI())) &&
-                (local == null || local.equals(next.getLocalPart())))) {
-            next = next.getNextInDocument((NodeImpl) anchor);
+                                         (uri == null || uri.equals(next.getURI())) &&
+                                         (local == null || local.equals(next.getLocalPart())))) {
+            next = next.getNextInDocument(anchor);
         }
         return next;
     }
@@ -743,11 +690,11 @@ public abstract class NodeImpl
         // otherwise the parent, up to the anchor element.
         // If this reaches the document root, return null.
 
-        NodeImpl prev = (NodeImpl) getPreviousSibling();
+        NodeImpl prev = getPreviousSibling();
         if (prev != null) {
             return prev.getLastDescendantOrSelf();
         }
-        return (NodeImpl) getParent();
+        return getParent();
     }
 
     /*@NotNull*/
@@ -765,92 +712,38 @@ public abstract class NodeImpl
      * @param buffer If this is non-null, and the result array fits in this buffer, then the result
      *               may overwrite the contents of this array, to avoid the cost of allocating a new array on the heap.
      * @return An array of integers representing the namespace declarations and undeclarations present on
-     *         this element. For a node other than an element, return null. Otherwise, the returned array is a
-     *         sequence of namespace codes, whose meaning may be interpreted by reference to the name pool. The
-     *         top half word of each namespace code represents the prefix, the bottom half represents the URI.
-     *         If the bottom half is zero, then this is a namespace undeclaration rather than a declaration.
-     *         The XML namespace is never included in the list. If the supplied array is larger than required,
-     *         then the first unused entry will be set to -1.
-     *         <p/>
-     *         <p>For a node other than an element, the method returns null.</p>
+     * this element. For a node other than an element, return null. Otherwise, the returned array is a
+     * sequence of namespace codes, whose meaning may be interpreted by reference to the name pool. The
+     * top half word of each namespace code represents the prefix, the bottom half represents the URI.
+     * If the bottom half is zero, then this is a namespace undeclaration rather than a declaration.
+     * The XML namespace is never included in the list. If the supplied array is larger than required,
+     * then the first unused entry will be set to -1.
+     * <p>For a node other than an element, the method returns null.</p>
      */
 
     /*@Nullable*/
     public NamespaceBinding[] getDeclaredNamespaces(NamespaceBinding[] buffer) {
         return null;
     }
-    /**
-     * Copy nodes. Copying type annotations is not yet supported for this tree
-     * structure, so we simply map the new interface onto the old
-     */
-
-//    public final void copy(Receiver out, int whichNamespaces, boolean copyAnnotations, int locationId)
-//    throws XPathException {
-//        copy(out, whichNamespaces);
-//    }
-//
-//    public abstract void copy(Receiver out, int whichNamespaces) throws XPathException;
-
-    // implement DOM Node methods
 
     /**
      * Determine whether the node has any children.
      *
      * @return <code>true</code> if the node has any children,
-     *         <code>false</code> if the node has no children.
+     * <code>false</code> if the node has no children.
      */
 
     public boolean hasChildNodes() {
         return getFirstChild() != null;
     }
 
-    /**
-     * Determine whether this node has the is-id property
-     *
-     * @return true if the node is an ID
-     */
-
-    public boolean isId() {
-        return false;
-    }
-
-    /**
-     * Determine whether this node has the is-idref property
-     *
-     * @return true if the node is an IDREF or IDREFS element or attribute
-     */
-
-    public boolean isIdref() {
-        return false;
-    }
-
-    /**
-     * Determine whether the node has the is-nilled property
-     *
-     * @return true if the node has the is-nilled property
-     */
-
-    public boolean isNilled() {
-        return false;
-    }
-
-    /**
-     * Ask whether this is a node in a streamed document
-     *
-     * @return true if the node is in a document being processed using streaming
-     */
-    @Override
-    public boolean isStreamed() {
-        return false;
-    }
-
     /**
      * Set the type annotation on a node. This must only be called when the caller has verified (by validation)
      * that the node is a valid instance of the specified type. The call is ignored if the node is not an element
      * or attribute node.
      *
      * @param type the type annotation (possibly including high bits set to indicate the isID, isIDREF, and
-     *                 isNilled properties)
+     *             isNilled properties)
      */
 
     public void setTypeAnnotation(SchemaType type) {
@@ -882,15 +775,13 @@ public abstract class NodeImpl
      */
 
     public boolean isDeleted() {
-        return (index == -1 || (parent != null && parent.isDeleted()));
+        return index == -1 || (parent != null && parent.isDeleted());
     }
 
     /**
      * Remove an attribute from this element node
-     * <p/>
      * <p>If this node is not an element, or if the specified node is not an attribute
      * of this element, this method takes no action.</p>
-     * <p/>
      * <p>The attribute object itself becomes unusable; any attempt to use this attribute object,
      * or any other object representing the same node, is likely to result in an exception.</p>
      *
@@ -903,11 +794,9 @@ public abstract class NodeImpl
 
     /**
      * Add an attribute to this element node.
-     * <p/>
      * <p>If this node is not an element, or if the supplied node is not an attribute, the method
      * takes no action. If the element already has an attribute with this name, the method
      * throws an exception.</p>
-     * <p/>
      * <p>This method does not perform any namespace fixup. It is the caller's responsibility
      * to ensure that any namespace prefix used in the name of the attribute (or in its value
      * if it has a namespace-sensitive type) is declared on this element.</p>
@@ -934,14 +823,13 @@ public abstract class NodeImpl
     }
 
 
-    public void addNamespace(NamespaceBinding nscode, boolean inherit) {
+    public void addNamespace(NamespaceBinding nscode, boolean unused) {
         // implemented for element nodes only
     }
 
     /**
      * Replace this node with a given sequence of nodes. This node is effectively deleted, and the replacement
      * nodes are attached to the parent of this node in its place.
-     * <p/>
      * <p>The supplied nodes will become children of this node's parent. Adjacent text nodes will be merged, and
      * zero-length text nodes removed. The supplied nodes may be modified in situ, for example to change their
      * parent property and to add namespace bindings, or they may be copied, at the discretion of
@@ -980,11 +868,9 @@ public abstract class NodeImpl
 
     /**
      * Insert a sequence of nodes as children of this node.
-     * <p/>
      * <p>This method takes no action unless the target node is a document node or element node. It also
      * takes no action in respect of any supplied nodes that are not elements, text nodes, comments, or
      * processing instructions.</p>
-     * <p/>
      * <p>The supplied nodes will form the new children. Adjacent text nodes will be merged, and
      * zero-length text nodes removed. The supplied nodes may be modified in situ, for example to change their
      * parent property and to add namespace bindings, or they may be copied, at the discretion of
@@ -1009,12 +895,10 @@ public abstract class NodeImpl
 
     /**
      * Insert copies of a sequence of nodes as siblings of this node.
-     * <p/>
      * <p>This method takes no action unless the target node is an element, text node, comment, or
      * processing instruction, and one that has a parent node. It also
      * takes no action in respect of any supplied nodes that are not elements, text nodes, comments, or
      * processing instructions.</p>
-     * <p/>
      * <p>The supplied nodes must use the same data model implementation as the tree into which they
      * will be inserted.</p>
      *
@@ -1028,7 +912,7 @@ public abstract class NodeImpl
         if (parent == null) {
             throw new IllegalStateException("Cannot add siblings if there is no parent");
         }
-        parent.insertChildrenAt(source, (before ? index : index + 1), inherit);
+        parent.insertChildrenAt(source, before ? index : index + 1, inherit);
     }
 
 
@@ -1045,12 +929,23 @@ public abstract class NodeImpl
      * Get a Builder suitable for building nodes that can be attached to this document.
      *
      * @return a new Builder that constructs nodes using the same object model implementation
-     *         as this one, suitable for attachment to this tree
+     * as this one, suitable for attachment to this tree
      */
 
     /*@NotNull*/
     public Builder newBuilder() {
         return getPhysicalRoot().newBuilder();
     }
+
+    /**
+     * Get the effective boolean value of this sequence
+     *
+     * @return the effective boolean value
+     * @throws XPathException if the sequence has no effective boolean value (for example a sequence of two integers)
+     */
+    @Override
+    public boolean effectiveBooleanValue() throws XPathException {
+        return true;
+    }
 }
 
