@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,16 +8,10 @@
 package net.sf.saxon.tree.tiny;
 
 import net.sf.saxon.Configuration;
-import net.sf.saxon.event.Builder;
-import net.sf.saxon.event.BuilderMonitor;
-import net.sf.saxon.event.PipelineConfiguration;
-import net.sf.saxon.event.ReceiverOptions;
+import net.sf.saxon.event.*;
 import net.sf.saxon.expr.parser.Location;
-import net.sf.saxon.lib.FeatureKeys;
-import net.sf.saxon.om.NamespaceBinding;
-import net.sf.saxon.om.NamespaceBindingSet;
-import net.sf.saxon.om.NodeName;
-import net.sf.saxon.om.StandardNames;
+import net.sf.saxon.lib.Feature;
+import net.sf.saxon.om.*;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.type.SchemaType;
@@ -47,6 +41,7 @@ public class TinyBuilder extends Builder {
     private boolean ended = false;
     private Statistics statistics = Statistics.TEMPORARY_TREE_STATISTICS;
     private boolean markDefaultedAttributes = false;
+    private int textualElementEligibilityState = 0;
 
     /**
      * Create a TinyTree builder
@@ -58,7 +53,7 @@ public class TinyBuilder extends Builder {
         super(pipe);
         Configuration config = pipe.getConfiguration();
         //markDefaultedAttributes = true;
-        markDefaultedAttributes = config.isExpandAttributeDefaults() && config.getBooleanProperty(FeatureKeys.MARK_DEFAULTED_ATTRIBUTES);
+        markDefaultedAttributes = config.isExpandAttributeDefaults() && config.getBooleanProperty(Feature.MARK_DEFAULTED_ATTRIBUTES);
         //System.err.println("TinyBuilder " + this);
     }
 
@@ -134,13 +129,6 @@ public class TinyBuilder extends Builder {
      */
 
     public void startDocument(int properties) throws XPathException {
-//        if (currentDepth == 0 && tree.numberOfNodes != 0) {
-//            System.err.println("**** FOREST DOCUMENT **** " + tree.numberOfNodes);
-//        }
-//        System.err.println("New doc: free memory: " + Runtime.getRuntime().freeMemory());
-//        if (sizeParams != null) {
-//            System.err.println("Size params: " + sizeParams[0] + ", " + sizeParams[1] + ", " + sizeParams[2] + ", " + sizeParams[3]);
-//        }
         if ((started && !ended) || currentDepth > 0) {
             // this happens when using an IdentityTransformer, or when copying a document node to form
             // the content of an element
@@ -219,15 +207,14 @@ public class TinyBuilder extends Builder {
      */
 
     public void startElement(/*@NotNull*/ NodeName elemName, SchemaType type, Location location, int properties) throws XPathException {
-//        if (currentDepth == 0 && tree.numberOfNodes != 0) {
-//            System.err.println("**** FOREST ELEMENT **** trees=" + tree.rootIndexUsed );
-//        }
 
         // if the number of siblings exceeds a certain threshold, add a parent pointer, in the form
         // of a pseudo-node
 
+        //System.err.println("startElement " + elemName);
         TinyTree tt = tree;
         assert tt != null;
+        textualElementEligibilityState = 0;
 
         if (siblingsAtDepth[currentDepth] > PARENT_POINTER_INTERVAL) {
             nodeNr = tt.addNode(Type.PARENT_POINTER, currentDepth, prevAtDepth[currentDepth - 1], 0, 0);
@@ -290,6 +277,10 @@ public class TinyBuilder extends Builder {
         if (lineNumbering) {
             tt.setLineNumber(nodeNr, location.getLineNumber(), location.getColumnNumber());
         }
+        if (location instanceof ReceivingContentHandler.LocalLocator &&
+                ((ReceivingContentHandler.LocalLocator)location).levelInEntity == 0 && currentDepth >= 1) {
+            tt.markTopWithinEntity(nodeNr);
+        }
     }
 
     public void namespace(NamespaceBindingSet namespaceBindings, int properties) throws XPathException {
@@ -299,9 +290,12 @@ public class TinyBuilder extends Builder {
         }
     }
 
+    //private static int counter = 0;
+
     public void attribute(/*@NotNull*/ NodeName attName, SimpleType type, CharSequence value, Location locationId, int properties)
             throws XPathException {
         // System.err.println("attribute " + nameCode + "=" + value);
+        //if (counter++ % 1000 == 0) System.err.println("T.addAttribute# = " + counter);
         int fp = attName.obtainFingerprint(namePool);
         String prefix = attName.getPrefix();
         int nameCode = prefix.isEmpty() ? fp : (tree.prefixPool.obtainPrefixCode(prefix) << 20) | fp;
@@ -314,7 +308,9 @@ public class TinyBuilder extends Builder {
     }
 
     public void startContent() {
+        //System.err.println("startContent");
         nodeNr++;
+        textualElementEligibilityState = 1;
     }
 
     /**
@@ -322,16 +318,34 @@ public class TinyBuilder extends Builder {
      */
 
     public void endElement() throws XPathException {
-        TinyTree tt = tree;
-        assert tt != null;
-//        System.err.println("End element");
+        assert tree != null;
+        //System.err.println("End element");
+
+        boolean eligibleAsTextualElement = textualElementEligibilityState == 2;
+        textualElementEligibilityState = 0;
         prevAtDepth[currentDepth] = -1;
         siblingsAtDepth[currentDepth] = 0;
         currentDepth--;
         if (isIDElement) {
             // we're relying on the fact that an ID element has no element children!
-            tt.indexIDElement(currentRoot, prevAtDepth[currentDepth]);
+            tree.indexIDElement(currentRoot, prevAtDepth[currentDepth]);
             isIDElement = false;
+        } else if (eligibleAsTextualElement &&
+                tree.nodeKind[nodeNr] == Type.TEXT &&
+                tree.nodeKind[nodeNr-1] == Type.ELEMENT &&
+                tree.alpha[nodeNr-1] == -1 &&
+                tree.beta[nodeNr-1] == -1) {
+            // Collapse a simple element with text content and no attributes or namespaces into a single node
+            // of type TRIVIAL_ELEMENT
+            //System.err.println("Created textual element");
+            tree.nodeKind[nodeNr-1] = Type.TEXTUAL_ELEMENT;
+            tree.alpha[nodeNr-1] = tree.alpha[nodeNr];
+            tree.beta[nodeNr-1] = tree.beta[nodeNr];
+            nodeNr--;
+            tree.numberOfNodes--;
+            if (currentDepth == 0) {
+                currentRoot = tree.getNode(nodeNr);
+            }
         }
     }
 
@@ -359,6 +373,7 @@ public class TinyBuilder extends Builder {
      */
 
     public void characters(/*@NotNull*/ CharSequence chars, Location locationId, int properties) throws XPathException {
+        //System.err.println("characters: " + chars);
         if (chars instanceof CompressedWhitespace &&
                 (properties & ReceiverOptions.WHOLE_TEXT_NODE) != 0) {
             TinyTree tt = tree;
@@ -373,12 +388,19 @@ public class TinyBuilder extends Builder {
             tt.next[nodeNr] = prevAtDepth[currentDepth - 1];   // *O* owner pointer in last sibling
             prevAtDepth[currentDepth] = nodeNr;
             siblingsAtDepth[currentDepth]++;
+            if (lineNumbering) {
+                tt.setLineNumber(nodeNr, locationId.getLineNumber(), locationId.getColumnNumber());
+            }
             return;
         }
 
         final int len = chars.length();
         if (len > 0) {
             nodeNr = makeTextNode(chars, len);
+            if (lineNumbering) {
+                tree.setLineNumber(nodeNr, locationId.getLineNumber(), locationId.getColumnNumber());
+            }
+            textualElementEligibilityState = textualElementEligibilityState == 1 ? 2 : 0;
         }
     }
 
@@ -416,13 +438,14 @@ public class TinyBuilder extends Builder {
     }
 
     /**
-     * Callback interface for SAX: not for application use<BR>
+     * Callback interface for SAX: not for application use
      */
 
     public void processingInstruction(String piname, /*@NotNull*/ CharSequence remainder, Location locationId, int properties) throws XPathException {
         TinyTree tt = tree;
         assert tt != null;
 
+        textualElementEligibilityState = 0;
         if (tt.commentBuffer == null) {
             tt.commentBuffer = new FastStringBuffer(FastStringBuffer.C256);
         }
@@ -452,10 +475,10 @@ public class TinyBuilder extends Builder {
      */
 
     public void comment(/*@NotNull*/ CharSequence chars, Location locationId, int properties) throws XPathException {
-
         TinyTree tt = tree;
         assert tt != null;
 
+        textualElementEligibilityState = 0;
         if (tt.commentBuffer == null) {
             tt.commentBuffer = new FastStringBuffer(FastStringBuffer.C256);
         }
@@ -493,6 +516,10 @@ public class TinyBuilder extends Builder {
         return new TinyBuilderMonitor(this);
     }
 
+    public boolean isPositionedAtElement() {
+        return (tree.nodeKind[tree.numberOfNodes-1]&0x0f) == Type.ELEMENT;
+    }
+
     /**
      * Copy an element node and its subtree from another TinyTree instance
      * @param source the TinyTree from which a subtree is to be copied
@@ -511,5 +538,24 @@ public class TinyBuilder extends Builder {
         siblingsAtDepth[currentDepth]++;
         //tree.diagnosticDump();
     }
+
+    /**
+     * Graft an element node and its subtree from another tree instance
+     *
+     * @param node the external node to be grafted
+     */
+
+    public void graft(NodeInfo node, boolean copyNamespaces) {
+        int newNodeNr = tree.numberOfNodes;
+        tree.graft(node, nodeNr, currentDepth, copyNamespaces);
+        int prev = prevAtDepth[currentDepth];
+        if (prev > 0) {
+            tree.next[prev] = newNodeNr;
+        }
+        tree.next[newNodeNr] = prevAtDepth[currentDepth - 1];
+        prevAtDepth[currentDepth] = newNodeNr;
+        siblingsAtDepth[currentDepth]++;
+        //tree.diagnosticDump();
+    }
 }
 
