@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -19,13 +19,14 @@ import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.tree.util.Navigator;
 import net.sf.saxon.type.SchemaType;
 import net.sf.saxon.type.Type;
+import net.sf.saxon.type.UType;
 
 
 /**
- * A node in a TinyTree representing an XML element, character content, or attribute.<P>
- * This is the top-level class in the implementation class hierarchy; it essentially contains
+ * A node in a TinyTree representing an XML element, character content, or attribute.
+ * <p>This is the top-level class in the implementation class hierarchy; it essentially contains
  * all those methods that can be defined using other primitive methods, without direct access
- * to data.
+ * to data.</p>
  *
  * @author Michael H. Kay
  */
@@ -36,6 +37,16 @@ public abstract class TinyNodeImpl implements NodeInfo {
     protected int nodeNr;
     /*@Nullable*/ protected TinyNodeImpl parent = null;
 
+    /**
+     * Get the genre of this item
+     *
+     * @return the genre
+     */
+    @Override
+    public Genre getGenre() {
+        return Genre.NODE;
+    }
+
     /**
      * Get information about the tree to which this NodeInfo belongs
      *
@@ -87,7 +98,6 @@ public abstract class TinyNodeImpl implements NodeInfo {
     /**
      * Get the type annotation of this node, if any. The type annotation is represented as
      * SchemaType object.
-     * <p/>
      * <p>Types derived from a DTD are not reflected in the result of this method.</p>
      *
      * @return For element and attribute nodes: the type annotation derived from schema
@@ -111,20 +121,10 @@ public abstract class TinyNodeImpl implements NodeInfo {
     }
 
     /**
-     * Get the public identifier of the document entity containing this node.
-     * The default implementation returns null, meaning unknown
-     */
-
-    /*@Nullable*/
-    public String getPublicId() {
-        return null;
-    }
-
-    /**
-     * Set the system id of this node. <br />
-     * This method is present to ensure that
+     * Set the system id of this node.
+     * <p>This method is present to ensure that
      * the class implements the javax.xml.transform.Source interface, so a node can
-     * be used as the source of a transformation.
+     * be used as the source of a transformation.</p>
      */
 
     public void setSystemId(String uri) {
@@ -251,64 +251,18 @@ public abstract class TinyNodeImpl implements NodeInfo {
         long a = getSequenceNumber();
         if (other instanceof TinyNodeImpl) {
             long b = ((TinyNodeImpl) other).getSequenceNumber();
-            if (a < b) {
-                return -1;
-            }
-            if (a > b) {
-                return +1;
-            }
-            return 0;
+            return Long.compare(a, b);
         } else {
-            // it must be a namespace node
+            // it must be a namespace node, or a TinyTextualElementText node
             return 0 - other.compareOrder(this);
         }
     }
 
-    /**
-     * Determine the relative position of this node and another node, in document order,
-     * distinguishing whether the first node is a preceding, following, descendant, ancestor,
-     * or the same node as the second.
-     * <p/>
-     * The other node must always be in the same tree; the effect of calling this method
-     * when the two nodes are in different trees is undefined. If either node is a namespace
-     * or attribute node, the method should throw UnsupportedOperationException.
-     *
-     * @param other The other node, whose position is to be compared with this
-     *              node
-     * @return {@link net.sf.saxon.om.AxisInfo#PRECEDING} if this node is on the preceding axis of the other node;
-     *         {@link net.sf.saxon.om.AxisInfo#FOLLOWING} if it is on the following axis; {@link net.sf.saxon.om.AxisInfo#ANCESTOR} if the first node is an
-     *         ancestor of the second; {@link net.sf.saxon.om.AxisInfo#DESCENDANT} if the first is a descendant of the second;
-     *         {@link net.sf.saxon.om.AxisInfo#SELF} if they are the same node.
-     * @throws UnsupportedOperationException if either node is an attribute or namespace
-     * @since 9.5
-     */
-    public int comparePosition(NodeInfo other) {
-        if (other instanceof TinyNodeImpl && !(other instanceof TinyAttributeImpl)) {
-            TinyNodeImpl a = this;
-            TinyNodeImpl b = (TinyNodeImpl) other;
-            if (a.isSameNodeInfo(b)) {
-                return AxisInfo.SELF;
-            }
-            if (a.isAncestorOrSelf(b)) {
-                return AxisInfo.ANCESTOR;
-            }
-            if (b.isAncestorOrSelf(a)) {
-                return AxisInfo.DESCENDANT;
-            }
-            if (a.compareOrder(b) < 0) {
-                return AxisInfo.PRECEDING;
-            }
-            return AxisInfo.FOLLOWING;
-        } else {
-            throw new UnsupportedOperationException();
-        }
-    }
-
     /**
      * Ask whether this NodeInfo implementation holds a fingerprint identifying the name of the
-     * node in the NamePool. If the answer is true, then the {link #getFingerprint} method must
-     * return the fingerprint of the node. If the answer is false, then the {link #getFingerprint}
-     * method should throw an {@link UnsupportedOperationException}. In the case of unnamed nodes
+     * node in the NamePool. If the answer is true, then the {@link #getFingerprint} method must
+     * return the fingerprint of the node. If the answer is false, then the {@link #getFingerprint}
+     * method should throw an {@code UnsupportedOperationException}. In the case of unnamed nodes
      * such as text nodes, the result can be either true (in which case getFingerprint() should
      * return -1) or false (in which case getFingerprint may throw an exception).
      *
@@ -454,7 +408,12 @@ public abstract class TinyNodeImpl implements NodeInfo {
 
             case AxisInfo.CHILD:
                 if (hasChildNodes()) {
-                    return new SiblingIterator(tree, this, nodeTest, true);
+                    if (nodeTest instanceof NameTest && ((NameTest) nodeTest).getNodeKind() == Type.ELEMENT) {
+                        // fast path for common case
+                        return new NamedChildIterator(tree, this, nodeTest.getFingerprint());
+                    } else {
+                        return new SiblingIterator(tree, this, nodeTest, true);
+                    }
                 } else {
                     return EmptyIterator.OfNodes.THE_INSTANCE;
                 }
@@ -465,21 +424,23 @@ public abstract class TinyNodeImpl implements NodeInfo {
                         nodeTest.getPrimitiveType() == Type.ELEMENT) {
                     return ((TinyDocumentImpl) this).getAllElements(nodeTest.getFingerprint());
                 } else if (hasChildNodes()) {
-                    return new DescendantIterator(tree, this, nodeTest);
+                    if (tree.externalNodes != null) {
+                        return new GraftingDescendantIterator(tree, this, nodeTest);
+                    } else if (nodeTest.getUType().overlaps(UType.TEXT)) {
+                        return new DescendantIterator(tree, this, nodeTest);
+                    } else {
+                        return new DescendantIteratorSansText(tree, this, nodeTest);
+                    }
                 } else {
                     return EmptyIterator.OfNodes.THE_INSTANCE;
                 }
 
             case AxisInfo.DESCENDANT_OR_SELF:
-                if (hasChildNodes()) {
-                    AxisIterator descendants =  new DescendantIterator(tree, this, nodeTest);
-                    if (nodeTest.matchesNode(this)) {
-                        return new PrependAxisIterator(this, descendants);
-                    } else {
-                        return descendants;
-                    }
+                AxisIterator descendants = iterateAxis(AxisInfo.DESCENDANT, nodeTest);
+                if (nodeTest.matchesNode(this)) {
+                    return new PrependAxisIterator(this, descendants);
                 } else {
-                    return Navigator.filteredSingleton(this, nodeTest);
+                    return descendants;
                 }
 
             case AxisInfo.FOLLOWING:
@@ -556,11 +517,10 @@ public abstract class TinyNodeImpl implements NodeInfo {
         }
         int p = getParentNodeNr(tree, nodeNr);
         if (p == -1) {
-            parent = null;
+            return null;
         } else {
-            parent = tree.getNode(p);
+            return parent = tree.getNode(p);
         }
-        return parent;
     }
 
     /**
