@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -17,8 +17,11 @@ import net.sf.saxon.om.*;
 import net.sf.saxon.pattern.*;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.iter.*;
+import net.sf.saxon.tree.tiny.TinyElementImpl;
 import net.sf.saxon.tree.tiny.TinyNodeImpl;
+import net.sf.saxon.tree.tiny.TinyTextualElement;
 import net.sf.saxon.tree.wrapper.SiblingCountingNode;
+import net.sf.saxon.tree.wrapper.VirtualCopy;
 import net.sf.saxon.type.*;
 
 import java.net.URI;
@@ -26,6 +29,7 @@ import java.net.URISyntaxException;
 import java.util.ArrayList;
 import java.util.LinkedList;
 import java.util.List;
+import java.util.function.Predicate;
 
 
 /**
@@ -126,6 +130,27 @@ public final class Navigator {
 
     /*@Nullable*/
     public static String getBaseURI(/*@NotNull*/ NodeInfo node) {
+        return getBaseURI(node, n -> {
+            NodeInfo parent = n.getParent();
+            return parent == null || !parent.getSystemId().equals(n.getSystemId());
+        });
+    }
+
+    /**
+     * Helper method to get the base URI of an element or processing instruction node
+     * @param node the node whose base URI is required
+     * @param isTopElementWithinEntity a predicate applied to a node that can be tested to determine
+     *                                 if the node is an element that originated as the outermost element within
+     *                                 an external entity. If this is the case, the rules for determining
+     *                                 the base URI are slightly different; in particular, any @xml:base
+     *                                 attribute on such a node is interpreted relative to the system ID of
+     *                                 the entity, not relative to the base URI of the parent node.
+     * @return the base URI of the node
+     * @since 9.9
+     */
+
+    /*@Nullable*/
+    public static String getBaseURI(/*@NotNull*/ NodeInfo node, Predicate<NodeInfo> isTopElementWithinEntity) {
         String xmlBase = node.getAttributeValue(NamespaceConstant.XML, "base");
         if (xmlBase != null) {
             URI baseURI;
@@ -145,7 +170,14 @@ public final class Navigator {
                         return null;
                     }
                     String parentSystemId = parent.getSystemId();
-                    URI base = new URI(startSystemId.equals(parentSystemId) ? parent.getBaseURI() : startSystemId);
+                    boolean isTopWithinEntity = false;
+                    if (node instanceof TinyElementImpl) {
+                        isTopWithinEntity = ((TinyElementImpl)node).getTree().isTopWithinEntity(((TinyElementImpl) node).getNodeNumber());
+                    } else {
+                        isTopWithinEntity = !startSystemId.equals(parentSystemId);
+                    }
+                    URI base = new URI(isTopElementWithinEntity.test(node) ? startSystemId : parent.getBaseURI());
+                    //URI base = new URI(parent.getBaseURI());  //bug 3530
                     baseURI = xmlBase.isEmpty() ? base : base.resolve(baseURI);
                 }
             } catch (URISyntaxException e) {
@@ -270,7 +302,7 @@ public final class Navigator {
      */
 
     public static AbsolutePath getAbsolutePath(NodeInfo node) {
-        List<AbsolutePath.PathElement> path = new LinkedList<AbsolutePath.PathElement>();
+        List<AbsolutePath.PathElement> path = new LinkedList<>();
         String sysId = node.getSystemId();
         while (node != null && node.getNodeKind() != Type.DOCUMENT) {
             path.add(0, new AbsolutePath.PathElement(node.getNodeKind(), NameOfNode.makeName(node), getNumberSimple(node, null)));
@@ -519,7 +551,7 @@ public final class Navigator {
             }
 
             if (count.matches(prev, context)) {
-                if (num == 1 && memoNode != null && prev.isSameNodeInfo(memoNode)) {
+                if (num == 1 && prev.equals(memoNode)) {
                     num = memoNumber + 1;
                     break;
                 }
@@ -566,7 +598,7 @@ public final class Navigator {
 
         //checkNumberable(node);
 
-        ArrayList<Long> v = new ArrayList<Long>(5);
+        ArrayList<Long> v = new ArrayList<>(5);
 
         if (count == null) {
             if (node.getLocalPart().isEmpty()) {    // unnamed node
@@ -743,7 +775,7 @@ public final class Navigator {
     public static int compareOrder(/*@NotNull*/ SiblingCountingNode first, /*@NotNull*/ SiblingCountingNode second) {
 
         // are they the same node?
-        if (first.isSameNodeInfo(second)) {
+        if (first.equals(second)) {
             return 0;
         }
 
@@ -760,7 +792,7 @@ public final class Navigator {
         }
 
         // do they have the same parent (common case)?
-        if (firstParent.isSameNodeInfo(secondParent)) {
+        if (firstParent.equals(secondParent)) {
             int cat1 = nodeCategories[first.getNodeKind()];
             int cat2 = nodeCategories[second.getNodeKind()];
             if (cat1 == cat2) {
@@ -789,7 +821,7 @@ public final class Navigator {
         while (depth1 > depth2) {
             p1 = p1.getParent();
             assert p1 != null;
-            if (p1.isSameNodeInfo(second)) {
+            if (p1.equals(second)) {
                 return +1;
             }
             depth1--;
@@ -799,7 +831,7 @@ public final class Navigator {
         while (depth2 > depth1) {
             p2 = p2.getParent();
             assert p2 != null;
-            if (p2.isSameNodeInfo(first)) {
+            if (p2.equals(first)) {
                 return -1;
             }
             depth2--;
@@ -812,7 +844,7 @@ public final class Navigator {
             if (par1 == null || par2 == null) {
                 throw new NullPointerException("Node order comparison - internal error");
             }
-            if (par1.isSameNodeInfo(par2)) {
+            if (par1.equals(par2)) {
                 if (p1.getNodeKind() == Type.ATTRIBUTE && p2.getNodeKind() != Type.ATTRIBUTE) {
                     return -1;  // attributes first
                 }
@@ -849,7 +881,7 @@ public final class Navigator {
         }
 
         // are they the same node?
-        if (first.isSameNodeInfo(second)) {
+        if (first.equals(second)) {
             return AxisInfo.SELF;
         }
 
@@ -866,7 +898,7 @@ public final class Navigator {
         }
 
         // do they have the same parent (common case)?
-        if (firstParent.isSameNodeInfo(secondParent)) {
+        if (firstParent.equals(secondParent)) {
             if (first.compareOrder(second) < 0) {
                 return AxisInfo.PRECEDING;
             } else {
@@ -893,7 +925,7 @@ public final class Navigator {
         while (depth1 > depth2) {
             p1 = p1.getParent();
             assert p1 != null;
-            if (p1.isSameNodeInfo(second)) {
+            if (p1.equals(second)) {
                 return AxisInfo.DESCENDANT;
             }
             depth1--;
@@ -903,7 +935,7 @@ public final class Navigator {
         while (depth2 > depth1) {
             p2 = p2.getParent();
             assert p2 != null;
-            if (p2.isSameNodeInfo(first)) {
+            if (p2.equals(first)) {
                 return AxisInfo.ANCESTOR;
             }
             depth2--;
@@ -1017,14 +1049,18 @@ public final class Navigator {
     public static boolean isAncestorOrSelf(/*@NotNull*/ NodeInfo a, /*@NotNull*/ NodeInfo d) {
         int k = a.getNodeKind();
         if (k != Type.ELEMENT && k != Type.DOCUMENT) {
-            return a.isSameNodeInfo(d);
+            return a.equals(d);
         }
         // Fast path for the TinyTree implementation
         if (a instanceof TinyNodeImpl) {
             if (d instanceof TinyNodeImpl) {
                 return ((TinyNodeImpl) a).isAncestorOrSelf((TinyNodeImpl) d);
+            } else if (d instanceof TinyTextualElement.TinyTextualElementText) {
+                return a.equals(d) || isAncestorOrSelf(a, d.getParent());
             } else if (d.getNodeKind() == Type.NAMESPACE) {
                 // fall through
+            } else if (d instanceof VirtualCopy) {
+                // fall through
             } else {
                 return false;
             }
@@ -1032,7 +1068,7 @@ public final class Navigator {
         // Generic implementation
         NodeInfo p = d;
         while (p != null) {
-            if (a.isSameNodeInfo(p)) {
+            if (a.equals(p)) {
                 return true;
             }
             p = p.getParent();
@@ -1093,7 +1129,7 @@ public final class Navigator {
      * the nodes returned by an underlying AxisIterator.
      */
 
-    public static class AxisFilter extends AxisIteratorImpl {
+    public static class AxisFilter implements AxisIterator {
         private AxisIterator base;
         private NodeTest nodeTest;
 
@@ -1132,7 +1168,7 @@ public final class Navigator {
      * nodes returned by an underlying AxisIterator.
      */
 
-    public static class EmptyTextFilter extends AxisIteratorImpl {
+    public static class EmptyTextFilter implements AxisIterator {
         private AxisIterator base;
 
         /**
@@ -1166,7 +1202,7 @@ public final class Navigator {
      * General-purpose implementation of the ancestor and ancestor-or-self axes
      */
 
-    public static final class AncestorEnumeration extends AxisIteratorImpl {
+    public static final class AncestorEnumeration implements AxisIterator {
 
         private boolean includeSelf;
         private boolean atStart;
@@ -1208,7 +1244,7 @@ public final class Navigator {
      * returned first.
      */
 
-    public static final class DescendantEnumeration extends AxisIteratorImpl {
+    public static final class DescendantEnumeration implements AxisIterator {
 
         /*@Nullable*/ private AxisIterator children = null;
         /*@Nullable*/ private AxisIterator descendants = null;
@@ -1246,10 +1282,10 @@ public final class Navigator {
                 if (n != null) {
                     if (n.hasChildNodes()) {
                         if (forwards) {
-                            descendants = new DescendantEnumeration(n, false, forwards);
+                            descendants = new DescendantEnumeration(n, false, true);
                             return n;
                         } else {
-                            descendants = new DescendantEnumeration(n, true, forwards);
+                            descendants = new DescendantEnumeration(n, true, false);
                             return next();
                         }
                     } else {
@@ -1276,7 +1312,7 @@ public final class Navigator {
                         if (children instanceof ReversibleIterator) {
                             children = (AxisIterator) ((ReversibleIterator) children).getReverseIterator();
                         } else {
-                            LinkedList<NodeInfo> list = new LinkedList<NodeInfo>();
+                            LinkedList<NodeInfo> list = new LinkedList<>();
                             AxisIterator forwards = start.iterateAxis(AxisInfo.CHILD);
                             NodeInfo n;
                             while ((n = forwards.next()) != null) {
@@ -1307,7 +1343,7 @@ public final class Navigator {
      * ancestor, child, and following-sibling axes
      */
 
-    public static final class FollowingEnumeration extends AxisIteratorImpl {
+    public static final class FollowingEnumeration implements AxisIterator {
         private NodeInfo start;
         /*@NotNull*/ private AxisIterator ancestorEnum;
         /*@Nullable*/ private AxisIterator siblingEnum = null;
@@ -1392,7 +1428,7 @@ public final class Navigator {
      * preceding-sibling axes.
      */
 
-    public static final class PrecedingEnumeration extends AxisIteratorImpl {
+    public static final class PrecedingEnumeration implements AxisIterator {
 
         private NodeInfo start;
         /*@NotNull*/ private AxisIterator ancestorEnum;
