@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -22,14 +22,14 @@ public interface SteppingNode<N extends SteppingNode> extends NodeInfo {
      * @return the parent of this node; or null if it is the root of the tree
      */
 
-    N getParent();
+    public N getParent();
 
     /**
      * Get the next sibling of this node
      *
      * @return the next sibling if there is one, or null otherwise
      */
-    N getNextSibling();
+    public N getNextSibling();
 
     /**
      * Get the previous sibling of this node
@@ -37,7 +37,7 @@ public interface SteppingNode<N extends SteppingNode> extends NodeInfo {
      * @return the previous sibling if there is one, or null otherwise
      */
 
-    N getPreviousSibling();
+    public N getPreviousSibling();
 
     /**
      * Get the first child of this node
@@ -45,7 +45,7 @@ public interface SteppingNode<N extends SteppingNode> extends NodeInfo {
      * @return the first child if there is one, or null otherwise
      */
 
-    N getFirstChild();
+    public N getFirstChild();
 
     /**
      * Find the next matching element in document order; that is, the first child element
@@ -60,7 +60,7 @@ public interface SteppingNode<N extends SteppingNode> extends NodeInfo {
      *         within the subtree being navigated
      */
 
-    N getSuccessorElement(N anchor, String uri, String local);
+    public N getSuccessorElement(N anchor, String uri, String local);
 
 }
 
