@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,9 +8,13 @@
 package net.sf.saxon.type;
 
 import net.sf.saxon.lib.ConversionRules;
+import net.sf.saxon.om.Genre;
+import net.sf.saxon.om.Item;
 import net.sf.saxon.om.StructuredQName;
 import net.sf.saxon.value.AtomicValue;
 
+import java.util.Optional;
+
 /**
  * Interface for atomic types (these are either built-in atomic types
  * or user-defined atomic types). An AtomicType is both an ItemType (a possible type
@@ -19,6 +23,16 @@ import net.sf.saxon.value.AtomicValue;
  */
 public interface AtomicType extends SimpleType, PlainType {
 
+    /**
+     * Determine the Genre (top-level classification) of this type
+     *
+     * @return the Genre to which this type belongs, specifically {@link Genre#ATOMIC}
+     */
+    @Override
+    default Genre getGenre() {
+        return Genre.ATOMIC;
+    }
+
     /**
      * Validate that a primitive atomic value is a valid instance of a type derived from the
      * same primitive type.
@@ -32,7 +46,7 @@ public interface AtomicType extends SimpleType, PlainType {
      * @throws UnsupportedOperationException in the case of an external object type
      */
 
-    public ValidationFailure validate(AtomicValue primValue, CharSequence lexicalValue, ConversionRules rules);
+    ValidationFailure validate(AtomicValue primValue, CharSequence lexicalValue, ConversionRules rules);
 
     /**
      * Determine whether the atomic type is ordered, that is, whether less-than and greater-than comparisons
@@ -44,7 +58,7 @@ public interface AtomicType extends SimpleType, PlainType {
      * @return true if ordering operations are permitted
      */
 
-    public boolean isOrdered(boolean optimistic);
+    boolean isOrdered(boolean optimistic);
 
     /**
      * Determine whether the type is abstract, that is, whether it cannot have instances that are not also
@@ -52,7 +66,7 @@ public interface AtomicType extends SimpleType, PlainType {
      * @return true if the type is abstract
      */
 
-    public boolean isAbstract();
+    boolean isAbstract();
 
     /**
      * Determine whether the atomic type is a primitive type.  The primitive types are
@@ -62,7 +76,7 @@ public interface AtomicType extends SimpleType, PlainType {
      * @return true if the type is considered primitive under the above rules
      */
 
-    public boolean isPrimitiveType();
+    boolean isPrimitiveType();
 
     /**
      * Determine whether the atomic type is a built-in type. The built-in atomic types are the 41 atomic types
@@ -70,7 +84,7 @@ public interface AtomicType extends SimpleType, PlainType {
      * xs:untypedAtomic, and all supertypes of these (xs:anyAtomicType, xs:numeric, ...)
      */
 
-    public boolean isBuiltInType();
+    boolean isBuiltInType();
 
     /**
      * Get the name of this type as a StructuredQName, unless the type is anonymous, in which case
@@ -79,7 +93,7 @@ public interface AtomicType extends SimpleType, PlainType {
      * @return the name of the atomic type, or null if the type is anonymous.
      */
 
-    public StructuredQName getTypeName();
+    StructuredQName getTypeName();
 
 
     /**
@@ -89,6 +103,24 @@ public interface AtomicType extends SimpleType, PlainType {
      * @return a StringConverter to do the conversion, or null if no built-in converter is available.
      */
 
-    public StringConverter getStringConverter(ConversionRules rules);
+    StringConverter<? extends AtomicValue> getStringConverter(ConversionRules rules);
+
+    /**
+     * Get extra diagnostic information about why a supplied item does not conform to this
+     * item type, if available. If extra information is returned, it should be in the form of a complete
+     * sentence, minus the closing full stop. No information should be returned for obvious cases.
+     *
+     * @param item the item that doesn't match this type
+     * @param th   the type hierarchy cache
+     * @return optionally, a message explaining why the item does not match the type
+     */
+    @Override
+    default Optional<String> explainMismatch(Item item, TypeHierarchy th) {
+        if (item instanceof AtomicValue) {
+            return Optional.of("The supplied value is of type " + ((AtomicValue) item).getItemType());
+        } else {
+            return Optional.of("The supplied value is " + item.getGenre().getDescription());
+        }
+    }
 }
 
