@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -21,6 +21,8 @@ import net.sf.saxon.value.*;
 import java.util.Collections;
 import java.util.Set;
 
+import static net.sf.saxon.type.SchemaComponent.ValidationStatus.VALIDATED;
+
 /**
  * This class represents a built-in atomic type, which may be either a primitive type
  * (such as xs:decimal or xs:anyURI) or a derived type (such as xs:ID or xs:dayTimeDuration).
@@ -33,7 +35,7 @@ public class BuiltInAtomicType implements AtomicType, ItemType.WithSequenceTypeC
     private int primitiveFingerprint;
     private UType uType;
     private boolean ordered = false;
-    public StringConverter stringConverter; // may be null for types where conversion rules can vary
+    public StringConverter<? extends AtomicValue> stringConverter; // may be null for types where conversion rules can vary
     private SequenceType _one;
     private SequenceType _oneOrMore;
     private SequenceType _zeroOrOne;
@@ -177,12 +179,56 @@ public class BuiltInAtomicType implements AtomicType, ItemType.WithSequenceTypeC
     public final static BuiltInAtomicType DATE_TIME_STAMP =
             makeAtomicType(StandardNames.XS_DATE_TIME_STAMP, DATE_TIME, true);
 
+    static {
+        // See bug 2524
+        ANY_ATOMIC          .stringConverter = StringConverter.StringToString.INSTANCE;
+        STRING              .stringConverter = StringConverter.StringToString.INSTANCE;
+        LANGUAGE            .stringConverter = StringConverter.StringToLanguage.INSTANCE;
+        NORMALIZED_STRING   .stringConverter = StringConverter.StringToNormalizedString.INSTANCE;
+        TOKEN               .stringConverter = StringConverter.StringToToken.INSTANCE;
+        NCNAME              .stringConverter = StringConverter.StringToNCName.TO_NCNAME;
+        NAME                .stringConverter = StringConverter.StringToName.INSTANCE;
+        NMTOKEN             .stringConverter = StringConverter.StringToNMTOKEN.INSTANCE;
+        ID                  .stringConverter = StringConverter.StringToNCName.TO_ID;
+        IDREF               .stringConverter = StringConverter.StringToNCName.TO_IDREF;
+        ENTITY              .stringConverter = StringConverter.StringToNCName.TO_ENTITY;
+        DECIMAL             .stringConverter = StringConverter.StringToDecimal.INSTANCE;
+        INTEGER             .stringConverter = StringConverter.StringToInteger.INSTANCE;
+        DURATION            .stringConverter = StringConverter.StringToDuration.INSTANCE;
+        G_MONTH             .stringConverter = StringConverter.StringToGMonth.INSTANCE;
+        G_MONTH_DAY         .stringConverter = StringConverter.StringToGMonthDay.INSTANCE;
+        G_DAY               .stringConverter = StringConverter.StringToGDay.INSTANCE;
+        DAY_TIME_DURATION   .stringConverter = StringConverter.StringToDayTimeDuration.INSTANCE;
+        YEAR_MONTH_DURATION .stringConverter = StringConverter.StringToYearMonthDuration.INSTANCE;
+        TIME                .stringConverter = StringConverter.StringToTime.INSTANCE;
+        BOOLEAN             .stringConverter = StringConverter.StringToBoolean.INSTANCE;
+        HEX_BINARY          .stringConverter = StringConverter.StringToHexBinary.INSTANCE;
+        BASE64_BINARY       .stringConverter = StringConverter.StringToBase64Binary.INSTANCE;
+        UNTYPED_ATOMIC      .stringConverter = StringConverter.StringToUntypedAtomic.INSTANCE;
 
+        NON_POSITIVE_INTEGER.stringConverter = new StringConverter.StringToIntegerSubtype(NON_POSITIVE_INTEGER);
+        NEGATIVE_INTEGER    .stringConverter = new StringConverter.StringToIntegerSubtype(NEGATIVE_INTEGER);
+        LONG                .stringConverter = new StringConverter.StringToIntegerSubtype(LONG);
+        INT                 .stringConverter = new StringConverter.StringToIntegerSubtype(INT);
+        SHORT               .stringConverter = new StringConverter.StringToIntegerSubtype(SHORT);
+        BYTE                .stringConverter = new StringConverter.StringToIntegerSubtype(BYTE);
+        NON_NEGATIVE_INTEGER.stringConverter = new StringConverter.StringToIntegerSubtype(NON_NEGATIVE_INTEGER);
+        POSITIVE_INTEGER    .stringConverter = new StringConverter.StringToIntegerSubtype(POSITIVE_INTEGER);
+        UNSIGNED_LONG       .stringConverter = new StringConverter.StringToIntegerSubtype(UNSIGNED_LONG);
+        UNSIGNED_INT        .stringConverter = new StringConverter.StringToIntegerSubtype(UNSIGNED_INT);
+        UNSIGNED_SHORT      .stringConverter = new StringConverter.StringToIntegerSubtype(UNSIGNED_SHORT);
+        UNSIGNED_BYTE       .stringConverter = new StringConverter.StringToIntegerSubtype(UNSIGNED_BYTE);
+
+        // We were getting an IntelliJ warning here about potential class loading deadlock. See bug #2524. Have moved the
+        // static initializers here, and removed the dependency on static initialization in StringConverter, which hopefully
+        // solves the problem.
+    }
 
     private BuiltInAtomicType(int fingerprint) {
         this.fingerprint = fingerprint;
     }
 
+
     /**
      * Get the local name of this type
      *
@@ -191,11 +237,7 @@ public class BuiltInAtomicType implements AtomicType, ItemType.WithSequenceTypeC
      */
 
     public String getName() {
-        if (fingerprint == StandardNames.XS_NUMERIC) {
-            return "numeric";
-        } else {
-            return StandardNames.getLocalName(fingerprint);
-        }
+        return StandardNames.getLocalName(fingerprint);
     }
 
     /**
@@ -358,7 +400,7 @@ public class BuiltInAtomicType implements AtomicType, ItemType.WithSequenceTypeC
      */
 
     public boolean isOrdered(boolean optimistic) {
-        return ordered || (optimistic && (this == BuiltInAtomicType.DURATION || this == BuiltInAtomicType.ANY_ATOMIC));
+        return ordered || (optimistic && (this == DURATION || this == ANY_ATOMIC));
     }
 
 
@@ -395,7 +437,7 @@ public class BuiltInAtomicType implements AtomicType, ItemType.WithSequenceTypeC
     /**
      * Get the validation status - always valid
      */
-    public final int getValidationStatus() {
+    public final ValidationStatus getValidationStatus() {
         return VALIDATED;
     }
 
@@ -482,11 +524,7 @@ public class BuiltInAtomicType implements AtomicType, ItemType.WithSequenceTypeC
      */
 
     public String getDisplayName() {
-        if (fingerprint == StandardNames.XS_NUMERIC) {
-            return "numeric";
-        } else {
-            return StandardNames.getDisplayName(fingerprint);
-        }
+        return StandardNames.getDisplayName(fingerprint);
     }
 
 
@@ -556,7 +594,7 @@ public class BuiltInAtomicType implements AtomicType, ItemType.WithSequenceTypeC
      *
      *
      * @param item    The item to be tested
-     * @param th
+     * @param th      The type hierarchy cache
      * @return true if the item is an instance of this type; false otherwise
      */
     public boolean matches(Item item, TypeHierarchy th) {
@@ -578,6 +616,7 @@ public class BuiltInAtomicType implements AtomicType, ItemType.WithSequenceTypeC
             return this;
         } else {
             ItemType s = (ItemType) getBaseType();
+            assert s != null;
             if (s.isPlainType()) {
                 return (BuiltInAtomicType)s.getPrimitiveItemType();
             } else {
@@ -613,20 +652,6 @@ public class BuiltInAtomicType implements AtomicType, ItemType.WithSequenceTypeC
         return getDisplayName();
     }
 
-    /**
-     * Return a string representation of this SequenceType suitable for use in export (SEF) files.
-     * The difference from the toString() method is that the representation will not contain
-     * references to anonymous types.
-     *
-     * @return the string representation as an instance of the XPath
-     * SequenceType construct, using only names that will be in-scope if the target environment
-     * imports the same schema as the source environment
-     */
-    @Override
-    public String toExportString() {
-        return toString();
-    }
-
     /**
      * Get the item type of the atomic values that will be produced when an item
      * of this type is atomized
@@ -642,9 +667,10 @@ public class BuiltInAtomicType implements AtomicType, ItemType.WithSequenceTypeC
      *
      * @return true unless it is known that these items will be elements with element-only
      *         content, in which case return false
+     * @param th The type hierarchy cache
      */
 
-    public boolean isAtomizable() {
+    public boolean isAtomizable(TypeHierarchy th) {
         return true;
     }
 
@@ -693,14 +719,14 @@ public class BuiltInAtomicType implements AtomicType, ItemType.WithSequenceTypeC
         } else {
             SchemaType base = getBaseType();
             if (base == null) {
-                throw new SchemaException("Type " + getDescription() +
-                        " is not validly derived from type " + type.getDescription());
+                throw new SchemaException("The type " + getDescription() +
+                        " is not validly derived from the type " + type.getDescription());
             }
             try {
                 base.checkTypeDerivationIsOK(type, block);
             } catch (SchemaException se) {
-                throw new SchemaException("Type " + getDescription() +
-                        " is not validly derived from type " + type.getDescription());
+                throw new SchemaException("The type " + getDescription() +
+                        " is not validly derived from the type " + type.getDescription());
             }
         }
     }
@@ -801,7 +827,8 @@ public class BuiltInAtomicType implements AtomicType, ItemType.WithSequenceTypeC
 
     /**
      * Test whether this simple type is namespace-sensitive, that is, whether
-     * it is derived from xs:QName or xs:NOTATION
+     * it is derived from xs:QName or xs:NOTATION.  Note that
+     * the result for xs:anyAtomicType is false, even though an instance might be a QName.
      *
      * @return true if this type is derived from xs:QName or xs:NOTATION
      */
@@ -811,6 +838,7 @@ public class BuiltInAtomicType implements AtomicType, ItemType.WithSequenceTypeC
         int fp = base.getFingerprint();
         while (fp > 1023) {
             base = (BuiltInAtomicType) base.getBaseType();
+            assert base != null;
             fp = base.getFingerprint();
         }
 
@@ -879,7 +907,7 @@ public class BuiltInAtomicType implements AtomicType, ItemType.WithSequenceTypeC
      */
  
     
-    public StringConverter getStringConverter(ConversionRules rules) {
+    public StringConverter<? extends AtomicValue> getStringConverter(ConversionRules rules) {
         if (stringConverter != null) {
             return stringConverter;
         }
@@ -906,7 +934,7 @@ public class BuiltInAtomicType implements AtomicType, ItemType.WithSequenceTypeC
             case StandardNames.XS_NOTATION:
                 return new StringConverter.StringToNotation(rules);
             default:
-                throw new AssertionError("No string converter available for " + toString());
+                throw new AssertionError("No string converter available for " + this);
         }
     }
 
@@ -931,7 +959,7 @@ public class BuiltInAtomicType implements AtomicType, ItemType.WithSequenceTypeC
         }
         StringConverter converter = stringConverter;
         if (converter == null) {
-            converter = getStringConverter(node.getTreeInfo().getConfiguration().getConversionRules());
+            converter = getStringConverter(node.getConfiguration().getConversionRules());
             if (isNamespaceSensitive()) {
                 converter = (StringConverter) converter.setNamespaceResolver(new InscopeNamespaceResolver(node));
             }
@@ -1159,20 +1187,10 @@ public class BuiltInAtomicType implements AtomicType, ItemType.WithSequenceTypeC
      * @return the value after postprocessing
      */
 
-    public CharSequence postprocess(CharSequence input) throws ValidationException {
+    public CharSequence postprocess(CharSequence input) {
         return input;
     }
 
-    /**
-     * Visit all the schema components used in this ItemType definition
-     *
-     * @param visitor the visitor class to be called when each component is visited
-     */
-
-    public void visitNamedSchemaComponents(SchemaComponentVisitor visitor) throws XPathException {
-        // no action
-    }
-
     /**
      * Get the list of plain types that are subsumed by this type
      *
@@ -1191,9 +1209,9 @@ public class BuiltInAtomicType implements AtomicType, ItemType.WithSequenceTypeC
 
     public boolean isNumericType() {
         ItemType p = getPrimitiveItemType();
-        return p == NumericType.getInstance() || p == BuiltInAtomicType.DECIMAL ||
-                p == BuiltInAtomicType.DOUBLE || p == BuiltInAtomicType.FLOAT ||
-                p == BuiltInAtomicType.INTEGER;
+        return p == NumericType.getInstance() || p == DECIMAL ||
+                p == DOUBLE || p == FLOAT ||
+                p == INTEGER;
     }
 
     /**
@@ -1203,13 +1221,11 @@ public class BuiltInAtomicType implements AtomicType, ItemType.WithSequenceTypeC
      * @return a Javascript instruction or sequence of instructions, which can be used as the body
      * of a Javascript function, and which returns a boolean indication whether the value of the
      * variable "item" is an instance of this item type.
-     * @throws XPathException if JS code cannot be generated for this item type, for example because
-     *                        the test is schema-aware.
      *
      */
     @Override
-    public String generateJavaScriptItemTypeTest(ItemType knownToBe, int targetVersion) throws XPathException {
-        if (this == BuiltInAtomicType.NOTATION) {
+    public String generateJavaScriptItemTypeTest(ItemType knownToBe, int targetVersion) {
+        if (this == NOTATION) {
             return "return false;";
         }
         if (targetVersion == 1) {
@@ -1232,7 +1248,7 @@ public class BuiltInAtomicType implements AtomicType, ItemType.WithSequenceTypeC
      * an error if conversion is not possible. The variable "val" will hold the supplied Javascript
      * value.
      */
-    public String generateJavaScriptItemTypeAcceptor(String errorCode, int targetVersion) throws XPathException {
+    public String generateJavaScriptItemTypeAcceptor(String errorCode, int targetVersion) {
         if (targetVersion == 1) {
             return "return SaxonJS.U.Atomic." + getName() + ".cast(val);";
         } else if (targetVersion == 2) {
