@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,7 +9,6 @@ package net.sf.saxon.type;
 
 import net.sf.saxon.lib.ConversionRules;
 import net.sf.saxon.om.NamespaceResolver;
-import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.value.*;
 
 import java.math.BigDecimal;
@@ -17,83 +16,19 @@ import java.math.BigDecimal;
 /**
  * A converter implements conversions from one atomic type to another - that is, it implements the casting
  * rules for a (source type, destination type) pair.
- * <p/>
  * <p>There is potentially one Converter implementation for each pair of (source, target) classes; though in many
  * cases the same implementation handles a number of such pairs.</p>
- * <p/>
  * <p>In cases where the conversion rules are fixed (specifically, where they do not depend on differences between
  * versions of the XSD or QT specifications), the appropriate Converter can be obtained as a static constant, for example
- * {@link #BOOLEAN_TO_DOUBLE}. In other cases the converter is paramaterized by the {@link ConversionRules} object,
+ * {@link BooleanToDouble#INSTANCE}. In other cases the converter is parameterized by the {@link ConversionRules} object,
  * and should be obtained by calling the appropriate factory method on the ConversionRules.</p>
- * <p/>
  * <p>Where the source type of the conversion is xs:string, the converter will always be a subclass of
  * {@link StringConverter}</p>
+ *
+ * @param <F> the source type
+ * @param <T> the target type
  */
-public abstract class Converter {
-    public final static StringConverter.IdentityConverter
-            IDENTITY_CONVERTER = new StringConverter.IdentityConverter();
-    public final static ToUntypedAtomicConverter
-            TO_UNTYPED_ATOMIC = new ToUntypedAtomicConverter();
-    public final static ToStringConverter
-            TO_STRING = new ToStringConverter();
-    public final static NumericToFloat
-            NUMERIC_TO_FLOAT = new NumericToFloat();
-    public final static BooleanToFloat
-            BOOLEAN_TO_FLOAT = new BooleanToFloat();
-    public final static NumericToDouble
-            NUMERIC_TO_DOUBLE = new NumericToDouble();
-    public final static BooleanToDouble
-            BOOLEAN_TO_DOUBLE = new BooleanToDouble();
-    public final static DoubleToDecimal
-            DOUBLE_TO_DECIMAL = new DoubleToDecimal();
-    public final static FloatToDecimal
-            FLOAT_TO_DECIMAL = new FloatToDecimal();
-    public final static IntegerToDecimal
-            INTEGER_TO_DECIMAL = new IntegerToDecimal();
-    public final static NumericToDecimal
-            NUMERIC_TO_DECIMAL = new NumericToDecimal();
-    public final static BooleanToDecimal
-            BOOLEAN_TO_DECIMAL = new BooleanToDecimal();
-    public final static DoubleToInteger
-            DOUBLE_TO_INTEGER = new DoubleToInteger();
-    public final static FloatToInteger
-            FLOAT_TO_INTEGER = new FloatToInteger();
-    public final static DecimalToInteger
-            DECIMAL_TO_INTEGER = new DecimalToInteger();
-    public final static NumericToInteger
-            NUMERIC_TO_INTEGER = new NumericToInteger();
-    public final static BooleanToInteger
-            BOOLEAN_TO_INTEGER = new BooleanToInteger();
-    public final static DurationToDayTimeDuration
-            DURATION_TO_DAY_TIME_DURATION = new DurationToDayTimeDuration();
-    public final static DurationToYearMonthDuration
-            DURATION_TO_YEAR_MONTH_DURATION = new DurationToYearMonthDuration();
-    public final static DateToDateTime
-            DATE_TO_DATE_TIME = new DateToDateTime();
-    public final static DateTimeToDate
-            DATE_TIME_TO_DATE = new DateTimeToDate();
-    public final static DateTimeToGMonth
-            DATE_TIME_TO_G_MONTH = new DateTimeToGMonth();
-    public final static DateTimeToGYearMonth
-            DATE_TIME_TO_G_YEAR_MONTH = new DateTimeToGYearMonth();
-    public final static DateTimeToGYear
-            DATE_TIME_TO_G_YEAR = new DateTimeToGYear();
-    public final static DateTimeToGMonthDay
-            DATE_TIME_TO_G_MONTH_DAY = new DateTimeToGMonthDay();
-    public final static DateTimeToGDay
-            DATE_TIME_TO_G_DAY = new DateTimeToGDay();
-    public final static DateTimeToTime
-            DATE_TIME_TO_TIME = new DateTimeToTime();
-    public final static NumericToBoolean
-            NUMERIC_TO_BOOLEAN = new NumericToBoolean();
-    public final static Base64BinaryToHexBinary
-            BASE64_BINARY_TO_HEX_BINARY = new Base64BinaryToHexBinary();
-    public final static HexBinaryToBase64Binary
-            HEX_BINARY_TO_BASE64_BINARY = new HexBinaryToBase64Binary();
-    public final static NotationToQName
-            NOTATION_TO_QNAME = new NotationToQName();
-    public final static QNameToNotation
-            QNAME_TO_NOTATION = new QNameToNotation();
+public abstract class Converter<F extends AtomicValue, T extends AtomicValue> {
 
 
     /**
@@ -109,7 +44,8 @@ public abstract class Converter {
 
     public static AtomicValue convert(AtomicValue value, AtomicType targetType, ConversionRules rules)
             throws ValidationException {
-        Converter converter = rules.getConverter(value.getPrimitiveType(), targetType);
+        Converter<? super AtomicValue, ? extends AtomicValue> converter =
+                rules.getConverter(value.getPrimitiveType(), targetType);
         if (converter == null) {
             ValidationFailure ve = new ValidationFailure("Cannot convert value from " + value.getPrimitiveType() + " to " + targetType);
             ve.setErrorCode("FORG0001");
@@ -149,7 +85,7 @@ public abstract class Converter {
      */
 
     /*@NotNull*/
-    public abstract ConversionResult convert(/*@NotNull*/ AtomicValue input);
+    public abstract ConversionResult convert(F input);
 
     /**
      * Set the conversion rules to be used by this Converter
@@ -191,7 +127,7 @@ public abstract class Converter {
      * unchanged (see bug 2754)
      */
 
-    public Converter setNamespaceResolver(NamespaceResolver resolver) {
+    public Converter<F, T> setNamespaceResolver(NamespaceResolver resolver) {
         return this;
         // no action
     }
@@ -207,27 +143,57 @@ public abstract class Converter {
         return null;
     }
 
+    /**
+     * Specialisation for converters that always succeed
+     * @param <F> the kind of atomic value supplied as input
+     * @param <T> the kind of atomic value returned as output
+     */
+
+    public static abstract class UnfailingConverter<F extends AtomicValue, T extends AtomicValue> extends Converter<F, T> {
+        @Override
+        public abstract T convert(F input);
+
+        @Override
+        public final boolean isAlwaysSuccessful() {
+            return true;
+        }
+    }
+
+    /**
+     * Converter that does nothing - it returns the input unchanged
+     */
+
+    public static class IdentityConverter extends Converter<AtomicValue, AtomicValue> {
+        public final static IdentityConverter INSTANCE = new IdentityConverter();
+
+        public ConversionResult convert(AtomicValue input) {
+            return input;
+        }
+
+        public boolean isAlwaysSuccessful() {
+            return true;
+        }
+
+    }
+
+
     /**
      * Converter that does nothing except change the type annotation of the value. The caller
      * is responsible for ensuring that this type annotation is legimite, that is, that the value
      * is in the value space of this type
      */
 
-    public static class UpCastingConverter extends Converter {
+    public static class UpCastingConverter extends UnfailingConverter<AtomicValue, AtomicValue> {
         private AtomicType newTypeAnnotation;
 
         public UpCastingConverter(AtomicType annotation) {
             this.newTypeAnnotation = annotation;
         }
 
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
+        public AtomicValue convert(/*@NotNull*/ AtomicValue input) {
             return input.copyAsSubType(newTypeAnnotation);
         }
 
-        public boolean isAlwaysSuccessful() {
-            return true;
-        }
     }
 
     /**
@@ -235,7 +201,7 @@ public abstract class Converter {
      * instance of a subtype, and returns an instance of the subtype
      */
 
-    public static class DownCastingConverter extends Converter {
+    public static class DownCastingConverter extends Converter<AtomicValue, AtomicValue> {
         private AtomicType newType;
 
         public DownCastingConverter(AtomicType annotation, ConversionRules rules) {
@@ -272,34 +238,35 @@ public abstract class Converter {
      * Converter that operates in two phases, via an intermediate type
      */
 
-    public static class TwoPhaseConverter extends StringConverter {
-        private Converter phaseOne;
-        private Converter phaseTwo;
+    public static class TwoPhaseConverter<F extends AtomicValue, T extends AtomicValue, V extends AtomicValue>
+            extends Converter<F, T> {
+        private Converter<F, V> phaseOne;
+        private Converter<V, T> phaseTwo;
 
-        public TwoPhaseConverter(Converter phaseOne, Converter phaseTwo) {
+        public TwoPhaseConverter(Converter<F, V> phaseOne, Converter<V, T> phaseTwo) {
             this.phaseOne = phaseOne;
             this.phaseTwo = phaseTwo;
         }
 
         /*@Nullable*/
         public static TwoPhaseConverter makeTwoPhaseConverter(/*@NotNull*/ AtomicType inputType, /*@NotNull*/ AtomicType viaType, /*@NotNull*/ AtomicType outputType, ConversionRules rules) {
-            return new TwoPhaseConverter(
+            return new TwoPhaseConverter<>(
                     rules.getConverter(inputType, viaType),
                     rules.getConverter(viaType, outputType));
         }
 
         @Override
-        public Converter setNamespaceResolver(NamespaceResolver resolver) {
-            return new TwoPhaseConverter(phaseOne.setNamespaceResolver(resolver), phaseTwo.setNamespaceResolver(resolver));
+        public Converter<F, T> setNamespaceResolver(NamespaceResolver resolver) {
+            return new TwoPhaseConverter<>(phaseOne.setNamespaceResolver(resolver), phaseTwo.setNamespaceResolver(resolver));
         }
 
         /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
+        public ConversionResult convert(/*@NotNull*/ F input) {
             ConversionResult temp = phaseOne.convert(input);
             if (temp instanceof ValidationFailure) {
                 return temp;
             }
-            AtomicValue aTemp = (AtomicValue)temp;
+            V aTemp = (V)temp;
             if (phaseTwo instanceof DownCastingConverter) {
                 return ((DownCastingConverter) phaseTwo).convert(aTemp, aTemp.getCanonicalLexicalRepresentation());
             } else {
@@ -308,57 +275,45 @@ public abstract class Converter {
         }
 
         /*@NotNull*/
-        public ConversionResult convertString(/*@NotNull*/ CharSequence input) {
-            ConversionResult temp = ((StringConverter) phaseOne).convertString(input);
-            if (temp instanceof ValidationFailure) {
-                return temp;
-            }
-            return ((DownCastingConverter) phaseTwo).convert((AtomicValue) temp, input);
-        }
+//        public ConversionResult convertString(/*@NotNull*/ CharSequence input) {
+//            ConversionResult temp = ((StringConverter) phaseOne).convertString(input);
+//            if (temp instanceof ValidationFailure) {
+//                return temp;
+//            }
+//            return ((DownCastingConverter) phaseTwo).convert((AtomicValue) temp, input);
+//        }
     }
 
     /**
      * Converts any value to untyped atomic
      */
 
-    public static class ToUntypedAtomicConverter extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
+    public static class ToUntypedAtomicConverter<F extends AtomicValue> extends UnfailingConverter<F, UntypedAtomicValue> {
+        public static final ToUntypedAtomicConverter INSTANCE = new ToUntypedAtomicConverter();
+        public UntypedAtomicValue convert(F input) {
             return new UntypedAtomicValue(input.getStringValueCS());
         }
-
-        public boolean isAlwaysSuccessful() {
-            return true;
-        }
     }
 
     /**
      * Converts any value to a string
      */
 
-    public static class ToStringConverter extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
+    public static class ToStringConverter extends UnfailingConverter<AtomicValue, StringValue> {
+        public static final ToStringConverter INSTANCE = new ToStringConverter();
+        public StringValue convert(AtomicValue input) {
             return new StringValue(input.getStringValueCS());
         }
-
-        public boolean isAlwaysSuccessful() {
-            return true;
-        }
     }
 
     /**
      * Converts any numeric value to xs:float
      */
 
-    public static class NumericToFloat extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
-            return new FloatValue(((NumericValue) input).getFloatValue());
-        }
-
-        public boolean isAlwaysSuccessful() {
-            return true;
+    public static class NumericToFloat extends UnfailingConverter<NumericValue, FloatValue> {
+        public final static NumericToFloat INSTANCE = new NumericToFloat();
+        public FloatValue convert(NumericValue input) {
+            return new FloatValue(input.getFloatValue());
         }
     }
 
@@ -366,14 +321,10 @@ public abstract class Converter {
      * Converts a boolean to an xs:float
      */
 
-    public static class BooleanToFloat extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
-            return new FloatValue(((BooleanValue) input).getBooleanValue() ? 1.0f : 0.0f);
-        }
-
-        public boolean isAlwaysSuccessful() {
-            return true;
+    public static class BooleanToFloat extends UnfailingConverter<BooleanValue, FloatValue> {
+        public final static BooleanToFloat INSTANCE = new BooleanToFloat();
+        public FloatValue convert(BooleanValue input) {
+            return new FloatValue(input.getBooleanValue() ? 1.0f : 0.0f);
         }
     }
 
@@ -381,33 +332,25 @@ public abstract class Converter {
      * Converts any numeric value to a double.
      */
 
-    public static class NumericToDouble extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
+    public static class NumericToDouble extends UnfailingConverter<NumericValue, DoubleValue> {
+        public static final NumericToDouble INSTANCE = new NumericToDouble();
+        public DoubleValue convert(NumericValue input) {
             if (input instanceof DoubleValue) {
-                return input;
+                return (DoubleValue)input;
             } else {
-                return new DoubleValue(((NumericValue) input).getDoubleValue());
+                return new DoubleValue(input.getDoubleValue());
             }
         }
-
-        public boolean isAlwaysSuccessful() {
-            return true;
-        }
     }
 
     /**
      * Converts a boolean to a double
      */
 
-    public static class BooleanToDouble extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
-            return new DoubleValue(((BooleanValue) input).getBooleanValue() ? 1.0e0 : 0.0e0);
-        }
-
-        public boolean isAlwaysSuccessful() {
-            return true;
+    public static class BooleanToDouble extends UnfailingConverter<BooleanValue, DoubleValue> {
+        public final static BooleanToDouble INSTANCE = new BooleanToDouble();
+        public DoubleValue convert(BooleanValue input) {
+            return new DoubleValue(input.getBooleanValue() ? 1.0e0 : 0.0e0);
         }
     }
 
@@ -415,11 +358,11 @@ public abstract class Converter {
      * Convers a double to a decimal
      */
 
-    public static class DoubleToDecimal extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
+    public static class DoubleToDecimal extends Converter<DoubleValue, DecimalValue> {
+        public final static DoubleToDecimal INSTANCE = new DoubleToDecimal();
+        public ConversionResult convert(DoubleValue input) {
             try {
-                return new BigDecimalValue(((DoubleValue) input).getDoubleValue());
+                return new BigDecimalValue(input.getDoubleValue());
             } catch (ValidationException e) {
                 return e.getValidationFailure();
             }
@@ -430,11 +373,11 @@ public abstract class Converter {
      * Converts a float to a decimal
      */
 
-    public static class FloatToDecimal extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
+    public static class FloatToDecimal extends Converter<FloatValue, DecimalValue> {
+        public final static FloatToDecimal INSTANCE = new FloatToDecimal();
+        public ConversionResult convert(FloatValue input) {
             try {
-                return new BigDecimalValue(((FloatValue) input).getFloatValue());
+                return new BigDecimalValue(input.getFloatValue());
             } catch (ValidationException e) {
                 return e.getValidationFailure();
             }
@@ -445,30 +388,26 @@ public abstract class Converter {
      * Converts an integer to a decimal
      */
 
-    public static class IntegerToDecimal extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
+    public static class IntegerToDecimal extends UnfailingConverter<IntegerValue, BigDecimalValue> {
+        public final static IntegerToDecimal INSTANCE = new IntegerToDecimal();
+        public BigDecimalValue convert(IntegerValue input) {
             if (input instanceof Int64Value) {
                 return new BigDecimalValue(((Int64Value) input).longValue());
             } else {
                 return new BigDecimalValue(((BigIntegerValue) input).asDecimal());
             }
         }
-
-        public boolean isAlwaysSuccessful() {
-            return true;
-        }
     }
 
     /**
      * Converts any numeric value to a decimal
      */
 
-    public static class NumericToDecimal extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
+    public static class NumericToDecimal extends Converter<NumericValue, DecimalValue> {
+        public final static NumericToDecimal INSTANCE = new NumericToDecimal();
+        public ConversionResult convert(NumericValue input) {
             try {
-                BigDecimal decimal = ((NumericValue) input).getDecimalValue();
+                BigDecimal decimal = input.getDecimalValue();
                 return new BigDecimalValue(decimal);
             } catch (ValidationException e) {
                 return e.getValidationFailure();
@@ -480,14 +419,10 @@ public abstract class Converter {
      * Converts a boolean to a decimal
      */
 
-    public static class BooleanToDecimal extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
-            return ((BooleanValue) input).getBooleanValue() ? BigDecimalValue.ONE : BigDecimalValue.ZERO;
-        }
-
-        public boolean isAlwaysSuccessful() {
-            return true;
+    public static class BooleanToDecimal extends UnfailingConverter<BooleanValue, DecimalValue> {
+        public final static BooleanToDecimal INSTANCE = new BooleanToDecimal();
+        public BigDecimalValue convert(BooleanValue input) {
+            return input.getBooleanValue() ? BigDecimalValue.ONE : BigDecimalValue.ZERO;
         }
     }
 
@@ -496,10 +431,10 @@ public abstract class Converter {
      * Converts a double to an integer
      */
 
-    public static class DoubleToInteger extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
-            return IntegerValue.makeIntegerValue((DoubleValue) input);
+    public static class DoubleToInteger extends Converter<DoubleValue, IntegerValue> {
+        public final static DoubleToInteger INSTANCE = new DoubleToInteger();
+        public ConversionResult convert(DoubleValue input) {
+            return IntegerValue.makeIntegerValue(input);
         }
     }
 
@@ -507,10 +442,10 @@ public abstract class Converter {
      * Converts a float to an integer
      */
 
-    public static class FloatToInteger extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
-            return IntegerValue.makeIntegerValue(new DoubleValue(((FloatValue) input).getDoubleValue()));
+    public static class FloatToInteger extends Converter<FloatValue, IntegerValue> {
+        public final static FloatToInteger INSTANCE = new FloatToInteger();
+        public ConversionResult convert(FloatValue input) {
+            return IntegerValue.makeIntegerValue(new DoubleValue(input.getDoubleValue()));
         }
     }
 
@@ -519,36 +454,32 @@ public abstract class Converter {
      * this must also be prepared to accept an xs:integer
      */
 
-    public static class DecimalToInteger extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
+    public static class DecimalToInteger extends UnfailingConverter<DecimalValue, IntegerValue> {
+        public final static DecimalToInteger INSTANCE = new DecimalToInteger();
+        public IntegerValue convert(DecimalValue input) {
             if (input instanceof IntegerValue) {
-                return input;
+                return (IntegerValue)input;
             }
             return BigIntegerValue.makeIntegerValue(((BigDecimalValue) input).getDecimalValue().toBigInteger());
         }
-
-        public boolean isAlwaysSuccessful() {
-            return true;
-        }
     }
 
     /**
      * Converts any numeric value to an integer.
      */
 
-    public static class NumericToInteger extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
+    public static class NumericToInteger extends Converter<NumericValue, IntegerValue> {
+        public final static NumericToInteger INSTANCE = new NumericToInteger();
+        public ConversionResult convert(NumericValue input) {
             try {
                 if (input instanceof IntegerValue) {
                     return input;
                 } else if (input instanceof DoubleValue) {
                     return IntegerValue.makeIntegerValue((DoubleValue) input);
                 } else if (input instanceof FloatValue) {
-                    return IntegerValue.makeIntegerValue(new DoubleValue(((FloatValue) input).getDoubleValue()));
+                    return IntegerValue.makeIntegerValue(new DoubleValue(input.getDoubleValue()));
                 } else {
-                    return BigIntegerValue.makeIntegerValue(((NumericValue) input).getDecimalValue().toBigInteger());
+                    return BigIntegerValue.makeIntegerValue(input.getDecimalValue().toBigInteger());
                 }
             } catch (ValidationException e) {
                 return e.getValidationFailure();
@@ -560,15 +491,10 @@ public abstract class Converter {
      * Converts a boolean to an integer
      */
 
-
-    public static class BooleanToInteger extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
-            return ((BooleanValue) input).getBooleanValue() ? Int64Value.PLUS_ONE : Int64Value.ZERO;
-        }
-
-        public boolean isAlwaysSuccessful() {
-            return true;
+    public static class BooleanToInteger extends UnfailingConverter<BooleanValue, Int64Value> {
+        public final static BooleanToInteger INSTANCE = new BooleanToInteger();
+        public Int64Value convert(BooleanValue input) {
+            return input.getBooleanValue() ? Int64Value.PLUS_ONE : Int64Value.ZERO;
         }
     }
 
@@ -576,11 +502,14 @@ public abstract class Converter {
      * Converts a duration to a dayTimeDuration
      */
 
-    public static class DurationToDayTimeDuration extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
-            DurationValue d = (DurationValue) input;
-            return new DayTimeDurationValue(d.signum(), d.getDays(), d.getHours(), d.getMinutes(), d.getSeconds(), d.getMicroseconds());
+    public static class DurationToDayTimeDuration extends UnfailingConverter<DurationValue, DayTimeDurationValue> {
+        public final static DurationToDayTimeDuration INSTANCE = new DurationToDayTimeDuration();
+        public DayTimeDurationValue convert(DurationValue d) {
+            if (d.signum() < 0) {
+                return new DayTimeDurationValue(-d.getDays(), -d.getHours(), -d.getMinutes(), -d.getSeconds(), -d.getNanoseconds());
+            } else {
+                return new DayTimeDurationValue(d.getDays(), d.getHours(), d.getMinutes(), d.getSeconds(), d.getNanoseconds());
+            }
         }
     }
 
@@ -588,11 +517,10 @@ public abstract class Converter {
      * Converts a duration to a yearMonthDuration
      */
 
-    public static class DurationToYearMonthDuration extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
-            DurationValue d = (DurationValue) input;
-            return YearMonthDurationValue.fromMonths(d.getTotalMonths());
+    public static class DurationToYearMonthDuration extends UnfailingConverter<DurationValue, YearMonthDurationValue> {
+        public final static DurationToYearMonthDuration INSTANCE = new DurationToYearMonthDuration();
+        public YearMonthDurationValue convert(DurationValue input) {
+            return YearMonthDurationValue.fromMonths(input.getTotalMonths());
         }
     }
 
@@ -600,14 +528,10 @@ public abstract class Converter {
      * Converts a date to a dateTime
      */
 
-    public static class DateToDateTime extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
-            return ((DateValue) input).toDateTime();
-        }
-
-        public boolean isAlwaysSuccessful() {
-            return true;
+    public static class DateToDateTime extends UnfailingConverter<DateValue, DateTimeValue> {
+        public final static DateToDateTime INSTANCE = new DateToDateTime();
+        public DateTimeValue convert(DateValue input) {
+            return input.toDateTime();
         }
     }
 
@@ -615,130 +539,87 @@ public abstract class Converter {
      * Converts a dateTime to a date
      */
 
-    public static class DateTimeToDate extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
-            DateTimeValue dt = (DateTimeValue) input;
+    public static class DateTimeToDate extends UnfailingConverter<DateTimeValue, DateValue> {
+        public final static DateTimeToDate INSTANCE = new DateTimeToDate();
+        public DateValue convert(DateTimeValue dt) {
             return new DateValue(dt.getYear(), dt.getMonth(), dt.getDay(), dt.getTimezoneInMinutes(), dt.isXsd10Rules());
         }
-
-        public boolean isAlwaysSuccessful() {
-            return true;
-        }
     }
 
     /**
      * Converts a dateTime to a gMonth
      */
 
-    public static class DateTimeToGMonth extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
-            DateTimeValue dt = (DateTimeValue) input;
+    public static class DateTimeToGMonth extends UnfailingConverter<DateTimeValue, GMonthValue> {
+        public static final DateTimeToGMonth INSTANCE = new DateTimeToGMonth();
+        public GMonthValue convert(DateTimeValue dt) {
             return new GMonthValue(dt.getMonth(), dt.getTimezoneInMinutes());
         }
-
-        public boolean isAlwaysSuccessful() {
-            return true;
-        }
     }
 
     /**
      * Converts a dateTime to a gYearMonth
      */
 
-    public static class DateTimeToGYearMonth extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
-            DateTimeValue dt = (DateTimeValue) input;
+    public static class DateTimeToGYearMonth extends UnfailingConverter<DateTimeValue, GYearMonthValue> {
+        public static final DateTimeToGYearMonth INSTANCE = new DateTimeToGYearMonth();
+        public GYearMonthValue convert(DateTimeValue dt) {
             return new GYearMonthValue(dt.getYear(), dt.getMonth(), dt.getTimezoneInMinutes(), dt.isXsd10Rules());
         }
-
-        public boolean isAlwaysSuccessful() {
-            return true;
-        }
     }
 
     /**
      * Converts a dateTime to a gYear
      */
 
-    public static class DateTimeToGYear extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
-            DateTimeValue dt = (DateTimeValue) input;
+    public static class DateTimeToGYear extends UnfailingConverter<DateTimeValue, GYearValue> {
+        public static final DateTimeToGYear INSTANCE = new DateTimeToGYear();
+        public GYearValue convert(DateTimeValue dt) {
             return new GYearValue(dt.getYear(), dt.getTimezoneInMinutes(), dt.isXsd10Rules());
         }
-
-        public boolean isAlwaysSuccessful() {
-            return true;
-        }
     }
 
     /**
      * Converts a dateTime to a gMonthDay
      */
 
-    public static class DateTimeToGMonthDay extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
-            DateTimeValue dt = (DateTimeValue) input;
+    public static class DateTimeToGMonthDay extends UnfailingConverter<DateTimeValue, GMonthDayValue> {
+        public static final DateTimeToGMonthDay INSTANCE = new DateTimeToGMonthDay();
+        public GMonthDayValue convert(DateTimeValue dt) {
             return new GMonthDayValue(dt.getMonth(), dt.getDay(), dt.getTimezoneInMinutes());
         }
-
-        public boolean isAlwaysSuccessful() {
-            return true;
-        }
     }
 
     /**
      * Converts a dateTime to a gDay
      */
 
-    public static class DateTimeToGDay extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
-            DateTimeValue dt = (DateTimeValue) input;
+    public static class DateTimeToGDay extends UnfailingConverter<DateTimeValue, GDayValue> {
+        public final static DateTimeToGDay INSTANCE = new DateTimeToGDay();
+        public GDayValue convert(DateTimeValue dt) {
             return new GDayValue(dt.getDay(), dt.getTimezoneInMinutes());
         }
-
-        public boolean isAlwaysSuccessful() {
-            return true;
-        }
     }
 
     /**
      * Converts a dateTime to a time
      */
 
-    public static class DateTimeToTime extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
-            DateTimeValue dt = (DateTimeValue) input;
+    public static class DateTimeToTime extends UnfailingConverter<DateTimeValue, TimeValue> {
+        public final static DateTimeToTime INSTANCE = new DateTimeToTime();
+        public TimeValue convert(DateTimeValue dt) {
             return new TimeValue(dt.getHour(), dt.getMinute(), dt.getSecond(), dt.getMicrosecond(), dt.getTimezoneInMinutes());
         }
-
-        public boolean isAlwaysSuccessful() {
-            return true;
-        }
     }
 
     /**
      * Converts a numeric value to a boolean
      */
 
-    public static class NumericToBoolean extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
-            try {
-                return BooleanValue.get(input.effectiveBooleanValue());
-            } catch (XPathException err) {
-                throw new AssertionError(err);
-            }
-        }
-
-        public boolean isAlwaysSuccessful() {
-            return true;
+    public static class NumericToBoolean extends UnfailingConverter<NumericValue, BooleanValue> {
+        public static final NumericToBoolean INSTANCE = new NumericToBoolean();
+        public BooleanValue convert(NumericValue input) {
+            return BooleanValue.get(input.effectiveBooleanValue());
         }
     }
 
@@ -746,49 +627,22 @@ public abstract class Converter {
      * Converts base64 to hexBinary
      */
 
-    public static class Base64BinaryToHexBinary extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
-            return new HexBinaryValue(((Base64BinaryValue) input).getBinaryValue());
-        }
-
-        public boolean isAlwaysSuccessful() {
-            return true;
+    public static class Base64BinaryToHexBinary extends UnfailingConverter<Base64BinaryValue, HexBinaryValue> {
+        public static final Base64BinaryToHexBinary INSTANCE = new Base64BinaryToHexBinary();
+        public HexBinaryValue convert(Base64BinaryValue input) {
+            return new HexBinaryValue(input.getBinaryValue());
         }
     }
 
-    /**
-     * Converts string to base64
-     */
-
-    public static class StringToBase64BinaryConverter extends StringConverter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
-            return convertString(input.getStringValueCS());
-        }
-
-        /*@NotNull*/
-        public ConversionResult convertString(/*@NotNull*/ CharSequence input) {
-            try {
-                return new Base64BinaryValue(input);
-            } catch (XPathException e) {
-                return ValidationFailure.fromException(e);
-            }
-        }
-    }
 
     /**
      * Converts hexBinary to base64Binary
      */
 
-    public static class HexBinaryToBase64Binary extends Converter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
-            return new Base64BinaryValue(((HexBinaryValue) input).getBinaryValue());
-        }
-
-        public boolean isAlwaysSuccessful() {
-            return true;
+    public static class HexBinaryToBase64Binary extends UnfailingConverter<HexBinaryValue, Base64BinaryValue> {
+        public static final HexBinaryToBase64Binary INSTANCE = new HexBinaryToBase64Binary();
+        public Base64BinaryValue convert(HexBinaryValue input) {
+            return new Base64BinaryValue(input.getBinaryValue());
         }
     }
 
@@ -796,15 +650,10 @@ public abstract class Converter {
      * Converts Notation to QName
      */
 
-    public static class NotationToQName extends Converter {
-
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
-            return new QNameValue(((NotationValue) input).getStructuredQName(), BuiltInAtomicType.QNAME);
-        }
-
-        public boolean isAlwaysSuccessful() {
-            return true;
+    public static class NotationToQName extends UnfailingConverter<NotationValue, QNameValue> {
+        public static final NotationToQName INSTANCE = new NotationToQName();
+        public QNameValue convert(NotationValue input) {
+            return new QNameValue(input.getStructuredQName(), BuiltInAtomicType.QNAME);
         }
     }
 
@@ -812,11 +661,10 @@ public abstract class Converter {
      * Converts QName to Notation
      */
 
-    public static class QNameToNotation extends Converter {
-
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
-            return new NotationValue(((QNameValue) input).getStructuredQName(), BuiltInAtomicType.NOTATION);
+    public static class QNameToNotation extends UnfailingConverter<QNameValue, NotationValue> {
+        public static final QNameToNotation INSTANCE = new QNameToNotation();
+        public NotationValue convert(QNameValue input) {
+            return new NotationValue(input.getStructuredQName(), BuiltInAtomicType.NOTATION);
         }
     }
 
@@ -824,13 +672,13 @@ public abstract class Converter {
      * Converter that implements the promotion rules to a required type of xs:double
      */
 
-    public static class PromoterToDouble extends Converter {
+    public static class PromoterToDouble extends Converter<AtomicValue, DoubleValue> {
 
         /*@Nullable*/ private StringConverter stringToDouble = null;
 
         /*@NotNull*/
         @Override
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
+        public ConversionResult convert(AtomicValue input) {
             if (input instanceof DoubleValue) {
                 return input;
             } else if (input instanceof NumericValue) {
@@ -839,7 +687,7 @@ public abstract class Converter {
                 if (stringToDouble == null) {
                     stringToDouble = BuiltInAtomicType.DOUBLE.getStringConverter(getConversionRules());
                 }
-                return stringToDouble.convert(input);
+                return stringToDouble.convert((UntypedAtomicValue)input);
             } else {
                 ValidationFailure err = new ValidationFailure(
                         "Cannot promote non-numeric value to xs:double");
@@ -853,7 +701,7 @@ public abstract class Converter {
      * Converter that implements the promotion rules to a required type of xs:float
      */
 
-    public static class PromoterToFloat extends Converter {
+    public static class PromoterToFloat extends Converter<AtomicValue, FloatValue> {
 
         /*@Nullable*/ private StringConverter stringToFloat = null;
 
@@ -873,7 +721,7 @@ public abstract class Converter {
                 if (stringToFloat == null) {
                     stringToFloat = BuiltInAtomicType.FLOAT.getStringConverter(getConversionRules());
                 }
-                return stringToFloat.convert(input);
+                return stringToFloat.convert((UntypedAtomicValue)input);
             } else {
                 ValidationFailure err = new ValidationFailure(
                         "Cannot promote non-numeric value to xs:double");
