@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -10,9 +10,7 @@ package net.sf.saxon.type;
 import net.sf.saxon.Configuration;
 import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.om.Item;
-import net.sf.saxon.om.StandardNames;
 import net.sf.saxon.om.StructuredQName;
-import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.value.ObjectValue;
 
@@ -21,11 +19,10 @@ import net.sf.saxon.value.ObjectValue;
  * an extension function, or supplied as an external variable/parameter.
  */
 
-public class JavaExternalObjectType extends ExternalObjectType implements ItemType {
+public class JavaExternalObjectType extends ExternalObjectType {
 
-    public static JavaExternalObjectType EXTERNAL_OBJECT_TYPE = new JavaExternalObjectType(Object.class);
-
-    private Class javaClass;
+    protected Configuration config;
+    protected Class javaClass;
 
     /**
      * Create an external object type.
@@ -33,29 +30,13 @@ public class JavaExternalObjectType extends ExternalObjectType implements ItemTy
      * @param javaClass the Java class to which this type corresponds
      */
 
-    public JavaExternalObjectType(Class javaClass) {
+    public JavaExternalObjectType(Configuration config, Class javaClass) {
+        this.config = config;
         this.javaClass = javaClass;
     }
 
-    /**
-     * Create an external object type.
-     *
-     * @param javaClass the Java class to which this type corresponds
-     * @param config    the Saxon configuration. This argument is no longer used (since 9.5).
-     */
-
-    public JavaExternalObjectType(/*@NotNull*/ Class javaClass, /*@NotNull*/ Configuration config) {
-        this(javaClass);
-    }
-
-    /**
-     * Get the corresponding {@link net.sf.saxon.type.UType}. A UType is a union of primitive item
-     * types.
-     *
-     * @return the smallest UType that subsumes this item type
-     */
-    public UType getUType() {
-        return UType.EXTENSION;
+    public Configuration getConfiguration() {
+        return config;
     }
 
     /**
@@ -80,17 +61,6 @@ public class JavaExternalObjectType extends ExternalObjectType implements ItemTy
         return NamespaceConstant.JAVA_TYPE;
     }
 
-    /**
-     * Return true if this is an external object type, that is, a Saxon-defined type for external
-     * Java or .NET objects
-     *
-     * @return true (always)
-     */
-
-    public boolean isExternalType() {
-        return true;
-    }
-
     /**
      * Get the name of this type as a StructuredQName, unless the type is anonymous, in which case
      * return null
@@ -112,41 +82,7 @@ public class JavaExternalObjectType extends ExternalObjectType implements ItemTy
 
     /*@NotNull*/
     public ItemType getPrimitiveItemType() {
-        return EXTERNAL_OBJECT_TYPE;
-    }
-
-    /**
-     * Get the primitive type corresponding to this item type. For item(),
-     * this is Type.ITEM. For node(), it is Type.NODE. For specific node kinds,
-     * it is the value representing the node kind, for example Type.ELEMENT.
-     * For anyAtomicValue it is Type.ATOMIC. For numeric it is Type.NUMBER.
-     * For other atomic types it is the primitive type as defined in XML Schema,
-     * except that INTEGER is considered to be a primitive type.
-     */
-
-    public int getPrimitiveType() {
-        return StandardNames.XS_ANY_ATOMIC_TYPE;
-    }
-
-    /**
-     * Get the item type of the atomic values that will be produced when an item
-     * of this type is atomized
-     *
-     * @return BuiltInAtomicType.STRING, because atomization returns the result of toString()
-     */
-
-    public AtomicType getAtomizedItemType() {
-        return BuiltInAtomicType.STRING;
-    }
-
-    /**
-     * Ask whether values of this type are atomizable
-     *
-     * @return true, because (since Saxon 9.5) atomization will always return the result of toString()
-     */
-
-    public boolean isAtomizable() {
-        return true;
+        return config.getJavaExternalObjectType(Object.class);
     }
 
     /**
@@ -172,21 +108,6 @@ public class JavaExternalObjectType extends ExternalObjectType implements ItemTy
         }
     }
 
-    /*@NotNull*/
-    public String getDescription() {
-        return getDisplayName();
-    }
-
-    /**
-     * Test whether this item type is an atomic type
-     *
-     * @return false, this is not considered to be an atomic type
-     */
-
-    public boolean isAtomicType() {
-        return false;
-    }
-
     /**
      * Get the Java class to which this external object type corresponds
      *
@@ -199,11 +120,8 @@ public class JavaExternalObjectType extends ExternalObjectType implements ItemTy
 
     /**
      * Test whether a given item conforms to this type
-     *
-     *
-     *
      * @param item    The item to be tested
-     * @param th
+     * @param th      The type hierarchy cache
      * @return true if the item is an instance of this type; false otherwise
      */
     public boolean matches(/*@NotNull*/ Item item, /*@NotNull*/TypeHierarchy th) {
@@ -214,51 +132,16 @@ public class JavaExternalObjectType extends ExternalObjectType implements ItemTy
         return false;
     }
 
-
     /*@NotNull*/
     public String toString() {
         return classNameToQName(javaClass.getName()).getEQName();
     }
 
-    /**
-     * Return a string representation of this SequenceType suitable for use in export (SEF) files.
-     * The difference from the toString() method is that the representation will not contain
-     * references to anonymous types.
-     *
-     * @return the string representation as an instance of the XPath
-     * SequenceType construct, using only names that will be in-scope if the target environment
-     * imports the same schema as the source environment
-     */
-    @Override
-    public String toExportString() {
-        return toString();
-    }
-
     /*@NotNull*/
     public String getDisplayName() {
         return "java-type:" + javaClass.getName();
     }
 
-    /**
-     * Determine the default priority of this item type when used on its own as a Pattern
-     *
-     * @return the default priority
-     */
-    public double getDefaultPriority() {
-        return 0;
-    }
-
-    /**
-     * Visit all the schema components used in this ItemType definition
-     *
-     * @param visitor the visitor class to be called when each component is visited
-     * @throws net.sf.saxon.trans.XPathException
-     *          if an error occurs
-     */
-    public void visitNamedSchemaComponents(SchemaComponentVisitor visitor) throws XPathException {
-        // no action
-    }
-
     /**
      * Returns a hash code value for the object.
      */
@@ -320,5 +203,4 @@ public class JavaExternalObjectType extends ExternalObjectType implements ItemTy
         return new StructuredQName("jt", NamespaceConstant.JAVA_TYPE, classNameToLocalName(className));
     }
 
-
 }
\ No newline at end of file
