@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -17,13 +17,18 @@ import net.sf.saxon.ma.map.MapItem;
 import net.sf.saxon.ma.map.MapType;
 import net.sf.saxon.om.Function;
 import net.sf.saxon.om.Item;
+import net.sf.saxon.om.GroundedValue;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.query.AnnotationList;
+import net.sf.saxon.trans.Err;
 import net.sf.saxon.trans.SaxonErrorCode;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.value.SequenceType;
 
+import java.util.Objects;
+import java.util.Optional;
+
 /**
  * An instance of this class represents a specific function item type, for example
  * function(xs:int) as xs:boolean
@@ -35,15 +40,15 @@ public class SpecificFunctionType extends AnyFunctionType {
     private AnnotationList annotations;
 
     public SpecificFunctionType(SequenceType[] argTypes, SequenceType resultType) {
-        this.argTypes = argTypes;
-        this.resultType = resultType;
+        this.argTypes = Objects.requireNonNull(argTypes);
+        this.resultType = Objects.requireNonNull(resultType);
         this.annotations = AnnotationList.EMPTY;
     }
 
     public SpecificFunctionType(SequenceType[] argTypes, SequenceType resultType, AnnotationList annotations) {
-        this.argTypes = argTypes;
-        this.resultType = resultType;
-        this.annotations = annotations;
+        this.argTypes = Objects.requireNonNull(argTypes);
+        this.resultType = Objects.requireNonNull(resultType);
+        this.annotations = Objects.requireNonNull(annotations);
     }
 
     /**
@@ -87,6 +92,24 @@ public class SpecificFunctionType extends AnyFunctionType {
         return annotations;
     }
 
+    /**
+     * Ask whether values of this type are atomizable
+     *
+     * @return true if some or all instances of this type can be successfully atomized; false
+     * * if no instances of this type can be atomized
+     * @param th The type hierarchy cache
+     */
+    @Override
+    public boolean isAtomizable(TypeHierarchy th) {
+        // An instance of a specific function type can be atomized only if it is an array, which
+        // means there must be a single argument and it must be of type xs:integer or a supertype.
+        if (getArity() != 1) {
+            return false;
+        }
+        ItemType argType = getArgumentTypes()[0].getPrimaryType();
+        return th.isSubType(BuiltInAtomicType.INTEGER, argType);
+    }
+
     /**
      * Produce a representation of this type name for use in error messages.
      *
@@ -108,16 +131,6 @@ public class SpecificFunctionType extends AnyFunctionType {
         return sb.toString();
     }
 
-    /**
-     * Return a string representation of this SequenceType suitable for use in export (SEF) files.
-     * The difference from the toString() method is that the representation will not contain
-     * references to anonymous types.
-     *
-     * @return the string representation as an instance of the XPath
-     * SequenceType construct, using only names that will be in-scope if the target environment
-     * imports the same schema as the source environment
-     */
-    @Override
     public String toExportString() {
         FastStringBuffer sb = new FastStringBuffer(100);
         sb.append("(function(");
@@ -133,6 +146,7 @@ public class SpecificFunctionType extends AnyFunctionType {
         return sb.toString();
     }
 
+
     /**
      * Test whether this function type equals another function type
      */
@@ -180,7 +194,7 @@ public class SpecificFunctionType extends AnyFunctionType {
      */
 
     public int relationship(FunctionItemType other, TypeHierarchy th) {
-        if (other == AnyFunctionType.getInstance()) {
+        if (other == AnyFunctionType.getInstance() || other instanceof AnyFunctionTypeWithAssertions) {
             return TypeHierarchy.SUBSUMED_BY;
         } else if (equals(other)) {
             return TypeHierarchy.SAME_TYPE;
@@ -278,7 +292,7 @@ public class SpecificFunctionType extends AnyFunctionType {
             if (getArity() == 1 &&
                     argTypes[0].getCardinality() == StaticProperty.EXACTLY_ONE &&
                     argTypes[0].getPrimaryType().isPlainType()) {
-                for (KeyValuePair pair : (MapItem) item) {
+                for (KeyValuePair pair : ((MapItem) item).keyValuePairs()) {
                     try {
                         if (!resultType.matches(pair.value, th)) {
                             return false;
@@ -303,7 +317,7 @@ public class SpecificFunctionType extends AnyFunctionType {
                 if (!(rel == TypeHierarchy.SAME_TYPE || rel == TypeHierarchy.SUBSUMED_BY)) {
                     return false;
                 }
-                for (Sequence member : (ArrayItem) item) {
+                for (Sequence member : ((ArrayItem) item).members()) {
                     try {
                         if (!resultType.matches(member, th)) {
                             return false;
@@ -322,24 +336,123 @@ public class SpecificFunctionType extends AnyFunctionType {
         return rel == TypeHierarchy.SAME_TYPE || rel == TypeHierarchy.SUBSUMED_BY;
     }
 
+    /**
+     * Get extra diagnostic information about why a supplied item does not conform to this
+     * item type, if available. If extra information is returned, it should be in the form of a complete
+     * sentence, minus the closing full stop. No information should be returned for obvious cases.
+     *
+     * @param item the item that doesn't match this type
+     * @param th   the type hierarchy cache
+     * @return optionally, a message explaining why the item does not match the type
+     */
+    @Override
+    public Optional<String> explainMismatch(Item item, TypeHierarchy th) {
+        if (!(item instanceof Function)) {
+            return Optional.empty();
+        }
+
+        if (item instanceof MapItem) {
+            if (getArity() == 1) {
+                if (argTypes[0].getCardinality() == StaticProperty.EXACTLY_ONE &&
+                    argTypes[0].getPrimaryType().isPlainType()) {
+                    for (KeyValuePair pair : ((MapItem) item).keyValuePairs()) {
+                        try {
+                            if (!resultType.matches(pair.value, th)) {
+                                String s = "The supplied map contains an entry with key (" + pair.key +
+                                        ") whose corresponding value (" + Err.depictSequence(pair.value) +
+                                        ") is not an instance of the return type in the function signature (" +
+                                        resultType + ")";
+                                Optional<String> more = resultType.explainMismatch(pair.value, th);
+                                if (more.isPresent()) {
+                                    s = s + ". " + more.get();
+                                }
+                                return Optional.of(s);
+                            }
+                        } catch (XPathException e) {
+                            return Optional.empty();
+                        }
+                    }
+                } else {
+                    String s = "The function argument is of type " + argTypes[0] +
+                            "; a map can only be supplied for a function type whose argument type is atomic";
+                    return Optional.of(s);
+                }
+            } else {
+                String s = "The function arity is " + getArity() + "; a map can only be supplied for a function type with arity 1";
+                return Optional.of(s);
+            }
+        }
+
+        if (item instanceof ArrayItem) {
+            // Bug 2938: Essentially a array is an instance of function(X) as Y
+            // if (a) X is a subtype of xs:integer, and (b) all the values in the array are instances of Y
+            if (getArity() == 1) {
+                if (argTypes[0].getCardinality() == StaticProperty.EXACTLY_ONE &&
+                        argTypes[0].getPrimaryType().isPlainType()) {
+                    int rel = th.relationship(argTypes[0].getPrimaryType(), BuiltInAtomicType.INTEGER);
+                    if (!(rel == TypeHierarchy.SAME_TYPE || rel == TypeHierarchy.SUBSUMED_BY)) {
+                        String s = "The function expects an argument of type " + argTypes[0] +
+                                "; an array can only be supplied for a function that expects an integer";
+                        return Optional.of(s);
+                    } else {
+                        for (GroundedValue member : (ArrayItem) item) {
+                            try {
+                                if (!resultType.matches(member, th)) {
+                                    String s = "The supplied array contains an entry (" + Err.depictSequence(member) +
+                                            ") is not an instance of the return type in the function signature (" +
+                                            resultType + ")";
+                                    Optional<String> more = resultType.explainMismatch(member, th);
+                                    if (more.isPresent()) {
+                                        s = s + ". " + more.get();
+                                    }
+                                    return Optional.of(s);
+                                }
+                            } catch (XPathException e) {
+                                return Optional.empty();
+                            }
+                        }
+                    }
+                } else {
+                    String s = "The function argument is of type " + argTypes[0] +
+                            "; an array can only be supplied for a function type whose argument type is xs:integer";
+                    return Optional.of(s);
+                }
+            } else {
+                String s = "The function arity is " + getArity() + "; an array can only be supplied for a function type with arity 1";
+                return Optional.of(s);
+            }
+        }
+
+        FunctionItemType other = ((Function) item).getFunctionItemType();
+        if (getArity() != ((Function) item).getArity()) {
+            String s = "The required function arity is " + getArity() +
+                    "; the supplied function has arity " + ((Function) item).getArity();
+            return Optional.of(s);
+        }
+        int rel = th.sequenceTypeRelationship(resultType, other.getResultType());
+        if (rel != TypeHierarchy.SAME_TYPE && rel != TypeHierarchy.SUBSUMES) {
+            String s = "The return type of the required function is " + resultType + " but the return"
+                    + "type of the supplied function is " + other.getResultType();
+            return Optional.of(s);
+        }
+        for (int j=0; j<getArity(); j++) {
+            rel = th.sequenceTypeRelationship(argTypes[j], other.getArgumentTypes()[j]);
+            if (rel != TypeHierarchy.SAME_TYPE && rel != TypeHierarchy.SUBSUMED_BY) {
+                String s = "The type of the " + RoleDiagnostic.ordinal(j+1) +
+                        " argument of the required function is " + argTypes[j] + " but the declared"
+                        + "type of the corresponding argument of the supplied function is " +
+                        other.getArgumentTypes()[j];
+                return Optional.of(s);
+            }
+        }
+        return Optional.empty();
+    }
+
     public Expression makeFunctionSequenceCoercer(Expression exp, RoleDiagnostic role)
             throws XPathException {
         return exp.getConfiguration().makeFunctionSequenceCoercer(this, exp, role);
     }
 
-    /**
-     * Visit all the schema components used in this ItemType definition
-     *
-     * @param visitor the visitor class to be called when each component is visited
-     */
-
-    public void visitNamedSchemaComponents(SchemaComponentVisitor visitor) throws XPathException {
-        resultType.getPrimaryType().visitNamedSchemaComponents(visitor);
-        for (SequenceType argType : argTypes) {
-            argType.getPrimaryType().visitNamedSchemaComponents(visitor);
-        }
-    }
-
     /**
      * Generate Javascript code to test whether an item conforms to this item type
      *
