@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -23,105 +23,16 @@ import java.util.regex.Pattern;
  * A {@link Converter} that accepts a string as input. This subclass of Converter is provided
  * to avoid having to wrap the string into a StringValue prior to conversion. Every Converter whose
  * source type is xs:string must be an instance of this subclass.
- * <p/>
  * <p>The input to a StringConverter can also be an xs:untypedAtomic value, since the conversion semantics
  * are always the same as from a string.</p>
- * <p/>
  * <p>A StringConverter also provides a method to validate that a string is valid against the target type,
  * without actually performing the conversion.</p>
  */
-public abstract class StringConverter extends Converter {
+public abstract class StringConverter<T extends AtomicValue> extends Converter<StringValue, T> {
 
     // Constants are defined only for converters that are independent of the conversion rules
 
-    /*@NotNull*/ public final static StringToString
-            STRING_TO_STRING = new StringToString();
-    /*@NotNull*/ public final static StringToLanguage
-            STRING_TO_LANGUAGE = new StringToLanguage();
-    /*@NotNull*/ public final static StringToNormalizedString
-            STRING_TO_NORMALIZED_STRING = new StringToNormalizedString();
-    /*@NotNull*/ public final static StringToName
-            STRING_TO_NAME = new StringToName();
-    /*@NotNull*/ public final static StringToNCName
-            STRING_TO_NCNAME = new StringToNCName(BuiltInAtomicType.NCNAME);
-    /*@NotNull*/ public final static StringToNMTOKEN
-            STRING_TO_NMTOKEN = new StringToNMTOKEN();
-    /*@NotNull*/ public final static StringToNCName
-            STRING_TO_ID = new StringToNCName(BuiltInAtomicType.ID);
-    /*@NotNull*/ public final static StringToNCName
-            STRING_TO_IDREF = new StringToNCName(BuiltInAtomicType.IDREF);
-    /*@NotNull*/ public final static StringToNCName
-            STRING_TO_ENTITY = new StringToNCName(BuiltInAtomicType.ENTITY);
-    /*@NotNull*/ public final static StringToToken
-            STRING_TO_TOKEN = new StringToToken();
-    /*@NotNull*/ public final static StringToDecimal
-            STRING_TO_DECIMAL = new StringToDecimal();
-    /*@NotNull*/ public final static StringToInteger
-            STRING_TO_INTEGER = new StringToInteger();
-    /*@NotNull*/ public final static StringToGMonth
-            STRING_TO_G_MONTH = new StringToGMonth();
-    /*@NotNull*/ public final static StringToGMonthDay
-            STRING_TO_G_MONTH_DAY = new StringToGMonthDay();
-    /*@NotNull*/ public final static StringToGDay
-            STRING_TO_G_DAY = new StringToGDay();
-    /*@NotNull*/ public final static StringToDuration
-            STRING_TO_DURATION = new StringToDuration();
-    /*@NotNull*/ public final static StringToDayTimeDuration
-            STRING_TO_DAY_TIME_DURATION = new StringToDayTimeDuration();
-    /*@NotNull*/ public final static StringToYearMonthDuration
-            STRING_TO_YEAR_MONTH_DURATION = new StringToYearMonthDuration();
-    /*@NotNull*/ public final static StringToTime
-            STRING_TO_TIME = new StringToTime();
-    /*@NotNull*/ public final static StringToBoolean
-            STRING_TO_BOOLEAN = new StringToBoolean();
-    /*@NotNull*/ public final static StringToHexBinary
-            STRING_TO_HEX_BINARY = new StringToHexBinary();
-    /*@NotNull*/ public final static StringToBase64BinaryConverter
-            STRING_TO_BASE64_BINARY = new StringToBase64BinaryConverter();
-    /*@NotNull*/ public final static StringToUntypedAtomic
-            STRING_TO_UNTYPED_ATOMIC = new StringToUntypedAtomic();
 
-    static {
-        // See bug 2524
-        BuiltInAtomicType.ANY_ATOMIC.stringConverter = Converter.IDENTITY_CONVERTER;
-        BuiltInAtomicType.STRING.stringConverter = STRING_TO_STRING;
-        BuiltInAtomicType.LANGUAGE.stringConverter = STRING_TO_LANGUAGE;
-        BuiltInAtomicType.NORMALIZED_STRING.stringConverter = STRING_TO_NORMALIZED_STRING;
-        BuiltInAtomicType.TOKEN.stringConverter = STRING_TO_TOKEN;
-        BuiltInAtomicType.NCNAME.stringConverter = STRING_TO_NCNAME;
-        BuiltInAtomicType.NAME.stringConverter = STRING_TO_NAME;
-        BuiltInAtomicType.NMTOKEN.stringConverter = STRING_TO_NMTOKEN;
-        BuiltInAtomicType.ID.stringConverter = STRING_TO_ID;
-        BuiltInAtomicType.IDREF.stringConverter = STRING_TO_IDREF;
-        BuiltInAtomicType.ENTITY.stringConverter = STRING_TO_ENTITY;
-        BuiltInAtomicType.DECIMAL.stringConverter = STRING_TO_DECIMAL;
-        BuiltInAtomicType.INTEGER.stringConverter = STRING_TO_INTEGER;
-        BuiltInAtomicType.DURATION.stringConverter = STRING_TO_DURATION;
-        BuiltInAtomicType.G_MONTH.stringConverter = STRING_TO_G_MONTH;
-        BuiltInAtomicType.G_MONTH_DAY.stringConverter = STRING_TO_G_MONTH_DAY;
-        BuiltInAtomicType.G_DAY.stringConverter = STRING_TO_G_DAY;
-        BuiltInAtomicType.DAY_TIME_DURATION.stringConverter = STRING_TO_DAY_TIME_DURATION;
-        BuiltInAtomicType.YEAR_MONTH_DURATION.stringConverter = STRING_TO_YEAR_MONTH_DURATION;
-        BuiltInAtomicType.TIME.stringConverter = STRING_TO_TIME;
-        BuiltInAtomicType.BOOLEAN.stringConverter = STRING_TO_BOOLEAN;
-        BuiltInAtomicType.HEX_BINARY.stringConverter = STRING_TO_HEX_BINARY;
-        BuiltInAtomicType.BASE64_BINARY.stringConverter = STRING_TO_BASE64_BINARY;
-        BuiltInAtomicType.UNTYPED_ATOMIC.stringConverter = STRING_TO_UNTYPED_ATOMIC;
-
-        BuiltInAtomicType.NON_POSITIVE_INTEGER.stringConverter = new StringToIntegerSubtype(BuiltInAtomicType.NON_POSITIVE_INTEGER);
-        BuiltInAtomicType.NEGATIVE_INTEGER.stringConverter = new StringToIntegerSubtype(BuiltInAtomicType.NEGATIVE_INTEGER);
-        BuiltInAtomicType.LONG.stringConverter = new StringToIntegerSubtype(BuiltInAtomicType.LONG);
-        BuiltInAtomicType.INT.stringConverter = new StringToIntegerSubtype(BuiltInAtomicType.INT);
-        BuiltInAtomicType.SHORT.stringConverter = new StringToIntegerSubtype(BuiltInAtomicType.SHORT);
-        BuiltInAtomicType.BYTE.stringConverter = new StringToIntegerSubtype(BuiltInAtomicType.BYTE);
-        BuiltInAtomicType.NON_NEGATIVE_INTEGER.stringConverter = new StringToIntegerSubtype(BuiltInAtomicType.NON_NEGATIVE_INTEGER);
-        BuiltInAtomicType.POSITIVE_INTEGER.stringConverter = new StringToIntegerSubtype(BuiltInAtomicType.POSITIVE_INTEGER);
-        BuiltInAtomicType.UNSIGNED_LONG.stringConverter = new StringToIntegerSubtype(BuiltInAtomicType.UNSIGNED_LONG);
-        BuiltInAtomicType.UNSIGNED_INT.stringConverter = new StringToIntegerSubtype(BuiltInAtomicType.UNSIGNED_INT);
-        BuiltInAtomicType.UNSIGNED_SHORT.stringConverter = new StringToIntegerSubtype(BuiltInAtomicType.UNSIGNED_SHORT);
-        BuiltInAtomicType.UNSIGNED_BYTE.stringConverter = new StringToIntegerSubtype(BuiltInAtomicType.UNSIGNED_BYTE);
-
-    }
 
     /**
      * Create a StringConverter
@@ -145,7 +56,7 @@ public abstract class StringConverter extends Converter {
      *
      * @param input the string to be converted
      * @return either an {@link net.sf.saxon.value.AtomicValue} of the appropriate type for this converter (if conversion
-     *         succeeded), or a {@link ValidationFailure} if conversion failed.
+     * succeeded), or a {@link ValidationFailure} if conversion failed.
      */
 
     /*@NotNull*/
@@ -157,7 +68,7 @@ public abstract class StringConverter extends Converter {
      *
      * @param input the string to be validated
      * @return null if validation is successful, or a ValidationFailure indicating the reasons for failure
-     *         if unsuccessful
+     * if unsuccessful
      */
 
     /*@Nullable*/
@@ -168,7 +79,7 @@ public abstract class StringConverter extends Converter {
 
     /*@NotNull*/
     @Override
-    public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
+    public ConversionResult convert(StringValue input) {
         return convertString(input.getStringValueCS());
     }
 
@@ -178,7 +89,7 @@ public abstract class StringConverter extends Converter {
      * string for validating against lexical facets such as pattern.
      */
 
-    public static class StringToNonStringDerivedType extends StringConverter {
+    public static class StringToNonStringDerivedType extends StringConverter<AtomicValue> {
         private StringConverter phaseOne;
         private DownCastingConverter phaseTwo;
 
@@ -188,14 +99,14 @@ public abstract class StringConverter extends Converter {
         }
 
         @Override
-        public Converter setNamespaceResolver(NamespaceResolver resolver) {
+        public StringToNonStringDerivedType setNamespaceResolver(NamespaceResolver resolver) {
             return new StringToNonStringDerivedType(
-                    (StringConverter)phaseOne.setNamespaceResolver(resolver),
-                    (DownCastingConverter)phaseTwo.setNamespaceResolver(resolver));
+                    (StringConverter) phaseOne.setNamespaceResolver(resolver),
+                    (DownCastingConverter) phaseTwo.setNamespaceResolver(resolver));
         }
 
         /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
+        public ConversionResult convert(StringValue input) {
             CharSequence in = input.getStringValueCS();
             try {
                 in = phaseTwo.getTargetType().preprocess(in);
@@ -229,7 +140,7 @@ public abstract class StringConverter extends Converter {
          *
          * @param input the string to be validated
          * @return null if validation is successful, or a ValidationFailure indicating the reasons for failure
-         *         if unsuccessful
+         * if unsuccessful
          */
         @Override
         public ValidationFailure validate(CharSequence input) {
@@ -240,7 +151,7 @@ public abstract class StringConverter extends Converter {
             }
             ConversionResult temp = phaseOne.convertString(input);
             if (temp instanceof ValidationFailure) {
-                return (ValidationFailure)temp;
+                return (ValidationFailure) temp;
             }
             return phaseTwo.validate((AtomicValue) temp, input);
         }
@@ -250,10 +161,11 @@ public abstract class StringConverter extends Converter {
      * Converts from xs:string or xs:untypedAtomic to xs:String
      */
 
-    public static class StringToString extends StringConverter {
-        /*@NotNull*/
+    public static class StringToString extends StringConverter<StringValue> {
+        public static final StringToString INSTANCE = new StringToString();
+
         @Override
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
+        public ConversionResult convert(StringValue input) {
             return new StringValue(input.getStringValueCS());
         }
 
@@ -276,10 +188,11 @@ public abstract class StringConverter extends Converter {
      * Converts from xs:string or xs:untypedAtomic to xs:untypedAtomic
      */
 
-    public static class StringToUntypedAtomic extends StringConverter {
-        /*@NotNull*/
+    public static class StringToUntypedAtomic extends StringConverter<UntypedAtomicValue> {
+        public static final StringToUntypedAtomic INSTANCE = new StringToUntypedAtomic();
+
         @Override
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
+        public UntypedAtomicValue convert(StringValue input) {
             return new UntypedAtomicValue(input.getStringValueCS());
         }
 
@@ -303,8 +216,9 @@ public abstract class StringConverter extends Converter {
      * Converts from xs:string to xs:normalizedString
      */
 
-    public static class StringToNormalizedString extends StringConverter {
-        /*@NotNull*/
+    public static class StringToNormalizedString extends StringConverter<StringValue> {
+        public static final StringToNormalizedString INSTANCE = new StringToNormalizedString();
+
         public ConversionResult convertString(/*@NotNull*/ CharSequence input) {
             return new StringValue(Whitespace.normalizeWhitespace(input), BuiltInAtomicType.NORMALIZED_STRING);
         }
@@ -323,8 +237,9 @@ public abstract class StringConverter extends Converter {
      * Converts from xs:string to xs:token
      */
 
-    public static class StringToToken extends StringConverter {
-        /*@NotNull*/
+    public static class StringToToken extends StringConverter<StringValue> {
+        public static final StringToToken INSTANCE = new StringToToken();
+
         public ConversionResult convertString(/*@NotNull*/ CharSequence input) {
             return new StringValue(Whitespace.collapseWhitespace(input), BuiltInAtomicType.TOKEN);
         }
@@ -343,9 +258,10 @@ public abstract class StringConverter extends Converter {
      * Converts from xs:string to xs:language
      */
 
-    public static class StringToLanguage extends StringConverter {
+    public static class StringToLanguage extends StringConverter<StringValue> {
         private final static Pattern regex = Pattern.compile("[a-zA-Z]{1,8}(-[a-zA-Z0-9]{1,8})*");
         // See erratum E2-25 to XML Schema Part 2.
+        public static final StringToLanguage INSTANCE = new StringToLanguage();
 
         /*@NotNull*/
         public ConversionResult convertString(/*@NotNull*/ CharSequence input) {
@@ -370,7 +286,13 @@ public abstract class StringConverter extends Converter {
      * Converts from xs:string to xs:NCName, xs:ID, xs:IDREF, or xs:ENTITY
      */
 
-    public static class StringToNCName extends StringConverter {
+    public static class StringToNCName extends StringConverter<StringValue> {
+
+        public static final StringToNCName TO_ID = new StringToNCName(BuiltInAtomicType.ID);
+        public static final StringToNCName TO_ENTITY = new StringToNCName(BuiltInAtomicType.ENTITY);
+        public static final StringToNCName TO_NCNAME = new StringToNCName(BuiltInAtomicType.NCNAME);
+        public static final StringToNCName TO_IDREF = new StringToNCName(BuiltInAtomicType.IDREF);
+
         AtomicType targetType;
 
         public StringToNCName(AtomicType targetType) {
@@ -401,10 +323,9 @@ public abstract class StringConverter extends Converter {
      * Converts from xs:string to xs:NMTOKEN
      */
 
-    public static class StringToNMTOKEN extends StringConverter {
+    public static class StringToNMTOKEN extends StringConverter<StringValue> {
 
-        public StringToNMTOKEN() {
-        }
+        public final static StringToNMTOKEN INSTANCE = new StringToNMTOKEN();
 
         /*@NotNull*/
         public ConversionResult convertString(/*@NotNull*/ CharSequence input) {
@@ -432,6 +353,7 @@ public abstract class StringConverter extends Converter {
      */
 
     public static class StringToName extends StringToNCName {
+        public static final StringToName INSTANCE = new StringToName();
 
         public StringToName() {
             super(BuiltInAtomicType.NAME);
@@ -475,7 +397,7 @@ public abstract class StringConverter extends Converter {
      * Converts from xs:string to a user-defined type derived directly from xs:string
      */
 
-    public static class StringToStringSubtype extends StringConverter {
+    public static class StringToStringSubtype extends StringConverter<StringValue> {
         AtomicType targetType;
         int whitespaceAction;
 
@@ -518,7 +440,7 @@ public abstract class StringConverter extends Converter {
      * Converts from xs;string to a user-defined type derived from a built-in subtype of xs:string
      */
 
-    public static class StringToDerivedStringSubtype extends StringConverter {
+    public static class StringToDerivedStringSubtype extends StringConverter<StringValue> {
         AtomicType targetType;
         StringConverter builtInValidator;
         int whitespaceAction;
@@ -558,7 +480,7 @@ public abstract class StringConverter extends Converter {
      * Converts a string to xs:float
      */
 
-    public static class StringToFloat extends StringConverter {
+    public static class StringToFloat extends StringConverter<FloatValue> {
         public StringToFloat(ConversionRules rules) {
             super(rules);
         }
@@ -569,42 +491,20 @@ public abstract class StringConverter extends Converter {
                 float flt = (float) getConversionRules().getStringToDoubleConverter().stringToNumber(input);
                 return new FloatValue(flt);
             } catch (NumberFormatException err) {
-                ValidationFailure ve = new ValidationFailure("Cannot convert string to float: " + input.toString());
+                ValidationFailure ve = new ValidationFailure("Cannot convert string to float: " + input);
                 ve.setErrorCode("FORG0001");
                 return ve;
             }
         }
     }
 
-    /**
-     * Converts a string to a double. The rules change in XSD 1.1 to permit "+INF"
-     */
-
-//    public static class StringToDouble extends StringConverter {
-//        net.sf.saxon.type.StringToDouble worker;
-//
-//        public StringToDouble(/*@NotNull*/ ConversionRules rules) {
-//            super(rules);
-//            worker = rules.getStringToDoubleConverter();
-//        }
-//
-//        /*@NotNull*/
-//        public ConversionResult convertString(/*@NotNull*/ CharSequence input) {
-//            try {
-//                double dble = worker.stringToNumber(input);
-//                return new DoubleValue(dble);
-//            } catch (NumberFormatException err) {
-//                return new ValidationFailure("Cannot convert string to double: " + Err.wrap(input.toString(), Err.VALUE));
-//            }
-//        }
-//    }
-
     /**
      * Converts a string to an xs:decimal
      */
 
-    public static class StringToDecimal extends StringConverter {
-        /*@NotNull*/
+    public static class StringToDecimal extends StringConverter<DecimalValue> {
+        public static final StringToDecimal INSTANCE = new StringToDecimal();
+
         public ConversionResult convertString(/*@NotNull*/ CharSequence input) {
             return BigDecimalValue.makeDecimalValue(input, true);
         }
@@ -615,7 +515,7 @@ public abstract class StringConverter extends Converter {
             if (BigDecimalValue.castableAsDecimal(input)) {
                 return null;
             } else {
-                return new ValidationFailure("Cannot convert string to decimal: " + input.toString());
+                return new ValidationFailure("Cannot convert string to decimal: " + input);
             }
         }
     }
@@ -624,9 +524,10 @@ public abstract class StringConverter extends Converter {
      * Converts a string to an integer
      */
 
-    public static class StringToInteger extends StringConverter {
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
+    public static class StringToInteger extends StringConverter<IntegerValue> {
+        public static final StringToInteger INSTANCE = new StringToInteger();
+
+        public ConversionResult convert(StringValue input) {
             return IntegerValue.stringToInteger(input.getStringValueCS());
         }
 
@@ -645,7 +546,7 @@ public abstract class StringConverter extends Converter {
      * Converts a string to a built-in subtype of integer
      */
 
-    public static class StringToIntegerSubtype extends StringConverter {
+    public static class StringToIntegerSubtype extends StringConverter<Int64Value> {
 
         BuiltInAtomicType targetType;
 
@@ -661,7 +562,7 @@ public abstract class StringConverter extends Converter {
                 if (ok) {
                     return ((Int64Value) iv).copyAsSubType(targetType);
                 } else {
-                    return new ValidationFailure("Integer value is out of range for type " + targetType.toString());
+                    return new ValidationFailure("Integer value is out of range for type " + targetType);
                 }
             } else if (iv instanceof BigIntegerValue) {
                 boolean ok = IntegerValue.checkBigRange(((BigIntegerValue) iv).asBigInteger(), targetType);
@@ -669,7 +570,7 @@ public abstract class StringConverter extends Converter {
                     ((BigIntegerValue) iv).setTypeLabel(targetType);
                     return iv;
                 } else {
-                    return new ValidationFailure("Integer value is out of range for type " + targetType.toString());
+                    return new ValidationFailure("Integer value is out of range for type " + targetType);
                 }
             } else {
                 assert iv instanceof ValidationFailure;
@@ -682,8 +583,9 @@ public abstract class StringConverter extends Converter {
      * Converts a string to a duration
      */
 
-    public static class StringToDuration extends StringConverter {
-        /*@NotNull*/
+    public static class StringToDuration extends StringConverter<DurationValue> {
+        public static final StringToDuration INSTANCE = new StringToDuration();
+
         public ConversionResult convertString(/*@NotNull*/ CharSequence input) {
             return DurationValue.makeDuration(input);
         }
@@ -694,9 +596,10 @@ public abstract class StringConverter extends Converter {
      */
 
 
-    public static class StringToDayTimeDuration extends StringConverter {
-        /*@NotNull*/
-        public ConversionResult convertString(/*@NotNull*/ CharSequence input) {
+    public static class StringToDayTimeDuration extends StringConverter<DayTimeDurationValue> {
+        public static final StringToDayTimeDuration INSTANCE = new StringToDayTimeDuration();
+
+        public ConversionResult convertString(CharSequence input) {
             return DayTimeDurationValue.makeDayTimeDurationValue(input);
         }
     }
@@ -705,8 +608,9 @@ public abstract class StringConverter extends Converter {
      * Converts a string to a yearMonthDuration
      */
 
-    public static class StringToYearMonthDuration extends StringConverter {
-        /*@NotNull*/
+    public static class StringToYearMonthDuration extends StringConverter<YearMonthDurationValue> {
+        public static final StringToYearMonthDuration INSTANCE = new StringToYearMonthDuration();
+
         public ConversionResult convertString(/*@NotNull*/ CharSequence input) {
             return YearMonthDurationValue.makeYearMonthDurationValue(input);
         }
@@ -716,7 +620,7 @@ public abstract class StringConverter extends Converter {
      * Converts a string to a dateTime
      */
 
-    public static class StringToDateTime extends StringConverter {
+    public static class StringToDateTime extends StringConverter<DateTimeValue> {
         public StringToDateTime(ConversionRules rules) {
             super(rules);
         }
@@ -731,7 +635,7 @@ public abstract class StringConverter extends Converter {
      * Converts a string to a dateTimeStamp
      */
 
-    public static class StringToDateTimeStamp extends StringConverter {
+    public static class StringToDateTimeStamp extends StringConverter<DateTimeValue> {
         public StringToDateTimeStamp(ConversionRules rules) {
             super(rules);
         }
@@ -740,10 +644,10 @@ public abstract class StringConverter extends Converter {
         public ConversionResult convertString(/*@NotNull*/ CharSequence input) {
             ConversionResult val = DateTimeValue.makeDateTimeValue(input, getConversionRules());
             if (val instanceof DateTimeValue) {
-                if (!((DateTimeValue)val).hasTimezone()) {
+                if (!((DateTimeValue) val).hasTimezone()) {
                     return new ValidationFailure("Supplied DateTimeStamp value " + input + " has no time zone");
                 } else {
-                    ((DateTimeValue)val).setTypeLabel(BuiltInAtomicType.DATE_TIME_STAMP);
+                    ((DateTimeValue) val).setTypeLabel(BuiltInAtomicType.DATE_TIME_STAMP);
                 }
             }
             return val;
@@ -754,7 +658,7 @@ public abstract class StringConverter extends Converter {
      * Converts a string to a date
      */
 
-    public static class StringToDate extends StringConverter {
+    public static class StringToDate extends StringConverter<DateValue> {
         public StringToDate(ConversionRules rules) {
             super(rules);
         }
@@ -769,11 +673,9 @@ public abstract class StringConverter extends Converter {
      * Converts a string to a gMonth
      */
 
-    public static class StringToGMonth extends StringConverter {
-        public StringToGMonth() {
-        }
+    public static class StringToGMonth extends StringConverter<GMonthValue> {
+        public static final StringToGMonth INSTANCE = new StringToGMonth();
 
-        /*@NotNull*/
         public ConversionResult convertString(/*@NotNull*/ CharSequence input) {
             return GMonthValue.makeGMonthValue(input);
         }
@@ -783,7 +685,7 @@ public abstract class StringConverter extends Converter {
      * Converts a string to a gYearMonth
      */
 
-    public static class StringToGYearMonth extends StringConverter {
+    public static class StringToGYearMonth extends StringConverter<GYearMonthValue> {
         public StringToGYearMonth(ConversionRules rules) {
             super(rules);
         }
@@ -798,7 +700,7 @@ public abstract class StringConverter extends Converter {
      * Converts a string to a gYear
      */
 
-    public static class StringToGYear extends StringConverter {
+    public static class StringToGYear extends StringConverter<GYearValue> {
         public StringToGYear(ConversionRules rules) {
             super(rules);
         }
@@ -813,12 +715,10 @@ public abstract class StringConverter extends Converter {
      * Converts a string to a gMonthDay
      */
 
-    public static class StringToGMonthDay extends StringConverter {
-        public StringToGMonthDay() {
-        }
+    public static class StringToGMonthDay extends StringConverter<GMonthDayValue> {
+        public static final StringToGMonthDay INSTANCE = new StringToGMonthDay();
 
-        /*@NotNull*/
-        public ConversionResult convertString(/*@NotNull*/ CharSequence input) {
+        public ConversionResult convertString(CharSequence input) {
             return GMonthDayValue.makeGMonthDayValue(input);
         }
     }
@@ -827,9 +727,8 @@ public abstract class StringConverter extends Converter {
      * Converts a string to a gDay
      */
 
-    public static class StringToGDay extends StringConverter {
-        public StringToGDay() {
-        }
+    public static class StringToGDay extends StringConverter<GDayValue> {
+        public static final StringToGDay INSTANCE = new StringToGDay();
 
         /*@NotNull*/
         public ConversionResult convertString(/*@NotNull*/ CharSequence input) {
@@ -841,9 +740,10 @@ public abstract class StringConverter extends Converter {
      * Converts a string to a time
      */
 
-    public static class StringToTime extends StringConverter {
-        /*@NotNull*/
-        public ConversionResult convertString(/*@NotNull*/ CharSequence input) {
+    public static class StringToTime extends StringConverter<TimeValue> {
+        public static final StringToTime INSTANCE = new StringToTime();
+
+        public ConversionResult convertString(CharSequence input) {
             return TimeValue.makeTimeValue(input);
         }
     }
@@ -852,8 +752,9 @@ public abstract class StringConverter extends Converter {
      * Converts a string to a boolean
      */
 
-    public static class StringToBoolean extends StringConverter {
-        /*@NotNull*/
+    public static class StringToBoolean extends StringConverter<BooleanValue> {
+        public static final StringToBoolean INSTANCE = new StringToBoolean();
+
         public ConversionResult convertString(/*@NotNull*/ CharSequence input) {
             return BooleanValue.fromString(input);
         }
@@ -863,8 +764,9 @@ public abstract class StringConverter extends Converter {
      * Converts a string to hexBinary
      */
 
-    public static class StringToHexBinary extends StringConverter {
-        /*@NotNull*/
+    public static class StringToHexBinary extends StringConverter<HexBinaryValue> {
+        public static final StringToHexBinary INSTANCE = new StringToHexBinary();
+
         public ConversionResult convertString(/*@NotNull*/ CharSequence input) {
             try {
                 return new HexBinaryValue(input);
@@ -874,11 +776,28 @@ public abstract class StringConverter extends Converter {
         }
     }
 
+    /**
+     * Converts string to base64
+     */
+
+    public static class StringToBase64Binary extends StringConverter {
+        public static final StringToBase64Binary INSTANCE = new StringToBase64Binary();
+
+        public ConversionResult convertString(/*@NotNull*/ CharSequence input) {
+            try {
+                return new Base64BinaryValue(input);
+            } catch (XPathException e) {
+                return ValidationFailure.fromException(e);
+            }
+        }
+    }
+
+
     /**
      * Converts String to QName
      */
 
-    public static class StringToQName extends StringConverter {
+    public static class StringToQName extends StringConverter<QNameValue> {
 
         NamespaceResolver nsResolver;
 
@@ -886,7 +805,7 @@ public abstract class StringConverter extends Converter {
             super(rules);
         }
 
-        public Converter setNamespaceResolver(NamespaceResolver resolver) {
+        public StringToQName setNamespaceResolver(NamespaceResolver resolver) {
             StringToQName c = new StringToQName(getConversionRules());
             c.nsResolver = resolver;
             return c;
@@ -906,17 +825,11 @@ public abstract class StringConverter extends Converter {
                 String[] parts = NameChecker.getQNameParts(Whitespace.trimWhitespace(input));
                 String uri = nsResolver.getURIForPrefix(parts[0], true);
                 if (uri == null) {
-                    ValidationFailure failure = new ValidationFailure("Namespace prefix " + Err.wrap(parts[0]) + " has not been declared");
+                    ValidationFailure failure = new ValidationFailure("Namespace prefix " +
+                                                                              Err.wrap(parts[0]) + " has not been declared");
                     failure.setErrorCode("FONS0004");
                     return failure;
                 }
-//                if (fingerprint == StandardNames.XS_NOTATION) {
-//                    // This check added in 9.3. The XSLT spec says that this check should not be performed during
-//                    // validation. However, this appears to be based on an incorrect assumption: see spec bug 6952
-//                    if (!rules.isDeclaredNotation(uri, parts[1])) {
-//                        return new ValidationFailure("Notation {" + uri + "}" + parts[1] + " is not declared in the schema");
-//                    }
-//                }
                 return new QNameValue(parts[0], uri, parts[1], BuiltInAtomicType.QNAME, false);
             } catch (QNameException err) {
                 return new ValidationFailure("Invalid lexical QName " + Err.wrap(input));
@@ -930,7 +843,7 @@ public abstract class StringConverter extends Converter {
      * Converts String to NOTATION
      */
 
-    public static class StringToNotation extends StringConverter {
+    public static class StringToNotation extends StringConverter<NotationValue> {
 
         NamespaceResolver nsResolver;
 
@@ -939,7 +852,7 @@ public abstract class StringConverter extends Converter {
         }
 
         @Override
-        public Converter setNamespaceResolver(NamespaceResolver resolver) {
+        public StringToNotation setNamespaceResolver(NamespaceResolver resolver) {
             StringToNotation c = new StringToNotation(getConversionRules());
             c.nsResolver = resolver;
             return c;
@@ -979,7 +892,7 @@ public abstract class StringConverter extends Converter {
      * Converts string to anyURI
      */
 
-    public static class StringToAnyURI extends StringConverter {
+    public static class StringToAnyURI extends StringConverter<AnyURIValue> {
         public StringToAnyURI(ConversionRules rules) {
             super(rules);
         }
@@ -989,7 +902,7 @@ public abstract class StringConverter extends Converter {
             if (getConversionRules().isValidURI(input)) {
                 return new AnyURIValue(input);
             } else {
-                return new ValidationFailure("Invalid URI: " + input.toString());
+                return new ValidationFailure("Invalid URI: " + input);
             }
         }
 
@@ -999,43 +912,16 @@ public abstract class StringConverter extends Converter {
             if (getConversionRules().isValidURI(input)) {
                 return null;
             } else {
-                return new ValidationFailure("Invalid URI: " + input.toString());
+                return new ValidationFailure("Invalid URI: " + input);
             }
         }
     }
 
-    /**
-     * Converter that does nothing - it returns the input unchanged
-     */
-
-    public static class IdentityConverter extends StringConverter {
-        /*@NotNull*/ public static IdentityConverter THE_INSTANCE = new IdentityConverter();
-
-        /*@NotNull*/
-        public ConversionResult convert(/*@NotNull*/ AtomicValue input) {
-            return input;
-        }
-
-        /*@NotNull*/
-        public ConversionResult convertString(/*@NotNull*/ CharSequence input) {
-            return StringValue.makeStringValue(input);
-        }
-
-        public boolean isAlwaysSuccessful() {
-            return true;
-        }
-
-        /*@Nullable*/
-        public ValidationFailure validate(/*@NotNull*/ CharSequence input) {
-            return null;
-        }
-    }
-
     /**
      * Converter from string to plain union types
      */
 
-    public static class StringToUnionConverter extends StringConverter {
+    public static class StringToUnionConverter extends StringConverter<AtomicValue> {
 
         SimpleType targetType;
         ConversionRules rules;
@@ -1056,7 +942,7 @@ public abstract class StringConverter extends Converter {
          *
          * @param input the string to be converted
          * @return either an {@link net.sf.saxon.value.AtomicValue} of the appropriate type for this converter (if conversion
-         *         succeeded), or a {@link net.sf.saxon.type.ValidationFailure} if conversion failed.
+         * succeeded), or a {@link net.sf.saxon.type.ValidationFailure} if conversion failed.
          */
         /*@NotNull*/
         @Override
