@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -14,7 +14,10 @@ import net.sf.saxon.expr.parser.RoleDiagnostic;
 import net.sf.saxon.lib.ConversionRules;
 import net.sf.saxon.lib.FunctionAnnotationHandler;
 import net.sf.saxon.ma.map.MapType;
-import net.sf.saxon.om.*;
+import net.sf.saxon.om.Item;
+import net.sf.saxon.om.Sequence;
+import net.sf.saxon.om.SequenceIterator;
+import net.sf.saxon.om.SequenceTool;
 import net.sf.saxon.pattern.*;
 import net.sf.saxon.query.Annotation;
 import net.sf.saxon.query.AnnotationList;
@@ -26,6 +29,7 @@ import net.sf.saxon.z.IntUniversalSet;
 
 import java.util.HashSet;
 import java.util.Map;
+import java.util.Optional;
 import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
 
@@ -71,7 +75,7 @@ public class TypeHierarchy {
 
     public TypeHierarchy(Configuration config) {
         this.config = config;
-        map = new ConcurrentHashMap<ItemTypePair, Integer>();
+        map = new ConcurrentHashMap<>();
     }
 
     /**
@@ -90,7 +94,7 @@ public class TypeHierarchy {
             throws XPathException {
         ItemType suppliedItemType = SequenceTool.getItemType(value, this);
 
-        SequenceIterator iterator = value.iterate();
+        SequenceIterator<? extends Item> iterator = value.iterate();
         final ItemType requiredItemType = requiredType.getPrimaryType();
 
         if (requiredItemType.isPlainType()) {
@@ -111,113 +115,95 @@ public class TypeHierarchy {
 
             // step 2: convert untyped atomic values to target item type
 
-            if (relationship(suppliedItemType, BuiltInAtomicType.UNTYPED_ATOMIC) != DISJOINT) {
+            if (relationship(suppliedItemType, BuiltInAtomicType.UNTYPED_ATOMIC) != DISJOINT &&
+                    !isSubType(BuiltInAtomicType.UNTYPED_ATOMIC, requiredItemType)) {
                 final boolean nsSensitive = ((SimpleType) requiredItemType).isNamespaceSensitive();
-                ItemMappingFunction converter;
+                ItemMappingFunction<Item, Item> converter;
                 if (nsSensitive) {
-                    converter = new ItemMappingFunction() {
-                        public Item mapItem(Item item) throws XPathException {
-                            if (item instanceof UntypedAtomicValue) {
-                                ValidationFailure vf = new ValidationFailure(
-                                        "Failed to convert the " + role.getMessage() + ": " +
-                                        "Implicit conversion of untypedAtomic value to " + requiredItemType.toString() +
-                                                " is not allowed");
-                                vf.setErrorCode("XPTY0117");
-                                throw vf.makeException();
-                            } else {
-                                return item;
-                            }
+                    converter = item -> {
+                        if (item instanceof UntypedAtomicValue) {
+                            ValidationFailure vf = new ValidationFailure(
+                                    "Failed to convert the " + role.getMessage() + ": " +
+                                    "Implicit conversion of untypedAtomic value to " + requiredItemType +
+                                            " is not allowed");
+                            vf.setErrorCode("XPTY0117");
+                            throw vf.makeException();
+                        } else {
+                            return item;
                         }
                     };
                 } else if (((SimpleType) requiredItemType).isUnionType()) {
                     final ConversionRules rules = config.getConversionRules();
-                    converter = new ItemMappingFunction() {
-                        public Item mapItem(Item item) throws XPathException {
-                            if (item instanceof UntypedAtomicValue) {
-                                try {
-                                    return ((SimpleType) requiredItemType).getTypedValue(
-                                            item.getStringValueCS(), null, rules).head();
-                                } catch (ValidationException ve) {
-                                    ve.setErrorCode("XPTY0004");
-                                    throw ve;
-                                }
-                            } else {
-                                return item;
+                    converter = item -> {
+                        if (item instanceof UntypedAtomicValue) {
+                            try {
+                                return ((SimpleType) requiredItemType).getTypedValue(
+                                        item.getStringValueCS(), null, rules).head();
+                            } catch (ValidationException ve) {
+                                ve.setErrorCode("XPTY0004");
+                                throw ve;
                             }
+                        } else {
+                            return item;
                         }
                     };
                 } else {
-                    converter = new ItemMappingFunction() {
-                        public Item mapItem(Item item) throws XPathException {
-                            if (item instanceof UntypedAtomicValue) {
-                                ConversionResult val = Converter.convert(
-                                        (UntypedAtomicValue) item, (AtomicType) requiredItemType, config.getConversionRules());
-                                if (val instanceof ValidationFailure) {
-                                    ValidationFailure vex = (ValidationFailure) val;
-                                    vex.setMessage("Failed to convert the " + role.getMessage() + ": " + vex.getMessage());
-                                    throw vex.makeException();
-                                }
-                                return (Item) val;
-                            } else {
-                                //noinspection unchecked
-                                return item;
-                            }
+                    converter = item -> {
+                        if (item instanceof UntypedAtomicValue) {
+                            return Converter.convert(
+                                    (UntypedAtomicValue) item, (AtomicType) requiredItemType, config.getConversionRules());
+                        } else {
+                            return item;
                         }
                     };
                 }
-                iterator = new ItemMappingIterator(iterator, converter, true);
+                iterator = new ItemMappingIterator<>(iterator, converter, true);
             }
 
             // step 3: apply numeric promotion
 
             if (requiredItemType.equals(BuiltInAtomicType.DOUBLE)) {
-                ItemMappingFunction promoter = new ItemMappingFunction() {
-                    public DoubleValue mapItem(Item item) throws XPathException {
-                        if (item instanceof NumericValue) {
-                            return (DoubleValue) Converter.convert(
-                                    (NumericValue)item, BuiltInAtomicType.DOUBLE, config.getConversionRules()).asAtomic();
-                        } else {
-                            throw new XPathException(
-                                    "Failed to convert the " + role.getMessage() + ": " +
-                                    "Cannot promote non-numeric value to xs:double", "XPTY0004");
-                        }
+                ItemMappingFunction<Item, Item> promoter = item -> {
+                    if (item instanceof NumericValue) {
+                        return (DoubleValue) Converter.convert(
+                                (NumericValue)item, BuiltInAtomicType.DOUBLE, config.getConversionRules()).asAtomic();
+                    } else {
+                        throw new XPathException(
+                                "Failed to convert the " + role.getMessage() + ": " +
+                                "Cannot promote non-numeric value to xs:double", "XPTY0004");
                     }
                 };
-                iterator = new ItemMappingIterator(iterator, promoter, true);
+                iterator = new ItemMappingIterator<>(iterator, promoter, true);
             } else if (requiredItemType.equals(BuiltInAtomicType.FLOAT)) {
-                ItemMappingFunction promoter = new ItemMappingFunction() {
-                    public FloatValue mapItem(Item item) throws XPathException {
-                        if (item instanceof DoubleValue) {
-                            throw new XPathException(
-                                    "Failed to convert the " + role.getMessage() + ": " +
-                                    "Cannot promote xs:double value to xs:float", "XPTY0004");
-                        } else if (item instanceof NumericValue) {
-                            return (FloatValue) Converter.convert(
-                                    (NumericValue)item, BuiltInAtomicType.FLOAT, config.getConversionRules()).asAtomic();
-                        } else {
-                            throw new XPathException(
-                                    "Failed to convert the " + role.getMessage() + ": " +
-                                    "Cannot promote non-numeric value to xs:float", "XPTY0004");
-                        }
+                ItemMappingFunction<Item, Item> promoter = item -> {
+                    if (item instanceof DoubleValue) {
+                        throw new XPathException(
+                                "Failed to convert the " + role.getMessage() + ": " +
+                                "Cannot promote xs:double value to xs:float", "XPTY0004");
+                    } else if (item instanceof NumericValue) {
+                        return (FloatValue) Converter.convert(
+                                (NumericValue)item, BuiltInAtomicType.FLOAT, config.getConversionRules()).asAtomic();
+                    } else {
+                        throw new XPathException(
+                                "Failed to convert the " + role.getMessage() + ": " +
+                                "Cannot promote non-numeric value to xs:float", "XPTY0004");
                     }
                 };
-                iterator = new ItemMappingIterator(iterator, promoter, true);
+                iterator = new ItemMappingIterator<>(iterator, promoter, true);
             }
 
             // step 4: apply URI-to-string promotion
 
             if (requiredItemType.equals(BuiltInAtomicType.STRING) &&
                     relationship(suppliedItemType, BuiltInAtomicType.ANY_URI) != DISJOINT) {
-                ItemMappingFunction promoter = new ItemMappingFunction() {
-                    public Item mapItem(Item item) throws XPathException {
-                        if (item instanceof AnyURIValue) {
-                            return new StringValue(item.getStringValueCS());
-                        } else {
-                            return item;
-                        }
+                ItemMappingFunction<Item, Item> promoter = item -> {
+                    if (item instanceof AnyURIValue) {
+                        return new StringValue(item.getStringValueCS());
+                    } else {
+                        return item;
                     }
                 };
-                iterator = new ItemMappingIterator(iterator, promoter, true);
+                iterator = new ItemMappingIterator<>(iterator, promoter, true);
             }
         }
 
@@ -374,12 +360,6 @@ public class TypeHierarchy {
             } else if (t1.isPlainType()) {
                 if (t2 instanceof NodeTest || t2 instanceof FunctionItemType || t2 instanceof JavaExternalObjectType) {
                     return DISJOINT;
-                } else if (t2 instanceof JavaExternalObjectType) {
-                    if (((AtomicType) t1).getFingerprint() == StandardNames.XS_ANY_ATOMIC_TYPE) {
-                        return SUBSUMES;
-                    } else {
-                        return DISJOINT;
-                    }
                 } else if (t1 == BuiltInAtomicType.ANY_ATOMIC && t2.isPlainType()) {
                     return SUBSUMES;
                 } else if (t2 == BuiltInAtomicType.ANY_ATOMIC) {
@@ -467,7 +447,7 @@ public class TypeHierarchy {
                 }
                 return DISJOINT;
             } else if (t1 instanceof NodeTest) {
-                if (t2.isPlainType() || (t2 instanceof FunctionItemType) || (t2 instanceof JavaExternalObjectType)) {
+                if (t2.isPlainType() || t2 instanceof FunctionItemType) {
                     return DISJOINT;
                 } else {
                     // both types are NodeTests
@@ -479,8 +459,6 @@ public class TypeHierarchy {
                         }
                     } else if (t2 instanceof AnyNodeTest) {
                         return SUBSUMED_BY;
-                    } else if (t1 instanceof ErrorType) {
-                        return DISJOINT;
                     } else if (t2 instanceof ErrorType) {
                         return DISJOINT;
                     } else {
@@ -500,26 +478,27 @@ public class TypeHierarchy {
                             nodeKindRelationship = OVERLAPS;
                         }
 
-                        // now find the relationship between the node names allowed.
+                        // Now find the relationship between the node names allowed.  See bug 3713
                         int nodeNameRelationship;
+                        Optional<IntSet> on1 = ((NodeTest) t1).getRequiredNodeNames();
+                        Optional<IntSet> on2 = ((NodeTest) t2).getRequiredNodeNames();
 
-                        IntSet n1 = ((NodeTest) t1).getRequiredNodeNames(); // null means all names allowed
-                        IntSet n2 = ((NodeTest) t2).getRequiredNodeNames(); // null means all names allowed
-
-                        if (t1 instanceof LocalNameTest || t1 instanceof NamespaceTest ||
-                                t2 instanceof LocalNameTest || t2 instanceof NamespaceTest) {
-                            // Bug 3712
+                        if (t1 instanceof QNameTest && t2 instanceof QNameTest) {
+                            nodeNameRelationship = nameTestRelationship((QNameTest)t1, (QNameTest)t2);
+                        } else if (on1.isPresent() && on1.get() instanceof IntUniversalSet) {
+                            if (on2.isPresent() && on2.get() instanceof IntUniversalSet) {
+                                nodeNameRelationship = SAME_TYPE;
+                            } else {
+                                nodeNameRelationship = SUBSUMES;
+                            }
+                        } else if (on2.isPresent() && on2.get() instanceof IntUniversalSet) {
+                            nodeNameRelationship = SUBSUMED_BY;
+                        } else if (!(on1.isPresent() && on2.isPresent())) {
                             nodeNameRelationship = t1.equals(t2) ? SAME_TYPE : OVERLAPS;
                         } else {
-                            if (n1 == null || n1 instanceof IntUniversalSet) {
-                                if (n2 == null || n2 instanceof IntUniversalSet) {
-                                    nodeNameRelationship = SAME_TYPE;
-                                } else {
-                                    nodeNameRelationship = SUBSUMES;
-                                }
-                            } else if (n2 == null || n2 instanceof IntUniversalSet) {
-                                nodeNameRelationship = SUBSUMED_BY;
-                            } else if (n1.containsAll(n2)) {
+                            IntSet n1 = on1.get();
+                            IntSet n2 = on2.get();
+                            if (n1.containsAll(n2)) {
                                 if (n1.size() == n2.size()) {
                                     nodeNameRelationship = SAME_TYPE;
                                 } else {
@@ -536,7 +515,7 @@ public class TypeHierarchy {
 
                         // now find the relationship between the content types allowed
 
-                        int contentRelationship = computeContentRelationship(t1, t2, n1, n2);
+                        int contentRelationship = computeContentRelationship(t1, t2, on1, on2);
 
                         // now analyse the three different relationsships
 
@@ -552,18 +531,28 @@ public class TypeHierarchy {
                                 (nodeNameRelationship == SAME_TYPE || nodeNameRelationship == SUBSUMED_BY) &&
                                 (contentRelationship == SAME_TYPE || contentRelationship == SUBSUMED_BY)) {
                             return SUBSUMED_BY;
-                        } else if (nodeKindRelationship == DISJOINT ||
-                                nodeNameRelationship == DISJOINT ||
-                                contentRelationship == DISJOINT) {
+                        } else if (nodeNameRelationship == DISJOINT || contentRelationship == DISJOINT) {
                             return DISJOINT;
                         } else {
                             return OVERLAPS;
                         }
                     }
                 }
-            } else if (t1 instanceof JavaExternalObjectType) {
+            } else if (t1 instanceof AnyExternalObjectType) {
+                if (!(t2 instanceof AnyExternalObjectType)) {
+                    return DISJOINT;
+                }
+                if (t1 instanceof JavaExternalObjectType) {
+                    if (t2 == AnyExternalObjectType.THE_INSTANCE) {
+                        return SUBSUMED_BY;
+                    } else if (t2 instanceof JavaExternalObjectType) {
+                        return ((JavaExternalObjectType)t1).getRelationship((JavaExternalObjectType)t2);
+                    } else {
+                        return DISJOINT;
+                    }
+                }
                 if (t2 instanceof JavaExternalObjectType) {
-                    return ((JavaExternalObjectType) t1).getRelationship((JavaExternalObjectType) t2);
+                    return SUBSUMES;
                 } else {
                     return DISJOINT;
                 }
@@ -601,7 +590,7 @@ public class TypeHierarchy {
                         int assertionRelationship = SAME_TYPE;
                         AnnotationList first = ((FunctionItemType) t1).getAnnotationAssertions();
                         AnnotationList second = ((FunctionItemType) t2).getAnnotationAssertions();
-                        Set<String> namespaces = new HashSet<String>();
+                        Set<String> namespaces = new HashSet<>();
                         for (Annotation a : first) {
                             namespaces.add(a.getAnnotationQName().getURI());
                         }
@@ -642,6 +631,31 @@ public class TypeHierarchy {
 
     }
 
+    private static int nameTestRelationship(QNameTest t1, QNameTest t2) {
+        if (t1.equals(t2)) {
+            return SAME_TYPE;
+        }
+        if (t2 instanceof NameTest) {
+            return t1.matches(((NameTest) t2).getMatchingNodeName()) ? SUBSUMES : DISJOINT;
+        }
+        if (t1 instanceof NameTest) {
+            return t2.matches(((NameTest) t1).getMatchingNodeName()) ? SUBSUMED_BY : DISJOINT;
+        }
+        if (t2 instanceof SameNameTest) {
+            return t1.matches(((SameNameTest) t2).getMatchingNodeName()) ? SUBSUMES : DISJOINT;
+        }
+        if (t1 instanceof SameNameTest) {
+            return t2.matches(((SameNameTest) t1).getMatchingNodeName()) ? SUBSUMED_BY : DISJOINT;
+        }
+        if (t1 instanceof NamespaceTest && t2 instanceof NamespaceTest) {
+            return DISJOINT;
+        }
+        if (t1 instanceof LocalNameTest && t2 instanceof LocalNameTest) {
+            return DISJOINT;
+        }
+        return OVERLAPS;
+    }
+
     private static int combineRelationships(int rel1, int rel2) {
         if (rel1 == SAME_TYPE &&
                 rel2 == SAME_TYPE) {
@@ -671,7 +685,7 @@ public class TypeHierarchy {
 
 
     private static <X> Set<X> toSet(Iterable<X> in) {
-        Set<X> s = new HashSet<X>();
+        Set<X> s = new HashSet<>();
         for (X x : in) {
             s.add(x);
         }
@@ -737,7 +751,7 @@ public class TypeHierarchy {
      * @param n2 the set of element names allowed by the second type
      * @return the relationship (same type, subsumes, overlaps, subsumed-by)
      */
-    protected int computeContentRelationship(ItemType t1, ItemType t2, IntSet n1, IntSet n2) {
+    protected int computeContentRelationship(ItemType t1, ItemType t2, Optional<IntSet> n1, Optional<IntSet> n2) {
         int contentRelationship;
         if (t1 instanceof DocumentNodeTest) {
             if (t2 instanceof DocumentNodeTest) {
@@ -818,7 +832,7 @@ public class TypeHierarchy {
 
         int itemRel = relationship(s1.getPrimaryType(), s2.getPrimaryType());
 
-        if (cardRel == DISJOINT || itemRel == DISJOINT) {
+        if (itemRel == DISJOINT) {
             return DISJOINT;
         }
 
