@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,16 +9,17 @@ package net.sf.saxon.value;
 
 import net.sf.saxon.om.GroundedValue;
 import net.sf.saxon.om.Item;
-import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.iter.EmptyIterator;
 import net.sf.saxon.tree.iter.UnfailingIterator;
 
+import java.util.Collections;
+
 /**
  * An EmptySequence object represents a sequence containing no members.
  */
 
 
-public final class EmptySequence implements GroundedValue {
+public final class EmptySequence<T extends Item> implements GroundedValue<T> {
 
     // This class has a single instance
     /*@NotNull*/ private static EmptySequence THE_INSTANCE = new EmptySequence();
@@ -37,16 +38,18 @@ public final class EmptySequence implements GroundedValue {
      * @return the singular instances of this class: an empty sequence
      */
 
-    /*@NotNull*/
-    public static EmptySequence getInstance() {
-        return THE_INSTANCE;
+    @SuppressWarnings("unchecked")
+    public static <T extends Item> EmptySequence<T> getInstance() {
+        return (EmptySequence<T>)THE_INSTANCE;
     }
 
-    public String getStringValue() throws XPathException {
+    public String getStringValue() {
         return "";
     }
 
-    public CharSequence getStringValueCS() throws XPathException {
+    public Collections a;
+
+    public CharSequence getStringValueCS() {
         return "";
     }
 
@@ -56,7 +59,7 @@ public final class EmptySequence implements GroundedValue {
      * @return the first item in the sequence if there is one, or null if the sequence
      *         is empty
      */
-    public Item head() {
+    public T head() {
         return null;
     }
 
@@ -65,7 +68,7 @@ public final class EmptySequence implements GroundedValue {
      */
 
     /*@NotNull*/
-    public UnfailingIterator iterate() {
+    public UnfailingIterator<T> iterate() {
         return EmptyIterator.emptyIterator();
     }
 
@@ -123,11 +126,11 @@ public final class EmptySequence implements GroundedValue {
      *
      * @param n position of the required item, counting from zero.
      * @return the n'th item in the sequence, where the first item in the sequence is
-     *         numbered zero. If n is negative or >= the length of the sequence, returns null.
+     *         numbered zero. If n is negative or &gt;= the length of the sequence, returns null.
      */
 
     /*@Nullable*/
-    public Item itemAt(int n) {
+    public T itemAt(int n) {
         return null;
     }
 
@@ -145,7 +148,7 @@ public final class EmptySequence implements GroundedValue {
      */
 
     /*@NotNull*/
-    public GroundedValue subsequence(int min, int length) {
+    public GroundedValue<T> subsequence(int min, int length) {
         return this;
     }
 
@@ -166,7 +169,7 @@ public final class EmptySequence implements GroundedValue {
      *
      * @return the simplified sequence
      */
-    public GroundedValue reduce() {
+    public GroundedValue<T> reduce() {
         return this;
     }
 }
